@@ -136,7 +136,7 @@ nsRootAccessible::~nsRootAccessible()
 }
 
   /* attribute wstring accName; */
-NS_IMETHODIMP nsRootAccessible::GetAccName(nsAWritableString& aAccName) 
+NS_IMETHODIMP nsRootAccessible::GetAccName(nsAString& aAccName) 
 { 
   return GetTitle(aAccName);
 }
@@ -201,6 +201,20 @@ NS_IMETHODIMP nsRootAccessible::GetAccRole(PRUint32 *aAccRole)
   */
 
   *aAccRole = ROLE_PANE;
+
+  // If it's a <dialog>, use ROLE_DIALOG instead
+  nsCOMPtr<nsIContent> rootContent;
+  mDocument->GetRootContent(getter_AddRefs(rootContent));
+  if (rootContent) {
+    nsCOMPtr<nsIDOMElement> rootElement(do_QueryInterface(rootContent));
+    if (rootElement) {
+      nsAutoString name;
+      rootElement->GetLocalName(name);
+      if (name.Equals(NS_LITERAL_STRING("dialog"))) 
+        *aAccRole = ROLE_DIALOG;
+    }
+  }
+
   return NS_OK;
 }
 
@@ -212,7 +226,7 @@ NS_IMETHODIMP nsRootAccessible::GetAccState(PRUint32 *aAccState)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsRootAccessible::GetAccValue(nsAWritableString& aAccValue)
+NS_IMETHODIMP nsRootAccessible::GetAccValue(nsAString& aAccValue)
 {
   return GetURL(aAccValue);
 }
@@ -296,12 +310,12 @@ NS_IMETHODIMP nsRootAccessible::AddAccessibleEventListener(nsIAccessibleEventLis
         nsCOMPtr<nsIDocShell> docShell(do_QueryInterface(container));
         if (docShell) {
           mWebProgress = do_GetInterface(docShell);
-          mWebProgress->AddProgressListener(this);
+          mWebProgress->AddProgressListener(this, nsIWebProgress::NOTIFY_LOCATION | 
+                                                  nsIWebProgress::NOTIFY_STATE_DOCUMENT);
         }
       }
     }
     NS_ASSERTION(mWebProgress, "Could not get nsIWebProgress for nsRootAccessible");
-    mWebProgress->AddProgressListener(this);
   }
 
   return NS_OK;
@@ -362,7 +376,7 @@ NS_IMETHODIMP nsRootAccessible::HandleEvent(nsIDOMEvent* aEvent)
     // Check to see if it's a select element. If so, need the currently focused option
     nsCOMPtr<nsIDOMHTMLSelectElement> selectElement(do_QueryInterface(targetNode));
     if (selectElement)     // ----- Target Node is an HTML <select> element ------
-      nsHTMLSelectOptionAccessible::GetFocusedOptionNode(mPresShell, targetNode, optionTargetNode);
+      nsHTMLSelectOptionAccessible::GetFocusedOptionNode(targetNode, optionTargetNode);
 
     // for focus events on Radio Groups we give the focus to the selected button
     nsCOMPtr<nsIDOMXULSelectControlElement> selectControl(do_QueryInterface(targetNode));
@@ -379,7 +393,7 @@ NS_IMETHODIMP nsRootAccessible::HandleEvent(nsIDOMEvent* aEvent)
 
     if (NS_SUCCEEDED(mAccService->GetAccessibleFor(targetNode, getter_AddRefs(accessible)))) {
       if (eventType.EqualsIgnoreCase("focus") || eventType.EqualsIgnoreCase("DOMMenuItemActive")) { 
-        if (selectControl && optionTargetNode &&
+        if (optionTargetNode &&
             NS_SUCCEEDED(mAccService->GetAccessibleFor(optionTargetNode, getter_AddRefs(accessible)))) {
           FireAccessibleFocusEvent(accessible, optionTargetNode);
         }
@@ -387,15 +401,10 @@ NS_IMETHODIMP nsRootAccessible::HandleEvent(nsIDOMEvent* aEvent)
           FireAccessibleFocusEvent(accessible, targetNode);
       }
       else if (eventType.EqualsIgnoreCase("change")) {
-        if (optionTargetNode) {   // Set to current option only for HTML selects
-          mListener->HandleEvent(nsIAccessibleEventListener::EVENT_SELECTION, accessible);
-          if (NS_SUCCEEDED(mAccService->GetAccessibleFor(optionTargetNode, getter_AddRefs(accessible)))) 
-            FireAccessibleFocusEvent(accessible, optionTargetNode);
-        }
-        else 
+        if (!selectControl)   // Don't use onchange to fire EVENT_STATE_CHANGE events for selects
           mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible);
       }
-      else if (eventType.EqualsIgnoreCase("ListitemStateChange")){
+      else if (eventType.EqualsIgnoreCase("ListitemStateChange")) {
         mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible);
         mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, accessible);
       }
@@ -488,27 +497,27 @@ NS_IMETHODIMP nsRootAccessible::CommandUpdate(nsIDOMEvent* aEvent) { return NS_O
 
 // ------- nsIAccessibleDocument Methods (5) ---------------
 
-NS_IMETHODIMP nsRootAccessible::GetURL(nsAWritableString& aURL)
+NS_IMETHODIMP nsRootAccessible::GetURL(nsAString& aURL)
 {
   return nsDocAccessibleMixin::GetURL(aURL);
 }
 
-NS_IMETHODIMP nsRootAccessible::GetTitle(nsAWritableString& aTitle)
+NS_IMETHODIMP nsRootAccessible::GetTitle(nsAString& aTitle)
 {
   return nsDocAccessibleMixin::GetTitle(aTitle);
 }
 
-NS_IMETHODIMP nsRootAccessible::GetMimeType(nsAWritableString& aMimeType)
+NS_IMETHODIMP nsRootAccessible::GetMimeType(nsAString& aMimeType)
 {
   return nsDocAccessibleMixin::GetMimeType(aMimeType);
 }
 
-NS_IMETHODIMP nsRootAccessible::GetDocType(nsAWritableString& aDocType)
+NS_IMETHODIMP nsRootAccessible::GetDocType(nsAString& aDocType)
 {
   return nsDocAccessibleMixin::GetDocType(aDocType);
 }
 
-NS_IMETHODIMP nsRootAccessible::GetNameSpaceURIForID(PRInt16 aNameSpaceID, nsAWritableString& aNameSpaceURI)
+NS_IMETHODIMP nsRootAccessible::GetNameSpaceURIForID(PRInt16 aNameSpaceID, nsAString& aNameSpaceURI)
 {
   return nsDocAccessibleMixin::GetNameSpaceURIForID(aNameSpaceID, aNameSpaceURI);
 }
@@ -519,7 +528,7 @@ NS_IMETHODIMP nsRootAccessible::GetDocument(nsIDocument **doc)
 }
 
 NS_IMETHODIMP nsRootAccessible::OnStateChange(nsIWebProgress *aWebProgress,
-  nsIRequest *aRequest, PRInt32 aStateFlags, PRUint32 aStatus)
+  nsIRequest *aRequest, PRUint32 aStateFlags, nsresult aStatus)
 {
   return NS_OK;
 }
@@ -529,6 +538,7 @@ NS_IMETHODIMP nsRootAccessible::OnProgressChange(nsIWebProgress *aWebProgress,
   nsIRequest *aRequest, PRInt32 aCurSelfProgress, PRInt32 aMaxSelfProgress,
   PRInt32 aCurTotalProgress, PRInt32 aMaxTotalProgress)
 {
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
   return NS_OK;
 }
 
@@ -550,13 +560,15 @@ NS_IMETHODIMP nsRootAccessible::OnLocationChange(nsIWebProgress *aWebProgress,
 NS_IMETHODIMP nsRootAccessible::OnStatusChange(nsIWebProgress *aWebProgress,
   nsIRequest *aRequest, nsresult aStatus, const PRUnichar *aMessage)
 {
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
   return NS_OK;
 }
 
-/* void onSecurityChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in long state); */
+/* void onSecurityChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in unsigned long state); */
 NS_IMETHODIMP nsRootAccessible::OnSecurityChange(nsIWebProgress *aWebProgress,
-  nsIRequest *aRequest, PRInt32 state)
+  nsIRequest *aRequest, PRUint32 state)
 {
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
   return NS_OK;
 }
 
@@ -575,7 +587,7 @@ nsDocAccessibleMixin::~nsDocAccessibleMixin()
 {
 }
 
-NS_IMETHODIMP nsDocAccessibleMixin::GetURL(nsAWritableString& aURL)
+NS_IMETHODIMP nsDocAccessibleMixin::GetURL(nsAString& aURL)
 { 
   nsCOMPtr<nsIPresShell> presShell;
   mDocument->GetShellAt(0, getter_AddRefs(presShell));
@@ -598,14 +610,14 @@ NS_IMETHODIMP nsDocAccessibleMixin::GetURL(nsAWritableString& aURL)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsDocAccessibleMixin::GetTitle(nsAWritableString& aTitle)
+NS_IMETHODIMP nsDocAccessibleMixin::GetTitle(nsAString& aTitle)
 {
   // This doesn't leak - we don't own the const pointer that's returned
   aTitle = *(mDocument->GetDocumentTitle());
   return NS_OK;
 }
 
-NS_IMETHODIMP nsDocAccessibleMixin::GetMimeType(nsAWritableString& aMimeType)
+NS_IMETHODIMP nsDocAccessibleMixin::GetMimeType(nsAString& aMimeType)
 {
   nsCOMPtr<nsIDOMNSDocument> domnsDocument(do_QueryInterface(mDocument));
   if (domnsDocument) {
@@ -614,7 +626,7 @@ NS_IMETHODIMP nsDocAccessibleMixin::GetMimeType(nsAWritableString& aMimeType)
   return NS_ERROR_FAILURE;
 }
 
-NS_IMETHODIMP nsDocAccessibleMixin::GetDocType(nsAWritableString& aDocType)
+NS_IMETHODIMP nsDocAccessibleMixin::GetDocType(nsAString& aDocType)
 {
   nsCOMPtr<nsIXULDocument> xulDoc(do_QueryInterface(mDocument));
   nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(mDocument));
@@ -631,7 +643,7 @@ NS_IMETHODIMP nsDocAccessibleMixin::GetDocType(nsAWritableString& aDocType)
   return NS_ERROR_FAILURE;
 }
 
-NS_IMETHODIMP nsDocAccessibleMixin::GetNameSpaceURIForID(PRInt16 aNameSpaceID, nsAWritableString& aNameSpaceURI)
+NS_IMETHODIMP nsDocAccessibleMixin::GetNameSpaceURIForID(PRInt16 aNameSpaceID, nsAString& aNameSpaceURI)
 {
   if (mDocument) {
     nsCOMPtr<nsINameSpaceManager> nameSpaceManager;
