@@ -43,6 +43,7 @@
 #include "nsIDocument.h"
 #include "nsINameSpaceManager.h"
 #include "nsString.h"
+#include "nsUnicharUtils.h"
 #include "nsXPIDLString.h"
 #include "nsParserCIID.h"
 #include "nsIServiceManager.h"
@@ -53,15 +54,18 @@
 #include "nsNetUtil.h"
 #include "nsEscape.h"
 #include "nsITextToSubURI.h"
+#include "nsCRT.h"
 
 static NS_DEFINE_CID(kParserServiceCID, NS_PARSERSERVICE_CID);
 
 #define kIndentStr NS_LITERAL_STRING("  ")
-#define kMozStr "_moz"
 #define kLessThan NS_LITERAL_STRING("<")
 #define kGreaterThan NS_LITERAL_STRING(">")
 #define kEndTag NS_LITERAL_STRING("</")
 
+static const PRUnichar kMozStr[] = {'m', 'o', 'z', 0};
+static const PRInt32 kMozStrLength = 3;
+
 static const PRInt32 kLongLineLen = 128;
 
 nsresult NS_NewHTMLContentSerializer(nsIContentSerializer** aSerializer)
@@ -84,6 +88,14 @@ nsHTMLContentSerializer::nsHTMLContentSerializer()
 
 nsHTMLContentSerializer::~nsHTMLContentSerializer()
 {
+  NS_ASSERTION(mOLStateStack.Count() == 0, "Expected OL State stack to be empty");
+  if (mOLStateStack.Count() > 0){
+    for (PRInt32 i = 0; i < mOLStateStack.Count(); i++){
+      olState* state = (olState*)mOLStateStack[i];
+      delete state;
+      mOLStateStack.RemoveElementAt(i);
+    }
+  }
 }
 
 nsresult
@@ -101,7 +113,7 @@ nsHTMLContentSerializer::GetParserService(nsIParserService** aParserService)
 
 NS_IMETHODIMP 
 nsHTMLContentSerializer::Init(PRUint32 aFlags, PRUint32 aWrapColumn,
-                              nsIAtom* aCharSet)
+                              nsIAtom* aCharSet, PRBool aIsCopying)
 {
   mFlags = aFlags;
   if (!aWrapColumn) {
@@ -111,6 +123,8 @@ nsHTMLContentSerializer::Init(PRUint32 aFlags, PRUint32 aWrapColumn,
     mMaxColumn = aWrapColumn;
   }
 
+  mIsCopying = aIsCopying;
+  mIsFirstChildOfOL = PR_FALSE;
   mDoFormat = (mFlags & nsIDocumentEncoder::OutputFormatted) ? PR_TRUE
                                                              : PR_FALSE;
   mBodyOnly = (mFlags & nsIDocumentEncoder::OutputBodyOnly) ? PR_TRUE
@@ -150,7 +164,7 @@ NS_IMETHODIMP
 nsHTMLContentSerializer::AppendText(nsIDOMText* aText, 
                                     PRInt32 aStartOffset,
                                     PRInt32 aEndOffset,
-                                    nsAWritableString& aStr)
+                                    nsAString& aStr)
 {
   NS_ENSURE_ARG(aText);
 
@@ -192,7 +206,7 @@ nsHTMLContentSerializer::AppendText(nsIDOMText* aText,
 }
 
 PRBool
-nsHTMLContentSerializer::IsJavaScript(nsIAtom* aAttrNameAtom, const nsAReadableString& aValueString)
+nsHTMLContentSerializer::IsJavaScript(nsIAtom* aAttrNameAtom, const nsAString& aValueString)
 {
   if (aAttrNameAtom == nsHTMLAtoms::href
   || aAttrNameAtom == nsHTMLAtoms::src) {
@@ -200,9 +214,8 @@ nsHTMLContentSerializer::IsJavaScript(nsIAtom* aAttrNameAtom, const nsAReadableS
     // this is covered in bug #59604
     static const char kJavaScript[] = "javascript";
     PRInt32 pos = aValueString.FindChar(':');
-    nsAutoString scheme;
+    const nsAutoString scheme(Substring(aValueString, 0, pos));
     if ((pos == (PRInt32)(sizeof kJavaScript - 1)) &&
-        (aValueString.Left(scheme, pos) != -1) &&
         scheme.EqualsIgnoreCase(kJavaScript))
       return PR_TRUE;
     else
@@ -231,7 +244,7 @@ nsHTMLContentSerializer::IsJavaScript(nsIAtom* aAttrNameAtom, const nsAReadableS
 }
 
 nsresult 
-nsHTMLContentSerializer::EscapeURI(const nsAReadableString& aURI, nsAWritableString& aEscapedURI)
+nsHTMLContentSerializer::EscapeURI(const nsAString& aURI, nsAString& aEscapedURI)
 {
   // URL escape %xx cannot be used in JS.
   // No escaping if the scheme is 'javascript'.
@@ -266,7 +279,7 @@ nsHTMLContentSerializer::EscapeURI(const nsAReadableString& aURI, nsAWritableStr
 
   // Loop and escape parts by avoiding escaping reserved characters (and '%', '#' ).
   while ((end = uri.FindCharInSet("%#;/?:@&=+$,", start)) != -1) {
-    aURI.Mid(part, start, (end-start));
+    part = Substring(aURI, start, (end-start));
     if (textToSubURI && !part.IsASCII()) {
       rv = textToSubURI->ConvertAndEscape(documentCharset, part.get(), getter_Copies(escapedURI));
       NS_ENSURE_SUCCESS(rv, rv);
@@ -277,14 +290,14 @@ nsHTMLContentSerializer::EscapeURI(const nsAReadableString& aURI, nsAWritableStr
     aEscapedURI.Append(NS_ConvertASCIItoUCS2(escapedURI));
 
     // Append a reserved character without escaping.
-    aURI.Mid(part, end, 1);
+    part = Substring(aURI, end, 1);
     aEscapedURI.Append(part);
     start = end + 1;
   }
 
   if (start < (PRInt32) aURI.Length()) {
     // Escape the remaining part.
-    aURI.Mid(part, start, aURI.Length()-start);
+    part = Substring(aURI, start, aURI.Length()-start);
     if (textToSubURI) {
       rv = textToSubURI->ConvertAndEscape(documentCharset, part.get(), getter_Copies(escapedURI));
       NS_ENSURE_SUCCESS(rv, rv);
@@ -301,7 +314,7 @@ nsHTMLContentSerializer::EscapeURI(const nsAReadableString& aURI, nsAWritableStr
 void 
 nsHTMLContentSerializer::SerializeAttributes(nsIContent* aContent,
                                              nsIAtom* aTagName,
-                                             nsAWritableString& aStr)
+                                             nsAString& aStr)
 {
   nsresult rv;
   PRInt32 index, count;
@@ -317,12 +330,11 @@ nsHTMLContentSerializer::SerializeAttributes(nsIContent* aContent,
                             *getter_AddRefs(attrName),
                             *getter_AddRefs(attrPrefix));
 
-    // Filter out any attribute starting with _moz
+    // Filter out any attribute starting with [-|_]moz
     const PRUnichar* sharedName;
     attrName->GetUnicode(&sharedName);
-    if (nsCRT::strncmp(sharedName, 
-                       NS_ConvertASCIItoUCS2(kMozStr).get(), 
-                       sizeof(kMozStr)-1) == 0) {
+    if ((('_' == *sharedName) || ('-' == *sharedName)) &&
+        !nsCRT::strncmp(sharedName+1, kMozStr, kMozStrLength)) {
       continue;
     }
 
@@ -334,7 +346,8 @@ nsHTMLContentSerializer::SerializeAttributes(nsIContent* aContent,
     //
     if ((aTagName == nsHTMLAtoms::br) &&
         (attrName.get() == nsHTMLAtoms::type) &&
-        (valueStr.EqualsWithConversion(kMozStr, PR_FALSE, sizeof(kMozStr)-1))) {
+        !valueStr.IsEmpty() && ('_' == valueStr[0]) &&
+        !nsCRT::strncmp(valueStr.get()+1, kMozStr, kMozStrLength)) {
       continue;
     }
     
@@ -346,6 +359,11 @@ nsHTMLContentSerializer::SerializeAttributes(nsIContent* aContent,
         continue;
     }
 
+    if ( mIsCopying && mIsFirstChildOfOL && (aTagName == nsHTMLAtoms::li) && 
+         (attrName.get() == nsHTMLAtoms::value)){
+      // This is handled separately in SerializeLIValueAttribute()
+      continue;
+    }
     PRBool isJS = IsJavaScript(attrName, valueStr);
     
     if (((attrName.get() == nsHTMLAtoms::href) || 
@@ -396,7 +414,8 @@ nsHTMLContentSerializer::SerializeAttributes(nsIContent* aContent,
 
 NS_IMETHODIMP
 nsHTMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
-                                            nsAWritableString& aStr)
+                                            PRBool aHasChildren,
+                                            nsAString& aStr)
 {
   NS_ENSURE_ARG(aElement);
   
@@ -411,6 +430,13 @@ nsHTMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
   nsCOMPtr<nsIAtom> name;
   content->GetTag(*getter_AddRefs(name));
 
+  if (name.get() == nsHTMLAtoms::br && mPreLevel > 0
+      && (mFlags & nsIDocumentEncoder::OutputNoFormattingInPre)) {
+    AppendToString(mLineBreak, aStr);
+    mColPos = 0;
+    return NS_OK;
+  }
+
   if (name.get() == nsHTMLAtoms::body) {
     mInBody = PR_TRUE;
   }
@@ -434,6 +460,40 @@ nsHTMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
   name->GetUnicode(&sharedName);
   AppendToString(sharedName, -1, aStr);
 
+  // Need to keep track of OL and LI elements in order to get ordinal number 
+  // for the LI.
+  if (mIsCopying && name.get() == nsHTMLAtoms::ol){
+    // We are copying and current node is an OL;
+    // Store it's start attribute value in olState->startVal.
+    nsAutoString start;
+    PRInt32 startAttrVal = 0;
+    aElement->GetAttribute(NS_LITERAL_STRING("start"), start);
+    if (!start.IsEmpty()){
+      PRInt32 rv = 0;
+      startAttrVal = start.ToInteger(&rv);
+      //If OL has "start" attribute, first LI element has to start with that value
+      //Therefore subtracting 1 as all the LI elements are incrementing it before using it;
+      //In failure of ToInteger(), default StartAttrValue to 0.
+      if (NS_SUCCEEDED(rv))
+        startAttrVal--; 
+      else
+        startAttrVal = 0;
+    }
+    olState* state = new olState(startAttrVal, PR_TRUE);
+    if (state)
+      mOLStateStack.AppendElement(state);
+  }
+
+  if (mIsCopying && name.get() == nsHTMLAtoms::li) {
+    mIsFirstChildOfOL = IsFirstChildOfOL(aElement);
+    if (mIsFirstChildOfOL){
+      // If OL is parent of this LI, serialize attributes in different manner.
+      SerializeLIValueAttribute(aElement, aStr);
+    }
+  }
+
+  // Even LI passed above have to go through this 
+  // for serializing attributes other than "value".
   SerializeAttributes(content, name, aStr);
 
   AppendToString(kGreaterThan, aStr);
@@ -464,7 +524,7 @@ nsHTMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
   
 NS_IMETHODIMP 
 nsHTMLContentSerializer::AppendElementEnd(nsIDOMElement *aElement,
-                                          nsAWritableString& aStr)
+                                          nsAString& aStr)
 {
   NS_ENSURE_ARG(aElement);
 
@@ -482,6 +542,17 @@ nsHTMLContentSerializer::AppendElementEnd(nsIDOMElement *aElement,
     mPreLevel--;
   }
 
+  if (mIsCopying && (name.get() == nsHTMLAtoms::ol)){
+    NS_ASSERTION((mOLStateStack.Count() > 0), "Cannot have an empty OL Stack");
+    /* Though at this point we must always have an state to be deleted as all 
+    the OL opening tags are supposed to push an olState object to the stack*/
+    if (mOLStateStack.Count() > 0) {
+      olState* state = (olState*)mOLStateStack.ElementAt(mOLStateStack.Count() -1);
+      mOLStateStack.RemoveElementAt(mOLStateStack.Count() -1);
+      delete state;
+    }
+  }
+  
   const PRUnichar* sharedName;
   name->GetUnicode(&sharedName);
 
@@ -522,7 +593,7 @@ nsHTMLContentSerializer::AppendElementEnd(nsIDOMElement *aElement,
 void
 nsHTMLContentSerializer::AppendToString(const PRUnichar* aStr,
                                         PRInt32 aLength,
-                                        nsAWritableString& aOutputStr)
+                                        nsAString& aOutputStr)
 {
   if (mBodyOnly && !mInBody) {
     return;
@@ -537,7 +608,7 @@ nsHTMLContentSerializer::AppendToString(const PRUnichar* aStr,
 
 void 
 nsHTMLContentSerializer::AppendToString(const PRUnichar aChar,
-                                        nsAWritableString& aOutputStr)
+                                        nsAString& aOutputStr)
 {
   if (mBodyOnly && !mInBody) {
     return;
@@ -549,8 +620,8 @@ nsHTMLContentSerializer::AppendToString(const PRUnichar aChar,
 }
 
 void 
-nsHTMLContentSerializer::AppendToStringWrapped(const nsAReadableString& aStr,
-                                               nsAWritableString& aOutputStr,
+nsHTMLContentSerializer::AppendToStringWrapped(const nsAString& aStr,
+                                               nsAString& aOutputStr,
                                                PRBool aTranslateEntities)
 {
   PRInt32 length = aStr.Length();
@@ -561,12 +632,6 @@ nsHTMLContentSerializer::AppendToStringWrapped(const nsAReadableString& aStr,
   PRInt32   strOffset = 0;
   PRInt32   lineLength, oldLineEnd;
   
-  // Make sure we haven't gone too far already
-  if (mColPos > mMaxColumn) {
-    AppendToString(mLineBreak, aOutputStr);
-    mColPos = 0;
-  }
-  
   // Find the end of the first old line
   oldLineEnd = aStr.FindChar(PRUnichar('\n'), 0);
   
@@ -587,12 +652,18 @@ nsHTMLContentSerializer::AppendToStringWrapped(const nsAReadableString& aStr,
 
     // if we can fill up the new line with less than what's
     // in the current old line...
-    if ((strOffset + leftInLine) < oldLineLimit) {
+    if (((strOffset + leftInLine) < oldLineLimit) || (leftInLine < 0))  {
       addLineBreak = PR_TRUE;
       
       // Look for the next word end to break
-      indx = aStr.FindChar(PRUnichar(' '), strOffset + leftInLine);
-      
+      if (leftInLine < 0) {
+        //if we have already crossed the limit of 72 columns, immediately start searching for next white space.
+        indx = aStr.FindChar(PRUnichar(' '), strOffset);
+      }
+      else {
+        //if the limit of 72 chars is not yet reached, start looking for next appropriate break only after 72 column.
+        indx = aStr.FindChar(PRUnichar(' '), strOffset + leftInLine);
+      }
       // If it's after the end of the current line, then break at
       // the current line
       if ((indx == kNotFound) || 
@@ -611,7 +682,7 @@ nsHTMLContentSerializer::AppendToStringWrapped(const nsAReadableString& aStr,
       }
       else {
         lineLength = length - strOffset;
-        aStr.Right(line, lineLength);
+        line = Substring(aStr, strOffset, lineLength);
         AppendToString(line, aOutputStr, aTranslateEntities);
       }
       done = PR_TRUE;
@@ -620,7 +691,7 @@ nsHTMLContentSerializer::AppendToStringWrapped(const nsAReadableString& aStr,
       // Add the part of the current old line that's part of the 
       // new line
       lineLength = indx - strOffset;
-      aStr.Mid(line, strOffset, lineLength);
+      line = Substring(aStr, strOffset, lineLength);
       AppendToString(line, aOutputStr, aTranslateEntities);
       
       // if we've reached the end of an old line, don't add the
@@ -661,8 +732,8 @@ static const char* kAttrEntities[] = {
 };
 
 void
-nsHTMLContentSerializer::AppendToString(const nsAReadableString& aStr,
-                                        nsAWritableString& aOutputStr,
+nsHTMLContentSerializer::AppendToString(const nsAString& aStr,
+                                        nsAString& aOutputStr,
                                         PRBool aTranslateEntities,
                                         PRBool aIncrColumn)
 {
@@ -740,8 +811,8 @@ nsHTMLContentSerializer::AppendToString(const nsAReadableString& aStr,
 }
 
 void
-nsHTMLContentSerializer::AppendToStringConvertLF(const nsAReadableString& aStr,
-                                                 nsAWritableString& aOutputStr)
+nsHTMLContentSerializer::AppendToStringConvertLF(const nsAString& aStr,
+                                                 nsAString& aOutputStr)
 {
   // Convert line-endings to mLineBreak
   PRUint32 start = 0;
@@ -924,7 +995,7 @@ nsHTMLContentSerializer::LineBreakAfterClose(nsIAtom* aName,
 void
 nsHTMLContentSerializer::StartIndentation(nsIAtom* aName,
                                           PRBool aHasDirtyAttr,
-                                          nsAWritableString& aStr)
+                                          nsAString& aStr)
 {
   if ((mDoFormat || aHasDirtyAttr) && !mPreLevel && !mColPos) {
     for (PRInt32 i = mIndent; --i >= 0; ) {
@@ -952,7 +1023,7 @@ nsHTMLContentSerializer::StartIndentation(nsIAtom* aName,
 void
 nsHTMLContentSerializer::EndIndentation(nsIAtom* aName,
                                         PRBool aHasDirtyAttr,
-                                        nsAWritableString& aStr)
+                                        nsAString& aStr)
 {
   if ((aName == nsHTMLAtoms::head) ||
       (aName == nsHTMLAtoms::table) ||
@@ -1002,3 +1073,102 @@ nsHTMLContentSerializer::HasLongLines(const nsString& text, PRInt32& aLastNewlin
   }
   return rv;
 }
+
+void 
+nsHTMLContentSerializer::SerializeLIValueAttribute(nsIDOMElement* aElement,
+                                                   nsAString& aStr)
+{
+  // We are copying and we are at the "first" LI node of OL in selected range.
+  // It may not be the first LI child of OL but it's first in the selected range.
+  // Note that we get into this condition only once per a OL.
+  nsCOMPtr<nsIDOMNode> node = do_QueryInterface(aElement);
+  PRBool found = PR_FALSE;
+  nsIDOMNode* currNode = node;
+  nsAutoString valueStr;
+  PRInt32 offset = 0;
+  olState defaultOLState(0, PR_FALSE);
+  olState* state = nsnull;
+  if (mOLStateStack.Count() > 0) 
+    state = (olState*)mOLStateStack.ElementAt(mOLStateStack.Count()-1);
+  /* Though we should never reach to a "state" as null or mOLStateStack.Count() == 0 
+  at this point as all LI are supposed to be inside some OL and OL tag should have 
+  pushed a state to the olStateStack.*/
+  if (!state || mOLStateStack.Count() == 0)
+    state = &defaultOLState;
+  PRInt32 startVal = state->startVal;
+  state->isFirstListItem = PR_FALSE;
+  // Traverse previous siblings until we find one with "value" attribute.
+  // offset keeps track of how many previous siblings we had tocurrNode traverse.
+  while (currNode && !found) {
+    nsCOMPtr<nsIDOMElement> currElement = do_QueryInterface(currNode);
+    // currElement may be null if it were a text node.
+    if (currElement) {
+      nsAutoString tagName;
+      currElement->GetTagName(tagName);
+      if (tagName.EqualsIgnoreCase("LI")) {
+        currElement->GetAttribute(NS_LITERAL_STRING("value"), valueStr);
+        if (valueStr.IsEmpty())
+          offset++;
+        else {
+          found = PR_TRUE;
+          PRInt32 rv = 0;
+          startVal = valueStr.ToInteger(&rv); 
+        }
+      }
+    }
+    currNode->GetPreviousSibling(&currNode);
+  }
+  // If LI was not having "value", Set the "value" attribute for it.
+  // Note that We are at the first LI in the selected range of OL.
+  if (offset == 0 && found) {
+    // offset = 0 => LI itself has the value attribute and we did not need to traverse back.
+    // Just serialize value attribute like other tags.
+    SerializeAttr(nsAutoString(), NS_LITERAL_STRING("value"), valueStr, aStr, PR_FALSE);
+  }
+  else if (offset == 1 && !found) {
+    /*(offset = 1 && !found) means either LI is the first child node of OL 
+    and LI is not having "value" attribute. 
+    In that case we would not like to set "value" attribute to reduce the changes.
+    */
+     //do nothing...
+  }
+  else if (offset > 0) {
+    // Set value attribute.
+    nsAutoString valueStr;
+
+    //As serializer needs to use this valueAttr we are creating here, 
+    valueStr.AppendInt(startVal + offset);
+    SerializeAttr(nsAutoString(), NS_LITERAL_STRING("value"), valueStr, aStr, PR_FALSE);
+  }
+}
+
+PRBool
+nsHTMLContentSerializer::IsFirstChildOfOL(nsIDOMElement* aElement){
+  nsCOMPtr<nsIDOMNode> node = do_QueryInterface(aElement);
+  nsIDOMNode* parentNode;
+  nsAutoString parentName;
+  node->GetParentNode(&parentNode);
+  if (parentNode)
+    parentNode->GetNodeName(parentName);
+  else
+    return PR_FALSE;
+  
+  if (parentName.EqualsIgnoreCase("OL")) {
+    olState defaultOLState(0, PR_FALSE);
+    olState* state = nsnull;
+    if (mOLStateStack.Count() > 0) 
+      state = (olState*)mOLStateStack.ElementAt(mOLStateStack.Count()-1);
+    /* Though we should never reach to a "state" as null at this point as 
+    all LI are supposed to be inside some OL and OL tag should have pushed
+    a state to the mOLStateStack.*/
+    if (!state)
+      state = &defaultOLState;
+    
+    if (state->isFirstListItem)
+      return PR_TRUE;
+
+    return PR_FALSE;
+  }
+  else
+    return PR_FALSE;
+}
