@@ -53,6 +53,9 @@
 #include "nsParserCIID.h"
 #include "nsContentUtils.h"
 #include "nsReadableUtils.h"
+#include "nsCRT.h"
+#include "nsUnicharUtils.h"
+#include "nsCRT.h"
 
 static NS_DEFINE_CID(kLWBrkCID, NS_LWBRK_CID);
 static NS_DEFINE_CID(kParserServiceCID, NS_PARSERSERVICE_CID);
@@ -148,7 +151,7 @@ NS_IMPL_ISUPPORTS4(nsPlainTextSerializer,
 
 NS_IMETHODIMP 
 nsPlainTextSerializer::Init(PRUint32 aFlags, PRUint32 aWrapColumn,
-                            nsIAtom* aCharSet)
+                            nsIAtom* aCharSet, PRBool aIsCopying)
 {
 #ifdef DEBUG
   // Check if the major control flags are set correctly.
@@ -219,10 +222,10 @@ nsPlainTextSerializer::Init(PRUint32 aFlags, PRUint32 aWrapColumn,
 }
 
 NS_IMETHODIMP
-nsPlainTextSerializer::Initialize(nsAWritableString* aOutString,
+nsPlainTextSerializer::Initialize(nsAString* aOutString,
                                   PRUint32 aFlags, PRUint32 aWrapCol)
 {
-  nsresult rv = Init(aFlags, aWrapCol, nsnull);
+  nsresult rv = Init(aFlags, aWrapCol, nsnull, PR_FALSE);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // XXX This is wrong. It violates XPCOM string ownership rules.
@@ -237,11 +240,15 @@ NS_IMETHODIMP
 nsPlainTextSerializer::AppendText(nsIDOMText* aText, 
                                   PRInt32 aStartOffset,
                                   PRInt32 aEndOffset, 
-                                  nsAWritableString& aStr)
+                                  nsAString& aStr)
 {
   if (mIgnoreAboveIndex != (PRUint32)kNotFound) {
     return NS_OK;
   }
+    
+  NS_ASSERTION(aStartOffset >= 0, "Negative start offset for text fragment!");
+  if ( aStartOffset < 0 )
+    return NS_ERROR_INVALID_ARG;
 
   NS_ENSURE_ARG(aText);
 
@@ -256,7 +263,14 @@ nsPlainTextSerializer::AppendText(nsIDOMText* aText,
   content->GetText(&frag);
 
   if (frag) {
-    length = ((aEndOffset == -1) ? frag->GetLength() : aEndOffset) - aStartOffset;
+    PRInt32 endoffset = (aEndOffset == -1) ? frag->GetLength() : aEndOffset;
+    NS_ASSERTION(aStartOffset <= endoffset, "A start offset is beyond the end of the text fragment!");
+
+    length = endoffset - aStartOffset;
+    if (length <= 0) {
+      return NS_OK;
+    }
+
     if (frag->Is2b()) {
       textstr.Assign(frag->Get2b() + aStartOffset, length);
     }
@@ -267,8 +281,6 @@ nsPlainTextSerializer::AppendText(nsIDOMText* aText,
 
   mOutputString = &aStr;
 
-  nsAutoString linebuffer;
-
   // We have to split the string across newlines
   // to match parser behavior
   PRInt32 start = 0;
@@ -277,8 +289,7 @@ nsPlainTextSerializer::AppendText(nsIDOMText* aText,
 
     if(offset>start) {
       // Pass in the line
-      textstr.Mid(linebuffer, start, offset-start);
-      rv = DoAddLeaf(eHTMLTag_text, linebuffer);
+      rv = DoAddLeaf(eHTMLTag_text, Substring(textstr, start, offset-start));
       if (NS_FAILED(rv)) break;
     }
 
@@ -293,8 +304,7 @@ nsPlainTextSerializer::AppendText(nsIDOMText* aText,
   // Consume the last bit of the string if there's any left
   if (NS_SUCCEEDED(rv) & (start < length)) {
     if (start) {
-      textstr.Mid(linebuffer, start, offset-start);
-      rv = DoAddLeaf(eHTMLTag_text, linebuffer);
+      rv = DoAddLeaf(eHTMLTag_text, Substring(textstr, start, length-start));
     }
     else {
       rv = DoAddLeaf(eHTMLTag_text, textstr);
@@ -308,7 +318,8 @@ nsPlainTextSerializer::AppendText(nsIDOMText* aText,
 
 NS_IMETHODIMP 
 nsPlainTextSerializer::AppendElementStart(nsIDOMElement *aElement,
-                                          nsAWritableString& aStr)
+                                          PRBool aHasChildren,
+                                          nsAString& aStr)
 {
   NS_ENSURE_ARG(aElement);
 
@@ -343,7 +354,7 @@ nsPlainTextSerializer::AppendElementStart(nsIDOMElement *aElement,
  
 NS_IMETHODIMP 
 nsPlainTextSerializer::AppendElementEnd(nsIDOMElement *aElement,
-                                        nsAWritableString& aStr)
+                                        nsAString& aStr)
 {
   NS_ENSURE_ARG(aElement);
 
@@ -374,7 +385,7 @@ nsPlainTextSerializer::AppendElementEnd(nsIDOMElement *aElement,
 }
 
 NS_IMETHODIMP 
-nsPlainTextSerializer::Flush(nsAWritableString& aStr)
+nsPlainTextSerializer::Flush(nsAString& aStr)
 {
   mOutputString = &aStr;
   FlushLine();
@@ -395,7 +406,7 @@ NS_IMETHODIMP
 nsPlainTextSerializer::CloseContainer(const nsIParserNode& aNode)
 {
   PRInt32 type = aNode.GetNodeType();
-  const nsAReadableString&   namestr = aNode.GetText();
+  const nsAString&   namestr = aNode.GetText();
   nsCOMPtr<nsIAtom> name = dont_AddRef(NS_NewAtom(namestr));
   
   mParserNode = NS_CONST_CAST(nsIParserNode *, &aNode);
@@ -410,7 +421,7 @@ nsPlainTextSerializer::AddLeaf(const nsIParserNode& aNode)
   }
 
   eHTMLTags type = (eHTMLTags)aNode.GetNodeType();
-  const nsAReadableString& text = aNode.GetText();
+  const nsAString& text = aNode.GetText();
 
   mParserNode = NS_CONST_CAST(nsIParserNode *, &aNode);
   if ((type == eHTMLTag_text) ||
@@ -639,15 +650,30 @@ nsPlainTextSerializer::DoOpenContainer(PRInt32 aTag)
   }
   else if (type == eHTMLTag_ol) {
     EnsureVerticalSpace(mULCount + mOLStackIndex == 0 ? 1 : 0);
-         // Must end the current line before we change indention
+    // Must end the current line before we change indention
     if (mOLStackIndex < OLStackSize) {
-      mOLStack[mOLStackIndex++] = 1;  // XXX should get it from the node!
+      nsAutoString startAttr;
+      PRInt32 startVal = 1;
+      if(NS_SUCCEEDED(GetAttributeValue(nsHTMLAtoms::start, startAttr))){
+        PRInt32 rv = 0;
+        startVal = startAttr.ToInteger(&rv);
+        if (NS_FAILED(rv))
+          startVal = 1;
+      }
+      mOLStack[mOLStackIndex++] = startVal;
     }
     mIndent += kIndentSizeList;  // see ul
   }
   else if (type == eHTMLTag_li) {
     if (mTagStackIndex > 1 && mTagStack[mTagStackIndex-2] == eHTMLTag_ol) {
       if (mOLStackIndex > 0) {
+        nsAutoString valueAttr;
+        if(NS_SUCCEEDED(GetAttributeValue(nsHTMLAtoms::value, valueAttr))){
+          PRInt32 rv = 0;
+          PRInt32 valueAttrVal = valueAttr.ToInteger(&rv);
+          if (NS_SUCCEEDED(rv))
+            mOLStack[mOLStackIndex-1] = valueAttrVal;
+        }
         // This is what nsBulletFrame does for OLs:
         mInIndentString.AppendInt(mOLStack[mOLStackIndex-1]++, 10);
       }
@@ -733,7 +759,8 @@ nsPlainTextSerializer::DoOpenContainer(PRInt32 aTag)
     nsAutoString value;
     nsresult rv = GetAttributeValue(nsHTMLAtoms::type, value);
 
-    if (NS_SUCCEEDED(rv) && value.EqualsIgnoreCase("cite")) {
+    if (NS_SUCCEEDED(rv) &&
+        NS_LossyConvertUCS2toASCII(value).Equals("cite", nsCaseInsensitiveCStringComparator())) {
       mCiteQuoteLevel++;
     }
     else {
@@ -840,7 +867,11 @@ nsPlainTextSerializer::DoCloseContainer(PRInt32 aTag)
   else if (type == eHTMLTag_span) {
     --mSpanLevel;
   }
-
+  else if (type == eHTMLTag_div) {
+    if (mFloatingLines < 0)
+      mFloatingLines = 0;
+    mLineBreakDue = PR_TRUE;
+  }
   else if (IsBlockLevel(aTag)
            && type != eHTMLTag_blockquote
            && type != eHTMLTag_script
@@ -889,7 +920,8 @@ nsPlainTextSerializer::DoCloseContainer(PRInt32 aTag)
     nsAutoString value;
     nsresult rv = GetAttributeValue(nsHTMLAtoms::type, value);
 
-    if (NS_SUCCEEDED(rv)  && value.EqualsIgnoreCase("cite")) {
+    if (NS_SUCCEEDED(rv)  &&
+        NS_LossyConvertUCS2toASCII(value).Equals("cite", nsCaseInsensitiveCStringComparator())) {
       mCiteQuoteLevel--;
     }
     else {
@@ -933,7 +965,7 @@ nsPlainTextSerializer::DoCloseContainer(PRInt32 aTag)
 
 nsresult
 nsPlainTextSerializer::DoAddLeaf(PRInt32 aTag, 
-                                 const nsAReadableString& aText)
+                                 const nsAString& aText)
 {
   // If we don't want any output, just return
   if (!DoOutput()) {
@@ -998,8 +1030,7 @@ nsPlainTextSerializer::DoAddLeaf(PRInt32 aTag,
     // we want to preserve whitespace.
     if (mFlags & nsIDocumentEncoder::OutputPreformatted ||
         (mPreFormatted && !mWrapColumn) ||
-        ((mFlags & nsIDocumentEncoder::OutputFormatted)
-         && IsInPre())) {
+        IsInPre()) {
       Write(aText);
     }
     else if(!mInWhitespace ||
@@ -1042,7 +1073,7 @@ nsPlainTextSerializer::DoAddLeaf(PRInt32 aTag,
     nsAutoString desc, temp;
     if (NS_SUCCEEDED(GetAttributeValue(nsHTMLAtoms::alt, desc))) {
       if (!desc.IsEmpty()) {
-        desc.StripChars("\"");
+        desc.StripChar(PRUnichar('"'));
         temp += desc;
       }
       // If the alt attribute has an empty value (|alt=""|), output nothing
@@ -1050,7 +1081,7 @@ nsPlainTextSerializer::DoAddLeaf(PRInt32 aTag,
     else if (NS_SUCCEEDED(GetAttributeValue(nsHTMLAtoms::title, desc))
              && !desc.IsEmpty()) {
       temp.Append(NS_LITERAL_STRING(" ["));
-      desc.StripChars("\"");
+      desc.StripChar(PRUnichar('"'));
       temp += desc;
       temp.Append(NS_LITERAL_STRING("] "));
     }
@@ -1458,7 +1489,7 @@ nsPlainTextSerializer::OutputQuotesAndIndent(PRBool stripTrailingSpaces /* = PR_
  * line wrapping, indentation, whitespace compression and other things.
  */
 void
-nsPlainTextSerializer::Write(const nsAReadableString& aString)
+nsPlainTextSerializer::Write(const nsAString& aString)
 {
 #ifdef DEBUG_wrapping
   printf("Write(%s): wrap col = %d\n",
@@ -1497,7 +1528,7 @@ nsPlainTextSerializer::Write(const nsAReadableString& aString)
         OutputQuotesAndIndent();
       }
 
-      // Find one of '\n' or '\r' using iterators since nsAReadableString
+      // Find one of '\n' or '\r' using iterators since nsAString
       // doesn't have the old FindCharInSet function.
       nsAString::const_iterator iter;           aString.BeginReading(iter);
       nsAString::const_iterator done_searching; aString.EndReading(done_searching);
@@ -1516,8 +1547,7 @@ nsPlainTextSerializer::Write(const nsAReadableString& aString)
       // Done searching
       if(newline == kNotFound) {
         // No new lines.
-        nsAutoString stringpart;
-        aString.Right(stringpart, totLen-bol);
+        nsAutoString stringpart(Substring(aString, bol, totLen - bol));
         if(!stringpart.IsEmpty()) {
           PRUnichar lastchar = stringpart[stringpart.Length()-1];
           if((lastchar == '\t') || (lastchar == ' ') ||
@@ -1535,8 +1565,7 @@ nsPlainTextSerializer::Write(const nsAReadableString& aString)
       } 
       else {
         // There is a newline
-        nsAutoString stringpart;
-        aString.Mid(stringpart, bol, newline-bol);
+        nsAutoString stringpart(Substring(aString, bol, newline-bol));
         mInWhitespace = PR_TRUE;
         Output(stringpart);
         // and write the newline
@@ -1650,10 +1679,10 @@ nsPlainTextSerializer::GetAttributeValue(nsIAtom* aName,
 
     PRInt32 count = mParserNode->GetAttributeCount();
     for (PRInt32 i=0;i<count;i++) {
-      const nsAReadableString& key = mParserNode->GetKeyAt(i);
-      if (key.Equals(name)) {
+      const nsAString& key = mParserNode->GetKeyAt(i);
+      if (key.Equals(name, nsCaseInsensitiveStringComparator())) {
         aValueRet = mParserNode->GetValueAt(i);
-        aValueRet.StripChars("\"");
+        aValueRet.StripChar(PRUnichar('"'));
         return NS_OK;
       }
     }
