@@ -52,6 +52,7 @@
 #include "nsString.h"
 #include "prprf.h"
 #include "nsUnicharUtils.h"
+#include "nsCRT.h"
 
 typedef struct {
   nsString mPrefix;
@@ -84,7 +85,7 @@ NS_IMPL_ISUPPORTS1(nsXMLContentSerializer, nsIContentSerializer)
 
 NS_IMETHODIMP 
 nsXMLContentSerializer::Init(PRUint32 flags, PRUint32 aWrapColumn,
-                             nsIAtom* aCharSet)
+                             nsIAtom* aCharSet, PRBool aIsCopying)
 {
   return NS_OK;
 }
@@ -93,7 +94,7 @@ nsresult
 nsXMLContentSerializer::AppendTextData(nsIDOMNode* aNode, 
                                        PRInt32 aStartOffset,
                                        PRInt32 aEndOffset,
-                                       nsAWritableString& aStr,
+                                       nsAString& aStr,
                                        PRBool aTranslateEntities,
                                        PRBool aIncrColumn)
 {
@@ -104,11 +105,13 @@ nsXMLContentSerializer::AppendTextData(nsIDOMNode* aNode,
   content->GetText(&frag);
 
   if (frag) {
-    PRInt32 length = ((aEndOffset == -1) ? frag->GetLength() : aEndOffset) - aStartOffset;
+    PRInt32 endoffset = (aEndOffset == -1) ? frag->GetLength() : aEndOffset;
+    PRInt32 length = endoffset - aStartOffset;
     
+    NS_ASSERTION(aStartOffset >= 0, "Negative start offset for text fragment!");
+    NS_ASSERTION(aStartOffset <= endoffset, "A start offset is beyond the end of the text fragment!");
+
     if (length <= 0) {
-      NS_ASSERTION(aStartOffset == 0,
-                   "a start offset is beyond the end of the text fragment!");
       return NS_OK;  // XXX Zero is a legal value, maybe non-zero values should
                      //     be an error.
     }
@@ -135,7 +138,7 @@ NS_IMETHODIMP
 nsXMLContentSerializer::AppendText(nsIDOMText* aText, 
                                    PRInt32 aStartOffset,
                                    PRInt32 aEndOffset,
-                                   nsAWritableString& aStr)
+                                   nsAString& aStr)
 {
   NS_ENSURE_ARG(aText);
 
@@ -146,7 +149,7 @@ NS_IMETHODIMP
 nsXMLContentSerializer::AppendCDATASection(nsIDOMCDATASection* aCDATASection,
                                            PRInt32 aStartOffset,
                                            PRInt32 aEndOffset,
-                                           nsAWritableString& aStr)
+                                           nsAString& aStr)
 {
   NS_ENSURE_ARG(aCDATASection);
   nsresult rv;
@@ -163,7 +166,7 @@ NS_IMETHODIMP
 nsXMLContentSerializer::AppendProcessingInstruction(nsIDOMProcessingInstruction* aPI,
                                                     PRInt32 aStartOffset,
                                                     PRInt32 aEndOffset,
-                                                    nsAWritableString& aStr)
+                                                    nsAString& aStr)
 {
   NS_ENSURE_ARG(aPI);
   nsresult rv;
@@ -190,7 +193,7 @@ NS_IMETHODIMP
 nsXMLContentSerializer::AppendComment(nsIDOMComment* aComment,
                                       PRInt32 aStartOffset,
                                       PRInt32 aEndOffset,
-                                      nsAWritableString& aStr)
+                                      nsAString& aStr)
 {
   NS_ENSURE_ARG(aComment);
   nsresult rv;
@@ -218,7 +221,7 @@ nsXMLContentSerializer::AppendComment(nsIDOMComment* aComment,
 
 NS_IMETHODIMP 
 nsXMLContentSerializer::AppendDoctype(nsIDOMDocumentType *aDoctype,
-                                      nsAWritableString& aStr)
+                                      nsAString& aStr)
 {
   NS_ENSURE_ARG(aDoctype);
   nsresult rv;
@@ -287,8 +290,8 @@ nsXMLContentSerializer::AppendDoctype(nsIDOMDocumentType *aDoctype,
 #define kXMLNS NS_LITERAL_STRING("xmlns")
 
 nsresult
-nsXMLContentSerializer::PushNameSpaceDecl(const nsAReadableString& aPrefix,
-                                          const nsAReadableString& aURI,
+nsXMLContentSerializer::PushNameSpaceDecl(const nsAString& aPrefix,
+                                          const nsAString& aURI,
                                           nsIDOMElement* aOwner)
 {
   NameSpaceDecl* decl = new NameSpaceDecl();
@@ -324,8 +327,8 @@ nsXMLContentSerializer::PopNameSpaceDeclsFor(nsIDOMElement* aOwner)
  * moved/modified elements/attributes 
  */
 PRBool
-nsXMLContentSerializer::ConfirmPrefix(nsAWritableString& aPrefix,
-                                      const nsAReadableString& aURI)
+nsXMLContentSerializer::ConfirmPrefix(nsAString& aPrefix,
+                                      const nsAString& aURI)
 {
   if (aPrefix.Equals(kXMLNS)) {
     return PR_FALSE;
@@ -380,10 +383,10 @@ nsXMLContentSerializer::ConfirmPrefix(nsAWritableString& aPrefix,
 }
 
 void
-nsXMLContentSerializer::SerializeAttr(const nsAReadableString& aPrefix,
-                                      const nsAReadableString& aName,
-                                      const nsAReadableString& aValue,
-                                      nsAWritableString& aStr,
+nsXMLContentSerializer::SerializeAttr(const nsAString& aPrefix,
+                                      const nsAString& aName,
+                                      const nsAString& aValue,
+                                      nsAString& aStr,
                                       PRBool aDoEscapeEntities)
 {
   AppendToString(PRUnichar(' '), aStr);
@@ -404,7 +407,8 @@ nsXMLContentSerializer::SerializeAttr(const nsAReadableString& aPrefix,
 
 NS_IMETHODIMP 
 nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
-                                           nsAWritableString& aStr)
+                                           PRBool aHasChildren,
+                                           nsAString& aStr)
 {
   NS_ENSURE_ARG(aElement);
 
@@ -419,10 +423,7 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
   aElement->GetLocalName(tagLocalName);
   aElement->GetNamespaceURI(tagNamespaceURI);
 
-  PRInt32 namespaceID, elementNamespaceID;
-  content->GetNameSpaceID(elementNamespaceID);
-  if (elementNamespaceID == kNameSpaceID_HTML)
-    ToLowerCase(tagLocalName); // XXX We shouldn't need this hack
+  PRInt32 namespaceID;
     
   PRInt32 index, count;
   nsAutoString nameStr, prefixStr, uriStr, valueStr;
@@ -438,26 +439,15 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
                            *getter_AddRefs(attrName),
                            *getter_AddRefs(attrPrefix));
     
-    if (namespaceID == kNameSpaceID_XMLNS ||
-      elementNamespaceID == kNameSpaceID_HTML /*XXX Hack*/) {
+    if (namespaceID == kNameSpaceID_XMLNS) {
       PRBool hasPrefix = attrPrefix ? PR_TRUE : PR_FALSE;
       content->GetAttr(namespaceID, attrName, uriStr);
 
-      attrName->ToString(nameStr);
-      // XXX We shouldn't need this hack
-      if (elementNamespaceID == kNameSpaceID_HTML) {
-        if (Substring(nameStr, 0, 6).Equals(NS_LITERAL_STRING("xmlns:"))) {
-          nameStr.Cut(0,6);
-          hasPrefix = PR_TRUE;
-        } else if (!nameStr.Equals(kXMLNS)) {
-          continue;
-        }
-      }
-
-      if (!hasPrefix) {
+      if (!attrPrefix) {
         // Default NS attribute does not have prefix (and the name is "xmlns")
         PushNameSpaceDecl(nsString(), uriStr, aElement);
       } else {
+        attrName->ToString(nameStr);
         PushNameSpaceDecl(nameStr, uriStr, aElement);
       }
     }
@@ -468,8 +458,7 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
   addNSAttr = ConfirmPrefix(tagPrefix, tagNamespaceURI);
   // Serialize the qualified name of the element
   AppendToString(NS_LITERAL_STRING("<"), aStr);
-  if (!tagPrefix.IsEmpty() &&
-    !(elementNamespaceID == kNameSpaceID_HTML && tagPrefix.Equals(kXMLNS) /*XXX Hack*/)) {
+  if (!tagPrefix.IsEmpty()) {
     AppendToString(tagPrefix, aStr);
     AppendToString(NS_LITERAL_STRING(":"), aStr);
   }
@@ -497,6 +486,7 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
                            namespaceID,
                            *getter_AddRefs(attrName),
                            *getter_AddRefs(attrPrefix));
+
     if (attrPrefix) {
       attrPrefix->ToString(prefixStr);
     }
@@ -512,9 +502,13 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
     
     content->GetAttr(namespaceID, attrName, valueStr);
     attrName->ToString(nameStr);
-    
-    if (elementNamespaceID == kNameSpaceID_HTML && nameStr.Equals(NS_LITERAL_STRING("xmlns:xmlns")))
-      nameStr.Assign(kXMLNS); // XXX Shouldn't need this hack, breaks case where there really is xmlns:xmlns
+
+    // XXX Hack to get around the fact that MathML can add
+    //     attributes starting with '-', which makes them
+    //     invalid XML.
+    if (!nameStr.IsEmpty() && nameStr.First() == '-')
+      continue;
+
     SerializeAttr(prefixStr, nameStr, valueStr, aStr, PR_TRUE);
     
     if (addNSAttr) {
@@ -524,9 +518,7 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
   }
 
   // We don't output a separate end tag for empty element
-  nsCOMPtr<nsIDOMNode> node(do_QueryInterface(aElement));
-  PRBool hasChildren;
-  if (NS_SUCCEEDED(node->HasChildNodes(&hasChildren)) && !hasChildren) {
+  if (!aHasChildren) {
     AppendToString(NS_LITERAL_STRING("/>"), aStr);    
   } else {
     AppendToString(NS_LITERAL_STRING(">"), aStr);    
@@ -537,7 +529,7 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
 
 NS_IMETHODIMP 
 nsXMLContentSerializer::AppendElementEnd(nsIDOMElement *aElement,
-                                         nsAWritableString& aStr)
+                                         nsAString& aStr)
 {
   NS_ENSURE_ARG(aElement);
 
@@ -557,16 +549,10 @@ nsXMLContentSerializer::AppendElementEnd(nsIDOMElement *aElement,
   aElement->GetLocalName(tagLocalName);
   aElement->GetNamespaceURI(tagNamespaceURI);
 
-  PRInt32 namespaceID;
-  content->GetNameSpaceID(namespaceID);
-  if (namespaceID == kNameSpaceID_HTML)
-    ToLowerCase(tagLocalName); // XXX We shouldn't need this hack
-
   ConfirmPrefix(tagPrefix, tagNamespaceURI);
 
   AppendToString(NS_LITERAL_STRING("</"), aStr);
-  if (!tagPrefix.IsEmpty() &&
-    !(namespaceID == kNameSpaceID_HTML && tagPrefix.Equals(kXMLNS) /*XXX Hack*/)) {
+  if (!tagPrefix.IsEmpty()) {
     AppendToString(tagPrefix, aStr);
     AppendToString(NS_LITERAL_STRING(":"), aStr);
   }
@@ -581,7 +567,7 @@ nsXMLContentSerializer::AppendElementEnd(nsIDOMElement *aElement,
 void
 nsXMLContentSerializer::AppendToString(const PRUnichar* aStr,
                                        PRInt32 aLength,
-                                       nsAWritableString& aOutputStr)
+                                       nsAString& aOutputStr)
 {
   PRInt32 length = (aLength == -1) ? nsCRT::strlen(aStr) : aLength;
   
@@ -590,7 +576,7 @@ nsXMLContentSerializer::AppendToString(const PRUnichar* aStr,
 
 void 
 nsXMLContentSerializer::AppendToString(const PRUnichar aChar,
-                                       nsAWritableString& aOutputStr)
+                                       nsAString& aOutputStr)
 {
   aOutputStr.Append(aChar);
 }
@@ -617,8 +603,8 @@ static const char* kAttrEntities[] = {
 };
 
 void
-nsXMLContentSerializer::AppendToString(const nsAReadableString& aStr,
-                                       nsAWritableString& aOutputStr,
+nsXMLContentSerializer::AppendToString(const nsAString& aStr,
+                                       nsAString& aOutputStr,
                                        PRBool aTranslateEntities,
                                        PRBool aIncrColumn)
 {
