@@ -61,6 +61,7 @@
 #include "nsDOMError.h"
 #include "nsContentUtils.h"
 #include "nsHashtable.h"
+#include "nsDoubleHashtable.h"
 #include "nsContentList.h"
 #include "nsGUIEvent.h"
 
@@ -74,7 +75,15 @@
 #include "nsRange.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsNetUtil.h"
+#include "nsIWebProgress.h"
+#include "nsIDocShell.h"
+#include "nsIWebProgressListener.h"
+#include "nsWeakReference.h"
 
+// radio buttons
+#include "nsIDOMHTMLInputElement.h"
+#include "nsIRadioControlElement.h"
+#include "nsIRadioVisitor.h"
 
 static const int NS_FORM_CONTROL_LIST_HASHTABLE_SIZE = 16;
 
@@ -83,14 +92,25 @@ class nsFormControlList;
 // nsHTMLFormElement
 
 class nsHTMLFormElement : public nsGenericHTMLContainerElement,
+                          public nsSupportsWeakReference,
                           public nsIDOMHTMLFormElement,
                           public nsIDOMNSHTMLFormElement,
+                          public nsIWebProgressListener,
                           public nsIForm
 {
 public:
-  nsHTMLFormElement();
+  nsHTMLFormElement() :
+    mGeneratingSubmit(PR_FALSE),
+    mGeneratingReset(PR_FALSE),
+    mDemotingForm(PR_FALSE),
+    mIsSubmitting(PR_FALSE),
+    mSubmittingRequest(nsnull) { }
+
+
   virtual ~nsHTMLFormElement();
 
+  virtual nsresult Init(nsINodeInfo* aNodeInfo);
+
   // nsISupports
   NS_DECL_ISUPPORTS_INHERITED
 
@@ -109,20 +129,28 @@ public:
   // nsIDOMNSHTMLFormElement
   NS_DECL_NSIDOMNSHTMLFORMELEMENT  
 
+  // nsIWebProgressListener
+  NS_DECL_NSIWEBPROGRESSLISTENER
+
   // nsIForm
   NS_IMETHOD AddElement(nsIFormControl* aElement);
   NS_IMETHOD AddElementToTable(nsIFormControl* aChild,
-                               const nsAReadableString& aName);
+                               const nsAString& aName);
   NS_IMETHOD GetElementAt(PRInt32 aIndex, nsIFormControl** aElement) const;
   NS_IMETHOD GetElementCount(PRUint32* aCount) const;
   NS_IMETHOD RemoveElement(nsIFormControl* aElement);
   NS_IMETHOD RemoveElementFromTable(nsIFormControl* aElement,
-                                    const nsAReadableString& aName);
-  NS_IMETHOD ResolveName(const nsAReadableString& aName,
-                         nsISupports **aReturn);
+                                    const nsAString& aName);
+  NS_IMETHOD ResolveName(const nsAString& aName,
+                         nsISupports** aReturn);
   NS_IMETHOD IndexOfControl(nsIFormControl* aControl, PRInt32* aIndex);
   NS_IMETHOD SetDemotingForm(PRBool aDemotingForm);
   NS_IMETHOD IsDemotingForm(PRBool* aDemotingForm);
+  NS_IMETHOD SetCurrentRadioButton(const nsAString& aName,
+                                   nsIDOMHTMLInputElement* aRadio);
+  NS_IMETHOD GetCurrentRadioButton(const nsAString& aName,
+                                   nsIDOMHTMLInputElement** aRadio);
+  NS_IMETHOD WalkRadioGroup(const nsAString& aName, nsIRadioVisitor* aVisitor);
 
 #ifdef DEBUG
   NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
@@ -130,14 +158,17 @@ public:
 
   // nsIContent
   NS_IMETHOD StringToAttribute(nsIAtom* aAttribute,
-                               const nsAReadableString& aValue,
+                               const nsAString& aValue,
                                nsHTMLValue& aResult);
   NS_IMETHOD AttributeToString(nsIAtom* aAttribute,
                                const nsHTMLValue& aValue,
-                               nsAWritableString& aResult) const;
+                               nsAString& aResult) const;
   NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
                             nsIDOMEvent** aDOMEvent, PRUint32 aFlags,
                             nsEventStatus* aEventStatus);
+  NS_IMETHOD SetDocument(nsIDocument* aDocument, PRBool aDeep,
+                         PRBool aCompileEventHandlers);
+
 protected:
   nsresult DoSubmitOrReset(nsIPresContext* aPresContext,
                            nsEvent* aEvent,
@@ -196,9 +227,12 @@ protected:
   // Data members
   //
   nsFormControlList *mControls;
+  nsDoubleHashtableStringSupports mSelectedRadioButtons;
   PRPackedBool mGeneratingSubmit;
   PRPackedBool mGeneratingReset;
   PRPackedBool mDemotingForm;
+  PRPackedBool mIsSubmitting;
+  nsCOMPtr<nsIRequest> mSubmittingRequest;
 
 protected:
   // Detection of first form to notify observers
@@ -227,13 +261,13 @@ public:
   // nsIDOMNSHTMLFormControlList interface
   NS_DECL_NSIDOMNSHTMLFORMCONTROLLIST
 
-  nsresult GetNamedObject(const nsAReadableString& aName,
+  nsresult GetNamedObject(const nsAString& aName,
                           nsISupports **aResult);
 
   nsresult AddElementToTable(nsIFormControl* aChild,
-                             const nsAReadableString& aName);
+                             const nsAString& aName);
   nsresult RemoveElementFromTable(nsIFormControl* aChild,
-                                  const nsAReadableString& aName);
+                                  const nsAString& aName);
   nsresult IndexOfControl(nsIFormControl* aControl,
                           PRInt32* aIndex);
 
@@ -335,14 +369,22 @@ NS_NewHTMLFormElement(nsIHTMLContent** aInstancePtrResult,
 }
 
 
-nsHTMLFormElement::nsHTMLFormElement():
-  mGeneratingSubmit(PR_FALSE),
-  mGeneratingReset(PR_FALSE),
-  mDemotingForm(PR_FALSE)
+nsresult
+nsHTMLFormElement::Init(nsINodeInfo *aNodeInfo)
 {
-  mControls = new nsFormControlList(this);
+  nsresult rv = nsGenericHTMLElement::Init(aNodeInfo);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  NS_IF_ADDREF(mControls);
+  mControls = new nsFormControlList(this);
+  if (!mControls) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+  NS_ADDREF(mControls);
+
+  rv = mSelectedRadioButtons.Init();
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return NS_OK;
 }
 
 nsHTMLFormElement::~nsHTMLFormElement()
@@ -367,9 +409,11 @@ NS_IMPL_RELEASE_INHERITED(nsHTMLFormElement, nsGenericElement)
 // QueryInterface implementation for nsHTMLFormElement
 NS_HTML_CONTENT_INTERFACE_MAP_BEGIN(nsHTMLFormElement,
                                     nsGenericHTMLContainerElement)
+  NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
   NS_INTERFACE_MAP_ENTRY(nsIDOMHTMLFormElement)
   NS_INTERFACE_MAP_ENTRY(nsIDOMNSHTMLFormElement)
   NS_INTERFACE_MAP_ENTRY(nsIForm)
+  NS_INTERFACE_MAP_ENTRY(nsIWebProgressListener)
   NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(HTMLFormElement)
 NS_HTML_CONTENT_INTERFACE_MAP_END
 
@@ -411,7 +455,7 @@ nsHTMLFormElement::GetElements(nsIDOMHTMLCollection** aElements)
 }
 
 NS_IMETHODIMP
-nsHTMLFormElement::GetName(nsAWritableString& aValue)
+nsHTMLFormElement::GetName(nsAString& aValue)
 {
   return nsGenericHTMLContainerElement::GetAttr(kNameSpaceID_HTML,
                                                 nsHTMLAtoms::name,
@@ -419,7 +463,7 @@ nsHTMLFormElement::GetName(nsAWritableString& aValue)
 }
 
 NS_IMETHODIMP
-nsHTMLFormElement::SetName(const nsAReadableString& aValue)
+nsHTMLFormElement::SetName(const nsAString& aValue)
 {
   return nsGenericHTMLContainerElement::SetAttr(kNameSpaceID_HTML,
                                                 nsHTMLAtoms::name,
@@ -432,7 +476,7 @@ NS_IMPL_STRING_ATTR(nsHTMLFormElement, Enctype, enctype)
 NS_IMPL_STRING_ATTR(nsHTMLFormElement, Method, method)
 
 NS_IMETHODIMP
-nsHTMLFormElement::GetTarget(nsAWritableString& aValue)
+nsHTMLFormElement::GetTarget(nsAString& aValue)
 {
   aValue.Truncate();
   nsresult rv = nsGenericHTMLContainerElement::GetAttr(kNameSpaceID_HTML,
@@ -492,7 +536,7 @@ static nsGenericHTMLElement::EnumTable kFormEnctypeTable[] = {
 
 NS_IMETHODIMP
 nsHTMLFormElement::StringToAttribute(nsIAtom* aAttribute,
-                                     const nsAReadableString& aValue,
+                                     const nsAString& aValue,
                                      nsHTMLValue& aResult)
 {
   if (aAttribute == nsHTMLAtoms::method) {
@@ -511,7 +555,7 @@ nsHTMLFormElement::StringToAttribute(nsIAtom* aAttribute,
 NS_IMETHODIMP
 nsHTMLFormElement::AttributeToString(nsIAtom* aAttribute,
                                      const nsHTMLValue& aValue,
-                                     nsAWritableString& aResult) const
+                                     nsAString& aResult) const
 {
   if (aAttribute == nsHTMLAtoms::method) {
     if (eHTMLUnit_Enumerated == aValue.GetUnit()) {
@@ -530,6 +574,28 @@ nsHTMLFormElement::AttributeToString(nsIAtom* aAttribute,
                                                           aValue, aResult);
 }
 
+NS_IMETHODIMP
+nsHTMLFormElement::SetDocument(nsIDocument* aDocument, PRBool aDeep,
+                               PRBool aCompileEventHandlers)
+{
+  nsCOMPtr<nsIHTMLDocument> oldDocument = do_QueryInterface(mDocument);
+  nsresult rv = nsGenericHTMLContainerElement::SetDocument(aDocument, aDeep,
+                                                           aCompileEventHandlers);
+  NS_ENSURE_SUCCESS(rv, rv);
+  
+  nsCOMPtr<nsIHTMLDocument> newDocument = do_QueryInterface(mDocument);
+  if (oldDocument != newDocument) {
+    if (oldDocument) {
+      oldDocument->RemovedForm();
+    }
+    if (newDocument) {
+      newDocument->AddedForm();
+    }
+  }
+  return rv;
+}
+
+
 NS_IMETHODIMP
 nsHTMLFormElement::HandleDOMEvent(nsIPresContext* aPresContext,
                                   nsEvent* aEvent,
@@ -626,9 +692,26 @@ nsHTMLFormElement::DoReset()
   return NS_OK;
 }
 
+#define NS_ENSURE_SUBMIT_SUCCESS(rv) \
+  if (NS_FAILED(rv)) { \
+    mIsSubmitting = PR_FALSE; \
+    return rv; \
+  }
+  
 nsresult
 nsHTMLFormElement::DoSubmit(nsIPresContext* aPresContext, nsEvent* aEvent)
 {
+  NS_ASSERTION(!mIsSubmitting, "Either two people are trying to submit or the "
+               "previous submit was not properly cancelled by the DocShell");
+  if (mIsSubmitting) {
+    // XXX Should this return an error?
+    return NS_OK;
+  }
+
+  // Mark us as submitting so that we don't try to submit again
+  mIsSubmitting = PR_TRUE;
+  mSubmittingRequest = nsnull;
+
   nsIContent *originatingElement = nsnull;
 
   // Get the originating frame (failure is non-fatal)
@@ -644,39 +727,74 @@ nsHTMLFormElement::DoSubmit(nsIPresContext* aPresContext, nsEvent* aEvent)
   nsCOMPtr<nsIFormSubmission> submission;
   nsresult rv = GetSubmissionFromForm(this, aPresContext,
                                       getter_AddRefs(submission));
-  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_SUBMIT_SUCCESS(rv);
 
   //
   // Dump the data into the submission object
   //
   rv = WalkFormElements(submission, originatingElement);
-  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_SUBMIT_SUCCESS(rv);
 
   //
   // Get the action and target
   //
   nsCOMPtr<nsIURI> actionURI;
   rv = GetActionURL(getter_AddRefs(actionURI));
-  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_SUBMIT_SUCCESS(rv);
 
-  if (actionURI) {
-    nsAutoString target;
-    rv = GetTarget(target);
-    NS_ENSURE_SUCCESS(rv, rv);
+  if (!actionURI) {
+    mIsSubmitting = PR_FALSE;
+    return NS_OK;
+  }
 
-    //
-    // Notify observers of submit
-    //
-    PRBool aCancelSubmit = PR_FALSE;
-    rv = NotifySubmitObservers(actionURI, &aCancelSubmit);
-    NS_ENSURE_SUCCESS(rv, rv);
+  // javascript URIs are not really submissions; they just call a function.
+  // Also, they may synchronously call submit(), and we want them to be able to
+  // do so while still disallowing other double submissions. (Bug 139798)
+  // Note that any other URI types that are of equivalent type should also be
+  // added here.
+  PRBool schemeIsJavaScript = PR_FALSE;
+  if (NS_SUCCEEDED(actionURI->SchemeIs("javascript", &schemeIsJavaScript)) &&
+      schemeIsJavaScript) {
+    mIsSubmitting = PR_FALSE;
+  }
 
-    if (!aCancelSubmit) {
-      //
-      // Submit
-      //
-      rv = submission->SubmitTo(actionURI, target, this, aPresContext);
-    }
+  nsAutoString target;
+  rv = GetTarget(target);
+  NS_ENSURE_SUBMIT_SUCCESS(rv);
+
+  //
+  // Notify observers of submit
+  //
+  PRBool aCancelSubmit = PR_FALSE;
+  rv = NotifySubmitObservers(actionURI, &aCancelSubmit);
+  NS_ENSURE_SUBMIT_SUCCESS(rv);
+
+  if (aCancelSubmit) {
+    mIsSubmitting = PR_FALSE;
+    return NS_OK;
+  }
+
+  //
+  // Submit
+  //
+  nsCOMPtr<nsIDocShell> docShell;
+  rv = submission->SubmitTo(actionURI, target, this, aPresContext,
+                            getter_AddRefs(docShell),
+                            getter_AddRefs(mSubmittingRequest));
+  NS_ENSURE_SUBMIT_SUCCESS(rv);
+
+  // Even if the submit succeeds, it's possible for there to be no docshell
+  // or request; for example, if it's to a named anchor within the same page
+  // the submit will not really do anything.
+  if (docShell) {
+    nsCOMPtr<nsIWebProgress> webProgress = do_GetInterface(docShell);
+    NS_ASSERTION(webProgress, "nsIDocShell not converted to nsIWebProgress!");
+    rv = webProgress->AddProgressListener(this, nsIWebProgress::NOTIFY_STATE_ALL);
+    NS_ENSURE_SUBMIT_SUCCESS(rv);
+  }
+  else {
+	  // in case we didn't do anything, reset mIsSubmitting
+	  mIsSubmitting = PR_FALSE;
   }
 
   return rv;
@@ -990,12 +1108,23 @@ nsHTMLFormElement::AddElement(nsIFormControl* aChild)
     }
   }
 
+  //
+  // Notify the radio button it's been added to a group
+  //
+  PRInt32 type;
+  aChild->GetType(&type);
+  if (type == NS_FORM_INPUT_RADIO) {
+    nsCOMPtr<nsIRadioControlElement> radio = do_QueryInterface(aChild);
+    nsresult rv = radio->AddedToRadioGroup();
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsHTMLFormElement::AddElementToTable(nsIFormControl* aChild,
-                                     const nsAReadableString& aName)
+                                     const nsAString& aName)
 {
   NS_ENSURE_TRUE(mControls, NS_ERROR_UNEXPECTED);
 
@@ -1016,12 +1145,23 @@ nsHTMLFormElement::RemoveElement(nsIFormControl* aChild)
     mControls->mNotInElements->Remove(&key);
   }
 
+  //
+  // Remove it from the radio group if it's a radio button
+  //
+  PRInt32 type;
+  aChild->GetType(&type);
+  if (type == NS_FORM_INPUT_RADIO) {
+    nsCOMPtr<nsIRadioControlElement> radio = do_QueryInterface(aChild);
+    nsresult rv = radio->RemovedFromRadioGroup(this, nsnull);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsHTMLFormElement::RemoveElementFromTable(nsIFormControl* aElement,
-                                          const nsAReadableString& aName)
+                                          const nsAString& aName)
 {
   NS_ENSURE_TRUE(mControls, NS_ERROR_UNEXPECTED);
 
@@ -1029,7 +1169,7 @@ nsHTMLFormElement::RemoveElementFromTable(nsIFormControl* aElement,
 }
 
 NS_IMETHODIMP
-nsHTMLFormElement::ResolveName(const nsAReadableString& aName,
+nsHTMLFormElement::ResolveName(const nsAString& aName,
                                nsISupports **aResult)
 {
   return mControls->GetNamedObject(aName, aResult);
@@ -1037,13 +1177,13 @@ nsHTMLFormElement::ResolveName(const nsAReadableString& aName,
 
 
 NS_IMETHODIMP
-nsHTMLFormElement::GetEncoding(nsAWritableString& aEncoding)
+nsHTMLFormElement::GetEncoding(nsAString& aEncoding)
 {
   return GetEnctype(aEncoding);
 }
  
 NS_IMETHODIMP
-nsHTMLFormElement::SetEncoding(const nsAReadableString& aEncoding)
+nsHTMLFormElement::SetEncoding(const nsAString& aEncoding)
 {
   return SetEnctype(aEncoding);
 }
@@ -1056,6 +1196,73 @@ nsHTMLFormElement::GetLength(PRInt32* aLength)
   return NS_OK;
 }
 
+// nsIWebProgressListener
+NS_IMETHODIMP
+nsHTMLFormElement::OnStateChange(nsIWebProgress* aWebProgress,
+                                 nsIRequest* aRequest,
+                                 PRUint32 aStateFlags,
+                                 PRUint32 aStatus)
+{
+  // If STATE_STOP is never fired for any reason (redirect?  Failed state
+  // change?) the form element will leak.  It will be kept around by the
+  // nsIWebProgressListener (assuming it keeps a strong pointer).  We will
+  // consequently leak the request.
+  if (aRequest == mSubmittingRequest &&
+      aStateFlags & nsIWebProgressListener::STATE_STOP) {
+    mIsSubmitting = PR_FALSE;
+    mSubmittingRequest = nsnull;
+    aWebProgress->RemoveProgressListener(this);
+  }
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsHTMLFormElement::OnProgressChange(nsIWebProgress* aWebProgress,
+                                    nsIRequest* aRequest,
+                                    PRInt32 aCurSelfProgress,
+                                    PRInt32 aMaxSelfProgress,
+                                    PRInt32 aCurTotalProgress,
+                                    PRInt32 aMaxTotalProgress)
+{
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsHTMLFormElement::OnLocationChange(nsIWebProgress* aWebProgress,
+                                    nsIRequest* aRequest,
+                                    nsIURI* location)
+{
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsHTMLFormElement::OnStatusChange(nsIWebProgress* aWebProgress,
+                                  nsIRequest* aRequest,
+                                  nsresult aStatus,
+                                  const PRUnichar* aMessage)
+{
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsHTMLFormElement::OnSecurityChange(nsIWebProgress* aWebProgress,
+                                    nsIRequest* aRequest,
+                                    PRUint32 state)
+{
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
+  return NS_OK;
+}
+
+
+
+
+
+
+
 NS_IMETHODIMP
 nsHTMLFormElement::IndexOfControl(nsIFormControl* aControl, PRInt32* aIndex)
 {
@@ -1083,8 +1290,111 @@ nsHTMLFormElement::IsDemotingForm(PRBool* aDemotingForm)
   return NS_OK;
 }
 
-//----------------------------------------------------------------------
+NS_IMETHODIMP
+nsHTMLFormElement::SetCurrentRadioButton(const nsAString& aName,
+                                         nsIDOMHTMLInputElement* aRadio)
+{
+  return mSelectedRadioButtons.Put(aName, (nsISupports*)aRadio);
+}
 
+NS_IMETHODIMP
+nsHTMLFormElement::GetCurrentRadioButton(const nsAString& aName,
+                                         nsIDOMHTMLInputElement** aRadio)
+{
+  *aRadio = (nsIDOMHTMLInputElement*)mSelectedRadioButtons.Get(aName).get();
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsHTMLFormElement::WalkRadioGroup(const nsAString& aName,
+                                  nsIRadioVisitor* aVisitor)
+{
+  nsresult rv = NS_OK;
+
+  PRBool stopIterating = PR_FALSE;
+
+  if (aName.IsEmpty()) {
+    //
+    // XXX If the name is empty, it's not stored in the control list.  There
+    // *must* be a more efficient way to do this.
+    //
+    nsCOMPtr<nsIFormControl> control;
+    PRUint32 len = 0;
+    GetElementCount(&len);
+    for (PRUint32 i=0; i<len; i++) {
+      GetElementAt(i, getter_AddRefs(control));
+      PRInt32 type;
+      control->GetType(&type);
+      if (type == NS_FORM_INPUT_RADIO) {
+        nsCOMPtr<nsIDOMHTMLInputElement> elem(do_QueryInterface(control));
+        if (elem) {
+          //
+          // XXX This is a particularly frivolous string copy just to determine
+          // if the string is empty or not
+          //
+          nsAutoString name;
+          elem->GetName(name);
+          if (name.IsEmpty()) {
+            aVisitor->Visit(control, &stopIterating);
+            if (stopIterating) {
+              break;
+            }
+          }
+        }
+      }
+    }
+  } else {
+    //
+    // Get the control / list of controls from the form using form["name"]
+    //
+    nsCOMPtr<nsISupports> item;
+    rv = ResolveName(aName, getter_AddRefs(item));
+
+    if (item) {
+      //
+      // If it's just a lone radio button, then select it.
+      //
+      nsCOMPtr<nsIFormControl> formControl(do_QueryInterface(item));
+      if (formControl) {
+        PRInt32 type;
+        formControl->GetType(&type);
+        if (type == NS_FORM_INPUT_RADIO) {
+          aVisitor->Visit(formControl, &stopIterating);
+        }
+      } else {
+        //
+        // If it's a list, we have to find the first non-disabled one and
+        // select it.
+        //
+        nsCOMPtr<nsIDOMNodeList> nodeList(do_QueryInterface(item));
+        if (nodeList) {
+          PRUint32 length = 0;
+          nodeList->GetLength(&length);
+          for (PRUint32 i=0; i<length; i++) {
+            nsCOMPtr<nsIDOMNode> node;
+            nodeList->Item(i, getter_AddRefs(node));
+            nsCOMPtr<nsIFormControl> formControl(do_QueryInterface(node));
+            if (formControl) {
+              PRInt32 type;
+              formControl->GetType(&type);
+              if (type == NS_FORM_INPUT_RADIO) {
+                aVisitor->Visit(formControl, &stopIterating);
+                if (stopIterating) {
+                  break;
+                }
+              }
+            }
+          }
+        }
+      }
+    }
+  }
+
+  return rv;
+}
+
+
+//----------------------------------------------------------------------
 // nsFormControlList implementation, this could go away if there were
 // a lightweight collection implementation somewhere
 
@@ -1182,7 +1492,7 @@ nsFormControlList::Item(PRUint32 aIndex, nsIDOMNode** aReturn)
 }
 
 nsresult
-nsFormControlList::GetNamedObject(const nsAReadableString& aName,
+nsFormControlList::GetNamedObject(const nsAString& aName,
                                   nsISupports** aResult)
 {
   *aResult = nsnull;
@@ -1201,7 +1511,7 @@ nsFormControlList::GetNamedObject(const nsAReadableString& aName,
 }
 
 NS_IMETHODIMP 
-nsFormControlList::NamedItem(const nsAReadableString& aName,
+nsFormControlList::NamedItem(const nsAString& aName,
                              nsIDOMNode** aReturn)
 {
   NS_ENSURE_ARG_POINTER(aReturn);
@@ -1234,7 +1544,7 @@ nsFormControlList::NamedItem(const nsAReadableString& aName,
 }
 
 NS_IMETHODIMP
-nsFormControlList::NamedItem(const nsAReadableString& aName,
+nsFormControlList::NamedItem(const nsAString& aName,
                              nsISupports** aReturn)
 {
   NS_ENSURE_ARG_POINTER(aReturn);
@@ -1248,7 +1558,7 @@ nsFormControlList::NamedItem(const nsAReadableString& aName,
 
 nsresult
 nsFormControlList::AddElementToTable(nsIFormControl* aChild,
-                                     const nsAReadableString& aName)
+                                     const nsAString& aName)
 {
   if (!ShouldBeInElements(aChild)) {
     if (!mNotInElements) {
@@ -1343,7 +1653,7 @@ nsFormControlList::IndexOfControl(nsIFormControl* aControl,
 
 nsresult
 nsFormControlList::RemoveElementFromTable(nsIFormControl* aChild,
-                                          const nsAReadableString& aName)
+                                          const nsAString& aName)
 {
   if (!ShouldBeInElements(aChild)) {
     if (mNotInElements) {
