@@ -50,7 +50,7 @@
 #include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
-#include "nsIHTMLAttributes.h"
+#include "nsHTMLAttributes.h"
 #include "nsIForm.h"
 #include "nsIFormSubmission.h"
 #include "nsIDOMHTMLCollection.h"
@@ -128,7 +128,7 @@ class nsHTMLSelectElement : public nsGenericHTMLContainerFormElement,
                             public nsISelectElement
 {
 public:
-  nsHTMLSelectElement();
+  nsHTMLSelectElement(PRBool aFromParser);
   virtual ~nsHTMLSelectElement();
 
   // nsISupports
@@ -183,7 +183,7 @@ public:
   NS_DECL_NSISELECTELEMENT
 
   NS_IMETHOD StringToAttribute(nsIAtom* aAttribute,
-                               const nsAReadableString& aValue,
+                               const nsAString& aValue,
                                nsHTMLValue& aResult);
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute,
@@ -238,7 +238,7 @@ protected:
   nsISelectControlFrame *GetSelectFrame();
 
   nsHTMLOptionCollection* mOptions;
-  PRBool    mIsDoneAddingContent;
+  PRBool    mIsDoneAddingChildren;
   PRUint32  mArtifactsAtTopLevel;
   PRInt32   mSelectedIndex;
   nsString* mRestoreState;
@@ -254,11 +254,12 @@ protected:
 
 nsresult
 NS_NewHTMLSelectElement(nsIHTMLContent** aInstancePtrResult,
-                        nsINodeInfo *aNodeInfo)
+                        nsINodeInfo *aNodeInfo,
+                        PRBool aFromParser)
 {
   NS_ENSURE_ARG_POINTER(aInstancePtrResult);
 
-  nsHTMLSelectElement* it = new nsHTMLSelectElement();
+  nsHTMLSelectElement* it = new nsHTMLSelectElement(aFromParser);
 
   if (!it) {
     return NS_ERROR_OUT_OF_MEMORY;
@@ -279,9 +280,11 @@ NS_NewHTMLSelectElement(nsIHTMLContent** aInstancePtrResult,
 }
 
 
-nsHTMLSelectElement::nsHTMLSelectElement()
+nsHTMLSelectElement::nsHTMLSelectElement(PRBool aFromParser)
 {
-  mIsDoneAddingContent = PR_TRUE;
+  // DoneAddingChildren() will be called later if it's from the parser,
+  // otherwise it is
+  mIsDoneAddingChildren = !aFromParser;
   mArtifactsAtTopLevel = 0;
 
   mOptions = new nsHTMLOptionCollection(this);
@@ -330,7 +333,7 @@ nsHTMLSelectElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
   NS_ENSURE_ARG_POINTER(aReturn);
   *aReturn = nsnull;
 
-  nsHTMLSelectElement* it = new nsHTMLSelectElement();
+  nsHTMLSelectElement* it = new nsHTMLSelectElement(PR_FALSE);
 
   if (!it) {
     return NS_ERROR_OUT_OF_MEMORY;
@@ -438,7 +441,7 @@ nsHTMLSelectElement::InsertOptionsIntoList(nsIContent* aOptions,
 
     // Actually select the options if the added options warrant it
     nsCOMPtr<nsIDOMNode> optionNode;
-    nsCOMPtr<nsIOptionElement> option;
+    nsCOMPtr<nsIDOMHTMLOptionElement> option;
     for (PRInt32 i=aListIndex;i<insertIndex;i++) {
       // Notify the frame that the option is added
       if (selectFrame) {
@@ -449,7 +452,7 @@ nsHTMLSelectElement::InsertOptionsIntoList(nsIContent* aOptions,
       option = do_QueryInterface(optionNode);
       if (option) {
         PRBool selected;
-        option->GetSelectedInternal(&selected);
+        option->GetSelected(&selected);
         if (selected) {
           // Clear all other options
           PRBool isMultiple;
@@ -860,7 +863,7 @@ nsHTMLSelectElement::GetOptions(nsIDOMHTMLCollection** aValue)
 }
 
 NS_IMETHODIMP
-nsHTMLSelectElement::GetType(nsAWritableString& aType)
+nsHTMLSelectElement::GetType(nsAString& aType)
 {
   PRBool isMultiple;
   nsresult rv = NS_OK;
@@ -1333,6 +1336,8 @@ nsHTMLSelectElement::IsOptionDisabled(PRInt32 aIndex, PRBool* aIsDisabled)
   *aIsDisabled = PR_FALSE;
   nsCOMPtr<nsIDOMNode> optionNode;
   Item(aIndex, getter_AddRefs(optionNode));
+  NS_ENSURE_TRUE(optionNode, NS_ERROR_FAILURE);
+
   nsCOMPtr<nsIDOMHTMLOptionElement> option = do_QueryInterface(optionNode);
   if (option) {
     PRBool isDisabled;
@@ -1387,7 +1392,7 @@ nsHTMLSelectElement::IsOptionDisabled(PRInt32 aIndex, PRBool* aIsDisabled)
 }
 
 NS_IMETHODIMP
-nsHTMLSelectElement::GetValue(nsAWritableString& aValue)
+nsHTMLSelectElement::GetValue(nsAString& aValue)
 {
   PRInt32 selectedIndex;
 
@@ -1443,7 +1448,7 @@ nsHTMLSelectElement::GetValue(nsAWritableString& aValue)
 }
 
 NS_IMETHODIMP
-nsHTMLSelectElement::SetValue(const nsAReadableString& aValue)
+nsHTMLSelectElement::SetValue(const nsAString& aValue)
 {
   nsresult rv = NS_OK;
 
@@ -1572,7 +1577,7 @@ nsHTMLSelectElement::Item(PRUint32 aIndex, nsIDOMNode** aReturn)
 }
 
 NS_IMETHODIMP 
-nsHTMLSelectElement::NamedItem(const nsAReadableString& aName,
+nsHTMLSelectElement::NamedItem(const nsAString& aName,
                                nsIDOMNode** aReturn)
 {
   return mOptions->NamedItem(aName, aReturn);
@@ -1581,7 +1586,7 @@ nsHTMLSelectElement::NamedItem(const nsAReadableString& aName,
 nsresult
 nsHTMLSelectElement::CheckSelectSomething()
 {
-  if (mIsDoneAddingContent) {
+  if (mIsDoneAddingChildren) {
     PRInt32 size = 1;
     GetSize(&size);
     PRBool isMultiple;
@@ -1598,7 +1603,7 @@ nsresult
 nsHTMLSelectElement::SelectSomething()
 {
   // If we're not done building the select, don't play with this yet.
-  if (!mIsDoneAddingContent) {
+  if (!mIsDoneAddingChildren) {
     return NS_OK;
   }
 
@@ -1642,33 +1647,38 @@ nsHTMLSelectElement::RemoveOption(nsIContent* aContent)
 }
 
 NS_IMETHODIMP
-nsHTMLSelectElement::IsDoneAddingContent(PRBool * aIsDone)
+nsHTMLSelectElement::IsDoneAddingChildren(PRBool * aIsDone)
 {
-  *aIsDone = mIsDoneAddingContent;
+  *aIsDone = mIsDoneAddingChildren;
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHTMLSelectElement::DoneAddingContent(PRBool aIsDone)
+nsHTMLSelectElement::DoneAddingChildren()
 {
-  mIsDoneAddingContent = aIsDone;
+  mIsDoneAddingChildren = PR_TRUE;
 
-  nsISelectControlFrame* sFrame = GetSelectFrame();
+  nsISelectControlFrame* selectFrame = GetSelectFrame();
 
   // If we foolishly tried to restore before we were done adding
   // content, restore the rest of the options proper-like
-  if (mIsDoneAddingContent && mRestoreState) {
+  if (mRestoreState) {
     RestoreStateTo(mRestoreState);
     delete mRestoreState;
     mRestoreState = nsnull;
   }
 
-  if (sFrame) {
-    sFrame->DoneAddingContent(mIsDoneAddingContent);
+  // Notify the frame
+  if (selectFrame) {
+    selectFrame->DoneAddingChildren(PR_TRUE);
   }
 
-  // Now that we're done, select something
+  // Restore state
+  RestoreFormControlState(this, this);
+
+  // Now that we're done, select something (if it's a single select something
+  // must be selected)
   CheckSelectSomething();
 
   return NS_OK;
@@ -1676,7 +1686,7 @@ nsHTMLSelectElement::DoneAddingContent(PRBool aIsDone)
 
 NS_IMETHODIMP
 nsHTMLSelectElement::StringToAttribute(nsIAtom* aAttribute,
-                                       const nsAReadableString& aValue,
+                                       const nsAString& aValue,
                                        nsHTMLValue& aResult)
 {
   if (aAttribute == nsHTMLAtoms::disabled) {
@@ -1803,8 +1813,7 @@ nsHTMLSelectElement::GetType(PRInt32* aType)
 }
 
 NS_IMETHODIMP
-nsHTMLSelectElement::SaveState(nsIPresContext* aPresContext,
-                               nsIPresState** aState)
+nsHTMLSelectElement::SaveState()
 {
   nsAutoString stateStr;
 
@@ -1826,24 +1835,19 @@ nsHTMLSelectElement::SaveState(nsIPresContext* aPresContext,
     }
   }
 
-  nsresult rv = GetPrimaryPresState(this, aState);
-  if (*aState) {
-    rv = (*aState)->SetStateProperty(NS_LITERAL_STRING("selecteditems"),
-                                     stateStr);
+  nsCOMPtr<nsIPresState> state;
+  nsresult rv = GetPrimaryPresState(this, getter_AddRefs(state));
+  if (state) {
+    rv = state->SetStateProperty(NS_LITERAL_STRING("selecteditems"),
+                                 stateStr);
     NS_ASSERTION(NS_SUCCEEDED(rv), "selecteditems set failed!");
   }
   return rv;
 }
 
 NS_IMETHODIMP
-nsHTMLSelectElement::RestoreState(nsIPresContext* aPresContext,
-                                  nsIPresState* aState)
+nsHTMLSelectElement::RestoreState(nsIPresState* aState)
 {
-  // XXX This works right now, but since this is only called from
-  // RestoreState() in the frame, this will happen at the first frame
-  // creation.  If JavaScript makes changes before then, and the page
-  // is being reloaded, these changes will be lost.
-  //
   // If RestoreState() is called a second time after SaveState() was
   // called, this will do nothing.
 
@@ -1880,7 +1884,7 @@ nsHTMLSelectElement::GetBoxObject(nsIBoxObject** aResult)
 nsresult
 nsHTMLSelectElement::RestoreStateTo(nsAString* aNewSelected)
 {
-  if (!mIsDoneAddingContent) {
+  if (!mIsDoneAddingChildren) {
     mRestoreState = new nsString;
     if (!mRestoreState) {
       return NS_OK;
@@ -2195,7 +2199,7 @@ nsHTMLOptionCollection::ItemAsOption(PRInt32 aIndex,
 }
 
 NS_IMETHODIMP 
-nsHTMLOptionCollection::NamedItem(const nsAReadableString& aName,
+nsHTMLOptionCollection::NamedItem(const nsAString& aName,
                                   nsIDOMNode** aReturn)
 {
   PRUint32 count;
