@@ -108,10 +108,12 @@ public:
   NS_IMETHOD OpenNoscript(const nsIParserNode& aNode);
   NS_IMETHOD CloseNoscript(const nsIParserNode& aNode);
   NS_IMETHOD GetPref(PRInt32 aTag,PRBool& aPref) { return NS_OK; }
+  NS_IMETHOD_(PRBool) IsFormOnStack() { return PR_FALSE; }
+  
   NS_IMETHOD OpenMap(const nsIParserNode& aNode);
   NS_IMETHOD CloseMap(const nsIParserNode& aNode);
   NS_IMETHOD FlushPendingNotifications() { return NS_OK; }
-  NS_IMETHOD SetDocumentCharset(nsAWritableString& aCharset) { return NS_OK; }
+  NS_IMETHOD SetDocumentCharset(nsAString& aCharset) { return NS_OK; }
   NS_IMETHOD WillProcessTokens(void) { return NS_OK; }
   NS_IMETHOD DidProcessTokens(void) { return NS_OK; }
   NS_IMETHOD WillProcessAToken(void) { return NS_OK; }
@@ -133,7 +135,7 @@ public:
   nsresult AddAttributes(const nsIParserNode& aNode,
                          nsIContent* aContent);
 
-  nsresult AddText(const nsAReadableString& aString);
+  nsresult AddText(const nsAString& aString);
   nsresult AddTextToContent(nsIHTMLContent* aContent,const nsString& aText);
   nsresult FlushText();
 
@@ -629,7 +631,15 @@ nsHTMLFragmentContentSink::AddLeaf(const nsIParserNode& aNode)
              nodeType == eHTMLTag_xmp) {
             
             // Create a text node holding the content
-            result=AddTextToContent(content,aNode.GetSkippedContent());
+            nsCOMPtr<nsIDTD> dtd;
+            mParser->GetDTD(getter_AddRefs(dtd));
+            NS_ENSURE_TRUE(dtd, NS_ERROR_FAILURE);
+    
+            nsAutoString skippedContent;
+            PRInt32 lineNo = 0;
+
+            dtd->CollectSkippedContent(nodeType, skippedContent, lineNo);
+            result=AddTextToContent(content, skippedContent);
           }
           else if (nodeType == eHTMLTag_img || nodeType == eHTMLTag_frame
               || nodeType == eHTMLTag_input)    // elements with 'SRC='
@@ -767,7 +777,7 @@ nsHTMLFragmentContentSink::PopContent()
 #define NS_ACCUMULATION_BUFFER_SIZE 4096
 
 nsresult
-nsHTMLFragmentContentSink::AddText(const nsAReadableString& aString)
+nsHTMLFragmentContentSink::AddText(const nsAString& aString)
 {
   PRInt32 addLen = aString.Length();
   if (0 == addLen) {
@@ -903,7 +913,7 @@ nsHTMLFragmentContentSink::AddAttributes(const nsIParserNode& aNode,
   PRInt32 ac = aNode.GetAttributeCount();
   for (PRInt32 i = 0; i < ac; i++) {
     // Get upper-cased key
-    const nsAReadableString& key = aNode.GetKeyAt(i);
+    const nsAString& key = aNode.GetKeyAt(i);
     k.Assign(key);
     ToLowerCase(k);
 
