@@ -68,7 +68,6 @@
 #include "nsINameSpaceManager.h"
 #include "nsINameSpace.h"
 #include "nsILookAndFeel.h"
-#include "xp_core.h"
 #include "nsRuleNode.h"
 
 #include "nsIStyleSet.h"
@@ -695,7 +694,7 @@ static PRBool IsPseudoElement(nsIAtom* aAtom)
   return PR_FALSE;
 }
 
-void nsCSSSelector::AppendNegationToString(nsAWritableString& aString)
+void nsCSSSelector::AppendNegationToString(nsAString& aString)
 {
   aString.Append(NS_LITERAL_STRING(":not("));
 }
@@ -704,7 +703,7 @@ void nsCSSSelector::AppendNegationToString(nsAWritableString& aString)
 // Builds the textual representation of a selector. Called by DOM 2 CSS 
 // StyleRule:selectorText
 //
-nsresult nsCSSSelector::ToString( nsAWritableString& aString, nsICSSStyleSheet* aSheet, PRBool aIsPseudoElem,
+nsresult nsCSSSelector::ToString( nsAString& aString, nsICSSStyleSheet* aSheet, PRBool aIsPseudoElem,
                                   PRInt8 aNegatedIndex) const
 {
   const PRUnichar* temp;
@@ -1068,8 +1067,8 @@ public:
   DOMCSSDeclarationImpl(nsICSSStyleRule *aRule);
   ~DOMCSSDeclarationImpl(void);
 
-  NS_IMETHOD RemoveProperty(const nsAReadableString& aPropertyName, 
-                            nsAWritableString& aReturn);
+  NS_IMETHOD RemoveProperty(const nsAString& aPropertyName, 
+                            nsAString& aReturn);
 
   virtual void DropReference(void);
   virtual nsresult GetCSSDeclaration(nsCSSDeclaration **aDecl,
@@ -1081,9 +1080,9 @@ public:
                                             nsIURI** aURI,
                                             nsICSSLoader** aCSSLoader,
                                             nsICSSParser** aCSSParser);
-  virtual nsresult ParsePropertyValue(const nsAReadableString& aPropName,
-                                      const nsAReadableString& aPropValue);
-  virtual nsresult ParseDeclaration(const nsAReadableString& aDecl,
+  virtual nsresult ParsePropertyValue(const nsAString& aPropName,
+                                      const nsAString& aPropValue);
+  virtual nsresult ParseDeclaration(const nsAString& aDecl,
                                     PRBool aParseOnlyOneDecl,
                                     PRBool aClearOldDecl);
   virtual nsresult GetParent(nsISupports **aParent);
@@ -1109,8 +1108,8 @@ DOMCSSDeclarationImpl::~DOMCSSDeclarationImpl(void)
 }
 
 NS_IMETHODIMP
-DOMCSSDeclarationImpl::RemoveProperty(const nsAReadableString& aPropertyName, 
-                                      nsAWritableString& aReturn)
+DOMCSSDeclarationImpl::RemoveProperty(const nsAString& aPropertyName, 
+                                      nsAString& aReturn)
 {
   aReturn.Truncate();
 
@@ -1118,6 +1117,19 @@ DOMCSSDeclarationImpl::RemoveProperty(const nsAReadableString& aPropertyName,
   nsresult rv = GetCSSDeclaration(&decl, PR_TRUE);
 
   if (NS_SUCCEEDED(rv) && decl) {
+    nsCOMPtr<nsICSSStyleSheet> cssSheet;
+    nsCOMPtr<nsIDocument> owningDoc;
+    if (mRule) {
+      nsCOMPtr<nsIStyleSheet> sheet;
+      mRule->GetStyleSheet(*getter_AddRefs(sheet));
+      cssSheet = do_QueryInterface(sheet);
+      if (sheet) {
+        sheet->GetOwningDocument(*getter_AddRefs(owningDoc));
+      }
+    }
+    if (owningDoc) {
+      owningDoc->BeginUpdate();
+    }
     nsCSSProperty prop = nsCSSProps::LookupProperty(aPropertyName);
     nsCSSValue val;
 
@@ -1127,11 +1139,20 @@ DOMCSSDeclarationImpl::RemoveProperty(const nsAReadableString& aPropertyName,
       // We pass in eCSSProperty_UNKNOWN here so that we don't get the
       // property name in the return string.
       val.ToString(aReturn, eCSSProperty_UNKNOWN);
+      if (cssSheet) {
+        cssSheet->SetModified(PR_TRUE);
+      }
+      if (owningDoc) {
+        owningDoc->StyleRuleChanged(cssSheet, mRule, nsCSSProps::kHintTable[prop]);
+      }
     } else {
       // If we tried to remove an invalid property or a property that wasn't 
       //  set we simply return success and an empty string
       rv = NS_OK;
     }
+    if (owningDoc) {
+      owningDoc->EndUpdate();
+    }
   }
 
   return rv;
@@ -1211,8 +1232,8 @@ DOMCSSDeclarationImpl::GetCSSParsingEnvironment(nsICSSStyleRule* aRule,
 }
 
 nsresult
-DOMCSSDeclarationImpl::ParsePropertyValue(const nsAReadableString& aPropName,
-                                          const nsAReadableString& aPropValue)
+DOMCSSDeclarationImpl::ParsePropertyValue(const nsAString& aPropName,
+                                          const nsAString& aPropValue)
 {
   nsCSSDeclaration* decl;
   nsresult result = GetCSSDeclaration(&decl, PR_TRUE);
@@ -1257,7 +1278,7 @@ DOMCSSDeclarationImpl::ParsePropertyValue(const nsAReadableString& aPropName,
 }
 
 nsresult 
-DOMCSSDeclarationImpl::ParseDeclaration(const nsAReadableString& aDecl,
+DOMCSSDeclarationImpl::ParseDeclaration(const nsAString& aDecl,
                                         PRBool aParseOnlyOneDecl,
                                         PRBool aClearOldDecl)
 {
@@ -2306,6 +2327,13 @@ MapDisplayForDeclaration(nsCSSDeclaration* aDecl, const nsStyleStructID& aID, ns
     if (aDisplay.mClear.GetUnit() == eCSSUnit_Null && ourDisplay->mClear.GetUnit() != eCSSUnit_Null)
       aDisplay.mClear = ourDisplay->mClear;
 
+    // temp fix for bug 24000
+    if (aDisplay.mBreakBefore.GetUnit() == eCSSUnit_Null && ourDisplay->mBreakBefore.GetUnit() != eCSSUnit_Null)
+      aDisplay.mBreakBefore = ourDisplay->mBreakBefore;
+    if (aDisplay.mBreakAfter.GetUnit() == eCSSUnit_Null && ourDisplay->mBreakAfter.GetUnit() != eCSSUnit_Null)
+      aDisplay.mBreakAfter = ourDisplay->mBreakAfter;
+    // end temp fix
+
     // float: enum, none, inherit
     if (aDisplay.mFloat.GetUnit() == eCSSUnit_Null && ourDisplay->mFloat.GetUnit() != eCSSUnit_Null)
       aDisplay.mFloat = ourDisplay->mFloat;
@@ -2377,6 +2405,9 @@ MapUIForDeclaration(nsCSSDeclaration* aDecl, const nsStyleStructID& aID, nsCSSUs
 
     if (aUI.mResizer.GetUnit() == eCSSUnit_Null && ourUI->mResizer.GetUnit() != eCSSUnit_Null)
       aUI.mResizer = ourUI->mResizer;
+
+    if (aUI.mForceBrokenImageIcon.GetUnit() == eCSSUnit_Null && ourUI->mForceBrokenImageIcon.GetUnit() == eCSSUnit_Integer)
+      aUI.mForceBrokenImageIcon = ourUI->mForceBrokenImageIcon;
   }
 
   return NS_OK;
@@ -2471,7 +2502,7 @@ CSSStyleRuleImpl::GetType(PRUint16* aType)
 }
 
 NS_IMETHODIMP    
-CSSStyleRuleImpl::GetCssText(nsAWritableString& aCssText)
+CSSStyleRuleImpl::GetCssText(nsAString& aCssText)
 {
   mSelector.ToString( aCssText, mSheet, IsPseudoElement(mSelector.mTag),
                       0 );
@@ -2490,7 +2521,7 @@ CSSStyleRuleImpl::GetCssText(nsAWritableString& aCssText)
 }
 
 NS_IMETHODIMP    
-CSSStyleRuleImpl::SetCssText(const nsAReadableString& aCssText)
+CSSStyleRuleImpl::SetCssText(const nsAString& aCssText)
 {
   // XXX TBI - need to re-parse rule & declaration
   return NS_OK;
@@ -2517,14 +2548,14 @@ CSSStyleRuleImpl::GetParentRule(nsIDOMCSSRule** aParentRule)
 }
 
 NS_IMETHODIMP    
-CSSStyleRuleImpl::GetSelectorText(nsAWritableString& aSelectorText)
+CSSStyleRuleImpl::GetSelectorText(nsAString& aSelectorText)
 {
   mSelector.ToString( aSelectorText, mSheet, IsPseudoElement(mSelector.mTag), 0 );
   return NS_OK;
 }
 
 NS_IMETHODIMP    
-CSSStyleRuleImpl::SetSelectorText(const nsAReadableString& aSelectorText)
+CSSStyleRuleImpl::SetSelectorText(const nsAString& aSelectorText)
 {
   // XXX TBI - get a parser and re-parse the selectors, 
   // XXX then need to re-compute the cascade
