@@ -77,6 +77,7 @@
 #include "nsXULAtoms.h"
 #include "nsXBLAtoms.h"
 #include "nsXBLProtoImpl.h"
+#include "nsCRT.h"
 
 #include "nsIScriptContext.h"
 
@@ -254,7 +255,7 @@ static const PRInt32 kInsInitialSize = (NS_SIZE_IN_HEAP(sizeof(nsXBLInsertionPoi
 NS_IMPL_ISUPPORTS2(nsXBLPrototypeBinding, nsIXBLPrototypeBinding, nsISupportsWeakReference)
 
 // Constructors/Destructors
-nsXBLPrototypeBinding::nsXBLPrototypeBinding(const nsAReadableCString& aID, nsIXBLDocumentInfo* aInfo,
+nsXBLPrototypeBinding::nsXBLPrototypeBinding(const nsACString& aID, nsIXBLDocumentInfo* aInfo,
                                              nsIContent* aElement)
 : mInheritStyle(PR_TRUE), 
   mHasBaseProto(PR_TRUE),
@@ -412,7 +413,7 @@ nsXBLPrototypeBinding::LoadResources(PRBool* aResult)
 }
 
 NS_IMETHODIMP
-nsXBLPrototypeBinding::AddResource(nsIAtom* aResourceType, const nsAReadableString& aSrc)
+nsXBLPrototypeBinding::AddResource(nsIAtom* aResourceType, const nsAString& aSrc)
 {
   if (!mResources) {
     mResources = new nsXBLPrototypeResources(this);
@@ -585,8 +586,8 @@ nsXBLPrototypeBinding::AttributeChanged(nsIAtom* aAttribute, PRInt32 aNameSpaceI
         // children of the real element and get the text nodes' values.
         if (aAttribute == nsXBLAtoms::xbltext) {
           nsXBLBinding::GetTextData(aChangedElement, value);
-          value.StripChar('\n');
-          value.StripChar('\r');
+          value.StripChar(PRUnichar('\n'));
+          value.StripChar(PRUnichar('\r'));
           nsAutoString stripVal(value);
           stripVal.StripWhitespace();
           if (stripVal.IsEmpty()) 
@@ -1070,8 +1071,8 @@ PRBool PR_CALLBACK SetAttrs(nsHashKey* aKey, void* aData, void* aClosure)
   PRBool attrPresent = PR_TRUE;
   if (src == nsXBLAtoms::xbltext) {
     nsXBLBinding::GetTextData(changeData->mBoundElement, value);
-    value.StripChar('\n');
-    value.StripChar('\r');
+    value.StripChar(PRUnichar('\n'));
+    value.StripChar(PRUnichar('\r'));
     nsAutoString stripVal(value);
     stripVal.StripWhitespace();
 
@@ -1174,13 +1175,8 @@ nsXBLPrototypeBinding::ShouldBuildChildFrames(PRBool* aResult)
 void
 nsXBLPrototypeBinding::ConstructAttributeTable(nsIContent* aElement)
 {
-  nsresult rv;
   nsAutoString inherits;
-  rv = aElement->GetAttr(kNameSpaceID_XBL, nsXBLAtoms::inherits, inherits);
-  // XXX fallback to using "inherits" in the null namespace
-  // We shouldn't do this once bug 119317 is fully fixed
-  if (rv == NS_CONTENT_ATTR_NOT_THERE)
-    aElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::inherits, inherits);
+  aElement->GetAttr(kNameSpaceID_XBL, nsXBLAtoms::inherits, inherits);
 
   if (!inherits.IsEmpty()) {
     if (!mAttributeTable) {
@@ -1352,7 +1348,7 @@ nsXBLPrototypeBinding::ConstructInsertionTable(nsIContent* aContent)
 }
 
 NS_IMETHODIMP
-nsXBLPrototypeBinding::ConstructInterfaceTable(const nsAReadableString& aImpls)
+nsXBLPrototypeBinding::ConstructInterfaceTable(const nsAString& aImpls)
 {
   if (!aImpls.IsEmpty()) {
     // Obtain the interface info manager that can tell us the IID
@@ -1426,7 +1422,7 @@ nsXBLPrototypeBinding::AddResourceListener(nsIContent* aBoundElement)
 // Creation Routine ///////////////////////////////////////////////////////////////////////
 
 nsresult
-NS_NewXBLPrototypeBinding(const nsAReadableCString& aRef, nsIContent* aElement, 
+NS_NewXBLPrototypeBinding(const nsACString& aRef, nsIContent* aElement, 
                           nsIXBLDocumentInfo* aInfo, nsIXBLPrototypeBinding** aResult)
 {
   nsXBLPrototypeBinding * binding = new nsXBLPrototypeBinding(aRef, aInfo, aElement);
