@@ -100,6 +100,7 @@
 #include "prlog.h"
 #include "prmem.h"
 #include "jsapi.h"  // for JSVERSION_*, JS_VersionToString, etc.
+#include "nsCRT.h"
 
 #include "nsIFastLoadService.h"         // XXXbe temporary
 #include "nsIObjectInputStream.h"       // XXXbe temporary
@@ -144,7 +145,7 @@ public:
     NS_IMETHOD WillResume(void);
     NS_IMETHOD SetParser(nsIParser* aParser);  
     NS_IMETHOD FlushPendingNotifications() { return NS_OK; }
-    NS_IMETHOD SetDocumentCharset(nsAWritableString& aCharset);
+    NS_IMETHOD SetDocumentCharset(nsAString& aCharset);
 
     // nsIXULContentSink
     NS_IMETHOD Init(nsIDocument* aDocument, nsIXULPrototypeDocument* aPrototype);
@@ -570,23 +571,6 @@ XULContentSinkImpl::SetParser(nsIParser* aParser)
     return NS_OK;
 }
 
-static void SplitMimeType(const nsString& aValue, nsString& aType, nsString& aParams)
-{
-  aType.Truncate();
-  aParams.Truncate();
-  PRInt32 semiIndex = aValue.FindChar(PRUnichar(';'));
-  if (-1 != semiIndex) {
-    aValue.Left(aType, semiIndex);
-    aValue.Right(aParams, (aValue.Length() - semiIndex) - 1);
-    aParams.StripWhitespace();
-  }
-  else {
-    aType = aValue;
-  }
-  aType.StripWhitespace();
-}
-
-
 nsresult
 XULContentSinkImpl::ProcessStyleLink(nsIContent* aElement,
                                      const nsString& aHref,
@@ -605,9 +589,9 @@ XULContentSinkImpl::ProcessStyleLink(nsIContent* aElement,
         }
     }
 
-    nsAutoString  mimeType;
-    nsAutoString  params;
-    SplitMimeType(aType, mimeType, params);
+    nsAutoString mimeType;
+    nsAutoString params;
+    nsParserUtils::SplitMimeType(aType, mimeType, params);
 
     if ((mimeType.IsEmpty()) || mimeType.EqualsIgnoreCase(kCSSType)) {
         nsCOMPtr<nsIURI> url;
@@ -658,7 +642,7 @@ XULContentSinkImpl::ProcessStyleLink(nsIContent* aElement,
 }
 
 NS_IMETHODIMP 
-XULContentSinkImpl::SetDocumentCharset(nsAWritableString& aCharset)
+XULContentSinkImpl::SetDocumentCharset(nsAString& aCharset)
 {
     nsCOMPtr<nsIDocument> doc = do_QueryReferent(mDocument);
     if (doc) {
@@ -1025,7 +1009,11 @@ XULContentSinkImpl::HandleCDataSection(const PRUnichar *aData, PRUint32 aLength)
 }
 
 NS_IMETHODIMP 
-XULContentSinkImpl::HandleDoctypeDecl(const PRUnichar *aDoctype, PRUint32 aLength)
+XULContentSinkImpl::HandleDoctypeDecl(const nsAString & aSubset, 
+                                      const nsAString & aName, 
+                                      const nsAString & aSystemId, 
+                                      const nsAString & aPublicId,
+                                      nsISupports* aCatalogData)
 {
     return NS_OK;
 }
@@ -1153,7 +1141,7 @@ XULContentSinkImpl::ReportError(const PRUnichar* aErrorText,
   mState = eInProlog;
 
   NS_NAMED_LITERAL_STRING(name, "xmlns");
-  NS_NAMED_LITERAL_STRING(value, "http://www.w3.org/1999/xhtml");
+  NS_NAMED_LITERAL_STRING(value, "http://www.mozilla.org/newlayout/xml/parsererror.xml");
 
   const PRUnichar* atts[] = {name.get(), value.get(), nsnull};;
     
@@ -1163,7 +1151,8 @@ XULContentSinkImpl::ReportError(const PRUnichar* aErrorText,
   rv = HandleCharacterData(aErrorText, nsCRT::strlen(aErrorText));
   NS_ENSURE_SUCCESS(rv,rv);  
   
-  rv = HandleStartElement(NS_LITERAL_STRING("sourcetext").get(), atts, 1, -1, -1);
+  const PRUnichar* noAtts[] = {0, 0};
+  rv = HandleStartElement(NS_LITERAL_STRING("sourcetext").get(), noAtts, 0, -1, -1);
   NS_ENSURE_SUCCESS(rv,rv);
   
   rv = HandleCharacterData(aSourceText, nsCRT::strlen(aSourceText));
@@ -1454,7 +1443,7 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
           nsAutoString  type(aAttributes[1]);
           nsAutoString  mimeType;
           nsAutoString  params;
-          SplitMimeType(type, mimeType, params);
+          nsParserUtils::SplitMimeType(type, mimeType, params);
 
           isJavaScript = mimeType.EqualsIgnoreCase("application/x-javascript") ||
                          mimeType.EqualsIgnoreCase("text/javascript");
