@@ -720,6 +720,7 @@ nsLDAPService::EstablishConnection(nsLDAPServiceEntry *aEntry,
     nsXPIDLString binddn;
     nsXPIDLString password;
     PRInt32 port;
+    PRUint32 options;
     nsresult rv;
 
     server = getter_AddRefs(aEntry->GetServer());
@@ -753,6 +754,10 @@ nsLDAPService::EstablishConnection(nsLDAPServiceEntry *aEntry,
         return NS_ERROR_FAILURE;
     }
 
+    rv = url->GetOptions(&options);
+    if (NS_FAILED(rv)) {
+      return NS_ERROR_FAILURE;
+    }
     // Create a new connection for this server.
     //
     conn = do_CreateInstance(kLDAPConnectionCID, &rv);
@@ -764,7 +769,9 @@ nsLDAPService::EstablishConnection(nsLDAPServiceEntry *aEntry,
 
     // Here we need to provide the binddn, see bug #75990
     //
-    rv = conn->Init(host.get(), port, 0, this);
+    rv = conn->Init(host.get(), port, 
+                    (options & nsILDAPURL::OPT_SECURE) ? PR_TRUE : PR_FALSE, 
+                    nsnull, this);
     if (NS_FAILED(rv)) {
         switch (rv) {
         // Only pass along errors we are aware of
@@ -862,12 +869,12 @@ nsLDAPService::EstablishConnection(nsLDAPServiceEntry *aEntry,
 
 /* AString createFilter (in unsigned long aMaxSize, in AString aPattern, in AString aPrefix, in AString aSuffix, in AString aAttr, in AString aValue); */
 NS_IMETHODIMP nsLDAPService::CreateFilter(PRUint32 aMaxSize, 
-                                          const nsAReadableString & aPattern,
-                                          const nsAReadableString & aPrefix,
-                                          const nsAReadableString & aSuffix,
-                                          const nsAReadableString & aAttr,
-                                          const nsAReadableString & aValue,
-                                          nsAWritableString & _retval)
+                                          const nsAString & aPattern,
+                                          const nsAString & aPrefix,
+                                          const nsAString & aSuffix,
+                                          const nsAString & aAttr,
+                                          const nsAString & aValue,
+                                          nsAString & _retval)
 {
     if (!aMaxSize) {
         return NS_ERROR_INVALID_ARG;
@@ -1028,7 +1035,7 @@ nsLDAPService::NextToken(nsReadingIterator<PRUnichar> & aIter,
 //
 // string UCS2ToUTF8 (in AString aString);
 NS_IMETHODIMP
-nsLDAPService::UCS2toUTF8(const nsAReadableString &aString,
+nsLDAPService::UCS2toUTF8(const nsAString &aString,
                                         char **_retval)
 {
     char *str;
@@ -1051,7 +1058,7 @@ nsLDAPService::UCS2toUTF8(const nsAReadableString &aString,
 // AString UTF8ToUCS2 (in string aString);
 NS_IMETHODIMP
 nsLDAPService::UTF8toUCS2(const char *aString,
-                                        nsAWritableString &_retval)
+                                        nsAString &_retval)
 {
     _retval = NS_ConvertUTF8toUCS2(aString);
     return NS_OK;
