@@ -455,17 +455,18 @@ struct OnLinkClickEvent : public PLEvent {
   ~OnLinkClickEvent();
 
   void HandleEvent() {
-    mHandler->HandleLinkClickEvent(mContent, mVerb, mURLSpec->get(),
-                                   mTargetSpec->get(), mPostDataStream,
-                                   mHeadersDataStream);
+    mHandler->OnLinkClickSync(mContent, mVerb, mURLSpec.get(),
+                              mTargetSpec.get(), mPostDataStream,
+                              mHeadersDataStream,
+                              nsnull, nsnull);
   }
 
-  nsWebShell*  mHandler;
-  nsString*    mURLSpec;
-  nsString*    mTargetSpec;
-  nsIInputStream* mPostDataStream;
-  nsIInputStream* mHeadersDataStream;
-  nsIContent*     mContent;
+  nsWebShell*     mHandler;
+  nsString       mURLSpec;
+  nsString       mTargetSpec;
+  nsCOMPtr<nsIInputStream> mPostDataStream;
+  nsCOMPtr<nsIInputStream> mHeadersDataStream;
+  nsCOMPtr<nsIContent>     mContent;
   nsLinkVerb      mVerb;
 };
 
@@ -489,14 +490,11 @@ OnLinkClickEvent::OnLinkClickEvent(nsWebShell* aHandler,
 {
   mHandler = aHandler;
   NS_ADDREF(aHandler);
-  mURLSpec = new nsString(aURLSpec);
-  mTargetSpec = new nsString(aTargetSpec);
+  mURLSpec.Assign(aURLSpec);
+  mTargetSpec.Assign(aTargetSpec);
   mPostDataStream = aPostDataStream;
-  NS_IF_ADDREF(mPostDataStream);
   mHeadersDataStream = aHeadersDataStream;
-  NS_IF_ADDREF(mHeadersDataStream);
   mContent = aContent;
-  NS_IF_ADDREF(mContent);
   mVerb = aVerb;
 
   PL_InitEvent(this, nsnull,
@@ -512,13 +510,7 @@ OnLinkClickEvent::OnLinkClickEvent(nsWebShell* aHandler,
 
 OnLinkClickEvent::~OnLinkClickEvent()
 {
-  NS_IF_RELEASE(mContent);
   NS_IF_RELEASE(mHandler);
-  NS_IF_RELEASE(mPostDataStream);
-  NS_IF_RELEASE(mHeadersDataStream);
-  if (nsnull != mURLSpec) delete mURLSpec;
-  if (nsnull != mTargetSpec) delete mTargetSpec;
-
 }
 
 //----------------------------------------
@@ -554,17 +546,27 @@ nsWebShell::GetEventQueue(nsIEventQueue **aQueue)
   return *aQueue ? NS_OK : NS_ERROR_FAILURE;
 }
 
-void
-nsWebShell::HandleLinkClickEvent(nsIContent *aContent,
-                                 nsLinkVerb aVerb,
-                                 const PRUnichar* aURLSpec,
-                                 const PRUnichar* aTargetSpec,
-                                 nsIInputStream* aPostDataStream,
-                                 nsIInputStream* aHeadersDataStream)
+NS_IMETHODIMP
+nsWebShell::OnLinkClickSync(nsIContent *aContent,
+                            nsLinkVerb aVerb,
+                            const PRUnichar* aURLSpec,
+                            const PRUnichar* aTargetSpec,
+                            nsIInputStream* aPostDataStream,
+                            nsIInputStream* aHeadersDataStream,
+                            nsIDocShell** aDocShell,
+                            nsIRequest** aRequest)
 {
   nsresult rv;
   nsAutoString target(aTargetSpec);
 
+  // Initialize the DocShell / Request
+  if (aDocShell) {
+    *aDocShell = nsnull;
+  }
+  if (aRequest) {
+    *aRequest = nsnull;
+  }
+
   switch(aVerb) {
     case eLinkVerb_New:
       target.Assign(NS_LITERAL_STRING("_blank"));
@@ -599,18 +601,22 @@ nsWebShell::HandleLinkClickEvent(nsIContent *aContent,
                     listener->OnStartURIOpen(uri, &abort);
                 }
             }
-            return;
+            // We didn't load the URI, so we failed
+            return NS_ERROR_FAILURE;
         }
 
-        rv = InternalLoad(uri,                // New URI
-                          mCurrentURI,        // Referer URI
-                          nsnull,             // No onwer
-                          PR_TRUE,            // Inherit owner from document
-                          target.get(),       // Window target
-                          aPostDataStream,    // Post data stream
-                          aHeadersDataStream, // Headers stream
-                          LOAD_LINK,          // Load type
-                          nsnull);            // No SHEntry
+        return InternalLoad(uri,                // New URI
+                            mCurrentURI,        // Referer URI
+                            nsnull,             // No onwer
+                            PR_TRUE,            // Inherit owner from document
+                            target.get(),       // Window target
+                            aPostDataStream,    // Post data stream
+                            aHeadersDataStream, // Headers stream
+                            LOAD_LINK,          // Load type
+                            nsnull,             // No SHEntry
+                            PR_TRUE,            // first party site
+                            aDocShell,          // DocShell out-param
+                            aRequest);          // Request out-param
       }
       break;
     case eLinkVerb_Embed:
@@ -618,6 +624,7 @@ nsWebShell::HandleLinkClickEvent(nsIContent *aContent,
       //          in NS 4.x
     default:
       NS_ABORT_IF_FALSE(0,"unexpected link verb");
+      return NS_ERROR_UNEXPECTED;
   }
 }
 
@@ -768,7 +775,8 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
       //
       if (aStatus == NS_ERROR_UNKNOWN_HOST ||
           aStatus == NS_ERROR_CONNECTION_REFUSED ||
-          aStatus == NS_ERROR_NET_TIMEOUT)
+          aStatus == NS_ERROR_NET_TIMEOUT ||
+          aStatus == NS_ERROR_NET_RESET)
       {
         mURIFixup->CreateFixupURI(oldSpecW.get(),
             nsIURIFixup::FIXUP_FLAG_ALLOW_KEYWORD_LOOKUP, getter_AddRefs(newURI));
@@ -777,7 +785,8 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
       //
       // Now try change the address, e.g. turn http://foo into http://www.foo.com
       //
-      if (aStatus == NS_ERROR_UNKNOWN_HOST)
+      if (aStatus == NS_ERROR_UNKNOWN_HOST ||
+          aStatus == NS_ERROR_NET_RESET)
       {
         // Test if keyword lookup produced a new URI or not
         PRBool doCreateAlternate = PR_TRUE;
@@ -792,6 +801,21 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
             doCreateAlternate = PR_FALSE;
           }
         }
+        // Skip fixup for anything except a normal document load operation
+        if (mLoadType != LOAD_NORMAL)
+        {
+          doCreateAlternate = PR_FALSE;
+        }
+        else
+        {
+          // Skip fixup for frames & iframes
+          nsCOMPtr<nsIDocShellTreeItem> targetParentTreeItem;
+          rv = GetSameTypeParent(getter_AddRefs(targetParentTreeItem));
+          if (NS_SUCCEEDED(rv) && targetParentTreeItem) 
+          {
+            doCreateAlternate = PR_FALSE;
+          }
+        }
         if (doCreateAlternate)
         {
           newURI = nsnull;
@@ -995,47 +1019,50 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
                        inputStream,                       // Post data stream
                        nsnull,                            // No headers stream
                        LOAD_RELOAD_BYPASS_PROXY_AND_CACHE,// Load type
-                       nsnull);                           // No SHEntry
-            
+                       nsnull,                            // No SHEntry
+                       PR_TRUE,                           // first party site
+                       nsnull,                            // No nsIDocShell
+                       nsnull);                           // No nsIRequest
           }
         }
     }
-    //
-    // Doc failed to load because the server generated too many redirects
-    //
-    else if (aStatus == NS_ERROR_REDIRECT_LOOP) {
-      nsCOMPtr<nsIPrompt> prompter;
-      nsCOMPtr<nsIStringBundle> stringBundle;
-
-      rv = GetPromptAndStringBundle(getter_AddRefs(prompter),
-                                    getter_AddRefs(stringBundle));
-      if (!stringBundle) {
-        return rv;
+    else {
+      //
+      // handle errors that have simple messages w/ no arguments.
+      //
+      const char *errorStr = nsnull;
+      switch (aStatus) {
+        case NS_ERROR_REDIRECT_LOOP:
+          // Doc failed to load because the server generated too many redirects
+          errorStr = "redirectLoop";
+          break;
+        case NS_ERROR_UNKNOWN_SOCKET_TYPE:
+          // Doc failed to load because PSM is not installed
+          errorStr = "unknownSocketType";
+          break;
+        case NS_ERROR_NET_RESET:
+          // Doc failed to load because the server kept reseting the connection
+          // before we could read any data from it
+          errorStr = "netReset";
+          break;
       }
+      if (errorStr) {
+        nsCOMPtr<nsIPrompt> prompter;
+        nsCOMPtr<nsIStringBundle> stringBundle;
 
-      nsXPIDLString messageStr;
-      rv = stringBundle->GetStringFromName(NS_LITERAL_STRING("redirectLoop").get(),
-                                           getter_Copies(messageStr));
-      if (NS_FAILED(rv)) return rv;
+        rv = GetPromptAndStringBundle(getter_AddRefs(prompter),
+                                      getter_AddRefs(stringBundle));
+        if (!stringBundle) {
+          return rv;
+        }
 
-      prompter->Alert(nsnull, messageStr);
-    }
-    else if (aStatus == NS_ERROR_UNKNOWN_SOCKET_TYPE) {
-      nsCOMPtr<nsIPrompt> prompter;
-      nsCOMPtr<nsIStringBundle> stringBundle;
+        nsXPIDLString messageStr;
+        rv = stringBundle->GetStringFromName(NS_ConvertASCIItoUCS2(errorStr).get(),
+                                             getter_Copies(messageStr));
+        if (NS_FAILED(rv)) return rv;
 
-      rv = GetPromptAndStringBundle(getter_AddRefs(prompter),
-                                    getter_AddRefs(stringBundle));
-      if (!stringBundle) {
-        return rv;
+        prompter->Alert(nsnull, messageStr);
       }
-
-      nsXPIDLString messageStr;
-      rv = stringBundle->GetStringFromName(NS_LITERAL_STRING("unknownSocketType").get(),
-                                           getter_Copies(messageStr));
-      if (NS_FAILED(rv)) return rv;
-
-      prompter->Alert(nsnull, messageStr);
     }
   } // if we have a host
 
@@ -1051,7 +1078,7 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
 #endif
 
 nsresult
-nsWebShell::GetControllerForCommand ( const nsAReadableString & inCommand, nsIController** outController )
+nsWebShell::GetControllerForCommand ( const nsAString & inCommand, nsIController** outController )
 {
   NS_ENSURE_ARG_POINTER(outController);
   *outController = nsnull;
@@ -1072,7 +1099,7 @@ nsWebShell::GetControllerForCommand ( const nsAReadableString & inCommand, nsICo
 
 
 nsresult
-nsWebShell::IsCommandEnabled ( const nsAReadableString & inCommand, PRBool* outEnabled )
+nsWebShell::IsCommandEnabled ( const nsAString & inCommand, PRBool* outEnabled )
 {
   NS_ENSURE_ARG_POINTER(outEnabled);
   *outEnabled = PR_FALSE;
@@ -1089,7 +1116,7 @@ nsWebShell::IsCommandEnabled ( const nsAReadableString & inCommand, PRBool* outE
 
 
 nsresult
-nsWebShell::DoCommand ( const nsAReadableString & inCommand )
+nsWebShell::DoCommand ( const nsAString & inCommand )
 {
   nsresult rv = NS_ERROR_FAILURE;
   
