@@ -64,9 +64,10 @@
 #include "nsDOMClassInfo.h"
 #include "nsICharsetConverterManager.h"
 #include "nsICharsetConverterManager2.h"
+#include "nsCRT.h"
 
 
-static nsresult EscapeNonAsciiInURI(nsAReadableString& aHref, nsAWritableCString& aEscapedHref)
+static nsresult EscapeNonAsciiInURI(const nsAString& aHref, nsACString& aEscapedHref)
 {
   aEscapedHref.Truncate(0);
 
@@ -114,12 +115,12 @@ static nsresult EscapeNonAsciiInURI(nsAReadableString& aHref, nsAWritableCString
   NS_ENSURE_SUCCESS(rv, rv);
 
   const nsAFlatString &tmp = PromiseFlatString(aHref);
-  PRInt32 len;
+  PRInt32 len = tmp.Length();
+  PRInt32 bufferLen;
 
-  rv = encoder->GetMaxLength(tmp.get(), tmp.Length(), &len);
+  rv = encoder->GetMaxLength(tmp.get(), len, &bufferLen);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRInt32 bufferLen = len;
   char* buffer = (char *) nsMemory::Alloc(bufferLen);
   NS_ENSURE_TRUE(buffer, NS_ERROR_OUT_OF_MEMORY);
 
@@ -234,10 +235,14 @@ LocationImpl::CheckURL(nsIURI* aURI, nsIDocShellLoadInfo** aLoadInfo)
 nsresult
 LocationImpl::GetURI(nsIURI** aURI)
 {
+  *aURI = nsnull;
+
   nsresult rv;
 
   nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(mDocShell, &rv));
-  if (NS_FAILED(rv)) return rv;
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
 
   return webNav->GetCurrentURI(aURI);
 }
@@ -245,10 +250,14 @@ LocationImpl::GetURI(nsIURI** aURI)
 nsresult
 LocationImpl::GetWritableURI(nsIURI** aURI)
 {
+  *aURI = nsnull;
+
   nsCOMPtr<nsIURI> uri;
 
   nsresult rv = GetURI(getter_AddRefs(uri));
-  if (NS_FAILED(rv)) return rv;
+  if (NS_FAILED(rv) || !uri) {
+    return rv;
+  }
 
   return uri->Clone(aURI);
 }
@@ -265,64 +274,61 @@ LocationImpl::SetURI(nsIURI* aURI)
 
     webNav->Stop(nsIWebNavigation::STOP_CONTENT);
     return mDocShell->LoadURI(aURI, loadInfo,
-                              nsIWebNavigation::LOAD_FLAGS_NONE);
+                              nsIWebNavigation::LOAD_FLAGS_NONE, PR_TRUE);
   }
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-LocationImpl::GetHash(nsAWritableString& aHash)
+LocationImpl::GetHash(nsAString& aHash)
 {
+  aHash.SetLength(0);
+
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
   result = GetURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
+  nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
+
+  if (url) {
     nsCAutoString ref;
 
-    if (url) {
-      result = url->GetRef(ref);
-      // XXX danger... this may result in non-ASCII octets!
-      NS_UnescapeURL(ref);
-    }
+    result = url->GetRef(ref);
+    // XXX danger... this may result in non-ASCII octets!
+    NS_UnescapeURL(ref);
 
     if (NS_SUCCEEDED(result) && !ref.IsEmpty()) {
       aHash.Assign(NS_LITERAL_STRING("#") + NS_ConvertASCIItoUCS2(ref));
     }
-    else {
-      aHash.SetLength(0);
-    }
   }
 
   return result;
 }
 
 NS_IMETHODIMP
-LocationImpl::SetHash(const nsAReadableString& aHash)
+LocationImpl::SetHash(const nsAString& aHash)
 {
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
   result = GetWritableURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURL> url(do_QueryInterface(uri, &result));
-    if (url) {
-      url->SetRef(NS_ConvertUCS2toUTF8(aHash));
+  nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
 
-      if (mDocShell) {
-        nsCOMPtr<nsIDocShellLoadInfo> loadInfo;
+  if (url) {
+    url->SetRef(NS_ConvertUCS2toUTF8(aHash));
 
-        if (NS_SUCCEEDED(CheckURL(url, getter_AddRefs(loadInfo))))
-          // We're not calling nsIWebNavigation->Stop, we don't
-          // want to stop the load when we're just scrolling to a
-          // named anchor in the document. See bug 114975.
-          mDocShell->LoadURI(url, loadInfo,
-                             nsIWebNavigation::LOAD_FLAGS_NONE);
-      }
+    if (mDocShell) {
+      nsCOMPtr<nsIDocShellLoadInfo> loadInfo;
+
+      if (NS_SUCCEEDED(CheckURL(url, getter_AddRefs(loadInfo))))
+        // We're not calling nsIWebNavigation->Stop, we don't want to
+        // stop the load when we're just scrolling to a named anchor
+        // in the document. See bug 114975.
+        mDocShell->LoadURI(url, loadInfo,
+                           nsIWebNavigation::LOAD_FLAGS_NONE, PR_TRUE);
     }
   }
 
@@ -330,34 +336,37 @@ LocationImpl::SetHash(const nsAReadableString& aHash)
 }
 
 NS_IMETHODIMP
-LocationImpl::GetHost(nsAWritableString& aHost)
+LocationImpl::GetHost(nsAString& aHost)
 {
+  aHost.SetLength(0);
+
   nsCOMPtr<nsIURI> uri;
   nsresult result;
 
   result = GetURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
+  if (uri) {
     nsCAutoString hostport;
 
     result = uri->GetHostPort(hostport);
 
-    if (NS_SUCCEEDED(result))
+    if (NS_SUCCEEDED(result)) {
       aHost = NS_ConvertUTF8toUCS2(hostport);
+    }
   }
 
   return result;
 }
 
 NS_IMETHODIMP
-LocationImpl::SetHost(const nsAReadableString& aHost)
+LocationImpl::SetHost(const nsAString& aHost)
 {
   nsCOMPtr<nsIURI> uri;
   nsresult result;
 
   result = GetWritableURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
+  if (uri) {
     uri->SetHostPort(NS_ConvertUCS2toUTF8(aHost));
     SetURI(uri);
   }
@@ -366,34 +375,37 @@ LocationImpl::SetHost(const nsAReadableString& aHost)
 }
 
 NS_IMETHODIMP
-LocationImpl::GetHostname(nsAWritableString& aHostname)
+LocationImpl::GetHostname(nsAString& aHostname)
 {
+  aHostname.SetLength(0);
+
   nsCOMPtr<nsIURI> uri;
   nsresult result;
 
   result = GetURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
+  if (uri) {
     nsCAutoString host;
 
     result = uri->GetHost(host);
 
-    if (NS_SUCCEEDED(result))
+    if (NS_SUCCEEDED(result)) {
       aHostname = NS_ConvertUTF8toUCS2(host);
+    }
   }
 
   return result;
 }
 
 NS_IMETHODIMP
-LocationImpl::SetHostname(const nsAReadableString& aHostname)
+LocationImpl::SetHostname(const nsAString& aHostname)
 {
   nsCOMPtr<nsIURI> uri;
   nsresult result;
 
   result = GetWritableURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
+  if (uri) {
     uri->SetHost(NS_ConvertUCS2toUTF8(aHostname));
     SetURI(uri);
   }
@@ -402,27 +414,30 @@ LocationImpl::SetHostname(const nsAReadableString& aHostname)
 }
 
 NS_IMETHODIMP
-LocationImpl::GetHref(nsAWritableString& aHref)
+LocationImpl::GetHref(nsAString& aHref)
 {
+  aHref.SetLength(0);
+
   nsCOMPtr<nsIURI> uri;
   nsresult result;
 
   result = GetURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result) && uri) {
+  if (uri) {
     nsCAutoString uriString;
 
     result = uri->GetSpec(uriString);
 
-    if (NS_SUCCEEDED(result))
+    if (NS_SUCCEEDED(result)) {
       aHref = NS_ConvertUTF8toUCS2(uriString);
+    }
   }
 
   return result;
 }
 
 NS_IMETHODIMP
-LocationImpl::SetHref(const nsAReadableString& aHref)
+LocationImpl::SetHref(const nsAString& aHref)
 {
   nsAutoString oldHref;
   nsresult rv = NS_OK;
@@ -459,7 +474,7 @@ LocationImpl::SetHref(const nsAReadableString& aHref)
 }
 
 nsresult
-LocationImpl::SetHrefWithContext(JSContext* cx, const nsAReadableString& aHref,
+LocationImpl::SetHrefWithContext(JSContext* cx, const nsAString& aHref,
                                  PRBool aReplace)
 {
   nsCOMPtr<nsIURI> base;
@@ -475,7 +490,7 @@ LocationImpl::SetHrefWithContext(JSContext* cx, const nsAReadableString& aHref,
 }
 
 nsresult
-LocationImpl::SetHrefWithBase(const nsAReadableString& aHref,
+LocationImpl::SetHrefWithBase(const nsAString& aHref,
                               nsIURI* aBase, PRBool aReplace)
 {
   nsresult result;
@@ -536,31 +551,30 @@ LocationImpl::SetHrefWithBase(const nsAReadableString& aHref,
 
     webNav->Stop(nsIWebNavigation::STOP_CONTENT);
     return mDocShell->LoadURI(newUri, loadInfo,
-                              nsIWebNavigation::LOAD_FLAGS_NONE);
+                              nsIWebNavigation::LOAD_FLAGS_NONE, PR_TRUE);
   }
 
   return result;
 }
 
 NS_IMETHODIMP
-LocationImpl::GetPathname(nsAWritableString& aPathname)
+LocationImpl::GetPathname(nsAString& aPathname)
 {
+  aPathname.SetLength(0);
+
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
-  aPathname.Truncate();
-
   result = GetURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
-    if (url) {
-      nsCAutoString file;
+  nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
+  if (url) {
+    nsCAutoString file;
 
-      result = url->GetFilePath(file);
+    result = url->GetFilePath(file);
 
-      if (NS_SUCCEEDED(result))
-        aPathname = NS_ConvertUTF8toUCS2(file);
+    if (NS_SUCCEEDED(result)) {
+      aPathname = NS_ConvertUTF8toUCS2(file);
     }
   }
 
@@ -568,14 +582,14 @@ LocationImpl::GetPathname(nsAWritableString& aPathname)
 }
 
 NS_IMETHODIMP
-LocationImpl::SetPathname(const nsAReadableString& aPathname)
+LocationImpl::SetPathname(const nsAString& aPathname)
 {
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
   result = GetWritableURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
+  if (uri) {
     uri->SetPath(NS_ConvertUCS2toUTF8(aPathname));
     SetURI(uri);
   }
@@ -584,16 +598,16 @@ LocationImpl::SetPathname(const nsAReadableString& aPathname)
 }
 
 NS_IMETHODIMP
-LocationImpl::GetPort(nsAWritableString& aPort)
+LocationImpl::GetPort(nsAString& aPort)
 {
+  aPort.SetLength(0);
+
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
   result = GetURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
-    aPort.SetLength(0);
-
+  if (uri) {
     PRInt32 port;
     uri->GetPort(&port);
 
@@ -608,14 +622,14 @@ LocationImpl::GetPort(nsAWritableString& aPort)
 }
 
 NS_IMETHODIMP
-LocationImpl::SetPort(const nsAReadableString& aPort)
+LocationImpl::SetPort(const nsAString& aPort)
 {
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
   result = GetWritableURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
+  if (uri) {
     // perhaps use nsReadingIterators at some point?
     NS_ConvertUCS2toUTF8 portStr(aPort);
     const char *buf = portStr.get();
@@ -629,6 +643,7 @@ LocationImpl::SetPort(const nsAReadableString& aPort)
         port = atol(buf);
       }
     }
+
     uri->SetPort(port);
     SetURI(uri);
   }
@@ -637,14 +652,16 @@ LocationImpl::SetPort(const nsAReadableString& aPort)
 }
 
 NS_IMETHODIMP
-LocationImpl::GetProtocol(nsAWritableString& aProtocol)
+LocationImpl::GetProtocol(nsAString& aProtocol)
 {
+  aProtocol.SetLength(0);
+
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
   result = GetURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
+  if (uri) {
     nsCAutoString protocol;
 
     result = uri->GetScheme(protocol);
@@ -659,14 +676,14 @@ LocationImpl::GetProtocol(nsAWritableString& aProtocol)
 }
 
 NS_IMETHODIMP
-LocationImpl::SetProtocol(const nsAReadableString& aProtocol)
+LocationImpl::SetProtocol(const nsAString& aProtocol)
 {
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
   result = GetWritableURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
+  if (uri) {
     uri->SetScheme(NS_ConvertUCS2toUTF8(aProtocol));
     SetURI(uri);
   }
@@ -675,46 +692,42 @@ LocationImpl::SetProtocol(const nsAReadableString& aProtocol)
 }
 
 NS_IMETHODIMP
-LocationImpl::GetSearch(nsAWritableString& aSearch)
+LocationImpl::GetSearch(nsAString& aSearch)
 {
+  aSearch.SetLength(0);
+
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
   result = GetURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
-    nsCAutoString search;
-    nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
+  nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
 
-    if (url) {
-      result = url->GetQuery(search);
-    }
+  if (url) {
+    nsCAutoString search;
+
+    result = url->GetQuery(search);
 
     if (NS_SUCCEEDED(result) && !search.IsEmpty()) {
       aSearch.Assign(NS_LITERAL_STRING("?") + NS_ConvertUTF8toUCS2(search));
     }
-    else {
-      aSearch.SetLength(0);
-    }
   }
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-LocationImpl::SetSearch(const nsAReadableString& aSearch)
+LocationImpl::SetSearch(const nsAString& aSearch)
 {
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
   result = GetWritableURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURL> url(do_QueryInterface(uri, &result));
-    if (url) {
-      result = url->SetQuery(NS_ConvertUCS2toUTF8(aSearch));
-      SetURI(uri);
-    }
+  nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
+  if (url) {
+    result = url->SetQuery(NS_ConvertUCS2toUTF8(aSearch));
+    SetURI(uri);
   }
 
   return result;
@@ -781,7 +794,7 @@ LocationImpl::Reload()
 }
 
 NS_IMETHODIMP
-LocationImpl::Replace(const nsAReadableString& aUrl)
+LocationImpl::Replace(const nsAString& aUrl)
 {
   nsresult rv = NS_OK;
 
@@ -813,7 +826,7 @@ LocationImpl::Replace(const nsAReadableString& aUrl)
 }
 
 NS_IMETHODIMP
-LocationImpl::Assign(const nsAReadableString& aUrl)
+LocationImpl::Assign(const nsAString& aUrl)
 {
   nsAutoString oldHref;
   nsresult result = NS_OK;
@@ -834,7 +847,7 @@ LocationImpl::Assign(const nsAReadableString& aUrl)
 }
 
 NS_IMETHODIMP
-LocationImpl::ToString(nsAWritableString& aReturn)
+LocationImpl::ToString(nsAString& aReturn)
 {
   return GetHref(aReturn);
 }
