@@ -64,6 +64,7 @@
 #include "nsIConsoleService.h"
 #include "nsXPIDLString.h"
 #include "prprf.h"
+#include "nsEscape.h"
 
 static NS_DEFINE_CID(kSimpleURICID, NS_SIMPLEURI_CID);
 static NS_DEFINE_CID(kWindowMediatorCID, NS_WINDOWMEDIATOR_CID);
@@ -137,6 +138,9 @@ nsresult nsJSThunk::EvaluateScript()
         return NS_ERROR_DOM_RETVAL_UNDEFINED;
     }
 
+    // Unescape the script
+    NS_UnescapeURL(script);
+
     // Get the url.
     nsCAutoString url;
     rv = mURI->GetSpec(url);
@@ -335,7 +339,13 @@ nsresult nsJSThunk::BringUpConsole()
 // nsIStreamIO implementation...
 //
 NS_IMETHODIMP
-nsJSThunk::Open(char* *contentType, PRInt32 *contentLength)
+nsJSThunk::Open()
+{
+    return NS_OK;
+}
+
+NS_IMETHODIMP 
+nsJSThunk::GetContentType(nsACString &aContentType)
 {
     //
     // At this point the script has already been evaluated...
@@ -343,9 +353,22 @@ nsJSThunk::Open(char* *contentType, PRInt32 *contentLength)
     //
     // If the resultant script evaluation actually does return a value, we
     // treat it as html.
-    *contentType = nsCRT::strdup("text/html");
-    *contentLength = mLength;
+    //
+    aContentType = NS_LITERAL_CSTRING("text/html");
+    return NS_OK;
+}
 
+NS_IMETHODIMP
+nsJSThunk::GetContentCharset(nsACString &aContentCharset)
+{
+    aContentCharset.Truncate();
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsJSThunk::GetContentLength(PRInt32 *result)
+{
+    *result = mLength;
     return NS_OK;
 }
 
@@ -387,15 +410,9 @@ nsJSThunk::GetOutputStream(nsIOutputStream* *aOutputStream)
 }
 
 NS_IMETHODIMP
-nsJSThunk::GetName(char* *aName)
+nsJSThunk::GetName(nsACString &aName)
 {
-    nsCAutoString buf;
-
-    nsresult rv = mURI->GetSpec(buf);
-    if (NS_FAILED(rv)) return rv;
-
-    *aName = ToNewCString(buf);
-    return *aName ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+    return mURI->GetSpec(aName);
 }
 
 
@@ -483,7 +500,7 @@ NS_INTERFACE_MAP_END
 //
 
 NS_IMETHODIMP
-nsJSChannel::GetName(PRUnichar * *aResult)
+nsJSChannel::GetName(nsACString &aResult)
 {
     return mStreamChannel->GetName(aResult);
 }
@@ -664,17 +681,29 @@ nsJSChannel::GetSecurityInfo(nsISupports * *aSecurityInfo)
 }
 
 NS_IMETHODIMP
-nsJSChannel::GetContentType(char * *aContentType)
+nsJSChannel::GetContentType(nsACString &aContentType)
 {
     return mStreamChannel->GetContentType(aContentType);
 }
 
 NS_IMETHODIMP
-nsJSChannel::SetContentType(const char *aContentType)
+nsJSChannel::SetContentType(const nsACString &aContentType)
 {
     return mStreamChannel->SetContentType(aContentType);
 }
 
+NS_IMETHODIMP
+nsJSChannel::GetContentCharset(nsACString &aContentCharset)
+{
+    return mStreamChannel->GetContentCharset(aContentCharset);
+}
+
+NS_IMETHODIMP
+nsJSChannel::SetContentCharset(const nsACString &aContentCharset)
+{
+    return mStreamChannel->SetContentCharset(aContentCharset);
+}
+
 NS_IMETHODIMP
 nsJSChannel::GetContentLength(PRInt32 *aContentLength)
 {
