@@ -119,13 +119,13 @@ static const PRBool gNoisy = PR_FALSE;
 // Add the CSS style corresponding to the HTML inline style defined
 // by aProperty aAttribute and aValue to the selection
 NS_IMETHODIMP nsHTMLEditor::SetCSSInlineProperty(nsIAtom *aProperty, 
-                            const nsAReadableString & aAttribute, 
-                            const nsAReadableString & aValue)
+                            const nsAString & aAttribute, 
+                            const nsAString & aValue)
 {
   nsresult res = NS_OK;
   PRBool useCSS;
 
-  IsCSSEnabled(&useCSS);
+  GetIsCSSEnabled(&useCSS);
   if (useCSS) {
     res = SetInlineProperty(aProperty, aAttribute, aValue);
   }
@@ -133,8 +133,8 @@ NS_IMETHODIMP nsHTMLEditor::SetCSSInlineProperty(nsIAtom *aProperty,
 }
 
 NS_IMETHODIMP nsHTMLEditor::SetInlineProperty(nsIAtom *aProperty, 
-                            const nsAReadableString & aAttribute, 
-                            const nsAReadableString & aValue)
+                            const nsAString & aAttribute, 
+                            const nsAString & aValue)
 {
   if (!aProperty) { return NS_ERROR_NULL_POINTER; }
   if (!mRules) { return NS_ERROR_NOT_INITIALIZED; }
@@ -313,8 +313,8 @@ nsHTMLEditor::SetInlinePropertyOnTextNode( nsIDOMCharacterData *aTextNode,
                                             PRInt32 aStartOffset,
                                             PRInt32 aEndOffset,
                                             nsIAtom *aProperty, 
-                                            const nsAReadableString *aAttribute,
-                                            const nsAReadableString *aValue)
+                                            const nsAString *aAttribute,
+                                            const nsAString *aValue)
 {
   if (!aTextNode) return NS_ERROR_NULL_POINTER;
   nsresult res = NS_OK;
@@ -385,8 +385,8 @@ nsHTMLEditor::SetInlinePropertyOnTextNode( nsIDOMCharacterData *aTextNode,
 nsresult
 nsHTMLEditor::SetInlinePropertyOnNode( nsIDOMNode *aNode,
                                        nsIAtom *aProperty, 
-                                       const nsAReadableString *aAttribute,
-                                       const nsAReadableString *aValue)
+                                       const nsAString *aAttribute,
+                                       const nsAString *aValue)
 {
   if (!aNode || !aProperty) return NS_ERROR_NULL_POINTER;
 
@@ -397,7 +397,7 @@ nsHTMLEditor::SetInlinePropertyOnNode( nsIDOMNode *aNode,
   ToLowerCase(tag);
   
   PRBool useCSS;
-  IsCSSEnabled(&useCSS);
+  GetIsCSSEnabled(&useCSS);
 
   if (useCSS) {
     // we are in CSS mode
@@ -524,7 +524,7 @@ nsHTMLEditor::SetInlinePropertyOnNode( nsIDOMNode *aNode,
 
 nsresult nsHTMLEditor::SplitStyleAboveRange(nsIDOMRange *inRange, 
                                             nsIAtom *aProperty, 
-                                            const nsAReadableString *aAttribute)
+                                            const nsAString *aAttribute)
 {
   if (!inRange) return NS_ERROR_NULL_POINTER;
   nsresult res;
@@ -568,7 +568,7 @@ nsresult nsHTMLEditor::SplitStyleAboveRange(nsIDOMRange *inRange,
 nsresult nsHTMLEditor::SplitStyleAbovePoint(nsCOMPtr<nsIDOMNode> *aNode,
                                            PRInt32 *aOffset,
                                            nsIAtom *aProperty,          // null here means we split all properties
-                                           const nsAReadableString *aAttribute,
+                                           const nsAString *aAttribute,
                                            nsCOMPtr<nsIDOMNode> *outLeftNode,
                                            nsCOMPtr<nsIDOMNode> *outRightNode)
 {
@@ -580,7 +580,7 @@ nsresult nsHTMLEditor::SplitStyleAbovePoint(nsCOMPtr<nsIDOMNode> *aNode,
   PRInt32 offset;
 
   PRBool useCSS;
-  IsCSSEnabled(&useCSS);
+  GetIsCSSEnabled(&useCSS);
 
   PRBool isSet;
   while (tmp && !IsBlockNode(tmp))
@@ -624,7 +624,7 @@ PRBool nsHTMLEditor::NodeIsProperty(nsIDOMNode *aNode)
 
 nsresult nsHTMLEditor::RemoveStyleInside(nsIDOMNode *aNode, 
                                    nsIAtom *aProperty,   // null here means remove all properties
-                                   const nsAReadableString *aAttribute, 
+                                   const nsAString *aAttribute, 
                                    PRBool aChildrenOnly)
 {
   if (!aNode) return NS_ERROR_NULL_POINTER;
@@ -646,7 +646,8 @@ nsresult nsHTMLEditor::RemoveStyleInside(nsIDOMNode *aNode,
   // then process the node itself
   if ( !aChildrenOnly && 
         (aProperty && NodeIsType(aNode, aProperty) || // node is prop we asked for
-        (aProperty == nsIEditProperty::href && nsHTMLEditUtils::IsLink(aNode))) || // but check for link (<a href=...)
+        (aProperty == nsIEditProperty::href && nsHTMLEditUtils::IsLink(aNode)) || // but check for link (<a href=...)
+        (aProperty == nsIEditProperty::name && nsHTMLEditUtils::IsNamedAnchor(aNode))) || // and for named anchors
         (!aProperty && NodeIsProperty(aNode)))  // or node is any prop and we asked for that
   {
     // if we weren't passed an attribute, then we want to 
@@ -677,7 +678,7 @@ nsresult nsHTMLEditor::RemoveStyleInside(nsIDOMNode *aNode,
   }
   else {
     PRBool useCSS;
-    IsCSSEnabled(&useCSS);
+    GetIsCSSEnabled(&useCSS);
 
     if (!aChildrenOnly
         && useCSS && mHTMLCSSUtils->IsCSSEditableProperty(aNode, aProperty, aAttribute)) {
@@ -712,7 +713,7 @@ nsresult nsHTMLEditor::RemoveStyleInside(nsIDOMNode *aNode,
 }
 
 PRBool nsHTMLEditor::IsOnlyAttribute(nsIDOMNode *aNode, 
-                                     const nsAReadableString *aAttribute)
+                                     const nsAString *aAttribute)
 {
   if (!aNode || !aAttribute) return PR_FALSE;  // ooops
   nsCOMPtr<nsIContent> content = do_QueryInterface(aNode);
@@ -743,7 +744,7 @@ PRBool nsHTMLEditor::IsOnlyAttribute(nsIDOMNode *aNode,
 }
 
 PRBool nsHTMLEditor::HasAttr(nsIDOMNode *aNode, 
-                             const nsAReadableString *aAttribute)
+                             const nsAString *aAttribute)
 {
   if (!aNode) return PR_FALSE;
   if (!aAttribute || aAttribute->IsEmpty()) return PR_TRUE;  // everybody has the 'null' attribute
@@ -761,8 +762,8 @@ PRBool nsHTMLEditor::HasAttr(nsIDOMNode *aNode,
 
 
 PRBool nsHTMLEditor::HasAttrVal(nsIDOMNode *aNode, 
-                                const nsAReadableString *aAttribute, 
-                                const nsAReadableString *aValue)
+                                const nsAString *aAttribute, 
+                                const nsAString *aValue)
 {
   if (!aNode) return PR_FALSE;
   if (!aAttribute || aAttribute->IsEmpty()) return PR_TRUE;  // everybody has the 'null' attribute
@@ -791,6 +792,63 @@ PRBool nsHTMLEditor::HasAttrVal(nsIDOMNode *aNode,
   return PR_FALSE;
 }
 
+nsresult nsHTMLEditor::PromoteRangeIfStartsOrEndsInNamedAnchor(nsIDOMRange *inRange)
+{
+  if (!inRange) return NS_ERROR_NULL_POINTER;
+  nsresult res;
+  nsCOMPtr<nsIDOMNode> startNode, endNode, parent, tmp;
+  PRInt32 startOffset, endOffset, tmpOffset;
+  
+  res = inRange->GetStartContainer(getter_AddRefs(startNode));
+  if (NS_FAILED(res)) return res;
+  res = inRange->GetStartOffset(&startOffset);
+  if (NS_FAILED(res)) return res;
+  res = inRange->GetEndContainer(getter_AddRefs(endNode));
+  if (NS_FAILED(res)) return res;
+  res = inRange->GetEndOffset(&endOffset);
+  if (NS_FAILED(res)) return res;
+
+  tmp = startNode;
+  while ( tmp && 
+          !nsTextEditUtils::IsBody(tmp) &&
+          !nsHTMLEditUtils::IsNamedAnchor(tmp))
+  {
+    res = GetNodeLocation(tmp, address_of(parent), &tmpOffset);
+    if (NS_FAILED(res)) return res;
+    tmp = parent;
+  }
+  if (!tmp) return NS_ERROR_NULL_POINTER;
+  if (nsHTMLEditUtils::IsNamedAnchor(tmp))
+  {
+    res = GetNodeLocation(tmp, address_of(parent), &tmpOffset);
+    if (NS_FAILED(res)) return res;
+    startNode = parent;
+    startOffset = tmpOffset;
+  }
+
+  tmp = endNode;
+  while ( tmp && 
+          !nsTextEditUtils::IsBody(tmp) &&
+          !nsHTMLEditUtils::IsNamedAnchor(tmp))
+  {
+    res = GetNodeLocation(tmp, address_of(parent), &tmpOffset);
+    if (NS_FAILED(res)) return res;
+    tmp = parent;
+  }
+  if (!tmp) return NS_ERROR_NULL_POINTER;
+  if (nsHTMLEditUtils::IsNamedAnchor(tmp))
+  {
+    res = GetNodeLocation(tmp, address_of(parent), &tmpOffset);
+    if (NS_FAILED(res)) return res;
+    endNode = parent;
+    endOffset = tmpOffset + 1;
+  }
+
+  res = inRange->SetStart(startNode, startOffset);
+  if (NS_FAILED(res)) return res;
+  res = inRange->SetEnd(endNode, endOffset);
+  return res;
+}
 
 nsresult nsHTMLEditor::PromoteInlineRange(nsIDOMRange *inRange)
 {
@@ -882,12 +940,12 @@ PRBool nsHTMLEditor::IsAtEndOfNode(nsIDOMNode *aNode, PRInt32 aOffset)
 
 nsresult
 nsHTMLEditor::GetInlinePropertyBase(nsIAtom *aProperty, 
-                             const nsAReadableString *aAttribute,
-                             const nsAReadableString *aValue,
+                             const nsAString *aAttribute,
+                             const nsAString *aValue,
                              PRBool *aFirst, 
                              PRBool *aAny, 
                              PRBool *aAll,
-                             nsAWritableString *outValue)
+                             nsAString *outValue)
 {
   if (!aProperty)
     return NS_ERROR_NULL_POINTER;
@@ -908,7 +966,7 @@ nsHTMLEditor::GetInlinePropertyBase(nsIAtom *aProperty,
   PRBool first=PR_TRUE;
 
   PRBool useCSS;
-  IsCSSEnabled(&useCSS);
+  GetIsCSSEnabled(&useCSS);
 
   nsCOMPtr<nsISelection>selection;
   result = GetSelection(getter_AddRefs(selection));
@@ -1090,18 +1148,18 @@ nsHTMLEditor::GetInlinePropertyBase(nsIAtom *aProperty,
 
 
 NS_IMETHODIMP nsHTMLEditor::GetInlineProperty(nsIAtom *aProperty, 
-                                              const nsAReadableString &aAttribute, 
-                                              const nsAReadableString &aValue,
+                                              const nsAString &aAttribute, 
+                                              const nsAString &aValue,
                                               PRBool *aFirst, 
                                               PRBool *aAny, 
                                               PRBool *aAll)
 {
   if (!aProperty || !aFirst || !aAny || !aAll)
     return NS_ERROR_NULL_POINTER;
-  const nsAReadableString *att = nsnull;
+  const nsAString *att = nsnull;
   if (aAttribute.Length())
     att = &aAttribute;
-  const nsAReadableString *val = nsnull;
+  const nsAString *val = nsnull;
   if (aValue.Length())
     val = &aValue;
   return GetInlinePropertyBase( aProperty, att, val, aFirst, aAny, aAll, nsnull);
@@ -1109,19 +1167,19 @@ NS_IMETHODIMP nsHTMLEditor::GetInlineProperty(nsIAtom *aProperty,
 
 
 NS_IMETHODIMP nsHTMLEditor::GetInlinePropertyWithAttrValue(nsIAtom *aProperty, 
-                                              const nsAReadableString &aAttribute, 
-                                              const nsAReadableString &aValue,
+                                              const nsAString &aAttribute, 
+                                              const nsAString &aValue,
                                               PRBool *aFirst, 
                                               PRBool *aAny, 
                                               PRBool *aAll,
-                                              nsAWritableString &outValue)
+                                              nsAString &outValue)
 {
   if (!aProperty || !aFirst || !aAny || !aAll)
     return NS_ERROR_NULL_POINTER;
-  const nsAReadableString *att = nsnull;
+  const nsAString *att = nsnull;
   if (aAttribute.Length())
     att = &aAttribute;
-  const nsAReadableString *val = nsnull;
+  const nsAString *val = nsnull;
   if (aValue.Length())
     val = &aValue;
   return GetInlinePropertyBase( aProperty, att, val, aFirst, aAny, aAll, &outValue);
@@ -1133,12 +1191,12 @@ NS_IMETHODIMP nsHTMLEditor::RemoveAllInlineProperties()
   return RemoveInlinePropertyImpl(nsnull, nsnull);
 }
 
-NS_IMETHODIMP nsHTMLEditor::RemoveInlineProperty(nsIAtom *aProperty, const nsAReadableString &aAttribute)
+NS_IMETHODIMP nsHTMLEditor::RemoveInlineProperty(nsIAtom *aProperty, const nsAString &aAttribute)
 {
   return RemoveInlinePropertyImpl(aProperty, &aAttribute);
 }
 
-nsresult nsHTMLEditor::RemoveInlinePropertyImpl(nsIAtom *aProperty, const nsAReadableString *aAttribute)
+nsresult nsHTMLEditor::RemoveInlinePropertyImpl(nsIAtom *aProperty, const nsAString *aAttribute)
 {
   if (!mRules)    return NS_ERROR_NOT_INITIALIZED;
   ForceCompositionEnd();
@@ -1154,14 +1212,15 @@ nsresult nsHTMLEditor::RemoveInlinePropertyImpl(nsIAtom *aProperty, const nsARea
   selection->GetIsCollapsed(&isCollapsed);
 
   PRBool useCSS;
-  IsCSSEnabled(&useCSS);
+  GetIsCSSEnabled(&useCSS);
 
   if (isCollapsed)
   {
     // manipulating text attributes on a collapsed selection only sets state for the next text insertion
 
     // For links, aProperty uses "href", use "a" instead
-    if (aProperty == nsIEditProperty::href)
+    if (aProperty == nsIEditProperty::href ||
+        aProperty == nsIEditProperty::name)
       aProperty = nsIEditProperty::a;
 
     if (aProperty) return mTypeInState->ClearProp(aProperty, nsAutoString(*aAttribute));
@@ -1195,10 +1254,18 @@ nsresult nsHTMLEditor::RemoveInlinePropertyImpl(nsIAtom *aProperty, const nsARea
       
       nsCOMPtr<nsIDOMRange> range( do_QueryInterface(currentItem) );
 
-      // adjust range to include any ancestors who's children are entirely selected
-      res = PromoteInlineRange(range);
+      if (aProperty == nsIEditProperty::name)
+      {
+        // promote range if it starts or end in a named anchor and we
+        // want to remove named anchors
+        res = PromoteRangeIfStartsOrEndsInNamedAnchor(range);
+      }
+      else {
+        // adjust range to include any ancestors who's children are entirely selected
+        res = PromoteInlineRange(range);
+      }
       if (NS_FAILED(res)) return res;
-      
+
       // remove this style from ancestors of our range endpoints, 
       // splitting them as appropriate
       res = SplitStyleAboveRange(range, aProperty, aAttribute);
@@ -1727,7 +1794,7 @@ nsHTMLEditor::RelativeFontChangeOnNode( PRInt32 aSizeChange,
 }
 
 NS_IMETHODIMP 
-nsHTMLEditor::GetFontFaceState(PRBool *aMixed, nsAWritableString &outFace)
+nsHTMLEditor::GetFontFaceState(PRBool *aMixed, nsAString &outFace)
 {
   if (!aMixed)
       return NS_ERROR_FAILURE;
@@ -1766,7 +1833,7 @@ nsHTMLEditor::GetFontFaceState(PRBool *aMixed, nsAWritableString &outFace)
 }
 
 NS_IMETHODIMP 
-nsHTMLEditor::GetFontColorState(PRBool *aMixed, nsAWritableString &aOutColor)
+nsHTMLEditor::GetFontColorState(PRBool *aMixed, nsAString &aOutColor)
 {
   if (!aMixed)
       return NS_ERROR_NULL_POINTER;
@@ -1799,7 +1866,7 @@ nsHTMLEditor::GetFontColorState(PRBool *aMixed, nsAWritableString &aOutColor)
 // can handle CSS styles (for instance, Composer can, Messenger can't) and if
 // the CSS preference is checked
 nsresult
-nsHTMLEditor::IsCSSEnabled(PRBool *aIsCSSEnabled)
+nsHTMLEditor::GetIsCSSEnabled(PRBool *aIsCSSEnabled)
 {
   *aIsCSSEnabled = PR_FALSE;
   if (mCSSAware) {
