@@ -99,6 +99,7 @@
 #include "nsIPref.h"
 #include "nsStyleConsts.h"
 #include "nsIStyleContext.h"
+#include "nsUnicharUtils.h"
 
 #include "nsAOLCiter.h"
 #include "nsInternetCiter.h"
@@ -112,11 +113,6 @@
 
 const PRUnichar nbsp = 160;
 
-// HACK - CID for NS_CTRANSITIONAL_DTD_CID so that we can get at transitional dtd
-#define NS_CTRANSITIONAL_DTD_CID \
-{ 0x4611d482, 0x960a, 0x11d4, { 0x8e, 0xb0, 0xb6, 0x17, 0x66, 0x1b, 0x6f, 0x7c } }
-
-
 static NS_DEFINE_CID(kCContentIteratorCID, NS_CONTENTITERATOR_CID);
 static NS_DEFINE_CID(kCRangeCID,      NS_RANGE_CID);
 static NS_DEFINE_CID(kCDOMSelectionCID,      NS_DOMSELECTION_CID);
@@ -256,7 +252,7 @@ nsPlaintextEditor::EndEditorInit()
 }
 
 NS_IMETHODIMP 
-nsPlaintextEditor::SetDocumentCharacterSet(const nsAReadableString & characterSet) 
+nsPlaintextEditor::SetDocumentCharacterSet(const nsAString & characterSet) 
 { 
   nsresult result; 
 
@@ -267,7 +263,6 @@ nsPlaintextEditor::SetDocumentCharacterSet(const nsAReadableString & characterSe
     nsCOMPtr<nsIDOMDocument>domdoc; 
     result = GetDocument(getter_AddRefs(domdoc)); 
     if (NS_SUCCEEDED(result) && domdoc) { 
-      nsAutoString newMetaString; 
       nsCOMPtr<nsIDOMNodeList>metaList; 
       nsCOMPtr<nsIDOMNode>metaNode; 
       nsCOMPtr<nsIDOMElement>metaElement; 
@@ -285,20 +280,26 @@ nsPlaintextEditor::SetDocumentCharacterSet(const nsAReadableString & characterSe
           metaElement = do_QueryInterface(metaNode); 
           if (!metaElement) continue; 
 
-          nsString currentValue; 
+          nsAutoString currentValue; 
           if (NS_FAILED(metaElement->GetAttribute(NS_LITERAL_STRING("http-equiv"), currentValue))) continue; 
 
-          if (kNotFound != currentValue.Find("content-type", PR_TRUE)) { 
+          if (FindInReadable(NS_LITERAL_STRING("content-type"),
+                             currentValue,
+                             nsCaseInsensitiveStringComparator())) { 
             NS_NAMED_LITERAL_STRING(content, "content");
             if (NS_FAILED(metaElement->GetAttribute(content, currentValue))) continue; 
 
-            NS_NAMED_LITERAL_STRING(charset, "charset=");
-            PRInt32 offset = currentValue.Find(charset.get(), PR_TRUE); 
-            if (kNotFound != offset) {
-              currentValue.Left(newMetaString, offset); // copy current value before "charset=" (e.g. text/html) 
-              newMetaString.Append(charset); 
-              newMetaString.Append(characterSet); 
-              result = nsEditor::SetAttribute(metaElement, content, newMetaString); 
+            NS_NAMED_LITERAL_STRING(charsetEquals, "charset=");
+            nsAString::const_iterator originalStart, start, end;
+            originalStart = currentValue.BeginReading(start);
+            currentValue.EndReading(end);
+            if (FindInReadable(charsetEquals, start, end,
+                               nsCaseInsensitiveStringComparator())) {
+
+              // set attribute to <original prefix> charset=text/html
+              result = nsEditor::SetAttribute(metaElement, content,
+                                              Substring(originalStart, start) +
+                                              charsetEquals + characterSet); 
               if (NS_SUCCEEDED(result)) 
                 newMetaCharset = PR_FALSE; 
               break; 
@@ -327,10 +328,9 @@ nsPlaintextEditor::SetDocumentCharacterSet(const nsAReadableString & characterSe
                 // not undoable, undo should undo CreateNode 
                 result = metaElement->SetAttribute(NS_LITERAL_STRING("http-equiv"), NS_LITERAL_STRING("Content-Type")); 
                 if (NS_SUCCEEDED(result)) { 
-                  newMetaString.Assign(NS_LITERAL_STRING("text/html;charset=")); 
-                  newMetaString.Append(characterSet); 
                   // not undoable, undo should undo CreateNode 
-                  result = metaElement->SetAttribute(NS_LITERAL_STRING("content"), newMetaString); 
+                  result = metaElement->SetAttribute(NS_LITERAL_STRING("content"),
+                                                     NS_LITERAL_STRING("text/html;charset=") + characterSet); 
                 } 
               } 
             } 
@@ -538,7 +538,7 @@ NS_IMETHODIMP nsPlaintextEditor::HandleKeyPress(nsIDOMKeyEvent* aKeyEvent)
    to TypedText() to determine what action to take, but without passing
    an event.
    */
-NS_IMETHODIMP nsPlaintextEditor::TypedText(const nsAReadableString& aString,
+NS_IMETHODIMP nsPlaintextEditor::TypedText(const nsAString& aString,
                                       PRInt32 aAction)
 {
   nsAutoPlaceHolderBatch batch(this, gTypingTxnName);
@@ -949,7 +949,7 @@ NS_IMETHODIMP nsPlaintextEditor::DeleteSelection(nsIEditor::EDirection aAction)
   return result;
 }
 
-NS_IMETHODIMP nsPlaintextEditor::InsertText(const nsAReadableString &aStringToInsert)
+NS_IMETHODIMP nsPlaintextEditor::InsertText(const nsAString &aStringToInsert)
 {
   if (!mRules) { return NS_ERROR_NOT_INITIALIZED; }
 
@@ -1427,9 +1427,9 @@ NS_IMETHODIMP nsPlaintextEditor::CanCopy(PRBool *aCanCopy)
 
 // Shared between OutputToString and OutputToStream
 NS_IMETHODIMP
-nsPlaintextEditor::GetAndInitDocEncoder(const nsAReadableString& aFormatType,
+nsPlaintextEditor::GetAndInitDocEncoder(const nsAString& aFormatType,
                                         PRUint32 aFlags,
-                                        const nsAReadableString& aCharset,
+                                        const nsAString& aCharset,
                                         nsIDocumentEncoder** encoder)
 {
   nsCOMPtr<nsIPresShell> presShell;
@@ -1515,9 +1515,9 @@ nsPlaintextEditor::GetAndInitDocEncoder(const nsAReadableString& aFormatType,
 
 
 NS_IMETHODIMP 
-nsPlaintextEditor::OutputToString(nsAWritableString& aOutputString,
-                                  const nsAReadableString& aFormatType,
-                                  PRUint32 aFlags)
+nsPlaintextEditor::OutputToString(const nsAString& aFormatType,
+                                  PRUint32 aFlags,
+                                  nsAString& aOutputString)
 {
   PRBool cancel, handled;
   nsString resultString;
@@ -1550,8 +1550,8 @@ nsPlaintextEditor::OutputToString(nsAWritableString& aOutputString,
 
 NS_IMETHODIMP
 nsPlaintextEditor::OutputToStream(nsIOutputStream* aOutputStream,
-                             const nsAReadableString& aFormatType,
-                             const nsAReadableString& aCharset,
+                             const nsAString& aFormatType,
+                             const nsAString& aCharset,
                              PRUint32 aFlags)
 {
   nsresult rv;
@@ -1676,7 +1676,7 @@ static nsICiter* MakeACiter()
 }
 
 NS_IMETHODIMP
-nsPlaintextEditor::InsertAsQuotation(const nsAReadableString& aQuotedText,
+nsPlaintextEditor::InsertAsQuotation(const nsAString& aQuotedText,
                                      nsIDOMNode **aNodeInserted)
 {
   // We have the text.  Cite it appropriately:
@@ -1723,17 +1723,17 @@ nsPlaintextEditor::InsertAsQuotation(const nsAReadableString& aQuotedText,
 }
 
 NS_IMETHODIMP
-nsPlaintextEditor::PasteAsCitedQuotation(const nsAReadableString& aCitation,
+nsPlaintextEditor::PasteAsCitedQuotation(const nsAString& aCitation,
                                          PRInt32 aSelectionType)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsPlaintextEditor::InsertAsCitedQuotation(const nsAReadableString& aQuotedText,
-                                          const nsAReadableString& aCitation,
+nsPlaintextEditor::InsertAsCitedQuotation(const nsAString& aQuotedText,
+                                          const nsAString& aCitation,
                                           PRBool aInsertHTML,
-                                          const nsAReadableString& aCharset,
+                                          const nsAString& aCharset,
                                           nsIDOMNode **aNodeInserted)
 {
   return InsertAsQuotation(aQuotedText, aNodeInserted);
@@ -1772,9 +1772,10 @@ nsPlaintextEditor::Rewrap(PRBool aRespectNewlines)
 
   if (isCollapsed)    // rewrap the whole document
   {
-    rv = OutputToString(current, format,
+    rv = OutputToString(format,
                         nsIDocumentEncoder::OutputFormatted
-                        | nsIDocumentEncoder::OutputLFLineBreak);
+                        | nsIDocumentEncoder::OutputLFLineBreak,
+                        current);
     if (NS_FAILED(rv)) return rv;
 
     nsCOMPtr<nsICiter> citer = dont_AddRef(MakeACiter());
@@ -1791,10 +1792,11 @@ nsPlaintextEditor::Rewrap(PRBool aRespectNewlines)
   }
   else                // rewrap only the selection
   {
-    rv = OutputToString(current, format,
+    rv = OutputToString(format,
                         nsIDocumentEncoder::OutputFormatted
                         | nsIDocumentEncoder::OutputLFLineBreak
-                        | nsIDocumentEncoder::OutputSelectionOnly);
+                        | nsIDocumentEncoder::OutputSelectionOnly,
+                        current);
     if (NS_FAILED(rv)) return rv;
 
     nsCOMPtr<nsICiter> citer = dont_AddRef(MakeACiter());
@@ -1835,8 +1837,7 @@ nsPlaintextEditor::StripCites()
 
   if (isCollapsed)    // rewrap the whole document
   {
-    rv = OutputToString(current, format,
-                               nsIDocumentEncoder::OutputFormatted);
+    rv = OutputToString(format, nsIDocumentEncoder::OutputFormatted, current);
     if (NS_FAILED(rv)) return rv;
 
     nsCOMPtr<nsICiter> citer = dont_AddRef(MakeACiter());
@@ -1853,9 +1854,10 @@ nsPlaintextEditor::StripCites()
   }
   else                // rewrap only the selection
   {
-    rv = OutputToString(current, format,
-                               nsIDocumentEncoder::OutputFormatted
-                                | nsIDocumentEncoder::OutputSelectionOnly);
+    rv = OutputToString(format,
+                        nsIDocumentEncoder::OutputFormatted
+                        | nsIDocumentEncoder::OutputSelectionOnly,
+                        current);
     if (NS_FAILED(rv)) return rv;
 
     nsCOMPtr<nsICiter> citer = dont_AddRef(MakeACiter());
@@ -1885,7 +1887,7 @@ nsPlaintextEditor::GetEmbeddedObjects(nsISupportsArray** aNodeList)
 #endif
 
 NS_IMETHODIMP
-nsPlaintextEditor::SetCompositionString(const nsAReadableString& aCompositionString, nsIPrivateTextRangeList* aTextRangeList,nsTextEventReply* aReply)
+nsPlaintextEditor::SetCompositionString(const nsAString& aCompositionString, nsIPrivateTextRangeList* aTextRangeList,nsTextEventReply* aReply)
 {
   NS_ASSERTION(aTextRangeList, "null ptr");
   if(nsnull == aTextRangeList)
@@ -1906,26 +1908,35 @@ nsPlaintextEditor::SetCompositionString(const nsAReadableString& aCompositionStr
   if (NS_FAILED(result)) return result;
 
   mIMETextRangeList = aTextRangeList;
-  nsAutoPlaceHolderBatch batch(this, gIMETxnName);
 
-  result = InsertText(aCompositionString);
-
-  mIMEBufferLength = aCompositionString.Length();
-
-  if (!mPresShellWeak) return NS_ERROR_NOT_INITIALIZED;
-  nsCOMPtr<nsIPresShell> ps = do_QueryReferent(mPresShellWeak);
-  if (!ps) return NS_ERROR_NOT_INITIALIZED;
-  ps->GetCaret(getter_AddRefs(caretP));
-  caretP->SetCaretDOMSelection(selection);
-
-  caretP->GetCaretCoordinates(nsICaret::eIMECoordinates, selection,
-            &(aReply->mCursorPosition), &(aReply->mCursorIsCollapsed));
-
-  // second part of 23558 fix:
-  if (aCompositionString.IsEmpty()) 
+  // we need the nsAutoPlaceHolderBatch destructor called before hitting
+  // GetCaretCoordinates so the states in Frame system sync with content
+  // therefore, we put the nsAutoPlaceHolderBatch into a inner block
   {
-    mIMETextNode = nsnull;
+    nsAutoPlaceHolderBatch batch(this, gIMETxnName);
+
+    result = InsertText(aCompositionString);
+
+    mIMEBufferLength = aCompositionString.Length();
+
+    if (!mPresShellWeak) 
+      return NS_ERROR_NOT_INITIALIZED;
+    nsCOMPtr<nsIPresShell> ps = do_QueryReferent(mPresShellWeak);
+    if (!ps) 
+      return NS_ERROR_NOT_INITIALIZED;
+    ps->GetCaret(getter_AddRefs(caretP));
+    caretP->SetCaretDOMSelection(selection);
+
+    // second part of 23558 fix:
+    if (aCompositionString.IsEmpty()) 
+    {
+      mIMETextNode = nsnull;
+    }
   }
+
+  result = caretP->GetCaretCoordinates(nsICaret::eIMECoordinates, selection,
+            &(aReply->mCursorPosition), &(aReply->mCursorIsCollapsed));
+  NS_ASSERTION(NS_SUCCEEDED(result), "cannot get caret position");
   
   return result;
 }
@@ -2107,9 +2118,15 @@ void nsPlaintextEditor::HandleEventListenerError()
 
 nsresult
 nsPlaintextEditor::SetAttributeOrEquivalent(nsIDOMElement * aElement,
-                                            const nsAReadableString & aAttribute,
-                                            const nsAReadableString & aValue)
+                                            const nsAString & aAttribute,
+                                            const nsAString & aValue)
 {
-  nsEditor::SetAttribute(aElement, aAttribute, aValue);
-  return NS_OK;
+  return nsEditor::SetAttribute(aElement, aAttribute, aValue);
+}
+
+nsresult
+nsPlaintextEditor::RemoveAttributeOrEquivalent(nsIDOMElement * aElement,
+                                               const nsAString & aAttribute)
+{
+  return nsEditor::RemoveAttribute(aElement, aAttribute);
 }
