@@ -59,6 +59,9 @@
 #include "nsWeakReference.h"
 #include "nsIChannel.h"
 #include "nsIWidget.h"
+#include "nsIWebBrowserPrint.h"
+#include "nsIMacTextInputEventSink.h"
+#include "nsCRT.h"
 
 // Local
 #include "ApplIDs.h"
@@ -125,7 +128,7 @@ CBrowserShellProgressListener::~CBrowserShellProgressListener()
 {
 }
 
-NS_IMETHODIMP CBrowserShellProgressListener::OnStateChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRInt32 aStateFlags, PRUint32 aStatus)
+NS_IMETHODIMP CBrowserShellProgressListener::OnStateChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRUint32 aStateFlags, nsresult aStatus)
 {
     NS_ENSURE_TRUE(mpOwner, NS_ERROR_NULL_POINTER);
 
@@ -171,9 +174,9 @@ NS_IMETHODIMP CBrowserShellProgressListener::OnStateChange(nsIWebProgress *aWebP
         {
             nsCOMPtr<nsIChannel> channel(do_QueryInterface(aRequest));
             NS_ENSURE_TRUE(channel, NS_ERROR_FAILURE);
-            nsXPIDLCString contentType;
-            channel->GetContentType(getter_Copies(contentType));
-            if (strcmp(contentType.get(), "text/html"))
+            nsCAutoString contentType;
+            channel->GetContentType(contentType);
+            if (contentType.Equals(NS_LITERAL_CSTRING("text/html")))
                 mUseRealProgFlag = true;
         }
         
@@ -227,9 +230,14 @@ NS_IMETHODIMP CBrowserShellProgressListener::OnStatusChange(nsIWebProgress *aWeb
     return NS_OK;
 }
 
-NS_IMETHODIMP CBrowserShellProgressListener::OnSecurityChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRInt32 state)
+NS_IMETHODIMP CBrowserShellProgressListener::OnSecurityChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRUint32 state)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    NS_ENSURE_TRUE(mpOwner, NS_ERROR_NULL_POINTER);
+
+    MsgSecurityChangeInfo info(mpOwner, state);
+    mpOwner->BroadcastMessage(msg_OnSecurityChange, &info);
+    
+    return NS_OK;
 }
 
 
@@ -872,6 +880,25 @@ NS_METHOD CBrowserShell::GetContentViewer(nsIContentViewer** aViewer)
 }
 
 
+NS_METHOD CBrowserShell::GetPrintSettings(nsIPrintSettings** aSettings)
+{
+    NS_ENSURE_ARG_POINTER(aSettings);
+    *aSettings = nsnull;
+    
+    if (!mPrintSettings) {
+        // If we don't have print settings yet, make new ones.
+        nsCOMPtr<nsIWebBrowserPrint> wbPrint(do_GetInterface(mWebBrowser));
+        NS_ENSURE_TRUE(wbPrint, NS_ERROR_NO_INTERFACE);
+        wbPrint->GetNewPrintSettings(getter_AddRefs(mPrintSettings));
+    }
+    if (mPrintSettings) {
+        *aSettings = mPrintSettings;
+        NS_ADDREF(*aSettings);
+        return NS_OK;
+    }
+    return NS_ERROR_FAILURE;
+}
+
 //*****************************************************************************
 //***    CBrowserShell: Navigation
 //*****************************************************************************
@@ -1465,3 +1492,86 @@ CBrowserShell::DoDragReceive( DragReference inDragRef )
     sDragHelper->Drop ( inDragRef, mEventSink, &dragAccepted );
   }
 }
+
+#pragma mark -
+
+OSStatus CBrowserShell::HandleUpdateActiveInputArea(
+  const nsAString& text, 
+  PRInt16 script,  
+  PRInt16 language, 
+  PRInt32 fixLen, 
+  const TextRangeArray * hiliteRng
+)
+{
+  nsCOMPtr<nsIMacTextInputEventSink> tieSink;
+  tieSink = do_QueryInterface(mEventSink);
+  if (!tieSink)
+    return eventNotHandledErr;
+    
+  OSStatus err = noErr;
+  nsresult res = tieSink->HandleUpdateActiveInputArea( text,
+                                                       script, language, 
+                                                       fixLen, (void*)hiliteRng, 
+                                                       &err);
+  if (noErr != err)
+    return err;
+  return NS_FAILED(res) ? eventNotHandledErr : noErr;
+}
+
+OSStatus CBrowserShell::HandleUnicodeForKeyEvent(
+  const nsAString& text, 
+  PRInt16 script,
+  PRInt16 language, 
+  const EventRecord * keyboardEvent)
+{
+  nsCOMPtr<nsIMacTextInputEventSink> tieSink;
+  tieSink = do_QueryInterface(mEventSink);
+  if (!tieSink)
+    return eventNotHandledErr;
+    
+  OSStatus err = noErr;
+  nsresult res = tieSink->HandleUnicodeForKeyEvent( text,
+                                                    script, language, 
+                                                    (void*)keyboardEvent, 
+                                                    &err);
+  if (noErr != err)
+    return err;
+  return NS_FAILED(res) ? eventNotHandledErr : noErr;
+}
+
+OSStatus CBrowserShell::HandleOffsetToPos(
+  PRInt32 offset, 
+  PRInt16 *pointX, 
+  PRInt16 *pointY)
+{
+  nsCOMPtr<nsIMacTextInputEventSink> tieSink;
+  tieSink = do_QueryInterface(mEventSink);
+  if (!tieSink)
+    return eventNotHandledErr;
+    
+  OSStatus err = noErr;
+  nsresult res = tieSink->HandleOffsetToPos( offset, pointX, pointY, &err);
+  if (noErr != err)
+    return err;
+  return NS_FAILED(res) ? eventNotHandledErr : noErr;
+}
+
+OSStatus CBrowserShell::HandlePosToOffset(
+  PRInt16 currentPointX, 
+  PRInt16 currentPointY, 
+  PRInt32 *offset,
+  PRInt16 *regionClass
+)
+{
+  nsCOMPtr<nsIMacTextInputEventSink> tieSink;
+  tieSink = do_QueryInterface(mEventSink);
+  if (!tieSink)
+    return eventNotHandledErr;
+    
+  OSStatus err = noErr;
+  nsresult res = tieSink->HandlePosToOffset( currentPointX, currentPointY, 
+                                             offset, regionClass, &err);
+  if (noErr != err)
+    return err;
+  return NS_FAILED(res) ? eventNotHandledErr : noErr;
+}
