@@ -235,7 +235,7 @@ NS_IMETHODIMP nsWebBrowser::BindListener(nsISupports *aListener, const nsIID& aI
     if (aIID.Equals(NS_GET_IID(nsIWebProgressListener))) {
         nsCOMPtr<nsIWebProgressListener> listener = do_QueryInterface(aListener, &rv);
         if (NS_FAILED(rv)) return rv;
-        rv = mWebProgress->AddProgressListener(listener);
+        rv = mWebProgress->AddProgressListener(listener, nsIWebProgress::NOTIFY_ALL);
     }
     else if (aIID.Equals(NS_GET_IID(nsISHistoryListener))) {      
       nsCOMPtr<nsISHistory> shistory(do_GetInterface(mDocShell, &rv));
@@ -443,6 +443,8 @@ NS_IMETHODIMP nsWebBrowser::SetItemType(PRInt32 aItemType)
 {
     NS_ENSURE_TRUE((aItemType == typeContentWrapper || aItemType == typeChromeWrapper), NS_ERROR_FAILURE);
     mContentType = aItemType;
+    if (mDocShellAsItem)
+        mDocShellAsItem->SetItemType(mContentType == typeChromeWrapper ? typeChrome : typeContent);
     return NS_OK;
 }
 
@@ -738,6 +740,12 @@ NS_IMETHODIMP nsWebBrowser::SetProperty(PRUint32 aId, PRUint32 aValue)
             // obsolete
         }
         break;
+    case nsIWebBrowserSetup::SETUP_IS_CHROME_WRAPPER:
+        {
+           NS_ENSURE_TRUE((aValue == PR_TRUE || aValue == PR_FALSE), NS_ERROR_INVALID_ARG);
+           SetItemType(aValue ? typeChromeWrapper : typeContentWrapper);
+        }
+        break;
     default:
         rv = NS_ERROR_INVALID_ARG;
   
@@ -750,8 +758,8 @@ NS_IMETHODIMP nsWebBrowser::SetProperty(PRUint32 aId, PRUint32 aValue)
 // nsWebBrowser::nsIWebProgressListener
 //*****************************************************************************
 
-/* void onStateChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in long aStateFlags, in unsigned long aStatus); */
-NS_IMETHODIMP nsWebBrowser::OnStateChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRInt32 aStateFlags, PRUint32 aStatus)
+/* void onStateChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in unsigned long aStateFlags, in nsresult aStatus); */
+NS_IMETHODIMP nsWebBrowser::OnStateChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRUint32 aStateFlags, nsresult aStatus)
 {
     if (mPersist)
     {
@@ -802,8 +810,8 @@ NS_IMETHODIMP nsWebBrowser::OnStatusChange(nsIWebProgress *aWebProgress, nsIRequ
     return NS_OK;
 }
 
-/* void onSecurityChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in long state); */
-NS_IMETHODIMP nsWebBrowser::OnSecurityChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRInt32 state)
+/* void onSecurityChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in unsigned long state); */
+NS_IMETHODIMP nsWebBrowser::OnSecurityChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRUint32 state)
 {
     if (mProgressListener)
     {
@@ -1111,7 +1119,7 @@ NS_IMETHODIMP nsWebBrowser::Create()
    if (NS_SUCCEEDED(rv))
    {
        mSecurityUI = do_CreateInstance(NS_SECURE_BROWSER_UI_CONTRACTID, &rv);
-       if (NS_SUCCEEDED(rv))mSecurityUI->Init(domWindow, nsnull);
+       if (NS_SUCCEEDED(rv))mSecurityUI->Init(domWindow);
    }
 
    mDocShellTreeOwner->AddToWatcher(); // evil twin of Remove in SetDocShell(0)
@@ -1597,6 +1605,7 @@ NS_IMETHODIMP nsWebBrowser::GetPrimaryContentWindow(nsIDOMWindowInternal **aDOMW
   *aDOMWindow = 0;
 
   nsCOMPtr<nsIDocShellTreeItem> item;
+  NS_ENSURE_TRUE(mDocShellTreeOwner, NS_ERROR_FAILURE);
   mDocShellTreeOwner->GetPrimaryContentShell(getter_AddRefs(item));
   NS_ENSURE_TRUE(item, NS_ERROR_FAILURE);
 
@@ -1634,21 +1643,23 @@ NS_IMETHODIMP nsWebBrowser::Activate(void)
   nsCOMPtr<nsIDOMWindowInternal> domWindow;
   domWindow = do_QueryInterface(domWindowExternal);
   nsCOMPtr<nsPIDOMWindow> piWin(do_QueryInterface(domWindow));
-  nsCOMPtr<nsIFocusController> focusController;
-  piWin->GetRootFocusController(getter_AddRefs(focusController));
   PRBool needToFocus = PR_TRUE;
-  if (focusController) {
-    // Go ahead and mark the focus controller as being active.  We have
-    // to do this even before the activate message comes in.
-    focusController->SetActive(PR_TRUE);
+  if (piWin) {
+    nsCOMPtr<nsIFocusController> focusController;
+    piWin->GetRootFocusController(getter_AddRefs(focusController));
+    if (focusController) {
+      // Go ahead and mark the focus controller as being active.  We have
+      // to do this even before the activate message comes in.
+      focusController->SetActive(PR_TRUE);
 
-    nsCOMPtr<nsIDOMWindowInternal> focusedWindow;
-    focusController->GetFocusedWindow(getter_AddRefs(focusedWindow));
-    if (focusedWindow) {
-      needToFocus = PR_FALSE;
-      focusController->SetSuppressFocus(PR_TRUE, "Activation Suppression");
-      domWindow->Focus(); // This sets focus, but we'll ignore it.  
-                         // A subsequent activate will cause us to stop suppressing.
+      nsCOMPtr<nsIDOMWindowInternal> focusedWindow;
+      focusController->GetFocusedWindow(getter_AddRefs(focusedWindow));
+      if (focusedWindow) {
+        needToFocus = PR_FALSE;
+        focusController->SetSuppressFocus(PR_TRUE, "Activation Suppression");
+        domWindow->Focus(); // This sets focus, but we'll ignore it.  
+                           // A subsequent activate will cause us to stop suppressing.
+      }
     }
   }
 
