@@ -44,7 +44,6 @@
 #include "nsIFileSpec.h"
 #include "nsVoidArray.h"
 #include "prprf.h"
-#include "xp_core.h"
 #include "prmem.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
@@ -53,12 +52,18 @@
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsIObserverService.h"
 #include "nsICookieConsent.h"
+#include "nsIURL.h"
+#include "nsIHttpChannel.h"
+#include "prnetdb.h"
+
+#include <time.h>
 
 #define MAX_NUMBER_OF_COOKIES 300
 #define MAX_COOKIES_PER_SERVER 20
 #define MAX_BYTES_PER_COOKIE 4096  /* must be at least 1 */
 
 #define cookie_behaviorPref "network.cookie.cookieBehavior"
+#define cookie_disableCookieForMailNewsPref "network.cookie.disableCookieForMailNews"
 #define cookie_warningPref "network.cookie.warnAboutCookies"
 #define cookie_strictDomainsPref "network.cookie.strictDomains"
 #define cookie_lifetimePref "network.cookie.lifetimeOption"
@@ -90,8 +95,8 @@ typedef struct _cookie_CookieStruct {
   time_t lastAccessed;
   PRBool isSecure;
   PRBool isDomain;   /* is it a domain instead of an absolute host? */
-  nsCookieStatus_t status;
-  nsCookiePolicy_t policy;
+  nsCookieStatus status;
+  nsCookiePolicy policy;
 } cookie_CookieStruct;
 
 typedef enum {
@@ -103,6 +108,7 @@ typedef enum {
 
 PRIVATE PRBool cookie_changed = PR_FALSE;
 PRIVATE PERMISSION_BehaviorEnum cookie_behavior = PERMISSION_Accept;
+PRIVATE PRBool cookie_disableCookieForMailNews = PR_TRUE; //default -- disable is true
 PRIVATE PRBool cookie_warning = PR_FALSE;
 PRIVATE COOKIE_LifetimeEnum cookie_lifetimeOpt = COOKIE_Normal;
 PRIVATE time_t cookie_lifetimeLimit = 90*24*60*60;
@@ -206,7 +212,7 @@ COOKIE_DeletePersistentUserData(void)
   nsCOMPtr<nsIFile> cookiesFile;
   res = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(cookiesFile));
   if (NS_SUCCEEDED(res)) {
-    res = cookiesFile->Append(kCookiesFileName);
+    res = cookiesFile->AppendNative(nsDependentCString(kCookiesFileName));
     if (NS_SUCCEEDED(res))
         (void) cookiesFile->Remove(PR_FALSE);
   }
@@ -267,6 +273,25 @@ cookie_RemoveExpiredCookies() {
   }
 }
 
+/* Remove any session cookies from memory */
+PUBLIC void
+COOKIE_RemoveSessionCookies() {
+  cookie_CookieStruct * cookie_s;
+  if (cookie_list == nsnull) {
+    return;
+  }
+  
+  for (PRInt32 i = cookie_list->Count(); i > 0;) {
+    i--;
+    cookie_s = NS_STATIC_CAST(cookie_CookieStruct*, cookie_list->ElementAt(i));
+    NS_ASSERTION(cookie_s, "corrupt cookie list");
+      if(!cookie_s->expires) {
+        cookie_list->RemoveElementAt(i);
+        deleteCookie((void*)cookie_s, nsnull);
+      }
+  }
+}
+
 /* checks to see if the maximum number of cookies per host
  * is being exceeded and deletes the oldest one in that
  * case
@@ -326,10 +351,15 @@ cookie_CheckForPrevCookie(char * path, char * hostname, char * name) {
 
 /* cookie utility functions */
 PRIVATE void
-cookie_SetBehaviorPref(PERMISSION_BehaviorEnum x) {
+cookie_SetBehaviorPref(PERMISSION_BehaviorEnum x, nsIPref* prefs) {
   cookie_behavior = x;
 }
 
+PRIVATE void
+cookie_SetDisableCookieForMailNewsPref(PRBool x) {
+  cookie_disableCookieForMailNews = x;
+}
+
 PRIVATE void
 cookie_SetWarningPref(PRBool x) {
   cookie_warning = x;
@@ -351,6 +381,11 @@ cookie_GetBehaviorPref() {
   return cookie_behavior;
 }
 
+PRIVATE PRBool
+cookie_GetDisableCookieForMailNewsPref() {
+  return cookie_disableCookieForMailNews;
+}
+
 PRIVATE PRBool
 cookie_GetWarningPref() {
   return cookie_warning;
@@ -405,7 +440,19 @@ cookie_BehaviorPrefChanged(const char * newpref, void * data) {
     n = PERMISSION_Accept;
   }
     
-  cookie_SetBehaviorPref((PERMISSION_BehaviorEnum)n);
+  cookie_SetBehaviorPref((PERMISSION_BehaviorEnum)n, prefs);
+  return 0;
+}
+
+MODULE_PRIVATE int PR_CALLBACK
+cookie_DisableCookieForMailNewsPrefChanged(const char * newpref, void * data) {
+  PRBool x;
+  nsresult rv;
+  nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
+  if (!prefs || NS_FAILED(prefs->GetBoolPref(cookie_disableCookieForMailNewsPref, &x))) {
+    x = PR_TRUE;
+  }
+  cookie_SetDisableCookieForMailNewsPref(x);
   return 0;
 }
 
@@ -510,10 +557,16 @@ COOKIE_RegisterPrefCallbacks(void) {
   if (NS_FAILED(prefs->GetIntPref(cookie_behaviorPref, &n))) {
     n = PERMISSION_Accept;
   }
-
-  cookie_SetBehaviorPref((PERMISSION_BehaviorEnum)n);
+  cookie_SetBehaviorPref((PERMISSION_BehaviorEnum)n, prefs);
   prefs->RegisterCallback(cookie_behaviorPref, cookie_BehaviorPrefChanged, nsnull);
 
+  // Initialize  cookie_disableCookieForMailNewsPref
+  if (NS_FAILED(prefs->GetBoolPref(cookie_disableCookieForMailNewsPref, &x))) {
+    x = PR_TRUE; //default --> disable is true
+  }
+  cookie_SetDisableCookieForMailNewsPref(x);
+  prefs->RegisterCallback(cookie_disableCookieForMailNewsPref, cookie_DisableCookieForMailNewsPrefChanged, nsnull);
+
   // Initialize for cookie_warningPref
   if (NS_FAILED(prefs->GetBoolPref(cookie_warningPref, &x))) {
     x = PR_FALSE;
@@ -558,6 +611,13 @@ COOKIE_RegisterPrefCallbacks(void) {
   prefs->RegisterCallback(cookie_p3pPref, cookie_P3PPrefChanged, nsnull);
 }
 
+PRBool
+cookie_IsIPAddress(const char* name) {
+  // determine if name is an IP address
+  PRNetAddr addr;
+  return (PR_StringToNetAddr(name, &addr) == PR_SUCCESS);
+}
+
 PRBool
 cookie_IsInDomain(char* domain, char* host, int hostLength) {
   int domainLength = PL_strlen(domain);
@@ -577,16 +637,9 @@ cookie_IsInDomain(char* domain, char* host, int hostLength) {
   }
 
   /*
-   * test for domain name being an IP address (e.g., 105.217) and reject if so
+   * test for domain name being an IP address (e.g., 105.217.180.21) and reject if so
    */
-  PRBool hasNonDigitChar = PR_FALSE;
-  for (int i = 0; i<domainLength; i++) {
-    if (!nsCRT::IsAsciiDigit(domain[i]) && domain[i] != '.') {
-      hasNonDigitChar = PR_TRUE;
-      break;
-    }
-  }
-  if (!hasNonDigitChar) {
+  if (cookie_IsIPAddress(domain)) {
     return PR_FALSE;
   }
 
@@ -654,6 +707,9 @@ COOKIE_GetCookie(char * address, nsIIOService* ioService) {
   if (NS_FAILED(result)) {
     return nsnull;
   }
+  if ((host.RFindChar(' ') != -1) || (host.RFindChar('\t') != -1)) {
+    return nsnull;
+  }
   result = ioService->ExtractUrlPart(nsDependentCString(address),
                                      nsIIOService::url_Path, path);
   if (NS_FAILED(result)) {
@@ -775,6 +831,27 @@ cookie_SameDomain(char * currentHost, char * firstHost) {
   return 0;
 }
 
+PRBool
+cookie_isFromMailNews(char *firstURL, nsIIOService* ioService) {
+  
+  if (!firstURL) 
+    return PR_FALSE;
+
+  NS_ASSERTION(ioService, "IOService not available");
+  if (!ioService)
+    return PR_FALSE;  // we cannot check the scheme of orignal uri
+
+  nsCAutoString schemeString;
+  nsresult rv = ioService->ExtractScheme(nsDependentCString(firstURL), schemeString);
+  if (NS_FAILED(rv))  //malformed uri
+    return PR_FALSE; 
+  
+  return (schemeString.Equals(NS_LITERAL_CSTRING("imap")) || 
+          schemeString.Equals(NS_LITERAL_CSTRING("news")) ||
+          schemeString.Equals(NS_LITERAL_CSTRING("mailbox")));
+}
+
+
 PRBool
 cookie_isForeign (char * curURL, char * firstURL, nsIIOService* ioService) {
   if (!firstURL) {
@@ -824,7 +901,7 @@ cookie_isForeign (char * curURL, char * firstURL, nsIIOService* ioService) {
   return retval;
 }
 
-nsCookieStatus_t
+nsCookieStatus
 cookie_GetStatus(char decision) {
   switch (decision) {
     case ' ':
@@ -835,11 +912,13 @@ cookie_GetStatus(char decision) {
       return nsICookie::STATUS_DOWNGRADED;
     case 'f':
       return nsICookie::STATUS_FLAGGED;
+    case 'r':
+      return nsICookie::STATUS_REJECTED;
   }
   return nsICookie::STATUS_UNKNOWN;
 }
 
-nsCookiePolicy_t
+nsCookiePolicy
 cookie_GetPolicy(int policy) {
   switch (policy) {
     case P3P_NoPolicy:
@@ -877,7 +956,8 @@ P3P_SitePolicy(char * curURL, nsIHttpChannel* aHttpChannel) {
  */
 int
 cookie_P3PUserPref(PRInt32 policy, PRBool foreign) {
-  NS_ASSERTION(policy == P3P_NoPolicy ||
+  NS_ASSERTION(policy == P3P_UnknownPolicy ||
+               policy == P3P_NoPolicy ||
                policy == P3P_NoConsent ||
                policy == P3P_ImplicitConsent ||
                policy == P3P_ExplicitConsent ||
@@ -888,7 +968,8 @@ cookie_P3PUserPref(PRInt32 policy, PRBool foreign) {
      * asked for explicit consent */
     policy = P3P_ExplicitConsent;
   }
-  if (cookie_P3P && PL_strlen(cookie_P3P) == 8) {
+  // note: P3P_UnknownPolicy means that the p3p module was not installed
+  if (cookie_P3P && PL_strlen(cookie_P3P) == 8 && policy != P3P_UnknownPolicy) {
     return (foreign ? cookie_P3P[policy+1] : cookie_P3P[policy]);
   } else {
     return P3P_Accept;
@@ -898,7 +979,7 @@ cookie_P3PUserPref(PRInt32 policy, PRBool foreign) {
 /*
  * returns STATUS_ACCEPT, STATUS_DOWNGRADE, STATUS_FLAG, or STATUS_REJECT based on user's preferences
  */
-nsCookieStatus_t
+nsCookieStatus
 cookie_P3PDecision (char * curURL, char * firstURL, nsIIOService* ioService, nsIHttpChannel* aHttpChannel) {
   return cookie_GetStatus(
            cookie_P3PUserPref(
@@ -916,6 +997,11 @@ PUBLIC char *
 COOKIE_GetCookieFromHttp(char * address, char * firstAddress, 
                          nsIIOService* ioService) {
 
+#ifdef DEBUG_morse
+   printf("--- curURL=%s\n",address);
+   printf("--- 1stURL=%s\n",firstAddress);
+#endif
+
   if ((cookie_GetBehaviorPref() == PERMISSION_DontAcceptForeign) &&
       (!firstAddress || cookie_isForeign(address, firstAddress, ioService))) {
 
@@ -980,7 +1066,7 @@ cookie_Count(char * host) {
 PRIVATE void
 cookie_SetCookieString(char * curURL, nsIPrompt *aPrompter, const char * setCookieHeader,
                        time_t timeToExpire, nsIIOService* ioService,
-                       nsIHttpChannel* aHttpChannel, nsCookieStatus_t status) {
+                       nsIHttpChannel* aHttpChannel, nsCookieStatus status) {
   cookie_CookieStruct * prev_cookie;
   char *path_from_header=nsnull, *host_from_header=nsnull;
   char *name_from_header=nsnull, *cookie_from_header=nsnull;
@@ -1087,7 +1173,13 @@ cookie_SetCookieString(char * curURL, nsIPrompt *aPrompter, const char * setCook
       int domain_length, cur_host_length;
 
       /* allocate more than we need */
-      nsCAutoString domain(ptr+7);
+      nsCAutoString domain;
+      if (*(ptr+7) != '.' && !cookie_IsIPAddress(cur_host.get())) {
+        // if host is not an IP address, force domain name to start with a dot
+        domain = '.';
+      }
+      domain.Append(ptr+7);
+
       domain.CompressWhitespace();
       CKutil_StrAllocCopy(domain_from_header, domain.get());
 
@@ -1126,7 +1218,7 @@ cookie_SetCookieString(char * curURL, nsIPrompt *aPrompter, const char * setCook
         *colon = '\0';
       }
       domain_length   = PL_strlen(domain_from_header);
-      cur_host_length = cur_host.Length();
+      cur_host_length = PL_strlen(cur_host.get());
 
       /* check to see if the host is in the domain */
       if (!cookie_IsInDomain(domain_from_header, (char*)cur_host.get(), cur_host_length)) {
@@ -1357,8 +1449,20 @@ cookie_SetCookieString(char * curURL, nsIPrompt *aPrompter, const char * setCook
 }
 
 PUBLIC void
-COOKIE_SetCookieString(char * curURL, nsIPrompt *aPrompter, const char * setCookieHeader, nsIIOService* ioService, nsIHttpChannel* aHttpChannel) {
-  COOKIE_SetCookieStringFromHttp(curURL, nsnull, aPrompter, setCookieHeader, 0, ioService, aHttpChannel);
+COOKIE_SetCookieString(char * aURL, nsIPrompt *aPrompter, const char * setCookieHeader, nsIIOService* ioService, nsIHttpChannel* aHttpChannel) {
+  nsCOMPtr<nsIURI> pFirstURL;
+  nsresult rv;
+  nsCAutoString firstSpec;
+
+  if (aHttpChannel) {
+    rv = aHttpChannel->GetDocumentURI(getter_AddRefs(pFirstURL));
+    if (NS_FAILED(rv)) return;
+    if (pFirstURL) {
+      rv = pFirstURL->GetSpec(firstSpec);
+      if (NS_FAILED(rv)) return;
+    }
+  }
+  COOKIE_SetCookieStringFromHttp(aURL, NS_CONST_CAST(char *, firstSpec.get()), aPrompter, setCookieHeader, 0, ioService, aHttpChannel);
 }
 
 /* This function wrapper wraps COOKIE_SetCookieString for the purposes of 
@@ -1372,6 +1476,11 @@ COOKIE_SetCookieString(char * curURL, nsIPrompt *aPrompter, const char * setCook
 PUBLIC void
 COOKIE_SetCookieStringFromHttp(char * curURL, char * firstURL, nsIPrompt *aPrompter, const char * setCookieHeader, char * server_date, nsIIOService* ioService, nsIHttpChannel* aHttpChannel) {
 
+#ifdef DEBUG_morse
+   printf("... curURL=%s\n",curURL);
+   printf("... 1stURL=%s\n",firstURL);
+#endif
+
   /* allow for multiple cookies separated by newlines */
    char *newline = PL_strchr(setCookieHeader, '\n');
    if(newline) {
@@ -1392,10 +1501,14 @@ COOKIE_SetCookieStringFromHttp(char * curURL, char * firstURL, nsIPrompt *aPromp
   time_t gmtCookieExpires=0, expires=0, sDate;
 
   /* check to see if P3P pref is satisfied */
-  nsCookieStatus_t status = nsICookie::STATUS_UNKNOWN;
+  nsCookieStatus status = nsICookie::STATUS_UNKNOWN;
   if (cookie_GetBehaviorPref() == PERMISSION_P3P) {
     status = cookie_P3PDecision(curURL, firstURL, ioService, aHttpChannel);
     if (status == nsICookie::STATUS_REJECTED) {
+      nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1"));
+      if (os) {
+        nsresult rv = os->NotifyObservers(nsnull, "cookieIcon", NS_ConvertASCIItoUCS2("on").get());
+      }
       return;
     }
   }
@@ -1407,6 +1520,10 @@ COOKIE_SetCookieStringFromHttp(char * curURL, char * firstURL, nsIPrompt *aPromp
     return;
   }
 
+  /* check if a Mail/News message is setting the cookie */
+  if (cookie_GetDisableCookieForMailNewsPref() && cookie_isFromMailNews(firstURL, ioService))
+    return;
+
   /* Determine when the cookie should expire. This is done by taking the difference between 
    * the server time and the time the server wants the cookie to expire, and adding that 
    * difference to the client time. This localizes the client time regardless of whether or
@@ -1428,7 +1545,7 @@ COOKIE_SetCookieStringFromHttp(char * curURL, char * firstURL, nsIPrompt *aPromp
     expires = cookie_ParseDate(date);
     *ptr=origLast;
   }
-  if (server_date) {
+  if (server_date && *server_date) {
     sDate = cookie_ParseDate(server_date);
   } else {
     sDate = get_current_time();
@@ -1748,8 +1865,8 @@ COOKIE_Enumerate
      char ** path,
      PRBool * isSecure,
      PRUint64 * expires,
-     nsCookieStatus_t * status,
-     nsCookiePolicy_t * policy) {
+     nsCookieStatus * status,
+     nsCookiePolicy * policy) {
   if (count > COOKIE_Count()) {
     return NS_ERROR_FAILURE;
   }
@@ -1776,7 +1893,7 @@ COOKIE_Enumerate
 
 PUBLIC void
 COOKIE_Remove
-    (const char* host, const char* name, const char* path, const PRBool permanent) {
+    (const char* host, const char* name, const char* path, const PRBool blocked) {
   cookie_CookieStruct * cookie;
   PRInt32 count = 0;
 
@@ -1790,7 +1907,7 @@ COOKIE_Remove
       if ((PL_strcmp(cookie->host, host) == 0) &&
           (PL_strcmp(cookie->name, name) == 0) &&
           (PL_strcmp(cookie->path, path) == 0)) {
-        if (permanent && cookie->host) {
+        if (blocked && cookie->host) {
           char * hostname = nsnull;
           char * hostnameAfterDot = cookie->host;
           while (*hostnameAfterDot == '.') {
