@@ -13,7 +13,7 @@
  * Portions created by ActiveState Tool Corp. are Copyright (C) 2000, 2001
  * ActiveState Tool Corp.  All Rights Reserved.
  *
- * Contributor(s): Mark Hammond <MarkH@ActiveState.com> (original author)
+ * Contributor(s): Mark Hammond <mhammond@skippinet.com.au> (original author)
  *
  */
 
@@ -63,6 +63,7 @@ PyXPCOM_INTERFACE_DEFINE(Py_nsISimpleEnumerator, nsISimpleEnumerator, PyMethods_
 PyXPCOM_INTERFACE_DEFINE(Py_nsIInterfaceInfo, nsIInterfaceInfo, PyMethods_IInterfaceInfo)
 PyXPCOM_INTERFACE_DEFINE(Py_nsIInputStream, nsIInputStream, PyMethods_IInputStream)
 PyXPCOM_INTERFACE_DEFINE(Py_nsIClassInfo, nsIClassInfo, PyMethods_IClassInfo)
+PyXPCOM_INTERFACE_DEFINE(Py_nsIVariant, nsIVariant, PyMethods_IVariant)
 
 ////////////////////////////////////////////////////////////
 // This is the main entry point called by the Python component
@@ -502,7 +503,8 @@ PRBool PyXPCOM_Globals_Ensure()
 			if (end > landmark) *end = '\0';
 
 			nsCOMPtr<nsILocalFile> ns_bin_dir;
-			NS_NewLocalFile(landmark, PR_FALSE, getter_AddRefs(ns_bin_dir));
+            NS_ConvertASCIItoUCS2 strLandmark(landmark);
+			NS_NewLocalFile(strLandmark, PR_FALSE, getter_AddRefs(ns_bin_dir));
 			nsresult rv = NS_InitXPCOM2(nsnull, ns_bin_dir, nsnull);
 #else
 			// Elsewhere, Mozilla can find it itself (we hope!)
@@ -594,6 +596,7 @@ init_xpcom() {
 	Py_nsIInterfaceInfo::InitType(dict);
 	Py_nsIInputStream::InitType(dict);
 	Py_nsIClassInfo::InitType(dict);
+	Py_nsIVariant::InitType(dict);
 
 	// yet another 
 	{ // temp scope nsIComponentManagerObsolete hack :(
