@@ -394,6 +394,9 @@ si_SelectDialog(const PRUnichar* szMessage, nsIPrompt* dialog, PRUnichar** pList
   PRUnichar * title_string = Wallet_Localize("SelectUserTitleLine");
   rv = dialog->Select( title_string, szMessage, *pCount, NS_CONST_CAST(const PRUnichar**, pList), &selectedIndex, &rtnValue );
   Recycle(title_string);
+  if (selectedIndex >= *pCount) {
+    return PR_FALSE; // out-of-range selection
+  }
   *pCount = selectedIndex;
   si_UserHasBeenSelected = PR_TRUE;
   return rtnValue;  
@@ -855,6 +858,16 @@ SINGSIGN_RemoveReject(const char *host) {
   return rv;
 }
 
+PRIVATE void
+si_PutReject(const char * passwordRealm, const nsString& userName, PRBool save);
+
+PUBLIC nsresult
+SINGSIGN_AddReject(const char *host /*, const char *userName*/) {
+  si_PutReject(host, nsString(/*thisParameter_isObsolete*/), PR_TRUE);
+// @see http://bonsai.mozilla.org/cvsblame.cgi?file=mozilla/extensions/wallet/src/singsign.cpp&rev=1.212&mark=1693#1650
+  return NS_OK;
+}
+
 /* Determine if a specified url/user exists */
 PRIVATE PRBool
 si_CheckForUser(const char *passwordRealm, const nsString& userName) {
@@ -2529,7 +2542,7 @@ SI_InSequence(const nsString& sequence, PRInt32 number)
 }
 
 PUBLIC void
-SI_FindValueInArgs(const nsAReadableString& results, const nsAReadableString& name, nsAWritableString& value)
+SI_FindValueInArgs(const nsAString& results, const nsAString& name, nsAString& value)
 {
   /* note: name must start and end with a vertical bar */
   nsReadingIterator<PRUnichar> start, end, barPos;
