@@ -224,7 +224,8 @@ NS_IMETHODIMP mozXMLTerminal::Init(nsIDocShell* aDocShell,
     nsCOMPtr<nsIWebProgress> progress(do_GetInterface(aDocShell, &result));
     if (NS_FAILED(result)) return result;
 
-    result = progress->AddProgressListener((nsIWebProgressListener*)this);
+    result = progress->AddProgressListener((nsIWebProgressListener*)this,
+                                           nsIWebProgress::NOTIFY_STATE_REQUEST);
     if (NS_FAILED(result))
       return NS_ERROR_FAILURE;
 
@@ -239,7 +240,7 @@ NS_IMETHODIMP mozXMLTerminal::Init(nsIDocShell* aDocShell,
     if (NS_FAILED(result))
       return NS_ERROR_FAILURE;
 
-    result = aDocShell->LoadURI(uri, nsnull, nsIWebNavigation::LOAD_FLAGS_NONE);
+    result = aDocShell->LoadURI(uri, nsnull, nsIWebNavigation::LOAD_FLAGS_NONE, PR_TRUE);
     if (NS_FAILED(result))
       return NS_ERROR_FAILURE;
 
@@ -973,7 +974,7 @@ NS_IMETHODIMP mozXMLTerminal::MatchesCookie(const PRUnichar* aCookie,
     return NS_ERROR_NULL_POINTER;
 
   // Check if supplied cookie matches XMLTerm cookie
-  *_retval = mCookie.EqualsWithConversion(aCookie);
+  *_retval = mCookie.Equals(aCookie);
 
   if (!(*_retval)) {
     XMLT_ERROR("mozXMLTerminal::MatchesCookie: Error - Cookie mismatch\n");
@@ -1035,7 +1036,7 @@ NS_IMETHODIMP mozXMLTerminal::ExportHTML(const PRUnichar* aFilename,
 NS_IMETHODIMP
 mozXMLTerminal::OnStateChange(nsIWebProgress* aWebProgress, 
                    nsIRequest *aRequest, 
-                   PRInt32 progressStateFlags, 
+                   PRUint32 progressStateFlags,
                    nsresult aStatus) {
     if (progressStateFlags & nsIWebProgressListener::STATE_IS_REQUEST)
         if (progressStateFlags & nsIWebProgressListener::STATE_START) {
@@ -1055,14 +1056,16 @@ mozXMLTerminal::OnProgressChange(nsIWebProgress *aWebProgress,
                                      PRInt32 aMaxSelfProgress,
                                      PRInt32 aCurTotalProgress,
                                      PRInt32 aMaxTotalProgress) {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    NS_NOTREACHED("notification excluded in AddProgressListener(...)");
+    return NS_OK;
 }
 
 NS_IMETHODIMP
 mozXMLTerminal::OnLocationChange(nsIWebProgress* aWebProgress,
                       nsIRequest* aRequest,
                       nsIURI *location) {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    NS_NOTREACHED("notification excluded in AddProgressListener(...)");
+    return NS_OK;
 }
 
 
@@ -1071,13 +1074,15 @@ mozXMLTerminal::OnStatusChange(nsIWebProgress* aWebProgress,
                     nsIRequest* aRequest,
                     nsresult aStatus,
                     const PRUnichar* aMessage) {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    NS_NOTREACHED("notification excluded in AddProgressListener(...)");
+    return NS_OK;
 }
 
 
 NS_IMETHODIMP
 mozXMLTerminal::OnSecurityChange(nsIWebProgress *aWebProgress, 
                       nsIRequest *aRequest, 
-                      PRInt32 state) {
-    return NS_ERROR_NOT_IMPLEMENTED;
+                      PRUint32 state) {
+    NS_NOTREACHED("notification excluded in AddProgressListener(...)");
+    return NS_OK;
 }
