@@ -364,9 +364,11 @@ NS_IMETHODIMP nsDeviceContextGTK::SupportsNativeWidgets(PRBool &aSupportsWidgets
 
 NS_IMETHODIMP nsDeviceContextGTK::GetScrollBarDimensions(float &aWidth, float &aHeight) const
 {
-  aWidth = mScrollbarWidth * mPixelsToTwips;
-  aHeight = mScrollbarHeight * mPixelsToTwips;
- 
+  float scale;
+  GetCanonicalPixelScale(scale);
+  aWidth = mScrollbarWidth * mPixelsToTwips * scale;
+  aHeight = mScrollbarHeight * mPixelsToTwips * scale;
+
   return NS_OK;
 }
 
