@@ -556,7 +556,7 @@ NS_IMETHODIMP nsRenderingContextMac::GetHints(PRUint32& aResult)
 {
 	PRUint32 result = 0;
 
-	// QuickDraw is prefered over to ATSUI for drawing 7-bit text
+	// QuickDraw is preferred over to ATSUI for drawing 7-bit text
 	// (it's not 8-bit: the name of the constant is misleading)
 	result |= NS_RENDERING_HINT_FAST_8BIT_TEXT;
 	aResult = result;
@@ -1533,3 +1533,30 @@ nsRenderingContextMac::ReleaseBackbuffer(void)
   SelectOffScreenDrawingSurface(nsnull);
   return NS_OK;
 }
+
+
+// override to not use a back buffer on MacOS X
+NS_IMETHODIMP 
+nsRenderingContextMac::UseBackbuffer(PRBool* aUseBackbuffer)
+{
+  *aUseBackbuffer = !OnMacOSX();
+  return NS_OK;
+}
+
+
+//
+// Return true if we are on Mac OS X, caching the result after the first call
+//
+PRBool
+nsRenderingContextMac::OnMacOSX()
+{
+  static PRBool gInitVer = PR_FALSE;
+  static PRBool gOnMacOSX = PR_FALSE;
+  if(! gInitVer) {
+    long version;
+    OSErr err = ::Gestalt(gestaltSystemVersion, &version);
+    gOnMacOSX = (err == noErr && version >= 0x00001000);
+    gInitVer = PR_TRUE;
+  }
+  return gOnMacOSX;
+}
