@@ -40,9 +40,9 @@
 #include "nsIServiceManager.h"
 #include "nsTextFormatter.h"
 #include "nsUnicodeMappingUtil.h"
-#include "nsUnicodeFontMappingCache.h"
 #include "nsDeviceContextMac.h"
 #include "nsReadableUtils.h"
+#include "nsCRT.h"
 static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
 
 #define BAD_FONT_NUM -1 
@@ -85,7 +85,6 @@ void nsUnicodeMappingUtil::Init()
 	InitFromPref();
 	InitScriptFontMapping();
 	InitBlockToScriptMapping(); // this must be called after InitScriptEnabled()
-	mCache = new nsUnicodeFontMappingCache();
 }
 void nsUnicodeMappingUtil::CleanUp()
 {
@@ -95,12 +94,6 @@ void nsUnicodeMappingUtil::CleanUp()
 	  		 delete mGenericFontMapping[i][j];
 	  	}
 	}
-	if (mCache)
-	{
-		delete mCache;
-		mCache = nsnull;
-	}
-
 }
 //--------------------------------------------------------------------------
 
