@@ -48,30 +48,32 @@ NS_IMPL_ISUPPORTS1(nsPrintSettings, nsIPrintSettings)
  *	@update 6/21/00 dwc
  */
 nsPrintSettings::nsPrintSettings() :
+  mPrintOptions(0L),
   mPrintRange(kRangeAllPages),
   mStartPageNum(1),
   mEndPageNum(1),
   mScaling(1.0),
-  mNumCopies(1),
   mPrintBGColors(PR_FALSE),
   mPrintBGImages(PR_FALSE),
-  mPrintOptions(0L),
-  mPrintReversed(PR_FALSE),
-  mPrintInColor(PR_TRUE),
-  mOrientation(kPortraitOrientation),
-  mPrintToFile(PR_FALSE),
   mPrintFrameTypeUsage(kUseInternalDefault),
   mPrintFrameType(kFramesAsIs),
+  mHowToEnableFrameUI(kFrameEnableNone),
+  mIsCancelled(PR_FALSE),
+  mPrintSilent(PR_FALSE),
+	mPrintPreview(PR_FALSE),
+  mShrinkToFit(PR_TRUE),
+  mShowPrintProgress(PR_TRUE),
   mPrintPageDelay(500),
-  mPaperSizeType(kPaperSizeDefined),
   mPaperData(0),
+  mPaperSizeType(kPaperSizeDefined),
   mPaperWidth(8.5),
   mPaperHeight(11.0),
   mPaperSizeUnit(kPaperSizeInches),
-  mPrintSilent(PR_FALSE),
-  mShrinkToFit(PR_FALSE),
-  mHowToEnableFrameUI(kFrameEnableNone),
-  mIsCancelled(PR_FALSE)
+  mPrintReversed(PR_FALSE),
+  mPrintInColor(PR_TRUE),
+  mOrientation(kPortraitOrientation),
+  mNumCopies(1),
+  mPrintToFile(PR_FALSE)
 {
   NS_INIT_ISUPPORTS();
 
@@ -578,6 +580,20 @@ NS_IMETHODIMP nsPrintSettings::SetPrintSilent(PRBool aPrintSilent)
   return NS_OK;
 }
 
+/* attribute boolean printPreview; */
+NS_IMETHODIMP nsPrintSettings::GetIsPrintPreview(PRBool *aPrintPreview)
+{
+  NS_ENSURE_ARG_POINTER(aPrintPreview);
+  *aPrintPreview = mPrintPreview;
+  return NS_OK;
+}
+NS_IMETHODIMP nsPrintSettings::SetIsPrintPreview(PRBool aPrintPreview)
+{
+  mPrintPreview = aPrintPreview;
+  return NS_OK;
+}
+
+
 /* attribute boolean shrinkToFit; */
 NS_IMETHODIMP nsPrintSettings::GetShrinkToFit(PRBool *aShrinkToFit)
 {
@@ -591,6 +607,37 @@ NS_IMETHODIMP nsPrintSettings::SetShrinkToFit(PRBool aShrinkToFit)
   return NS_OK;
 }
 
+/* attribute boolean showPrintProgress; */
+NS_IMETHODIMP nsPrintSettings::GetShowPrintProgress(PRBool *aShowPrintProgress)
+{
+  NS_ENSURE_ARG_POINTER(aShowPrintProgress);
+  *aShowPrintProgress = mShowPrintProgress;
+  return NS_OK;
+}
+NS_IMETHODIMP nsPrintSettings::SetShowPrintProgress(PRBool aShowPrintProgress)
+{
+  mShowPrintProgress = aShowPrintProgress;
+  return NS_OK;
+}
+
+/* attribute wstring paperName; */
+NS_IMETHODIMP nsPrintSettings::GetPaperName(PRUnichar * *aPaperName)
+{
+  NS_ENSURE_ARG_POINTER(aPaperName);
+  if (mPaperName.Length()) {
+    *aPaperName = ToNewUnicode(mPaperName);
+  } else {
+    *aPaperName = nsnull;
+  }
+  return NS_OK;
+}
+NS_IMETHODIMP nsPrintSettings::SetPaperName(const PRUnichar * aPaperName)
+{
+  NS_ENSURE_ARG_POINTER(aPaperName);
+  mPaperName = aPaperName;
+  return NS_OK;
+}
+
 /* attribute boolean howToEnableFrameUI; */
 NS_IMETHODIMP nsPrintSettings::GetHowToEnableFrameUI(PRInt16 *aHowToEnableFrameUI)
 {
