@@ -60,6 +60,7 @@
 #include "nsSupportsArray.h"
 
 #include "nsString.h"
+#include "nsCRT.h"
 #include "nsIServiceManager.h"
 #include "nsReadableUtils.h"
 #include "nsGfxCIID.h"
@@ -106,7 +107,7 @@ static UINT gFrameSelectedRadioBtn = 0;
 // Indicates whether the native print dialog was successfully extended
 static PRPackedBool gDialogWasExtended     = PR_FALSE;
 
-#define PRINTDLG_PROPERTIES "chrome://communicator-platform/locale/gfx/printdialog.properties"
+#define PRINTDLG_PROPERTIES "chrome://global/locale/printdialog.properties"
 
 static HWND gParentWnd = NULL;
 
@@ -352,6 +353,16 @@ nsDeviceContextSpecWin::SetPrintSettingsFromDevMode(nsIPrintSettings* aPrintSett
     aPrintSettings->SetNumCopies(PRInt32(aDevMode->dmCopies));
   }
 
+  if (aDevMode->dmFields & DM_SCALE) {
+    double scale = double(aDevMode->dmScale) / 100.0f;
+    if (scale != 1.0) {
+      aPrintSettings->SetScaling(scale);
+      aDevMode->dmScale = 100;
+      // To turn this on you must change where the mPrt->mShrinkToFit is being set in the DocumentViewer
+      //aPrintSettings->SetShrinkToFit(PR_FALSE);
+    }
+  }
+
   if (doingPaperSize) {
     aPrintSettings->SetPaperSizeType(nsIPrintSettings::kPaperSizeNativeData);
     aPrintSettings->SetPaperData(aDevMode->dmPaperSize);
@@ -1092,6 +1103,12 @@ nsDeviceContextSpecWin::ShowNativePrintDialog(nsIWidget *aWidget, PRBool aQuiet)
   BOOL result = ::PrintDlg(&prntdlg);
 
   if (TRUE == result) {
+    if (mPrintSettings && prntdlg.hDevMode != NULL) {
+      // Transfer the settings from the native data to the PrintSettings
+      LPDEVMODE devMode = (LPDEVMODE)::GlobalLock(prntdlg.hDevMode);
+      SetPrintSettingsFromDevMode(mPrintSettings, devMode);
+      ::GlobalUnlock(prntdlg.hDevMode);
+    }
     DEVNAMES *devnames = (DEVNAMES *)::GlobalLock(prntdlg.hDevNames);
     if ( NULL != devnames ) {
 
@@ -1528,7 +1545,8 @@ nsDeviceContextSpecWin::GetDataFromPrinter(const PRUnichar * aName, nsIPrintSett
   nsresult rv = NS_ERROR_FAILURE;
 
   if (!GlobalPrinters::GetInstance()->PrintersAreAllocated()) {
-    return rv;
+    rv = GlobalPrinters::GetInstance()->EnumeratePrinterList();
+    NS_ENSURE_SUCCESS(rv, rv);
   }
 
   HANDLE hPrinter = NULL;
@@ -1794,7 +1812,7 @@ nsPrinterEnumeratorWin::EnumeratePrinters(PRUint32* aCount, PRUnichar*** aResult
   PRInt32 count      = 0;
   PRInt32 printerInx = 0;
   while( count < numItems ) {
-    LPTSTR name = GlobalPrinters::GetInstance()->GetItemFromList(printerInx);
+    LPTSTR name = GlobalPrinters::GetInstance()->GetItemFromList(printerInx++);
     nsString newName; 
     newName.AssignWithConversion(name);
     PRUnichar *str = ToNewUnicode(newName);
