@@ -425,6 +425,11 @@ nsImageWin :: CreateDDB(nsDrawingSurface aSurface)
 
   ((nsDrawingSurfaceWin *)aSurface)->GetDC(&TheHDC);
 
+  int   rasterCaps = ::GetDeviceCaps(TheHDC, RASTERCAPS);
+  PRBool isPaletteDevice = RC_PALETTE == (rasterCaps & RC_PALETTE);
+  if (isPaletteDevice) return;  
+
+
   if (TheHDC != NULL){
     if (mSizeImage > 0){
        if (mAlphaDepth == 8) {
@@ -499,9 +504,8 @@ nsImageWin :: Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
   }
 
   if (nsnull != TheHDC){
+    PRBool  didComposite = PR_FALSE;
     if (!IsOptimized() || nsnull==mHBitmap){
-      PRBool  didComposite = PR_FALSE;
-
       rop = SRCCOPY;
 
       if (nsnull != mAlphaBits){
@@ -550,11 +554,30 @@ nsImageWin :: Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
         if (nsnull != mAlphaBits){
           if (1==mAlphaDepth){
             MONOBITMAPINFO  bmi(mAlphaWidth, mAlphaHeight);
-
-            ::StretchDIBits(TheHDC, aDX, aDY, aDWidth, aDHeight,aSX, srcy, aSWidth, aSHeight, mAlphaBits,
-                                          (LPBITMAPINFO)&bmi, DIB_RGB_COLORS, SRCAND);
-             rop = SRCPAINT;
-             //rop = SRCCOPY;
+            if (canRaster == DT_RASPRINTER) {
+              MONOBITMAPINFO  bmi(mAlphaWidth, mAlphaHeight);
+              if (mImageBits != nsnull) {
+                CompositeBitsInMemory(TheHDC,aDX,aDY,aDWidth,aDHeight,aSX,aSY,aSWidth,aSHeight,srcy,
+                                      mAlphaBits,&bmi,mImageBits,mBHead,mNumPaletteColors);
+                didComposite = PR_TRUE;  
+              } else {
+                ConvertDDBtoDIB(); // Create mImageBits
+                if (mImageBits != nsnull) {  
+                  CompositeBitsInMemory(TheHDC,aDX,aDY,aDWidth,aDHeight,aSX,aSY,aSWidth,aSHeight,srcy,
+                                        mAlphaBits,&bmi,mImageBits,mBHead,mNumPaletteColors);
+                  // Clean up the image bits    
+                  delete [] mImageBits;
+                  mImageBits = nsnull;
+                  didComposite = PR_TRUE;         
+                } else {
+                  NS_WARNING("Could not composite bits in memory because conversion to DIB failed\n");
+                }
+              }
+            } else {
+              ::StretchDIBits(TheHDC, aDX, aDY, aDWidth, aDHeight,aSX, srcy, aSWidth, aSHeight, mAlphaBits,
+                             (LPBITMAPINFO)&bmi, DIB_RGB_COLORS, SRCAND);
+              rop = SRCPAINT;
+            }
           }
         }
         // if this is for a printer.. we have to convert it back to a DIB
@@ -571,12 +594,13 @@ nsImageWin :: Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
                 blendFunction.BlendFlags = 0;
                 blendFunction.SourceConstantAlpha = 255;
                 blendFunction.AlphaFormat = 1 /*AC_SRC_ALPHA*/;
-                gAlphaBlend(TheHDC, aDX, aDY, aDWidth, aDHeight, srcDC, aSX, srcy, aSWidth, aSHeight, blendFunction);
+                gAlphaBlend(TheHDC, aDX, aDY, aDWidth, aDHeight, srcDC, aSX, aSY, aSWidth, aSHeight, blendFunction);
               } else {
                 ::StretchBlt(TheHDC, aDX, aDY, aDWidth, aDHeight, srcDC, aSX, aSY,aSWidth, aSHeight, rop);
               }
             }else{
-              PrintDDB(aSurface,aDX,aDY,aDWidth,aDHeight,rop);
+              if (! didComposite) 
+                PrintDDB(aSurface,aDX,aDY,aDWidth,aDHeight,rop);
             }
           }
         } else {
@@ -588,7 +612,7 @@ nsImageWin :: Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
             blendFunction.BlendFlags = 0;
             blendFunction.SourceConstantAlpha = 255;
             blendFunction.AlphaFormat = 1 /*AC_SRC_ALPHA*/;
-            gAlphaBlend(TheHDC, aDX, aDY, aDWidth, aDHeight, srcDC, aSX, srcy, aSWidth, aSHeight, blendFunction);
+            gAlphaBlend(TheHDC, aDX, aDY, aDWidth, aDHeight, srcDC, aSX, aSY, aSWidth, aSHeight, blendFunction);
           } else {
             ::StretchBlt(TheHDC,aDX,aDY,aDWidth,aDHeight,srcDC,aSX,aSY,aSWidth,aSHeight,rop);
           }
@@ -654,7 +678,7 @@ void nsImageWin::DrawComposited(HDC TheHDC, int aDX, int aDY, int aDWidth, int a
 
   ::SelectObject(memDC, oldBitmap);
   ::DeleteObject(tmpBitmap);
-  ::DeleteObject(memDC);
+  ::DeleteDC(memDC);
 }
 
 /** ---------------------------------------------------
@@ -746,6 +770,10 @@ NS_IMETHODIMP nsImageWin::DrawTile(nsIRenderingContext &aContext,
 
 
   if( mAlphaDepth == 8){
+    if(!mImageBits){
+      ConvertDDBtoDIB();
+    }
+
     /** 
      *  do alpha depth equal to 8 here.. this needs some special attention
      *  draw the alpha and the bitmap to an offscreen buffer.. for the blend.. first 
@@ -764,7 +792,7 @@ NS_IMETHODIMP nsImageWin::DrawTile(nsIRenderingContext &aContext,
     HBITMAP tmpBitmap = ::CreateDIBSection(memDC, (LPBITMAPINFO)&bmi, DIB_RGB_COLORS, (LPVOID *)&screenBits, NULL, 0);
     HBITMAP oldBitmap = (HBITMAP)::SelectObject(memDC, tmpBitmap);
     if(0 == tmpBitmap){
-      ::DeleteObject(memDC);
+      ::DeleteDC(memDC);
       tryAgain = PR_TRUE;
     } else {
       // Copy from the HDC to the memory DC
@@ -807,16 +835,44 @@ NS_IMETHODIMP nsImageWin::DrawTile(nsIRenderingContext &aContext,
 
       ::SelectObject(memDC, oldBitmap);
       ::DeleteObject(tmpBitmap);
-      ::DeleteObject(memDC);
+      ::DeleteDC(memDC);
   
     return(PR_TRUE);
+    } 
+  }
+
+
+  // figure out which case to use for tiling
+  PRBool  useSlow = PR_FALSE;
+
+  if ((mAlphaDepth>8) || ((mAlphaDepth==8)&&tryAgain) || (canRaster==DT_RASPRINTER) 
+      || (256==mNumPaletteColors)){
+    // CASE 1 -- ALPHA DEPTH IS TO HIGH OR WE ARE PRINTING OR ALPHA ALGORITHM FROM ABOVE FAILED
+    useSlow = PR_TRUE;
+  } else if ( (imageScaledWidth>MAX_BUFFER_WIDTH) || (imageScaledHeight>MAX_BUFFER_HEIGHT)) {
+    if(PR_TRUE != gIsWinNT){
+      // CASE 2 -- THE PLATFORM IS NOT ON NT AND CAN NOT USE A PATBLT
+      useSlow = PR_TRUE; 
+    } else {
+      if( (imageScaledWidth < MAX_BUFFER_WIDTH) || (imageScaledHeight < MAX_BUFFER_HEIGHT) ) {
+        // CASE 3 -- THE PLATFORM IS ON NT AND WE HAVE ONE LARGE AND ONE SMALL WIDTH AND HEIGHT
+        if (PatBltTile(aContext,aSurface,aX0,aY0,aX1,aY1)) {
+          return(PR_TRUE);
+        }
+        // If PatBltTile returns PR_FALSE then we must drop through to the slow tiling
+        // code because either the width or height of the tiling buffer has been exceeded
+        useSlow = PR_TRUE; 
+      } else {
+      // CASE 4 -- THE PLATFORM IS ON NT AND BOTH THE WIDTH AND HEIGHT ARE LARGE.  
+      //        -- THIS IS AN ODD CASE.. SEEMS PATBLT WITH LARGER BRUSHES HAS A DIFFICULT TIME
+      //        -- AND THE TIMES ARE SLOWER.
+      useSlow = PR_TRUE;
+      }
     }
   }
 
 
-  // we have to use the old way.. for 256 color mode and printing.. slow, but will always work.
-  if ((mAlphaDepth>8) || ((mAlphaDepth==8)&&tryAgain) || (canRaster==DT_RASPRINTER) || (256==mNumPaletteColors)
-      || (imageScaledWidth>MAX_BUFFER_WIDTH) || (imageScaledHeight>MAX_BUFFER_HEIGHT)){
+  if(useSlow){
     for(y=aY0;y<aY1;y+=imageScaledHeight){
       for(x=aX0;x<aX1;x+=imageScaledWidth){
       Draw(aContext, aSurface,
@@ -825,12 +881,6 @@ NS_IMETHODIMP nsImageWin::DrawTile(nsIRenderingContext &aContext,
       }
     } 
     return(PR_TRUE);
-  } 
-  
-  // now try to pattern blit.. if that fails.. we use our own algorithm to get
-  // things fast
-  if ( PR_TRUE == PatBltTile(aContext,aSurface,aX0,aY0,aX1,aY1) ){
-    return(PR_TRUE);
   }
 
 
@@ -840,7 +890,8 @@ NS_IMETHODIMP nsImageWin::DrawTile(nsIRenderingContext &aContext,
     return (PR_FALSE);
   }
 
-  // can not create a compatible bitmap with an offscreen DC (it will be black and white)
+  // IF WE MADE IT THIS FAR.. A PROGRESSIVE DOUBLING ALGORITHM WILL BE USED TO TILE
+
   // so we will create a screen compatible bitmap and then install this into the offscreen DC
   tvrect.SetRect(0,0,aX1-aX0,aY1-aY0);
   offDC = ::CreateCompatibleDC(TheHDC);
@@ -863,7 +914,7 @@ NS_IMETHODIMP nsImageWin::DrawTile(nsIRenderingContext &aContext,
   tileBits = ::CreateCompatibleBitmap(TheHDC, TileBufferWidth,TileBufferHeight);
 
   if (NULL == tileBits){
-    ::DeleteObject(offDC);
+    ::DeleteDC(offDC);
     return (PR_FALSE);
   }
   oldBits =(HBITMAP) ::SelectObject(offDC,tileBits);
@@ -875,15 +926,15 @@ NS_IMETHODIMP nsImageWin::DrawTile(nsIRenderingContext &aContext,
     if (NULL ==maskDC){
       ::SelectObject(offDC,oldBits);
       ::DeleteObject(tileBits);
-      ::DeleteObject(offDC);
+      ::DeleteDC(offDC);
       return (PR_FALSE);
     }
     maskBits = ::CreateCompatibleBitmap(TheHDC, TileBufferWidth, TileBufferHeight);
     if (NULL ==maskBits){
       ::SelectObject(offDC,oldBits);
       ::DeleteObject(tileBits);
-      ::DeleteObject(offDC);
-      ::DeleteObject(maskDC);
+      ::DeleteDC(offDC);
+      ::DeleteDC(maskDC);
       return (PR_FALSE);
     }
 
@@ -940,12 +991,12 @@ NS_IMETHODIMP nsImageWin::DrawTile(nsIRenderingContext &aContext,
     } 
     ::SelectObject(maskDC,oldMaskBits);
     ::DeleteObject(maskBits);
-    ::DeleteObject(maskDC);
+    ::DeleteDC(maskDC);
   }
 
   ::SelectObject(offDC,oldBits);
   ::DeleteObject(tileBits);
-  ::DeleteObject(offDC);
+  ::DeleteDC(offDC);
 
   return NS_OK;
 }
@@ -1163,7 +1214,7 @@ nsImageWin::PrintDDB(nsDrawingSurface aSurface,PRInt32 aX, PRInt32 aY, PRInt32 a
 
   if (mIsOptimized == PR_TRUE){
     if (mHBitmap != nsnull){
-      ConvertDDBtoDIB(aWidth,aHeight);
+      ConvertDDBtoDIB();
       ((nsDrawingSurfaceWin *)aSurface)->GetDC(&theHDC);
 
       if (mBHead->biBitCount == 8) {
@@ -1192,7 +1243,7 @@ nsImageWin::PrintDDB(nsDrawingSurface aSurface,PRInt32 aX, PRInt32 aY, PRInt32 a
  * @return the result of the operation, if NS_OK, then the pixelmap is unoptimized
  */
 nsresult 
-nsImageWin::ConvertDDBtoDIB(PRInt32 aWidth, PRInt32 aHeight)
+nsImageWin::ConvertDDBtoDIB()
 {
   PRInt32             numbytes,tWidth,tHeight;
   BITMAP              srcinfo;
@@ -1387,7 +1438,7 @@ nsImageWin::GetBits()
 {
   // if mImageBits did not exist.. then
   if (!mImageBits) {
-    ConvertDDBtoDIB(GetWidth(), GetHeight());
+    ConvertDDBtoDIB();
     mDIBTemp = PR_TRUE;   // only set to true if the DIB is being created here as temporary
   }
 
@@ -1547,6 +1598,6 @@ CompositeBitsInMemory(HDC aTheHDC, int aDX, int aDY, int aDWidth, int aDHeight,
       }
       ::DeleteObject(tmpBitmap);
     }
-    ::DeleteObject(memDC);
+    ::DeleteDC(memDC);
   }
 }
