@@ -57,9 +57,6 @@
 #include "xprintutil.h"
 #include "prenv.h" /* for PR_GetEnv */
 
-/* misc defines */
-#define XPRINT_MAKE_24BIT_VISUAL_AVAILABLE_FOR_TESTING 1
-
 /* NS_XPRINT_RGB_DITHER: Macro to check whether we should dither or not.
  * In theory we only have to look at the visual and depth ("TrueColor" with 
  * enougth bits for the colors or GrayScale/StaticGray with enougth bits for
@@ -186,19 +183,17 @@ nsXPrintContext::Init(nsDeviceContextXp *dc, nsIDeviceContextSpecXp *aSpec)
   PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("nsXPrintContext::Init()\n"));
   nsresult rv = NS_ERROR_FAILURE;
 
-  int   prefDepth = 8;  /* 24 or 8 for PS DDX, 24, 8, 1 for PCL DDX... 
-                         * I wish current Xprt would have a 1bit/8bit StaticGray 
-                         * visual for the PS DDX... ;-( 
-                         */
-/* I can't get any other visual than the 8bit peudocolor one working... BAD...
- * This env var allows others to test this without hacking their own binaries...
- */
-#ifdef XPRINT_MAKE_24BIT_VISUAL_AVAILABLE_FOR_TESTING  
-  if( PR_GetEnv("MOZILLA_XPRINT_EXPERIMENTAL_USE_24BIT_VISUAL") != nsnull )
+  int   prefDepth = 24;  /* 24 or 8 for PS DDX, 24, 8, 1 for PCL DDX... 
+                          * I wish current Xprt would have a 1bit/8bit StaticGray 
+                          * visual for the PS DDX... ;-( 
+                          */
+  /* Safeguard for production use (see bug 80562),
+   * Make sure we can switch back to the
+   * extensively tested 8bit Pseudocolor visual on demand... */
+  if( PR_GetEnv("MOZILLA_XPRINT_EXPERIMENTAL_DISABLE_24BIT_VISUAL") != nsnull )
   {
-    prefDepth = 24;
+    prefDepth = 8;
   }
-#endif /* XPRINT_MAKE_24BIT_VISUAL_AVAILABLE_FOR_TESTING */  
   
   unsigned short width, height;
   XRectangle rect;
@@ -256,10 +251,23 @@ nsXPrintContext::Init(nsDeviceContextXp *dc, nsIDeviceContextSpecXp *aSpec)
   else
   {
     PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("printing color\n"));
-    xargs.xtemplate.depth   = prefDepth;
-    xargs.xtemplate.c_class = (prefDepth>12)?(TrueColor):(PseudoColor);
-    xargs.xtemplate_mask    = VisualDepthMask|VisualClassMask;
-    mXlibRgbHandle = xxlib_rgb_create_handle(mPDisplay, mScreen, &xargs);
+    if (prefDepth > 12)
+    {
+      PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("trying TrueColor %d bit\n", prefDepth));
+      xargs.xtemplate.depth   = prefDepth;
+      xargs.xtemplate.c_class = TrueColor;
+      xargs.xtemplate_mask    = VisualDepthMask|VisualClassMask;
+      mXlibRgbHandle = xxlib_rgb_create_handle(mPDisplay, mScreen, &xargs);
+    }  
+
+    if (!mXlibRgbHandle)
+    {
+      PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("trying PseudoColor 8 bit\n"));
+      xargs.xtemplate.depth   = 8;
+      xargs.xtemplate.c_class = PseudoColor;
+      xargs.xtemplate_mask    = VisualDepthMask|VisualClassMask;
+      mXlibRgbHandle = xxlib_rgb_create_handle(mPDisplay, mScreen, &xargs);
+    }  
   }
   
   /* No XlibRgb handle ? Either we do not have a matching visual or no memory... */
@@ -319,9 +327,18 @@ nsXPrintContext::SetupWindow(int x, int y, int width, int height)
   xattributes.border_pixel     = foreground;
   xattributes.colormap         = xxlib_rgb_get_cmap(mXlibRgbHandle);
   xattributes_mask             = CWBorderPixel | CWBackPixel;
-  if( xattributes.colormap )
+  if( xattributes.colormap != None )
+  {
     xattributes_mask |= CWColormap;
 
+    /* XXX: Why do we need this ? See bug 80562 ("Xprint does not support any
+     * other visuals than Xprt's default one..").*/
+    if (mDepth > 12)
+    {
+      XInstallColormap(mPDisplay, xattributes.colormap);
+    }  
+  }
+
   mDrawable = (Drawable)XCreateWindow(mPDisplay, parent_win, x, y,
                                       width, height, 0,
                                       mDepth, InputOutput, mVisual, xattributes_mask,
@@ -336,15 +353,19 @@ nsXPrintContext::SetupWindow(int x, int y, int width, int height)
 }
 
 
-nsresult nsXPrintContext::SetPageSize(float page_width_mm, float page_height_mm)
+nsresult nsXPrintContext::SetMediumSize(const char *aPaperName)
 {
   nsresult                rv = NS_ERROR_GFX_PRINTER_PAPER_SIZE_NOT_SUPPORTED;
   XpuMediumSourceSizeList mlist;
   int                     mlist_count;
   int                     i;
+  char                   *paper_name,
+                         *alloc_paper_name; /* for free() ... */
+  paper_name = alloc_paper_name = strdup(aPaperName);
+  if (!paper_name)
+    return NS_ERROR_OUT_OF_MEMORY;
   
-  PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("SetPageSize: Requested page width_mm=%f, page height_mm=%f\n", 
-         (double)page_width_mm, (double)page_height_mm));
+  PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("SetMediumSize: Requested page '%s'\n", paper_name));
 
   mlist = XpuGetMediumSourceSizeList(mPDisplay, mPContext, &mlist_count);
   if( !mlist )
@@ -365,29 +386,38 @@ nsresult nsXPrintContext::SetPageSize(float page_width_mm, float page_height_mm)
   }
 #endif /* PR_LOGGING */
 
-  /* Tolerate +/- 2mm due conversion/rounding errors and different notations */
-  match = XpuFindMediumSourceSizeBySize(mlist, mlist_count, page_width_mm, page_height_mm, 2.0f);
- 
-  /* No match ?
-   * The "try again" with a tolerance if +/- 10mm
-   */
-  if (!match)
+  char *s;
+  
+  /* Did we get a tray name and paper name (e.g. "manual/din-a4") ? */
+  if (s = strchr(paper_name, '/'))
   {
-    PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, 
-           ("No match found in first attempt, trying again with 10mm tolerance...\n"));
-    match = XpuFindMediumSourceSizeBySize(mlist, mlist_count, page_width_mm, page_height_mm, 10.0f); 
-  }    
+    const char *tray_name;
+    *s = '\0';
+    tray_name  = paper_name;
+    paper_name = s+1;
+    
+    PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("SetMediumSize: searching for '%s'/'%s'\n", tray_name, paper_name));
+    match = XpuFindMediumSourceSizeByName(mlist, mlist_count, tray_name, paper_name);
+  }
+  else
+  {
+    PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("SetMediumSize: searching for '%s'\n", paper_name));
+    match = XpuFindMediumSourceSizeByName(mlist, mlist_count, nsnull, paper_name);
+  }
   
   /* Found a match ? */
   if (match)
   {
     PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG,
-           ("match %s/%s !\n", XPU_NULLXSTR(match->tray_name), match->medium_name));
+           ("match '%s'/'%s' !\n", XPU_NULLXSTR(match->tray_name), match->medium_name));
+           
+    /* Set document's paper size */
     if( XpuSetDocMediumSourceSize(mPDisplay, mPContext, match) == 1 )
       rv = NS_OK;  
   }
   
   XpuFreeMediumSourceSizeList(mlist);
+  free(alloc_paper_name);
   
   return rv;
 }
@@ -537,13 +567,10 @@ nsXPrintContext::SetupPrintContext(nsIDeviceContextSpecXp *aSpec)
   dumpXpAttributes(mPDisplay, mPContext);
 #endif /* XPRINT_DEBUG_SOMETIMES_USEFULL */
   
-  PRInt32 page_width_in_twips,
-          page_height_in_twips;
-
-  aSpec->GetPageSizeInTwips(&page_width_in_twips, &page_height_in_twips);
+  const char *paper_name = nsnull;
+  aSpec->GetPaperName(&paper_name);
   
-  if (NS_FAILED(XPU_TRACE(rv = SetPageSize(NS_TWIPS_TO_MILLIMETERS(page_width_in_twips),
-                                           NS_TWIPS_TO_MILLIMETERS(page_height_in_twips)))))
+  if (NS_FAILED(XPU_TRACE(rv = SetMediumSize(paper_name))))
     return rv;
   
   if (NS_FAILED(XPU_TRACE(rv = SetOrientation(landscape))))
