@@ -597,7 +597,7 @@ public:
 	  while(expn--) {
 		  ndex = ((root<=aValue) && (root)) ? (aValue/root): 0;
 		  aValue%=root;
-      aString.AppendWithConversion(aCharSet[ndex+((root>1)*offset)]);
+      aString.Append(PRUnichar(aCharSet[ndex+((root>1)*offset)]));
 		  root/=base;
     }
   }
@@ -712,8 +712,14 @@ public:
    * @param   aString -- will hold result
    */
   static void RomanString(PRInt32 aValue,nsString& aString) {
-	  static char digitsA[] = "ixcm";
-	  static char digitsB[] = "vld?";
+	  static PRUnichar digitsA[] = { PRUnichar('i'),
+                                   PRUnichar('x'),
+                                   PRUnichar('c'),
+                                   PRUnichar('m') };
+	  static PRUnichar digitsB[] = { PRUnichar('v'),
+                                   PRUnichar('l'),
+                                   PRUnichar('d'),
+                                   PRUnichar('?') };
 
     aString.Truncate();
     if(aValue<0)
@@ -732,20 +738,20 @@ public:
 		  romanPos--;
 		  switch(decStr[digitPos]) {
 			  case	'0':	break;
-			  case	'3':	aString.AppendWithConversion(digitsA[romanPos]);
-			  case	'2':	aString.AppendWithConversion(digitsA[romanPos]);
-			  case	'1':	aString.AppendWithConversion(digitsA[romanPos]);
+			  case	'3':	aString.Append(digitsA[romanPos]);
+			  case	'2':	aString.Append(digitsA[romanPos]);
+			  case	'1':	aString.Append(digitsA[romanPos]);
 				  break;
-			  case	'4':	aString.AppendWithConversion(digitsA[romanPos]);
+			  case	'4':	aString.Append(digitsA[romanPos]);
 			  case 	'5':	case	'6':
 			  case	'7':	case 	'8':	
-          aString.AppendWithConversion(digitsB[romanPos]);
+          aString.Append(digitsB[romanPos]);
 					for(n=0;n<(decStr[digitPos]-'5');n++)
-            aString.AppendWithConversion(digitsA[romanPos]);
+            aString.Append(digitsA[romanPos]);
 				  break;
 			  case	'9':	          
-          aString.AppendWithConversion(digitsA[romanPos]);
-          aString.AppendWithConversion(digitsA[romanPos]);
+          aString.Append(digitsA[romanPos]);
+          aString.Append(digitsA[romanPos]);
 					break;
 		  }
 	  }
@@ -789,7 +795,7 @@ public:
 	  modChar=aValue-(repCount*seriesLen);
 	  
     for(count=0;count<=repCount;count++) {
-      aString.AppendWithConversion(kFootnoteSet[modChar]);
+      aString.Append(PRUnichar(kFootnoteSet[modChar]));
     }
   }
 
@@ -1234,7 +1240,7 @@ public:
  *
  * @return  ptr to new token (or 0).
  */
-CToken* nsTokenAllocator::CreateTokenOfType(eHTMLTokenTypes aType,eHTMLTags aTag, const nsAReadableString& aString) {
+CToken* nsTokenAllocator::CreateTokenOfType(eHTMLTokenTypes aType,eHTMLTags aTag, const nsAString& aString) {
 
   CToken* result=0;
 
@@ -1377,7 +1383,8 @@ nsNodeAllocator::~nsNodeAllocator() {
 #endif
 }
   
-nsCParserNode* nsNodeAllocator::CreateNode(CToken* aToken,PRInt32 aLineNumber,nsTokenAllocator* aTokenAllocator) {
+nsCParserNode* nsNodeAllocator::CreateNode(CToken* aToken,  
+                                           nsTokenAllocator* aTokenAllocator) {
   nsCParserNode* result=0;
 
 #ifdef HEAP_ALLOCATED_NODES
@@ -1389,10 +1396,10 @@ nsCParserNode* nsNodeAllocator::CreateNode(CToken* aToken,PRInt32 aLineNumber,ns
 
   result=NS_STATIC_CAST(nsCParserNode*,mSharedNodes.Pop());
   if(result) {
-    result->Init(aToken,aLineNumber,aTokenAllocator,this);
+    result->Init(aToken, aTokenAllocator,this);
   }
   else{
-    result=nsCParserNode::Create(aToken,aLineNumber,aTokenAllocator,this);
+    result=nsCParserNode::Create(aToken, aTokenAllocator,this);
 #ifdef DEBUG_TRACK_NODES
     mCount++;
     AddNode(NS_STATIC_CAST(nsCParserNode*,result));
@@ -1400,7 +1407,7 @@ nsCParserNode* nsNodeAllocator::CreateNode(CToken* aToken,PRInt32 aLineNumber,ns
     IF_HOLD(result);
   }
 #else
-  result=nsCParserNode::Create(aToken,aLineNumber,aTokenAllocator,this);
+  result=nsCParserNode::Create(aToken, aTokenAllocator,this);
   IF_HOLD(result);
 #endif
   return result;
@@ -1516,7 +1523,8 @@ nsObserverEntry::~nsObserverEntry() {
 NS_IMETHODIMP
 nsObserverEntry::Notify(nsIParserNode* aNode,
                         nsIParser* aParser,
-                        nsISupports* aWebShell) 
+                        nsISupports* aWebShell,
+                        const PRUint32 aFlags) 
 {
   NS_ENSURE_ARG_POINTER(aNode);
   NS_ENSURE_ARG_POINTER(aParser);
@@ -1566,7 +1574,7 @@ nsObserverEntry::Notify(nsIParserNode* aNode,
           if (observer) {
             result = observer->Notify(aWebShell, channel,
                                       nsHTMLTags::GetStringValue(theTag),
-                                      &keys, &values);
+                                      &keys, &values, aFlags);
             if (NS_FAILED(result)) {
               break;
             }
