@@ -97,7 +97,7 @@ CStartToken::CStartToken(eHTMLTags aTag) : CHTMLToken(aTag) {
 #endif
 }
 
-CStartToken::CStartToken(const nsAReadableString& aName) : CHTMLToken(eHTMLTag_unknown) {
+CStartToken::CStartToken(const nsAString& aName) : CHTMLToken(eHTMLTag_unknown) {
   mEmpty=PR_FALSE;
   mContainerInfo=eFormUnknown;
   mTextValue.Assign(aName);
@@ -106,7 +106,7 @@ CStartToken::CStartToken(const nsAReadableString& aName) : CHTMLToken(eHTMLTag_u
 #endif
 }
 
-CStartToken::CStartToken(const nsAReadableString& aName,eHTMLTags aTag) : CHTMLToken(aTag) {
+CStartToken::CStartToken(const nsAString& aName,eHTMLTags aTag) : CHTMLToken(aTag) {
   mEmpty=PR_FALSE;
   mContainerInfo=eFormUnknown;
   mTextValue.Assign(aName);
@@ -227,6 +227,10 @@ nsresult CStartToken::Consume(PRUnichar aChar, nsScanner& aScanner,PRInt32 aFlag
     mTypeID = nsHTMLTags::LookupTag(mTextValue);
   }
 
+  if (NS_SUCCEEDED(result) && !(aFlag & NS_IPARSER_FLAG_VIEW_SOURCE)) {
+    result = aScanner.SkipWhitespace(mNewlineCount);
+  }
+
   return result;
 }
 
@@ -246,7 +250,7 @@ void CStartToken::DebugDumpSource(nsOutputStream& out) {
 }
 #endif
 
-const nsAReadableString& CStartToken::GetStringValue()
+const nsAString& CStartToken::GetStringValue()
 {
   if((eHTMLTag_unknown<mTypeID) && (mTypeID<eHTMLTag_text)) {
     if(!mTextValue.Length()) {
@@ -286,7 +290,7 @@ void CStartToken::GetSource(nsString& anOutputString){
  *  @param   result appended to the output string.
  *  @return  nada
  */
-void CStartToken::AppendSource(nsString& anOutputString){
+void CStartToken::AppendSourceTo(nsAString& anOutputString){
   anOutputString.Append(PRUnichar('<'));
   /*
    * Watch out for Bug 15204 
@@ -312,11 +316,11 @@ void CStartToken::AppendSource(nsString& anOutputString){
 CEndToken::CEndToken(eHTMLTags aTag) : CHTMLToken(aTag) {
 }
 
-CEndToken::CEndToken(const nsAReadableString& aName) : CHTMLToken(eHTMLTag_unknown) {
+CEndToken::CEndToken(const nsAString& aName) : CHTMLToken(eHTMLTag_unknown) {
   mTextValue.Assign(aName);
 }
 
-CEndToken::CEndToken(const nsAReadableString& aName,eHTMLTags aTag) : CHTMLToken(aTag) {
+CEndToken::CEndToken(const nsAString& aName,eHTMLTags aTag) : CHTMLToken(aTag) {
   mTextValue.Assign(aName);
 }
 
@@ -329,46 +333,30 @@ CEndToken::CEndToken(const nsAReadableString& aName,eHTMLTags aTag) : CHTMLToken
  *  @param   aFlag - contains information such as |dtd mode|view mode|doctype|etc...
  *  @return  error result
  */
-nsresult CEndToken::Consume(PRUnichar aChar, nsScanner& aScanner,PRInt32 aFlag) {
-  //if you're here, we've already Consumed the <! chars, and are
-   //ready to Consume the rest of the open tag identifier.
-   //Stop consuming as soon as you see a space or a '>'.
-   //NOTE: We don't Consume the tag attributes here, nor do we eat the ">"
-
-  nsresult result=NS_OK;
-  nsAutoString buffer;
-  PRInt32 offset;
+nsresult CEndToken::Consume(PRUnichar aChar, nsScanner& aScanner,PRInt32 aFlag) 
+{
+  nsresult result = NS_OK;
   if (aFlag & NS_IPARSER_FLAG_HTML) {
     nsAutoString theSubstr;
-    result=aScanner.ReadUntil(theSubstr,kGreaterThan,PR_FALSE);
-    if (NS_FAILED(result)) {
-      return result;
-    }
+    result=aScanner.GetIdentifier(theSubstr,PR_TRUE);
+    NS_ENSURE_SUCCESS(result, result);
     
-    offset = theSubstr.FindCharInSet(" \r\n\t\b",0);
-    if (offset != kNotFound) {
-      theSubstr.Left(buffer, offset);
-      mTypeID = nsHTMLTags::LookupTag(buffer);
-    }
-    else {
-      mTypeID = nsHTMLTags::LookupTag(theSubstr);
-    }
-
+    mTypeID = (PRInt32)nsHTMLTags::LookupTag(theSubstr);
     if(eHTMLTag_userdefined==mTypeID) {
       mTextValue=theSubstr;
     }
-
   }
   else {
-    mTextValue.SetLength(0);
-    result=aScanner.ReadUntil(mTextValue,kGreaterThan,PR_FALSE);
-    if (NS_FAILED(result)) {
-      return result;
-    }
-    mTypeID = eHTMLTag_userdefined;
+    result = aScanner.ReadIdentifier(mTextValue,PR_TRUE);
+    NS_ENSURE_SUCCESS(result, result);
+
+    mTypeID = nsHTMLTags::LookupTag(mTextValue);
   }
 
-  result=aScanner.GetChar(aChar); //eat the closing '>;
+  if (!(aFlag & NS_IPARSER_FLAG_VIEW_SOURCE)) {
+    result = aScanner.SkipWhitespace(mNewlineCount);
+    NS_ENSURE_SUCCESS(result, result);
+  }
 
   return result;
 }
@@ -433,7 +421,7 @@ void CEndToken::DebugDumpSource(nsOutputStream& out) {
 }
 #endif
 
-const nsAReadableString& CEndToken::GetStringValue()
+const nsAString& CEndToken::GetStringValue()
 {
   if((eHTMLTag_unknown<mTypeID) && (mTypeID<eHTMLTag_text)) {
     if(!mTextValue.Length()) {
@@ -466,7 +454,7 @@ void CEndToken::GetSource(nsString& anOutputString){
  *  @param   result appended to the output string.
  *  @return  nada
  */
-void CEndToken::AppendSource(nsString& anOutputString){
+void CEndToken::AppendSourceTo(nsAString& anOutputString){
   anOutputString.Append(NS_LITERAL_STRING("</"));
   if(mTextValue.Length()>0)
     anOutputString.Append(mTextValue);
@@ -493,7 +481,7 @@ CTextToken::CTextToken() : CHTMLToken(eHTMLTag_text) {
  *  @param   aName -- string to init token name with
  *  @return  
  */
-CTextToken::CTextToken(const nsAReadableString& aName) : CHTMLToken(eHTMLTag_text) {
+CTextToken::CTextToken(const nsAString& aName) : CHTMLToken(eHTMLTag_text) {
   mTextValue.Rebind(aName);
 }
 
@@ -636,7 +624,7 @@ nsresult CTextToken::ConsumeUntil(PRUnichar aChar,PRBool aIgnoreComments,nsScann
     while (FindCharInReadable(PRUnichar(kLessThan), ltOffset, endPos) &&
            Distance(ltOffset, endPos) >= termStrLen) {
       // Make a copy of the (presumed) end tag and
-      // do a case-insensitive comparision
+      // do a case-insensitive comparison
 
       nsReadingIterator<PRUnichar> start(ltOffset), end(ltOffset);
       end.advance(termStrLen);
@@ -719,12 +707,12 @@ nsresult CTextToken::ConsumeUntil(PRUnichar aChar,PRBool aIgnoreComments,nsScann
   return result;
 }
 
-void CTextToken::CopyTo(nsAWritableString& aStr)
+void CTextToken::CopyTo(nsAString& aStr)
 {
   aStr.Assign(mTextValue);
 }
 
-const nsAReadableString& CTextToken::GetStringValue(void)
+const nsAString& CTextToken::GetStringValue(void)
 {
   return mTextValue;
 }
@@ -734,7 +722,7 @@ void CTextToken::Bind(nsScanner* aScanner, nsReadingIterator<PRUnichar>& aStart,
   aScanner->BindSubstring(mTextValue, aStart, aEnd);
 }
 
-void CTextToken::Bind(const nsAReadableString& aStr)
+void CTextToken::Bind(const nsAString& aStr)
 {
   mTextValue.Rebind(aStr);
 }
@@ -757,7 +745,7 @@ CCDATASectionToken::CCDATASectionToken() : CHTMLToken(eHTMLTag_unknown) {
  *  @param   aName -- string to init token name with
  *  @return  
  */
-CCDATASectionToken::CCDATASectionToken(const nsAReadableString& aName) : CHTMLToken(eHTMLTag_unknown) {
+CCDATASectionToken::CCDATASectionToken(const nsAString& aName) : CHTMLToken(eHTMLTag_unknown) {
   mTextValue.Assign(aName);
 }
 
@@ -825,11 +813,12 @@ nsresult CCDATASectionToken::Consume(PRUnichar aChar, nsScanner& aScanner,PRInt3
         mTextValue.Append(aChar);
         result=aScanner.Peek(aChar);    //then see what's next.
         if((NS_OK==result) && (kRightSquareBracket==aChar)) {
-          result=aScanner.GetChar(aChar);    //strip off the second ]
+          result=aScanner.GetChar(aChar); //strip off the second ]
           mTextValue.Append(aChar);
-          result=aScanner.Peek(aChar);    //then see what's next.
         }
-        if((NS_OK==result) && (kGreaterThan==aChar)) {
+        nsAutoString dummy; // skip any bad data
+        result=aScanner.ReadUntil(dummy,kGreaterThan,PR_FALSE);
+        if (NS_OK==result) {
           result=aScanner.GetChar(aChar); //strip off the >
           done=PR_TRUE;
         }
@@ -840,7 +829,7 @@ nsresult CCDATASectionToken::Consume(PRUnichar aChar, nsScanner& aScanner,PRInt3
   return result;
 }
 
-const nsAReadableString& CCDATASectionToken::GetStringValue(void)
+const nsAString& CCDATASectionToken::GetStringValue(void)
 {
   return mTextValue;
 }
@@ -862,7 +851,7 @@ CMarkupDeclToken::CMarkupDeclToken() : CHTMLToken(eHTMLTag_markupDecl) {
  *  @param   aName -- string to init token name with
  *  @return  
  */
-CMarkupDeclToken::CMarkupDeclToken(const nsAReadableString& aName) : CHTMLToken(eHTMLTag_markupDecl) {
+CMarkupDeclToken::CMarkupDeclToken(const nsAString& aName) : CHTMLToken(eHTMLTag_markupDecl) {
   mTextValue.Rebind(aName);
 }
 
@@ -977,7 +966,7 @@ nsresult CMarkupDeclToken::Consume(PRUnichar aChar, nsScanner& aScanner,PRInt32
   return result;
 }
 
-const nsAReadableString& CMarkupDeclToken::GetStringValue(void)
+const nsAString& CMarkupDeclToken::GetStringValue(void)
 {
   return mTextValue;
 }
@@ -1001,176 +990,249 @@ CCommentToken::CCommentToken() : CHTMLToken(eHTMLTag_comment) {
  *  @param   
  *  @return  
  */
-CCommentToken::CCommentToken(const nsAReadableString& aName) : CHTMLToken(eHTMLTag_comment) {
-  mTextValue.Assign(aName);
+CCommentToken::CCommentToken(const nsAString& aName) : CHTMLToken(eHTMLTag_comment) {
+  mComment.Rebind(aName);
 }
 
-/*
- *  This method consumes a comment using the (CORRECT) comment parsing
- *  algorithm according to ISO 8879 (SGML).
- *  
- *  @update  gess 01/04/99
- *  @param   
- *  @param   
- *  @return  
- */
-static
-nsresult ConsumeStrictComment(PRUnichar aChar, nsScanner& aScanner,nsString& aString) {
-  nsresult  result=NS_OK;
-  static const PRUnichar theTerminalsChars[] = 
-    { PRUnichar('-'), PRUnichar('>'), PRUnichar(0) };
-  static const nsReadEndCondition endCommentCondition(theTerminalsChars);
- 
+void CCommentToken::AppendSourceTo(nsAString& anOutputString){
+  anOutputString.Append(mCommentDecl);
+}
+
+static PRBool IsCommentEnd(
+  const nsReadingIterator<PRUnichar>& aCurrent, 
+  const nsReadingIterator<PRUnichar>& aEnd, 
+  nsReadingIterator<PRUnichar>& aGt)
+{
+  nsReadingIterator<PRUnichar> current = aCurrent;
+  PRInt32 dashes = 0;
+
+  while ((current != aEnd) && (dashes != 2)) {
+    if (*current == kGreaterThan) {
+      aGt = current;
+      return PR_TRUE;
+    }
+    if (*current == PRUnichar('-')) {
+      dashes++;
+    } else {
+      dashes = 0;
+    }
+    ++current;
+  }
+
+  return PR_FALSE;
+}
+
+nsresult CCommentToken::ConsumeStrictComment(nsScanner& aScanner) 
+{
+  // <!--[... -- ... -- ...]*-->
   /*********************************************************
     NOTE: This algorithm does a fine job of handling comments
           when they're formatted per spec, but if they're not
-          we don't handle them well. For example, we gack
-          on the following:
-
-          <!-- xx -- xx --> 
+          we don't handle them well.
    *********************************************************/
+  nsReadingIterator<PRUnichar> end, current, gt, lt;
+  aScanner.EndReading(end);
+  aScanner.CurrentPosition(current);
 
-  aString.Assign(NS_LITERAL_STRING("<!"));
-  while(NS_OK==result) {
-    result=aScanner.GetChar(aChar);
-    if(NS_OK==result) {
-      aString+=aChar;
-      if(kMinus==aChar) {
-        result=aScanner.GetChar(aChar);
-        if(NS_OK==result) {
-          if(kMinus==aChar) {
-               //in this case, we're reading a long-form comment <-- xxx -->
-            aString+=aChar;
-            if(NS_OK==result) {
-              //Find the first ending sequence '--'
-              PRBool found = PR_FALSE;
-              nsAutoString temp;
-              do {
-                result = aScanner.ReadUntil(temp,kMinus,PR_TRUE);
-                if (NS_FAILED(result)) return result;
-                  
-                result = aScanner.Peek(aChar);
-                if (NS_FAILED(result)) return result;
-                  
-                if (aChar == kMinus) {
-                  // found matching '--'
-                  found = PR_TRUE;
-                  aScanner.GetChar(aChar);
-                  aString += temp;
-                  aString += aChar;
-                  // Read until "-" or ">"
-                  result = aScanner.ReadUntil(aString,endCommentCondition,PR_FALSE);
-                }
-              } while (!found);
-            }
-          } //
-          else break; //go find '>'
+  nsReadingIterator<PRUnichar> beginData = end;
+
+  lt = current;
+  lt.advance(-2); // <!
+
+  // Regular comment must start with <!--
+  if (current != end && *current == kMinus &&
+      ++current != end && *current == kMinus &&
+      ++current != end) {
+    nsReadingIterator<PRUnichar> currentEnd = end;
+    PRBool balancedComment = PR_FALSE;
+    static NS_NAMED_LITERAL_STRING(dashes,"--");
+    beginData = current;
+
+    while (FindInReadable(dashes, current, currentEnd)) {
+      current.advance(2);
+
+      balancedComment = !balancedComment; // We need to match '--' with '--'
+    
+      if (balancedComment && IsCommentEnd(current, end, gt)) {
+        // done
+        current.advance(-2);
+        if (beginData != current) { // protects from <!---->
+          aScanner.BindSubstring(mComment, beginData, current);
         }
-      }//if
-      else if(kGreaterThan==aChar) {
-        return result;
+        aScanner.BindSubstring(mCommentDecl, lt, ++gt);
+        aScanner.SetPosition(gt);
+        return NS_OK;
+      } else {
+        // Continue after the last '--'
+        currentEnd = end;
       }
-      else break; //go find '>'
-    }//if
-  }//while
-  if(NS_OK==result) {
-     //Read up to the closing '>', unless you already did!  (such as <!>).
-    if(kGreaterThan!=aChar) {
-      result=aScanner.ReadUntil(aString,kGreaterThan,PR_TRUE);
     }
   }
-  return result;
+
+  // If beginData == end, we did not find opening '--'
+  if (beginData == end) {
+    // This might have been empty comment: <!>
+    // Or it could have been something completely bogus like: <!This is foobar>
+    // Handle both cases below
+    aScanner.CurrentPosition(current);
+    beginData = current;
+    if (FindCharInReadable('>', current, end)) {
+      aScanner.BindSubstring(mComment, beginData, current); 
+      aScanner.BindSubstring(mCommentDecl, lt, ++current);
+      aScanner.SetPosition(current);
+      return NS_OK;
+    }
+  }
+
+  if (aScanner.IsIncremental()) {
+    // We got here because we saw the beginning of a comment,
+    // but not yet the end, and we are still loading the page. In that
+    // case the return value here will cause us to unwind,
+    // wait for more content, and try again.
+    // XXX For performance reasons we should cache where we were, and
+    //     continue from there for next call
+    return kEOF; // not really an nsresult, but...
+  }
+
+  // XXX We should return kNotAComment, parse comment open as text, and parse
+  //     the rest of the document normally. Now we ALMOST do that: <! is
+  //     missing from the content model.
+  return NS_OK;
 }
 
-/*
- *  This method consumes a comment using common (actually non-standard)
- *  algorithm that seems to work against the content on the web.
- *  
- *  @update  gess 01/04/99
- *  @param   
- *  @param   
- *  @return  
- */
-static
-nsresult ConsumeComment(PRUnichar aChar, nsScanner& aScanner,nsString& aString) {
-  
-
-  nsresult  result=NS_OK;
- 
+nsresult CCommentToken::ConsumeQuirksComment(nsScanner& aScanner) 
+{
+  // <![-[-]] ... [[-]-|--!]>
   /*********************************************************
     NOTE: This algorithm does a fine job of handling comments
           commonly used, but it doesn't really consume them
           per spec (But then, neither does IE or Nav).
    *********************************************************/
+  nsReadingIterator<PRUnichar> end, current;
+  aScanner.EndReading(end);
+  aScanner.CurrentPosition(current);
+  nsReadingIterator<PRUnichar> beginData = current, 
+                               beginLastMinus = end,
+                               bestAltCommentEnd = end,
+                               lt = current;
+  lt.advance(-2); // <!
 
-  nsReadingIterator<PRUnichar> theStartOffset, theCurrOffset, theBestAltPos, endPos;
-  aScanner.EndReading(endPos);
-  theBestAltPos = endPos;
+  // When we get here, we have always already consumed <!
+  // Skip over possible leading minuses
+  if (current != end && *current == kMinus) {
+    beginLastMinus = current;
+    ++current;
+    ++beginData;
+    if (current != end && *current == kMinus) { // <!--
+      beginLastMinus = current;
+      ++current;
+      ++beginData;
+      // Long form comment
 
-  aScanner.CurrentPosition(theStartOffset);
-  result=aScanner.GetChar(aChar);
-  if(NS_OK==result) {
-    if(kMinus==aChar) {
-      result=aScanner.GetChar(aChar);
-      if(NS_OK==result) {
-        if(kMinus==aChar) {
-          //in this case, we're reading a long-form comment <-- xxx -->
-
-          theCurrOffset = theStartOffset;
-        
-          while((NS_OK==result)) {
-            if (FindCharInReadable(PRUnichar(kGreaterThan), theCurrOffset, endPos)) {
-              ++theCurrOffset;
-              nsReadingIterator<PRUnichar> temp = theCurrOffset;
-              temp.advance(-3);
-              aChar=*temp;
-              if(kMinus==aChar) {
-                ++temp;
-                aChar=*temp;
-                if(kMinus==aChar) {
-                  theStartOffset.advance(-2); // Include "<!" also..
-                  CopyUnicodeTo(theStartOffset, theCurrOffset, aString);
-                  aScanner.SetPosition(theCurrOffset);
-                  return result; // We have found the dflt end comment delimiter ("-->")
-                }
-              }
-              if(theBestAltPos == endPos) {
-                // If we did not find the dflt then assume that '>' is the end comment
-                // until we find '-->'. Nav. Compatibility -- Ref: Bug# 24006
-                theBestAltPos=theCurrOffset;
-              }
-            }
-            else {
-              result=kEOF;
-            }
-          } //while
-          if((endPos==theCurrOffset) && (!aScanner.IsIncremental())) {
-            //if you're here, then we're in a special state. 
-            //The problem at hand is that we've hit the end of the document without finding the normal endcomment delimiter "-->".
-            //In this case, the first thing we try is to see if we found one of the alternate endcomment delimiter ">".
-            //If so, rewind just pass than, and use everything up to that point as your comment.
-            //If not, the document has no end comment and should be treated as one big comment.
-            if(endPos != theBestAltPos) {
-              theStartOffset.advance(-2);// Include "<!" also..
-              CopyUnicodeTo(theStartOffset, theBestAltPos, aString);
-              aScanner.SetPosition(theBestAltPos);
-              result=NS_OK;
+      nsReadingIterator<PRUnichar> currentEnd = end, gt = end;
+      
+      // Find the end of the comment
+      while (FindCharInReadable(kGreaterThan, current, currentEnd)) {
+        gt = current;
+        if (bestAltCommentEnd == end) {
+          bestAltCommentEnd = gt;
+        }
+        --current;
+        PRBool goodComment = PR_FALSE;
+        if (current != beginLastMinus && *current == kMinus) { // ->
+          --current;
+          if (current != beginLastMinus && *current == kMinus) { // -->
+            goodComment = PR_TRUE;
+            --current;
+          }
+        } else if (current != beginLastMinus && *current == '!') {
+          --current;
+          if (current != beginLastMinus && *current == kMinus) {
+            --current;
+            if (current != beginLastMinus && *current == kMinus) { // --!>
+              --current;
+              goodComment = PR_TRUE;
             }
           }
-          return result;
+        } else if (current == beginLastMinus) {
+          goodComment = PR_TRUE;
+        }
+    
+        if (goodComment) {
+          // done
+          if (beginLastMinus != current) { // protects from <!---->
+            aScanner.BindSubstring(mComment, beginData, ++current);
+          }
+          aScanner.BindSubstring(mCommentDecl, lt, ++gt);
+          aScanner.SetPosition(gt);
+          return NS_OK;
+        } else {
+          // try again starting after the last '>'
+          current = ++gt;
+          currentEnd = end;
+        }
+      } //while
+  
+      if (aScanner.IsIncremental()) {
+        // We got here because we saw the beginning of a comment,
+        // but not yet the end, and we are still loading the page. In that
+        // case the return value here will cause us to unwind,
+        // wait for more content, and try again.
+        // XXX For performance reasons we should cache where we were, and
+        //     continue from there for next call
+        return kEOF;  // not really an nsresult, but...
+      }
 
-        } //if
-      }//if
-    }//if
-  }//if
-  if(NS_OK==result) {
-     //Read up to the closing '>', unless you already did!  (such as <!>).
-    if(kGreaterThan!=aChar) {
-      aString.Append(NS_LITERAL_STRING("<!- "));
-      result=aScanner.ReadUntil(aString,kGreaterThan,PR_TRUE);
+      // If you're here, then we're in a special state. 
+      // The problem at hand is that we've hit the end of the document without finding the normal endcomment delimiter "-->".
+      // In this case, the first thing we try is to see if we found an alternate endcomment delimiter ">".
+      // If so, rewind just pass that, and use everything up to that point as your comment.
+      // If not, the document has no end comment and should be treated as one big comment.
+      gt = bestAltCommentEnd;
+      if (beginData != gt) { // protects from <!-->
+        aScanner.BindSubstring(mComment, beginData, gt);
+      }
+      if (gt != end) {
+        ++gt;
+      }
+      aScanner.BindSubstring(mCommentDecl, lt, gt);
+      aScanner.SetPosition(gt);
+      return NS_OK;
     }
   }
-  return result;
+  
+  // This could be short form of comment
+  // Find the end of the comment
+  current = beginData;
+  if (FindCharInReadable(kGreaterThan, current, end)) {
+    nsReadingIterator<PRUnichar> gt = current;
+    if (current != beginData) {
+      --current;
+      if (current != beginData && *current == kMinus) { // ->
+        --current;
+        if (current != beginData && *current == kMinus) { // -->
+          --current;
+        }
+      } else if (current != beginData && *current == '!') { // !>
+        --current;
+        if (current != beginData && *current == kMinus) { // -!>
+          --current;
+          if (current != beginData && *current == kMinus) { // --!>
+            --current;
+          }
+        }
+      }
+    }
+
+    if (current != gt) {
+      aScanner.BindSubstring(mComment, beginData, ++current);
+    }
+    aScanner.BindSubstring(mCommentDecl, lt, ++gt);
+    aScanner.SetPosition(gt);
+    return NS_OK;
+  }
+
+  return kEOF; // not really an nsresult, but...
 }
 
 /*
@@ -1187,17 +1249,22 @@ nsresult CCommentToken::Consume(PRUnichar aChar, nsScanner& aScanner,PRInt32 aFl
   
   if (aFlag & NS_IPARSER_FLAG_STRICT_MODE) {
     //Enabling strict comment parsing for Bug 53011 and  2749 contradicts!!!!
-    result=ConsumeStrictComment(aChar,aScanner,mTextValue);
+    result = ConsumeStrictComment(aScanner);
   }
   else {
-    result=ConsumeComment(aChar,aScanner,mTextValue);
+    result = ConsumeQuirksComment(aScanner);
   }
+
+  if (NS_SUCCEEDED(result)) {
+    mNewlineCount = !(aFlag & NS_IPARSER_FLAG_VIEW_SOURCE) ? mCommentDecl.CountChar(kNewLine) : -1;
+  }
+
   return result;
 }
 
-const nsAReadableString& CCommentToken::GetStringValue(void)
+const nsAString& CCommentToken::GetStringValue(void)
 {
-  return mTextValue;
+  return mComment;
 }
 
 /* 
@@ -1276,7 +1343,7 @@ void CNewlineToken::FreeNewline()
  *  @update gess 3/25/98
  *  @return nsString reference to internal string value
  */
-const nsAReadableString& CNewlineToken::GetStringValue(void) {
+const nsAString& CNewlineToken::GetStringValue(void) {
   return *gNewlineStr;
 }
 
@@ -1319,7 +1386,9 @@ nsresult CNewlineToken::Consume(PRUnichar aChar, nsScanner& aScanner,PRInt32 aFl
       default:
         break;
     }
-  }  
+  }
+
+  mNewlineCount = 1;
   return result;
 }
 
@@ -1344,7 +1413,7 @@ CAttributeToken::CAttributeToken() : CHTMLToken(eHTMLTag_unknown) {
  *  @param   aName -- string value to init token name with
  *  @return  
  */
-CAttributeToken::CAttributeToken(const nsAReadableString& aName) : CHTMLToken(eHTMLTag_unknown) {
+CAttributeToken::CAttributeToken(const nsAString& aName) : CHTMLToken(eHTMLTag_unknown) {
   mTextValue.Assign(aName);
   mHasEqualWithoutValue=PR_FALSE;
 #ifdef DEBUG
@@ -1360,7 +1429,7 @@ CAttributeToken::CAttributeToken(const nsAReadableString& aName) : CHTMLToken(eH
  *  @param   aName -- string value to init token name with
  *  @return  
  */
-CAttributeToken::CAttributeToken(const nsAReadableString& aKey, const nsAReadableString& aName) : CHTMLToken(eHTMLTag_unknown) {
+CAttributeToken::CAttributeToken(const nsAString& aKey, const nsAString& aName) : CHTMLToken(eHTMLTag_unknown) {
   mTextValue.Assign(aName);
   mTextKey.Rebind(aKey);
   mHasEqualWithoutValue=PR_FALSE;
@@ -1418,7 +1487,7 @@ void CAttributeToken::SanitizeKey() {
     // legal part
     if (iter != --end) {
       nsAutoString str;
-      CopyUnicodeTo(begin, iter, str);
+      CopyUnicodeTo(begin, ++iter, str);
       mTextKey.Rebind(str);
     }
   }
@@ -1442,7 +1511,7 @@ void CAttributeToken::DebugDumpToken(nsOutputStream& out) {
 }
 #endif
 
-const nsAReadableString& CAttributeToken::GetStringValue(void)
+const nsAString& CAttributeToken::GetStringValue(void)
 {
   return mTextValue;
 }
@@ -1456,7 +1525,7 @@ const nsAReadableString& CAttributeToken::GetStringValue(void)
  */
 void CAttributeToken::GetSource(nsString& anOutputString){
   anOutputString.Truncate();
-  AppendSource(anOutputString);
+  AppendSourceTo(anOutputString);
 }
 
 /*
@@ -1466,7 +1535,7 @@ void CAttributeToken::GetSource(nsString& anOutputString){
  *  @param   result appended to the output string.
  *  @return  nada
  */
-void CAttributeToken::AppendSource(nsString& anOutputString){
+void CAttributeToken::AppendSourceTo(nsAString& anOutputString){
   anOutputString.Append(mTextKey);
   if(mTextValue.Length() || mHasEqualWithoutValue) 
     anOutputString.Append(NS_LITERAL_STRING("="));
@@ -1652,10 +1721,10 @@ nsresult CAttributeToken::Consume(PRUnichar aChar, nsScanner& aScanner,PRInt32 a
   nsReadingIterator<PRUnichar> wsstart, wsend;
   
   if (aFlag & NS_IPARSER_FLAG_VIEW_SOURCE) {
-    result = aScanner.ReadWhitespace(wsstart, wsend);
+    result = aScanner.ReadWhitespace(wsstart, wsend, mNewlineCount);
   }
   else {
-    result = aScanner.SkipWhitespace();
+    result = aScanner.SkipWhitespace(mNewlineCount);
   }
 
   if (NS_OK==result) {
@@ -1677,11 +1746,11 @@ nsresult CAttributeToken::Consume(PRUnichar aChar, nsScanner& aScanner,PRInt32 a
     //now it's time to Consume the (optional) value...
     if (NS_OK==result) {
       if (aFlag & NS_IPARSER_FLAG_VIEW_SOURCE) {
-        result = aScanner.ReadWhitespace(start, wsend);
+        result = aScanner.ReadWhitespace(start, wsend, mNewlineCount);
         aScanner.BindSubstring(mTextKey, wsstart, wsend);
       }
       else {
-        result = aScanner.SkipWhitespace();
+        result = aScanner.SkipWhitespace(mNewlineCount);
       }
 
       if (NS_OK==result) { 
@@ -1691,10 +1760,10 @@ nsresult CAttributeToken::Consume(PRUnichar aChar, nsScanner& aScanner,PRInt32 a
             result=aScanner.GetChar(aChar);  //skip the equal sign...
             if (NS_OK==result) {
               if (aFlag & NS_IPARSER_FLAG_VIEW_SOURCE) {
-                result = aScanner.ReadWhitespace(mTextValue);
+                result = aScanner.ReadWhitespace(mTextValue, mNewlineCount);
               }
               else {
-                result = aScanner.SkipWhitespace();
+                result = aScanner.SkipWhitespace(mNewlineCount);
               }
 
               if (NS_OK==result) {
@@ -1730,10 +1799,10 @@ nsresult CAttributeToken::Consume(PRUnichar aChar, nsScanner& aScanner,PRInt32 a
                 }//if
                 if (NS_OK==result) {
                   if (aFlag & NS_IPARSER_FLAG_VIEW_SOURCE) {
-                    result = aScanner.ReadWhitespace(mTextValue);
+                    result = aScanner.ReadWhitespace(mTextValue, mNewlineCount);
                   }
                   else {
-                    result = aScanner.SkipWhitespace();
+                    result = aScanner.SkipWhitespace(mNewlineCount);
                   }
                 }
               }//if
@@ -1784,7 +1853,7 @@ void CAttributeToken::DebugDumpSource(nsOutputStream& out) {
 }
 #endif
 
-void CAttributeToken::SetKey(const nsAReadableString& aKey)
+void CAttributeToken::SetKey(const nsAString& aKey)
 {
   mTextKey.Rebind(aKey);
 }
@@ -1814,7 +1883,7 @@ CWhitespaceToken::CWhitespaceToken() : CHTMLToken(eHTMLTag_whitespace) {
  *  @param   aName -- string value to init token name with
  *  @return  
  */
-CWhitespaceToken::CWhitespaceToken(const nsAReadableString& aName) : CHTMLToken(eHTMLTag_whitespace) {
+CWhitespaceToken::CWhitespaceToken(const nsAString& aName) : CHTMLToken(eHTMLTag_whitespace) {
   mTextValue.Assign(aName);
 }
 
@@ -1851,14 +1920,14 @@ PRInt32 CWhitespaceToken::GetTokenType(void) {
  */
 nsresult CWhitespaceToken::Consume(PRUnichar aChar, nsScanner& aScanner,PRInt32 aFlag) {
   mTextValue.Assign(aChar);
-  nsresult result=aScanner.ReadWhitespace(mTextValue);
+  nsresult result=aScanner.ReadWhitespace(mTextValue, mNewlineCount);
   if(NS_OK==result) {
     mTextValue.StripChar(kCR);
   }
   return result;
 }
 
-const nsAReadableString& CWhitespaceToken::GetStringValue(void)
+const nsAString& CWhitespaceToken::GetStringValue(void)
 {
   return mTextValue;
 }
@@ -1880,7 +1949,7 @@ CEntityToken::CEntityToken() : CHTMLToken(eHTMLTag_entity) {
  *  @param   aName -- string value to init token name with
  *  @return  
  */
-CEntityToken::CEntityToken(const nsAReadableString& aName) : CHTMLToken(eHTMLTag_entity) {
+CEntityToken::CEntityToken(const nsAString& aName) : CHTMLToken(eHTMLTag_entity) {
   mTextValue.Assign(aName);
 #ifdef VERBOSE_DEBUG
   if(!VerifyEntityTable())  {
@@ -2096,12 +2165,12 @@ PRInt32 CEntityToken::TranslateToUnicodeStr(nsString& aString) {
       value=mTextValue.ToInteger(&err,kAutoDetect);
 
       if(0==err) {
-  #ifdef PA_REMAP_128_TO_160_ILLEGAL_NCR
+#ifdef PA_REMAP_128_TO_160_ILLEGAL_NCR
         /* for some illegal, but popular usage */
         if ((value >= 0x0080) && (value <= 0x009f)) {
           value = PA_HackTable[value - 0x0080];
         }
-  #endif
+#endif
         if (IS_IN_BMP(value))
           aString.Append(PRUnichar(value));
         else {
@@ -2137,7 +2206,7 @@ void CEntityToken::DebugDumpSource(nsOutputStream& out) {
 }
 #endif
 
-const nsAReadableString& CEntityToken::GetStringValue(void)
+const nsAString& CEntityToken::GetStringValue(void)
 {
   return mTextValue;
 }
@@ -2162,7 +2231,7 @@ void CEntityToken::GetSource(nsString& anOutputString){
  *  @param   result appended to the output string.
  *  @return  nada
  */
-void CEntityToken::AppendSource(nsString& anOutputString){
+void CEntityToken::AppendSourceTo(nsAString& anOutputString){
   anOutputString.Append(NS_LITERAL_STRING("&"));
   anOutputString+=mTextValue;
   //anOutputString+=";";
@@ -2185,7 +2254,7 @@ CScriptToken::CScriptToken() : CHTMLToken(eHTMLTag_script) {
  *  @param   aName -- string to init token name with
  *  @return  
  */
-CScriptToken::CScriptToken(const nsAReadableString& aString) : CHTMLToken(eHTMLTag_script) {
+CScriptToken::CScriptToken(const nsAString& aString) : CHTMLToken(eHTMLTag_script) {
   mTextValue.Assign(aString);
 }
                         
@@ -2212,7 +2281,7 @@ PRInt32 CScriptToken::GetTokenType(void) {
   return eToken_script;
 }
 
-const nsAReadableString& CScriptToken::GetStringValue(void)
+const nsAString& CScriptToken::GetStringValue(void)
 {
   return mTextValue;
 }
@@ -2227,7 +2296,7 @@ const nsAReadableString& CScriptToken::GetStringValue(void)
 CStyleToken::CStyleToken() : CHTMLToken(eHTMLTag_style) {
 }
 
-CStyleToken::CStyleToken(const nsAReadableString& aString) : CHTMLToken(eHTMLTag_style) {
+CStyleToken::CStyleToken(const nsAString& aString) : CHTMLToken(eHTMLTag_style) {
   mTextValue.Assign(aString);
 }
 
@@ -2253,7 +2322,7 @@ PRInt32 CStyleToken::GetTokenType(void) {
   return eToken_style;
 }
 
-const nsAReadableString& CStyleToken::GetStringValue(void)
+const nsAString& CStyleToken::GetStringValue(void)
 {
   return mTextValue;
 }
@@ -2297,7 +2366,7 @@ CInstructionToken::CInstructionToken() : CHTMLToken(eHTMLTag_instruction) {
  *  @param   
  *  @return  
  */
-CInstructionToken::CInstructionToken(const nsAReadableString& aString) : CHTMLToken(eHTMLTag_unknown) {
+CInstructionToken::CInstructionToken(const nsAString& aString) : CHTMLToken(eHTMLTag_unknown) {
   mTextValue.Assign(aString);
 }
 
@@ -2336,7 +2405,7 @@ PRInt32 CInstructionToken::GetTokenType(void){
   return eToken_instruction;
 }
 
-const nsAReadableString& CInstructionToken::GetStringValue(void)
+const nsAString& CInstructionToken::GetStringValue(void)
 {
   return mTextValue;
 }
@@ -2369,7 +2438,7 @@ const nsParserError * CErrorToken::GetError(void)
   return mError; 
 }
 
-const nsAReadableString& CErrorToken::GetStringValue(void)
+const nsAString& CErrorToken::GetStringValue(void)
 {
   return mTextValue;
 }
@@ -2380,7 +2449,7 @@ CDoctypeDeclToken::CDoctypeDeclToken(eHTMLTags aTag)
   : CHTMLToken(aTag) {
 }
 
-CDoctypeDeclToken::CDoctypeDeclToken(const nsAReadableString& aString,eHTMLTags aTag)
+CDoctypeDeclToken::CDoctypeDeclToken(const nsAString& aString,eHTMLTags aTag)
   : CHTMLToken(aTag), mTextValue(aString) {
 }
 
@@ -2440,12 +2509,12 @@ PRInt32 CDoctypeDeclToken::GetTokenType(void) {
   return eToken_doctypeDecl;
 }
 
-const nsAReadableString& CDoctypeDeclToken::GetStringValue(void)
+const nsAString& CDoctypeDeclToken::GetStringValue(void)
 {
   return mTextValue;
 }
 
-void CDoctypeDeclToken::SetStringValue(const nsAReadableString& aStr)
+void CDoctypeDeclToken::SetStringValue(const nsAString& aStr)
 {
   mTextValue.Assign(aStr);
 }
