@@ -41,6 +41,7 @@
 #include "nsReadableUtils.h"
 #include "prprf.h"
 
+
 static NS_DEFINE_IID(kIContentSinkIID, NS_ICONTENT_SINK_IID);
 static NS_DEFINE_IID(kIHTMLContentSinkIID, NS_IHTML_CONTENT_SINK_IID);
 static NS_DEFINE_IID(kILoggingSinkIID, NS_ILOGGING_SINK_IID);
@@ -197,6 +198,12 @@ nsLoggingSink::SetParser(nsIParser* aParser)  {
     theResult=mSink->SetParser(aParser);
   }
   
+  NS_IF_RELEASE(mParser);
+  
+  mParser = aParser;
+  
+  NS_IF_ADDREF(mParser);
+
   return theResult;
 }
 
@@ -575,7 +582,7 @@ nsLoggingSink::WriteAttributes(const nsIParserNode& aNode) {
   for (PRInt32 i = 0; i < ac; i++) {
     char* key=nsnull;
     char* value=nsnull;
-    const nsAReadableString& k = aNode.GetKeyAt(i);
+    const nsAString& k = aNode.GetKeyAt(i);
     const nsAString& v = aNode.GetValueAt(i);
 
     GetNewCString(k, &key);
@@ -610,8 +617,16 @@ nsLoggingSink::WriteAttributes(const nsIParserNode& aNode) {
   }
 
   if (0 != strchr(gSkippedContentTags, aNode.GetNodeType())) {
+    nsCOMPtr<nsIDTD> dtd;
+    mParser->GetDTD(getter_AddRefs(dtd));
+    NS_ENSURE_TRUE(dtd, NS_ERROR_FAILURE);
+    
+    nsString theString;
+    PRInt32 lineNo = 0;
+
+    dtd->CollectSkippedContent(aNode.GetNodeType(), theString, lineNo);
     char* content;
-    GetNewCString(aNode.GetSkippedContent(), &content);
+    GetNewCString(theString, &content);
     if(content) {
       PR_fprintf(mOutput, " <content value=\"");
       PR_fprintf(mOutput, "%s\"/>\n", content) ;
@@ -630,7 +645,14 @@ nsLoggingSink::WillWriteAttributes(const nsIParserNode& aNode)
     return PR_TRUE;
   }
   if (0 != strchr(gSkippedContentTags, aNode.GetNodeType())) {
-    const nsString& content = aNode.GetSkippedContent();
+    nsCOMPtr<nsIDTD> dtd;
+    mParser->GetDTD(getter_AddRefs(dtd));
+    NS_ENSURE_TRUE(dtd, NS_ERROR_FAILURE);
+    
+    nsString content;
+    PRInt32 lineNo = 0;
+
+    dtd->CollectSkippedContent(aNode.GetNodeType(), content, lineNo);
     if (content.Length() > 0) {
       return PR_TRUE;
     }
@@ -697,7 +719,7 @@ nsLoggingSink::LeafNode(const nsIParserNode& aNode)
 }
 
 nsresult 
-nsLoggingSink::QuoteText(const nsAReadableString& aValue, nsString& aResult) {
+nsLoggingSink::QuoteText(const nsAString& aValue, nsString& aResult) {
   aResult.Truncate();
     /*
       if you're stepping through the string anyway, why not use iterators instead of forcing the string to copy?
@@ -734,7 +756,7 @@ nsLoggingSink::QuoteText(const nsAReadableString& aValue, nsString& aResult) {
  * @param aResult - String coverted to char*.
  */
 nsresult
-nsLoggingSink::GetNewCString(const nsAReadableString& aValue, char** aResult)
+nsLoggingSink::GetNewCString(const nsAString& aValue, char** aResult)
 {
   nsresult result=NS_OK;
   nsAutoString temp;
