@@ -180,15 +180,15 @@ public:
     mIndirectString=0;
   }
   
-  void SetIndirectString(const nsAReadableString& aString) {
+  void SetIndirectString(const nsAString& aString) {
     mIndirectString=&aString;
   }
 
-  virtual const nsAReadableString& GetStringValue(void){
-    return (nsAReadableString&)*mIndirectString;
+  virtual const nsAString& GetStringValue(void){
+    return (const nsAString&)*mIndirectString;
   }
 
-  const nsAReadableString* mIndirectString;
+  const nsAString* mIndirectString;
 };
 
 
@@ -358,14 +358,8 @@ CViewSourceHTML::CViewSourceHTML() : mFilename(), mTags(), mErrors() {
   mLineNumber=0;
   mTokenizer=0;
   mDocType=eHTML3_Quirks; // why?
-  mValidator=0;
   mHasOpenRoot=PR_FALSE;
   mHasOpenBody=PR_FALSE;
-  mInCDATAContainer = PR_FALSE;
-
-  //set this to 1 if you want to see errors in your HTML markup.
-  char* theEnvString = PR_GetEnv("MOZ_VALIDATE_HTML"); 
-  mShowErrors=PRBool(theEnvString != nsnull);
 
   mTokenCount=0;
 
@@ -386,9 +380,6 @@ CViewSourceHTML::CViewSourceHTML() : mFilename(), mTags(), mErrors() {
  */
 CViewSourceHTML::~CViewSourceHTML(){
   mParser=0; //just to prove we destructed...
-
-  NS_IF_RELEASE(mTokenizer);
-
 }
 
 /**
@@ -428,22 +419,22 @@ CViewSourceHTML::CanParse(CParserContext& aParserContext,
   eAutoDetectResult result=eUnknownDetect;
 
   if(eViewSource==aParserContext.mParserCommand) {
-    if(aParserContext.mMimeType.EqualsWithConversion(kPlainTextContentType) ||
-       aParserContext.mMimeType.EqualsWithConversion(kTextCSSContentType) ||
-       aParserContext.mMimeType.EqualsWithConversion(kTextJSContentType) ||
-       aParserContext.mMimeType.EqualsWithConversion(kApplicationJSContentType)) {
+    if(aParserContext.mMimeType.Equals(NS_LITERAL_CSTRING(kPlainTextContentType)) ||
+       aParserContext.mMimeType.Equals(NS_LITERAL_CSTRING(kTextCSSContentType)) ||
+       aParserContext.mMimeType.Equals(NS_LITERAL_CSTRING(kTextJSContentType)) ||
+       aParserContext.mMimeType.Equals(NS_LITERAL_CSTRING(kApplicationJSContentType))) {
       result=eValidDetect;
     }
-    if(aParserContext.mMimeType.EqualsWithConversion(kXMLTextContentType) ||
-       aParserContext.mMimeType.EqualsWithConversion(kXMLApplicationContentType) ||
-       aParserContext.mMimeType.EqualsWithConversion(kXHTMLApplicationContentType) ||
-       aParserContext.mMimeType.EqualsWithConversion(kRDFTextContentType) ||
-       aParserContext.mMimeType.EqualsWithConversion(kHTMLTextContentType) ||
-       aParserContext.mMimeType.EqualsWithConversion(kXULTextContentType) ||
+    if(aParserContext.mMimeType.Equals(NS_LITERAL_CSTRING(kXMLTextContentType)) ||
+       aParserContext.mMimeType.Equals(NS_LITERAL_CSTRING(kXMLApplicationContentType)) ||
+       aParserContext.mMimeType.Equals(NS_LITERAL_CSTRING(kXHTMLApplicationContentType)) ||
+       aParserContext.mMimeType.Equals(NS_LITERAL_CSTRING(kRDFTextContentType)) ||
+       aParserContext.mMimeType.Equals(NS_LITERAL_CSTRING(kHTMLTextContentType)) ||
+       aParserContext.mMimeType.Equals(NS_LITERAL_CSTRING(kXULTextContentType)) ||
 #ifdef MOZ_SVG
-       aParserContext.mMimeType.Equals(NS_LITERAL_STRING(kSVGTextContentType)) ||
+       aParserContext.mMimeType.Equals(NS_LITERAL_CSTRING(kSVGTextContentType)) ||
 #endif
-       aParserContext.mMimeType.EqualsWithConversion(kSGMLTextContentType)) {
+       aParserContext.mMimeType.Equals(kSGMLTextContentType)) {
       result=ePrimaryDetect;
     }
   }
@@ -460,7 +451,9 @@ CViewSourceHTML::CanParse(CParserContext& aParserContext,
   * @param	aSink
   * @return	error code (almost always 0)
   */
-nsresult CViewSourceHTML::WillBuildModel(  const CParserContext& aParserContext,nsIContentSink* aSink){
+nsresult CViewSourceHTML::WillBuildModel(const CParserContext& aParserContext,
+                                         nsITokenizer* aTokenizer,
+                                         nsIContentSink* aSink){
 
   nsresult result=NS_OK;
 
@@ -478,11 +471,11 @@ nsresult CViewSourceHTML::WillBuildModel(  const CParserContext& aParserContext,
     mTags.Truncate();
     mErrors.Assign(NS_LITERAL_STRING(" HTML 4.0 Strict-DTD validation (enabled); [Should use Transitional?].\n"));
 
-    mValidator=aParserContext.mValidator;
     mDocType=aParserContext.mDocType;
     mMimeType=aParserContext.mMimeType;
     mDTDMode=aParserContext.mDTDMode;
     mParserCommand=aParserContext.mParserCommand;
+    mTokenizer = aTokenizer;
     mErrorCount=0;
     mTagCount=0;
 
@@ -542,12 +535,12 @@ NS_IMETHODIMP CViewSourceHTML::BuildModel(nsIParser* aParser,nsITokenizer* aToke
 
       tag.Assign(NS_LITERAL_STRING("HTML"));
       CStartToken htmlToken(tag, eHTMLTag_html);
-      nsCParserNode htmlNode(&htmlToken,0,0/*stack token*/);
+      nsCParserNode htmlNode(&htmlToken, 0/*stack token*/);
       mSink->OpenHTML(htmlNode);
 
       tag.Assign(NS_LITERAL_STRING("HEAD"));
       CStartToken headToken(tag, eHTMLTag_head);
-      nsCParserNode headNode(&headToken,0,0/*stack token*/);
+      nsCParserNode headNode(&headToken, 0/*stack token*/);
       mSink->OpenHead(headNode);
 
       // Note that XUL with automatically add the prefix "Source of: "
@@ -558,7 +551,7 @@ NS_IMETHODIMP CViewSourceHTML::BuildModel(nsIParser* aParser,nsITokenizer* aToke
         CStartToken* theToken=NS_STATIC_CAST(CStartToken*,theAllocator->CreateTokenOfType(eToken_start,eHTMLTag_link,tag));
         if(theToken) {
           CAttributeToken *theAttr;
-          nsCParserNode theNode(theToken,0,theAllocator);
+          nsCParserNode theNode(theToken, theAllocator);
 
           theAttr=(CAttributeToken*)theAllocator->CreateTokenOfType(eToken_attribute,eHTMLTag_unknown,NS_LITERAL_STRING("stylesheet"));
           theAttr->SetKey(NS_LITERAL_STRING("rel"));
@@ -578,7 +571,7 @@ NS_IMETHODIMP CViewSourceHTML::BuildModel(nsIParser* aParser,nsITokenizer* aToke
       }
 
       CEndToken endHeadToken(eHTMLTag_head);
-      nsCParserNode endHeadNode(&endHeadToken,0,0/*stack token*/);
+      nsCParserNode endHeadNode(&endHeadToken, 0/*stack token*/);
       result = mSink->CloseHead(endHeadNode);
       if(NS_SUCCEEDED(result)) {
         mHasOpenRoot = PR_TRUE;
@@ -592,7 +585,7 @@ NS_IMETHODIMP CViewSourceHTML::BuildModel(nsIParser* aParser,nsITokenizer* aToke
         tag.Assign(NS_LITERAL_STRING("BODY"));
         CStartToken* bodyToken=NS_STATIC_CAST(CStartToken*,theAllocator->CreateTokenOfType(eToken_start, eHTMLTag_body, tag));
         if (bodyToken) {
-          nsCParserNode bodyNode(bodyToken,0,theAllocator);
+          nsCParserNode bodyNode(bodyToken, theAllocator);
           CAttributeToken *theAttr=nsnull;
           theAttr=(CAttributeToken*)theAllocator->CreateTokenOfType(eToken_attribute,eHTMLTag_unknown,NS_ConvertASCIItoUCS2(kBodyId));
           theAttr->SetKey(NS_LITERAL_STRING("id"));
@@ -610,7 +603,7 @@ NS_IMETHODIMP CViewSourceHTML::BuildModel(nsIParser* aParser,nsITokenizer* aToke
         
         if (NS_SUCCEEDED(result)) {
           CStartToken theToken(eHTMLTag_pre);
-          nsCParserNode theNode(&theToken,0,0/*stack token*/);
+          nsCParserNode theNode(&theToken, 0/*stack token*/);
           result=mSink->OpenContainer(theNode);
         }
       }
@@ -671,11 +664,11 @@ nsresult  CViewSourceHTML::GenerateSummary() {
  */
 void CViewSourceHTML::StartNewPreBlock(void){
   CEndToken endToken(eHTMLTag_pre);
-  nsCParserNode endNode(&endToken,0,0/*stack token*/);
+  nsCParserNode endNode(&endToken, 0/*stack token*/);
   mSink->CloseContainer(endNode);
 
   CStartToken startToken(eHTMLTag_pre);
-  nsCParserNode startNode(&startToken,0,0/*stack token*/);
+  nsCParserNode startNode(&startToken, 0/*stack token*/);
   mSink->OpenContainer(startNode);
 
 #ifdef DUMP_TO_FILE
@@ -719,15 +712,15 @@ NS_IMETHODIMP CViewSourceHTML::DidBuildModel(nsresult anErrorCode,PRBool aNotify
 
       if(ePlainText!=mDocType) {
         CEndToken theToken(eHTMLTag_pre);
-        nsCParserNode preNode(&theToken,0,0/*stack token*/);
+        nsCParserNode preNode(&theToken, 0/*stack token*/);
         mSink->CloseContainer(preNode);
         
         CEndToken bodyToken(eHTMLTag_body);
-        nsCParserNode bodyNode(&bodyToken,0,0/*stack token*/);
+        nsCParserNode bodyNode(&bodyToken, 0/*stack token*/);
         mSink->CloseBody(bodyNode);
         
         CEndToken htmlToken(eHTMLTag_html);
-        nsCParserNode htmlNode(&htmlToken,0,0/*stack token*/);
+        nsCParserNode htmlNode(&htmlToken, 0/*stack token*/);
         mSink->CloseHTML(htmlNode);
       }
       result = mSink->DidBuildModel(1);
@@ -761,23 +754,16 @@ NS_IMETHODIMP_(void)
 CViewSourceHTML::Terminate() {
 }
 
-
-/**
- * Retrieve the preferred tokenizer for use by this DTD.
- * @update  gess12/28/98
- * @param   none
- * @return  ptr to tokenizer
- */
-nsresult CViewSourceHTML::GetTokenizer(nsITokenizer*& aTokenizer) {
-  nsresult result=NS_OK;
-  if(!mTokenizer) {
-
-    result=NS_NewHTMLTokenizer(&mTokenizer,eDTDMode_quirks,mDocType,mParserCommand);
-  }
-  aTokenizer=mTokenizer;
-  return result;
+NS_IMETHODIMP_(PRInt32)  
+CViewSourceHTML::GetType() {
+  return NS_IPARSER_FLAG_HTML;
 }
 
+NS_IMETHODIMP 
+CViewSourceHTML::CollectSkippedContent(PRInt32 aTag, nsAString& aContent, PRInt32 &aLineNo)
+{
+  return NS_OK;
+}
 
 /**
  * 
@@ -837,7 +823,7 @@ PRBool CViewSourceHTML::CanContain(PRInt32 aParent,PRInt32 aChild) const{
  * become useful.
  */
 NS_IMETHODIMP
-CViewSourceHTML::StringTagToIntTag(const nsAReadableString &aTag,
+CViewSourceHTML::StringTagToIntTag(const nsAString &aTag,
                                    PRInt32* aIntTag) const
 {
   *aIntTag = nsHTMLTags::LookupTag(aTag);
@@ -857,7 +843,7 @@ CViewSourceHTML::IntTagToStringTag(PRInt32 aIntTag) const
 }
 
 NS_IMETHODIMP
-CViewSourceHTML::ConvertEntityToUnicode(const nsAReadableString& aEntity,
+CViewSourceHTML::ConvertEntityToUnicode(const nsAString& aEntity,
                                         PRInt32* aUnicode) const
 {
   *aUnicode = nsHTMLEntities::EntityToUnicode(aEntity);
@@ -913,7 +899,7 @@ nsresult CViewSourceHTML::WriteAttributes(PRInt32 attrCount) {
           theContext.mTokenNode.AddAttribute(theToken);  //and add it to the node.
 
           CAttributeToken* theAttrToken=(CAttributeToken*)theToken;
-          const nsAReadableString& theKey=theAttrToken->GetKey();
+          const nsAString& theKey=theAttrToken->GetKey();
 
           result=WriteTag(mKey,theKey,0,PR_FALSE);
           const nsAString& theValue=theAttrToken->GetValue();
@@ -937,7 +923,7 @@ nsresult CViewSourceHTML::WriteAttributes(PRInt32 attrCount) {
  *  @param   
  *  @return  result status
  */
-nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsAReadableString & aText,PRInt32 attrCount,PRBool aNewlineRequired) {
+nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsAString & aText,PRInt32 attrCount,PRBool aNewlineRequired) {
   static nsString       theString;
 
   nsresult result=NS_OK;
@@ -953,7 +939,7 @@ nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsAReadableString & aT
     nsAutoString beforeText;
     beforeText.AssignWithConversion(kBeforeText[aTagType]);
     theContext.mITextToken.SetIndirectString(beforeText);
-    nsCParserNode theNode(&theContext.mITextToken,0,0/*stack token*/);
+    nsCParserNode theNode(&theContext.mITextToken, 0/*stack token*/);
     mSink->AddLeaf(theNode);
   }
 #ifdef DUMP_TO_FILE
@@ -964,7 +950,7 @@ nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsAReadableString & aT
   if (mSyntaxHighlight && aTagType != mText) {
     CStartToken* theTagToken=NS_STATIC_CAST(CStartToken*,theAllocator->CreateTokenOfType(eToken_start,eHTMLTag_span,NS_LITERAL_STRING("SPAN")));
 
-    theContext.mStartNode.Init(theTagToken,mLineNumber,theAllocator);
+    theContext.mStartNode.Init(theTagToken, theAllocator);
     CAttributeToken* theAttr=(CAttributeToken*)theAllocator->CreateTokenOfType(eToken_attribute,eHTMLTag_unknown,NS_ConvertASCIItoUCS2(kElementClasses[aTagType]));
     theAttr->SetKey(NS_LITERAL_STRING("class"));
     theContext.mStartNode.AddAttribute(theAttr);
@@ -982,7 +968,7 @@ nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsAReadableString & aT
 
   theContext.mITextToken.SetIndirectString(aText);  //now emit the tag name...
 
-  nsCParserNode theNode(&theContext.mITextToken,0,0/*stack token*/);
+  nsCParserNode theNode(&theContext.mITextToken, 0/*stack token*/);
   mSink->AddLeaf(theNode);
 #ifdef DUMP_TO_FILE
   if (gDumpFile) {
@@ -995,7 +981,7 @@ nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsAReadableString & aT
   if (mSyntaxHighlight && aTagType != mText) {
     theContext.mStartNode.ReleaseAll(); 
     CEndToken theEndToken(eHTMLTag_span);
-    theContext.mEndNode.Init(&theEndToken,mLineNumber,0/*stack token*/);
+    theContext.mEndNode.Init(&theEndToken, 0/*stack token*/);
     mSink->CloseContainer(theContext.mEndNode);  //emit </starttag>...
 #ifdef DUMP_TO_FILE
     if (gDumpFile)
@@ -1011,7 +997,7 @@ nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsAReadableString & aT
     nsAutoString afterText;
     afterText.AssignWithConversion(kAfterText[aTagType]);
     theContext.mITextToken.SetIndirectString(afterText);
-    nsCParserNode theNode(&theContext.mITextToken,0,0/*stack token*/);
+    nsCParserNode theNode(&theContext.mITextToken, 0/*stack token*/);
     mSink->AddLeaf(theNode);
   }
 #ifdef DUMP_TO_FILE
@@ -1025,67 +1011,6 @@ nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsAReadableString & aT
   return result;
 }
 
-/**
- *  This method gets called when a tag needs to be sent out but is known to be misplaced (error)
- *  
- *  @update  gess 6June2000 -- 
- *  @param   
- *  @return  result status
- */
-nsresult CViewSourceHTML::WriteTagWithError(PRInt32 aTagType,const nsAReadableString& aStr,PRInt32 attrCount,PRBool aNewlineRequired) {
-
-  STOP_TIMER();
-
-  CSharedVSContext& theContext=CSharedVSContext::GetSharedContext();
-  nsresult result=NS_OK;
-  
-  if(ePlainText!=mDocType) {
-    //first write the error tag itself...
-    theContext.mErrorNode.Init(&theContext.mErrorToken,mLineNumber,0/*stack token*/);
-    result=mSink->OpenContainer(theContext.mErrorNode);  //emit <error>...
-  }
-
-  //now write the tag from the source file...
-  result=WriteTag(aTagType,aStr,attrCount,aNewlineRequired);
-
-  if(ePlainText!=mDocType) {
-    //now close the error tag...
-    STOP_TIMER();
-    theContext.mErrorNode.Init(&theContext.mErrorToken,0,0/*stack token*/);
-    mSink->CloseContainer(theContext.mErrorNode);
-    START_TIMER();
-  }
-
-  return result; 
-}
-
-void CViewSourceHTML::AddContainmentError(eHTMLTags aChildTag,eHTMLTags aParentTag,PRInt32 aLineNumber) {
-
-  if (mShowErrors) {
-    mErrorCount++;
-
-    if(mErrorCount<=gErrorThreshold) {
-
-      char theChildMsg[100];
-      if(eHTMLTag_text==aChildTag) 
-        strcpy(theChildMsg,"text");
-      else sprintf(theChildMsg,"<%s>",
-                   NS_ConvertUCS2toUTF8(nsHTMLTags::GetStringValue(aChildTag)).get());
-
-      char theMsg[256];
-      sprintf(theMsg,"\n -- Line (%i) error: %s is not a legal child of <%s>",
-              aLineNumber, theChildMsg,
-              NS_ConvertUCS2toUTF8(nsHTMLTags::GetStringValue(aParentTag)).get());
-
-      mErrors.AppendWithConversion(theMsg);
-    }
-    else if(gErrorThreshold+1==mErrorCount){
-      mErrors.Append(NS_LITERAL_STRING("\n -- Too many errors -- terminating output."));
-    }
-  }
-
-}
-
 /**
  *  
  *  @update  gess 3/25/98
@@ -1101,7 +1026,7 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
   mSink=(nsIHTMLContentSink*)aParser->GetContentSink();
  
   CSharedVSContext& theContext=CSharedVSContext::GetSharedContext();
-  theContext.mTokenNode.Init(theToken,mLineNumber,mTokenizer->GetTokenAllocator());
+  theContext.mTokenNode.Init(theToken, mTokenizer->GetTokenAllocator());
 
   eHTMLTags theParent=(mTags.Length()) ? (eHTMLTags)mTags.Last() : eHTMLTag_unknown;
   eHTMLTags theChild=(eHTMLTags)aToken->GetTypeID();
@@ -1112,29 +1037,8 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
       {
         mTagCount++;
 
-        if (gHTMLElements[theChild].CanContainType(kCDATA)) {
-          mInCDATAContainer = PR_TRUE;
-        }
-
-        const nsAReadableString& startValue = aToken->GetStringValue();
-        if(mShowErrors) {
-          PRBool theChildIsValid=PR_TRUE;
-          if(mValidator) {
-            theChildIsValid=mValidator->CanContain(theParent,theChild);
-            if(theChildIsValid) {
-              if(mValidator->IsContainer(theChild))
-                mTags.Append(PRUnichar(theChild));
-            }
-          }
-          
-          if(theChildIsValid)
-            result=WriteTag(mStartTag,startValue,aToken->GetAttributeCount(),PR_TRUE);
-          else {
-            AddContainmentError(theChild,theParent,mLineNumber);
-            result=WriteTagWithError(mStartTag,startValue,aToken->GetAttributeCount(),PR_TRUE);
-          }
-        }
-        else result=WriteTag(mStartTag,startValue,aToken->GetAttributeCount(),PR_TRUE);
+        const nsAString& startValue = aToken->GetStringValue();
+        result=WriteTag(mStartTag,startValue,aToken->GetAttributeCount(),PR_TRUE);
 
         if((ePlainText!=mDocType) && mParser && (NS_OK==result)) {
           result = mSink->NotifyTagObservers(&theContext.mTokenNode);
@@ -1144,15 +1048,12 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
 
     case eToken_end:
       {
-        if (gHTMLElements[theChild].CanContainType(kCDATA)) {
-          mInCDATAContainer = PR_FALSE;
-        }
-
         if(theParent==theChild) {
           mTags.Truncate(mTags.Length()-1);
         }
-        const nsAReadableString& endValue = aToken->GetStringValue();
-        result=WriteTag(mEndTag,endValue,0,PR_TRUE);
+
+        const nsAString& endValue = aToken->GetStringValue();
+        result=WriteTag(mEndTag,endValue,aToken->GetAttributeCount(),PR_TRUE);
       }
       break;
 
@@ -1178,21 +1079,22 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
 
     case eToken_comment: 
       {
-        const nsAReadableString& commentValue = aToken->GetStringValue();
-        result=WriteTag(mCommentTag,commentValue,0,PR_TRUE);
+        nsAutoString theStr;
+        aToken->AppendSourceTo(theStr);
+        result=WriteTag(mCommentTag,theStr,0,PR_TRUE);
       }
       break;
 
     case eToken_doctypeDecl:
       {
-        const nsAReadableString& doctypeValue = aToken->GetStringValue();
+        const nsAString& doctypeValue = aToken->GetStringValue();
         result=WriteTag(mDocTypeTag,doctypeValue,0,PR_TRUE);
       }
       break;
 
     case eToken_newline:
       {
-        const nsAReadableString& newlineValue = aToken->GetStringValue();
+        const nsAString& newlineValue = aToken->GetStringValue();
         mLineNumber++; 
         result=WriteTag(mText,newlineValue,0,PR_FALSE);
         mTokenCount++;
@@ -1204,7 +1106,7 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
 
     case eToken_whitespace:
       {
-        const nsAReadableString& wsValue = aToken->GetStringValue();
+        const nsAString& wsValue = aToken->GetStringValue();
         result=WriteTag(mText,wsValue,0,PR_FALSE);
         mTokenCount++;
         if (NS_VIEWSOURCE_TOKENS_PER_BLOCK > 0 &&
@@ -1219,34 +1121,14 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
 
     case eToken_text:
       {
-        if(mShowErrors) {
-          const nsAReadableString& str = aToken->GetStringValue();
-          if((0==mValidator) || 
-             mValidator->CanContain(theParent,eHTMLTag_text))
-            result=WriteTag(mText,str,aToken->GetAttributeCount(),PR_TRUE);
-          else {
-            AddContainmentError(eHTMLTag_text,theParent,mLineNumber);
-            result=WriteTagWithError(mText,str,aToken->GetAttributeCount(),PR_FALSE);
-          }
-        }
-        else if (mInCDATAContainer) {
-          // Fix bug 40809
-          nsAutoString theStr;
-          aToken->GetSource(theStr);
-          theStr.ReplaceSubstring(NS_LITERAL_STRING("\r\n").get(), NS_LITERAL_STRING("\n").get());
-          theStr.ReplaceChar(kCR,kLF);  
-          result=WriteTag(mText,theStr,aToken->GetAttributeCount(),PR_TRUE);
-        }
-        else {
-          const nsAReadableString& str = aToken->GetStringValue();         
-          result=WriteTag(mText,str,aToken->GetAttributeCount(),PR_TRUE);
-          mTokenCount++;
-          if (NS_VIEWSOURCE_TOKENS_PER_BLOCK > 0 &&
-              mTokenCount > NS_VIEWSOURCE_TOKENS_PER_BLOCK && !str.IsEmpty()) {
-            PRUnichar ch = str.Last();
-            if (ch == kLF || ch == kCR)
-              StartNewPreBlock();
-          }
+        const nsAString& str = aToken->GetStringValue();         
+        result=WriteTag(mText,str,aToken->GetAttributeCount(),PR_TRUE);
+        mTokenCount++;
+        if (NS_VIEWSOURCE_TOKENS_PER_BLOCK > 0 &&
+            mTokenCount > NS_VIEWSOURCE_TOKENS_PER_BLOCK && !str.IsEmpty()) {
+          PRUnichar ch = str.Last();
+          if (ch == kLF || ch == kCR)
+            StartNewPreBlock();
         }
       }
 
