@@ -6,7 +6,7 @@
  * the License at http://www.mozilla.org/NPL/
  *
  * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
@@ -543,12 +543,26 @@ array_nyi(JSContext *cx, const char *what)
 }
 #endif
 
+static JSBool
+InitArrayElements(JSContext *cx, JSObject *obj, jsuint length, jsval *vector)
+{
+    jsuint index;
+    jsid id;
+
+    for (index = 0; index < length; index++) {
+	if (!IndexToId(cx, index, &id))
+	    return JS_FALSE;
+	if (!OBJ_SET_PROPERTY(cx, obj, id, &vector[index]))
+	    return JS_FALSE;
+    }
+    return JS_TRUE;
+}
+
 static JSBool
 InitArrayObject(JSContext *cx, JSObject *obj, jsuint length, jsval *vector)
 {
     jsval v;
     jsid id;
-    jsuint index;
 
     if (!IndexToValue(cx, length, &v))
 	return JS_FALSE;
@@ -561,17 +575,7 @@ InitArrayObject(JSContext *cx, JSObject *obj, jsuint length, jsval *vector)
     }
     if (!vector)
 	return JS_TRUE;
-    for (index = 0; index < length; index++) {
-	if (!IndexToId(cx, index, &id))
-	    return JS_FALSE;
-	if (!OBJ_DEFINE_PROPERTY(cx, obj, id, vector[index],
-                                 JS_PropertyStub, JS_PropertyStub,
-                                 JSPROP_ENUMERATE,
-                                 NULL)) {
-	    return JS_FALSE;
-	}
-    }
-    return JS_TRUE;
+    return InitArrayElements(cx, obj, length, vector);
 }
 
 static JSBool
@@ -618,29 +622,30 @@ array_reverse(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
     return JS_TRUE;
 }
 
-typedef struct QSortArgs {
+typedef struct HSortArgs {
     void         *vec;
     size_t       elsize;
     void         *pivot;
     JSComparator cmp;
     void         *arg;
-} QSortArgs;
+} HSortArgs;
 
 static int
 sort_compare(const void *a, const void *b, void *arg);
 
 static void
-js_qsort_r(QSortArgs *qa, int lo, int hi)
+HeapSortHelper(HSortArgs *qa, int lo, int hi)
 {
-    void *pivot, *vec, *arg, *a, *b;
+    void *pivot, *vec, *vec2, *arg, *a, *b;
     size_t elsize;
     JSComparator cmp;
     JSBool fastmove;
-    int i, j, lohi, hilo;
+    int j, hiDiv2;
 
     pivot = qa->pivot;
     vec = qa->vec;
     elsize = qa->elsize;
+    vec2 =  (char *)vec - 2 * elsize;
     cmp = qa->cmp;
     arg = qa->arg;
 
@@ -648,51 +653,51 @@ js_qsort_r(QSortArgs *qa, int lo, int hi)
 #define MEMMOVE(p,q,n) \
     (fastmove ? (void)(*(jsval*)(p) = *(jsval*)(q)) : (void)memmove(p, q, n))
 
-    while (lo < hi) {
-        i = lo;
-        j = hi;
-        a = (char *)vec + i * elsize;
+    if (lo == 1) {
+        j = 2;
+        b = (char *)vec + elsize;
+        if (j < hi && cmp((char *)vec, b, arg) < 0)
+            j++;
+        a = (char *)vec + (hi - 1) * elsize;
+        b = (char *)vec2 + j * elsize;
+        if (cmp(a, b, arg) >= 0)
+            return;
+
+        MEMMOVE(pivot, a, elsize);
+        MEMMOVE(a, b, elsize);
+        lo = j;
+    } else {
+        a = (char *)vec2 + lo * elsize;
         MEMMOVE(pivot, a, elsize);
-        while (i < j) {
-            b = (char *)vec + j * elsize;
-            if (cmp(b, pivot, arg) >= 0) {
-                j--;
-                continue;
-            }
-            MEMMOVE(a, b, elsize);
-            while (cmp(a, pivot, arg) <= 0) {
-                i++;
-                a = (char *)vec + i * elsize;
-                if (i == j)
-                    goto store_pivot;
-            }
-            MEMMOVE(b, a, elsize);
-        }
-        if (i > lo) {
-      store_pivot:
-            MEMMOVE(a, pivot, elsize);
-        }
-        if (i - lo < hi - i) {
-            lohi = i - 1;
-            if (lo < lohi)
-                js_qsort_r(qa, lo, lohi);
-            lo = i + 1;
-        } else {
-            hilo = i + 1;
-            if (hilo < hi)
-                js_qsort_r(qa, hilo, hi);
-            hi = i - 1;
-        }
     }
 
+    hiDiv2 = hi/2;
+    while (lo <= hiDiv2) {
+        j = lo + lo;
+        a = (char *)vec2 + j * elsize;
+        b = (char *)vec + (j - 1) * elsize;
+        if (j < hi && cmp(a, b, arg) < 0)
+            j++;
+        b = (char *)vec2 + j * elsize;
+        if (cmp(pivot, b, arg) >= 0)
+            break;
+
+        a = (char *)vec2 + lo * elsize;
+        MEMMOVE(a, b, elsize);
+        lo = j;
+    }
+
+    a = (char *)vec2 + lo * elsize;
+    MEMMOVE(a, pivot, elsize);
 #undef MEMMOVE
 }
 
 JSBool
-js_qsort(void *vec, size_t nel, size_t elsize, JSComparator cmp, void *arg)
+js_HeapSort(void *vec, size_t nel, size_t elsize, JSComparator cmp, void *arg) 
 {
     void *pivot;
-    QSortArgs qa;
+    HSortArgs qa;
+    int i;
 
     pivot = malloc(elsize);
     if (!pivot)
@@ -702,7 +707,12 @@ js_qsort(void *vec, size_t nel, size_t elsize, JSComparator cmp, void *arg)
     qa.pivot = pivot;
     qa.cmp = cmp;
     qa.arg = arg;
-    js_qsort_r(&qa, 0, (int)(nel - 1));
+
+    for (i = nel/2; i > 0; i--)
+        HeapSortHelper(&qa, i, (int) nel);
+    while (nel > 2)
+        HeapSortHelper(&qa, 1, (int) --nel);
+
     free(pivot);
     return JS_TRUE;
 }
@@ -829,13 +839,13 @@ array_sort(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
     ca.context = cx;
     ca.fval = fval;
     ca.status = JS_TRUE;
-    if (!js_qsort(vec, (size_t) len, sizeof(jsval), sort_compare, &ca)) {
+    if (!js_HeapSort(vec, (size_t) len, sizeof(jsval), sort_compare, &ca)) {
 	JS_ReportOutOfMemory(cx);
 	ca.status = JS_FALSE;
     }
 
     if (ca.status) {
-	ca.status = InitArrayObject(cx, obj, newlen, vec);
+	ca.status = InitArrayElements(cx, obj, newlen, vec);
 	if (ca.status)
 	    *rval = OBJECT_TO_JSVAL(obj);
 #if JS_HAS_SPARSE_ARRAYS
