@@ -6,7 +6,7 @@
  * the License at http://www.mozilla.org/NPL/
  *
  * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
@@ -74,7 +74,7 @@ js_NewContext(JSRuntime *rt, size_t stackChunkSize)
     js_InitContextForLocking(cx);
 #endif
 
-    JS_LOCK_RUNTIME(rt);
+    JS_LOCK_GC(rt);
     for (;;) {
         first = (rt->contextList.next == &rt->contextList);
         if (rt->state == JSRTS_UP) {
@@ -89,7 +89,7 @@ js_NewContext(JSRuntime *rt, size_t stackChunkSize)
         JS_WAIT_CONDVAR(rt->stateChange, JS_NO_TIMEOUT);
     }
     JS_APPEND_LINK(&cx->links, &rt->contextList);
-    JS_UNLOCK_RUNTIME(rt);
+    JS_UNLOCK_GC(rt);
 
     /*
      * First we do the infallible, every-time per-context initializations.
@@ -139,10 +139,10 @@ js_NewContext(JSRuntime *rt, size_t stackChunkSize)
             return NULL;
         }
 
-        JS_LOCK_RUNTIME(rt);
+        JS_LOCK_GC(rt);
         rt->state = JSRTS_UP;
         JS_NOTIFY_ALL_CONDVAR(rt->stateChange);
-        JS_UNLOCK_RUNTIME(rt);
+        JS_UNLOCK_GC(rt);
     }
 
     return cx;
@@ -158,15 +158,30 @@ js_DestroyContext(JSContext *cx, JSGCMode gcmode)
     rt = cx->runtime;
 
     /* Remove cx from context list first. */
-    JS_LOCK_RUNTIME(rt);
+    JS_LOCK_GC(rt);
     JS_ASSERT(rt->state == JSRTS_UP || rt->state == JSRTS_LAUNCHING);
     JS_REMOVE_LINK(&cx->links);
     last = (rt->contextList.next == &rt->contextList);
     if (last)
         rt->state = JSRTS_LANDING;
-    JS_UNLOCK_RUNTIME(rt);
+    JS_UNLOCK_GC(rt);
 
     if (last) {
+#ifdef JS_THREADSAFE
+        /*
+         * If cx is not in a request already, begin one now so that we wait
+         * for any racing GC started on a not-last context to finish, before
+         * we plow ahead and unpin atoms.  Note that even though we begin a
+         * request here if necessary, we end all requests on cx below before
+         * forcing a final GC.  This lets any not-last context destruction
+         * racing in another thread try to force or maybe run the GC, but by
+         * that point, rt->state will not be JSRTS_UP, and that GC attempt
+         * will return early.
+         */
+        if (cx->requestDepth == 0)
+            JS_BeginRequest(cx);
+#endif
+
         /* Unpin all pinned atoms before final GC. */
         js_UnpinPinnedAtoms(&rt->atomState);
 
@@ -207,7 +222,7 @@ js_DestroyContext(JSContext *cx, JSGCMode gcmode)
 
     if (last) {
         /* Always force, so we wait for any racing GC to finish. */
-        js_ForceGC(cx);
+        js_ForceGC(cx, GC_LAST_CONTEXT);
 
         /* Iterate until no finalizer removes a GC root or lock. */
         while (rt->gcPoke)
@@ -218,13 +233,13 @@ js_DestroyContext(JSContext *cx, JSGCMode gcmode)
             js_FreeAtomState(cx, &rt->atomState);
 
         /* Take the runtime down, now that it has no contexts or atoms. */
-        JS_LOCK_RUNTIME(rt);
+        JS_LOCK_GC(rt);
         rt->state = JSRTS_DOWN;
         JS_NOTIFY_ALL_CONDVAR(rt->stateChange);
-        JS_UNLOCK_RUNTIME(rt);
+        JS_UNLOCK_GC(rt);
     } else {
         if (gcmode == JS_FORCE_GC)
-            js_ForceGC(cx);
+            js_ForceGC(cx, 0);
         else if (gcmode == JS_MAYBE_GC)
             JS_MaybeGC(cx);
     }
@@ -256,7 +271,7 @@ js_DestroyContext(JSContext *cx, JSGCMode gcmode)
 }
 
 JSBool
-js_LiveContext(JSRuntime *rt, JSContext *cx)
+js_ValidContextPointer(JSRuntime *rt, JSContext *cx)
 {
     JSCList *cl;
 
@@ -269,18 +284,20 @@ js_LiveContext(JSRuntime *rt, JSContext *cx)
 }
 
 JSContext *
-js_ContextIterator(JSRuntime *rt, JSContext **iterp)
+js_ContextIterator(JSRuntime *rt, JSBool unlocked, JSContext **iterp)
 {
     JSContext *cx = *iterp;
 
-    JS_LOCK_RUNTIME(rt);
+    if (unlocked)
+        JS_LOCK_GC(rt);
     if (!cx)
         cx = (JSContext *)&rt->contextList;
     cx = (JSContext *)cx->links.next;
     if (&cx->links == &rt->contextList)
         cx = NULL;
     *iterp = cx;
-    JS_UNLOCK_RUNTIME(rt);
+    if (unlocked)
+        JS_UNLOCK_GC(rt);
     return cx;
 }
 
@@ -378,12 +395,12 @@ js_ReportErrorVA(JSContext *cx, uintN flags, const char *format, va_list ap)
     char *last;
     JSBool warning;
 
-    fp = cx->fp;
+    if ((flags & JSREPORT_STRICT) && !JS_HAS_STRICT_OPTION(cx))
+        return JS_TRUE;
 
-    /* Walk stack until we find a frame that is associated with
-       some script rather than a native frame. */
-    while (fp && (!fp->script || !fp->pc))
-        fp = fp->down;
+    /* Find the top-most active script frame, for best line number blame. */
+    for (fp = cx->fp; fp && (!fp->script || !fp->pc); fp = fp->down)
+        continue;
 
     reportp = &report;
     memset(reportp, 0, sizeof (struct JSErrorReport));
@@ -578,6 +595,9 @@ js_ReportErrorNumberVA(JSContext *cx, uintN flags, JSErrorCallback callback,
     char *message;
     JSBool warning;
 
+    if ((flags & JSREPORT_STRICT) && !JS_HAS_STRICT_OPTION(cx))
+        return JS_TRUE;
+
     report.messageArgs = NULL;
     report.ucmessage = NULL;
     message = NULL;
