@@ -6,7 +6,7 @@
  * the License at http://www.mozilla.org/NPL/
  *
  * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
@@ -1926,6 +1926,11 @@ JS_dtoa(double d, int mode, JSBool biasUp, int ndigits,
         }
         return JS_TRUE;
     }
+    
+    b = NULL;                           /* initialize for abort protection */
+    S = NULL;
+    mlo = mhi = NULL;
+    
     if (!d) {
       no_digits:
         *decpt = 1;
@@ -1937,6 +1942,14 @@ JS_dtoa(double d, int mode, JSBool biasUp, int ndigits,
         buf[0] = '0'; buf[1] = '\0';  /* copy "0" to buffer */
         if (rve)
             *rve = buf + 1;
+        /* We might have jumped to "no_digits" from below, so we need
+         * to be sure to free the potentially allocated Bigints to avoid
+         * memory leaks. */
+        Bfree(b);
+        Bfree(S);
+        if (mlo != mhi)
+            Bfree(mlo);
+        Bfree(mhi);
         return JS_TRUE;
     }
 
@@ -2216,7 +2229,6 @@ JS_dtoa(double d, int mode, JSBool biasUp, int ndigits,
 
     m2 = b2;
     m5 = b5;
-    mhi = mlo = 0;
     if (leftright) {
         if (mode < 2) {
             i =
