@@ -552,8 +552,9 @@ nsContainerFrame::SyncFrameViewAfterReflow(nsIPresContext* aPresContext,
     // See if the view should be hidden or visible
     PRBool  viewIsVisible = PR_TRUE;
     PRBool  viewHasTransparentContent =
-        (!hasBG ||
-         (bg->mBackgroundFlags & NS_STYLE_BG_COLOR_TRANSPARENT));
+         (!hasBG ||
+         (bg->mBackgroundFlags & NS_STYLE_BG_COLOR_TRANSPARENT) ||
+         !aFrame->CanPaintBackground());
     if (isCanvas && viewHasTransparentContent) {
       nsIView* rootView;
       vm->GetRootView(rootView);
@@ -576,14 +577,15 @@ nsContainerFrame::SyncFrameViewAfterReflow(nsIPresContext* aPresContext,
         NS_RELEASE(widget);
       }
       else {
-        // If it's a scroll frame, then hide the view. This means that
+        // If it's a scroll frame or a list control frame which is derived from the scrollframe, 
+        // then hide the view. This means that
         // child elements can't override their parent's visibility, but
         // it's not practical to leave it visible in all cases because
         // the scrollbars will be showing
         nsIAtom*  frameType;
         aFrame->GetFrameType(&frameType);
 
-        if (frameType == nsLayoutAtoms::scrollFrame) {
+        if (frameType == nsLayoutAtoms::scrollFrame || frameType == nsLayoutAtoms::listControlFrame) {
           viewIsVisible = PR_FALSE;
 
         } else {
