@@ -114,6 +114,12 @@
 #define NS_ICON_LOADING_IMAGE (0)
 #define NS_ICON_BROKEN_IMAGE  (1)
 
+// sizes (pixels) for image icon, padding and border frame
+#define ICON_SIZE        (16)
+#define ICON_PADDING     (3)
+#define ALT_BORDER_WIDTH (1)
+
+
 // Default alignment value (so we can tell an unset value from a set value)
 #define ALIGN_UNSET PRUint8(-1)
 
@@ -653,7 +659,7 @@ NS_IMETHODIMP nsImageFrame::OnStopDecode(imgIRequest *aRequest, nsIPresContext *
       mContent->GetAttr(kNameSpaceID_HTML, nsHTMLAtoms::usemap, usemap);    
       if (usemap.IsEmpty()) {
         // check if we want to honor the ALT text in the IMG frame, or let the preShell make it into inline text
-        //  - if QuirksMode, and the IMG has a size, then render the ALT text in the ING frame
+        //  - if QuirksMode, and the IMG has a size, then render the ALT text in the IMG frame
         //    UNLESS there is a pref set to force inline alt text
         PRBool useSizedBox = PR_FALSE;
         PRBool prefForceInlineAltText = mIconLoad ? mIconLoad->mPrefForceInlineAltText : PR_FALSE;
@@ -667,11 +673,24 @@ NS_IMETHODIMP nsImageFrame::OnStopDecode(imgIRequest *aRequest, nsIPresContext *
         nsCompatibility mode;
         aPresContext->GetCompatibilityMode(&mode);
         
-        // wrap it all up
-        useSizedBox = !prefForceInlineAltText &&
-                      HaveFixedSize(*stylePosition) && 
-                      mode == eCompatibility_NavQuirks &&
-                      !mImageBlocked;
+        // check for being in Composer: if we are, we always show the size-box
+        PRBool forceIcon = PR_FALSE;
+
+        // check for style property that indicates the icon should always be shown
+        const nsStyleUIReset* styleData;
+        GetStyleData(eStyleStruct_UIReset, (const nsStyleStruct*&) styleData);
+        if (styleData->mForceBrokenImageIcon) {
+          forceIcon = PR_TRUE;
+        }
+
+        // wrap it all up: use the size box if
+        // - in editor
+        // - not forcing inline alt text, have fixed size, in quirks mode, image not blocked
+        useSizedBox = forceIcon ||
+                      (!prefForceInlineAltText &&
+                       HaveFixedSize(*stylePosition) && 
+                       mode == eCompatibility_NavQuirks &&
+                       !mImageBlocked);
 
         if (!useSizedBox) {
           // let the presShell handle converting this into the inline alt text frame
@@ -745,18 +764,39 @@ nsImageFrame::GetDesiredSize(nsIPresContext* aPresContext,
   // check to see if the size is already known by the image container.
   if (mLoads[0].mIntrinsicSize.width == 0 &&
       mLoads[0].mIntrinsicSize.height == 0) {
+    PRBool needMinIconSizing = PR_FALSE;
+    float p2t;
+    aPresContext->GetPixelsToTwips(&p2t);
     if (mLoads[0].mRequest) {
       nsCOMPtr<imgIContainer> con;
       mLoads[0].mRequest->GetImage(getter_AddRefs(con));
       if (con) {
-        float p2t;
         nscoord width, height;
 
-        aPresContext->GetPixelsToTwips(&p2t);
         con->GetWidth(&width);
         con->GetHeight(&height);
         mLoads[0].mIntrinsicSize.width = NSIntPixelsToTwips(width, p2t);
         mLoads[0].mIntrinsicSize.height = NSIntPixelsToTwips(height, p2t);
+      } else {
+        // no image container, so we may need to synthesize a minimum size for the image icons
+        needMinIconSizing = PR_TRUE;
+      }
+    } else {
+      // no request, so we may need to synthesize a minimum size for the image icons
+      needMinIconSizing = PR_TRUE;
+    }
+
+    if (needMinIconSizing) {
+      // XXX:  we need this in composer, but it is also good for general quirks mode to always
+      // XXX:  have room for the icon
+      // check for quirks mode
+      nsCompatibility mode;
+      aPresContext->GetCompatibilityMode(&mode);
+      if (mode == eCompatibility_NavQuirks) {
+        // image request is null or image size not known, probably an invalid image specified
+        // - make the image big enough for the icon (it may not be used if inline alt expansion is used instead)
+        mLoads[0].mIntrinsicSize.width = NSIntPixelsToTwips(ICON_SIZE+(2*(ICON_PADDING+ALT_BORDER_WIDTH)), p2t);
+        mLoads[0].mIntrinsicSize.height = NSIntPixelsToTwips(ICON_SIZE+(2*(ICON_PADDING+ALT_BORDER_WIDTH)), p2t);
       }
     }
   }
@@ -1088,10 +1128,6 @@ struct nsRecessedBorder : public nsStyleBorder {
   }
 };
 
-#define ICON_SIZE        (16)
-#define ICON_PADDING     (6)
-#define ALT_BORDER_WIDTH (1)
-
 void
 nsImageFrame::DisplayAltFeedback(nsIPresContext*      aPresContext,
                                  nsIRenderingContext& aRenderingContext,
@@ -1107,6 +1143,12 @@ nsImageFrame::DisplayAltFeedback(nsIPresContext*      aPresContext,
   aPresContext->GetScaledPixelsToTwips(&p2t);
   borderEdgeWidth = NSIntPixelsToTwips(ALT_BORDER_WIDTH, p2t);
 
+  // if inner area is empty, then make it big enough for at least the icon
+  if (inner.IsEmpty()){
+    inner.SizeBy(2*(NSIntPixelsToTwips(ICON_SIZE+ICON_PADDING+ALT_BORDER_WIDTH,p2t)),
+                 2*(NSIntPixelsToTwips(ICON_SIZE+ICON_PADDING+ALT_BORDER_WIDTH,p2t)));
+  }
+
   // Make sure we have enough room to actually render the border within
   // our frame bounds
   if ((inner.width < 2 * borderEdgeWidth) || (inner.height < 2 * borderEdgeWidth)) {
@@ -1142,46 +1184,51 @@ nsImageFrame::DisplayAltFeedback(nsIPresContext*      aPresContext,
   aRenderingContext.PushState();
   aRenderingContext.SetClipRect(inner, nsClipCombine_kIntersect, clipState);
 
-  PRInt32 size = NSIntPixelsToTwips(ICON_SIZE, p2t);
+  PRBool dispIcon = mIconLoad ? mIconLoad->mPrefShowPlaceholders : PR_TRUE;
 
-  PRBool iconUsed = PR_FALSE;
+  // Check if we should display image placeholders
+  if (dispIcon) {
+    PRInt32 size = NSIntPixelsToTwips(ICON_SIZE, p2t);
 
-  // see if the icon images are present...
-  if (mIconLoad && mIconLoad->mIconsLoaded) {
-    // pick the correct image
-    NS_ASSERTION( aIconId == NS_ICON_LOADING_IMAGE ||
-                  aIconId == NS_ICON_BROKEN_IMAGE, "Invalid Icon ID in DisplayAltFeedback");
+    PRBool iconUsed = PR_FALSE;
 
-    nsCOMPtr<imgIContainer> imgCon;
-    if (mIconLoad->mIconLoads[aIconId].mRequest) {
-      mIconLoad->mIconLoads[aIconId].mRequest->GetImage(getter_AddRefs(imgCon));
-    }
-    if (imgCon) { 
-      // draw it
-      nsPoint p(inner.x, inner.y);
-      nsRect r(0,0,size,size);
-      aRenderingContext.DrawImage(imgCon, &r, &p);
-      iconUsed = PR_TRUE;
+    // see if the icon images are present...
+    if (mIconLoad && mIconLoad->mIconsLoaded) {
+      // pick the correct image
+      NS_ASSERTION( aIconId == NS_ICON_LOADING_IMAGE ||
+                    aIconId == NS_ICON_BROKEN_IMAGE, "Invalid Icon ID in DisplayAltFeedback");
+
+      nsCOMPtr<imgIContainer> imgCon;
+      if (mIconLoad->mIconLoads[aIconId].mRequest) {
+        mIconLoad->mIconLoads[aIconId].mRequest->GetImage(getter_AddRefs(imgCon));
+      }
+      if (imgCon) { 
+        // draw it
+        nsPoint p(inner.x, inner.y);
+        nsRect r(0,0,size,size);
+        aRenderingContext.DrawImage(imgCon, &r, &p);
+        iconUsed = PR_TRUE;
+      }
     }
+
+    // if we could not draw the image, then just draw some grafitti
+    if (!iconUsed) {
+      nscolor oldColor;
+      aRenderingContext.DrawRect(0,0,size,size);
+      aRenderingContext.GetColor(oldColor);
+      aRenderingContext.SetColor( aIconId == NS_ICON_BROKEN_IMAGE ? NS_RGB(0xFF,0,0) : NS_RGB(0,0xFF,0));
+      aRenderingContext.FillEllipse(NS_STATIC_CAST(int,size/2),NS_STATIC_CAST(int,size/2),
+                                    NS_STATIC_CAST(int,(size/2)-(2*p2t)),NS_STATIC_CAST(int,(size/2)-(2*p2t)));
+      aRenderingContext.SetColor(oldColor);
+    }  
+
+    // Reduce the inner rect by the width of the icon, and leave an
+    // additional ICON_PADDING pixels for padding
+    PRInt32 iconWidth = NSIntPixelsToTwips(ICON_SIZE + ICON_PADDING, p2t);
+    inner.x += iconWidth;
+    inner.width -= iconWidth;
   }
 
-  // if we could not draw the image, then just draw some grafitti
-  if (!iconUsed) {
-    nscolor oldColor;
-    aRenderingContext.DrawRect(0,0,size,size);
-    aRenderingContext.GetColor(oldColor);
-    aRenderingContext.SetColor( aIconId == NS_ICON_BROKEN_IMAGE ? NS_RGB(0xFF,0,0) : NS_RGB(0,0xFF,0));
-    aRenderingContext.FillEllipse(NS_STATIC_CAST(int,size/2),NS_STATIC_CAST(int,size/2),
-                                  NS_STATIC_CAST(int,(size/2)-(2*p2t)),NS_STATIC_CAST(int,(size/2)-(2*p2t)));
-    aRenderingContext.SetColor(oldColor);
-  }  
-
-  // Reduce the inner rect by the width of the icon, and leave an
-  // additional ICON_PADDING pixels for padding
-  PRInt32 iconWidth = NSIntPixelsToTwips(ICON_SIZE + ICON_PADDING, p2t);
-  inner.x += iconWidth;
-  inner.width -= iconWidth;
-
   // If there's still room, display the alt-text
   if (!inner.IsEmpty()) {
     nsCOMPtr<nsIContent> content;
@@ -1221,24 +1268,27 @@ nsImageFrame::Paint(nsIPresContext*      aPresContext,
       return NS_OK;
 
 
-    // First paint background and borders
-    if (NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer) {
+    // First paint background and borders, which should be in the
+    // FOREGROUND or BACKGROUND paint layer if the element is
+    // inline-level or block-level, respectively.  (See CSS2 9.5, which
+    // is the rationale for paint layers.)
+    const nsStyleDisplay *display;
+    ::GetStyleData(NS_STATIC_CAST(nsIStyleContext*, mStyleContext), &display);
+    nsFramePaintLayer backgroundLayer = display->IsBlockLevel()
+                                            ? NS_FRAME_PAINT_LAYER_BACKGROUND
+                                            : NS_FRAME_PAINT_LAYER_FOREGROUND;
+    if (aWhichLayer == backgroundLayer) {
       const nsStyleVisibility* vis = 
         (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
       if (vis->IsVisibleOrCollapsed()) {
         const nsStyleBorder* myBorder = (const nsStyleBorder*)
           mStyleContext->GetStyleData(eStyleStruct_Border);
-        const nsStyleOutline* myOutline = (const nsStyleOutline*)
-          mStyleContext->GetStyleData(eStyleStruct_Outline);
         nsRect rect(0, 0, mRect.width, mRect.height);
         nsCSSRendering::PaintBackground(aPresContext, aRenderingContext, this,
                                         aDirtyRect, rect, *myBorder, 0, 0);
         nsCSSRendering::PaintBorder(aPresContext, aRenderingContext, this,
                                     aDirtyRect, rect, *myBorder,
                                     mStyleContext, 0);
-        nsCSSRendering::PaintOutline(aPresContext, aRenderingContext, this,
-                                     aDirtyRect, rect, *myBorder,
-                                     *myOutline, mStyleContext, 0);
       }
     }
 
@@ -1254,9 +1304,10 @@ nsImageFrame::Paint(nsIPresContext*      aPresContext,
     if (loadStatus & imgIRequest::STATUS_ERROR || !imgCon) {
       // No image yet, or image load failed. Draw the alt-text and an icon
       // indicating the status (unless image is blocked, in which case we show nothing)
+
 #ifndef SUPPRESS_LOADING_ICON
       if (NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer &&
-          !mImageBlocked) {
+          (!mImageBlocked || mIconLoad->mPrefAllImagesBlocked)) {
 #else
       if (NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer &&
           mInitialLoadCompleted) {
@@ -1268,6 +1319,7 @@ nsImageFrame::Paint(nsIPresContext*      aPresContext,
       }
     }
     else {
+      PRBool paintOutline   = PR_FALSE;
       mInitialLoadCompleted = PR_TRUE;
       if (NS_FRAME_PAINT_LAYER_FOREGROUND == aWhichLayer) {
         // Now render the image into our content area (the area inside the
@@ -1330,6 +1382,7 @@ nsImageFrame::Paint(nsIPresContext*      aPresContext,
 
             aRenderingContext.DrawScaledImage(imgCon, &r, &d);
           }
+          paintOutline = PR_TRUE;
         }
       }
 
@@ -1344,6 +1397,20 @@ nsImageFrame::Paint(nsIPresContext*      aPresContext,
         aRenderingContext.Translate(inner.x, inner.y);
         map->Draw(aPresContext, aRenderingContext);
         aRenderingContext.PopState(clipState);
+        paintOutline = PR_TRUE;
+      }
+
+      // paint the outline in the overlay layer (or if there is an image map) until the 
+      // general problem of painting it outside the border box is solved.
+      if (paintOutline) {
+        const nsStyleBorder* myBorder = (const nsStyleBorder*)
+          mStyleContext->GetStyleData(eStyleStruct_Border);
+        const nsStyleOutline* myOutline = (const nsStyleOutline*)
+          mStyleContext->GetStyleData(eStyleStruct_Outline);
+        nsRect rect(0, 0, mRect.width, mRect.height);
+        nsCSSRendering::PaintOutline(aPresContext, aRenderingContext, this,
+                                     aDirtyRect, rect, *myBorder,
+                                     *myOutline, mStyleContext, 0);
       }
 
 #ifdef DEBUG
@@ -1378,7 +1445,7 @@ nsImageFrame::Paint(nsIPresContext*      aPresContext,
   if (!(displaySelection & nsISelectionDisplay::DISPLAY_IMAGES))
     return NS_OK;//no need to check the blue border, we cannot be drawn selected
   
-  return nsFrame::Paint(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
+  return nsFrame::Paint(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer, nsISelectionDisplay::DISPLAY_IMAGES);
 }
 
 nsImageMap*
@@ -1712,26 +1779,25 @@ nsImageFrame::AttributeChanged(nsIPresContext* aPresContext,
     nsAutoString newSRC;
     aChild->GetAttr(kNameSpaceID_None, nsHTMLAtoms::src, newSRC);
 
-    PRUint32 loadStatus = imgIRequest::STATUS_ERROR;
 
-    if (mLoads[0].mRequest)
+    if (mLoads[0].mRequest) {
+      PRUint32 loadStatus = imgIRequest::STATUS_ERROR;
       mLoads[0].mRequest->GetImageStatus(&loadStatus);
 
-    if (!(loadStatus & imgIRequest::STATUS_SIZE_AVAILABLE)) {
-      if (mLoads[0].mRequest) {
+      if (!(loadStatus & imgIRequest::STATUS_SIZE_AVAILABLE)) {
         mFailureReplace = PR_FALSE; // don't cause a CantRenderReplacedElement call
         mLoads[0].mRequest->Cancel(NS_ERROR_FAILURE);
         mLoads[0].mRequest = nsnull;
       }
-
-      mCanSendLoadEvent = PR_TRUE;
     }
 
+    mCanSendLoadEvent = PR_TRUE;
+
     if (mLoads[1].mRequest) {
       mLoads[1].mRequest->Cancel(NS_ERROR_FAILURE);
       mLoads[1].mRequest = nsnull;
     }
-    
+
     nsCOMPtr<imgIRequest> req(do_CreateInstance("@mozilla.org/image/request;1"));
     if (!mLoads[0].mRequest) {
       mLoads[0].mRequest = req;
@@ -1784,12 +1850,13 @@ mRect.height);
   fprintf(out, " [content=%p]", mContent);
 
   // output the img src url
-  nsCOMPtr<nsIURI> uri;
-  mLoads[0].mRequest->GetURI(getter_AddRefs(uri));
-  nsCAutoString uristr;
-  uri->GetAsciiSpec(uristr);
-  fprintf(out, " [src=%s]", uristr.get());
-
+  if (mLoads[0].mRequest) {
+    nsCOMPtr<nsIURI> uri;
+    mLoads[0].mRequest->GetURI(getter_AddRefs(uri));
+    nsCAutoString uristr;
+    uri->GetAsciiSpec(uristr);
+    fprintf(out, " [src=%s]", uristr.get());
+  }
   fputs("\n", out);
   return NS_OK;
 }
@@ -1850,9 +1917,9 @@ nsImageFrame::IsImageComplete(PRBool* aComplete)
 }
 
 nsresult
-nsImageFrame::LoadImage(const nsAReadableString& aSpec, nsIPresContext *aPresContext, imgIRequest *aRequest)
+nsImageFrame::LoadImage(const nsAString& aSpec, nsIPresContext *aPresContext, imgIRequest *aRequest, PRBool aCheckContentPolicy)
 {
-  nsresult rv = RealLoadImage(aSpec, aPresContext, aRequest);
+  nsresult rv = RealLoadImage(aSpec, aPresContext, aRequest, aCheckContentPolicy);
 
   if (NS_FAILED(rv)) {
     int whichLoad = GetImageLoad(aRequest);
@@ -1864,7 +1931,7 @@ nsImageFrame::LoadImage(const nsAReadableString& aSpec, nsIPresContext *aPresCon
 }
 
 nsresult
-nsImageFrame::RealLoadImage(const nsAReadableString& aSpec, nsIPresContext *aPresContext, imgIRequest *aRequest)
+nsImageFrame::RealLoadImage(const nsAString& aSpec, nsIPresContext *aPresContext, imgIRequest *aRequest, PRBool aCheckContentPolicy)
 {
   nsresult rv = NS_OK;
 
@@ -1878,7 +1945,8 @@ nsImageFrame::RealLoadImage(const nsAReadableString& aSpec, nsIPresContext *aPre
 
   /* don't load the image if some security check fails... */
   GetRealURI(aSpec, getter_AddRefs(realURI));
-  if (!CanLoadImage(realURI)) return NS_ERROR_FAILURE;
+  if (aCheckContentPolicy)
+    if (!CanLoadImage(realURI)) return NS_ERROR_FAILURE;
 
   if (!mListener) {
     nsImageListener *listener = new nsImageListener(this);
@@ -1909,14 +1977,29 @@ nsImageFrame::RealLoadImage(const nsAReadableString& aSpec, nsIPresContext *aPre
   /* set this back to FALSE before we do the real load */
   mInitialLoadCompleted = PR_FALSE;
 
+  nsCOMPtr<nsIURI> baseURI;
+  rv = aPresContext->GetBaseURL(getter_AddRefs(baseURI));
   nsCOMPtr<imgIRequest> tempRequest;
-  return il->LoadImage(uri, loadGroup, mListener, aPresContext, loadFlags, nsnull, aRequest, getter_AddRefs(tempRequest));
+  return il->LoadImage(uri, baseURI, loadGroup, mListener, aPresContext, loadFlags, nsnull, aRequest, getter_AddRefs(tempRequest));
 }
 
 #define INTERNAL_GOPHER_LENGTH 16 /* "internal-gopher-" length */
 
 void
-nsImageFrame::GetURI(const nsAReadableString& aSpec, nsIURI **aURI)
+nsImageFrame::GetDocumentCharacterSet(nsAString& aCharset) const
+{
+  nsresult rv;
+  nsCOMPtr<nsIHTMLContent> htmlContent(do_QueryInterface(mContent, &rv));
+  if (NS_SUCCEEDED(rv)) {
+    nsCOMPtr<nsIDocument> doc;
+    rv = htmlContent->GetDocument(*getter_AddRefs(doc));
+    if (NS_SUCCEEDED(rv))
+      (void) doc->GetDocumentCharacterSet(aCharset);
+  }
+}
+
+void
+nsImageFrame::GetURI(const nsAString& aSpec, nsIURI **aURI)
 {
   *aURI = nsnull;
 
@@ -1938,11 +2021,15 @@ nsImageFrame::GetURI(const nsAReadableString& aSpec, nsIURI **aURI)
 }
 
 void
-nsImageFrame::GetRealURI(const nsAReadableString& aSpec, nsIURI **aURI)
+nsImageFrame::GetRealURI(const nsAString& aSpec, nsIURI **aURI)
 {
   nsCOMPtr<nsIURI> baseURI;
   GetBaseURI(getter_AddRefs(baseURI));
-  NS_NewURI(aURI, aSpec, nsnull, baseURI);
+  nsAutoString charset;
+  GetDocumentCharacterSet(charset);
+  NS_NewURI(aURI, aSpec, 
+            charset.IsEmpty() ? nsnull : NS_ConvertUCS2toUTF8(charset).get(), 
+            baseURI);
 }
 
 void
@@ -2020,7 +2107,7 @@ nsImageFrame::CanLoadImage(nsIURI *aURI)
     nsCOMPtr<nsIDOMWindow> domWin(do_QueryInterface(globalScript));
 
     rv = NS_CheckContentLoadPolicy(nsIContentPolicy::IMAGE,
-                                 aURI, element, domWin, &shouldLoad);
+                                   aURI, element, domWin, &shouldLoad);
     if (NS_SUCCEEDED(rv) && !shouldLoad) {
       // this image has been blocked, so flag it
       mImageBlocked = PR_TRUE;
@@ -2068,14 +2155,14 @@ nsresult nsImageFrame::LoadIcons(nsIPresContext *aPresContext)
 #ifdef NOISY_ICON_LOADING
       printf( "Loading request %p\n", mIconLoad->mIconLoads[NS_ICON_LOADING_IMAGE].mRequest.get() );
 #endif
-      rv = LoadImage(loadingSrc, aPresContext, mIconLoad->mIconLoads[NS_ICON_LOADING_IMAGE].mRequest);
+      rv = LoadImage(loadingSrc, aPresContext, mIconLoad->mIconLoads[NS_ICON_LOADING_IMAGE].mRequest, PR_FALSE);
       if (NS_SUCCEEDED(rv)) {
         mIconLoad->mIconLoads[NS_ICON_BROKEN_IMAGE].mRequest = do_CreateInstance("@mozilla.org/image/request;1");
         if (mIconLoad->mIconLoads[NS_ICON_BROKEN_IMAGE].mRequest) {
 #ifdef NOISY_ICON_LOADING
           printf( "Loading request %p\n", mIconLoad->mIconLoads[NS_ICON_BROKEN_IMAGE].mRequest.get() );
 #endif
-          rv = LoadImage(brokenSrc, aPresContext, mIconLoad->mIconLoads[NS_ICON_BROKEN_IMAGE].mRequest);
+          rv = LoadImage(brokenSrc, aPresContext, mIconLoad->mIconLoads[NS_ICON_BROKEN_IMAGE].mRequest, PR_FALSE);
           // ImageLoader will callback into OnStartContainer, which will handle the mIconsLoaded flag
         }
       }
@@ -2131,7 +2218,7 @@ void nsImageFrame::InvalidateIcon(nsIPresContext *aPresContext)
   Invalidate(aPresContext, rect, PR_FALSE);
 }
 
-void nsImageFrame::IconLoad::GetAltModePref(nsIPresContext *aPresContext)
+void nsImageFrame::IconLoad::GetPrefs(nsIPresContext *aPresContext)
 {
   NS_ASSERTION(aPresContext, "null presContext is not allowed in GetAltModePref");
   // NOTE: the presContext could be used to fetch a cached pref if needed, but is not for now
@@ -2139,11 +2226,22 @@ void nsImageFrame::IconLoad::GetAltModePref(nsIPresContext *aPresContext)
   nsCOMPtr<nsIPref> prefs = do_GetService(NS_PREF_CONTRACTID);
   if (prefs) {
     PRBool boolPref;
+    PRInt32 intPref;
     if (NS_SUCCEEDED(prefs->GetBoolPref("browser.display.force_inline_alttext", &boolPref))) {
       mPrefForceInlineAltText = boolPref;
     } else {
       mPrefForceInlineAltText = PR_FALSE;
     }
+    if (NS_SUCCEEDED(prefs->GetIntPref("network.image.imageBehavior", &intPref)) && intPref == 2) {
+      mPrefAllImagesBlocked = PR_TRUE;
+    } else {
+      mPrefAllImagesBlocked = PR_FALSE;
+    }
+    if (NS_SUCCEEDED(prefs->GetBoolPref("browser.display.show_image_placeholders", &boolPref))) {
+      mPrefShowPlaceholders = boolPref;
+    } else {
+      mPrefShowPlaceholders = PR_TRUE;
+    }
   }
 }
 
