@@ -249,6 +249,16 @@ nsInlineFrame::RemoveFrame(nsIPresContext* aPresContext,
   if (aOldFrame) {
     // Loop and destroy the frame and all of its continuations.
     PRBool generateReflowCommand = PR_FALSE;
+
+    // If the frame we are removing is a brFrame and we have a
+    // nextInFlow, we need a reflow so we can attempt to pull up
+    // any frames in the nextInFlow that can fit on the line
+    // the brFrame was on.
+    nsCOMPtr<nsIAtom> frameType;
+    aOldFrame->GetFrameType(getter_AddRefs(frameType));
+    if (frameType == nsLayoutAtoms::brFrame && mNextInFlow)
+      generateReflowCommand = PR_TRUE;
+
     nsIFrame* oldFrameParent;
     aOldFrame->GetParent(&oldFrameParent);
     nsInlineFrame* parent = (nsInlineFrame*) oldFrameParent;
@@ -602,25 +612,34 @@ nsInlineFrame::ReflowFrames(nsIPresContext* aPresContext,
       aMetrics.width += aReflowState.mComputedBorderPadding.right;
     }
 
+    nsCOMPtr<nsIFontMetrics> fm;
     const nsStyleFont* font;
     GetStyleData(eStyleStruct_Font, (const nsStyleStruct*&)font);
-    aReflowState.rendContext->SetFont(font->mFont);
-    nsCOMPtr<nsIFontMetrics> fm;
-    aReflowState.rendContext->GetFontMetrics(*getter_AddRefs(fm));
+    if (font) {
+      aReflowState.rendContext->SetFont(font->mFont);
+      aReflowState.rendContext->GetFontMetrics(*getter_AddRefs(fm));
+    } else {
+      NS_WARNING("No font retrieved from style context - sorry but font metrics are not available");
+    }
 
-    // Compute final height of the frame.
-    //
-    // Do things the standard css2 way -- though it's hard to find it
-    // in the css2 spec! It's actually found in the css1 spec section
-    // 4.4 (you will have to read between the lines to really see
-    // it).
-    //
-    // The height of our box is the sum of our font size plus the top
-    // and bottom border and padding. The height of children do not
-    // affect our height.
-    fm->GetMaxAscent(aMetrics.ascent);
-    fm->GetMaxDescent(aMetrics.descent);
-    fm->GetHeight(aMetrics.height);
+    if (fm) {
+      // Compute final height of the frame.
+      //
+      // Do things the standard css2 way -- though it's hard to find it
+      // in the css2 spec! It's actually found in the css1 spec section
+      // 4.4 (you will have to read between the lines to really see
+      // it).
+      //
+      // The height of our box is the sum of our font size plus the top
+      // and bottom border and padding. The height of children do not
+      // affect our height.
+      fm->GetMaxAscent(aMetrics.ascent);
+      fm->GetMaxDescent(aMetrics.descent);
+      fm->GetHeight(aMetrics.height);
+    } else {
+      NS_WARNING("Cannot get font metrics - defaulting sizes to 0");
+      aMetrics.ascent = aMetrics.descent = aMetrics.height = 0;
+    }
     aMetrics.ascent += aReflowState.mComputedBorderPadding.top;
     aMetrics.descent += aReflowState.mComputedBorderPadding.bottom;
     aMetrics.height += aReflowState.mComputedBorderPadding.top +
@@ -1046,9 +1065,9 @@ nsFirstLineFrame::Reflow(nsIPresContext* aPresContext,
         // of the parent frame.
         nsIStyleContext* newSC;
         aPresContext->ResolvePseudoStyleContextFor(mContent,
-                                                  nsHTMLAtoms::mozLineFrame,
-                                                  parentContext,
-                                                  PR_FALSE, &newSC);
+                                                   nsHTMLAtoms::mozLineFrame,
+                                                   parentContext,
+                                                   &newSC);
         if (newSC) {
           // Switch to the new style context.
           SetStyleContext(aPresContext, newSC);
