@@ -106,7 +106,8 @@ static NS_DEFINE_CID(kCChildCID, NS_CHILD_CID);
 // Bug 8065: Limit content frame depth to some reasonable level.
 //           This does not count chrome frames when determining depth,
 //           nor does it prevent chrome recursion.
-#define MAX_DEPTH_CONTENT_FRAMES 25
+//           Also see bug 126466.
+#define MAX_DEPTH_CONTENT_FRAMES 8
 
 /*******************************************************************************
  * FrameLoadingInfo 
@@ -208,6 +209,7 @@ public:
 #endif
 
   NS_IMETHOD GetFrameType(nsIAtom** aType) const;
+  NS_IMETHOD Destroy(nsIPresContext* aPresContext);
 
   /**
     * @see nsIFrame::Paint
@@ -254,8 +256,6 @@ protected:
   nsresult CreateViewAndWidget(nsIPresContext* aPresContext,
                                nsIWidget**     aWidget);
 
-  virtual ~nsHTMLFrameInnerFrame();
-
   virtual void GetDesiredSize(nsIPresContext* aPresContext,
                               const nsHTMLReflowState& aReflowState,
                               nsHTMLReflowMetrics& aDesiredSize);
@@ -360,7 +360,6 @@ nsHTMLFrameOuterFrame::Init(nsIPresContext*  aPresContext,
     nsresult rv = NS_OK;
     nsIStyleContext *innerStyleContext = nsnull;
     rv = aPresContext->ResolveStyleContextFor(mContent, mStyleContext,
-                                              PR_FALSE,
                                               &innerStyleContext);
     if (NS_SUCCEEDED(rv)) {
       rv = firstChild->Init(aPresContext, mContent, this, innerStyleContext, nsnull);
@@ -649,7 +648,9 @@ nsHTMLFrameInnerFrame::nsHTMLFrameInnerFrame()
   mPresShellWeak = nsnull;
 }
 
-nsHTMLFrameInnerFrame::~nsHTMLFrameInnerFrame()
+
+NS_IMETHODIMP
+nsHTMLFrameInnerFrame::Destroy(nsIPresContext* aPresContext)
 {
    //printf("nsHTMLFrameInnerFrame destructor %X \n", this);
 
@@ -674,6 +675,8 @@ nsHTMLFrameInnerFrame::~nsHTMLFrameInnerFrame()
   }
   mSubShell = nsnull; // This is the location it was released before...
                       // Not sure if there is ordering depending on this.
+
+  return nsLeafFrame::Destroy(aPresContext);
 }
 
 PRBool nsHTMLFrameInnerFrame::GetURL(nsIContent* aContent, nsString& aResult)
@@ -827,7 +830,7 @@ nsHTMLFrameInnerFrame::QueryInterface(REFNSIID aIID, void** aInstancePtr)
 NS_IMETHODIMP
 nsHTMLFrameInnerFrame::OnStateChange(nsIWebProgress *aWebProgress,
                                      nsIRequest *aRequest,
-                                     PRInt32 aStateFlags, PRUint32 aStatus)
+                                     PRUint32 aStateFlags, nsresult aStatus)
 {
   if (!((~aStateFlags) & (nsIWebProgressListener::STATE_IS_DOCUMENT |
                           nsIWebProgressListener::STATE_TRANSFERRING))) {
@@ -873,7 +876,7 @@ nsHTMLFrameInnerFrame::OnStatusChange(nsIWebProgress *aWebProgress,
 
 NS_IMETHODIMP
 nsHTMLFrameInnerFrame::OnSecurityChange(nsIWebProgress *aWebProgress,
-                                        nsIRequest *aRequest, PRInt32 state)
+                                        nsIRequest *aRequest, PRUint32 state)
 {
   return NS_OK;
 }
@@ -978,8 +981,10 @@ nsHTMLFrameInnerFrame::CreateDocShell(nsIPresContext* aPresContext)
     nsCOMPtr<nsIDocShellTreeItem> parentAsItem(do_QueryInterface(parentAsSupports));
     while (parentAsItem) {
       depth++;
-      if (MAX_DEPTH_CONTENT_FRAMES < depth)
+      if (MAX_DEPTH_CONTENT_FRAMES < depth) {
+        NS_WARNING("Too many nested content frames so giving up");
         return NS_ERROR_UNEXPECTED; // Too deep, give up!  (silently?)
+      }
 
       // Only count depth on content, not chrome.
       // If we wanted to limit total depth, skip the following check:
@@ -1148,11 +1153,12 @@ nsHTMLFrameInnerFrame::DoLoadURL(nsIPresContext* aPresContext)
   // Make an absolute URL
   nsCOMPtr<nsIURI> baseURL;
   nsCOMPtr<nsIHTMLContent> htmlContent = do_QueryInterface(parentContent, &rv);
+  nsCOMPtr<nsIDocument> doc;
   if (NS_SUCCEEDED(rv) && htmlContent) {
     htmlContent->GetBaseURL(*getter_AddRefs(baseURL));
+    (void) htmlContent->GetDocument(*getter_AddRefs(doc));
   }
   else {
-    nsCOMPtr<nsIDocument> doc;
     rv = parentContent->GetDocument(*getter_AddRefs(doc));
     if (NS_SUCCEEDED(rv) && doc) {
       doc->GetBaseURL(*getter_AddRefs(baseURL));
@@ -1160,12 +1166,17 @@ nsHTMLFrameInnerFrame::DoLoadURL(nsIPresContext* aPresContext)
   }
   if (!baseURL) return NS_ERROR_NULL_POINTER;
 
+  nsAutoString docCharset;
+  if (doc)
+    (void) doc->GetDocumentCharacterSet(docCharset);
+
   nsAutoString absURL;
   rv = NS_MakeAbsoluteURI(absURL, url, baseURL);
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIURI> uri;
-  NS_NewURI(getter_AddRefs(uri), absURL);
+  NS_NewURI(getter_AddRefs(uri), absURL, 
+            docCharset.IsEmpty() ? nsnull : NS_ConvertUCS2toUTF8(docCharset).get());
 
   // Check for security
   nsCOMPtr<nsIScriptSecurityManager> secMan = 
@@ -1218,10 +1229,10 @@ nsHTMLFrameInnerFrame::DoLoadURL(nsIPresContext* aPresContext)
   nsCOMPtr<nsIWebProgress> webProgress(do_GetInterface(mSubShell));
 
   if (webProgress) {
-    webProgress->AddProgressListener(this);
+    webProgress->AddProgressListener(this, nsIWebProgress::NOTIFY_STATE_DOCUMENT);
   }
 
-  rv = docShell->LoadURI(uri, loadInfo, nsIWebNavigation::LOAD_FLAGS_NONE);
+  rv = docShell->LoadURI(uri, loadInfo, nsIWebNavigation::LOAD_FLAGS_NONE, PR_FALSE);
   NS_ASSERTION(NS_SUCCEEDED(rv), "failed to load URL");
 
   return rv;
