@@ -122,7 +122,8 @@ public:
 
   NS_IMETHOD ContentStatesChanged(nsIPresContext* aPresContext, 
                                   nsIContent*     aContent1,
-                                  nsIContent*     aContent2);
+                                  nsIContent*     aContent2,
+                                  PRInt32         aStateMask);
 
   NS_IMETHOD AttributeChanged(nsIPresContext* aPresContext,
                               nsIContent*     aContent,
@@ -176,15 +177,6 @@ public:
                                nsIFrame**    aInsertionPoint,
                                PRBool*       aMultiple = nsnull);
 
-  NS_IMETHOD CreateTreeWidgetContent(nsIPresContext*           aPresContext,
-                                        nsIFrame*              aParentFrame,
-                                        nsIFrame*              aPrevFrame,
-                                        nsIContent*            aChild,
-                                        nsIFrame**             aResult,
-                                        PRBool                 aIsAppend,
-                                        PRBool                 aIsScrollbar,
-                                        nsILayoutHistoryState* aFrameState);
-
   NS_IMETHOD CreateListBoxContent(nsIPresContext* aPresContext,
                                   nsIFrame*       aParentFrame,
                                   nsIFrame*       aPrevFrame,
@@ -534,6 +526,25 @@ protected:
                               nsIStyleContext*         aStyleContext,
                               nsFrameItems&            aFrameItems);
 
+  nsresult ConstructPageBreakFrame(nsIPresShell*            aPresShell, 
+                                   nsIPresContext*          aPresContext,
+                                   nsFrameConstructorState& aState,
+                                   nsIContent*              aContent,
+                                   nsIFrame*                aParentFrame,
+                                   nsIStyleContext*         aStyleContext,
+                                   nsFrameItems&            aFrameItems);
+
+  // Construct a page break frame if page-break-before:always is set in aStyleContext
+  // and add it to aFrameItems. Return true if page-break-after:always is set on aStyleContext.
+  // Don't do this for row groups, rows or cell, because tables handle those internally.
+  PRBool PageBreakBefore(nsIPresShell*            aPresShell,
+                         nsIPresContext*          aPresContext,
+                         nsFrameConstructorState& aState,
+                         nsIContent*              aContent,
+                         nsIFrame*                aParentFrame,
+                         nsIStyleContext*         aStyleContext,
+                         nsFrameItems&            aFrameItems);
+
   nsresult ConstructHTMLFrame(nsIPresShell*            aPresShell, 
                               nsIPresContext*          aPresContext,
                               nsFrameConstructorState& aState,
@@ -855,6 +866,16 @@ protected:
                              nsIFrame*                aFrame,
                              nsIFrame*                aFrameList);
 
+  PRBool NeedSpecialFrameReframe(nsIPresShell*    aPresShell,
+                                 nsIPresContext*  aPresContext,
+                                 nsIContent*      aParent1,
+                                 nsIContent*      aParent2,
+                                 nsIFrame*&       aParentFrame,
+                                 nsIContent*      aChild,
+                                 PRInt32          aIndexInContainer,
+                                 nsIFrame*&       aPrevSibling,
+                                 nsIFrame*        aNextSibling);
+
   nsresult SplitToContainingBlock(nsIPresContext*          aPresContext,
                                   nsFrameConstructorState& aState,
                                   nsIFrame*                aFrame,
@@ -990,8 +1011,9 @@ protected:
                                  nsIFrame*                aPrevSibling,
                                  nsFrameItems&            aFrameItems);
 
-  nsresult RemoveFixedItems(nsIPresContext& aPresContext,
-                            nsIPresShell&   aPresShell);
+  nsresult RemoveFixedItems(nsIPresContext*  aPresContext,
+                            nsIPresShell*    aPresShell,
+                            nsIFrameManager* aFrameManager);
 
 protected:
   nsIDocument*        mDocument;
