@@ -47,6 +47,7 @@
 #include "nsTableColFrame.h"
 #include "nsTableColGroupFrame.h"
 
+class nsCellMap;
 class nsTableCellMap;
 class nsTableCellFrame;
 class nsTableColFrame;
@@ -217,7 +218,20 @@ public:
                               float        aPixelToTwips,
                               nsPixelRound aRound= eAlwaysRoundUp);
 
-  static void NotifyAncestorsOfSpecialReflow(nsIFrame& aFrame);
+  // See if a special height reflow will occur due to having a pct height when
+  // the pct height basis may not yet be valid.
+  static void CheckRequestSpecialHeightReflow(const nsHTMLReflowState& aReflowState);
+
+  // Notify the frame and its ancestors (up to the containing table) that a special
+  // height reflow will occur. 
+  static void RequestSpecialHeightReflow(const nsHTMLReflowState& aReflowState);
+
+  // Return true (and set aMetrics's desiredSize to aRect) if the special height reflow
+  // was not initiated by aReflowState.frame's containing table.
+  static PRBool IsPrematureSpecialHeightReflow(const nsHTMLReflowState& aReflowState,
+                                               const nsRect&            aRect,
+                                               PRBool                   aNeedSpecialHeightReflow,
+                                               nsHTMLReflowMetrics&     aMetrics);
 
   NS_IMETHOD IsPercentageBase(PRBool& aBase) const;
 
@@ -228,6 +242,9 @@ public:
   static void RePositionViews(nsIPresContext* aPresContext,
                               nsIFrame*       aFrame);
 
+  static PRBool PageBreakAfter(nsIFrame& aSourceFrame,
+                               nsIFrame* aNextFrame);
+
   nsPoint GetFirstSectionOrigin(const nsHTMLReflowState& aReflowState) const;
   /*
    * Notification that aAttribute has changed for content inside a table (cell, row, etc)
@@ -364,14 +381,11 @@ public:
                        const nsHTMLReflowState& aReflowState,
                        nscoord                  aAvailHeight,
                        nsReflowReason           aReason,
+                       nsIFrame*&               aLastChildReflowed,
                        PRBool&                  aDoCollapse,
                        PRBool&                  aDidBalance,
                        nsReflowStatus&          aStatus);
 
-  NS_IMETHOD GetParentStyleContextProvider(nsIPresContext* aPresContext,
-                                           nsIFrame** aProviderFrame, 
-                                           nsContextProviderRelationship& aRelationship);
-
   static nsMargin GetPadding(const nsHTMLReflowState& aReflowState,
                              const nsTableCellFrame*  aCellFrame);
 
@@ -380,6 +394,10 @@ public:
 
   nsFrameList& GetColGroups();
 
+  NS_IMETHOD GetParentStyleContextFrame(nsIPresContext* aPresContext,
+                                        nsIFrame**      aProviderFrame,
+                                        PRBool*         aIsChild);
+
   /**
    * Get the "type" of the frame
    *
@@ -418,7 +436,8 @@ public:
     */
   virtual PRInt32  GetEffectiveRowSpan(PRInt32                 aStartRowIndex,
                                        const nsTableCellFrame& aCell) const;
-  virtual PRInt32  GetEffectiveRowSpan(const nsTableCellFrame& aCell) const;
+  virtual PRInt32  GetEffectiveRowSpan(const nsTableCellFrame& aCell,
+                                       nsCellMap*              aCellMap = nsnull);
 
   /** return the col span of a cell, taking into account col span magic at the edge
     * of a table.
@@ -428,7 +447,8 @@ public:
     * @return  the col span, correcting for col spans that extend beyond the edge
     *          of the table.
     */
-  virtual PRInt32  GetEffectiveColSpan(const nsTableCellFrame& aCell) const;
+  virtual PRInt32  GetEffectiveColSpan(const nsTableCellFrame& aCell,
+                                       nsCellMap*              aCellMap = nsnull) const;
 
   /** return the value of the COLS attribute, adjusted for the 
     * actual number of columns in the table
@@ -534,9 +554,13 @@ public:
 
   PRBool NeedSpecialReflow() const;
   void   SetNeedSpecialReflow(PRBool aValue);
+
   PRBool NeedToInitiateSpecialReflow() const;
   void   SetNeedToInitiateSpecialReflow(PRBool aValue);
 
+  PRBool InitiatedSpecialReflow() const;
+  void   SetInitiatedSpecialReflow(PRBool aValue);
+
 protected:
 
   /** protected constructor. 
@@ -603,6 +627,7 @@ protected:
                            PRBool               aDoColGroups,
                            PRBool               aDirtyOnly,
                            nsReflowStatus&      aStatus,
+                           nsIFrame*&           aLastChildReflowed,
                            PRBool*              aReflowedAtLeastOne = nsnull);
 // begin incremental reflow methods
   
@@ -896,11 +921,12 @@ public: /* ----- Cell Map public methods ----- */
 
 public:
   static nsIAtom* gColGroupAtom;
+#ifdef DEBUG
   void Dump(nsIPresContext* aPresContext,
             PRBool          aDumpRows,
             PRBool          aDumpCols, 
             PRBool          aDumpCellMap);
-
+#endif
   nsAutoVoidArray mColFrames; // XXX temporarily public 
 
 #ifdef DEBUG
@@ -909,9 +935,9 @@ public:
 #endif
 
 protected:
+#ifdef DEBUG
   void DumpRowGroup(nsIPresContext* aPresContext, nsIFrame* aChildFrame);
-  void DebugPrintCount() const; // Debugging routine
-
+#endif
   // DATA MEMBERS
 
   struct TableBits {
@@ -933,8 +959,9 @@ protected:
     unsigned mRowInserted:1;
     unsigned mNeedSpecialReflow:1;
     unsigned mNeedToInitiateSpecialReflow:1;
+    unsigned mInitiatedSpecialReflow:1;
     unsigned mNeedToCalcBCBorders:1;
-    unsigned : 18;                     // unused
+    unsigned : 17;                     // unused
   } mBits;
 
   nsTableCellMap*         mCellMap;            // maintains the relationships between rows, cols, and cells
@@ -1057,6 +1084,17 @@ inline void nsTableFrame::SetNeedToInitiateSpecialReflow(PRBool aValue)
 {
   mBits.mNeedToInitiateSpecialReflow = (unsigned)aValue;
 }
+
+inline PRBool nsTableFrame::InitiatedSpecialReflow() const
+{
+  return (PRBool)mBits.mInitiatedSpecialReflow;
+}
+
+inline void nsTableFrame::SetInitiatedSpecialReflow(PRBool aValue)
+{
+  mBits.mInitiatedSpecialReflow = (unsigned)aValue;
+}
+
 inline PRBool nsTableFrame::IsRowInserted() const
 {
   return (PRBool)mBits.mRowInserted;
