@@ -116,20 +116,12 @@ nsMathMLFrame::ResolveMathMLCharStyle(nsIPresContext*  aPresContext,
     nsMathMLAtoms::fontstyle_anonymous; // savings
   nsCOMPtr<nsIStyleContext> newStyleContext;
   nsresult rv = aPresContext->ResolvePseudoStyleContextFor(aContent, fontAtom, 
-                                             aParentStyleContext, PR_FALSE,
+                                             aParentStyleContext,
                                              getter_AddRefs(newStyleContext));
   if (NS_SUCCEEDED(rv) && newStyleContext)
     aMathMLChar->SetStyleContext(newStyleContext);
 }
 
-/* static */ PRBool
-nsMathMLFrame::IsEmbellishOperator(nsIFrame* aFrame)
-{
-  nsEmbellishData embellishData;
-  GetEmbellishDataFrom(aFrame, embellishData);
-  return NS_MATHML_IS_EMBELLISH_OPERATOR(embellishData.flags);
-}
-
 /* static */ void
 nsMathMLFrame::GetEmbellishDataFrom(nsIFrame*        aFrame,
                                     nsEmbellishData& aEmbellishData)
@@ -179,6 +171,9 @@ nsMathMLFrame::GetPresentationDataFrom(nsIFrame*           aFrame,
     nsCOMPtr<nsIAtom> tag;
     nsCOMPtr<nsIContent> content;
     frame->GetContent(getter_AddRefs(content));
+    NS_ASSERTION(content, "dangling frame without a content node");
+    if (!content)
+      break;
     content->GetTag(*getter_AddRefs(tag));
     if (tag.get() == nsMathMLAtoms::math) {
       const nsStyleDisplay* display;
@@ -190,6 +185,7 @@ nsMathMLFrame::GetPresentationDataFrom(nsIFrame*           aFrame,
     }
     frame->GetParent(&frame);
   }
+  NS_ASSERTION(frame, "bad MathML markup - could not find the top <math> element");
 }
 
 /* static */ PRBool
@@ -561,7 +557,7 @@ GetMathMLAttributeStyleSheet(nsIPresContext* aPresContext,
     return;
   cssSheet->Init(uri);
   cssSheet->SetTitle(NS_ConvertASCIItoUCS2(kTitle));
-  cssSheet->SetDefaultNameSpaceID(nsMathMLAtoms::nameSpaceID);
+  cssSheet->SetDefaultNameSpaceID(kNameSpaceID_MathML);
   nsCOMPtr<nsIStyleSheet> sheet(do_QueryInterface(cssSheet));
 
   // insert the stylesheet into the styleset without notifying observers
@@ -575,8 +571,10 @@ nsMathMLFrame::MapAttributesIntoCSS(nsIPresContext* aPresContext,
                                     nsIContent*     aContent)
 {
   // normal case, quick return if there are no attributes
-  PRInt32 attrCount;
-  aContent->GetAttrCount(attrCount);
+  NS_ASSERTION(aContent, "null arg");
+  PRInt32 attrCount = 0;
+  if (aContent)
+    aContent->GetAttrCount(attrCount);
   if (!attrCount)
     return 0;
 
@@ -638,8 +636,8 @@ nsMathMLFrame::MapAttributesIntoCSS(nsIPresContext* aPresContext,
     // make a style rule that maps to the equivalent CSS property
     nsAutoString cssRule;
     cssRule.Assign(NS_LITERAL_STRING("[")  + nsDependentString(attrName) +
-                   NS_LITERAL_STRING("='") + attrValue + NS_LITERAL_STRING("']{"));
-    cssRule.Append(cssProperty + NS_LITERAL_STRING("}"));
+                   NS_LITERAL_STRING("='") + attrValue +
+                   NS_LITERAL_STRING("']{") + cssProperty + NS_LITERAL_STRING("}"));
 
     if (!sheet) {
       // first time... we do this to defer the lookup up to the
@@ -661,17 +659,35 @@ nsMathMLFrame::MapAttributesIntoCSS(nsIPresContext* aPresContext,
       sheet->SetOwningDocument(nsnull);
     }
 
-    // insert the rule (note: when the sheet has @namespace and friends,
-    // insert after them, e.g., at the end, otherwise it won't work)
-    PRInt32 pos = 0;
-    if (map->compatibility == kMathMLversion2) {
-      // MathML 2, insert at the end to give it precedence
-      cssSheet->StyleRuleCount(pos);
+    // check for duplicate, if a similar rule is already there, don't bother to add another one
+    // XXX bug 142648 - GetSourceSelectorText is in the format *[color=blue] (i.e., no quotes...) 
+    // XXXrbs need to keep this in sync with the fix for bug 142648
+    nsAutoString selector;
+    selector.Assign(NS_LITERAL_STRING("*[") + nsDependentString(attrName) +
+                    NS_LITERAL_STRING("=") + attrValue +
+                    NS_LITERAL_STRING("]"));
+    PRInt32 k, count;
+    cssSheet->StyleRuleCount(count);
+    for (k = 0; k < count; ++k) {
+      nsAutoString tmpSelector;
+      nsCOMPtr<nsICSSRule> tmpRule;
+      cssSheet->GetStyleRuleAt(k, *getter_AddRefs(tmpRule));
+      nsCOMPtr<nsICSSStyleRule> tmpStyleRule(do_QueryInterface(tmpRule));
+      tmpStyleRule->GetSourceSelectorText(tmpSelector);
+      if (tmpSelector.Equals(selector)) {
+        k = -1;
+        break;
+      }
+    }
+    if (k >= 0) {
+      // insert the rule (note: when the sheet has @namespace and friends,
+      // insert after them, e.g., at the end, otherwise it won't work)
+      // For MathML 2, insert at the end to give it precedence
+      PRInt32 pos = (map->compatibility == kMathMLversion2) ? count : 0;
+      PRUint32 index;
+      domSheet->InsertRule(cssRule, pos, &index);
+      ++ruleCount;
     }
-    //XXX possibly check for duplicate, but might be faster to just insert
-    PRUint32 index;
-    domSheet->InsertRule(cssRule, pos, &index);
-    ++ruleCount;
   }
   // restore the sheet to its owner
   if (sheet) {
