@@ -750,7 +750,6 @@ nsListBoxBodyFrame::ComputeIntrinsicWidth(nsBoxLayoutState& aBoxLayoutState)
   if (firstRowContent) {
     nsCOMPtr<nsIStyleContext> styleContext;
     aBoxLayoutState.GetPresContext()->ResolveStyleContextFor(firstRowContent, nsnull,
-                                                             PR_FALSE, 
                                                              getter_AddRefs(styleContext));
 
     nscoord width = 0;
@@ -1223,7 +1222,7 @@ nsListBoxBodyFrame::GetNextItemBox(nsIBox* aBox, PRInt32 aOffset, PRBool* aCreat
     prevContent->GetParent(*getter_AddRefs(parentContent));
     parentContent->IndexOf(prevContent, i);
     parentContent->ChildCount(childCount);
-    if (i+1 < childCount) {
+    if (i+aOffset+1 < childCount) {
       // There is a content node that wants a frame.
       nsCOMPtr<nsIContent> nextContent;
       parentContent->ChildAt(i+aOffset+1, *getter_AddRefs(nextContent));
