@@ -434,6 +434,12 @@ nsMenuPopupFrame::GetViewOffset(nsIView* aView, nsPoint& aPoint)
   aPoint.x = 0;
   aPoint.y = 0;
  
+  // Keep track of the root view so that we know to stop there
+  nsCOMPtr<nsIViewManager> vm;
+  aView->GetViewManager(*getter_AddRefs(vm));
+  nsIView* rootView;
+  vm->GetRootView(rootView);
+
   nsIView *parent;
   nsRect bounds;
 
@@ -481,6 +487,8 @@ nsMenuPopupFrame::GetViewOffset(nsIView* aView, nsPoint& aPoint)
       aPoint.y += bounds.y;
       aPoint.x += bounds.x;
     }
+    if (parent == rootView)
+      break;
     parent->GetParent(parent);
   }
 }
@@ -585,11 +593,20 @@ nsMenuPopupFrame::AdjustClientXYForNestedDocuments ( nsIDOMXULDocument* inPopupD
   }
   NS_WARN_IF_FALSE(popupDocumentWidget, "ACK, BAD WIDGET");
   
-  // Find the widget associated with the target's document. Recall that we cached the
-  // target popup node in the document of the popup in the nsXULPopupListener.
+  // Find the widget associated with the target's document.
+  // For tooltips, we check the document's tooltipNode (which is set by
+  // nsXULTooltipListener).  For regular popups, use popupNode (set by
+  // nsXULPopupListener).
+
+  nsCOMPtr<nsIAtom> tag;
+  mContent->GetTag(*getter_AddRefs(tag));
   nsCOMPtr<nsIDOMNode> targetNode;
-  inPopupDoc->GetPopupNode(getter_AddRefs(targetNode));
-  NS_WARN_IF_FALSE(targetNode, "ACK, BAD TARGET NODE");
+  if (tag == nsXULAtoms::tooltip)
+    inPopupDoc->GetTooltipNode(getter_AddRefs(targetNode));
+  else
+    inPopupDoc->GetPopupNode(getter_AddRefs(targetNode));
+
+  NS_WARN_IF_FALSE(targetNode, "no popup/tooltip node on document!");
   nsCOMPtr<nsIContent> targetAsContent ( do_QueryInterface(targetNode) );
   nsCOMPtr<nsIWidget> targetDocumentWidget;
   if ( targetAsContent ) {
@@ -1025,8 +1042,11 @@ nsMenuPopupFrame::SyncViewWithFrame(nsIPresContext* aPresContext,
   if (!parentView)
     return NS_OK;
 
+  // Use containingView instead of parentView, to account for the scrollarrows
+  // that a parent menu might have.
+
   nsCOMPtr<nsIWidget> parentViewWidget;
-  GetWidgetForView ( parentView, *getter_AddRefs(parentViewWidget) );
+  GetWidgetForView ( containingView, *getter_AddRefs(parentViewWidget) );
   nsRect localParentWidgetRect(0,0,0,0), screenParentWidgetRect;
   parentViewWidget->WidgetToScreen ( localParentWidgetRect, screenParentWidgetRect );
   PRInt32 screenViewLocX = NSIntPixelsToTwips(screenParentWidgetRect.x,p2t) + (xpos - parentPos.x);
@@ -1039,6 +1059,25 @@ nsMenuPopupFrame::SyncViewWithFrame(nsIPresContext* aPresContext,
     // shouldn't do anything funky if it will already fit on the screen as is.
     //
 
+    ///////////////////////////////////////////////////////////////////////////////
+    //
+    //                +------------------------+          
+    //                |           /\           |
+    // parentPos -> - +------------------------+
+    //              | |                        |
+    //       offset | |                        |
+    //              | |                        |
+    //              | |                        | (screenViewLocX,screenViewLocY)
+    //              - |========================|+--------------
+    //                | parentRect           > ||
+    //                |========================||
+    //                |                        || Submenu 
+    //                +------------------------+|  ( = mRect )
+    //                |           \/           ||
+    //                +------------------------+
+
+
+
     // compute screen coordinates of parent frame so we can play with it. Make sure we put it
     // into twips as everything else is as well.
     nsRect screenParentFrameRect ( NSTwipsToIntPixels(offset.x,t2p), NSTwipsToIntPixels(offset.y,t2p),
@@ -1078,7 +1117,24 @@ nsMenuPopupFrame::SyncViewWithFrame(nsIPresContext* aPresContext,
       }
       else {
         // move it up to be on screen, but don't let it go off the screen at the top
+
+        //  |
+        //  |
+        //  |+----  screenViewLocY
+        //  ||
+        //  ||  Submenu ( = mRect )
+        // -+|
+        //   |
+        //   |
+        // - - - - - - - - - - screenBottomTwips (bottom of the screen)
+        //   |    \ 
+        //   |     }  moveDistY
+        //   |    /
+        //   +----  screenViewLocY + mRect.height
+        //
+
         if ( (screenViewLocY + mRect.height) > screenBottomTwips ) {
+          // XXX Bug 84121 comment 48 says the next line has to use screenHeightTwips, why not screenBottomTwips?
           PRInt32 moveDistY = (screenViewLocY + mRect.height) - screenHeightTwips;
           if ( screenViewLocY - moveDistY < screenTopTwips )
             moveDistY = screenViewLocY - screenTopTwips;          
@@ -1090,10 +1146,13 @@ nsMenuPopupFrame::SyncViewWithFrame(nsIPresContext* aPresContext,
       // Resize it to fit on the screen. By this point, we've given the popup as much
       // room as we can w/out covering the parent. If it still can't be as big
       // as it wants to be, well, it just has to suck up and deal. 
+      //
+      // ySpillage is calculated the same way as moveDistY above. see picture there.
+
       PRInt32 xSpillage = (screenViewLocX + mRect.width) - screenRightTwips;
       if ( xSpillage > 0 )
         mRect.width -= xSpillage;
-      PRInt32 ySpillage = (screenViewLocY + mRect.height + parentRect.height) - screenHeightTwips;
+      PRInt32 ySpillage = (screenViewLocY + mRect.height) - screenBottomTwips;
       if ( ySpillage > 0 )
         mRect.height -= ySpillage;
 
@@ -1162,14 +1221,10 @@ nsMenuPopupFrame::SyncViewWithFrame(nsIPresContext* aPresContext,
   aFrame->GetContent(getter_AddRefs(parentContent));
   nsCOMPtr<nsIAtom> tag;
   mContent->GetTag(*getter_AddRefs(tag));
-  if (tag.get() != nsXULAtoms::tooltip) {
-    nsAutoString sizeToContent;
-    parentContent->GetAttr(kNameSpaceID_None, nsXULAtoms::sizetopopup, sizeToContent);
-    if (sizeToContent == NS_LITERAL_STRING("pref") ||
-        sizeToContent == NS_LITERAL_STRING("always")) {
+  if (tag.get() != nsXULAtoms::tooltip &&
+      nsMenuFrame::IsSizedToPopup(parentContent, PR_FALSE)) {
       nsBoxLayoutState state(mPresContext);
       SetBounds(state, nsRect(mRect.x, mRect.y, parentRect.width, mRect.height));
-    }
   }
     
   nsAutoString shouldDisplay, menuActive;
@@ -1440,6 +1495,8 @@ NS_IMETHODIMP nsMenuPopupFrame::SetCurrentMenuItem(nsIMenuFrame* aMenuItem)
 NS_IMETHODIMP
 nsMenuPopupFrame::Escape(PRBool& aHandledFlag)
 {
+  mIncrementalString = NS_LITERAL_STRING("");
+
   if (!mCurrentMenu)
     return NS_OK;
 
@@ -1462,6 +1519,8 @@ nsMenuPopupFrame::Escape(PRBool& aHandledFlag)
 NS_IMETHODIMP
 nsMenuPopupFrame::Enter()
 {
+  mIncrementalString = NS_LITERAL_STRING("");
+
   // Give it to the child.
   if (mCurrentMenu)
     mCurrentMenu->Enter();
@@ -1470,8 +1529,13 @@ nsMenuPopupFrame::Enter()
 }
 
 nsIMenuFrame*
-nsMenuPopupFrame::FindMenuWithShortcut(PRUint32 aLetter)
+nsMenuPopupFrame::FindMenuWithShortcut(nsIDOMKeyEvent* aKeyEvent, PRBool& doAction)
 {
+  PRUint32 charCode, keyCode;
+  aKeyEvent->GetCharCode(&charCode);
+  aKeyEvent->GetKeyCode(&keyCode);
+
+  doAction = PR_FALSE;
 
   // Enumerate over our list of frames.
   nsIFrame* immediateParent = nsnull;
@@ -1481,67 +1545,181 @@ nsMenuPopupFrame::FindMenuWithShortcut(PRUint32 aLetter)
   if (!immediateParent)
     immediateParent = this;
 
+  PRUint32 matchCount = 0, matchShortcutCount = 0;
+  PRBool foundActive = PR_FALSE;
+  PRBool isMenu = PR_FALSE;
+  PRBool isShortcut;
+  nsIMenuFrame* frameBefore = nsnull;
+  nsIMenuFrame* frameAfter = nsnull;
+  nsIMenuFrame* frameShortcut = nsnull;
+
+  nsCOMPtr<nsIContent> parentContent;
+  mContent->GetParent(*getter_AddRefs(parentContent));
+  if (parentContent) {
+    nsCOMPtr<nsIAtom> tag;
+    parentContent->GetTag(*getter_AddRefs(tag));
+    if (tag != nsXULAtoms::menulist)
+      isMenu = PR_TRUE;
+  }
+
+  static DOMTimeStamp lastKeyTime = 0;
+  DOMTimeStamp keyTime;
+  aKeyEvent->GetTimeStamp(&keyTime);
+
+  if (charCode == 0) {
+    if (keyCode == NS_VK_BACK) {
+      if (!isMenu && !mIncrementalString.IsEmpty()) {
+        mIncrementalString.SetLength(mIncrementalString.Length() - 1);
+        return nsnull;
+      }
+      else {
+#ifdef XP_WIN
+        nsCOMPtr<nsISound> soundInterface = do_CreateInstance("@mozilla.org/sound;1");
+        if (soundInterface)
+          soundInterface->Beep();
+#endif  // #ifdef XP_WIN
+      }
+    }
+    return nsnull;
+  }
+  else {
+    nsAutoString pressKey(NS_STATIC_CAST(PRUnichar, charCode));
+    // UpperCase to compare-no-case
+    ToUpperCase(pressKey);
+    if (isMenu || // Menu supports only first-letter navigation
+        keyTime - lastKeyTime > INC_TYP_INTERVAL) // Interval too long, treat as new typing
+      mIncrementalString = pressKey;
+    else {
+      if (mIncrementalString.Length() != 1 ||
+          GetCharAt(mIncrementalString, 0) != GetCharAt(pressKey, 0))
+          // If user typed the same key more than once, we should do a cycled one-key navigation
+      mIncrementalString.do_AppendFromElement(GetCharAt(pressKey, 0));
+    }
+  }
+
+  lastKeyTime = keyTime;
+
   nsIFrame* currFrame;
+  // NOTE: If you crashed here due to a bogus |immediateParent| it is 
+  //       possible that the menu whose shortcut is being looked up has 
+  //       been destroyed already.  One strategy would be to 
+  //       setTimeout(<func>,0) as detailed in:
+  //       <http://bugzilla.mozilla.org/show_bug.cgi?id=126675#c32>
   immediateParent->FirstChild(mPresContext, nsnull, &currFrame);
 
+  // We start searching from first child. This process is divided into two parts
+  //   -- before current and after current -- by the current item
   while (currFrame) {
     nsCOMPtr<nsIContent> current;
     currFrame->GetContent(getter_AddRefs(current));
     
     // See if it's a menu item.
     if (IsValidItem(current)) {
+      nsAutoString activeKey, textKey;
       // Get the shortcut attribute.
-      nsAutoString shortcutKey;
-      current->GetAttr(kNameSpaceID_None, nsXULAtoms::accesskey, shortcutKey);
-      if (shortcutKey.Length() > 0) {
-        // We've got something.
-        char tempChar[2];
-        tempChar[0] = aLetter;
-        tempChar[1] = 0;
-  
-        if (shortcutKey.EqualsIgnoreCase(tempChar)) {
-          // We match!
-          nsCOMPtr<nsIMenuFrame> menuFrame = do_QueryInterface(currFrame);
-          if (menuFrame)
-            return menuFrame.get();
+      current->GetAttr(kNameSpaceID_None, nsXULAtoms::accesskey, textKey);
+      if (textKey.IsEmpty()) { // No shortcut, try first letter
+        isShortcut = PR_FALSE;
+        current->GetAttr(kNameSpaceID_None, nsXULAtoms::label, textKey);
+        if (textKey.IsEmpty()) // No label, try another attribute (value)
+          current->GetAttr(kNameSpaceID_None, nsXULAtoms::value, textKey);
+      }
+      else
+        isShortcut = PR_TRUE;
+
+      ToUpperCase(textKey);
+      if (Substring(textKey, 0, mIncrementalString.Length()) == mIncrementalString) {
+        // mIncrementalString is a prefix of textKey
+        nsCOMPtr<nsIMenuFrame> menuFrame = do_QueryInterface(currFrame);
+        if (menuFrame) {
+          // There is one match
+          matchCount++;
+          if (isShortcut) {
+            // There is one shortcut-key match
+            matchShortcutCount++;
+            // Record the matched item. If there is only one matched shortcut item, do it
+            frameShortcut = menuFrame.get();
+          }
+          if (!foundActive) {
+            // It's a first candidate item located before/on the current item
+            if (!frameBefore)
+              frameBefore = menuFrame.get();
+          }
+          else {
+            // It's a first candidate item located after the current item
+            if (!frameAfter)
+              frameAfter = menuFrame.get();
+          }
+        }
+        else
           return nsnull;
+      }
+
+      // Get the active status
+      current->GetAttr(kNameSpaceID_None, nsXULAtoms::menuactive, activeKey);
+      if (activeKey == NS_LITERAL_STRING("true")) {
+        foundActive = PR_TRUE;
+        if (mIncrementalString.Length() > 1) {
+          // If there is more than one char typed, the current item has highest priority,
+          //   otherwise the item next to current has highest priority
+          nsCOMPtr<nsIMenuFrame> menuFrame = do_QueryInterface(currFrame);
+          if (menuFrame && menuFrame.get() == frameBefore) {
+            return frameBefore;
+          }
         }
       }
     }
     currFrame->GetNextSibling(&currFrame);
   }
 
+  doAction = (isMenu && (matchCount == 1 || matchShortcutCount == 1));
+
+  if (matchShortcutCount == 1) // We have one matched shortcut item
+    return frameShortcut;
+  if (frameAfter) // If we have matched item after the current, use it
+    return frameAfter;
+  else if (frameBefore) // If we haven't, use the item before the current
+    return frameBefore;
+
+  // If we don't match anything, rollback the last typing
+  mIncrementalString.SetLength(mIncrementalString.Length() - 1);
+
   // didn't find a matching menu item
 #ifdef XP_WIN
-  // behavior on Windows - this item is in a popup, beep and do nothing else
-  nsCOMPtr<nsISound> soundInterface = do_CreateInstance("@mozilla.org/sound;1");
-  if (soundInterface)
-    soundInterface->Beep();    
+  // behavior on Windows - this item is in a menu popup off of the
+  // menu bar, so beep and do nothing else
+  if (isMenu) {
+    nsCOMPtr<nsISound> soundInterface = do_CreateInstance("@mozilla.org/sound;1");
+    if (soundInterface)
+      soundInterface->Beep();
+  }
 #endif  // #ifdef XP_WIN
 
   return nsnull;
 }
 
 NS_IMETHODIMP 
-nsMenuPopupFrame::ShortcutNavigation(PRUint32 aLetter, PRBool& aHandledFlag)
+nsMenuPopupFrame::ShortcutNavigation(nsIDOMKeyEvent* aKeyEvent, PRBool& aHandledFlag)
 {
   if (mCurrentMenu) {
     PRBool isOpen = PR_FALSE;
     mCurrentMenu->MenuIsOpen(isOpen);
     if (isOpen) {
       // No way this applies to us. Give it to our child.
-      mCurrentMenu->ShortcutNavigation(aLetter, aHandledFlag);
+      mCurrentMenu->ShortcutNavigation(aKeyEvent, aHandledFlag);
       return NS_OK;
     }
   }
 
   // This applies to us. Let's see if one of the shortcuts applies
-  nsIMenuFrame* result = FindMenuWithShortcut(aLetter);
+  PRBool action;
+  nsIMenuFrame* result = FindMenuWithShortcut(aKeyEvent, action);
   if (result) {
     // We got one!
     aHandledFlag = PR_TRUE;
     SetCurrentMenuItem(result);
-    result->Enter();
+    if (action)
+      result->Enter();
   }
 
   return NS_OK;
@@ -1550,6 +1728,8 @@ nsMenuPopupFrame::ShortcutNavigation(PRUint32 aLetter, PRBool& aHandledFlag)
 NS_IMETHODIMP
 nsMenuPopupFrame::KeyboardNavigation(PRUint32 aDirection, PRBool& aHandledFlag)
 {
+  mIncrementalString = NS_LITERAL_STRING("");
+
   // This method only gets called if we're open.
   if (!mCurrentMenu && (aDirection == NS_VK_RIGHT || aDirection == NS_VK_LEFT)) {
     // We've been opened, but we haven't had anything selected.
