@@ -48,7 +48,7 @@
 #include "nsIPopupBoxObject.h"
 #include "nsIPref.h"
 #include "nsIServiceManager.h"
-#include "nsIOutlinerView.h"
+#include "nsITreeView.h"
 #include "nsGUIEvent.h"
 #include "nsIPrivateDOMEvent.h"
 #include "nsIPresContext.h"
@@ -63,8 +63,8 @@ nsXULTooltipListener::nsXULTooltipListener()
   : mSourceNode(nsnull), mTargetNode(nsnull),
     mCurrentTooltip(nsnull),
     mMouseClientX(0), mMouseClientY(0),
-    mIsTargetOutliner(PR_FALSE), mNeedTitletip(PR_FALSE),
-    mOutlinerBox(nsnull), mLastOutlinerRow(-1)
+    mIsSourceTree(PR_FALSE), mNeedTitletip(PR_FALSE),
+    mLastTreeRow(-1)
 {
 	 NS_INIT_REFCNT();
 }
@@ -141,11 +141,10 @@ nsXULTooltipListener::MouseOut(nsIDOMEvent* aMouseEvent)
     if (tooltipNode == targetNode) {
       HideTooltip();
 
-      // reset special outliner tracking
-      if (mIsTargetOutliner) {
-        mOutlinerBox = nsnull;
-        mLastOutlinerRow = -1;
-        mLastOutlinerCol.Truncate();
+      // reset special tree tracking
+      if (mIsSourceTree) {
+        mLastTreeRow = -1;
+        mLastTreeCol.Truncate();
       }
     }
   }
@@ -175,8 +174,8 @@ nsXULTooltipListener::MouseMove(nsIDOMEvent* aMouseEvent)
   mMouseClientX = newMouseX;
   mMouseClientY = newMouseY;
 
-  if (mIsTargetOutliner)
-    CheckOutlinerBodyMove(mouseEvent);
+  if (mIsSourceTree)
+    CheckTreeBodyMove(mouseEvent);
 
   // as the mouse moves, we want to make sure we reset the timer to show it, 
   // so that the delay is from when the mouse stops moving, not when it enters
@@ -252,11 +251,11 @@ nsXULTooltipListener::Init(nsIContent* aSourceNode, nsIRootBox* aRootBox)
   mSourceNode = aSourceNode;
   AddTooltipSupport(aSourceNode);
   
-  // if the target is an outlinerchildren, we may have some special
+  // if the target is an treechildren, we may have some special
   // case handling to do
   nsCOMPtr<nsIAtom> tag;
   mSourceNode->GetTag(*getter_AddRefs(tag));
-  mIsTargetOutliner = tag == nsXULAtoms::outlinerchildren;
+  mIsSourceTree = tag == nsXULAtoms::treechildren;
 
   static PRBool prefChangeRegistered = PR_FALSE;
 
@@ -305,56 +304,39 @@ nsXULTooltipListener::RemoveTooltipSupport(nsIContent* aNode)
 }
 
 void
-nsXULTooltipListener::CheckOutlinerBodyMove(nsIDOMMouseEvent* aMouseEvent)
+nsXULTooltipListener::CheckTreeBodyMove(nsIDOMMouseEvent* aMouseEvent)
 {
-  if (!mOutlinerBox) {
-    // This will be called from MouseMove before the tooltip timer is
-    // killed so that mTargetNode isn't yet set to null
-    if (mTargetNode) {
-      nsCOMPtr<nsIDocument> doc;
-      mSourceNode->GetDocument(*getter_AddRefs(doc));
-      nsCOMPtr<nsIPresShell> shell;
-      doc->GetShellAt(0, getter_AddRefs(shell));
-      if (shell) {
-        nsIFrame* bodyFrame = nsnull;
-        shell->GetPrimaryFrameFor(mTargetNode, &bodyFrame);
-        if (bodyFrame) {
-          nsCOMPtr<nsIOutlinerBoxObject> bx(do_QueryInterface(bodyFrame));
-          mOutlinerBox = bx;
-        }
-      }
-    }
-  }
-
-  if (mOutlinerBox) {
+  nsCOMPtr<nsITreeBoxObject> obx;
+  GetSourceTreeBoxObject(getter_AddRefs(obx));
+  if (obx) {
     PRInt32 x, y;
     aMouseEvent->GetClientX(&x);
     aMouseEvent->GetClientY(&y);
     PRInt32 row;
     nsXPIDLString colId, obj;
 
-    mOutlinerBox->GetCellAt(x, y, &row, getter_Copies(colId), getter_Copies(obj));
+    obx->GetCellAt(x, y, &row, getter_Copies(colId), getter_Copies(obj));
     
     // determine if we are going to need a titletip
-    // XXX check the disabletitletips attribute on the outliner content
+    // XXX check the disabletitletips attribute on the tree content
     mNeedTitletip = PR_FALSE;
 #ifdef DEBUG_crap
     if (row >= 0 && obj.Equals(NS_LITERAL_STRING("text"))) {
-      nsCOMPtr<nsIOutlinerView> view;
-      mOutlinerBox->GetView(getter_AddRefs(view));
+      nsCOMPtr<nsITreeView> view;
+      obx->GetView(getter_AddRefs(view));
       PRBool isCropped;
-      mOutlinerBox->IsCellCropped(row, colId, &isCropped);
+      obx->IsCellCropped(row, colId, &isCropped);
       mNeedTitletip = isCropped;
     }
 #endif
 
     if (mCurrentTooltip && 
-        (row != mLastOutlinerRow || !mLastOutlinerCol.Equals(colId))) {
+        (row != mLastTreeRow || !mLastTreeCol.Equals(colId))) {
       HideTooltip();
     } 
 
-    mLastOutlinerRow = row;
-    mLastOutlinerCol.Assign(colId);
+    mLastTreeRow = row;
+    mLastTreeCol.Assign(colId);
   }
 }
 
@@ -376,14 +358,13 @@ nsXULTooltipListener::ShowTooltip()
     nsCOMPtr<nsIDocument> targetDoc;
     mSourceNode->GetDocument(*getter_AddRefs(targetDoc));
     if (targetDoc) {
-      if (!mIsTargetOutliner) {
-        mLastOutlinerRow = -1;
-        mLastOutlinerCol.Truncate();
+      if (!mIsSourceTree) {
+        mLastTreeRow = -1;
+        mLastTreeCol.Truncate();
       }
 
       nsCOMPtr<nsIDOMNode> targetNode(do_QueryInterface(mTargetNode));
       xulDoc->SetTooltipNode(targetNode);
-      xulDoc->SetPopupNode(targetNode);
       LaunchTooltip(mSourceNode, mMouseClientX, mMouseClientY);
       mTargetNode = nsnull;
 
@@ -391,6 +372,8 @@ nsXULTooltipListener::ShowTooltip()
       // the tooltip. If there is an attribute on the popup telling us
       // not to create the auto-hide timer, don't.
       nsCOMPtr<nsIDOMElement> tooltipEl(do_QueryInterface(mCurrentTooltip));
+      if (!tooltipEl)
+        return NS_ERROR_FAILURE;
       nsAutoString noAutoHide;
       tooltipEl->GetAttribute(NS_LITERAL_STRING("noautohide"), noAutoHide);
       if (noAutoHide != NS_LITERAL_STRING("true"))
@@ -421,12 +404,12 @@ nsXULTooltipListener::ShowTooltip()
 }
 
 static void
-GetOutlinerCellCoords(nsIOutlinerBoxObject* aOutlinerBox, nsIContent* aSourceNode, 
-                      PRInt32 aRow, nsAutoString aCol, PRInt32* aX, PRInt32* aY)
+GetTreeCellCoords(nsITreeBoxObject* aTreeBox, nsIContent* aSourceNode, 
+                  PRInt32 aRow, nsAutoString aCol, PRInt32* aX, PRInt32* aY)
 {
   PRInt32 junk;
   const PRUnichar empty[] = {'\0'};
-  aOutlinerBox->GetCoordsForCellItem(aRow, aCol.get(), empty, aX, aY, &junk, &junk);
+  aTreeBox->GetCoordsForCellItem(aRow, aCol.get(), empty, aX, aY, &junk, &junk);
   nsCOMPtr<nsIDOMXULElement> xulEl(do_QueryInterface(aSourceNode));
   nsCOMPtr<nsIBoxObject> bx;
   xulEl->GetBoxObject(getter_AddRefs(bx));
@@ -438,11 +421,11 @@ GetOutlinerCellCoords(nsIOutlinerBoxObject* aOutlinerBox, nsIContent* aSourceNod
 }
 
 static void
-SetTitletipLabel(nsIOutlinerBoxObject* aOutlinerBox, nsIContent* aTooltip,
+SetTitletipLabel(nsITreeBoxObject* aTreeBox, nsIContent* aTooltip,
                  PRInt32 aRow, nsAutoString aCol)
 {
-  nsCOMPtr<nsIOutlinerView> view;
-  aOutlinerBox->GetView(getter_AddRefs(view));
+  nsCOMPtr<nsITreeView> view;
+  aTreeBox->GetView(getter_AddRefs(view));
 
   nsAutoString label;
   view->GetCellText(aRow, aCol.get(), label);
@@ -471,10 +454,12 @@ nsXULTooltipListener::LaunchTooltip(nsIContent* aTarget, PRInt32 aX, PRInt32 aY)
     PRInt32 x = aX;
     PRInt32 y = aY;
     if (mNeedTitletip) {
-      GetOutlinerCellCoords(mOutlinerBox, mSourceNode,
-                            mLastOutlinerRow, mLastOutlinerCol, &x, &y);
+      nsCOMPtr<nsITreeBoxObject> obx;
+      GetSourceTreeBoxObject(getter_AddRefs(obx));
+      GetTreeCellCoords(obx, mSourceNode,
+                            mLastTreeRow, mLastTreeCol, &x, &y);
 
-      SetTitletipLabel(mOutlinerBox, mCurrentTooltip, mLastOutlinerRow, mLastOutlinerCol);
+      SetTitletipLabel(obx, mCurrentTooltip, mLastTreeRow, mLastTreeCol);
       mCurrentTooltip->SetAttr(nsnull, nsXULAtoms::titletip, NS_LITERAL_STRING("true"), PR_FALSE);
     } else
       mCurrentTooltip->UnsetAttr(nsnull, nsXULAtoms::titletip, PR_FALSE);
@@ -594,7 +579,7 @@ nsXULTooltipListener::GetTooltipFor(nsIContent* aTarget, nsIContent** aTooltip)
         }
 
         // titletips should just use the default tooltip
-        if (mIsTargetOutliner && mNeedTitletip) {
+        if (mIsSourceTree && mNeedTitletip) {
           mRootBox->GetDefaultTooltip(aTooltip);
           NS_IF_ADDREF(*aTooltip);
           return NS_OK;
@@ -615,10 +600,8 @@ nsXULTooltipListener::DestroyTooltip()
     mCurrentTooltip->GetDocument(*getter_AddRefs(doc));
     if (doc) {
       nsCOMPtr<nsIDOMXULDocument> xulDoc(do_QueryInterface(doc));
-      if (xulDoc) {
+      if (xulDoc)
         xulDoc->SetTooltipNode(nsnull);
-        xulDoc->SetPopupNode(nsnull);
-      }
 
       // remove the mousedown and keydown listener from document
       nsCOMPtr<nsIDOMEventTarget> evtTarget(do_QueryInterface(doc));
@@ -629,9 +612,12 @@ nsXULTooltipListener::DestroyTooltip()
 
     // remove the popuphidden listener from tooltip
     nsCOMPtr<nsIDOMEventTarget> evtTarget(do_QueryInterface(mCurrentTooltip));
-    evtTarget->RemoveEventListener(NS_LITERAL_STRING("popuphiding"), (nsIDOMMouseListener*)this, PR_FALSE);
 
+    // release tooltip before removing listener to prevent our destructor from
+    // being called recursively (bug 120863)
     mCurrentTooltip = nsnull;
+
+    evtTarget->RemoveEventListener(NS_LITERAL_STRING("popuphiding"), (nsIDOMMouseListener*)this, PR_FALSE);
   }
   
   // kill any ongoing timers
@@ -692,3 +678,26 @@ nsXULTooltipListener::sTooltipPrefChanged(const char* aPref, void* aData)
 
   return NS_OK;
 }
+
+nsresult
+nsXULTooltipListener::GetSourceTreeBoxObject(nsITreeBoxObject** aBoxObject)
+{
+  *aBoxObject = nsnull;
+
+  if (mIsSourceTree && mSourceNode) {
+    nsCOMPtr<nsIContent> treeParent;
+    mSourceNode->GetParent(*getter_AddRefs(treeParent));
+    nsCOMPtr<nsIDOMXULElement> xulEl(do_QueryInterface(treeParent));
+    if (xulEl) {
+      nsCOMPtr<nsIBoxObject> bx;
+      xulEl->GetBoxObject(getter_AddRefs(bx));
+      nsCOMPtr<nsITreeBoxObject> obx(do_QueryInterface(bx));
+      if (obx) {
+        *aBoxObject = obx;
+        NS_ADDREF(*aBoxObject);
+        return NS_OK;
+      }
+    }
+  }
+  return NS_ERROR_FAILURE;
+}
