@@ -67,7 +67,6 @@
 #include "nsIWindowWatcher.h"
 
 static NS_DEFINE_CID(kCAbSyncPostEngineCID, NS_ABSYNC_POST_ENGINE_CID); 
-static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
 static NS_DEFINE_CID(kAddrBookSessionCID, NS_ADDRBOOKSESSION_CID);
 static NS_DEFINE_CID(kAddressBookDBCID, NS_ADDRDATABASE_CID);
 static NS_DEFINE_CID(kRDFServiceCID,  NS_RDFSERVICE_CID);
@@ -626,7 +625,7 @@ NS_IMETHODIMP nsAbSync::PerformAbSync(nsIDOMWindowInternal *aDOMWindow, PRInt32
   // this request as well as the local address book we will be 
   // syncing with...
   //
-  nsCOMPtr<nsIPref> prefs(do_GetService(kPrefCID, &rv)); 
+  nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv)); 
   NS_ENSURE_SUCCESS(rv, rv);
 
   prefs->CopyCharPref("mail.absync.address_book",     &mAbSyncAddressBook);
@@ -795,7 +794,10 @@ nsAbSync::GenerateProtocolForCard(nsIAbCard *aCard, PRBool aAddId, nsString &pro
     if (NS_SUCCEEDED(aCard->GetCardValue(mSchemaMappingList[i].abField, &aName)) && (aName) && (*aName))
     {
       // These are unknown tags we are omitting...
-      if (!nsCRT::strncasecmp(mSchemaMappingList[i].serverField, "OMIT:", 5))
+      const nsAString& prefix =
+        Substring(mSchemaMappingList[i].serverField, 0, 5);
+      if (NS_LITERAL_STRING("OMIT:").Equals(prefix,
+                                            nsCaseInsensitiveStringComparator()))
         continue;
 
       // Reset this flag...
@@ -834,26 +836,12 @@ nsAbSync::GenerateProtocolForCard(nsIAbCard *aCard, PRBool aAddId, nsString &pro
         char *pVal = PR_smprintf("phone%d", phoneCount);
         if (pVal)
         {
-          char *utfString = ToNewUTF8String(nsDependentString(aName));
-
-          // Now, URL Encode the value string....
-          char *myTStr = nsEscape(utfString, url_Path);
-          if (myTStr)
-          {
-            PR_FREEIF(utfString);
-            utfString = myTStr;
-          }
-
-          tProtLine.Append(NS_LITERAL_STRING("&") + 
+           tProtLine.Append(NS_LITERAL_STRING("&") + 
                            NS_ConvertASCIItoUCS2(pVal) +
                            NS_LITERAL_STRING("="));
-          if (utfString)
-          {
-            tProtLine.Append(NS_ConvertASCIItoUCS2(utfString));
-            PR_FREEIF(utfString);
-          }
-          else
-            tProtLine.Append(aName);
+
+          AddValueToProtocolLine(aName, tProtLine);
+
           tProtLine.Append(NS_LITERAL_STRING("&") + 
                            NS_ConvertASCIItoUCS2(pVal) + 
                            NS_LITERAL_STRING("_type=") +
@@ -864,26 +852,10 @@ nsAbSync::GenerateProtocolForCard(nsIAbCard *aCard, PRBool aAddId, nsString &pro
       }
       else    // Good ole' normal tag...
       {
-        char *utfString = ToNewUTF8String(nsDependentString(aName));
-
-        // Now, URL Encode the value string....
-        char *myTStr = nsEscape(utfString, url_Path);
-        if (myTStr)
-        {
-          PR_FREEIF(utfString);
-          utfString = myTStr;
-        }
-
         tProtLine.Append(NS_LITERAL_STRING("&") + 
-                         NS_ConvertASCIItoUCS2(mSchemaMappingList[i].serverField) +
+                         mSchemaMappingList[i].serverField +
                          NS_LITERAL_STRING("="));
-        if (utfString)
-        {
-          tProtLine.Append(NS_ConvertASCIItoUCS2(utfString));
-          PR_FREEIF(utfString);
-        }
-        else
-          tProtLine.Append(aName);
+        AddValueToProtocolLine(aName, tProtLine);
       }
 
       PR_FREEIF(aName);
@@ -905,31 +877,49 @@ nsAbSync::GenerateProtocolForCard(nsIAbCard *aCard, PRBool aAddId, nsString &pro
       // Just some sanity...
       if (aName)
       {
-        char *utfString = ToNewUTF8String(nsDependentString(aName));
-      
-        // Now, URL Encode the value string....
-        char *myTStr = nsEscape(utfString, url_Path);
-        if (myTStr)
-        {
-          PR_FREEIF(utfString);
-          utfString = myTStr;
-        }
-      
         tProtLine.Append(NS_LITERAL_STRING("&") + 
-                         NS_ConvertASCIItoUCS2(kServerPlainTextColumn) +
+                         kServerPlainTextColumn +
                          NS_LITERAL_STRING("="));
-        if (utfString)
-        {
-          tProtLine.Append(NS_ConvertASCIItoUCS2(utfString));
-          PR_FREEIF(utfString);
-        }
-        else
-          tProtLine.Append(aName);
+        AddValueToProtocolLine(aName, tProtLine);
       
         PR_FREEIF(aName);
       }
     }
 
+    // If the aim screen name is present then add it to the protocol string.
+    nsresult rv;
+    nsCOMPtr<nsIAbMDBCard> dbcard(do_QueryInterface(aCard, &rv));
+    if (NS_SUCCEEDED(rv) && dbcard)
+    {
+      nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv)); 
+      if (NS_SUCCEEDED(rv))
+      {
+        PRUint32 i, prefCount, prefixLen=strlen(SYNC_PREF_PREFIX_CLIENT_MAP);
+        char** prefNames;
+
+        rv = prefs->GetChildList(SYNC_PREF_PREFIX_CLIENT_MAP, &prefCount, &prefNames);
+        if (NS_SUCCEEDED(rv) && prefCount > 0)
+          for (i = 0; i < prefCount; i++)
+          {
+            nsXPIDLString genericValue;
+            rv = dbcard->GetStringAttribute(prefNames[i]+prefixLen, getter_Copies(genericValue));
+            if (NS_SUCCEEDED(rv) && genericValue.Length())
+            {
+              nsXPIDLCString mapValue;
+              rv = prefs->CopyCharPref(prefNames[i], getter_Copies(mapValue));
+              if (NS_SUCCEEDED(rv) && mapValue.Length())
+              {
+                // Add name tag and value to the protocol string.
+                tProtLine.Append(NS_LITERAL_STRING("&"));
+                tProtLine.AppendWithConversion(mapValue);
+                tProtLine.Append(NS_LITERAL_STRING("="));
+                AddValueToProtocolLine(genericValue.get(), tProtLine);
+              }
+            }
+          } // end of for (i = 0; i < prefCount; i++)
+       }
+    }
+
     char *tLine = ToNewCString(tProtLine);
     if (!tLine)
       return NS_ERROR_OUT_OF_MEMORY;
@@ -948,6 +938,27 @@ nsAbSync::GenerateProtocolForCard(nsIAbCard *aCard, PRBool aAddId, nsString &pro
   return NS_OK;
 }
 
+nsresult nsAbSync::AddValueToProtocolLine(const PRUnichar *value, nsString &protocolLine)
+{
+  // Escape the value string.
+  char *utfString = ToNewUTF8String(nsDependentString(value));
+  char *escapedStr = nsEscape(utfString, url_Path);
+  if (escapedStr)
+  {
+    PR_FREEIF(utfString);
+    utfString = escapedStr;
+  }
+  if (utfString)
+  {
+    protocolLine.Append(NS_ConvertASCIItoUCS2(utfString));
+    PR_FREEIF(utfString);
+  }
+  else
+    protocolLine.Append(value);
+
+  return NS_OK;
+}
+
 long
 GetCRC(char *str)
 {
@@ -1164,19 +1175,15 @@ nsAbSync::AnalyzeAllRecords(nsIAddrDatabase *aDatabase, nsIAbDirectory *director
   
   rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(historyFile));
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = historyFile->Append("absync.dat");
+  rv = historyFile->AppendNative(NS_LITERAL_CSTRING("absync.dat"));
   NS_ENSURE_SUCCESS(rv, rv);
 
   // TODO: Convert the rest of the code to use
   // nsIFile and avoid this conversion hack.
   do
   {
-    nsXPIDLCString pathBuf;
-    rv = historyFile->GetPath(getter_Copies(pathBuf));
+    rv = NS_NewFileSpecFromIFile(historyFile, getter_AddRefs(mHistoryFile));
     if (NS_FAILED(rv)) break;
-    rv = NS_NewFileSpec(getter_AddRefs(mHistoryFile));
-    if (NS_FAILED(rv)) break;
-    rv = mHistoryFile->SetNativePath(pathBuf);
   }
   while (0);
 
@@ -1216,7 +1223,7 @@ nsAbSync::AnalyzeAllRecords(nsIAddrDatabase *aDatabase, nsIAbDirectory *director
   rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(lockFile));
   if (NS_SUCCEEDED(rv)) 
   {
-    rv = lockFile->Append((const char *)"absync.lck");
+    rv = lockFile->AppendNative(NS_LITERAL_CSTRING("absync.lck"));
     if (NS_SUCCEEDED(rv)) 
     {
     
@@ -1224,12 +1231,8 @@ nsAbSync::AnalyzeAllRecords(nsIAddrDatabase *aDatabase, nsIAbDirectory *director
       // nsIFile and avoid this conversion hack.
       do
       {
-        nsXPIDLCString pathBuf;
-        rv = lockFile->GetPath(getter_Copies(pathBuf));
+        rv = NS_NewFileSpecFromIFile(lockFile, getter_AddRefs(mLockFile));
         if (NS_FAILED(rv)) break;
-        rv = NS_NewFileSpec(getter_AddRefs(mLockFile));
-        if (NS_FAILED(rv)) break;
-        rv = mLockFile->SetNativePath(pathBuf);
       }
       while (0);
       
@@ -2195,10 +2198,16 @@ nsAbSync::ProcessServerResponse(const char *aProtocolResponse)
 
   if (ErrorFromServer(&errorString))
   {
-    PRUnichar   *outValue = GetString(NS_LITERAL_STRING("syncServerError").get());
-    PRUnichar   *msgValue;
+    PRUnichar   *msgValue, *outValue=nsnull;
 
+    // Covert server exceed max record msg here.
+    if (! nsCRT::strncasecmp(errorString, SYNC_ERROR_EXCEED_MAX_RECORD, nsCRT::strlen(SYNC_ERROR_EXCEED_MAX_RECORD)))
+      msgValue = GetString(NS_LITERAL_STRING("exceedMaxRecordError").get());
+    else
+    {
+      outValue = GetString(NS_LITERAL_STRING("syncServerError").get());
     msgValue = nsTextFormatter::smprintf(outValue, errorString);
+    }
 
     DisplayErrorMessage(msgValue);
 
@@ -2274,7 +2283,7 @@ nsAbSync::ProcessServerResponse(const char *aProtocolResponse)
 ExitEarly:
   if (mLastChangeNum > 1)
   {
-    nsCOMPtr<nsIPref> prefs(do_GetService(kPrefCID, &rv)); 
+    nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv)); 
     if (NS_SUCCEEDED(rv) && prefs) 
     {
       prefs->SetIntPref("mail.absync.last_change", mLastChangeNum);
@@ -2735,12 +2744,47 @@ nsAbSync::AddNewUsers()
         rv = aDatabase->EditCard(newCard, PR_TRUE);
       else
       {
+        // hack
+        // there is no way (currently) to set generic attributes on simple abcardproperties
+        // so we have to use a mdbcard instead. We can remove this once #128567 is fixed.
+        nsCOMPtr<nsIAbMDBCard> dbcard = do_CreateInstance(NS_ABMDBCARD_CONTRACTID, &rv);
+        if (NS_FAILED(rv))
+          continue;
+        
+        nsCOMPtr<nsIAbCard> card = do_QueryInterface(dbcard, &rv);
+        if (NS_FAILED(rv))
+          continue;
+        
+        rv = card->Copy(newCard);
+        if (NS_FAILED(rv))
+          continue;
+        
+        dbcard->SetAbDatabase(aDatabase);
         PRUint32  tID;
-        rv = aDatabase->CreateNewCardAndAddToDBWithKey(newCard, PR_TRUE, &tID);
+        // card has to be a mdbcard, or tID will come back as 0
+        rv = aDatabase->CreateNewCardAndAddToDBWithKey(card, PR_TRUE, &tID);
         localID = (PRInt32) tID;
+        newCard = card;
       }
     }
 
+    // hack
+    // we only support one generic column/value and we need to do this
+    // AFTER we have a mdbcard. We can remove this once #128567 is fixed.
+    if (!mCurrentGenericColumn.IsEmpty())
+    {
+      nsCOMPtr <nsIAbMDBCard> dbcard = do_QueryInterface(newCard, &rv);
+      if (NS_FAILED(rv))
+          continue;
+
+      rv = dbcard->SetStringAttribute(mCurrentGenericColumn.get(), mCurrentGenericValue.get());
+      if (NS_FAILED(rv))
+          continue;
+
+      mCurrentGenericColumn.Truncate();
+      mCurrentGenericValue.Truncate();
+    }
+
     //
     // Now, calculate the NEW CRC for the new or updated card...
     //
@@ -2939,15 +2983,15 @@ nsAbSync::AddValueToNewCard(nsIAbCard *aCard, nsString *aTagName, nsString *aTag
   // Ok, we need to figure out what the tag name from the server maps to and assign
   // this value the new nsIAbCard
   //
-  if (!aTagName->CompareWithConversion(kServerFirstNameColumn))
+  if (aTagName->Equals(kServerFirstNameColumn))
     aCard->SetFirstName(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerLastNameColumn))
+  else if (aTagName->Equals(kServerLastNameColumn))
     aCard->SetLastName(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerDisplayNameColumn))
+  else if (aTagName->Equals(kServerDisplayNameColumn))
     aCard->SetDisplayName(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerNicknameColumn))
+  else if (aTagName->Equals(kServerNicknameColumn))
     aCard->SetNickName(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerPriEmailColumn))
+  else if (aTagName->Equals(kServerPriEmailColumn))
   {
 #ifdef DEBUG_rhp
   char *t = ToNewCString(*aTagValue);
@@ -2956,53 +3000,53 @@ nsAbSync::AddValueToNewCard(nsIAbCard *aCard, nsString *aTagName, nsString *aTag
 #endif
     aCard->SetPrimaryEmail(aTagValue->get());
   }
-  else if (!aTagName->CompareWithConversion(kServer2ndEmailColumn))
+  else if (aTagName->Equals(kServer2ndEmailColumn))
     aCard->SetSecondEmail(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerHomeAddressColumn))
+  else if (aTagName->Equals(kServerHomeAddressColumn))
     aCard->SetHomeAddress(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerHomeAddress2Column))
+  else if (aTagName->Equals(kServerHomeAddress2Column))
     aCard->SetHomeAddress2(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerHomeCityColumn))
+  else if (aTagName->Equals(kServerHomeCityColumn))
     aCard->SetHomeCity(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerHomeStateColumn))
+  else if (aTagName->Equals(kServerHomeStateColumn))
     aCard->SetHomeState(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerHomeZipCodeColumn))
+  else if (aTagName->Equals(kServerHomeZipCodeColumn))
     aCard->SetHomeZipCode(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerHomeCountryColumn))
+  else if (aTagName->Equals(kServerHomeCountryColumn))
     aCard->SetHomeCountry(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerWorkAddressColumn))
+  else if (aTagName->Equals(kServerWorkAddressColumn))
     aCard->SetWorkAddress(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerWorkAddress2Column))
+  else if (aTagName->Equals(kServerWorkAddress2Column))
     aCard->SetWorkAddress2(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerWorkCityColumn))
+  else if (aTagName->Equals(kServerWorkCityColumn))
     aCard->SetWorkCity(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerWorkStateColumn))
+  else if (aTagName->Equals(kServerWorkStateColumn))
     aCard->SetWorkState(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerWorkZipCodeColumn))
+  else if (aTagName->Equals(kServerWorkZipCodeColumn))
     aCard->SetWorkZipCode(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerWorkCountryColumn))
+  else if (aTagName->Equals(kServerWorkCountryColumn))
     aCard->SetWorkCountry(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerJobTitleColumn))
+  else if (aTagName->Equals(kServerJobTitleColumn))
     aCard->SetJobTitle(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerNotesColumn))
+  else if (aTagName->Equals(kServerNotesColumn))
     aCard->SetNotes(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerWebPage1Column))
+  else if (aTagName->Equals(kServerWebPage1Column))
     aCard->SetWebPage1(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerDepartmentColumn))
+  else if (aTagName->Equals(kServerDepartmentColumn))
     aCard->SetDepartment(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerCompanyColumn))
+  else if (aTagName->Equals(kServerCompanyColumn))
     aCard->SetCompany(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerWebPage2Column))
+  else if (aTagName->Equals(kServerWebPage2Column))
     aCard->SetWebPage2(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerCustom1Column))
+  else if (aTagName->Equals(kServerCustom1Column))
     aCard->SetCustom1(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerCustom2Column))
+  else if (aTagName->Equals(kServerCustom2Column))
     aCard->SetCustom2(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerCustom3Column))
+  else if (aTagName->Equals(kServerCustom3Column))
     aCard->SetCustom3(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerCustom4Column))
+  else if (aTagName->Equals(kServerCustom4Column))
     aCard->SetCustom4(aTagValue->get());
-  else if (!aTagName->CompareWithConversion(kServerPlainTextColumn))
+  else if (aTagName->Equals(kServerPlainTextColumn))
   {
     // This is plain text pref...have to add a little logic.
     if (aTagValue->Equals(NS_LITERAL_STRING("1")))
@@ -3010,6 +3054,29 @@ nsAbSync::AddValueToNewCard(nsIAbCard *aCard, nsString *aTagName, nsString *aTag
     else if (aTagValue->Equals(NS_LITERAL_STRING("0")))
       aCard->SetPreferMailFormat(nsIAbPreferMailFormat::unknown);
   }
+  else if (aTagName->Length() && aTagValue->Length())
+  {
+    // See if it's a generic tag defined by external source (like netscape).
+    nsCAutoString prefName;
+    nsXPIDLCString mapValue;
+
+    nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv)); 
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    prefName = SYNC_PREF_PREFIX_SERVER_MAP;
+    prefName += NS_LossyConvertUCS2toASCII((*aTagName).get());
+
+    rv = prefs->CopyCharPref(prefName.get(), getter_Copies(mapValue));
+    if (NS_SUCCEEDED(rv) && mapValue.Length())
+    {
+      // OK, this generic column is predefined so remember it.
+      // Note: we only support one generic column for now and
+      // this code can be replaced by what's provided by #128567
+      // to set generic column/value pairs once #128567 is fixed.
+      mCurrentGenericColumn = mapValue;
+      mCurrentGenericValue = aTagValue->get();
+    }
+  }
 
   return rv;
 }
