@@ -60,6 +60,7 @@ var messagesBox = null;
 var accountCentralBox = null;
 var gSearchBox = null;
 var gAccountCentralLoaded = false;
+var gFakeAccountPageLoaded = false;
 var gPaneConfig = null;
 //End progress and Status variables
 
@@ -247,8 +248,7 @@ nsMsgStatusFeedback.prototype =
   statusTextFld : null,
   statusBar     : null,
   throbber      : null,
-  stopMenu      : null,
-  stopButton    : null,
+  stopCmd       : null,
   startTimeoutID : null,
   stopTimeoutID  : null,
   pendingStartRequests : 0,
@@ -259,8 +259,7 @@ nsMsgStatusFeedback.prototype =
       if (!this.statusTextFld ) this.statusTextFld = document.getElementById("statusText");
       if (!this.statusBar) this.statusBar = document.getElementById("statusbar-icon");
       if(!this.throbber)   this.throbber = document.getElementById("navigator-throbber");
-      if(!this.stopButton) this.stopButton = document.getElementById("button-stop");
-      if(!this.stopMenu)   this.stopMenu = document.getElementById("stopMenuitem");
+      if(!this.stopCmd)   this.stopCmd = document.getElementById("cmd_stop");
     },
 
   // nsIXULBrowserWindow implementation
@@ -304,8 +303,7 @@ nsMsgStatusFeedback.prototype =
       this.throbber.setAttribute("busy", true);
 
       //turn on stop button and menu
-    this.stopButton.setAttribute("disabled", false);
-    this.stopMenu.setAttribute("disabled", false);
+    this.stopCmd.removeAttribute("disabled");
 
       // Remember when loading commenced.
       this.startTime = (new Date()).getTime();
@@ -347,8 +345,7 @@ nsMsgStatusFeedback.prototype =
       this.statusBar.setAttribute("mode","normal");
       this.statusBar.value = 0;  // be sure to clear the progress bar
       this.statusBar.label = "";
-      this.stopButton.setAttribute("disabled", true);
-      this.stopMenu.setAttribute("disabled", true);
+      this.stopCmd.setAttribute("disabled", "true");
 
       this.meteorsSpinning = false;
       this.stopTimeoutID = null;
@@ -514,6 +511,7 @@ function HideAccountCentral()
         switch (gPaneConfig)
         {
             case 0:
+                window.frames["accountCentralPane"].location = "about:blank";
                 accountCentralBox.setAttribute("collapsed", "true");
                 gSearchBox.removeAttribute("collapsed");
                 messagesBox.removeAttribute("collapsed");
@@ -521,6 +519,7 @@ function HideAccountCentral()
                 break;
 
             case 1:
+                window.frames["accountCentralPane"].location = "about:blank";
                 accountCentralBox.setAttribute("collapsed", "true");
                 var messagePaneBox = document.getElementById("messagepanebox");
                 messagePaneBox.removeAttribute("collapsed");
