@@ -55,6 +55,7 @@
 #include "nsDirectoryServiceDefs.h"
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsIFileSpec.h" 
+#include "nsILocalFile.h"
 #include "nsIURL.h"
 #include "nsNetCID.h"
 #include "nsIStringBundle.h"
@@ -185,6 +186,10 @@ static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
 #define ESCAPE_USER_NAME(outName,inName) \
     *((char **)getter_Copies(outName)) = nsEscape((const char *)inName, url_XAlphas);
 
+#define ESCAPE_FOLDER_NAME(outName,inName) \
+    *((char **)getter_Copies(outName)) = nsEscape((const char *)inName, url_Path);
+
+
 #define CONVERT_4X_URI(IDENTITY,FOR_NEWS,USERNAME,HOSTNAME,DEFAULT_FOLDER_NAME,MACRO_GETTER,MACRO_SETTER,DEFAULT_PREF) \
 { \
   nsXPIDLCString macro_oldStr; \
@@ -347,10 +352,7 @@ static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
 NS_IMPL_ISUPPORTS2(nsMessengerMigrator, nsIMessengerMigrator, nsIObserver)
 
 nsMessengerMigrator::nsMessengerMigrator() :
-  m_haveShutdown(PR_FALSE),
-  m_oldMailType(-1),
-  m_alreadySetNntpDefaultLocalPath(PR_FALSE),
-  m_alreadySetImapDefaultLocalPath(PR_FALSE)
+  m_haveShutdown(PR_FALSE)
 {
 
   NS_INIT_REFCNT();
@@ -390,7 +392,7 @@ nsresult nsMessengerMigrator::Init()
   rv = getPrefService();
   if (NS_FAILED(rv)) return rv;   
 
-  rv = m_prefs->GetIntPref(PREF_4X_MAIL_SERVER_TYPE, &m_oldMailType);
+  rv = ResetState();
   return rv;
 }
 
@@ -558,14 +560,8 @@ nsMessengerMigrator::CreateLocalMailAccount(PRBool migrating)
   // Convert the nsILocalFile into an nsIFileSpec
   // TODO: convert users os nsIFileSpec to nsILocalFile
   // and avoid this step.
-  nsXPIDLCString mailDirPath;
-  rv = mailDir->GetPath(getter_Copies(mailDirPath));
+  rv = NS_NewFileSpecFromIFile(mailDir, getter_AddRefs(mailDirSpec));
   if (NS_FAILED(rv)) return rv;
-  rv = NS_NewFileSpec(getter_AddRefs(mailDirSpec));
-  if (NS_FAILED(rv)) return rv;
-  rv = mailDirSpec->SetNativePath(mailDirPath);
-  if (NS_FAILED(rv)) return rv;
-
 
   // set the default local path for "none"
   rv = server->SetDefaultLocalPath(mailDirSpec);
@@ -608,6 +604,20 @@ nsMessengerMigrator::CreateLocalMailAccount(PRBool migrating)
   return NS_OK;
 }
 
+nsresult
+nsMessengerMigrator::ResetState()
+{
+  m_alreadySetNntpDefaultLocalPath = PR_FALSE;
+  m_alreadySetImapDefaultLocalPath = PR_FALSE;
+
+  // Reset 'm_oldMailType' in case the prefs file has changed. This is possible in quick launch
+  // mode where the profile to be migrated is IMAP type but the current working profile is POP.
+  nsresult rv = m_prefs->GetIntPref(PREF_4X_MAIL_SERVER_TYPE, &m_oldMailType);
+  if (NS_FAILED(rv))
+    m_oldMailType = -1;
+  return rv;
+}
+
 NS_IMETHODIMP
 nsMessengerMigrator::UpgradePrefs()
 {
@@ -616,6 +626,9 @@ nsMessengerMigrator::UpgradePrefs()
     rv = getPrefService();
     if (NS_FAILED(rv)) return rv;
 
+    // Reset some control vars, necessary in turbo mode.
+    ResetState();
+
     // because mail.server_type defaults to 0 (pop) it will look the user
     // has something to migrate, even with an empty prefs.js file
     // ProceedWithMigration will check if there is something to migrate
@@ -967,10 +980,19 @@ nsMessengerMigrator::Convert4XUri(const char *old_uri, PRBool for_news, const ch
     }
     else {
 		// IMAP uri's began with "IMAP:/".  we need that to be "imap:/"
+    // 4.x escapes imap uri in case folder names contain spaces. For example:
+    //   "imap://qatest20@nsmail-2/Folders%2FJan%20Sent"
+    // and it needs to be converted to:
+    //   "imap://qatest20@nsmail-2/Folders/Jan Sent"
+    // ie, we need to unescape the folder names.
 #ifdef DEBUG_MIGRATOR
 		printf("new_uri = %s%s\n",IMAP_SCHEMA,old_uri+IMAP_SCHEMA_LENGTH);
 #endif /* DEBUG_MIGRATOR */
-		*new_uri = PR_smprintf("%s%s",IMAP_SCHEMA,old_uri+IMAP_SCHEMA_LENGTH);
+
+    char *unescaped_uri = nsCRT::strdup(old_uri);
+    nsUnescape(unescaped_uri);
+    *new_uri = PR_smprintf("%s%s",IMAP_SCHEMA,unescaped_uri+IMAP_SCHEMA_LENGTH);
+    nsCRT::free(unescaped_uri);
 		return NS_OK;
 	}
   }
@@ -1009,13 +1031,18 @@ nsMessengerMigrator::Convert4XUri(const char *old_uri, PRBool for_news, const ch
     rv = m_prefs->CopyCharPref(PREF_4X_NETWORK_HOSTS_POP_SERVER, getter_Copies(pop_hostname));
     if (NS_FAILED(rv)) return rv;
 
-    nsXPIDLCString escaped_pop_username;
+    // Need to escape hostname in case it contains spaces.
+    nsXPIDLCString escaped_pop_username, escaped_pop_hostname;
+    ESCAPE_USER_NAME(escaped_pop_hostname, pop_hostname);
     ESCAPE_USER_NAME(escaped_pop_username,pop_username);
 
-    usernameAtHostname = PR_smprintf("%s@%s",(const char *)escaped_pop_username, (const char *)pop_hostname);
+    usernameAtHostname = PR_smprintf("%s@%s", escaped_pop_username.get(), escaped_pop_hostname.get());
   }
   else if (m_oldMailType == IMAP_4X_MAIL_TYPE) {
-    usernameAtHostname = PR_smprintf("%s@%s",LOCAL_MAIL_FAKE_USER_NAME,mLocalFoldersHostname.get());
+    // Need to escape hostname in case it contains spaces.
+    nsXPIDLCString escaped_localfolder_hostname;
+    ESCAPE_USER_NAME(escaped_localfolder_hostname,mLocalFoldersHostname.get());
+    usernameAtHostname = PR_smprintf("%s@%s",LOCAL_MAIL_FAKE_USER_NAME, escaped_localfolder_hostname.get());
   }
 #ifdef HAVE_MOVEMAIL
   else if (m_oldMailType == MOVEMAIL_4X_MAIL_TYPE) {
@@ -1061,14 +1088,26 @@ nsMessengerMigrator::Convert4XUri(const char *old_uri, PRBool for_news, const ch
   // and the directory pref was <foobar>
   // this meant it was reall <foobar>/<default folder name>
   // this insanity only happened on mac and windows.
+  // Need to escape spaces in folder name (ie, "A Folder" --> "A%20Folder").
   if (!folderPath || (PL_strlen(folderPath) == 0)) {
-	*new_uri = PR_smprintf("%s/%s/%s",MAILBOX_SCHEMA,usernameAtHostname, default_folder_name);
+    nsXPIDLCString escaped_default_folder;
+    ESCAPE_FOLDER_NAME(escaped_default_folder, default_folder_name);
+    *new_uri = PR_smprintf("%s/%s/%s",MAILBOX_SCHEMA,usernameAtHostname, escaped_default_folder.get());
   }
   else {
 	// if the folder path starts with a /, we don't need to add one.
 	// the reason for this is on UNIX, we store mail.directory as "/home/sspitzer/nsmail"
 	// but on windows, its "C:\foo\bar\mail"
-	*new_uri = PR_smprintf("%s/%s%s%s",MAILBOX_SCHEMA,usernameAtHostname,(folderPath[0] == '/')?"":"/",folderPath);
+    // In 4.x if the local folder names have hierarchy in it then the uri is like:
+    //   "/My Folders.sbd/New Drafts#1"
+    // and it needs to be converted to:
+    //   "/My%20Folders/New%20Drafts%231"
+    // ie, we need to get rid of all ".sbd" and then escape the URIs (ie, #->%23).
+    nsCAutoString clean_folderPath(folderPath);
+    clean_folderPath.ReplaceSubstring(".sbd/", "/");
+    nsXPIDLCString escaped_folderPath;
+    ESCAPE_FOLDER_NAME(escaped_folderPath,clean_folderPath.get());
+    *new_uri = PR_smprintf("%s/%s%s%s",MAILBOX_SCHEMA,usernameAtHostname,(folderPath[0] == '/')?"":"/",escaped_folderPath.get());
   }
 
   if (!*new_uri) {
@@ -1136,17 +1175,12 @@ nsMessengerMigrator::MigrateLocalMailAccount()
     rv = mailDir->Create(nsIFile::DIRECTORY_TYPE, 0775);
   if (NS_FAILED(rv)) return rv;  
     
-  nsXPIDLCString mailDirPath;
   nsCOMPtr<nsIFileSpec> mailDirSpec;
   
   // Convert the nsILocalFile into an nsIFileSpec
   // TODO: convert users of nsIFileSpec to nsILocalFile
   // and avoid this step.
-  rv = mailDir->GetPath(getter_Copies(mailDirPath));
-  if (NS_FAILED(rv)) return rv;
-  rv = NS_NewFileSpec(getter_AddRefs(mailDirSpec));
-  if (NS_FAILED(rv)) return rv;
-  rv = mailDirSpec->SetNativePath(mailDirPath);
+  rv = NS_NewFileSpecFromIFile(mailDir, getter_AddRefs(mailDirSpec));
   if (NS_FAILED(rv)) return rv;
 
   // set the default local path for "none"
@@ -1250,15 +1284,11 @@ nsMessengerMigrator::MigrateMovemailAccount(nsIMsgIdentity *identity)
   if (NS_FAILED(rv)) {
     // we wan't <profile>/Mail
     nsCOMPtr<nsIFile> aFile;
-    nsXPIDLCString pathBuf;
     
     rv = NS_GetSpecialDirectory(NS_APP_MAIL_50_DIR, getter_AddRefs(aFile));
     if (NS_FAILED(rv)) return rv;
-    rv = aFile->GetPath(getter_Copies(pathBuf));
-    if (NS_FAILED(rv)) return rv;
-    rv = NS_NewFileSpec(getter_AddRefs(mailDir));
-    if (NS_FAILED(rv)) return rv;
-    rv = mailDir->SetNativePath((const char *)pathBuf);
+
+    NS_NewFileSpecFromIFile(aFile, getter_AddRefs(mailDir));
     if (NS_FAILED(rv)) return rv;
   }
 
@@ -1403,17 +1433,12 @@ nsMessengerMigrator::MigratePopAccount(nsIMsgIdentity *identity)
     rv = mailDir->Create(nsIFile::DIRECTORY_TYPE, 0775);
   if (NS_FAILED(rv)) return rv;
   
-  nsXPIDLCString mailDirPath;
   nsCOMPtr<nsIFileSpec> mailDirSpec;
   
   // Convert the nsILocalFile into an nsIFileSpec
   // TODO: convert users os nsIFileSpec to nsILocalFile
   // and avoid this step.
-  rv = mailDir->GetPath(getter_Copies(mailDirPath));
-  if (NS_FAILED(rv)) return rv;
-  rv = NS_NewFileSpec(getter_AddRefs(mailDirSpec));
-  if (NS_FAILED(rv)) return rv;
-  rv = mailDirSpec->SetNativePath(mailDirPath);
+  rv = NS_NewFileSpecFromIFile(mailDir, getter_AddRefs(mailDirSpec));
   if (NS_FAILED(rv)) return rv;
 
   // set the default local path for "pop3"
@@ -1500,8 +1525,12 @@ nsMessengerMigrator::SetSendLaterUriPref(nsIMsgIncomingServer *server)
 	rv = server->GetHostName(getter_Copies(hostname));
 	if (NS_FAILED(rv)) return rv;
 
+  nsXPIDLCString escaped_username, escaped_hostname;
+  ESCAPE_USER_NAME(escaped_hostname, hostname);
+  ESCAPE_USER_NAME(escaped_username,username);
+
 	char *sendLaterUriStr = nsnull;
-	sendLaterUriStr = PR_smprintf("%s/%s@%s/%s", MAILBOX_SCHEMA, (const char *)username, (const char *)hostname, UNSENT_MESSAGES_FOLDER_NAME);
+	sendLaterUriStr = PR_smprintf("%s/%s@%s/%s", MAILBOX_SCHEMA, escaped_username.get(), escaped_hostname.get(), UNSENT_MESSAGES_FOLDER_NAME);
 	m_prefs->SetCharPref(PREF_MAIL_DEFAULT_SENDLATER_URI, sendLaterUriStr);
 	PR_FREEIF(sendLaterUriStr);
 
@@ -1699,17 +1728,12 @@ nsMessengerMigrator::MigrateImapAccount(nsIMsgIdentity *identity, const char *ho
     rv = imapMailDir->Create(nsIFile::DIRECTORY_TYPE, 0775);
   if (NS_FAILED(rv)) return rv;
   
-  nsXPIDLCString pathBuf;
   nsCOMPtr<nsIFileSpec> imapMailDirSpec;
 
   // Convert the nsILocalFile into an nsIFileSpec
   // TODO: convert users os nsIFileSpec to nsILocalFile
   // and avoid this step.
-  rv = imapMailDir->GetPath(getter_Copies(pathBuf));
-  if (NS_FAILED(rv)) return rv;
-  rv = NS_NewFileSpec(getter_AddRefs(imapMailDirSpec));
-  if (NS_FAILED(rv)) return rv;
-  rv = imapMailDirSpec->SetNativePath(pathBuf);
+  rv = NS_NewFileSpecFromIFile(imapMailDir, getter_AddRefs(imapMailDirSpec));
   if (NS_FAILED(rv)) return rv;
 
   // we only need to do this once
@@ -1898,15 +1922,8 @@ nsMessengerMigrator::migrateAddressBookPrefEnum(const char *aPref, void *aClosur
   if (NS_FAILED(rv) || !ab4xFile) return;
 
   // TODO: Change users of nsIFileSpec to nsIFile and avoid this.
-  {
-     nsXPIDLCString pathBuf;  
-     rv = ab4xFile->GetPath(getter_Copies(pathBuf));
-     if (NS_FAILED(rv)) return;
-     rv = NS_NewFileSpec(getter_AddRefs(ab4xFileSpec));
-     if (NS_FAILED(rv)) return;
-     rv = ab4xFileSpec->SetNativePath(pathBuf);
-     if (NS_FAILED(rv)) return;
-  }
+  rv = NS_NewFileSpecFromIFile(ab4xFile, getter_AddRefs(ab4xFileSpec));
+  if (NS_FAILED(rv)) return;
   
   rv = ab4xFileSpec->AppendRelativeUnixPath((const char *)abFileName);
   NS_ASSERTION(NS_SUCCEEDED(rv),"ab migration failed:  failed to append filename");
@@ -1918,22 +1935,15 @@ nsMessengerMigrator::migrateAddressBookPrefEnum(const char *aPref, void *aClosur
   // do the migration in a subdirectory of temp, to prevent 
   // collision (between multiple users), deleting TMPDIR
   // and privacy issues (where the temp ldif files are readable)
-  rv = tmpLDIFFile->Append("addr-migrate");
+  rv = tmpLDIFFile->AppendNative(NS_LITERAL_CSTRING("addr-migrate"));
   if (NS_FAILED(rv) || !tmpLDIFFile) return;
 
   rv = tmpLDIFFile->CreateUnique(nsIFile::DIRECTORY_TYPE, 0700);
   if (NS_FAILED(rv) || !tmpLDIFFile) return;
 
   // TODO: Change users of nsIFileSpec to nsIFile and avoid this.
-  {
-     nsXPIDLCString pathBuf;  
-     rv = tmpLDIFFile->GetPath(getter_Copies(pathBuf));
-     if (NS_FAILED(rv)) return;
-     rv = NS_NewFileSpec(getter_AddRefs(tmpLDIFFileSpec));
-     if (NS_FAILED(rv)) return;
-     rv = tmpLDIFFileSpec->SetNativePath(pathBuf);
-     if (NS_FAILED(rv)) return;
-  }
+  rv = NS_NewFileSpecFromIFile(tmpLDIFFile, getter_AddRefs(tmpLDIFFileSpec));
+  if (NS_FAILED(rv)) return;
 
   // get the csid from the prefs
   nsCAutoString csidPrefName;
@@ -2076,10 +2086,10 @@ nsMessengerMigrator::MigrateNewsAccounts(nsIMsgIdentity *identity)
     
     // TODO: convert users os nsIFileSpec to nsILocalFile
     // and avoid this step.
-    nsXPIDLCString pathBuf;
-    rv = newsDir->GetPath(getter_Copies(pathBuf));
+    nsCAutoString pathBuf;
+    rv = newsDir->GetNativePath(pathBuf);
     if (NS_FAILED(rv)) return rv;
-    newsHostsDir = pathBuf;    
+    newsHostsDir = pathBuf.get();    
     
 #ifdef USE_NEWSRC_MAP_FILE	
     // if we are using the fat file, it lives in the newsHostsDir.
@@ -2216,10 +2226,10 @@ nsMessengerMigrator::MigrateNewsAccounts(nsIMsgIdentity *identity)
     newsDir = prefLocal;
     
     {
-        nsXPIDLCString pathBuf;
-        newsDir->GetPath(getter_Copies(pathBuf));
+        nsCAutoString pathBuf;
+        newsDir->GetNativePath(pathBuf);
         if (NS_FAILED(rv)) return rv;
-        newsrcDir = (const char *)pathBuf;
+        newsrcDir = pathBuf.get();
     }
 
     for (nsDirectoryIterator i(newsrcDir, PR_FALSE); i.Exists(); i++) {
