@@ -37,7 +37,6 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "msgCore.h"
-#include "xp_core.h"
 #include "nsReadableUtils.h"
 #include "nsMsgDBFolder.h"
 #include "nsMsgFolderFlags.h"
@@ -61,6 +60,7 @@
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
 
+#include <time.h>
 
 #define oneHour 3600000000
 #include "nsMsgUtils.h"
@@ -131,6 +131,8 @@ NS_IMETHODIMP nsMsgDBFolder::Shutdown(PRBool shutdownChildren)
 				}
 			}
 		}
+    // Ask base class shutdown itself.
+    nsMsgFolder::Shutdown(shutdownChildren);
 	}
 	return NS_OK;
 }
@@ -426,10 +428,10 @@ nsresult nsMsgDBFolder::ReadDBFolderInfo(PRBool force)
               if (!mInitializedFromCache)
               {
                 folderInfo->GetFlags((PRInt32 *)&mFlags);
-#ifdef DEBUG_bienvenu
+#ifdef DEBUG_bienvenu1
                 nsXPIDLString name;
                 GetName(getter_Copies(name));
-                NS_ASSERTION(Compare(name, NS_LITERAL_STRING("Trash")) || (mFlags & MSG_FOLDER_FLAG_TRASH), "lost trash flag");
+                NS_ASSERTION(Compare(name, kLocalizedTrashName) || (mFlags & MSG_FOLDER_FLAG_TRASH), "lost trash flag");
 #endif
                 mInitializedFromCache = PR_TRUE;
               }
@@ -529,7 +531,7 @@ NS_IMETHODIMP nsMsgDBFolder::GetOfflineFileTransport(nsMsgKey msgKey, PRUint32 *
     mPath->GetNativePath(getter_Copies(nativePath));
 
     nsCOMPtr <nsILocalFile> localStore;
-    rv = NS_NewLocalFile(nativePath, PR_TRUE, getter_AddRefs(localStore));
+    rv = NS_NewNativeLocalFile(nativePath, PR_TRUE, getter_AddRefs(localStore));
     if (NS_SUCCEEDED(rv) && localStore)
     {
       NS_DEFINE_CID(kFileTransportServiceCID, NS_FILETRANSPORTSERVICE_CID);
@@ -539,7 +541,11 @@ NS_IMETHODIMP nsMsgDBFolder::GetOfflineFileTransport(nsMsgKey msgKey, PRUint32 *
       if (NS_FAILED(rv))
         return rv;
       
-      rv = fts->CreateTransport(localStore, PR_RDWR | PR_CREATE_FILE, 0664, aFileChannel);
+      rv = fts->CreateTransport(localStore,
+                                PR_RDWR | PR_CREATE_FILE,
+                                0664,
+                                PR_TRUE,
+                                aFileChannel);
 
       if (NS_SUCCEEDED(rv))
       {
@@ -749,39 +755,31 @@ NS_IMETHODIMP nsMsgDBFolder::OnKeyAdded(nsMsgKey aKeyChanged, nsMsgKey  aParentK
 nsresult nsMsgDBFolder::OnKeyAddedOrDeleted(nsMsgKey aKeyChanged, nsMsgKey  aParentKey , PRInt32 aFlags, 
                         nsIDBChangeListener * aInstigator, PRBool added, PRBool doFlat, PRBool doThread)
 {
-	nsCOMPtr<nsIMsgDBHdr> msgDBHdr;
-	nsCOMPtr<nsIMsgDBHdr> parentDBHdr;
-	nsresult rv = mDatabase->GetMsgHdrForKey(aKeyChanged, getter_AddRefs(msgDBHdr));
-	if(NS_FAILED(rv))
-		return rv;
-
-	rv = mDatabase->GetMsgHdrForKey(aParentKey, getter_AddRefs(parentDBHdr));
-	if(NS_FAILED(rv))
-		return rv;
-
-	if(msgDBHdr)
-	{
-		nsCOMPtr<nsISupports> msgSupports(do_QueryInterface(msgDBHdr));
-		nsCOMPtr<nsISupports> folderSupports;
-		rv = QueryInterface(NS_GET_IID(nsISupports), getter_AddRefs(folderSupports));
-		if(msgSupports && NS_SUCCEEDED(rv) && doFlat)
-		{
-			if(added)
-				NotifyItemAdded(folderSupports, msgSupports, "flatMessageView");
-			else
-				NotifyItemDeleted(folderSupports, msgSupports, "flatMessageView");
-		}
-		if(msgSupports && folderSupports)
-		{
-			if(added)
-				NotifyItemAdded(folderSupports, msgSupports, "threadMessageView");
-			else
-				NotifyItemDeleted(folderSupports, msgSupports, "threadMessageView");
-		}
-		UpdateSummaryTotals(PR_TRUE);
-	}
-	return NS_OK;
-
+  nsCOMPtr<nsIMsgDBHdr> msgDBHdr;
+  nsresult rv = mDatabase->GetMsgHdrForKey(aKeyChanged, getter_AddRefs(msgDBHdr));
+  if(NS_SUCCEEDED(rv) && msgDBHdr)
+  {
+    nsCOMPtr<nsISupports> msgSupports(do_QueryInterface(msgDBHdr));
+    nsCOMPtr<nsISupports> folderSupports;
+    rv = QueryInterface(NS_GET_IID(nsISupports), getter_AddRefs(folderSupports));
+    if(msgSupports && NS_SUCCEEDED(rv) && doFlat)
+    {
+      if(added)
+        NotifyItemAdded(folderSupports, msgSupports, "flatMessageView");
+      else
+        NotifyItemDeleted(folderSupports, msgSupports, "flatMessageView");
+    }
+    if(msgSupports && folderSupports)
+    {
+      if(added)
+        NotifyItemAdded(folderSupports, msgSupports, "threadMessageView");
+      else
+        NotifyItemDeleted(folderSupports, msgSupports, "threadMessageView");
+    }
+    UpdateSummaryTotals(PR_TRUE);
+  }
+  return NS_OK;
+  
 }
 
 
@@ -1333,7 +1331,8 @@ nsresult nsMsgDBFolder::WriteStartOfNewLocalMessage()
 nsresult nsMsgDBFolder::StartNewOfflineMessage()
 {
   nsresult rv = GetOfflineStoreOutputStream(getter_AddRefs(m_tempMessageStream));
-  WriteStartOfNewLocalMessage();
+  if (NS_SUCCEEDED(rv))
+    WriteStartOfNewLocalMessage();
   m_numOfflineMsgLines = 0;
   return rv;
 }
