@@ -472,7 +472,8 @@ PRBool nsMsgI18N7bit_data_part(const char *charset, const char *inString, const
         srcLen = ((size - consumedLen) >= 512) ? 512 : (size - consumedLen);  // buffer len or remaining src len
         unicharLength = 512;
         res = decoder->Convert(currentSrcPtr, &srcLen, unicharBuff, &unicharLength);
-        if (NS_FAILED(res))
+        // break, if error or the input string not consumed
+        if (NS_FAILED(res) || !srcLen)
           break;
         for (PRInt32 i = 0; i < unicharLength; i++) {
           if (unicharBuff[i] > 127) {
@@ -579,7 +580,8 @@ nsMsgI18NParseMetaCharset(nsFileSpec* fileSpec)
       token = nsCRT::strtok(cp, seps, &newStr); 
       if (token != NULL) 
       { 
-        PL_strcpy(charset, token); 
+        PL_strncpy(charset, token, sizeof(charset));
+        charset[sizeof(charset)-1] = '\0';
       } 
     } 
   } 
@@ -658,7 +660,7 @@ nsresult nsMsgI18NSaveAsCharset(const char* contentType, const char *charset, co
   else
     // fallback for text/plain: first try transliterate then '?'
     res = conv->Init(NS_ConvertUCS2toUTF8(charsetName).get(), 
-                     nsISaveAsCharset::attr_FallbackQuestionMark + nsISaveAsCharset::attr_EntityNone, 
+                     nsISaveAsCharset::attr_FallbackQuestionMark + nsISaveAsCharset::attr_EntityAfterCharsetConv, 
                      nsIEntityConverter::transliterate);
   NS_ENSURE_SUCCESS(res, res);
 
@@ -716,6 +718,7 @@ nsMsgI18NGetAcceptLanguage(void)
 	  if (NS_SUCCEEDED(res) && prefValue) 
     {
       PL_strncpy(lang, NS_ConvertUCS2toUTF8(prefValue).get(), sizeof(lang));
+      lang[sizeof(lang)-1] = '\0';
     }
 	  else 
 		  PL_strcpy(lang, "en");
@@ -727,7 +730,7 @@ nsMsgI18NGetAcceptLanguage(void)
 }
 
 
-// taken from nsFileSpec::GetNativePAthString()
+// taken from nsFileSpec::GetNativePathString()
 void
 nsMsgGetNativePathString(const char *aPath, nsString& aResult)
 {
