@@ -178,10 +178,10 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetServer(nsIMsgIncomingServer ** aIncomingServe
 
 	nsresult rv = GetAsciiHost(host);
 
-	/* GetUsername() returns an escaped string.
+	/* GetUsername() returns an escaped string, so we need to manually unescape it.
 	 */
 	GetUsername(userName);
-    NS_UnescapeURL(userName); // XXX may result in non-ASCII octets
+	NS_UnescapeURL(userName); // XXX may result in non-ASCII octets!
 
 	rv = GetScheme(scheme);
     if (NS_SUCCEEDED(rv))
@@ -200,6 +200,17 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetServer(nsIMsgIncomingServer ** aIncomingServe
                                         host.get(),
                                         scheme.get(),
                                         aIncomingServer);
+        if (!*aIncomingServer && scheme.Equals("imap"))
+        {
+          // look for any imap server with this host name so clicking on 
+          // other users folder urls will work. We could override this method
+          // for imap urls, or we could make caching of servers work and
+          // just set the server in the imap code for this case.
+          rv = accountManager->FindServer("",
+                                          host.get(),
+                                          scheme.get(),
+                                          aIncomingServer);
+        }
     }
 
     return rv;
@@ -296,7 +307,8 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetLoadGroup(nsILoadGroup **aLoadGroup)
               if (statusFeedback && docShell)
               {
                 webProgressListener = do_QueryInterface(statusFeedback);
-                webProgress->AddProgressListener(webProgressListener);
+                webProgress->AddProgressListener(webProgressListener,
+                                                 nsIWebProgress::NOTIFY_ALL);
               }
             }
 #endif
@@ -641,7 +653,7 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetFileExtension(nsACString &aFileExtension)
   if (!mAttachmentFileName.IsEmpty())
   {
     nsCAutoString extension;
-    PRInt32 pos = mAttachmentFileName.RFindCharInSet(".");
+    PRInt32 pos = mAttachmentFileName.RFindChar(PRUnichar('.'));
     if (pos > 0)
       mAttachmentFileName.Right(extension,
                                 mAttachmentFileName.Length() -
@@ -703,6 +715,16 @@ NS_IMETHODIMP nsMsgMailNewsUrl::SetFilePath(const nsACString &i_DirFile)
 	return m_baseURL->SetFilePath(i_DirFile);
 }
 
+NS_IMETHODIMP nsMsgMailNewsUrl::GetCommonBaseSpec(nsIURI *uri2, nsACString &result)
+{
+  return m_baseURL->GetCommonBaseSpec(uri2, result);
+}
+
+NS_IMETHODIMP nsMsgMailNewsUrl::GetRelativeSpec(nsIURI *uri2, nsACString &result)
+{
+  return m_baseURL->GetRelativeSpec(uri2, result);
+}
+
 NS_IMETHODIMP nsMsgMailNewsUrl::SetMemCacheEntry(nsICacheEntryDescriptor *memCacheEntry)
 {
   m_memCacheEntry = memCacheEntry;
@@ -767,3 +789,16 @@ NS_IMETHODIMP nsMsgMailNewsUrl:: RemoveCacheEntry(nsICacheEntryDescriptor *cache
   }
   return NS_OK;
 }
+
+NS_IMETHODIMP nsMsgMailNewsUrl::GetMimeHeaders(nsIMimeHeaders * *mimeHeaders)
+{
+    NS_ENSURE_ARG_POINTER(mimeHeaders);
+    NS_IF_ADDREF(*mimeHeaders = mMimeHeaders);
+    return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgMailNewsUrl::SetMimeHeaders(nsIMimeHeaders *mimeHeaders)
+{
+    mMimeHeaders = mimeHeaders;
+    return NS_OK;
+}
