@@ -57,10 +57,13 @@
 #include "nsNetCID.h"
 #include "nsIIOService.h"
 #include "nsIRDFService.h"
+#include "nsIMimeConverter.h"
+#include "nsMsgMimeCID.h"
 
 static NS_DEFINE_CID(kImapUrlCID, NS_IMAPURL_CID);
 static NS_DEFINE_CID(kCMailboxUrl, NS_MAILBOXURL_CID);
 static NS_DEFINE_CID(kCNntpUrlCID, NS_NNTPURL_CID);
+static NS_DEFINE_CID(kCMimeConverterCID, NS_MIME_CONVERTER_CID);
 
 nsresult GetMessageServiceContractIDForURI(const char *uri, nsCString &contractID)
 {
@@ -300,7 +303,7 @@ nsresult NS_MsgCreatePathStringFromFolderURI(const char *folderURI, nsCString& p
 	PRInt32 startSlashPos = oldPath.FindChar('/');
 	PRInt32 endSlashPos = (startSlashPos >= 0) 
 		? oldPath.FindChar('/', startSlashPos + 1) - 1 : oldPath.Length() - 1;
-	if (endSlashPos == -1)
+	if (endSlashPos < 0)
 		endSlashPos = oldPath.Length();
     // trick to make sure we only add the path to the first n-1 folders
     PRBool haveFirst=PR_FALSE;
@@ -324,10 +327,10 @@ nsresult NS_MsgCreatePathStringFromFolderURI(const char *folderURI, nsCString& p
 
 	  endSlashPos = (startSlashPos >= 0) 
 			? oldPath.FindChar('/', startSlashPos + 1)  - 1: oldPath.Length() - 1;
-	  if (endSlashPos == -1)
+	  if (endSlashPos < 0)
 			endSlashPos = oldPath.Length();
 
-      if (startSlashPos == endSlashPos)
+      if (startSlashPos >= endSlashPos)
 		  break;
     }
 
@@ -342,7 +345,7 @@ nsresult NS_MsgCreatePathStringFromFolderURI(const char *folderURI, nsCString& p
    The string is not altered: the pointer to its head is merely advanced,
    and the length correspondingly decreased.
  */
-PRBool NS_MsgStripRE(const char **stringP, PRUint32 *lengthP)
+PRBool NS_MsgStripRE(const char **stringP, PRUint32 *lengthP, char **modifiedSubject)
 {
   const char *s, *s_end;
   const char *last;
@@ -350,7 +353,20 @@ PRBool NS_MsgStripRE(const char **stringP, PRUint32 *lengthP)
   PRBool result = PR_FALSE;
   NS_ASSERTION(stringP, "bad null param");
   if (!stringP) return PR_FALSE;
-  s = *stringP;
+
+  // decode the string
+  nsXPIDLCString decodedString;
+  nsCOMPtr<nsIMimeConverter> mimeConverter;
+  nsresult rv;
+  // we cannot strip "Re:" for MIME encoded subject without modifying the original
+  if (modifiedSubject && strstr(*stringP, "=?"))
+  {
+    mimeConverter = do_GetService(kCMimeConverterCID, &rv);
+    if (NS_SUCCEEDED(rv))
+      rv = mimeConverter->DecodeMimeHeader(*stringP, getter_Copies(decodedString));
+  }
+
+  s = decodedString ? decodedString.get() : *stringP;
   L = lengthP ? *lengthP : strlen(s);
 
   s_end = s + L;
@@ -391,6 +407,35 @@ PRBool NS_MsgStripRE(const char **stringP, PRUint32 *lengthP)
 		}
 	}
 
+  if (decodedString)
+  {
+    // encode the string back if any modification is made
+    if (s != decodedString.get())
+    {
+      // extract between "=?" and "?"
+      // e.g. =?ISO-2022-JP?
+      const char *p1 = strstr(*stringP, "=?");
+      if (p1) 
+      {
+        p1 += sizeof("=?")-1;         // skip "=?"
+        const char *p2 = strchr(p1, '?');   // then search for '?' 
+        if (p2) 
+        {
+          char charset[kMAX_CSNAME] = "";
+          if (kMAX_CSNAME >= (p2 - p1))
+            strncpy(charset, p1, p2 - p1);
+          rv = mimeConverter->EncodeMimePartIIStr_UTF8(s, PR_FALSE, charset, sizeof("Subject:"), 
+                                                       kMIME_ENCODED_WORD_SIZE, modifiedSubject);
+          if (NS_SUCCEEDED(rv)) 
+            return result;
+        }
+      }
+    }
+    else 
+      s = *stringP;   // no modification, set the original encoded string
+  }
+
+
   /* Decrease length by difference between current ptr and original ptr.
 	 Then store the current ptr back into the caller. */
   if (lengthP) 
