@@ -256,7 +256,6 @@ void nsSmtpProtocol::Initialize(nsIURI * aURL)
     m_prefAuthMethod = PREF_AUTH_NONE;
     m_usernamePrompted = PR_FALSE;
     m_prefTrySSL = PREF_SSL_TRY;
-    m_port = SMTP_PORT;
     m_tlsInitiated = PR_FALSE;
 
     m_urlErrorState = NS_ERROR_FAILURE;
@@ -301,7 +300,7 @@ void nsSmtpProtocol::Initialize(nsIURI * aURL)
         smtpServer->GetAuthMethod(&m_prefAuthMethod);
         smtpServer->GetTrySSL(&m_prefTrySSL);
     }
-    
+
     rv = RequestOverrideInfo(smtpServer);
     // if we aren't waiting for a login override, then go ahead an
     // open the network connection like we normally would have.
@@ -1672,7 +1671,7 @@ nsresult nsSmtpProtocol::RequestOverrideInfo(nsISmtpServer * aSmtpServer)
     PRBool requiresPassword = PR_TRUE;
 
 		aSmtpServer->GetUsername(getter_Copies(userName));
-    m_logonRedirector->RequiresPassword(userName, &requiresPassword);
+    m_logonRedirector->RequiresPassword(userName, redirectionTypeStr.get(), &requiresPassword);
     if (requiresPassword)
 		  GetPassword(getter_Copies(password));
 
@@ -1707,14 +1706,9 @@ NS_IMETHODIMP nsSmtpProtocol::OnLogonRedirectionError(const PRUnichar *pErrMsg,
   NS_ENSURE_TRUE(smtpServer, NS_ERROR_FAILURE);
   NS_ENSURE_TRUE(m_logonRedirector, NS_ERROR_FAILURE);
 
-  // step (1) force a log off...
-  // logoff 
-  nsXPIDLCString userName;
-	smtpServer->GetUsername(getter_Copies(userName));
-  m_logonRedirector->Logoff(userName);
   m_logonRedirector = nsnull; // we don't care about it anymore
 	
-  // step (2) alert the user about the error
+  // step (1) alert the user about the error
   nsCOMPtr<nsIPrompt> dialog;
   if (m_runningURL && pErrMsg && pErrMsg[0]) 
   {
@@ -1723,11 +1717,11 @@ NS_IMETHODIMP nsSmtpProtocol::OnLogonRedirectionError(const PRUnichar *pErrMsg,
       dialog->Alert(nsnull, pErrMsg);
   }
 
-  // step (3) if they entered a bad password, forget about it!
+  // step (2) if they entered a bad password, forget about it!
   if (aBadPassword && smtpServer)
     smtpServer->ForgetPassword();
 
-  // step (4) we need to let the originator of the send url request know that an
+  // step (3) we need to let the originator of the send url request know that an
   // error occurred and we aren't sending the message...in our case, this will
   // force the user back into the compose window and they can try to send it 
   // again.
