@@ -55,7 +55,7 @@ struct mdbOid gAllOfflineOpsTableOID;
 
 
 nsMailDatabase::nsMailDatabase()
-    : m_reparse(PR_FALSE), m_folderSpec(nsnull), m_folderStream(nsnull)
+    : m_reparse(PR_FALSE), m_folderSpec(nsnull), m_folderStream(nsnull), m_ownFolderStream(PR_FALSE)
 {
   m_mdbAllOfflineOpsTable = nsnull;
 }
@@ -68,6 +68,12 @@ nsMailDatabase::~nsMailDatabase()
     m_mdbAllOfflineOpsTable->Release();
 }
 
+NS_IMETHODIMP nsMailDatabase::SetFolderStream(nsIOFileStream *aFileStream)
+{
+  m_folderStream = aFileStream; //m_folderStream is set externally, so m_ownFolderStream is false
+  m_ownFolderStream = PR_FALSE;
+  return NS_OK;
+}
 
 
 NS_IMETHODIMP nsMailDatabase::Open(nsIFileSpec *aFolderName, PRBool create, PRBool upgrading, nsIMsgDatabase** pMessageDB)
@@ -234,21 +240,28 @@ nsresult nsMailDatabase::OnNewPath (nsFileSpec &newPath)
 NS_IMETHODIMP nsMailDatabase::StartBatch()
 {
 #ifndef XP_MAC
-  if (!m_folderStream)
-	  m_folderStream = new nsIOFileStream(nsFileSpec(*m_folderSpec));
+  if (!m_folderStream)  //only if we create a stream, set m_ownFolderStream to true.
+  {
+    m_folderStream = new nsIOFileStream(nsFileSpec(*m_folderSpec));
+    m_ownFolderStream = PR_TRUE;
+  }
 #endif  
 return NS_OK;
 }
 
 NS_IMETHODIMP nsMailDatabase::EndBatch()
 {
-#ifndef XP_MAC
-  if (m_folderStream)
+#ifndef XP_MAC  //only if we own the stream, then we should close it
+  if (m_ownFolderStream)
   {
-    m_folderStream->close();  
-    delete m_folderStream;
+    if (m_folderStream)
+    {
+      m_folderStream->close();  
+      delete m_folderStream;
+    }
+    m_folderStream = nsnull;
+    m_ownFolderStream = PR_FALSE;
   }
-  m_folderStream = nsnull;
 #endif
   return NS_OK;
 }
@@ -258,14 +271,21 @@ NS_IMETHODIMP nsMailDatabase::DeleteMessages(nsMsgKeyArray* nsMsgKeys, nsIDBChan
 {
 	nsresult ret = NS_OK;
   if (!m_folderStream)
+  {
 	  m_folderStream = new nsIOFileStream(nsFileSpec(*m_folderSpec));
+    m_ownFolderStream = PR_TRUE;
+  }
 	ret = nsMsgDatabase::DeleteMessages(nsMsgKeys, instigator);
-	if (m_folderStream)
+  if (m_ownFolderStream)//only if we own the stream, then we should close it
+  {
+	  if (m_folderStream)
     {
       m_folderStream->close();
-	  delete m_folderStream;
+	    delete m_folderStream;
     }
-	m_folderStream = NULL;
+	  m_folderStream = nsnull;
+    m_ownFolderStream = PR_FALSE;
+  }
 	SetFolderInfoValid(m_folderSpec, 0, 0);
 	return ret;
 }
@@ -276,7 +296,7 @@ PRBool nsMailDatabase::SetHdrFlag(nsIMsgDBHdr *msgHdr, PRBool bSet, MsgFlags fla
 {
   nsIOFileStream *fileStream = NULL;
   PRBool		ret = PR_FALSE;
-  
+
   if (nsMsgDatabase::SetHdrFlag(msgHdr, bSet, flag))
   {
     UpdateFolderFlag(msgHdr, bSet, flag, &fileStream);
@@ -288,6 +308,7 @@ PRBool nsMailDatabase::SetHdrFlag(nsIMsgDBHdr *msgHdr, PRBool bSet, MsgFlags fla
     }
     ret = PR_TRUE;
   }
+
   return ret;
 }
 
@@ -703,73 +724,70 @@ NS_IMETHODIMP nsMailDatabase::ListAllOfflineDeletes(nsMsgKeyArray *offlineDelete
 /* static */
 nsresult nsMailDatabase::SetFolderInfoValid(nsFileSpec *folderName, int num, int numunread)
 {
-	nsLocalFolderSummarySpec	summarySpec(*folderName);
-	nsFileSpec					summaryPath(summarySpec);
-	nsresult		err = NS_OK;
-	PRBool			bOpenedDB = PR_FALSE;
-
-	if (!folderName->Exists())
-		return NS_MSG_ERROR_FOLDER_SUMMARY_MISSING;
-
-	// should we have type safe downcast methods again?
-	nsMailDatabase *pMessageDB = (nsMailDatabase *) nsMailDatabase::FindInCache(summaryPath);
-	if (pMessageDB == NULL)
-	{
-		pMessageDB = new nsMailDatabase();
-		if(!pMessageDB)
-			return NS_ERROR_OUT_OF_MEMORY;
-
-		pMessageDB->m_folderSpec = new nsLocalFolderSummarySpec();
-		if(!pMessageDB->m_folderSpec)
-		{
-			delete pMessageDB;
-			return NS_ERROR_OUT_OF_MEMORY;
-		}
-
-		*(pMessageDB->m_folderSpec) = summarySpec;
-		// ### this does later stuff (marks latered messages unread), which may be a problem
-		err = pMessageDB->OpenMDB(summaryPath, PR_FALSE);
-		if (err != NS_OK)
-		{
-			delete pMessageDB;
-			pMessageDB = NULL;
-		}
-		bOpenedDB = PR_TRUE;
-	}
-	else
-		pMessageDB->AddRef();
-
-
-	if (pMessageDB == NULL)
-	{
+  nsLocalFolderSummarySpec	summarySpec(*folderName);
+  nsFileSpec					summaryPath(summarySpec);
+  nsresult		err = NS_OK;
+  PRBool			bOpenedDB = PR_FALSE;
+  
+  if (!folderName->Exists())
+    return NS_MSG_ERROR_FOLDER_SUMMARY_MISSING;
+  
+  // should we have type safe downcast methods again?
+  nsMailDatabase *pMessageDB = (nsMailDatabase *) nsMailDatabase::FindInCache(summaryPath);
+  if (pMessageDB == nsnull)
+  {
+    pMessageDB = new nsMailDatabase();
+    if(!pMessageDB)
+      return NS_ERROR_OUT_OF_MEMORY;
+    
+    pMessageDB->m_folderSpec = new nsLocalFolderSummarySpec();
+    if(!pMessageDB->m_folderSpec)
+    {
+      delete pMessageDB;
+      return NS_ERROR_OUT_OF_MEMORY;
+    }
+    
+    *(pMessageDB->m_folderSpec) = summarySpec;
+    // ### this does later stuff (marks latered messages unread), which may be a problem
+    err = pMessageDB->OpenMDB(summaryPath, PR_FALSE);
+    if (err != NS_OK)
+    {
+      delete pMessageDB;
+      pMessageDB = nsnull;
+    }
+    bOpenedDB = PR_TRUE;
+  }
+  
+  if (pMessageDB == nsnull)
+  {
 #ifdef DEBUG
-		printf("Exception opening summary file\n");
+    printf("Exception opening summary file\n");
 #endif
-		return NS_MSG_ERROR_FOLDER_SUMMARY_OUT_OF_DATE;
-	}
-	
-	{
-		nsFileSpec::TimeStamp actualFolderTimeStamp;
-		folderName->GetModDate(actualFolderTimeStamp) ;
-
-		pMessageDB->m_dbFolderInfo->SetFolderSize(folderName->GetFileSize());
-		pMessageDB->m_dbFolderInfo->SetFolderDate(actualFolderTimeStamp);
-		pMessageDB->m_dbFolderInfo->ChangeNumVisibleMessages(num);
-		pMessageDB->m_dbFolderInfo->ChangeNumNewMessages(numunread);
-		pMessageDB->m_dbFolderInfo->ChangeNumMessages(num);
-	}
-	// if we opened the db, then we'd better close it. Otherwise, we found it in the cache,
-	// so just commit and release.
-	if (bOpenedDB)
-	{
-		pMessageDB->Close(PR_TRUE);
-	}
-	else if (pMessageDB)
-	{ 
-		err = pMessageDB->Commit(nsMsgDBCommitType::kLargeCommit);
-		pMessageDB->Release();
-	}
-	return err;
+    return NS_MSG_ERROR_FOLDER_SUMMARY_OUT_OF_DATE;
+  }
+  
+  {
+    nsFileSpec::TimeStamp actualFolderTimeStamp;
+    folderName->GetModDate(actualFolderTimeStamp) ;
+    
+    pMessageDB->m_dbFolderInfo->SetFolderSize(folderName->GetFileSize());
+    pMessageDB->m_dbFolderInfo->SetFolderDate(actualFolderTimeStamp);
+    pMessageDB->m_dbFolderInfo->ChangeNumVisibleMessages(num);
+    pMessageDB->m_dbFolderInfo->ChangeNumNewMessages(numunread);
+    pMessageDB->m_dbFolderInfo->ChangeNumMessages(num);
+  }
+  // if we opened the db, then we'd better close it. Otherwise, we found it in the cache,
+  // so just commit and release.
+  if (bOpenedDB)
+  {
+    pMessageDB->Close(PR_TRUE);
+  }
+  else if (pMessageDB)
+  { 
+    err = pMessageDB->Commit(nsMsgDBCommitType::kLargeCommit);
+    pMessageDB->Release();
+  }
+  return err;
 }
 
 
@@ -919,4 +937,3 @@ NS_IMETHODIMP nsMsgOfflineOpEnumerator::HasMoreElements(PRBool *aResult)
   *aResult = !mDone;
   return NS_OK;
 }
-
