@@ -61,11 +61,19 @@ nsOEScanBoxes::nsOEScanBoxes()
 
 nsOEScanBoxes::~nsOEScanBoxes()
 {
-	int max = m_entryArray.Count();
-	for (int i = 0; i < max; i++) {
-		MailboxEntry *pEntry = (MailboxEntry *) m_entryArray.ElementAt( i);
-		delete pEntry;
-	}
+  int i, max;
+  MailboxEntry *pEntry;
+  for (i = 0, max = m_entryArray.Count(); i < max; i++) {
+    pEntry = (MailboxEntry *) m_entryArray.ElementAt( i);
+    delete pEntry;
+  }
+  // Now free the unprocessed child entries (ie, those without parents for some reason).
+  for (i = 0, max = m_pendingChildArray.Count(); i < max; i++)
+  {
+    pEntry = (MailboxEntry *) m_pendingChildArray.ElementAt(i);
+    if (!pEntry->processed)
+      delete pEntry;
+  }
 }
 
 
@@ -383,6 +391,7 @@ PRBool nsOEScanBoxes::Find50MailBoxes( nsIFileSpec* descFile)
 		while (dataOffset < strOffset) {
 			tag = (PRUint8) pBytes[dataOffset];
 
+      data = 0; // make sure all bytes are 0 before copying 3 bytes over.
 			memcpy( &data, &(pBytes[dataOffset + 1]), 3);
 			switch( tag) {
 				case 0x80: // id record
@@ -409,7 +418,13 @@ PRBool nsOEScanBoxes::Find50MailBoxes( nsIFileSpec* descFile)
 		// now build an entry if necessary!
 		if (pDataSource) {
 			if (!nsCRT::strcasecmp( pDataSource, "LocalStore"))
+      {
 				localStoreId = id;	
+        // See if we have any child folders that need to be added/processed for this top level parent.
+        ProcessPendingChildEntries(localStoreId, localStoreId, m_pendingChildArray);
+        // Clean up the pending list.
+        RemoveProcessedChildEntries();
+      }
 		}
 		else if (id && localStoreId && parent) {
 			// veryify that this mailbox is in the local store
@@ -423,23 +438,38 @@ PRBool nsOEScanBoxes::Find50MailBoxes( nsIFileSpec* descFile)
 			}
 			if (data == localStoreId) {
 				// Create an entry for this bugger
-				pEntry = new MailboxEntry();
-				pEntry->index = id;
-				pEntry->parent = parent;
-				pEntry->child = 0;
-				pEntry->type = 0;
-				pEntry->sibling = -1;
-				ConvertToUnicode((const char *) (pBytes + strOffset), pEntry->mailName);
-				if (pFileName)
-					pEntry->fileName = pFileName;
-				AddChildEntry( pEntry, localStoreId);
+        pEntry = NewMailboxEntry(id, parent, (const char *) (pBytes + strOffset), pFileName);
+        if (pEntry)
+        {
+				  AddChildEntry( pEntry, localStoreId);
+          pEntry->processed =  PR_TRUE;
+          // See if we have any child folders that need to be added/processed.
+          ProcessPendingChildEntries(id, localStoreId, m_pendingChildArray);
+          // Clean up the pending list.
+          RemoveProcessedChildEntries();
+        }
 			}
+      else
+      {
+        // Put this folder into child array and process it when its parent shows up.
+        pEntry = NewMailboxEntry(id, parent, (const char *) (pBytes + strOffset), pFileName);
+        if (pEntry)
+          m_pendingChildArray.AppendElement(pEntry);
+      }
 		}
+    else if (pFileName)
+    {
+      // Put this folder into child array and process it when its parent shows up.
+      // For some reason, it's likely that child folders come before their parents.
+      pEntry = NewMailboxEntry(id, parent, (const char *) (pBytes + strOffset), pFileName);
+      if (pEntry)
+        m_pendingChildArray.AppendElement(pEntry);
+    }
 
 		delete [] pBytes;
 	}
-
 	
+
 	delete [] pIndex;
 
 	if (m_entryArray.Count())
@@ -448,6 +478,55 @@ PRBool nsOEScanBoxes::Find50MailBoxes( nsIFileSpec* descFile)
 		return( PR_FALSE);
 }
 
+nsOEScanBoxes::MailboxEntry *nsOEScanBoxes::NewMailboxEntry(PRUint32 id, PRUint32 parent, const char *prettyName, char *pFileName)
+{
+  MailboxEntry *pEntry = new MailboxEntry();
+  if (!pEntry)
+    return nsnull;
+
+  pEntry->index = id;
+  pEntry->parent = parent;
+  pEntry->child = 0;
+  pEntry->type = 0;
+  pEntry->sibling = -1;
+  pEntry->processed =  PR_FALSE;
+  ConvertToUnicode(prettyName, pEntry->mailName);
+  if (pFileName)
+    pEntry->fileName = pFileName;
+  return pEntry;
+}
+
+void nsOEScanBoxes::ProcessPendingChildEntries(PRUint32 parent, PRUint32 rootIndex, nsVoidArray	&childArray)
+{
+  PRInt32 i, max;
+  MailboxEntry *pEntry;
+  for (i = 0, max = childArray.Count(); i < max; i++)
+  {
+    pEntry = (MailboxEntry *) childArray.ElementAt(i);
+    if ((!pEntry->processed) && (pEntry->parent == parent))
+    {
+      AddChildEntry(pEntry, rootIndex);
+      pEntry->processed =  PR_TRUE; // indicate it's been processed.
+      // See if there are unprocessed child folders for this child in the
+      // array as well (ie, both child and grand-child are on the list).
+      ProcessPendingChildEntries(pEntry->index, rootIndex, childArray);
+    }
+  }
+}
+
+void nsOEScanBoxes::RemoveProcessedChildEntries()
+{
+  // Remove already processed entries from the pending list. Note that these entries are also
+  // on 'm_entryArray' list so we don't want to deallocate the space for the entries now.
+  MailboxEntry * pEntry;
+  PRInt32 i;
+  for (i = m_pendingChildArray.Count()-1; i >= 0; i--)
+  {
+    pEntry = (MailboxEntry *) m_pendingChildArray.ElementAt(i);
+    if (pEntry->processed)
+      m_pendingChildArray.RemoveElementAt(i);
+  }
+}
 
 void nsOEScanBoxes::AddChildEntry( MailboxEntry *pEntry, PRUint32 rootIndex)
 {
