@@ -52,7 +52,7 @@
 #include "nsMsgDBCID.h"
 #include "nsIMsgMailNewsUrl.h"
 #include "nsICopyMsgStreamListener.h"
-//#include "allxpstr.h"
+#include "nsMsgMessageFlags.h"
 #include "prtime.h"
 #include "prlog.h"
 #include "prerror.h"
@@ -69,12 +69,14 @@ PRLogModuleInfo *MAILBOX;
 #include "nsXPIDLString.h"
 #include "nsNetUtil.h"
 #include "nsIMsgWindow.h"
+#include "nsIMimeHeaders.h"
+
+#include "nsIMsgMdnGenerator.h"
 
 static NS_DEFINE_CID(kIStreamConverterServiceCID, NS_STREAMCONVERTERSERVICE_CID);
 static NS_DEFINE_CID(kCMailDB, NS_MAILDB_CID);
 static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
 
-
 /* the output_buffer_size must be larger than the largest possible line
  * 2000 seems good for news
  *
@@ -158,8 +160,11 @@ nsresult nsMailboxProtocol::OpenFileSocketForReuse(nsIURI * aURL, PRUint32 aStar
   LL_L2UI( length, fileSize );  
 
   // probably should pass in the file size instead of aReadCount
-  rv = fts->CreateTransportFromStream("mailbox", m_multipleMsgMoveCopyStream,
-                            "", length, PR_FALSE, getter_AddRefs(m_transport));
+  rv = fts->CreateTransportFromStream(NS_LITERAL_CSTRING("mailbox"),
+                            m_multipleMsgMoveCopyStream,
+                            NS_LITERAL_CSTRING(""),
+                            NS_LITERAL_CSTRING(""),
+                            length, PR_FALSE, getter_AddRefs(m_transport));
   m_socketIsOpen = PR_FALSE;
 
 	return rv;
@@ -382,6 +387,7 @@ PRInt32 nsMailboxProtocol::DoneReadingMessage()
 	{
 		// now mark the message as read
 		nsCOMPtr<nsIMsgDBHdr> msgHdr;
+        nsMsgKey msgKey;
     if (m_runningUrl)
 		  rv = m_runningUrl->GetMessageHeader(getter_AddRefs(msgHdr));
     NS_ASSERTION(msgHdr, "no msg hdr!");
@@ -389,7 +395,54 @@ PRInt32 nsMailboxProtocol::DoneReadingMessage()
     PRBool isRead;
     msgHdr->GetIsRead(&isRead);
     if (NS_SUCCEEDED(rv) && !isRead)
+        {
+            PRUint32 msgFlags, newFlags;
+            msgHdr->GetFlags(&msgFlags);
+            if (msgFlags & MSG_FLAG_MDN_REPORT_NEEDED)
+            {
+                nsCOMPtr<nsIMsgMdnGenerator> mdnGenerator;
+                nsCOMPtr<nsIMimeHeaders> mimeHeaders;
+
+                mdnGenerator =
+                    do_CreateInstance(NS_MSGMDNGENERATOR_CONTRACTID, &rv);
+
+                // To ensure code works w/o MDN enabled
+                if (NS_SUCCEEDED(rv) && mdnGenerator)
+                {
+                    mimeHeaders =
+                        do_CreateInstance(NS_IMIMEHEADERS_CONTRACTID, &rv);
+                    if (NS_SUCCEEDED(rv) && mimeHeaders)
+                    {
+                        nsCOMPtr<nsIMsgFolder> msgFolder;
+                        msgHdr->GetFolder(getter_AddRefs(msgFolder));
+                        nsCOMPtr<nsIMsgMailNewsUrl> msgUrl =
+                            do_QueryInterface(m_runningUrl);
+                        if (msgUrl) 
+                        {
+                            nsCOMPtr<nsIMsgWindow> msgWindow;
+                            msgUrl->GetMsgWindow(getter_AddRefs(msgWindow));
+                            msgHdr->GetMessageKey(&msgKey);
+                            nsCOMPtr<nsIMimeHeaders> mimeHeaders;
+                            msgUrl->GetMimeHeaders(getter_AddRefs(mimeHeaders));
+                            mdnGenerator->Process(nsIMsgMdnGenerator::eDisplayed,
+                                                  msgWindow, msgFolder, msgKey,
+                                                  mimeHeaders, PR_FALSE);
+                            msgUrl->SetMimeHeaders(nsnull); 
+                            // no longer needed
+                        }
+                    }
+                }
+                // unsetting MDN_REPORT_NEEDED flag and mark the message as
+                // MDN_REPORT_SENT
+                // There are cases that: a) user wishes not to send MDN, b)
+                // mdn module is not installed, c) the message can be marked
+                // as unread and force it back to the original mdn
+                // needed state. 
+                msgHdr->SetFlags(msgFlags & ~MSG_FLAG_MDN_REPORT_NEEDED);
+                msgHdr->OrFlags(MSG_FLAG_MDN_REPORT_SENT, &newFlags);
+            }
 			msgHdr->MarkRead(PR_TRUE);
+        }
 	}
 
 	return rv;
