@@ -44,6 +44,7 @@
 #include "nsMsgMessageFlags.h"
 #include "nsIDBFolderInfo.h"
 #include "nsIInputStream.h"
+#include "nsILocalFile.h"
 #include "nsMsgLocalFolderHdrs.h"
 #include "nsMsgBaseCID.h"
 #include "nsMsgDBCID.h"
@@ -80,8 +81,7 @@ NS_IMPL_ISUPPORTS_INHERITED2(nsMsgMailboxParser, nsParseMailMessageState, nsIStr
 
 // Whenever data arrives from the connection, core netlib notifices the protocol by calling
 // OnDataAvailable. We then read and process the incoming data from the input stream. 
-NS_IMETHODIMP nsMsgMailboxParser::OnDataAvailable(nsIRequest *request, nsISupports *ctxt, nsIInputStream *aIStream, PRUint32 sourceOffset, 
-												  PRUint32 aLength)
+NS_IMETHODIMP nsMsgMailboxParser::OnDataAvailable(nsIRequest *request, nsISupports *ctxt, nsIInputStream *aIStream, PRUint32 sourceOffset, PRUint32 aLength)
 {
     // right now, this really just means turn around and process the url
     nsresult rv = NS_OK;
@@ -641,6 +641,14 @@ NS_IMETHODIMP nsParseMailMessageState::SetMailDB(nsIMsgDatabase *mailDB)
 	return NS_OK;
 }
 
+NS_IMETHODIMP nsParseMailMessageState::SetDBFolderStream(nsIOFileStream *fileStream)
+{
+  NS_ASSERTION(m_mailDB, "m_mailDB is not set");
+  if (m_mailDB)
+    m_mailDB->SetFolderStream(fileStream);
+  return NS_OK;
+}
+
 /* #define STRICT_ENVELOPE */
 
 PRBool
@@ -1058,7 +1066,8 @@ int nsParseMailMessageState::InternSubject (struct message_header *header)
 	 we just parsed the subject header anyway, we expect that parsing
 	 to be smartest.  (After all, what if someone just went in and
 	 edited the subject line by hand?) */
-	if (NS_MsgStripRE((const char **) &key, &L))
+  nsXPIDLCString modifiedSubject;
+	if (NS_MsgStripRE((const char **) &key, &L, getter_Copies(modifiedSubject)))
     flags |= MSG_FLAG_HAS_RE;
 	else
     flags &= ~MSG_FLAG_HAS_RE;
@@ -1068,11 +1077,12 @@ int nsParseMailMessageState::InternSubject (struct message_header *header)
 
 	// Condense the subject text into as few MIME-2 encoded words as possible.
 #ifdef WE_CONDENSE_MIME_STRINGS
-	char *condensedKey = msg_condense_mime2_string(key);
+  char *condensedKey = msg_condense_mime2_string(modifiedSubject.IsEmpty() ? key : modifiedSubject.get());
 #else
 	char *condensedKey = nsnull;
 #endif
-	m_newMsgHdr->SetSubject(condensedKey ? condensedKey : key);
+  m_newMsgHdr->SetSubject(condensedKey ? condensedKey : 
+                          (modifiedSubject.IsEmpty() ? key : modifiedSubject.get()));
 	PR_FREEIF(condensedKey);
 
 	return 0;
@@ -1205,7 +1215,8 @@ int nsParseMailMessageState::FinalizeHeaders()
       
       flags &= ~MSG_FLAG_OFFLINE; // don't keep MSG_FLAG_OFFLINE for local msgs
       if (mdn_dnt && !(origFlags & MSG_FLAG_READ) &&
-        !(origFlags & MSG_FLAG_MDN_REPORT_SENT))
+          !(origFlags & MSG_FLAG_MDN_REPORT_SENT) &&
+          !(flags & MSG_FLAG_MDN_REPORT_SENT))
         flags |= MSG_FLAG_MDN_REPORT_NEEDED;
       
       m_newMsgHdr->SetFlags(flags);
@@ -1442,56 +1453,10 @@ nsParseNewMailState::Init(nsIFolder *rootFolder, nsIMsgFolder *downloadFolder, n
     if (NS_SUCCEEDED(rv))
       rv = server->GetFilterList(aMsgWindow, getter_AddRefs(m_filterList));
     m_logFile = nsnull;
-#ifdef DOING_MDN
+   
 	if (m_filterList)
-	{
-		const char *folderName = nsnull;
-		PRInt32 int_pref = 0;
-		PREF_GetIntPref("mail.incorporate.return_receipt", &int_pref);
-		if (int_pref == 1)
-		{
-			nsIMsgFolder *folderInfo = nsnull;
-			int status = 0;
-			char *defaultFolderName =
-				msg_MagicFolderName(master->GetPrefs(),
-									MSG_FOLDER_FLAG_SENTMAIL, &status); 
-			if (defaultFolderName)
-			{
-				folderInfo = master->FindMailFolder(defaultFolderName, PR_FALSE);
-				if (folderInfo && folderInfo->GetMailFolderInfo())
-					folderName = folderInfo->GetMailFolderInfo()->GetPathname();
-				XP_FREE(defaultFolderName);
-			}
-		}
-		if (folderName)
-		{
-			MSG_Filter *newFilter = new MSG_Filter(filterInboxRule, "receipt");
-			if (newFilter)
-			{
-				MSG_Rule *rule = nsnull;
-				MSG_SearchValue value;
-				newFilter->SetDescription("incorporate mdn report");
-				newFilter->SetEnabled(PR_TRUE);
-				newFilter->GetRule(&rule);
-				newFilter->SetFilterList(m_filterList);
-				value.attribute = attribOtherHeader;
-				value.u.string = "multipart/report";
-				rule->AddTerm(attribOtherHeader, opContains,
-							  &value, PR_TRUE, "Content-Type");
-				value.u.string = "disposition-notification";
-				rule->AddTerm(attribOtherHeader, opContains,
-							  &value, PR_TRUE, "Content-Type");
-#if 0
-				value.u.string = "delivery-status";
-				rule->AddTerm(attribOtherHeader, opContains,
-							  &value, PR_FALSE, "Content-Type");
-#endif
-				rule->SetAction(nsMsgFilterActionMoveToFolder, (void*)folderName);
-				m_filterList->InsertFilterAt(0, newFilter);
-			}
-		}
-	}
-#endif // DOING_MDN
+      rv = server->ConfigureTemporaryReturnReceiptsFilter(m_filterList);
+
 	m_usingTempDB = PR_FALSE;
 	m_tmpdbName = nsnull;
 	m_disableFilters = PR_FALSE;
@@ -1597,13 +1562,14 @@ nsOutputFileStream * nsParseNewMailState::GetLogFile ()
     nsCOMPtr<nsIFile> logDir;
     NS_GetSpecialDirectory(NS_APP_MAIL_50_DIR, getter_AddRefs(logDir));
 
-    nsXPIDLCString pathBuf;
-    logDir->Append("filter.log");
-    nsresult rv = logDir->GetPath(getter_Copies(pathBuf));
+    logDir->AppendNative(NS_LITERAL_CSTRING("filter.log"));
+
+    nsCAutoString pathBuf;
+    nsresult rv = logDir->GetNativePath(pathBuf);
     if (NS_FAILED(rv)) return nsnull;
-    nsCOMPtr<nsIFileSpec> outSpec;
+
+		nsFileSpec logFile(pathBuf.get());
 
-		nsFileSpec logFile(pathBuf);
 		m_logFile = new nsOutputFileStream(logFile, PR_WRONLY | PR_CREATE_FILE, 00600);
 	}
 	return m_logFile;
@@ -1710,52 +1676,12 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
 			{
 				msgHdr->GetFlags(&msgFlags);
 
-				if (msgFlags & MSG_FLAG_MDN_REPORT_NEEDED &&
-					!isRead)
-				{
-					struct message_header to;
-					struct message_header cc;
-					GetAggregateHeader (m_toList, &to);
-					GetAggregateHeader (m_ccList, &cc);
-					msgHdr->SetFlags(msgFlags & ~MSG_FLAG_MDN_REPORT_NEEDED);
-					msgHdr->OrFlags(MSG_FLAG_MDN_REPORT_SENT, &newFlags);
-
-#if DOING_MDN	// leave it to the user action
-					if (actionType == nsMsgFilterActionDelete)
-					{
-						MSG_ProcessMdnNeededState processMdnNeeded
-							(MSG_ProcessMdnNeededState::eDeleted,
-							 m_pane, m_folder, msgHdr->GetMessageKey(),
-							 &state->m_return_path, &state->m_mdn_dnt, 
-							 &to, &cc, &state->m_subject, 
-							 &state->m_date, &state->m_mdn_original_recipient,
-							 &state->m_message_id, state->m_headers, 
-							 (PRInt32) state->m_headers_fp, PR_TRUE);
-					}
-					else
-					{
-						MSG_ProcessMdnNeededState processMdnNeeded
-							(MSG_ProcessMdnNeededState::eProcessed,
-							 m_pane, m_folder, msgHdr->GetMessageKey(),
-							 &state->m_return_path, &state->m_mdn_dnt, 
-							 &to, &cc, &state->m_subject, 
-							 &state->m_date, &state->m_mdn_original_recipient,
-							 &state->m_message_id, state->m_headers, 
-							 (PRInt32) state->m_headers_fp, PR_TRUE);
-					}
-#endif
-					char *tmp = (char*) to.value;
-					PR_FREEIF(tmp);
-					tmp = (char*) cc.value;
-					PR_FREEIF(tmp);
-				}
-				nsresult err = MoveIncorporatedMessage(msgHdr, m_mailDB, (const char *) actionTargetFolderUri, filter, msgWindow);
-				if (NS_SUCCEEDED(err))
-					m_msgMovedByFilter = PR_TRUE;
-
-			}
+        nsresult err = MoveIncorporatedMessage(msgHdr, m_mailDB, (const char *) actionTargetFolderUri, filter, msgWindow);
+        if (NS_SUCCEEDED(err))
+          m_msgMovedByFilter = PR_TRUE;
+      }
       *applyMore = PR_FALSE; 
-			break;
+      break;
 		case nsMsgFilterAction::MarkRead:
 			MarkFilteredMessageRead(msgHdr);
 			break;
@@ -1783,7 +1709,9 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
             {
                 nsMsgLabelValue filterLabel;
                 filter->GetActionLabel(&filterLabel);
-                msgHdr->SetLabel(filterLabel);
+                nsMsgKey msgKey;
+                msgHdr->GetMessageKey(&msgKey);
+                m_mailDB->SetLabel(msgKey, filterLabel);
             }
 			break;
 		default:
