@@ -62,6 +62,9 @@
 #include "nsMsgMimeCID.h"
 #include "prlog.h"
 #include "prprf.h"
+#include "nsIMimeHeaders.h"
+#include "nsIMsgWindow.h"
+#include "nsIMsgMailNewsUrl.h"
 
 static PRLogModuleInfo * gMimeEmitterLogModule = nsnull;
 
@@ -558,15 +561,15 @@ nsMimeBaseEmitter::UpdateCharacterSet(const char *aCharset)
         (PL_strcasecmp(aCharset, "ISO-8859-1")) &&
         (PL_strcasecmp(aCharset, "UTF-8")) )
   {
-    char    *contentType = nsnull;
+    nsCAutoString contentType;
     
-    if (NS_SUCCEEDED(mChannel->GetContentType(&contentType)) && contentType)
+    if (NS_SUCCEEDED(mChannel->GetContentType(contentType)) && !contentType.IsEmpty())
     {
-      char *cPtr = (char *) PL_strcasestr(contentType, "charset=");
+      char *cPtr = (char *) PL_strcasestr(contentType.get(), "charset=");
 
       if (cPtr)
       {
-        char  *ptr = contentType;
+        char  *ptr = (char *) contentType.get();
         while (*ptr)
         {
           if ( (*ptr == ' ') || (*ptr == ';') ) 
@@ -582,14 +585,9 @@ nsMimeBaseEmitter::UpdateCharacterSet(const char *aCharset)
         }
       }
 
-      char *newContentType = (char *) PR_smprintf("%s; charset=%s", contentType, aCharset);
-      if (newContentType)
-      {
-        mChannel->SetContentType(newContentType); 
-        PR_FREEIF(newContentType);
-      }
-      
-      PR_FREEIF(contentType);
+      // have to recompute strlen since contentType could have an embedded null byte
+      mChannel->SetContentType(nsDependentCString(contentType.get()));
+      mChannel->SetContentCharset(nsDependentCString(aCharset));
     }
   }
 
@@ -625,6 +623,22 @@ nsMimeBaseEmitter::AddHeaderField(const char *field, const char *value)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMimeBaseEmitter::AddAllHeaders(const char *allheaders, 
+                                 const PRInt32 allheadersize)
+{
+    nsresult rv;
+    nsCOMPtr<nsIMsgMailNewsUrl> msgurl (do_QueryInterface(mURL));
+    if (msgurl)
+    {
+        nsCOMPtr<nsIMimeHeaders> mimeHeaders = do_CreateInstance(NS_IMIMEHEADERS_CONTRACTID, &rv);
+        NS_ENSURE_SUCCESS(rv,rv);
+        mimeHeaders->Initialize(allheaders, allheadersize);
+        msgurl->SetMimeHeaders(mimeHeaders);
+    }
+    return NS_OK;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // The following code is responsible for formatting headers in a manner that is
 // identical to the normal XUL output.
