@@ -168,6 +168,7 @@ typedef enum {
                                   // DON'T remove.
 } nsMsgBoolHeaderSet;
 
+#ifdef NS_DEBUG
 extern "C" void
 mime_dump_attachments ( nsMsgAttachmentData *attachData )
 {
@@ -185,16 +186,17 @@ mime_dump_attachments ( nsMsgAttachmentData *attachData )
       printf("URL               : %s\n", spec.get());
     }
 
-    printf("Desired Type      : %s\n", tmp->desired_type);
-    printf("Real Type         : %s\n", tmp->real_type);
-    printf("Real Encoding     : %s\n", tmp->real_encoding); 
-    printf("Description       : %s\n", tmp->description);
-    printf("Mac Type          : %s\n", tmp->x_mac_type);
-    printf("Mac Creator       : %s\n", tmp->x_mac_creator);
+    printf("Desired Type      : %s\n", tmp->desired_type ? tmp->desired_type : "nsnull");
+    printf("Real Type         : %s\n", tmp->real_type ? tmp->real_type : "nsnull");
+    printf("Real Encoding     : %s\n", tmp->real_encoding ? tmp->real_encoding : "nsnull"); 
+    printf("Description       : %s\n", tmp->description) ? tmp->description : "nsnull";
+    printf("Mac Type          : %s\n", tmp->x_mac_type ? tmp->x_mac_type : "nsnull");
+    printf("Mac Creator       : %s\n", tmp->x_mac_creator ? tmp->x_mac_creator : "nsnull");
     i++;
     tmp++;
   }
 }
+#endif
 
 nsresult
 CreateTheComposeWindow(nsIMsgCompFields     *compFields,
@@ -306,7 +308,7 @@ CreateCompositionFields(const char        *from,
   NS_ENSURE_TRUE(cFields, NS_ERROR_OUT_OF_MEMORY);
   
   // Now set all of the passed in stuff...
-  cFields->SetCharacterSet(charset);
+  cFields->SetCharacterSet(!nsCRT::strcasecmp("us-ascii", charset) ? "ISO-8859-1" : charset);
   
   char *val;
   
@@ -518,7 +520,8 @@ mime_draft_process_attachments(mime_draft_data *mdd)
 
   //It's possible we must treat the message body as attachment!
   PRBool bodyAsAttachment = PR_FALSE;
-  if (  mdd->messageBody->type && *mdd->messageBody->type &&
+  if (  mdd->messageBody &&
+        mdd->messageBody->type && *mdd->messageBody->type &&
         ( PL_strcasestr(mdd->messageBody->type, "text/html") == nsnull ) &&
         ( PL_strcasestr(mdd->messageBody->type, "text/plain") == nsnull ) &&
         ( PL_strcasecmp(mdd->messageBody->type, "text") != 0 )
@@ -1151,7 +1154,7 @@ mime_insert_forwarded_message_headers(char            **body,
     // convert body from mail charset to UTF-8
     char *utf8 = NULL;
     nsAutoString ucs2;
-    if (NS_SUCCEEDED(nsMsgI18NConvertToUnicode(nsCAutoString(mailcharset), nsCAutoString(*body), ucs2))) {
+    if (NS_SUCCEEDED(ConvertToUnicode(mailcharset, *body, ucs2))) {
       utf8 = ToNewUTF8String(ucs2);
       if (NULL != utf8) {
         PR_Free(*body);
@@ -1348,7 +1351,11 @@ mime_parse_stream_complete (nsMIMESession *stream)
         int receiptType = 0;
         fields->SetReturnReceipt(PR_TRUE); 
         sscanf(parm, "%d", &receiptType);
-        fields->SetReturnReceipt((PRInt32) receiptType);
+        // slight change compared to 4.x; we used to use receipt= to tell
+        // whether the draft/template has request for either MDN or DNS or both
+        // return receipt; since the DNS is out of the picture we now use the
+        // header type - 1 to tell whether user has requested the return receipt
+        fields->SetReceiptHeaderType(((PRInt32)receiptType) - 1);
       }
       PR_FREEIF(parm);
       parm = MimeHeaders_get_parameter(draftInfo, "uuencode", NULL, NULL);
