@@ -35,9 +35,9 @@
  * the terms of any one of the NPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
+#include "prlog.h"
 #include "nsCOMPtr.h"
 #include "modlmime.h"
-#include "xp_core.h"
 #include "nsCRT.h"
 #include "mimeobj.h"
 #include "mimemsg.h"
@@ -294,6 +294,10 @@ GenerateAttachmentData(MimeObject *object, const char *aMessageURL, MimeDisplayO
   nsXPIDLCString part;
   PRBool isIMAPPart;
 
+  /* be sure the object has not be marked as Not to be an attachment */
+  if (object->dontShowAsAttachment)
+    return NS_OK;
+
   part.Adopt(mime_part_address(object));
   if (part.IsEmpty()) 
     return NS_ERROR_OUT_OF_MEMORY;
@@ -513,10 +517,13 @@ MimeGetAttachmentList(MimeObject *tobj, const char *aMessageURL, nsMsgAttachment
 
   if (!data) 
     return 0;
-  *data = NULL;
+  *data = nsnull;
 
   obj = mime_get_main_object(tobj);
-  if ( (obj) && (!mime_subclass_p(obj->clazz, (MimeObjectClass*) &mimeContainerClass)) )
+  if (nsnull == obj)
+    return 0;
+
+  if (!mime_subclass_p(obj->clazz, (MimeObjectClass*) &mimeContainerClass))
   {
     if (!PL_strcasecmp(obj->content_type, MESSAGE_RFC822))
       return 0;
@@ -524,8 +531,7 @@ MimeGetAttachmentList(MimeObject *tobj, const char *aMessageURL, nsMsgAttachment
       return ProcessBodyAsAttachment(obj, data);
   }
 
-  (void) MimeObjectChildIsMessageBody(obj, &isAlternativeOrRelated);
-  if (isAlternativeOrRelated)
+  if (mime_subclass_p(obj->clazz, (MimeObjectClass*) &mimeMultipartAlternativeClass))
     return 0;
   
   isAnInlineMessage = mime_typep(obj, (MimeObjectClass *) &mimeMessageClass);
@@ -913,8 +919,6 @@ mime_display_stream_complete (nsMIMESession *stream)
     //
     if (!msd->options->part_to_load)
     {
-      extern void  mime_dump_attachments ( nsMsgAttachmentData *attachData );
-
       nsMsgAttachmentData *attachments;
       nsresult rv = MimeGetAttachmentList(obj, msd->url_name, &attachments);
       if (NS_SUCCEEDED(rv))
@@ -1261,7 +1265,7 @@ mime_get_main_object(MimeObject* obj)
         obj = nsnull;
     }
   }
-  return NULL;
+  return nsnull;
 }
 
 PRBool MimeObjectChildIsMessageBody(MimeObject *obj, 
@@ -1735,6 +1739,26 @@ mimeEmitterAddHeaderField(MimeDisplayOptions *opt, const char *field, const char
   return NS_ERROR_FAILURE;
 }
 
+extern "C" nsresult     
+mimeEmitterAddAllHeaders(MimeDisplayOptions *opt, const char *allheaders, const PRInt32 allheadersize)
+{
+  // Check for draft processing...
+  if (NoEmitterProcessing(opt->format_out))
+    return NS_OK;
+
+  mime_stream_data  *msd = GetMSD(opt);
+  if (!msd) 
+    return NS_ERROR_FAILURE;
+
+  if (msd->output_emitter)
+  {
+    nsIMimeEmitter *emitter = (nsIMimeEmitter *)msd->output_emitter;
+    return emitter->AddAllHeaders(allheaders, allheadersize);
+  }
+
+  return NS_ERROR_FAILURE;
+}
+
 extern "C" nsresult     
 mimeEmitterStartAttachment(MimeDisplayOptions *opt, const char *name, const char *contentType, const char *url,
                            PRBool aNotDownloaded)
@@ -1975,11 +1999,11 @@ ResetChannelCharset(MimeObject *obj)
     char *ct = MimeHeaders_get (obj->headers, HEADER_CONTENT_TYPE, PR_FALSE, PR_FALSE); 
     if ( (ct) && (msd) && (msd->channel) ) 
     { 
-      char *ptr = PL_strstr(ct, "charset="); 
+      char *ptr = strstr(ct, "charset="); 
       if (ptr)
       {
         // First, setup the channel!
-        msd->channel->SetContentType(ct); 
+        msd->channel->SetContentType(nsDependentCString(ct));
 
         // Second, if this is a Save As operation, then we need to convert
         // to override the output charset!
