@@ -266,7 +266,7 @@ MimeMultipart_parse_line (char *line, PRInt32 length, MimeObject *obj)
         // If "multipart/alternative" or the first part is a message body
         // then we should check for a charset and notify the emitter  
         // if one exists.                                                     
-        if (obj->options && (isAlternative || isBody))
+        if (obj->options && ((isAlternative && mult->state != MimeMultipartSkipPartLine) || isBody))
         {
           {
            char *ct = MimeHeaders_get(mult->hdrs, HEADER_CONTENT_TYPE, PR_FALSE, PR_FALSE);
@@ -314,6 +314,10 @@ MimeMultipart_parse_line (char *line, PRInt32 length, MimeObject *obj)
 	  if (status < 0) return status;
 	  break;
 
+  case MimeMultipartSkipPartLine:
+    /* we are skipping that part, therefore just ignore the line */
+    break;
+
 	default:
 	  PR_ASSERT(0);
 	  return -1;
@@ -457,7 +461,7 @@ MimeMultipart_create_child(MimeObject *obj)
       struct mime_stream_data *msd = (struct mime_stream_data *)body->options->stream_closure;
       if (!body->options->write_html_p && body->content_type && !nsCRT::strcasecmp(body->content_type, APPLICATION_APPLEFILE))
 				if (msd && msd->channel)
-        	msd->channel->SetContentType(APPLICATION_APPLEFILE);
+        	msd->channel->SetContentType(NS_LITERAL_CSTRING(APPLICATION_APPLEFILE));
     }
 #endif
 
