@@ -22,6 +22,7 @@
  *     Daniel Veditz <dveditz@netscape.com>
  *     Samir Gehani <sgehani@netscape.com>
  *     Mitch Stoltz <mstoltz@netscape.com>
+ *     Jeroen Dobbelaere <jeroen.dobbelaere@acunia.com>
  */
 
 /* 
@@ -55,6 +56,9 @@
 #define BY4ALLOC_ITEMS 320
 nsRecyclingAllocator *gZlibAllocator = NULL;
 
+// For placement new used for arena allocations of zip file list
+#include <new.h>
+
 #else /* STANDALONE */
 
 #ifdef XP_WIN
@@ -507,6 +511,17 @@ PRInt32 nsZipArchive::CloseArchive()
 	mFd = 0;
   }
 
+#ifndef STANDALONE
+  // delete arena for filelist
+  PL_FinishArenaPool(&mArena);
+
+  // CAUTION:
+  // We dont need to delete each of the nsZipItem as the memory for
+  // the zip item and the filename it holds are both allocated from the Arena.
+  // Hence, destroying the Arena is like destroying all the memory
+  // for all the nsZipItem in one shot. But if the ~nsZipItem is doing
+  // anything more than cleaning up memory, we should start calling it.
+#else
   // delete nsZipItems in table
   nsZipItem* pItem;
   for ( int i = 0; i < ZIP_TABSIZE; ++i)
@@ -519,6 +534,8 @@ PRInt32 nsZipArchive::CloseArchive()
       pItem = mFiles[i];
     }
   }
+#endif
+
   return ZIP_OK;
 }
 
@@ -893,7 +910,7 @@ PRInt32 nsZipArchive::BuildFileList()
     PRUint32 endsig;
     PRBool bEndsigFound = PR_FALSE;
 
-    for (endp -= sizeof(ZipEnd); endp >= buf; endp--)
+    for (endp -= ZIPEND_SIZE; endp >= buf; endp--)
     {
 		endsig = xtolong(endp);
 		if (endsig == ENDSIG)
@@ -921,7 +938,7 @@ PRInt32 nsZipArchive::BuildFileList()
       status = ZIP_ERR_CORRUPT;
 
     //-- backward read must overlap ZipEnd length
-    pos += sizeof(ZipEnd);
+    pos += ZIPEND_SIZE;
 
   } /* while looking for end signature */
 
@@ -934,7 +951,7 @@ PRInt32 nsZipArchive::BuildFileList()
     //-- we think we found the central directory, read in the first chunk
     pos = 0;
     bufsize = PR_Read( mFd, &buf, sizeof(buf) );
-    if (bufsize < (PRInt32)(sizeof(ZipCentral) + sizeof(ZipEnd)))
+    if (bufsize < (PRInt32)(ZIPCENTRAL_SIZE + ZIPEND_SIZE))
     {
       // We know we read the end sig and got pointed at the central
       // directory--there should be at least this much
@@ -963,8 +980,15 @@ PRInt32 nsZipArchive::BuildFileList()
     PRUint32 namelen = xtoint( central->filename_len );
     PRUint32 extralen = xtoint( central->extrafield_len );
     PRUint32 commentlen = xtoint( central->commentfield_len );
-    
+#ifndef STANDALONE
+    // Arena allocate the nsZipItem
+    void *mem;
+    PL_ARENA_ALLOCATE(mem, &mArena, sizeof(nsZipItem));
+    // Use placement new to arena allcoate the nsZipItem
+    nsZipItem* item = mem ? new (mem) nsZipItem() : nsnull;
+#else
     nsZipItem* item = new nsZipItem();
+#endif
     if (!item)
     {
       status = ZIP_ERR_MEMORY;
@@ -991,12 +1015,24 @@ PRInt32 nsZipArchive::BuildFileList()
     item->time = xtoint( central->time );
     item->date = xtoint( central->date );
 
-    pos += sizeof(ZipCentral);
+    pos += ZIPCENTRAL_SIZE;
 
     //-------------------------------------------------------
     // get the item name
     //-------------------------------------------------------
+#ifndef STANDALONE
+    PL_ARENA_ALLOCATE(mem, &mArena, (namelen + 1));
+    item->name = (char *) mem;
+    if ( !item->name )
+    {
+      status = ZIP_ERR_MEMORY;
+      // No need to delete name. It gets deleted only when the entire arena
+      // goes away.
+      break;
+    }
+#else
     item->name = new char[namelen + 1];
+#endif
     if ( !item->name )
     {
       status = ZIP_ERR_MEMORY;
@@ -1034,7 +1070,7 @@ PRInt32 nsZipArchive::BuildFileList()
     // set up to process the next item at the top of loop
     //-------------------------------------------------------
     leftover = (PRUint32)(bufsize - pos);
-    if ( leftover < (extralen + commentlen + sizeof(ZipCentral)) )
+    if ( leftover < (extralen + commentlen + ZIPCENTRAL_SIZE) )
     {
       //-- not enough data left to process at top of loop.
       //-- move leftover and read more
@@ -1055,13 +1091,20 @@ PRInt32 nsZipArchive::BuildFileList()
     }
 
     //-- make sure we've read enough
-    if ( (PRUint32)bufsize < pos + sizeof(ZipCentral) )
+    if ( bufsize < pos + ZIPCENTRAL_SIZE )
     {
       status = ZIP_ERR_CORRUPT;
       break;
     }
   } /* while reading central directory records */
 
+#if defined(DEBUG)
+  if(status != ZIP_OK) {
+    const char* msgs[] = { "ZIP_OK", "ZIP_ERR_GENERAL", "ZIP_ERR_MEMORY", "ZIP_ERR_DISK", "ZIP_ERR_CORRUPT", "ZIP_ERR_PARAM", "ZIP_ERR_FNF", "ZIP_ERR_UNSUPPORTED", "ZIP_ERR_SMALLBUF", "UNKNOWN" };
+    printf("nsZipArchive::BuildFileList  status = %d '%s'\n", (int)status, msgs[(status <= 0 && status >= -8) ? -status : 9]);
+  }
+#endif
+
   return status;
 }
   
@@ -1122,14 +1165,14 @@ PRInt32  nsZipArchive::SeekToItem(const nsZipItem* aItem)
       return ZIP_ERR_CORRUPT;
 
     ZipLocal   Local;
-    if ( PR_Read(mFd, (char*)&Local, sizeof(ZipLocal)) != (READTYPE)sizeof(ZipLocal)
+    if ( PR_Read(mFd, (char*)&Local, ZIPLOCAL_SIZE) != (READTYPE) ZIPLOCAL_SIZE
          || xtolong( Local.signature ) != LOCALSIG )
     {
       //-- read error or local header not found
       return ZIP_ERR_CORRUPT;
     }
 
-    ((nsZipItem*)aItem)->offset += sizeof(Local) +
+    ((nsZipItem*)aItem)->offset += ZIPLOCAL_SIZE +
                                    xtoint( Local.filename_len ) +
                                    xtoint( Local.extrafield_len );
     ((nsZipItem*)aItem)->flags |= ZIFLAG_DATAOFFSET;
@@ -1275,7 +1318,7 @@ PRInt32 nsZipArchive::InflateItem( const nsZipItem* aItem, PRFileDesc* fOut,
 #ifndef STANDALONE
   //-- ensure we have our zlib allocator for better performance
   if (!gZlibAllocator) {
-    gZlibAllocator = new nsRecyclingAllocator(NBUCKETS);
+    gZlibAllocator = new nsRecyclingAllocator(NBUCKETS, NS_DEFAULT_RECYCLE_TIMEOUT, "libjar");
   }
 
   zs.zalloc = zlibAlloc;
@@ -1574,15 +1617,17 @@ cleanup:
 MOZ_DECL_CTOR_COUNTER(nsZipArchive)
 
 nsZipArchive::nsZipArchive()
-  : kMagic(ZIP_MAGIC), mFd(0)
+  : kMagic(ZIP_MAGIC), kArenaBlockSize(1*1024), mFd(0)
 {
   MOZ_COUNT_CTOR(nsZipArchive);
 
   // initialize the table to NULL
-  for ( int i = 0; i < ZIP_TABSIZE; ++i) {
-    mFiles[i] = 0;
-  }
+  memset(mFiles, 0, sizeof mFiles);
 
+#ifndef STANDALONE
+  // Initialize our arena
+  PL_INIT_ARENA_POOL(&mArena, "ZipArena", kArenaBlockSize);
+#endif
 }
 
 nsZipArchive::~nsZipArchive()
@@ -1599,22 +1644,19 @@ nsZipArchive::~nsZipArchive()
 // nsZipItem constructor and destructor
 //------------------------------------------
 
-MOZ_DECL_CTOR_COUNTER(nsZipItem)
-
 nsZipItem::nsZipItem() : name(0), offset(0), next(0), flags(0)
 {
-  MOZ_COUNT_CTOR(nsZipItem);
 }
 
 nsZipItem::~nsZipItem()
 {
+#ifdef STANDALONE
   if (name != 0 )
   {
     delete [] name;
     name = 0;
   }
-
-  MOZ_COUNT_DTOR(nsZipItem);
+#endif
 }
 
 //------------------------------------------
