@@ -42,9 +42,11 @@
 #include "nsDirectoryServiceDefs.h"
 #include "nsICategoryManager.h"
 #include "nsIFileStreams.h"
+#include "nsILocalFile.h"
 #include "nsIObserverService.h"
 #include "nsPrefBranch.h"
 #include "nsXPIDLString.h"
+#include "nsCRT.h"
 
 #include "nsQuickSort.h"
 #include "prmem.h"
@@ -57,7 +59,7 @@ class nsIFileSpec;	// needed for prefapi_private_data.h inclusion
 // supporting PREF_Init()
 #include "nsIJSRuntimeService.h"
 
-// lose these if possible (supporting nsIFileToFileSpec)
+// lose these if possible (need NS_NewFileSpecFromIFile)
 #include "nsIFileSpec.h"
 #include "nsFileStream.h"
 
@@ -70,7 +72,6 @@ class nsIFileSpec;	// needed for prefapi_private_data.h inclusion
 
 
 // Prototypes
-static nsresult nsIFileToFileSpec(nsIFile* inFile, nsIFileSpec **aFileSpec);
 static nsresult openPrefFile(nsIFile* aFile, PRBool aIsErrorFatal,
                              PRBool aIsGlobalContext, PRBool aSkipFirstLine);
 static nsresult savePrefFile(nsIFile* aFile);
@@ -153,6 +154,9 @@ nsresult nsPrefService::Init()
     rv = observerService->AddObserver(this, "profile-before-change", PR_TRUE);
     if (NS_SUCCEEDED(rv)) {
       rv = observerService->AddObserver(this, "profile-do-change", PR_TRUE);
+      if (NS_SUCCEEDED(rv)) {
+        rv = observerService->AddObserver(this, "session-logout", PR_TRUE);
+      }
     }
   }
   return(rv);
@@ -174,6 +178,8 @@ NS_IMETHODIMP nsPrefService::Observe(nsISupports *aSubject, const char *aTopic,
   } else if (!nsCRT::strcmp(aTopic, "profile-do-change")) {
     ResetUserPrefs();
     rv = ReadUserPrefs(nsnull);
+  } else if (!nsCRT::strcmp(aTopic, "session-logout")) {
+    rv = SavePrefFile(nsnull);
   }
   return rv;
 }
@@ -318,7 +324,7 @@ nsresult nsPrefService::useDefaultPrefFile()
     // knows about NS_APP_PREFS_50_FILE. Put the file in NS_XPCOM_CURRENT_PROCESS_DIR.
     rv = NS_GetSpecialDirectory(NS_XPCOM_CURRENT_PROCESS_DIR, getter_AddRefs(aFile));
     if (NS_FAILED(rv)) return rv;
-    rv = aFile->Append("default_prefs.js");
+    rv = aFile->AppendNative(NS_LITERAL_CSTRING("default_prefs.js"));
     if (NS_FAILED(rv)) return rv;
   } 
 
@@ -338,11 +344,9 @@ nsresult nsPrefService::useUserPrefFile()
   nsresult rv = NS_OK;
   nsCOMPtr<nsIFile> aFile;
 
-  static const char* userFiles[] = {"user.js"};
-
   rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(aFile));
   if (NS_SUCCEEDED(rv) && aFile) {
-    rv = aFile->Append(userFiles[0]);
+    rv = aFile->AppendNative(NS_LITERAL_CSTRING("user.js"));
     if (NS_SUCCEEDED(rv)) {
       rv = openPrefFile(aFile, PR_FALSE, PR_FALSE, PR_FALSE);
     }
@@ -361,8 +365,8 @@ static nsresult openPrefFile(nsIFile* aFile, PRBool aIsErrorFatal,
 
 #if MOZ_TIMELINE
   {
-    nsXPIDLCString str;
-    aFile->GetPath(getter_Copies(str));
+    nsCAutoString str;
+    aFile->GetNativePath(str);
     NS_TIMELINE_MARK_FUNCTION1("load pref file", str.get());
   }
 #endif
@@ -415,7 +419,7 @@ static nsresult savePrefFile(nsIFile* aFile)
     return NS_OK;
 
   // TODO: Convert the rest of this code to nsIFile and avoid this conversion to nsIFileSpec
-  rv = nsIFileToFileSpec(aFile, getter_AddRefs(fileSpec));
+  rv = NS_NewFileSpecFromIFile(aFile, getter_AddRefs(fileSpec));
   if (NS_FAILED(rv))
     return rv;        
 
@@ -447,30 +451,6 @@ static nsresult savePrefFile(nsIFile* aFile)
 }
 
 
-//----------------------------------------------------------------------------------------
-// So discouraged is the use of nsIFileSpec, nobody wanted to have this routine be
-// public - It might lead to continued use of nsIFileSpec. Right now, this code has
-// such a need for it, here it is. Let's stop having to use it though.
-static nsresult nsIFileToFileSpec(nsIFile* inFile, nsIFileSpec **aFileSpec)
-//----------------------------------------------------------------------------------------
-{
-   nsresult rv;
-   nsCOMPtr<nsIFileSpec> newFileSpec;
-   nsXPIDLCString pathBuf;
-   
-   rv = inFile->GetPath(getter_Copies(pathBuf));
-   if (NS_FAILED(rv)) return rv;
-   rv = NS_NewFileSpec(getter_AddRefs(newFileSpec));
-   if (NS_FAILED(rv)) return rv;
-   rv = newFileSpec->SetNativePath((const char *)pathBuf);
-   if (NS_FAILED(rv)) return rv;
-   
-   *aFileSpec = newFileSpec;
-   NS_ADDREF(*aFileSpec);
-   
-   return NS_OK;
-}
-
 /*
  * some stuff that gets called from Pref_Init()
  */
@@ -479,28 +459,24 @@ static nsresult nsIFileToFileSpec(nsIFile* inFile, nsIFileSpec **aFileSpec)
 static int PR_CALLBACK
 inplaceSortCallback(const void *data1, const void *data2, void *privateData)
 {
-  char *name1 = nsnull;
-  char *name2 = nsnull;
+  nsCAutoString name1;
+  nsCAutoString name2;
   nsIFile *file1= *(nsIFile **)data1;
   nsIFile *file2= *(nsIFile **)data2;
   nsresult rv;
   int sortResult = 0;
 
-  rv = file1->GetLeafName(&name1);
+  rv = file1->GetNativeLeafName(name1);
   NS_ASSERTION(NS_SUCCEEDED(rv),"failed to get the leaf name");
   if (NS_SUCCEEDED(rv)) {
-    rv = file2->GetLeafName(&name2);
+    rv = file2->GetNativeLeafName(name2);
     NS_ASSERTION(NS_SUCCEEDED(rv),"failed to get the leaf name");
     if (NS_SUCCEEDED(rv)) {
-      if (name1 && name2) {
+      if (!name1.IsEmpty() && !name2.IsEmpty()) {
         // we want it so foo.js will come before foo-<bar>.js
         // "foo." is before "foo-", so we have to reverse the order to accomplish
-        sortResult = PL_strcmp(name2,name1);
+        sortResult = Compare(name2, name1); // XXX i18n
       }
-      if (name1)
-        nsCRT::free((char*)name1);
-      if (name2)
-        nsCRT::free((char*)name2);
     }
   }
   return sortResult;
@@ -524,6 +500,9 @@ JSBool pref_InitInitialObjects()
 #endif
 #ifdef XP_MAC
       , "macprefs.js"
+#if defined (TARGET_CARBON)
+      , "macxprefs.js"
+#endif
 #elif defined(XP_WIN)
       , "winpref.js"
 #elif defined(XP_UNIX)
@@ -567,7 +546,7 @@ JSBool pref_InitInitialObjects()
   if (NS_FAILED(rv))
     return JS_FALSE;
 
-  rv = aFile->Append((char *)specialFiles[0]);
+  rv = aFile->AppendNative(nsDependentCString((char *)specialFiles[0]));
   if (NS_FAILED(rv))
     return JS_FALSE;
 
@@ -578,20 +557,20 @@ JSBool pref_InitInitialObjects()
 
   while (hasMoreElements) {
     PRBool shouldParse = PR_TRUE;
-    char* leafName;
+    nsCAutoString leafName;
 
     dirIterator->GetNext(getter_AddRefs(aFile));
     dirIterator->HasMoreElements(&hasMoreElements);
 
-    rv = aFile->GetLeafName(&leafName);
+    rv = aFile->GetNativeLeafName(leafName);
     if (NS_SUCCEEDED(rv)) {
       // Skip non-js files
-      if (PL_strstr(leafName, ".js") + PL_strlen(".js") != leafName + PL_strlen(leafName))
+      if ((leafName.Length() < 3) || !Substring(leafName, leafName.Length() - 3, 3).Equals(NS_LITERAL_CSTRING(".js")))
         shouldParse = PR_FALSE;
       // Skip files in the special list.
       if (shouldParse) {
         for (int j = 0; j < (int) (sizeof(specialFiles) / sizeof(char *)); j++)
-          if (!PL_strcmp(leafName, specialFiles[j]))
+          if (!strcmp(leafName.get(), specialFiles[j]))
             shouldParse = PR_FALSE;
       }
       if (shouldParse) {
@@ -605,8 +584,6 @@ JSBool pref_InitInitialObjects()
           }
         }
       }
-      if (leafName)
-        nsCRT::free((char*)leafName);
     }
   };
 
@@ -626,7 +603,7 @@ JSBool pref_InitInitialObjects()
     // because SetLeafName will not work here.
     rv = defaultPrefDir->Clone(getter_AddRefs(aFile));
     if (NS_SUCCEEDED(rv)) {
-      rv = aFile->Append((char*)specialFiles[k]);
+      rv = aFile->AppendNative(nsDependentCString(specialFiles[k]));
       if (NS_SUCCEEDED(rv)) {
         rv = openPrefFile(aFile, PR_FALSE, PR_FALSE, PR_FALSE);
         NS_ASSERTION(NS_SUCCEEDED(rv), "<platform>.js was not parsed successfully");
