@@ -169,9 +169,10 @@ private:
   nsCOMPtr<nsITimer> mPluginTimer;
 };
 
-  static void GetWidgetPosAndClip(nsIWidget* aWidget,nscoord& aAbsX, nscoord& aAbsY,
-                                nsRect& aClipRect);
-
+#ifdef XP_MAC
+  static void GetWidgetPosAndClip(nsIWidget* aWidget,nscoord& aAbsX, nscoord& aAbsY, 
+                                  nsRect& aClipRect);
+#endif
 
 class PluginViewerImpl : public nsIPluginViewer,
                          public nsIContentViewer,
@@ -344,10 +345,9 @@ PluginViewerImpl::StartLoad(nsIRequest* request, nsIStreamListener*& aResult)
   NS_ADDREF(mChannel);
 
 #ifdef DEBUG
-  char* contentType;
-  channel->GetContentType(&contentType);
-  printf("PluginViewerImpl::StartLoad: content-type=%s\n", contentType);
-  nsCRT::free(contentType);
+  nsCAutoString contentType;
+  channel->GetContentType(contentType);
+  printf("PluginViewerImpl::StartLoad: content-type=%s\n", contentType.get());
 #endif
 
   aResult = nsnull;
@@ -377,6 +377,13 @@ PluginViewerImpl::StartLoad(nsIRequest* request, nsIStreamListener*& aResult)
     nsRect r;
     mWindow->GetClientBounds(r);
     rv = CreatePlugin(request, host, nsRect(0, 0, r.width, r.height), aResult);
+
+#ifdef XP_MAC
+    // On Mac, we need to initiate the intial invalidate for full-page plugins to ensure
+    // the entire window gets cleared. Otherwise, Acrobat won't initially repaint on top 
+    // of our previous presentation and we may have garbage leftover
+    mWindow->Invalidate(PR_FALSE);
+#endif
   }
 
   return rv;
@@ -415,14 +422,12 @@ PluginViewerImpl::CreatePlugin(nsIRequest* request, nsIPluginHost* aHost, const
     if (NS_FAILED(rv)) return rv;
     NS_ConvertUTF8toUCS2 str(spec);
 
-    char* ct;
+    nsCAutoString ct;
   
     nsCOMPtr<nsIChannel> channel = do_QueryInterface(request);
-    channel->GetContentType(&ct);
+    channel->GetContentType(ct);
     if (NS_FAILED(rv)) return rv;
-    rv = aHost->InstantiateFullPagePlugin(ct, str, aResult, mOwner);
-   
-    delete[] ct;
+    rv = aHost->InstantiateFullPagePlugin(ct.get(), str, aResult, mOwner);
   }
 
   return rv;
@@ -541,7 +546,7 @@ HandlePluginEvent(nsGUIEvent *aEvent)
     void *clientData;
     (nsIWidget*)(aEvent->widget)->GetClientData(clientData);
     PluginViewerImpl * pluginViewer = (PluginViewerImpl *)clientData;
-    if (pluginViewer != nsnull && pluginViewer->mOwner != nsnull)
+    if (pluginViewer && pluginViewer->mOwner && pluginViewer->mWindow == aEvent->widget)
       return pluginViewer->mOwner->ProcessEvent(*aEvent);
 #endif // XP_MAC
   }
@@ -617,7 +622,10 @@ PluginViewerImpl::SetBounds(const nsRect& aBounds)
         win->clipRect.left = aBounds.x;
         win->clipRect.bottom = aBounds.YMost();
         win->clipRect.right = aBounds.XMost();
-
+        
+#ifdef XP_MAC   // On Mac we also need to add in the widget offset to the plugin window
+        mOwner->FixUpPluginWindow();
+#endif        
         inst->SetWindow(win);
       }
       NS_RELEASE(inst);
@@ -643,6 +651,9 @@ PluginViewerImpl::Move(PRInt32 aX, PRInt32 aY)
         win->clipRect.top = aY;
         win->clipRect.left = aX;
 
+#ifdef XP_MAC   // On Mac we also need to add in the widget offset to the plugin window
+        mOwner->FixUpPluginWindow();
+#endif
         inst->SetWindow(win);
       }
       NS_RELEASE(inst);
@@ -843,18 +854,6 @@ PluginViewerImpl::GetGlobalPrintSettings(nsIPrintSettings * *aGlobalPrintSetting
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-/* attribute nsIPrintSettings globalPrintSettingsValues; */
-NS_IMETHODIMP 
-PluginViewerImpl::GetGlobalPrintSettingsValues(nsIPrintSettings * *aGlobalPrintSettingsValues)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP 
-PluginViewerImpl::SetGlobalPrintSettingsValues(nsIPrintSettings * aGlobalPrintSettingsValues)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
 /* void printPreview (in nsIPrintSettings aThePrintSettings); */
 NS_IMETHODIMP 
 PluginViewerImpl::PrintPreview(nsIPrintSettings *aThePrintSettings)
@@ -876,6 +875,13 @@ PluginViewerImpl::GetDoingPrintPreview(PRBool *aDoingPrintPreview)
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+/* readonly attribute nsIPrintSettings currentPrintSettings; */
+NS_IMETHODIMP 
+PluginViewerImpl::GetCurrentPrintSettings(nsIPrintSettings * *aCurrentPrintSettings)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 /* void cancel (); */
 NS_IMETHODIMP 
 PluginViewerImpl::Cancel()
@@ -883,6 +889,20 @@ PluginViewerImpl::Cancel()
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+/* void initPrintSettingsFromPrefs (in nsIPrintSettings aPrintSettings, in boolean aUsePrinterNamePrefix, in unsigned long aFlags); */
+NS_IMETHODIMP 
+PluginViewerImpl::InitPrintSettingsFromPrefs(nsIPrintSettings *aPrintSettings, PRBool aUsePrinterNamePrefix, PRUint32 aFlags)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* void savePrintSettingsToPrefs (in nsIPrintSettings aPrintSettings, in boolean aUsePrinterNamePrefix, in unsigned long aFlags); */
+NS_IMETHODIMP 
+PluginViewerImpl::SavePrintSettingsToPrefs(nsIPrintSettings *aPrintSettings, PRBool aUsePrinterNamePrefix, PRUint32 aFlags)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 /* void enumerateDocumentNames (out PRUint32 aCount, [array, size_is (aCount), retval] out wstring aResult); */
 NS_IMETHODIMP 
 PluginViewerImpl::EnumerateDocumentNames(PRUint32 *aCount, PRUnichar ***aResult)
@@ -897,6 +917,27 @@ PluginViewerImpl::GetIsFramesetDocument(PRBool *aIsFramesetDocument)
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+/* readonly attribute boolean isIFrameSelected; */
+NS_IMETHODIMP 
+PluginViewerImpl::GetIsIFrameSelected(PRBool *aIsIFrameSelected)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* readonly attribute boolean isRangeSelection; */
+NS_IMETHODIMP 
+PluginViewerImpl::GetIsRangeSelection(PRBool *aIsRangeSelection)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* readonly attribute boolean isFramesetFrameSelected; */
+NS_IMETHODIMP 
+PluginViewerImpl::GetIsFramesetFrameSelected(PRBool *aIsFramesetFrameSelected)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 /* readonly attribute long printPreviewNumPages; */
 NS_IMETHODIMP 
 PluginViewerImpl::GetPrintPreviewNumPages(PRInt32 *aPrintPreviewNumPages)
@@ -967,14 +1008,7 @@ NS_IMETHODIMP
 PluginListener::OnStartRequest(nsIRequest *request, nsISupports *ctxt)
 {
   nsresult rv;
-  char* contentType = nsnull;
 
-  nsCOMPtr<nsIChannel> channel = do_QueryInterface(request);
-  rv = channel->GetContentType(&contentType);
-
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
   rv = mViewer->StartLoad(request, mNextStream);
 
   if (NS_FAILED(rv)) {
@@ -1218,92 +1252,80 @@ NS_IMETHODIMP pluginInstanceOwner :: Init(PluginViewerImpl *aViewer, nsIWidget *
 #ifdef XP_MAC
 
 #if TARGET_CARBON
-inline Boolean OSEventAvail(EventMask mask, EventRecord* event) { return EventAvail(mask, event); }
+static void InitializeEventRecord(EventRecord* event)
+{
+    memset(event, 0, sizeof(EventRecord));
+    GetGlobalMouse(&event->where);
+    event->when = TickCount();
+    event->modifiers = GetCurrentKeyModifiers();
+}
+#else
+inline void InitializeEventRecord(EventRecord* event) { ::OSEventAvail(0, event); }
 #endif
 
 void pluginInstanceOwner::GUItoMacEvent(const nsGUIEvent& anEvent, EventRecord& aMacEvent)
 {
-       ::OSEventAvail(0, &aMacEvent);
-       switch (anEvent.message) {
-       case NS_GOTFOCUS:
-       case NS_FOCUS_EVENT_START:
-               aMacEvent.what = nsPluginEventType_GetFocusEvent;
-               break;
-       case NS_LOSTFOCUS:
-       case NS_MOUSE_EXIT:
-               aMacEvent.what = nsPluginEventType_LoseFocusEvent;
-               break;
-       case NS_MOUSE_MOVE:
-       case NS_MOUSE_ENTER:
-         mWindow->SetFocus();
-               aMacEvent.what = nsPluginEventType_AdjustCursorEvent;
-               break;
-       case NS_PAINT:
-          aMacEvent.what = updateEvt;
-         break;
-       case NS_KEY_DOWN:
-       case NS_KEY_PRESS:
-         break;
-         
-       default:
-               aMacEvent.what = nullEvent;
-               break;
-       }
+    InitializeEventRecord(&aMacEvent);
+    switch (anEvent.message) {
+    case NS_GOTFOCUS:
+    case NS_FOCUS_EVENT_START:
+        aMacEvent.what = nsPluginEventType_GetFocusEvent;
+        break;
+    case NS_LOSTFOCUS:
+    case NS_MOUSE_EXIT:
+        aMacEvent.what = nsPluginEventType_LoseFocusEvent;
+        break;
+    case NS_MOUSE_MOVE:
+    case NS_MOUSE_ENTER:
+        mWindow->SetFocus();
+        aMacEvent.what = nsPluginEventType_AdjustCursorEvent;
+        break;
+    case NS_PAINT:
+        aMacEvent.what = updateEvt;
+        break;
+    case NS_KEY_DOWN:
+    case NS_KEY_PRESS:
+        break;
+
+    default:
+        aMacEvent.what = nullEvent;
+        break;
+    }
 }
 #endif
 
 nsEventStatus pluginInstanceOwner::ProcessEvent(const nsGUIEvent& anEvent)
 {
   nsEventStatus rv = nsEventStatus_eIgnore;
-  if (!mInstance || !mWindow)   // if the instance or the window is null, we shouldn't be here
+  if (!mInstance || !mWindow || anEvent.message == NS_MENU_SELECTED)
     return rv;
 
 #ifdef XP_MAC
     //if (mWidget != NULL) {  // check for null mWidget
         EventRecord* event = (EventRecord*)anEvent.nativeMsg;
         if (event == NULL || event->what == nullEvent ||
-            anEvent.message == NS_KEY_PRESS           ||
             anEvent.message == NS_CONTEXTMENU_MESSAGE_START) {
             EventRecord macEvent;
             GUItoMacEvent(anEvent, macEvent);
             event = &macEvent;
             if (event->what == updateEvt) {
-              nsPluginPort* pluginPort = GetPluginPort();
+                nsPluginPort* pluginPort = GetPluginPort();
                 // Add in child windows absolute position to get make the dirty rect
                 // relative to the top-level window.
-                nscoord absWidgetX = 0;
-                nscoord absWidgetY = 0;
-               nsRect widgetClip(0,0,0,0);
-               GetWidgetPosAndClip(mWindow,absWidgetX,absWidgetY,widgetClip);
-               //mViewer->GetBounds(widgetClip);
-               //absWidgetX = widgetClip.x;
-               //absWidgetY = widgetClip.y;
-               
-                // set the port
-    mPluginWindow.x = absWidgetX;
-    mPluginWindow.y = absWidgetY;
+                FixUpPluginWindow();
+                
+                EventRecord updateEvent;
+                InitializeEventRecord(&updateEvent);
+                updateEvent.what = updateEvt;
+                updateEvent.message = UInt32(pluginPort->port);
 
-
-    // fix up the clipping region
-    mPluginWindow.clipRect.top = widgetClip.y;
-    mPluginWindow.clipRect.left = widgetClip.x;
-    mPluginWindow.clipRect.bottom =  mPluginWindow.clipRect.top + widgetClip.height;
-    mPluginWindow.clipRect.right =  mPluginWindow.clipRect.left + widgetClip.width;  
-
-    EventRecord updateEvent;
-    ::OSEventAvail(0, &updateEvent);
-    updateEvent.what = updateEvt;
-    updateEvent.message = UInt32(pluginPort->port);
-
-    nsPluginEvent pluginEvent = { &updateEvent, nsPluginPlatformWindowRef(pluginPort->port) };
-    PRBool eventHandled = PR_FALSE;
-    mInstance->HandleEvent(&pluginEvent, &eventHandled);
-     }
-            
-            return  nsEventStatus_eConsumeNoDefault;
+                nsPluginEvent pluginEvent = { &updateEvent, nsPluginPlatformWindowRef(pluginPort->port) };
+                PRBool eventHandled = PR_FALSE;
+                mInstance->HandleEvent(&pluginEvent, &eventHandled);
+            }
+            return nsEventStatus_eConsumeNoDefault;
             
         }
-        //nsPluginPort* port = (nsPluginPort*)mWidget->GetNativeData(NS_NATIVE_PLUGIN_PORT);
         nsPluginPort* port = (nsPluginPort*)mWindow->GetNativeData(NS_NATIVE_PLUGIN_PORT);
         nsPluginEvent pluginEvent = { event, nsPluginPlatformWindowRef(port->port) };
         PRBool eventHandled = PR_FALSE;
@@ -1316,10 +1338,12 @@ nsEventStatus pluginInstanceOwner::ProcessEvent(const nsGUIEvent& anEvent)
 //~~~
 #ifdef XP_WIN
         nsPluginEvent * pPluginEvent = (nsPluginEvent *)anEvent.nativeMsg;
-        PRBool eventHandled = PR_FALSE;
-        mInstance->HandleEvent(pPluginEvent, &eventHandled);
-        if (eventHandled)
-            rv = nsEventStatus_eConsumeNoDefault;
+        if (pPluginEvent) {
+            PRBool eventHandled = PR_FALSE;
+            mInstance->HandleEvent(pPluginEvent, &eventHandled);
+            if (eventHandled)
+                rv = nsEventStatus_eConsumeNoDefault;
+        }
 #endif
 
   return rv;
@@ -1337,7 +1361,7 @@ NS_IMETHODIMP_(void) pluginInstanceOwner::Notify(nsITimer* /* timer */)
     FixUpPluginWindow();
     if (mInstance != NULL) {
         EventRecord idleEvent;
-        ::OSEventAvail(0, &idleEvent);
+        InitializeEventRecord(&idleEvent);
         idleEvent.what = nullEvent;
         
         nsPluginPort* pluginPort = GetPluginPort();
