@@ -116,17 +116,16 @@ nsFileIO::nsFileIO()
     if (nsnull == gFileIOLog) {
         gFileIOLog = PR_NewLogModule("nsFileIO");
     }
-
-    mSpec = nsnull;
 #endif /* PR_LOGGING */
 }
 
 nsFileIO::~nsFileIO()
 {
     (void)Close(NS_OK);
-#ifdef PR_LOGGING
-    if (mSpec) nsCRT::free(mSpec);
-#endif      
+    if (mFD) {
+        PR_Close(mFD);
+        mFD = nsnull;
+    }
 }
 
 NS_METHOD
@@ -154,7 +153,7 @@ nsFileIO::Init(nsIFile* file, PRInt32 ioFlags, PRInt32 perm)
     mIOFlags = ioFlags;
     mPerm = perm;
 #ifdef PR_LOGGING
-    nsresult rv = mFile->GetPath(&mSpec);
+    nsresult rv = mFile->GetNativePath(mSpec);
     NS_ASSERTION(NS_SUCCEEDED(rv), "GetSpec failed");
 #endif      
     return NS_OK;
@@ -169,17 +168,12 @@ nsFileIO::GetFile(nsIFile* *aFile)
 }
 
 NS_IMETHODIMP
-nsFileIO::Open(char **contentType, PRInt32 *contentLength)
+nsFileIO::Open()
 {
     NS_ASSERTION(mFile, "File must not be null");
     if (mFile == nsnull)
         return NS_ERROR_NOT_INITIALIZED;
 
-    if (contentLength)
-        *contentLength = 0;
-    if (contentType)
-        *contentType = nsnull;
-
     nsresult rv = NS_OK;
     nsCOMPtr<nsILocalFile> localFile = do_QueryInterface(mFile, &rv);
     if (NS_FAILED(rv)) return rv;
@@ -190,56 +184,85 @@ nsFileIO::Open(char **contentType, PRInt32 *contentLength)
 
     rv = localFile->OpenNSPRFileDesc(mIOFlags, mPerm, &mFD);
     if (NS_FAILED(rv)) {
+        mFD = nsnull; // just in case
+#ifdef PR_LOGGING
+        nsresult openError = rv;
+#endif
+        // maybe we can't open this because it is a directory...
         PRBool isDir;
         rv = localFile->IsDirectory(&isDir);
         if (NS_SUCCEEDED(rv) && isDir) {
-            // Directories turn into an HTTP-index stream, with
-            // unbounded (i.e., read 'til the stream says it's done)
-            // length.
-            if (contentType)
-                *contentType = nsCRT::strdup(APPLICATION_HTTP_INDEX_FORMAT);
-            if (contentLength) 
-                *contentLength = -1;
             return NS_OK;
         }
+        PR_LOG(gFileIOLog, PR_LOG_DEBUG,
+               ("nsFileIO: OpenNSPRFileDesc failed [rv=%x]\n", openError));
         return NS_ERROR_FILE_NOT_FOUND;
     }
 
-    if (contentLength) {
-        // We'll try to use the file's length, if it has one. If not,
-        // assume the file to be special, and set the content length
-        // to -1, which means "read the stream until exhausted".
-        PRInt64 size;
-        rv = mFile->GetFileSize(&size);
-        if (NS_SUCCEEDED(rv)) {
-            *contentLength = nsInt64(size);
-        if (! *contentLength)
-            *contentLength = -1;
-        }
-        else 
-            *contentLength = -1;
-    }
-    if (contentType) {
-        // must we really go though this? dougt
-        nsIMIMEService* mimeServ = nsnull;
-        nsFileTransportService* fileTransportService = nsFileTransportService::GetInstance();
-        if (fileTransportService) {
-            mimeServ = fileTransportService->GetCachedMimeService();
-            if (mimeServ)
-                rv = mimeServ->GetTypeFromFile(mFile, contentType);
-        }
-        
-        if (!mimeServ || (NS_FAILED(rv))) {
-            // if all else fails treat it as text/html?
-            *contentType = nsCRT::strdup(UNKNOWN_CONTENT_TYPE);
-            if (*contentType == nsnull)
-                rv = NS_ERROR_OUT_OF_MEMORY;
-            else
-                rv = NS_OK;
-        }
-    }
     PR_LOG(gFileIOLog, PR_LOG_DEBUG,
-           ("nsFileIO: logically opening %s", mSpec));
+           ("nsFileIO: logically opening %s", mSpec.get()));
+    return rv;
+}
+
+NS_IMETHODIMP 
+nsFileIO::GetContentType(nsACString &result)
+{
+    if (!mContentType.IsEmpty()) {
+        result = mContentType;
+        return NS_OK;
+    }
+
+    if (mFile == nsnull)
+        return NS_ERROR_NOT_INITIALIZED;
+
+    nsresult rv = NS_OK;
+    nsIMIMEService* mimeServ = nsnull;
+    nsFileTransportService* fileTransportService = nsFileTransportService::GetInstance();
+    if (fileTransportService) {
+        mimeServ = fileTransportService->GetCachedMimeService();
+        if (mimeServ) {
+            nsXPIDLCString mimeType; // XXX fix mime service to use |ACString|
+            rv = mimeServ->GetTypeFromFile(mFile, getter_Copies(mimeType));
+            if (NS_SUCCEEDED(rv))
+                result = mimeType;
+        }
+    }
+        
+    if (!mimeServ || (NS_FAILED(rv))) {
+        // if all else fails treat it as text/html?
+        result = NS_LITERAL_CSTRING(UNKNOWN_CONTENT_TYPE);
+    }
+
+    mContentType = result;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsFileIO::GetContentCharset(nsACString &result)
+{
+    result.Truncate();
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsFileIO::GetContentLength(PRInt32 *result)
+{
+    NS_ENSURE_ARG_POINTER(result);
+    *result = -1;
+
+    if (!mFile)
+        return NS_ERROR_NOT_INITIALIZED;
+
+    // We'll try to use the file's length, if it has one. If not,
+    // assume the file to be special, and set the content length
+    // to -1, which means "read the stream until exhausted".
+    PRInt64 size;
+    nsresult rv = mFile->GetFileSize(&size);
+    if (NS_SUCCEEDED(rv)) {
+        *result = nsInt64(size);
+        if (*result == 0)
+            *result = -1;
+    }
     return rv;
 }
 
@@ -249,7 +272,7 @@ nsFileIO::Close(nsresult status)
     if (mFile) {
         PR_LOG(gFileIOLog, PR_LOG_DEBUG,
                ("nsFileIO: logically closing %s: status=%x",
-                mSpec, status));
+                mSpec.get(), status));
         mFile = nsnull;
     }
     mStatus = status;
@@ -266,7 +289,7 @@ nsFileIO::GetInputStream(nsIInputStream * *aInputStream)
     nsresult rv;
 
     if (!mFD) {
-        rv = Open(nsnull, nsnull);        
+        rv = Open();        
         if (NS_FAILED(rv))  // file or directory does not exist
             return rv;
     }
@@ -277,12 +300,14 @@ nsFileIO::GetInputStream(nsIInputStream * *aInputStream)
         return rv;
     
     if (isDir) {
-        if (mFD)
+        if (mFD) {
             PR_Close(mFD);
+            mFD = nsnull;
+        }
         rv = nsDirectoryIndexStream::Create(mFile, aInputStream);
         PR_LOG(gFileIOLog, PR_LOG_DEBUG,
                ("nsFileIO: opening local dir %s for input (%x)",
-                mSpec, rv));
+                mSpec.get(), rv));
         return rv;
     }
 
@@ -290,7 +315,7 @@ nsFileIO::GetInputStream(nsIInputStream * *aInputStream)
     if (fileIn == nsnull)
         return NS_ERROR_OUT_OF_MEMORY;
     NS_ADDREF(fileIn);
-    rv = fileIn->InitWithFileDescriptor(mFD, mFile, PR_FALSE);
+    rv = fileIn->InitWithFileDescriptor(mFD, this);
     if (NS_SUCCEEDED(rv)) {
 #ifdef NS_NO_INPUT_BUFFERING
         *aInputStream = fileIn;
@@ -305,7 +330,7 @@ nsFileIO::GetInputStream(nsIInputStream * *aInputStream)
 
     PR_LOG(gFileIOLog, PR_LOG_DEBUG,
            ("nsFileIO: opening local file %s for input (%x)",
-            mSpec, rv));
+            mSpec.get(), rv));
     return rv;
 }
 
@@ -319,7 +344,7 @@ nsFileIO::GetOutputStream(nsIOutputStream * *aOutputStream)
     nsresult rv;
 
     if (!mFD) {
-        rv = Open(nsnull, nsnull);
+        rv = Open();
         if (NS_FAILED(rv))  // file or directory does not exist
             return rv;
     }
@@ -334,7 +359,7 @@ nsFileIO::GetOutputStream(nsIOutputStream * *aOutputStream)
     if (fileOut == nsnull)
         return NS_ERROR_OUT_OF_MEMORY;
     NS_ADDREF(fileOut);
-    rv = fileOut->InitWithFileDescriptor(mFD, mFile);
+    rv = fileOut->InitWithFileDescriptor(mFD, this);
     if (NS_SUCCEEDED(rv)) {
         nsCOMPtr<nsIOutputStream> bufStr;
 #ifdef NS_NO_OUTPUT_BUFFERING
@@ -350,18 +375,23 @@ nsFileIO::GetOutputStream(nsIOutputStream * *aOutputStream)
 
     PR_LOG(gFileIOLog, PR_LOG_DEBUG,
            ("nsFileIO: opening local file %s for output (%x)",
-            mSpec, rv));
+            mSpec.get(), rv));
     return rv;
 }
 
 NS_IMETHODIMP
-nsFileIO::GetName(char* *aName)
+nsFileIO::GetName(nsACString &aName)
 {
     NS_ASSERTION(mFile, "File must not be null");
     if (mFile == nsnull)
         return NS_ERROR_NOT_INITIALIZED;
 
-    return mFile->GetPath(aName);
+    nsAutoString path;
+    nsresult rv = mFile->GetPath(path);
+    if (NS_FAILED(rv)) return rv;
+
+    aName = NS_ConvertUCS2toUTF8(path);
+    return NS_OK;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -369,22 +399,40 @@ nsFileIO::GetName(char* *aName)
 
 nsFileStream::nsFileStream()
     : mFD(nsnull)
+    , mCloseFD(PR_TRUE)
 {
     NS_INIT_REFCNT();
 }
 
 nsFileStream::~nsFileStream()
 {
-    Close();
+    if (mCloseFD)
+        Close();
 }
 
 NS_IMPL_THREADSAFE_ISUPPORTS1(nsFileStream, nsISeekableStream)
 
+nsresult
+nsFileStream::InitWithFileDescriptor(PRFileDesc* fd, nsISupports* parent)
+{
+    NS_ENSURE_TRUE(mFD == nsnull, NS_ERROR_ALREADY_INITIALIZED);
+    //
+    // this file stream is dependent on its parent to keep the
+    // file descriptor valid.  an owning reference to the parent
+    // prevents the file descriptor from going away prematurely.
+    //
+    mFD = fd;
+    mCloseFD = PR_FALSE;
+    mParent = parent;
+    return NS_OK;
+}
+
 nsresult
 nsFileStream::Close()
 {
     if (mFD) {
-        PR_Close(mFD);
+        if (mCloseFD)
+            PR_Close(mFD);
         mFD = nsnull;
     }
     return NS_OK;
@@ -480,47 +528,66 @@ nsFileInputStream::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
     return rv;
 }
 
-NS_IMETHODIMP
-nsFileInputStream::Init(nsIFile* file, PRInt32 ioFlags, PRInt32 perm, PRBool deleteOnClose)
+nsresult
+nsFileInputStream::Open(nsIFile* aFile, PRInt32 aIOFlags, PRInt32 aPerm)
 {   
     nsresult rv = NS_OK;
-    nsCOMPtr<nsILocalFile> localFile = do_QueryInterface(file, &rv);
+
+    // If the previous file is open, close it
+    if (mFD) {
+        rv = Close();
+        if (NS_FAILED(rv)) return rv;
+    }
+
+    // Open the file
+    nsCOMPtr<nsILocalFile> localFile = do_QueryInterface(aFile, &rv);
     if (NS_FAILED(rv)) return rv;
-    if (ioFlags == -1)
-        ioFlags = PR_RDONLY;
-    if (perm == -1)
-        perm = 0;
+    if (aIOFlags == -1)
+        aIOFlags = PR_RDONLY;
+    if (aPerm == -1)
+        aPerm = 0;
 
     PRFileDesc* fd;
-    rv = localFile->OpenNSPRFileDesc(ioFlags, perm, &fd);
+    rv = localFile->OpenNSPRFileDesc(aIOFlags, aPerm, &fd);
     if (NS_FAILED(rv)) return rv;
-    
-    return InitWithFileDescriptor(fd, file, deleteOnClose);
-}
 
-nsresult
-nsFileInputStream::InitWithFileDescriptor(PRFileDesc* fd, nsIFile* file, PRBool deleteOnClose)
-{
-    NS_ASSERTION(mFD == nsnull, "already inited");
-    if (mFD || !fd)
-        return NS_ERROR_FAILURE;
-    
-    mLineBuffer = nsnull;    
     mFD = fd;
 
-    if (deleteOnClose) {
+    if (mBehaviorFlags & DELETE_ON_CLOSE) {
         // POSIX compatible filesystems allow a file to be unlinked while a
         // file descriptor is still referencing the file.  since we've already
         // opened the file descriptor, we'll try to remove the file.  if that
         // fails, then we'll just remember the nsIFile and remove it after we
         // close the file descriptor.
-        nsresult rv = file->Remove(PR_FALSE);
-        if (NS_FAILED(rv))
-            mFileToDelete = file;
+        rv = aFile->Remove(PR_FALSE);
+        if (NS_FAILED(rv) && !(mBehaviorFlags & REOPEN_ON_REWIND)) {
+            // If REOPEN_ON_REWIND is not happenin', we haven't saved the file yet
+            mFile = aFile;
+        }
     }
+
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsFileInputStream::Init(nsIFile* aFile, PRInt32 aIOFlags, PRInt32 aPerm,
+                        PRInt32 aBehaviorFlags)
+{
+    NS_ENSURE_TRUE(!mFD, NS_ERROR_ALREADY_INITIALIZED);
+    NS_ENSURE_TRUE(!mParent, NS_ERROR_ALREADY_INITIALIZED);
+
+    mBehaviorFlags = aBehaviorFlags;
+
+    // If the file will be reopened on rewind, save the info to open the file
+    if (mBehaviorFlags & REOPEN_ON_REWIND) {
+        mFile = aFile;
+        mIOFlags = aIOFlags;
+        mPerm = aPerm;
+    }
+
+    return Open(aFile, aIOFlags, aPerm);
+}
+
 NS_IMETHODIMP
 nsFileInputStream::Close()
 {
@@ -528,90 +595,98 @@ nsFileInputStream::Close()
     mLineBuffer = nsnull;       // in case Close() is called again after failing
     nsresult rv = nsFileStream::Close();
     if (NS_FAILED(rv)) return rv;
-    if (mFileToDelete) {
-        rv = mFileToDelete->Remove(PR_FALSE);
+    if (mFile && (mBehaviorFlags & DELETE_ON_CLOSE)) {
+        rv = mFile->Remove(PR_FALSE);
         NS_ASSERTION(NS_SUCCEEDED(rv), "failed to delete file");
+        // If we don't need to save the file for reopening, free it up
+        if (!(mBehaviorFlags & REOPEN_ON_REWIND)) {
+          mFile = nsnull;
+        }
     }
     return rv;
 }
 
 NS_IMETHODIMP
-nsFileInputStream::Available(PRUint32 *result)
+nsFileInputStream::Available(PRUint32* aResult)
 {
-    if (mFD == nsnull)
+    if (!mFD) {
         return NS_BASE_STREAM_CLOSED;
+    }
 
     PRInt32 avail = PR_Available(mFD);
     if (avail == -1) {
         return NS_ErrorAccordingToNSPR();
     }
-    *result = avail;
+    *aResult = avail;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileInputStream::Read(char * buf, PRUint32 count, PRUint32 *result)
+nsFileInputStream::Read(char* aBuf, PRUint32 aCount, PRUint32* aResult)
 {
-    if (mFD == nsnull)
+    if (!mFD) {
         return NS_BASE_STREAM_CLOSED;
+    }
 
-    PRInt32 cnt = PR_Read(mFD, buf, count);
-    if (cnt == -1) {
+    PRInt32 bytesRead = PR_Read(mFD, aBuf, aCount);
+    if (bytesRead == -1) {
         return NS_ErrorAccordingToNSPR();
     }
-    *result = cnt;
+    // Check if we're at the end of file and need to close
+    if (mBehaviorFlags & CLOSE_ON_EOF) {
+        if (bytesRead == 0) {
+            Close();
+        }
+    }
+
+    *aResult = bytesRead;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileInputStream::ReadLine(nsAWritableString & aLine, PRBool *_retval)
+nsFileInputStream::ReadLine(nsAString& aLine, PRBool* aResult)
 {
     if (!mLineBuffer) {
         nsresult rv = NS_InitLineBuffer(&mLineBuffer);
         if (NS_FAILED(rv)) return rv;
     }
-    return NS_ReadLine(this, mLineBuffer, aLine, _retval);
+    return NS_ReadLine(this, mLineBuffer, aLine, aResult);
 }
 
 NS_IMETHODIMP
-nsFileInputStream::ReadSegments(nsWriteSegmentFun writer, void * closure, PRUint32 count, PRUint32 *_retval)
+nsFileInputStream::ReadSegments(nsWriteSegmentFun aWriter, void* aClosure,
+                                PRUint32 aCount, PRUint32* aResult)
 {
-     PRUint32 nBytes;
-     char *readBuf = (char *)nsMemory::Alloc(count);
-     if (!readBuf)
-         return NS_ERROR_OUT_OF_MEMORY;
-     
-     nsresult rv = Read(readBuf, count, &nBytes);
-   
-     *_retval = 0;
-     if (NS_SUCCEEDED(rv)) {
-         rv = writer(this, closure, readBuf, 0, nBytes, _retval);
-         NS_ASSERTION(NS_SUCCEEDED(rv) ? nBytes == *_retval : PR_TRUE, "Didn't write all Data.");
-     }
-   
-     nsMemory::Free(readBuf);
-     return rv;
-}
-
-NS_IMETHODIMP
-nsFileInputStream::GetNonBlocking(PRBool *aNonBlocking)
-{
-    NS_NOTREACHED("GetNonBlocking");
+    // ReadSegments is not implemented because it would be inefficient when
+    // the writer does not consume all data.  If you want to call ReadSegments,
+    // wrap a BufferedInputStream around the file stream.  That will call
+    // Read().
+    NS_NOTREACHED("ReadSegments");
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsFileInputStream::GetObserver(nsIInputStreamObserver * *aObserver)
+nsFileInputStream::IsNonBlocking(PRBool *aNonBlocking)
 {
-    NS_NOTREACHED("GetObserver");
-    return NS_ERROR_NOT_IMPLEMENTED;
+    *aNonBlocking = PR_FALSE;
+    return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileInputStream::SetObserver(nsIInputStreamObserver * aObserver)
+nsFileInputStream::Seek(PRInt32 aWhence, PRInt32 aOffset)
 {
-    NS_NOTREACHED("SetObserver");
-    return NS_ERROR_NOT_IMPLEMENTED;
+    if (!mFD) {
+        if (mBehaviorFlags & REOPEN_ON_REWIND) {
+            nsresult rv = Reopen();
+            if (NS_FAILED(rv)) {
+                return rv;
+            }
+        } else {
+            return NS_BASE_STREAM_CLOSED;
+        }
+    }
+
+    return nsFileStream::Seek(aWhence, aOffset);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -637,8 +712,11 @@ nsFileOutputStream::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
 }
 
 NS_IMETHODIMP
-nsFileOutputStream::Init(nsIFile* file, PRInt32 ioFlags, PRInt32 perm)
+nsFileOutputStream::Init(nsIFile* file, PRInt32 ioFlags, PRInt32 perm,
+                         PRInt32 behaviorFlags)
 {
+    NS_ENSURE_TRUE(mFD == nsnull, NS_ERROR_ALREADY_INITIALIZED);
+
     nsresult rv;
     nsCOMPtr<nsILocalFile> localFile = do_QueryInterface(file, &rv);
     if (NS_FAILED(rv)) return rv;
@@ -646,20 +724,11 @@ nsFileOutputStream::Init(nsIFile* file, PRInt32 ioFlags, PRInt32 perm)
         ioFlags = PR_WRONLY | PR_CREATE_FILE | PR_TRUNCATE;
     if (perm <= 0)
         perm = 0664;
+
     PRFileDesc* fd;
     rv = localFile->OpenNSPRFileDesc(ioFlags, perm, &fd);
     if (NS_FAILED(rv)) return rv;
 
-    return InitWithFileDescriptor(fd, file);
-}
-
-
-nsresult
-nsFileOutputStream::InitWithFileDescriptor(PRFileDesc* fd, nsIFile* file)
-{
-    NS_ASSERTION(mFD == nsnull, "already inited");
-    if (mFD || !fd)
-        return NS_ERROR_FAILURE;
     mFD = fd;
     return NS_OK;
 }
@@ -712,30 +781,10 @@ nsFileOutputStream::WriteSegments(nsReadSegmentFun reader, void * closure, PRUin
 }
 
 NS_IMETHODIMP
-nsFileOutputStream::GetNonBlocking(PRBool *aNonBlocking)
+nsFileOutputStream::IsNonBlocking(PRBool *aNonBlocking)
 {
     *aNonBlocking = PR_FALSE;
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsFileOutputStream::SetNonBlocking(PRBool aNonBlocking)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsFileOutputStream::GetObserver(nsIOutputStreamObserver * *aObserver)
-{
-    NS_NOTREACHED("GetObserver");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsFileOutputStream::SetObserver(nsIOutputStreamObserver * aObserver)
-{
-    NS_NOTREACHED("SetObserver");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
