@@ -101,14 +101,8 @@ public:
         mLastError = rv;
         return rv;
     }
-    NS_IMETHOD GetNonBlocking(PRBool *aValue) {
-        return mSource->GetNonBlocking(aValue);
-    }
-    NS_IMETHOD GetObserver(nsIInputStreamObserver **aObserver) {
-        return mSource->GetObserver(aObserver);
-    }
-    NS_IMETHOD SetObserver(nsIInputStreamObserver *aObserver) {
-        return mSource->SetObserver(aObserver);
+    NS_IMETHOD IsNonBlocking(PRBool *aValue) {
+        return mSource->IsNonBlocking(aValue);
     }
 
     //
@@ -182,17 +176,8 @@ public:
         mLastError = rv;
         return rv;
     }
-    NS_IMETHOD GetNonBlocking(PRBool *aValue) {
-        return mSink->GetNonBlocking(aValue);
-    }
-    NS_IMETHOD SetNonBlocking(PRBool aValue) {
-        return mSink->SetNonBlocking(aValue);
-    }
-    NS_IMETHOD GetObserver(nsIOutputStreamObserver **aObserver) {
-        return mSink->GetObserver(aObserver);
-    }
-    NS_IMETHOD SetObserver(nsIOutputStreamObserver *aObserver) {
-        return mSink->SetObserver(aObserver);
+    NS_IMETHOD IsNonBlocking(PRBool *aValue) {
+        return mSink->IsNonBlocking(aValue);
     }
 
     //
@@ -223,8 +208,7 @@ NS_IMPL_THREADSAFE_ISUPPORTS1(nsFileTransportSinkWrapper, nsIOutputStream)
 //////////////////////////////////////////////////////////////////////////////////
 
 nsFileTransport::nsFileTransport()
-    : mContentType(nsnull),
-      mBufferSegmentSize(NS_FILE_TRANSPORT_DEFAULT_SEGMENT_SIZE),
+    : mBufferSegmentSize(NS_FILE_TRANSPORT_DEFAULT_SEGMENT_SIZE),
       mBufferMaxSize(NS_FILE_TRANSPORT_DEFAULT_BUFFER_SIZE),
       mXferState(CLOSED),
       mRunState(RUNNING),
@@ -250,31 +234,41 @@ nsFileTransport::nsFileTransport()
 }
 
 nsresult
-nsFileTransport::Init(nsFileTransportService *aService, nsIFile* file, PRInt32 ioFlags, PRInt32 perm)
+nsFileTransport::Init(nsFileTransportService *aService,
+                      nsIFile *file,
+                      PRInt32 ioFlags,
+                      PRInt32 perm,
+                      PRBool closeStreamWhenDone)
 {
     nsresult rv;
     nsCOMPtr<nsIFileIO> io;
     rv = NS_NewFileIO(getter_AddRefs(io), file, ioFlags, perm);
     if (NS_FAILED(rv)) return rv;
 
-    return Init(aService, io);
+    return Init(aService, io, closeStreamWhenDone);
 }
 
 nsresult
-nsFileTransport::Init(nsFileTransportService *aService, const char* name, nsIInputStream* inStr,
-                      const char* contentType, PRInt32 contentLength, PRBool closeStreamWhenDone)
+nsFileTransport::Init(nsFileTransportService *aService,
+                      const nsACString &name,
+                      nsIInputStream* inStr,
+                      const nsACString &contentType,
+                      const nsACString &contentCharset,
+                      PRInt32 contentLength,
+                      PRBool closeStreamWhenDone)
 {
     nsresult rv;
     nsCOMPtr<nsIInputStreamIO> io;
-    rv = NS_NewInputStreamIO(getter_AddRefs(io),
-                             name, inStr, contentType, contentLength);
+    rv = NS_NewInputStreamIO(getter_AddRefs(io), name, inStr,
+                             contentType, contentCharset, contentLength);
     if (NS_FAILED(rv)) return rv;
-    mCloseStreamWhenDone = closeStreamWhenDone;
-    return Init(aService, io);
+    return Init(aService, io, closeStreamWhenDone);
 }
 
 nsresult
-nsFileTransport::Init(nsFileTransportService *aService, nsIStreamIO* io)
+nsFileTransport::Init(nsFileTransportService *aService,
+                      nsIStreamIO* io,
+                      PRBool closeStreamWhenDone)
 {
     nsresult rv = NS_OK;
     if (mLock == nsnull) {
@@ -283,11 +277,11 @@ nsFileTransport::Init(nsFileTransportService *aService, nsIStreamIO* io)
             return NS_ERROR_OUT_OF_MEMORY;
     }
     mStreamIO = io;
-    nsXPIDLCString name;
-    rv = mStreamIO->GetName(getter_Copies(name));
-    mStreamName = NS_STATIC_CAST(const char*, name);
+    rv = mStreamIO->GetName(mStreamName);
     NS_ASSERTION(NS_SUCCEEDED(rv), "GetName failed");
 
+    mCloseStreamWhenDone = closeStreamWhenDone;
+
     NS_ADDREF(mService = aService);
     PR_AtomicIncrement(&mService->mTotalTransports);
 
@@ -307,11 +301,6 @@ nsFileTransport::~nsFileTransport()
         PR_DestroyLock(mLock);
         mLock = nsnull;
     }
-    if (mContentType) {
-        nsCRT::free(mContentType);
-        mContentType = nsnull;
-    }
-
     NS_IF_RELEASE(mService);
 }
 
@@ -338,10 +327,10 @@ nsFileTransport::Create(nsISupports* aOuter, const nsIID& aIID, void* *aResult)
 ////////////////////////////////////////////////////////////////////////////////
 
 NS_IMETHODIMP
-nsFileTransport::GetName(PRUnichar* *result)
+nsFileTransport::GetName(nsACString &result)
 {
-    *result = ToNewUnicode(mStreamName);
-    return *result ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+    result = mStreamName;
+    return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -667,7 +656,9 @@ nsFileTransport::Process(nsIProgressEventSink *progressSink)
 
     switch (mXferState) {
       case OPEN_FOR_READ: { 
-        mStatus = mStreamIO->Open(&mContentType, &mTotalAmount);
+        mStatus = mStreamIO->Open();
+        if (NS_SUCCEEDED(mStatus))
+            mStreamIO->GetContentLength(&mTotalAmount);
         LOG(("nsFileTransport: OPEN_FOR_READ [this=%x %s] status=%x\n", this, mStreamName.get(), mStatus));
         if (mListener) {
             nsresult rv = mListener->OnStartRequest(this, mContext);  // always send the start notification
@@ -841,23 +832,27 @@ nsFileTransport::Process(nsIProgressEventSink *progressSink)
         NS_IF_RELEASE(mSourceWrapper);
         mSourceWrapper = nsnull;
 
-        if (saveListener) {
-            saveListener->OnStopRequest(this, saveContext, mStatus);
-            saveListener = 0;
-        }
         if (progressSink) {
             progressSink->OnStatus(this, saveContext, 
                                     NS_NET_STATUS_READ_FROM, 
                                     NS_ConvertASCIItoUCS2(mStreamName).get());
         }
 
+        if (saveListener) {
+            saveListener->OnStopRequest(this, saveContext, mStatus);
+            saveListener = 0;
+        }
+        saveContext = 0;
+
         break;
       }
 
       case OPEN_FOR_WRITE: {
         LOG(("nsFileTransport: OPEN_FOR_WRITE [this=%x %s]\n",
             this, mStreamName.get()));
-        mStatus = mStreamIO->Open(&mContentType, &mTotalAmount);
+        mStatus = mStreamIO->Open();
+        if (NS_SUCCEEDED(mStatus)) // XXX why do we care about this when writing?
+            mStatus = mStreamIO->GetContentLength(&mTotalAmount);
         
         if (mStatus == NS_ERROR_FILE_NOT_FOUND)
             mStatus = NS_OK;
