@@ -415,12 +415,26 @@ nsIOService::GetProtocolHandler(const char* scheme, nsIProtocolHandler* *result)
     rv = GetCachedProtocolHandler(scheme, result);
     if (NS_SUCCEEDED(rv)) return NS_OK;
 
-    nsCAutoString contractID(NS_NETWORK_PROTOCOL_CONTRACTID_PREFIX);
-    contractID += scheme;
-    ToLowerCase(contractID);
+    PRBool externalProtocol = PR_FALSE;
+    nsCOMPtr<nsIPrefBranch> prefBranch;
+    GetPrefBranch(getter_AddRefs(prefBranch));
+    if (prefBranch) {
+        nsCAutoString externalProtocolPref("network.protocol-handler.external.");
+        externalProtocolPref += scheme;
+        rv = prefBranch->GetBoolPref(externalProtocolPref.get(), &externalProtocol);
+        if (NS_FAILED(rv))
+            externalProtocol = PR_FALSE;
+    }
 
-    rv = CallGetService(contractID.get(), result);
-    if (NS_FAILED(rv)) 
+    if (!externalProtocol) {
+        nsCAutoString contractID(NS_NETWORK_PROTOCOL_CONTRACTID_PREFIX);
+        contractID += scheme;
+        ToLowerCase(contractID);
+
+        rv = CallGetService(contractID.get(), result);
+    }
+    
+    if (externalProtocol || NS_FAILED(rv)) 
     {
       // okay we don't have a protocol handler to handle this url type, so use the default protocol handler.
       // this will cause urls to get dispatched out to the OS ('cause we can't do anything with them) when 
@@ -791,7 +805,14 @@ nsIOService::SetOffline(PRBool offline)
     nsresult rv1 = NS_OK;
     nsresult rv2 = NS_OK;
     if (offline) {
+        NS_NAMED_LITERAL_STRING(offlineString, "offline");
     	mOffline = PR_TRUE;		// indicate we're trying to shutdown
+        // don't care if notification fails
+        if (observerService)
+        // this allows users to attempt a little cleanup before dns and socket transport are shut down.
+            (void)observerService->NotifyObservers(NS_STATIC_CAST(nsIIOService *, this),
+                                                   "network:offline-about-to-go-offline",
+                                                   offlineString.get());
         // be sure to try and shutdown both (even if the first fails)
         if (mDNSService)
             rv1 = mDNSService->Shutdown();  // shutdown dns service first, because it has callbacks for socket transport
@@ -804,7 +825,7 @@ nsIOService::SetOffline(PRBool offline)
         if (observerService)
             (void)observerService->NotifyObservers(NS_STATIC_CAST(nsIIOService *, this),
                                                    "network:offline-status-changed",
-                                                   NS_LITERAL_STRING("offline").get());
+                                                   offlineString.get());
     }
     else if (!offline && mOffline) {
         // go online
