@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *   Darin Fisher <darin@netscape.com> (original author)
+ *   Andreas Otte <andreas.otte@debitel.net>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or 
@@ -92,27 +93,36 @@ EncodeString(nsIUnicodeEncoder *encoder, const nsAFlatString &str, nsACString &r
     PRInt32 maxlen;
 
     rv = encoder->GetMaxLength(str.get(), len, &maxlen);
-    if (NS_FAILED(rv)) return rv;
+    if (NS_FAILED(rv))
+        return rv;
 
     char buf[256], *p = buf;
     if (PRUint32(maxlen) > sizeof(buf) - 1) {
         p = (char *) malloc(maxlen + 1);
-        if (!p) return NS_ERROR_OUT_OF_MEMORY;
+        if (!p)
+            return NS_ERROR_OUT_OF_MEMORY;
     }
 
     rv = encoder->Convert(str.get(), &len, p, &maxlen);
-    if (NS_FAILED(rv)) goto end;
+    if (NS_FAILED(rv))
+        goto end;
+    if (rv == NS_ERROR_UENC_NOMAPPING) {
+        NS_WARNING("unicode conversion failed");
+        rv = NS_ERROR_UNEXPECTED;
+        goto end;
+    }
     p[maxlen] = 0;
     result = p;
 
     rv = encoder->Finish(p, &len);
-    if (NS_FAILED(rv)) goto end;
+    if (NS_FAILED(rv))
+        goto end;
     p[len] = 0;
     result += p;
 
-    rv = encoder->Reset();
-
 end:
+    encoder->Reset();
+
     if (p != buf)
         free(p);
     return rv;
@@ -125,6 +135,14 @@ FilterString(const char *str, nsCString &result)
     PRBool writing = PR_FALSE;
     result.Truncate();
     const char *p = str;
+
+    // Remove leading spaces, tabs, CR, LF if any.
+    while (*p == ' ' || *p == '\t' || *p == '\r' || *p == '\n') {
+        writing = PR_TRUE;
+        str = p + 1;
+        p++;
+    }
+
     for (; *p; ++p) {
         if (*p == '\t' || *p == '\r' || *p == '\n') {
             writing = PR_TRUE;
@@ -134,8 +152,16 @@ FilterString(const char *str, nsCString &result)
             str = p + 1;
         }
     }
+
+    // Remove trailing spaces if any
+    while (((p-1) >= str) && (*(p-1) == ' ')) {
+        writing = PR_TRUE;
+        p--;
+    }
+
     if (writing && p > str)
         result.Append(str, p - str);
+
     return writing ? result.get() : str;
 }
 
@@ -236,6 +262,7 @@ nsSegmentEncoder::EncodeSegmentCount(const char *str,
                 pos = 0;
                 len = encBuf.Length();
             }
+            // else some failure occured... assume UTF-8 is ok.
         }
 
         // escape per RFC2396 unless UTF-8 and allowed by preferences
@@ -244,7 +271,7 @@ nsSegmentEncoder::EncodeSegmentCount(const char *str,
         PRUint32 initLen = result.Length();
 
         // now perform any required escaping
-        if (NS_EscapeURL(str + pos, len, mask|escapeFlags, result))
+        if (NS_EscapeURL(str + pos, len, mask | escapeFlags, result))
             len = result.Length() - initLen;
         else if (str == encBuf.get()) {
             result += encBuf; // append only!!
@@ -505,7 +532,7 @@ nsStandardURL::BuildNormalizedSpec(const char *spec)
     //
     // generate the normalized URL string
     //
-    char *buf = (char *) malloc(approxLen + 32);
+    char *buf = (char *) nsMemory::Alloc(approxLen + 32);
     if (!buf)
         return NS_ERROR_OUT_OF_MEMORY;
     PRUint32 i = 0;
@@ -916,7 +943,7 @@ nsStandardURL::GetAsciiSpec(nsACString &result)
 
     result = Substring(mSpec, 0, mScheme.mLen + 3);
 
-    NS_EscapeURL(Userpass(PR_TRUE), esc_OnlyNonASCII|esc_AlwaysCopy, result);
+    NS_EscapeURL(Userpass(PR_TRUE), esc_OnlyNonASCII | esc_AlwaysCopy, result);
 
     // get escaped host
     nsCAutoString escHostport;
@@ -931,7 +958,7 @@ nsStandardURL::GetAsciiSpec(nsACString &result)
     }
     result += escHostport;
 
-    NS_EscapeURL(Path(), esc_OnlyNonASCII|esc_AlwaysCopy, result);
+    NS_EscapeURL(Path(), esc_OnlyNonASCII | esc_AlwaysCopy, result);
     return NS_OK;
 }
 
@@ -959,7 +986,7 @@ nsStandardURL::GetAsciiHost(nsACString &result)
 
     if (gIDNService) {
         nsresult rv;
-        rv = gIDNService->UTF8ToIDNHostName(PromiseFlatCString(Host()).get(), &mHostA);
+        rv = gIDNService->ConvertUTF8toACE(PromiseFlatCString(Host()).get(), &mHostA);
         if (NS_SUCCEEDED(rv)) {
             result = mHostA;
             return NS_OK;
@@ -994,6 +1021,9 @@ nsStandardURL::SetSpec(const nsACString &input)
 
     Clear();
 
+    if (!spec)
+        return NS_OK;
+
     // filter out unexpected chars "\r\n\t" if necessary
     nsCAutoString buf1;
     spec = FilterString(spec, buf1);
@@ -1085,7 +1115,7 @@ nsStandardURL::SetUserPass(const nsACString &input)
         NS_ERROR("uninitialized");
         return NS_ERROR_NOT_INITIALIZED;
     }
-    NS_ASSERTION(mHost.mLen >= 0, "unitialized");
+    NS_ASSERTION(mHost.mLen >= 0, "uninitialized");
 
     InvalidateCache();
 
@@ -1118,13 +1148,17 @@ nsStandardURL::SetUserPass(const nsACString &input)
     if (usernameLen > 0) {
         GET_SEGMENT_ENCODER(encoder);
         usernameLen = encoder.EncodeSegmentCount(userpass.get(),
-                                                 URLSegment(usernamePos, usernameLen),
-                                                 esc_Username|esc_AlwaysCopy, buf);
+                                                 URLSegment(usernamePos,
+                                                            usernameLen),
+                                                 esc_Username | esc_AlwaysCopy,
+                                                 buf);
         if (passwordLen >= 0) {
             buf.Append(':');
             passwordLen = encoder.EncodeSegmentCount(userpass.get(),
-                                                     URLSegment(passwordPos, passwordLen),
-                                                     esc_Password|esc_AlwaysCopy, buf);
+                                                     URLSegment(passwordPos,
+                                                                passwordLen),
+                                                     esc_Password |
+                                                     esc_AlwaysCopy, buf);
         }
         if (mUsername.mLen < 0)
             buf.Append('@');
@@ -1514,7 +1548,11 @@ nsStandardURL::Resolve(const nsACString &in, nsACString &out)
                                  &scheme.mPos, &scheme.mLen,
                                  nsnull, nsnull,
                                  nsnull, nsnull);
-    if (NS_FAILED(rv)) return rv;
+
+    // if the parser fails (for example because there is no valid scheme)
+    // reset the scheme and assume a relative url
+    if (NS_FAILED(rv)) 
+        scheme.Reset();
 
     if (scheme.mLen >= 0) {
         // this URL appears to be absolute
@@ -1577,6 +1615,151 @@ nsStandardURL::Resolve(const nsACString &in, nsACString &out)
     return NS_OK;
 }
 
+// result may contain unescaped UTF-8 characters
+NS_IMETHODIMP
+nsStandardURL::GetCommonBaseSpec(nsIURI *uri2, nsACString &aResult)
+{
+    NS_ENSURE_ARG_POINTER(uri2);
+
+    // if uri's are equal, then return uri as is
+    PRBool isEquals = PR_FALSE;
+    if (NS_SUCCEEDED(Equals(uri2, &isEquals)) && isEquals)
+        return GetSpec(aResult);
+
+    aResult.Truncate();
+
+    // check pre-path; if they don't match, then return empty string
+    nsStandardURL *stdurl2;
+    nsresult rv = uri2->QueryInterface(kThisImplCID, (void **) &stdurl2);
+    isEquals = NS_SUCCEEDED(rv)
+            && SegmentIs(mScheme, stdurl2->mSpec.get(), stdurl2->mScheme)    
+            && HostsAreEquivalent(stdurl2)
+            && SegmentIs(mUsername, stdurl2->mSpec.get(), stdurl2->mUsername)
+            && SegmentIs(mPassword, stdurl2->mSpec.get(), stdurl2->mPassword)
+            && (Port() == stdurl2->Port());
+    if (!isEquals)
+    {
+        if (NS_SUCCEEDED(rv))
+            NS_RELEASE(stdurl2);
+        return NS_OK;
+    }
+
+    // scan for first mismatched character
+    const char *thisIndex, *thatIndex, *startCharPos;
+    startCharPos = mSpec.get() + mDirectory.mPos;
+    thisIndex = startCharPos;
+    thatIndex = stdurl2->mSpec.get() + mDirectory.mPos;
+    while ((*thisIndex == *thatIndex) && *thisIndex)
+    {
+        thisIndex++;
+        thatIndex++;
+    }
+
+    // backup to just after previous slash so we grab an appropriate path
+    // segment such as a directory (not partial segments)
+    // todo:  also check for file matches which include '?', '#', and ';'
+    while ((*(thisIndex-1) != '/') && (thisIndex != startCharPos))
+        thisIndex--;
+
+    // grab spec from beginning to thisIndex
+    aResult = Substring(mSpec, mScheme.mPos, thisIndex - mSpec.get());
+
+    NS_RELEASE(stdurl2);
+    return rv;
+}
+
+NS_IMETHODIMP
+nsStandardURL::GetRelativeSpec(nsIURI *uri2, nsACString &aResult)
+{
+    NS_ENSURE_ARG_POINTER(uri2);
+
+    aResult.Truncate();
+
+    // if uri's are equal, then return empty string
+    PRBool isEquals = PR_FALSE;
+    if (NS_SUCCEEDED(Equals(uri2, &isEquals)) && isEquals)
+        return NS_OK;
+
+    nsStandardURL *stdurl2;
+    nsresult rv = uri2->QueryInterface(kThisImplCID, (void **) &stdurl2);
+    isEquals = NS_SUCCEEDED(rv)
+            && SegmentIs(mScheme, stdurl2->mSpec.get(), stdurl2->mScheme)    
+            && SegmentIs(mHost, stdurl2->mSpec.get(), stdurl2->mHost)
+            && SegmentIs(mUsername, stdurl2->mSpec.get(), stdurl2->mUsername)
+            && SegmentIs(mPassword, stdurl2->mSpec.get(), stdurl2->mPassword)
+            && (Port() == stdurl2->Port());
+    if (!isEquals)
+    {
+        if (NS_SUCCEEDED(rv))
+            NS_RELEASE(stdurl2);
+
+        return uri2->GetSpec(aResult);
+    }
+
+    // scan for first mismatched character
+    const char *thisIndex, *thatIndex, *startCharPos;
+    startCharPos = mSpec.get() + mDirectory.mPos;
+    thisIndex = startCharPos;
+    thatIndex = stdurl2->mSpec.get() + mDirectory.mPos;
+
+#ifdef XP_WIN
+    PRBool isFileScheme = SegmentIs(mScheme, "file");
+    if (isFileScheme)
+    {
+        // on windows, we need to match the first segment of the path
+        // if these don't match then we need to return an absolute path
+        // skip over any leading '/' in path
+        while ((*thisIndex == *thatIndex) && (*thisIndex == '/'))
+        {
+            thisIndex++;
+            thatIndex++;
+        }
+        // look for end of first segment
+        while ((*thisIndex == *thatIndex) && *thisIndex && (*thisIndex != '/'))
+        {
+            thisIndex++;
+            thatIndex++;
+        }
+
+        // if we didn't match through the first segment, return absolute path
+        if ((*thisIndex != '/') || (*thatIndex != '/'))
+        {
+            NS_RELEASE(stdurl2);
+            return uri2->GetSpec(aResult);
+        }
+    }
+#endif
+
+    while ((*thisIndex == *thatIndex) && *thisIndex)
+    {
+        thisIndex++;
+        thatIndex++;
+    }
+
+    // backup to just after previous slash so we grab an appropriate path
+    // segment such as a directory (not partial segments)
+    // todo:  also check for file matches with '#', '?' and ';'
+    while ((*(thatIndex-1) != '/') && (thatIndex != startCharPos))
+        thatIndex--;
+
+    // need to account for slashes and add corresponding "../"
+    while (*thisIndex)
+    {
+        if (*thisIndex == '/')
+            aResult.Append("../");
+
+        thisIndex++;
+    }
+
+    // grab spec from thisIndex to end
+    PRUint32 startPos = stdurl2->mScheme.mPos + thatIndex - stdurl2->mSpec.get();
+    aResult.Append(Substring(stdurl2->mSpec, startPos, 
+                             stdurl2->mSpec.Length() - startPos));
+
+    NS_RELEASE(stdurl2);
+    return rv;
+}
+
 //----------------------------------------------------------------------------
 // nsStandardURL::nsIURL
 //----------------------------------------------------------------------------
@@ -1683,16 +1866,20 @@ nsStandardURL::SetFilePath(const nsACString &input)
 
         // append encoded filepath components
         if (dirLen > 0)
-            encoder.EncodeSegment(Substring(filepath + dirPos, filepath + dirLen),
-                                  esc_Directory|esc_AlwaysCopy, spec);
+            encoder.EncodeSegment(Substring(filepath + dirPos,
+                                            filepath + dirPos + dirLen),
+                                  esc_Directory | esc_AlwaysCopy, spec);
         if (baseLen > 0)
-            encoder.EncodeSegment(Substring(filepath + basePos, filepath + baseLen),
-                                  esc_FileBaseName|esc_AlwaysCopy, spec);
+            encoder.EncodeSegment(Substring(filepath + basePos,
+                                            filepath + basePos + baseLen),
+                                  esc_FileBaseName | esc_AlwaysCopy, spec);
         if (extLen >= 0) {
             spec.Append('.');
             if (extLen > 0)
-                encoder.EncodeSegment(Substring(filepath + extPos, filepath + extLen),
-                                      esc_FileExtension|esc_AlwaysCopy, spec);
+                encoder.EncodeSegment(Substring(filepath + extPos,
+                                                filepath + extPos + extLen),
+                                      esc_FileExtension | esc_AlwaysCopy,
+                                      spec);
         }
 
         // compute the ending position of the current filepath
@@ -1905,7 +2092,8 @@ nsStandardURL::SetFileName(const nsACString &input)
 	        nsCAutoString newFilename;
             GET_SEGMENT_ENCODER(encoder);
             basename.mLen = encoder.EncodeSegmentCount(filename, basename,
-                                                       esc_FileBaseName|esc_AlwaysCopy,
+                                                       esc_FileBaseName |
+                                                       esc_AlwaysCopy,
                                                        newFilename);
 	        if (extension.mLen >= 0) {
 	            newFilename.Append('.');
@@ -1989,8 +2177,8 @@ nsStandardURL::GetFile(nsIFile **result)
 
 #if defined(PR_LOGGING)
     if (LOG_ENABLED()) {
-        nsXPIDLCString path;
-        localFile->GetPath(getter_Copies(path));
+        nsCAutoString path;
+        localFile->GetNativePath(path);
         LOG(("nsStandardURL::GetFile [this=%p spec=%s resulting_path=%s]\n",
             this, mSpec.get(), path.get()));
     }
