@@ -27,6 +27,7 @@
 #include "nsCRT.h"
 #include "prmem.h"
 #include "netCore.h"
+#include "nsAString.h"
 
 #define MAX_IO_CHUNK 8192 // maximum count reported per OnDataAvailable
 #define MAX_COUNT ((PRUint32) -1)
@@ -459,8 +460,8 @@ nsStorageTransport::nsReadRequest::Process()
     mWaitingForWrite = PR_FALSE;
 
     if (!mOnStartFired) {
-        // no need to proxy this callback
-        (void) mListener->OnStartRequest(this, mListenerContext);
+        // must proxy since called from AsyncRead - bug 136216
+        (void) mListenerProxy->OnStartRequest(this, mListenerContext);
         mOnStartFired = PR_TRUE;
     }
 
@@ -497,12 +498,7 @@ nsStorageTransport::nsReadRequest::Process()
         mTransport->ReadRequestCompleted(this);
         
         // no need to proxy this callback
-        (void) mListener->OnStopRequest(this, mListenerContext, mStatus);
-
-        //OnStopRequest completed and listeners no longer needed. 
-        mListener=nsnull;
-        mListenerContext=nsnull;
-        mListenerProxy=nsnull;
+        (void) mListenerProxy->OnStopRequest(this, mListenerContext, mStatus);
     }
     else
         mWaitingForWrite = PR_TRUE;
@@ -526,10 +522,9 @@ nsStorageTransport::nsReadRequest::GetTransport(nsITransport **aTransport)
 }
 
 NS_IMETHODIMP
-nsStorageTransport::nsReadRequest::GetName(PRUnichar **aName)
+nsStorageTransport::nsReadRequest::GetName(nsACString &aName)
 {
-    NS_ENSURE_ARG_POINTER(aName);
-    *aName = nsnull;
+    aName.Truncate();
     return NS_OK;
 }
 
@@ -599,8 +594,8 @@ NS_IMETHODIMP
 nsStorageTransport::nsReadRequest::OnStartRequest(nsIRequest *aRequest,
                                                   nsISupports *aContext)
 {
-    NS_NOTREACHED("nsStorageTransport::nsReadRequest::OnStartRequest");
-    return NS_ERROR_FAILURE;
+    NS_ASSERTION(mListener, "no listener");
+    return mListener->OnStartRequest(aRequest, aContext);
 }
 
 NS_IMETHODIMP
@@ -608,8 +603,13 @@ nsStorageTransport::nsReadRequest::OnStopRequest(nsIRequest *aRequest,
                                                  nsISupports *aContext,
                                                  nsresult aStatus)
 {
-    NS_NOTREACHED("nsStorageTransport::nsReadRequest::OnStopRequest");
-    return NS_ERROR_FAILURE;
+    NS_ASSERTION(mListener, "no listener");
+    (void) mListener->OnStopRequest(aRequest, aContext, aStatus);
+    // OnStopRequest completed, so listeners are no longer needed. 
+    mListener = 0;
+    mListenerContext = 0;
+    mListenerProxy = 0;
+    return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -711,27 +711,13 @@ nsStorageTransport::nsReadRequest::ReadSegments(nsWriteSegmentFun aWriter,
 }
 
 NS_IMETHODIMP
-nsStorageTransport::nsReadRequest::GetNonBlocking(PRBool *aNonBlocking)
+nsStorageTransport::nsReadRequest::IsNonBlocking(PRBool *aNonBlocking)
 {
     NS_ENSURE_ARG_POINTER(aNonBlocking);
     *aNonBlocking = PR_FALSE;
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsStorageTransport::nsReadRequest::GetObserver(nsIInputStreamObserver **aObserver)
-{
-    NS_ENSURE_ARG_POINTER(aObserver);
-    *aObserver = nsnull;
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsStorageTransport::nsReadRequest::SetObserver(nsIInputStreamObserver *aObserver)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 //----------------------------------------------------------------------------
 // nsStorageTransport::nsBlockingStream
 //----------------------------------------------------------------------------
@@ -791,27 +777,13 @@ nsStorageTransport::nsInputStream::ReadSegments(nsWriteSegmentFun aWriter,
 }
 
 NS_IMETHODIMP
-nsStorageTransport::nsInputStream::GetNonBlocking(PRBool *aNonBlocking)
+nsStorageTransport::nsInputStream::IsNonBlocking(PRBool *aNonBlocking)
 {
     NS_ENSURE_ARG_POINTER(aNonBlocking);
     *aNonBlocking = PR_TRUE;
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsStorageTransport::nsInputStream::GetObserver(nsIInputStreamObserver **aObserver)
-{
-    NS_ENSURE_ARG_POINTER(aObserver);
-    *aObserver = nsnull;
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsStorageTransport::nsInputStream::SetObserver(nsIInputStreamObserver *aObserver)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 //----------------------------------------------------------------------------
 // nsStorageTransport::nsOutputStream
 //----------------------------------------------------------------------------
@@ -901,29 +873,9 @@ nsStorageTransport::nsOutputStream::WriteSegments(nsReadSegmentFun aReader,
 }
 
 NS_IMETHODIMP
-nsStorageTransport::nsOutputStream::GetNonBlocking(PRBool *aNonBlocking)
+nsStorageTransport::nsOutputStream::IsNonBlocking(PRBool *aNonBlocking)
 {
     NS_ENSURE_ARG_POINTER(aNonBlocking);
     *aNonBlocking = PR_FALSE;
     return NS_OK;
 }
-
-NS_IMETHODIMP
-nsStorageTransport::nsOutputStream::SetNonBlocking(PRBool aNonBlocking)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsStorageTransport::nsOutputStream::GetObserver(nsIOutputStreamObserver **aObserver)
-{
-    NS_ENSURE_ARG_POINTER(aObserver);
-    *aObserver = nsnull;
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsStorageTransport::nsOutputStream::SetObserver(nsIOutputStreamObserver *aObserver)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
