@@ -39,6 +39,7 @@
 #include "nsReadableUtils.h"
 #include "nsIInputStream.h"
 #include "nsIOutputStream.h"
+#include "nsCRT.h"
 
 
 static const char DISK_CACHE_DEVICE_ID[] = { "disk" };
@@ -178,11 +179,11 @@ NS_IMETHODIMP nsDiskCacheDeviceInfo::GetUsageReport(char ** usageReport)
 
     buffer.Append("<tr><td><b>Cache Directory:</b></td><td><tt> ");
     nsCOMPtr<nsILocalFile> cacheDir;
-    char *                 path;
+    nsAutoString           path;
     mDevice->getCacheDirectory(getter_AddRefs(cacheDir)); 
-    nsresult rv = cacheDir->GetPath(&path);
+    nsresult rv = cacheDir->GetPath(path);
     if (NS_SUCCEEDED(rv)) {
-        buffer.Append(path);
+        buffer.Append(NS_ConvertUCS2toUTF8(path));
     } else {
         buffer.Append("directory unavailable");
     }
@@ -580,7 +581,7 @@ nsDiskCacheDevice::GetTransportForEntry(nsCacheEntry * entry,
         break;
     }
 
-    rv = gFileTransportService->CreateTransport(file, ioFlags, PR_IRUSR | PR_IWUSR, result);
+    rv = gFileTransportService->CreateTransport(file, ioFlags, PR_IRUSR | PR_IWUSR, PR_FALSE, result);
     return rv;
 }
 
@@ -631,8 +632,15 @@ nsDiskCacheDevice::OnDataSizeChange(nsCacheEntry * entry, PRInt32 deltaSize)
 
     NS_ASSERTION(binding->mRecord.ValidRecord(), "bad record");
 
+    PRUint32  newSize = entry->DataSize() + deltaSize;
+    if (newSize > mCacheCapacity) {
+        nsresult rv = nsCacheService::GlobalInstance()->DoomEntry_Locked(entry);
+        NS_ASSERTION(NS_SUCCEEDED(rv),"DoomEntry_Locked() failed.");
+        return NS_ERROR_ABORT;
+    }
+
     PRUint32  sizeK = ((entry->DataSize() + 0x0399) >> 10); // round up to next 1k
-    PRUint32  newSizeK =  ((entry->DataSize() + deltaSize + 0x399) >> 10);
+    PRUint32  newSizeK =  ((newSize + 0x399) >> 10);
 
     NS_ASSERTION((sizeK < USHRT_MAX) && (sizeK == binding->mRecord.DataFileSize()),
                  "data size out of sync");
@@ -758,7 +766,7 @@ nsDiskCacheDevice::InitializeCacheDirectory()
         }
         
         // create a directory with unique name to contain existing cache directory
-        rv = cacheTrashDir->Append("Cache");
+        rv = cacheTrashDir->AppendNative(NS_LITERAL_CSTRING("Cache"));
         if (NS_FAILED(rv))  return rv;
         rv = cacheTrashDir->CreateUnique(nsIFile::DIRECTORY_TYPE, 0777); 
         if (NS_FAILED(rv))  return rv;
@@ -767,7 +775,7 @@ nsDiskCacheDevice::InitializeCacheDirectory()
         nsCOMPtr<nsIFile> existingCacheDir;
         rv = mCacheDirectory->Clone(getter_AddRefs(existingCacheDir));
         if (NS_FAILED(rv))  return rv;
-        rv = existingCacheDir->MoveTo(cacheTrashDir, nsnull);
+        rv = existingCacheDir->MoveToNative(cacheTrashDir, nsCString());
         if (NS_FAILED(rv))  return rv;
     }
     
@@ -786,7 +794,7 @@ nsDiskCacheDevice::GetCacheTrashDirectory(nsIFile ** result)
     nsCOMPtr<nsIFile> cacheTrashDir;
     nsresult rv = mCacheDirectory->Clone(getter_AddRefs(cacheTrashDir));
     if (NS_FAILED(rv))  return rv;
-    rv = cacheTrashDir->SetLeafName("Cache.Trash");
+    rv = cacheTrashDir->SetNativeLeafName(NS_LITERAL_CSTRING("Cache.Trash"));
     if (NS_FAILED(rv))  return rv;
     
     *result = cacheTrashDir.get();
@@ -844,7 +852,7 @@ nsDiskCacheDevice::SetCacheParentDirectory(nsILocalFile * parentDir)
     
     rv = parentDir->Clone(getter_AddRefs(directory));
     if (NS_FAILED(rv))  return;
-    rv = directory->Append("Cache");
+    rv = directory->AppendNative(NS_LITERAL_CSTRING("Cache"));
     if (NS_FAILED(rv))  return;
     
     rv = directory->Exists(&exists);
@@ -857,7 +865,7 @@ nsDiskCacheDevice::SetCacheParentDirectory(nsILocalFile * parentDir)
     // clean up Cache.Trash directories
     rv = parentDir->Clone(getter_AddRefs(directory));
     if (NS_FAILED(rv))  return;    
-    rv = directory->Append("Cache.Trash");
+    rv = directory->AppendNative(NS_LITERAL_CSTRING("Cache.Trash"));
     if (NS_FAILED(rv))  return;
     
     rv = directory->Exists(&exists);
@@ -867,7 +875,7 @@ nsDiskCacheDevice::SetCacheParentDirectory(nsILocalFile * parentDir)
     // clean up obsolete NewCache directory
     rv = parentDir->Clone(getter_AddRefs(directory));
     if (NS_FAILED(rv))  return;    
-    rv = directory->Append("NewCache");
+    rv = directory->AppendNative(NS_LITERAL_CSTRING("NewCache"));
     if (NS_FAILED(rv))  return;
     
     rv = directory->Exists(&exists);
