@@ -34,6 +34,7 @@
 #include "nsHttpResponseHead.h"
 #include "nsHttpTransaction.h"
 #include "nsHttpAuthCache.h"
+#include "nsHttpPipeline.h"
 #include "nsIHttpChannel.h"
 #include "nsIHttpNotify.h"
 #include "nsIURL.h"
@@ -110,6 +111,7 @@ nsHttpHandler::nsHttpHandler()
     , mMaxConnectionsPerServer(8)
     , mMaxPersistentConnectionsPerServer(2)
     , mMaxPersistentConnectionsPerProxy(4)
+    , mMaxPipelinedRequests(2)
     , mRedirectionLimit(10)
     , mLastUniqueID(NowInSeconds())
     , mSessionStartTime(0)
@@ -119,6 +121,7 @@ nsHttpHandler::nsHttpHandler()
     , mConnectionLock(nsnull)
     , mUserAgentIsDirty(PR_TRUE)
     , mUseCache(PR_TRUE)
+    , mSendSecureXSiteReferrer(PR_FALSE)
 {
     NS_INIT_ISUPPORTS();
 
@@ -236,7 +239,7 @@ nsHttpHandler::Init()
     LOG(("> product = %s\n", mProduct.get()));
     LOG(("> product-sub = %s\n", mProductSub.get()));
     LOG(("> product-comment = %s\n", mProductComment.get()));
-    LOG(("> user-agent = %s\n", UserAgent()));
+    LOG(("> user-agent = %s\n", UserAgent().get()));
 #endif
 
     mSessionStartTime = NowInSeconds();
@@ -287,22 +290,22 @@ nsHttpHandler::AddStandardRequestHeaders(nsHttpHeaderArray *request,
 
     // MIME based content negotiation lives!
     // Add the "Accept" header
-    rv = request->SetHeader(nsHttp::Accept, mAccept.get());
+    rv = request->SetHeader(nsHttp::Accept, mAccept);
     if (NS_FAILED(rv)) return rv;
 
     // Add the "Accept-Language" header
     if (!mAcceptLanguages.IsEmpty()) {
         // Add the "Accept-Language" header
-        rv = request->SetHeader(nsHttp::Accept_Language, mAcceptLanguages.get());
+        rv = request->SetHeader(nsHttp::Accept_Language, mAcceptLanguages);
         if (NS_FAILED(rv)) return rv;
     }
 
     // Add the "Accept-Encoding" header
-    rv = request->SetHeader(nsHttp::Accept_Encoding, mAcceptEncodings.get());
+    rv = request->SetHeader(nsHttp::Accept_Encoding, mAcceptEncodings);
     if (NS_FAILED(rv)) return rv;
 
     // Add the "Accept-Charset" header
-    rv = request->SetHeader(nsHttp::Accept_Charset, mAcceptCharsets.get());
+    rv = request->SetHeader(nsHttp::Accept_Charset, mAcceptCharsets);
     if (NS_FAILED(rv)) return rv;
 
     // RFC2616 section 19.6.2 states that the "Connection: keep-alive"
@@ -316,22 +319,17 @@ nsHttpHandler::AddStandardRequestHeaders(nsHttpHeaderArray *request,
     
     const char* connectionType = "close";
     if (caps & NS_HTTP_ALLOW_KEEPALIVE) {
-        char buf[32];
-        
-        PR_snprintf(buf, sizeof(buf), "%u", (PRUintn) mIdleTimeout);
-        
-        rv = request->SetHeader(nsHttp::Keep_Alive, buf);
+        rv = request->SetHeader(nsHttp::Keep_Alive, nsPrintfCString("%u", mIdleTimeout));
         if (NS_FAILED(rv)) return rv;
-        
         connectionType = "keep-alive";
     } else if (useProxy) {
         // Bug 92006
-        request->SetHeader(nsHttp::Connection, "close");
+        request->SetHeader(nsHttp::Connection, NS_LITERAL_CSTRING("close"));
     }
 
     const nsHttpAtom &header =
         useProxy ? nsHttp::Proxy_Connection : nsHttp::Connection;
-    return request->SetHeader(header, connectionType);
+    return request->SetHeader(header, nsDependentCString(connectionType));
 }
 
 PRBool
@@ -412,7 +410,7 @@ nsHttpHandler::InitiateTransaction(nsHttpTransaction *trans,
         PR_Unlock(mConnectionLock);
     }
     else {
-        rv = DispatchTransaction_Locked(trans, conn);
+        rv = DispatchTransaction_Locked(trans, trans->Capabilities(), conn);
         NS_RELEASE(conn);
     }
     return rv;
@@ -423,9 +421,7 @@ nsresult
 nsHttpHandler::ReclaimConnection(nsHttpConnection *conn)
 {
     NS_ENSURE_ARG_POINTER(conn);
-#ifdef DEBUG
-    NS_PRECONDITION(PR_GetCurrentThread() == NS_SOCKET_THREAD, "wrong thread");
-#endif
+    NS_ASSERTION(PR_GetCurrentThread() == NS_SOCKET_THREAD, "wrong thread");
 
     PRBool reusable = conn->CanReuse();
 
@@ -475,9 +471,7 @@ nsresult
 nsHttpHandler::ProcessTransactionQ()
 {
     LOG(("nsHttpHandler::ProcessTransactionQ\n"));
-#ifdef DEBUG
-    NS_PRECONDITION(PR_GetCurrentThread() == NS_SOCKET_THREAD, "wrong thread");
-#endif
+    NS_ASSERTION(PR_GetCurrentThread() == NS_SOCKET_THREAD, "wrong thread");
 
     PR_Lock(mConnectionLock);
 
@@ -665,12 +659,12 @@ nsHttpHandler::OnExamineResponse(nsIHttpChannel *chan)
 // nsHttpHandler <private>
 //-----------------------------------------------------------------------------
 
-const char *
+const nsAFlatCString &
 nsHttpHandler::UserAgent()
 {
     if (mUserAgentOverride) {
         LOG(("using general.useragent.override : %s\n", mUserAgentOverride.get()));
-        return mUserAgentOverride.get();
+        return mUserAgentOverride;
     }
 
     if (mUserAgentIsDirty) {
@@ -678,7 +672,7 @@ nsHttpHandler::UserAgent()
         mUserAgentIsDirty = PR_FALSE;
     }
 
-    return mUserAgent.get();
+    return mUserAgent;
 }
 
 nsresult
@@ -783,7 +777,8 @@ nsHttpHandler::EnqueueTransaction_Locked(nsHttpTransaction *trans,
 
 // dispatch this transaction, return unlocked
 nsresult
-nsHttpHandler::DispatchTransaction_Locked(nsHttpTransaction *trans,
+nsHttpHandler::DispatchTransaction_Locked(nsAHttpTransaction *trans,
+                                          PRUint8 transCaps,
                                           nsHttpConnection *conn)
 {
     nsresult rv;
@@ -803,7 +798,7 @@ nsHttpHandler::DispatchTransaction_Locked(nsHttpTransaction *trans,
     // we must not hold the connection lock while making the call to
     // SetTransaction, as it could lead to deadlocks.
     PR_Unlock(mConnectionLock);
-    rv = conn->SetTransaction(trans, trans->Capabilities());
+    rv = conn->SetTransaction(trans, transCaps);
 
     if (NS_FAILED(rv)) {
         LOG(("nsHttpConnection::SetTransaction failed [rv=%x]\n", rv));
@@ -856,24 +851,47 @@ nsHttpHandler::ProcessTransactionQ_Locked()
     mTransactionQ.RemoveElementAt(i);
 
     // 
-    // step 3: dispatch this transaction
+    // step 3: determine if we can pipeline any other transactions along with
+    // this transaction.  if pipelining is disabled or there are no other
+    // transactions that can be sent with this transaction, then simply send
+    // this transaction.
     //
-    nsresult rv = DispatchTransaction_Locked(pt->Transaction(), conn);
+    nsAHttpTransaction *trans = pt->Transaction();
+    PRUint8 caps = pt->Transaction()->Capabilities();
+    nsPipelineEnqueueState pipelineState;
+    if (conn->SupportsPipelining() &&
+        caps & NS_HTTP_ALLOW_PIPELINING &&
+        BuildPipeline_Locked(pipelineState,
+                             pt->Transaction(),
+                             pt->ConnectionInfo())) {
+        // ok... let's rock!
+        trans = pipelineState.Transaction();
+        caps = pipelineState.TransactionCaps();
+        NS_ASSERTION(trans, "no transaction");
+    }
+
+    // 
+    // step 4: dispatch this transaction
+    //
+    nsresult rv = DispatchTransaction_Locked(trans, caps, conn);
 
     // we're no longer inside mConnectionLock
 
     // 
-    // step 4: handle errors / cleanup
+    // step 5: handle errors / cleanup
     //
     if (NS_FAILED(rv)) {
         LOG((">> DispatchTransaction_Locked failed [rv=%x]\n", rv));
         nsAutoLock lock(mConnectionLock);
         // there must have been something wrong with the connection,
         // requeue... we'll try again later.
+        if (caps & NS_HTTP_ALLOW_PIPELINING)
+            PipelineFailed_Locked(pipelineState);
         mTransactionQ.AppendElement(pt);
     }
     else
         delete pt;
+    pipelineState.Cleanup();
     NS_RELEASE(conn);
 }
 
@@ -961,18 +979,109 @@ nsHttpHandler::DropConnections(nsVoidArray &connections)
     connections.Clear();
 }
 
+PRBool
+nsHttpHandler::BuildPipeline_Locked(nsPipelineEnqueueState &state,
+                                    nsHttpTransaction *firstTrans,
+                                    nsHttpConnectionInfo *ci)
+{
+    if (mMaxPipelinedRequests < 2)
+        return PR_FALSE;
+
+    LOG(("BuildPipeline_Locked [trans=%x]\n", firstTrans));
+
+    //
+    // need to search the pending transaction list for other transactions
+    // that can be pipelined along with |firstTrans|.
+    //
+    nsresult rv = NS_ERROR_FAILURE; // by default, nothing to pipeline
+    PRUint8 numAppended = 0;
+    PRInt32 i = 0;
+    while (i < mTransactionQ.Count()) {
+        nsPendingTransaction *pt = (nsPendingTransaction *) mTransactionQ[i];
+        if (pt->Transaction()->Capabilities() & (NS_HTTP_ALLOW_KEEPALIVE |
+                                                 NS_HTTP_ALLOW_PIPELINING) &&
+            pt->ConnectionInfo()->Equals(ci)) {
+
+            //
+            // ok, we can add this transaction to our pipeline
+            //
+            if (numAppended == 0) {
+                rv = state.Init(firstTrans);
+                if (NS_FAILED(rv)) break;
+            }
+            rv = state.AppendTransaction(pt);
+            if (NS_FAILED(rv)) break;
+
+            //
+            // ok, remove the transaction from the pending queue; next time
+            // around the loop we'll still check the i-th element :-)
+            //
+            mTransactionQ.RemoveElementAt(i);
+
+            //
+            // we may have reached the pipelined requests limit...
+            //
+            if (++numAppended == (mMaxPipelinedRequests - 1))
+                break;
+        }
+        else
+            i++; // advance to next pending transaction
+    }
+    if (NS_FAILED(rv)) {
+        LOG(("  unable to pipeline any transactions with this one\n"));
+        state.Cleanup();
+        return PR_FALSE;
+    }
+    LOG(("  pipelined %u transactions\n", numAppended + 1));
+    return PR_TRUE;
+}
+
+void
+nsHttpHandler::PipelineFailed_Locked(nsPipelineEnqueueState &state)
+{
+    if ((mMaxPipelinedRequests < 2) || !state.HasPipeline())
+        return;
+
+    LOG(("PipelineFailed_Locked\n"));
+
+    // need to put any "appended" transactions back on the queue
+    PRInt32 i = 0;
+    while (i < state.NumAppendedTrans()) {
+        mTransactionQ.AppendElement(state.GetAppendedTrans(i));
+        i++;
+    }
+    state.DropAppendedTrans();
+}
+
 void
 nsHttpHandler::BuildUserAgent()
 {
     LOG(("nsHttpHandler::BuildUserAgent\n"));
 
-    NS_ASSERTION(mAppName &&
-                 mAppVersion &&
-                 mPlatform &&
-                 mSecurity &&
-                 mOscpu,
+    NS_ASSERTION(!mAppName.IsEmpty() &&
+                 !mAppVersion.IsEmpty() &&
+                 !mPlatform.IsEmpty() &&
+                 !mSecurity.IsEmpty() &&
+                 !mOscpu.IsEmpty(),
                  "HTTP cannot send practical requests without this much");
 
+    // preallocate to worst-case size, which should always be better
+    // than if we didn't preallocate at all.
+    mUserAgent.SetCapacity(mAppName.Length() + 
+                           mAppVersion.Length() + 
+                           mPlatform.Length() + 
+                           mSecurity.Length() +
+                           mOscpu.Length() +
+                           mLanguage.Length() +
+                           mMisc.Length() +
+                           mProduct.Length() +
+                           mProductSub.Length() +
+                           mProductComment.Length() +
+                           mVendor.Length() +
+                           mVendorSub.Length() +
+                           mVendorComment.Length() +
+                           22);
+
     // Application portion
     mUserAgent.Assign(mAppName);
     mUserAgent += '/';
@@ -986,25 +1095,25 @@ nsHttpHandler::BuildUserAgent()
     mUserAgent += mSecurity;
     mUserAgent += "; ";
     mUserAgent += mOscpu;
-    if (mLanguage) {
+    if (!mLanguage.IsEmpty()) {
         mUserAgent += "; ";
         mUserAgent += mLanguage;
     }
-    if (mMisc) {
+    if (!mMisc.IsEmpty()) {
         mUserAgent += "; ";
         mUserAgent += mMisc;
     }
     mUserAgent += ')';
 
     // Product portion
-    if (mProduct) {
+    if (!mProduct.IsEmpty()) {
         mUserAgent += ' ';
         mUserAgent += mProduct;
-        if (mProductSub) {
+        if (!mProductSub.IsEmpty()) {
             mUserAgent += '/';
             mUserAgent += mProductSub;
         }
-        if (mProductComment) {
+        if (!mProductComment.IsEmpty()) {
             mUserAgent += " (";
             mUserAgent += mProductComment;
             mUserAgent += ')';
@@ -1012,14 +1121,14 @@ nsHttpHandler::BuildUserAgent()
     }
 
     // Vendor portion
-    if (mVendor) {
+    if (!mVendor.IsEmpty()) {
         mUserAgent += ' ';
         mUserAgent += mVendor;
-        if (mVendorSub) {
+        if (!mVendorSub.IsEmpty()) {
             mUserAgent += '/';
             mUserAgent += mVendorSub;
         }
-        if (mVendorComment) {
+        if (!mVendorComment.IsEmpty()) {
             mUserAgent += " (";
             mUserAgent += mVendorComment;
             mUserAgent += ')';
@@ -1322,10 +1431,11 @@ nsHttpHandler::PrefsChanged(nsIPrefBranch *prefs, const char *pref)
         }
     }
 
-    /*
-    mPipelineMaxRequests  = DEFAULT_PIPELINE_MAX_REQUESTS;
-    rv = prefs->GetIntPref("network.http.pipelining.maxrequests", &mPipelineMaxRequests );
-    */
+    if (PREF_CHANGED(HTTP_PREF("pipelining.maxrequests"))) {
+        rv = prefs->GetIntPref(HTTP_PREF("pipelining.maxrequests"), &val);
+        if (NS_SUCCEEDED(rv))
+            mMaxPipelinedRequests = CLAMP(val, 1, NS_HTTP_MAX_PIPELINED_REQUESTS);
+    }
 
     if (PREF_CHANGED(HTTP_PREF("proxy.pipelining"))) {
         rv = prefs->GetBoolPref(HTTP_PREF("proxy.pipelining"), &cVar);
@@ -1337,6 +1447,12 @@ nsHttpHandler::PrefsChanged(nsIPrefBranch *prefs, const char *pref)
         }
     }
 
+    if (PREF_CHANGED(HTTP_PREF("sendSecureXSiteReferrer"))) {
+        rv = prefs->GetBoolPref(HTTP_PREF("sendSecureXSiteReferrer"), &cVar);
+        if (NS_SUCCEEDED(rv))
+            mSendSecureXSiteReferrer = cVar;
+    }
+
     /*
     if (bChangedAll || PL_strcmp(pref, "network.http.connect.timeout") == 0)
         prefs->GetIntPref("network.http.connect.timeout", &mConnectTimeout);
@@ -1513,7 +1629,7 @@ nsHttpHandler::SetAcceptLanguages(const char *aAcceptLanguages)
     nsCString buf;
     nsresult rv = PrepareAcceptLanguages(aAcceptLanguages, buf);
     if (NS_SUCCEEDED(rv))
-        mAcceptLanguages.Assign(buf.get());
+        mAcceptLanguages.Assign(buf);
     return rv;
 }
 
@@ -1641,7 +1757,7 @@ nsHttpHandler::SetAcceptCharsets(const char *aAcceptCharsets)
     nsCString buf;
     nsresult rv = PrepareAcceptCharsets(aAcceptCharsets, buf);
     if (NS_SUCCEEDED(rv))
-        mAcceptCharsets.Assign(buf.get());
+        mAcceptCharsets.Assign(buf);
     return rv;
 }
 
@@ -1785,135 +1901,148 @@ nsHttpHandler::NewProxiedChannel(nsIURI *uri,
 //-----------------------------------------------------------------------------
 
 NS_IMETHODIMP
-nsHttpHandler::GetUserAgent(char **aUserAgent)
+nsHttpHandler::GetUserAgent(nsACString &value)
 {
-    return DupString(UserAgent(), aUserAgent);
+    value = UserAgent();
+    return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetAppName(char **aAppName)
+nsHttpHandler::GetAppName(nsACString &value)
 {
-    return DupString(mAppName, aAppName);
+    value = mAppName;
+    return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetAppVersion(char **aAppVersion)
+nsHttpHandler::GetAppVersion(nsACString &value)
 {
-    return DupString(mAppVersion, aAppVersion);
+    value = mAppVersion;
+    return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetVendor(char **aVendor)
+nsHttpHandler::GetVendor(nsACString &value)
 {
-    return DupString(mVendor, aVendor);
+    value = mVendor;
+    return NS_OK;
 }
 NS_IMETHODIMP
-nsHttpHandler::SetVendor(const char *aVendor)
+nsHttpHandler::SetVendor(const nsACString &value)
 {
-    mVendor.Adopt(aVendor ? nsCRT::strdup(aVendor) : 0);
+    mVendor = value;
     mUserAgentIsDirty = PR_TRUE;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetVendorSub(char **aVendorSub)
+nsHttpHandler::GetVendorSub(nsACString &value)
 {
-    return DupString(mVendorSub, aVendorSub);
+    value = mVendorSub;
+    return NS_OK;
 }
 NS_IMETHODIMP
-nsHttpHandler::SetVendorSub(const char *aVendorSub)
+nsHttpHandler::SetVendorSub(const nsACString &value)
 {
-    mVendorSub.Adopt(aVendorSub ? nsCRT::strdup(aVendorSub) : 0);
+    mVendorSub = value;
     mUserAgentIsDirty = PR_TRUE;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetVendorComment(char **aVendorComment)
+nsHttpHandler::GetVendorComment(nsACString &value)
 {
-    return DupString(mVendorComment, aVendorComment);
+    value = mVendorComment;
+    return NS_OK;
 }
 NS_IMETHODIMP
-nsHttpHandler::SetVendorComment(const char *aVendorComment)
+nsHttpHandler::SetVendorComment(const nsACString &value)
 {
-    mVendorComment.Adopt(aVendorComment ? nsCRT::strdup(aVendorComment) : 0);
+    mVendorComment = value;
     mUserAgentIsDirty = PR_TRUE;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetProduct(char **aProduct)
+nsHttpHandler::GetProduct(nsACString &value)
 {
-    return DupString(mProduct, aProduct);
+    value = mProduct;
+    return NS_OK;
 }
 NS_IMETHODIMP
-nsHttpHandler::SetProduct(const char *aProduct)
+nsHttpHandler::SetProduct(const nsACString &value)
 {
-    mProduct.Adopt(aProduct ? nsCRT::strdup(aProduct) : 0);
+    mProduct = value;
     mUserAgentIsDirty = PR_TRUE;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetProductSub(char **aProductSub)
+nsHttpHandler::GetProductSub(nsACString &value)
 {
-    return DupString(mProductSub, aProductSub);
+    value = mProductSub;
+    return NS_OK;
 }
 NS_IMETHODIMP
-nsHttpHandler::SetProductSub(const char *aProductSub)
+nsHttpHandler::SetProductSub(const nsACString &value)
 {
-    mProductSub.Adopt(aProductSub ? nsCRT::strdup(aProductSub) : 0);
+    mProductSub = value;
     mUserAgentIsDirty = PR_TRUE;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetProductComment(char **aProductComment)
+nsHttpHandler::GetProductComment(nsACString &value)
 {
-    return DupString(mProductComment, aProductComment);
+    value = mProductComment;
+    return NS_OK;
 }
 NS_IMETHODIMP
-nsHttpHandler::SetProductComment(const char *aProductComment)
+nsHttpHandler::SetProductComment(const nsACString &value)
 {
-    mProductComment.Adopt(aProductComment ? nsCRT::strdup(aProductComment) : 0);
+    mProductComment = value;
     mUserAgentIsDirty = PR_TRUE;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetPlatform(char **aPlatform)
+nsHttpHandler::GetPlatform(nsACString &value)
 {
-    return DupString(mPlatform, aPlatform);
+    value = mPlatform;
+    return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetOscpu(char **aOscpu)
+nsHttpHandler::GetOscpu(nsACString &value)
 {
-    return DupString(mOscpu, aOscpu);
+    value = mOscpu;
+    return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetLanguage(char **aLanguage)
+nsHttpHandler::GetLanguage(nsACString &value)
 {
-    return DupString(mLanguage, aLanguage);
+    value = mLanguage;
+    return NS_OK;
 }
 NS_IMETHODIMP
-nsHttpHandler::SetLanguage(const char *aLanguage)
+nsHttpHandler::SetLanguage(const nsACString &value)
 {
-    mLanguage.Adopt(aLanguage ? nsCRT::strdup(aLanguage) : 0);
+    mLanguage = value;
     mUserAgentIsDirty = PR_TRUE;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetMisc(char **aMisc)
+nsHttpHandler::GetMisc(nsACString &value)
 {
-    return DupString(mMisc, aMisc);
+    value = mMisc;
+    return NS_OK;
 }
 NS_IMETHODIMP
-nsHttpHandler::SetMisc(const char *aMisc)
+nsHttpHandler::SetMisc(const nsACString &value)
 {
-    mMisc.Adopt(aMisc ? nsCRT::strdup(aMisc) : 0);
+    mMisc = value;
     mUserAgentIsDirty = PR_TRUE;
     return NS_OK;
 }
@@ -1927,6 +2056,8 @@ nsHttpHandler::Observe(nsISupports *subject,
                        const char *topic,
                        const PRUnichar *data)
 {
+    LOG(("nsHttpHandler::Observe [topic=\"%s\")]\n", topic));
+
     if (!nsCRT::strcmp(topic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID)) {
         nsCOMPtr<nsIPrefBranch> prefBranch = do_QueryInterface(subject);
         if (prefBranch)
@@ -1980,3 +2111,42 @@ nsPendingTransaction::~nsPendingTransaction()
     NS_RELEASE(mTransaction);
     NS_RELEASE(mConnectionInfo);
 }
+
+//-----------------------------------------------------------------------------
+// nsHttpHandler::nsPipelineEnqueueState
+//-----------------------------------------------------------------------------
+
+nsresult nsHttpHandler::
+nsPipelineEnqueueState::Init(nsHttpTransaction *firstTrans)
+{
+    NS_ASSERTION(mPipeline == nsnull, "already initialized");
+
+    mPipeline = new nsHttpPipeline;
+    if (!mPipeline)
+        return NS_ERROR_OUT_OF_MEMORY;
+    NS_ADDREF(mPipeline);
+
+    return mPipeline->Init(firstTrans);
+}
+
+nsresult nsHttpHandler::
+nsPipelineEnqueueState::AppendTransaction(nsPendingTransaction *pt)
+{
+    nsresult rv = mPipeline->AppendTransaction(pt->Transaction());
+    if (NS_FAILED(rv)) return rv;
+
+    // remember this pending transaction object
+    mAppendedTrans.AppendElement(pt);
+    return NS_OK;
+}
+
+void nsHttpHandler::
+nsPipelineEnqueueState::Cleanup()
+{
+    NS_IF_RELEASE(mPipeline);
+
+    // free any appended pending transaction objects
+    for (PRInt32 i=0; i < mAppendedTrans.Count(); i++)
+        delete GetAppendedTrans(i);
+    mAppendedTrans.Clear();
+}
