@@ -95,6 +95,7 @@ nsHttpTransaction::nsHttpTransaction(nsIStreamListener *listener,
     , mCapabilities(caps)
     , mHaveStatusLine(PR_FALSE)
     , mHaveAllHeaders(PR_FALSE)
+    , mResponseIsComplete(PR_FALSE)
     , mFiredOnStart(PR_FALSE)
     , mNoContent(PR_FALSE)
     , mPrematureEOF(PR_FALSE)
@@ -119,7 +120,8 @@ nsHttpTransaction::~nsHttpTransaction()
 nsresult
 nsHttpTransaction::SetupRequest(nsHttpRequestHead *requestHead,
                                 nsIInputStream *requestBody,
-                                PRBool requestBodyHasHeaders)
+                                PRBool requestBodyHasHeaders,
+                                PRBool pruneProxyHeaders)
 {
     nsresult rv;
 
@@ -155,7 +157,7 @@ nsHttpTransaction::SetupRequest(nsHttpRequestHead *requestHead,
     mRequestHead = requestHead;
 
     mReqHeaderBuf.SetLength(0);
-    requestHead->Flatten(mReqHeaderBuf);
+    requestHead->Flatten(mReqHeaderBuf, pruneProxyHeaders);
 
 #if defined(PR_LOGGING)
     if (LOG2_ENABLED()) {
@@ -198,6 +200,15 @@ nsHttpTransaction::TakeResponseHead()
 // nsHttpTransaction::nsAHttpTransaction
 //----------------------------------------------------------------------------
 
+PRUint32
+nsHttpTransaction::GetRequestSize()
+{
+    PRUint32 n = 0;
+    if (mReqHeaderStream)
+        mReqHeaderStream->Available(&n);
+    return n;
+}
+
 // called on the socket transport thread
 nsresult
 nsHttpTransaction::OnDataWritable(nsIOutputStream *os)
@@ -266,12 +277,19 @@ nsHttpTransaction::OnStopTransaction(nsresult status)
 
     // if the connection was reset before we read any part of the response,
     // then we must try to restart the transaction.
-    if ((status == NS_ERROR_NET_RESET) && (mContentRead == 0)) {
+    if (status == NS_ERROR_NET_RESET) {
+        // if some data was read, then mask the reset error, so our listener
+        // will treat this as a normal failure.  XXX we might want to map
+        // this error to a special error code to indicate that the transfer
+        // was abnormally interrupted.
+        if (mContentRead > 0)
+            status = NS_ERROR_ABORT;
         // if restarting fails, then we must notify our listener.
-        if (NS_SUCCEEDED(Restart()))
+        else if (NS_SUCCEEDED(Restart()))
             return NS_OK;
     }
 
+    mTransactionDone = 1; // force this flag
     mStatus = status;
 
 	if (mListener) {
@@ -309,7 +327,7 @@ nsHttpTransaction::Restart()
     // limit the number of restart attempts - bug 92224
     if (++mRestartCount >= nsHttpHandler::get()->MaxRequestAttempts()) {
         LOG(("reached max request attempts, failing transaction @%x\n", this));
-        return NS_BINDING_FAILED;
+        return NS_ERROR_NET_RESET;
     }
 
     LOG(("restarting transaction @%x\n", this));
@@ -552,19 +570,21 @@ nsHttpTransaction::HandleContentStart()
 nsresult
 nsHttpTransaction::HandleContent(char *buf,
                                  PRUint32 count,
-                                 PRUint32 *countRead)
+                                 PRUint32 *contentRead,
+                                 PRUint32 *bufRead)
 {
     nsresult rv;
 
     LOG(("nsHttpTransaction::HandleContent [this=%x count=%u]\n",
         this, count));
 
-    *countRead = 0;
+    *contentRead = 0;
+    *bufRead = 0;
 
     if (mTransactionDone)
         return NS_OK;
 
-    NS_PRECONDITION(mConnection, "no connection");
+    NS_ASSERTION(mConnection, "no connection");
 
     if (!mFiredOnStart) {
         rv = HandleContentStart();
@@ -574,7 +594,7 @@ nsHttpTransaction::HandleContent(char *buf,
     if (mChunkedDecoder) {
         // give the buf over to the chunked decoder so it can reformat the
         // data and tell us how much is really there.
-        rv = mChunkedDecoder->HandleChunkedContent(buf, count, countRead);
+        rv = mChunkedDecoder->HandleChunkedContent(buf, count, contentRead, bufRead);
         if (NS_FAILED(rv)) return rv;
     }
     else if (mContentLength >= 0) {
@@ -583,31 +603,33 @@ nsHttpTransaction::HandleContent(char *buf,
         // allowances for a possibly invalid Content-Length header. Thus, if
         // NOT persistent, we simply accept everything in |buf|.
         if (mConnection->IsPersistent()) {
-            *countRead = PRUint32(mContentLength) - mContentRead;
-            *countRead = PR_MIN(count, *countRead);
+            *bufRead = PRUint32(mContentLength) - mContentRead;
+            *bufRead = PR_MIN(count, *bufRead);
         }
         else {
-            *countRead = count;
+            *bufRead = count;
             // mContentLength might need to be increased...
-            if (*countRead + mContentRead > (PRUint32) mContentLength) {
-                mContentLength = *countRead + mContentRead;
+            if (*bufRead + mContentRead > (PRUint32) mContentLength) {
+                mContentLength = *bufRead + mContentRead;
                 //mResponseHead->SetContentLength(mContentLength);
             }
         }
+        *contentRead = *bufRead;
     }
-    else
+    else {
         // when we are just waiting for the server to close the connection...
-        *countRead = count;
+        *contentRead = *bufRead = count;
+    }
 
-    if (*countRead) {
+    if (*contentRead) {
         // update count of content bytes read and report progress...
-        mContentRead += *countRead;
+        mContentRead += *contentRead;
         if (mProgressSink)
             mProgressSink->OnProgress(nsnull, nsnull, mContentRead, PR_MAX(0, mContentLength));
     }
 
     LOG(("nsHttpTransaction [this=%x count=%u read=%u mContentRead=%u mContentLength=%d]\n",
-        this, count, *countRead, mContentRead, mContentLength));
+        this, count, *contentRead, mContentRead, mContentLength));
 
     // check for end-of-file
     if ((mContentRead == PRUint32(mContentLength)) ||
@@ -616,6 +638,7 @@ nsHttpTransaction::HandleContent(char *buf,
         // OnTransactionComplete is fired only once!
         PRInt32 priorVal = PR_AtomicSet(&mTransactionDone, 1);
         if (priorVal == 0) {
+            mResponseIsComplete = PR_TRUE;
             // let the connection know that we are done with it; this should
             // result in OnStopTransaction being fired.
             return mConnection->OnTransactionComplete(this, NS_OK);
@@ -625,7 +648,7 @@ nsHttpTransaction::HandleContent(char *buf,
 
     // if we didn't "read" anything and this is not a no-content response,
     // then we must return NS_BASE_STREAM_WOULD_BLOCK so we'll be called again.
-    return (!mNoContent && !*countRead) ? NS_BASE_STREAM_WOULD_BLOCK : NS_OK;
+    return (!mNoContent && !*contentRead) ? NS_BASE_STREAM_WOULD_BLOCK : NS_OK;
 }
 
 void
@@ -711,7 +734,7 @@ NS_IMPL_THREADSAFE_QUERY_INTERFACE2(nsHttpTransaction,
 //-----------------------------------------------------------------------------
 
 NS_IMETHODIMP
-nsHttpTransaction::GetName(PRUnichar **aName)
+nsHttpTransaction::GetName(nsACString &aName)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -742,10 +765,6 @@ nsHttpTransaction::Cancel(nsresult status)
         return NS_OK;
     }
 
-    // the status must be set immediately as the cancelation may only take
-    // action asynchronously.
-    mStatus = status;
-
     // if the transaction is already "done" then there is nothing more to do.
     // ie., our consumer _will_ eventually receive their OnStopRequest.
     PRInt32 priorVal = PR_AtomicSet(&mTransactionDone, 1);
@@ -754,6 +773,10 @@ nsHttpTransaction::Cancel(nsresult status)
         return NS_OK;
     }
 
+    // the status must be set immediately as the cancelation may only take
+    // action asynchronously.
+    mStatus = status;
+
     return nsHttpHandler::get()->CancelTransaction(this, status);
 }
 
@@ -875,8 +898,35 @@ nsHttpTransaction::Read(char *buf, PRUint32 count, PRUint32 *bytesWritten)
 
     // even though count may be 0, we still want to call HandleContent
     // so it can complete the transaction if this is a "no-content" response.
-    if (mHaveAllHeaders)
-        return HandleContent(buf, count, bytesWritten);
+    if (mHaveAllHeaders) {
+        PRUint32 countConsumed = 0;
+        //
+        // buf layout:
+        // 
+        //  +----------------------------------------+----+-------+
+        //  | buf                                    |    |       |
+        //  +----------------------------------------+----+-------+
+        //                                           |    |       |
+        //                                           |    |       count
+        //                                           |    |
+        //                                           |    countConsumed
+        //                                           |
+        //                                           bytesWritten
+        //
+        // count         : size of buf
+        // countConsumed : portion of buf corresponding to this transaction
+        // bytesWritten  : portion of buf corresponding to this transaction,
+        //                 minus chunked encoding boundaries.
+        //
+        rv = HandleContent(buf, count, bytesWritten, &countConsumed);
+        if (NS_FAILED(rv)) return rv;
+        // we may have read more than our share, in which case we must give
+        // the excess bytes back to the connection
+        if (mResponseIsComplete && (count > countConsumed)) {
+            NS_ASSERTION(mConnection, "no connection");
+            mConnection->PushBack(buf + countConsumed, count - countConsumed);
+        }
+    }
 
     // wait for more data
     return NS_BASE_STREAM_WOULD_BLOCK;
@@ -890,18 +940,7 @@ nsHttpTransaction::ReadSegments(nsWriteSegmentFun writer, void *closure,
 }
 
 NS_IMETHODIMP
-nsHttpTransaction::GetNonBlocking(PRBool *result)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsHttpTransaction::GetObserver(nsIInputStreamObserver **obs)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-NS_IMETHODIMP
-nsHttpTransaction::SetObserver(nsIInputStreamObserver *obs)
+nsHttpTransaction::IsNonBlocking(PRBool *result)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
