@@ -1,3 +1,4 @@
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* 
  * The contents of this file are subject to the Mozilla Public
  * License Version 1.1 (the "License"); you may not use this file
@@ -30,23 +31,6 @@
  * may use your version of this file under either the MPL or the
  * GPL.
  */
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/*
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "NPL"); you may not use this file except in
- * compliance with the NPL.  You may obtain a copy of the NPL at
- * http://www.mozilla.org/NPL/
- * 
- * Software distributed under the NPL is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
- * for the specific language governing rights and limitations under the
- * NPL.
- * 
- * The Initial Developer of this code under the NPL is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation.  All Rights
- * Reserved.
- */
 
 #include <string.h>
 
@@ -97,8 +81,7 @@ static void AsyncIOCompletion (ExtendedParamBlock *pbAsyncPtr)
     if (_PR_MD_GET_INTSOFF()) {
         thread->md.missedIONotify = PR_TRUE;
         cpu->u.missed[cpu->where] |= _PR_MISSED_IO;
-    }
-    else {
+    } else {
         _PR_INTSOFF(is);
 
         thread->md.osErrCode = noErr;
@@ -483,8 +466,6 @@ PROffset32 _MD_LSeek(PRFileDesc *fd, PROffset32 offset, PRSeekWhence how)
 	OSErr 	err = noErr;
 	long	curPos, endPos;
 
-	PR_ASSERT(offset >= 0);
-
 	/* compute new mark */
 	switch (how) {
 		case PR_SEEK_SET:
