@@ -48,7 +48,10 @@
 #include "nsIWindowMediator.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIScriptContext.h"
-#include "nsSpecialSystemDirectory.h"
+#include "nsILocalFile.h"
+#include "nsDirectoryServiceUtils.h"
+#include "nsDirectoryServiceDefs.h"
+#include "nsDependentString.h"
 #include "nsFileStream.h"
 #include "nsIFileSpec.h"
 #include "nsCOMPtr.h"
@@ -178,20 +181,6 @@
 
 #define NEW_DIR_SUFFIX "5"
 
-/* who's going to win the file name battle? */
-#if defined(XP_UNIX)
-#define PREF_FILE_NAME_IN_4x "preferences.js"
-#elif defined(XP_MAC)
-#define PREF_FILE_NAME_IN_4x "Netscape Preferences"
-#elif defined(XP_PC)
-#define PREF_FILE_NAME_IN_4x "prefs.js"
-#else
-/* this will cause a failure at run time, as it should, since we don't know
-   how to migrate platforms other than Mac, Windows and UNIX */
-#define PREF_FILE_NAME_IN_4x ""
-#endif /* XP_UNIX */
-
-/* and the winner is:  Windows */
 #define PREF_FILE_NAME_IN_5x "prefs.js"
 
 #define PREF_MIGRATION_PROGRESS_URL "chrome://communicator/content/profile/profileMigrationProgress.xul"
@@ -414,7 +403,7 @@ extern "C" void ProfileMigrationController(void *data)
 
     if (migrator->mErrorCode != 0)
     {
-      if (migrator->mErrorCode == RETRY)
+      if (migrator->mErrorCode == MIGRATION_RETRY)
       {
         rv = prefProxy->ShowSpaceDialog(&choice);
         if (NS_FAILED(rv))
@@ -426,7 +415,7 @@ extern "C" void ProfileMigrationController(void *data)
       }
     }
 
-  } while (choice == RETRY);
+  } while (choice == MIGRATION_RETRY);
 
   prefProxy->WindowCloseCallback();
   migrator->mErrorCode = choice;
@@ -628,34 +617,28 @@ nsPrefMigration::ProcessPrefsCallback(const char* oldProfilePathStr, const char
                      getter_AddRefs(PrefsFile4xAsIFile));
   if (NS_FAILED(rv)) return rv;
 
-  nsSpecialSystemDirectory systemTempDir(nsSpecialSystemDirectory::OS_TemporaryDirectory);
-  nsFileSpec tempPrefsFile = systemTempDir;
-  
-  systemTempDir += "migrate"; 
-  
-  //Convert the systemTempDir to an nsILocalFile
-  nsCOMPtr<nsILocalFile> systemTempIFileDir;
-  rv = NS_NewLocalFile(systemTempDir, PR_TRUE, getter_AddRefs(systemTempIFileDir));
+  nsCOMPtr<nsIFile> systemTempDir;
+  rv = NS_GetSpecialDirectory(NS_OS_TEMP_DIR, getter_AddRefs(systemTempDir));
   if (NS_FAILED(rv)) return rv;
 
+  systemTempDir->AppendNative(NS_LITERAL_CSTRING("migrate"));
+  
   //Create a unique directory in the system temp dir based on the name of the 4.x prefs file
-  rv = systemTempIFileDir->CreateUnique(nsIFile::DIRECTORY_TYPE, 0700); 
+  rv = systemTempDir->CreateUnique(nsIFile::DIRECTORY_TYPE, 0700); 
   if (NS_FAILED(rv)) return rv;
 
-  rv = PrefsFile4xAsIFile->CopyTo(systemTempIFileDir, PREF_FILE_NAME_IN_4x);
+  rv = PrefsFile4xAsIFile->CopyToNative(systemTempDir, NS_LITERAL_CSTRING(PREF_FILE_NAME_IN_4x));
   if (NS_FAILED(rv)) return rv;
   
-  //Get the new unique directory name and append it to the system temp dir
-  nsXPIDLCString uniquePrefsDir;
-  systemTempIFileDir->GetLeafName(getter_Copies(uniquePrefsDir));
-  tempPrefsFile += uniquePrefsDir;
-  tempPrefsFile += PREF_FILE_NAME_IN_4x;
+  nsCOMPtr<nsIFile> cloneFile;
+  rv = systemTempDir->Clone(getter_AddRefs(cloneFile));
+  if (NS_FAILED(rv)) return rv;
 
-  //Create the m_prefsFile fileSpec for use in ReadUserPrefsFrom
-//  rv = NS_NewFileSpecWithSpec(tempPrefsFile, getter_AddRefs(m_prefsFile));
-//  if (NS_FAILED(rv)) return rv;
+  m_prefsFile = do_QueryInterface(cloneFile, &rv);
+  if (NS_FAILED(rv)) return rv;
 
-  NS_NewLocalFile((const char *)tempPrefsFile, PR_TRUE, getter_AddRefs(m_prefsFile));
+  rv = m_prefsFile->AppendNative(NS_LITERAL_CSTRING(PREF_FILE_NAME_IN_4x));
+  if (NS_FAILED(rv)) return rv;
 
   //Clear the prefs in case a previous set was read in.
   m_prefs->ResetPrefs();
@@ -846,7 +829,7 @@ nsPrefMigration::ProcessPrefsCallback(const char* oldProfilePathStr, const char
 
     if (!enoughSpace)
     {
-      mErrorCode = RETRY; 
+      mErrorCode = MIGRATION_RETRY; 
       return NS_OK;
     }
 
@@ -1134,9 +1117,9 @@ nsPrefMigration::ProcessPrefsCallback(const char* oldProfilePathStr, const char
   nsXPIDLCString path;
 
   newProfilePath->GetNativePath(getter_Copies(path));
-  NS_NewLocalFile(path, PR_TRUE, getter_AddRefs(newPrefsFile));
+  NS_NewNativeLocalFile(path, PR_TRUE, getter_AddRefs(newPrefsFile));
 
-  rv = newPrefsFile->Append(PREF_FILE_NAME_IN_5x);
+  rv = newPrefsFile->AppendNative(NS_LITERAL_CSTRING(PREF_FILE_NAME_IN_5x));
   if (NS_FAILED(rv)) return rv;
 
   rv=m_prefs->SavePrefFile(newPrefsFile);
@@ -1149,9 +1132,9 @@ nsPrefMigration::ProcessPrefsCallback(const char* oldProfilePathStr, const char
   if (flagExists)
     m_prefsFile->Remove(PR_FALSE);
   
-  systemTempIFileDir->Exists(&flagExists); //Delete the unique dir in the system temp dir.
+  systemTempDir->Exists(&flagExists); //Delete the unique dir in the system temp dir.
   if (flagExists)
-    systemTempIFileDir->Remove(PR_FALSE);
+    systemTempDir->Remove(PR_FALSE);
 
   return rv;
 }
@@ -1264,7 +1247,7 @@ nsPrefMigration::GetDirFromPref(nsIFileSpec * oldProfilePath, nsIFileSpec * newP
   if (NS_FAILED(rv)) return rv;
   
   // convert nsILocalFile to nsIFileSpec
-  rv = oldPrefPathFile->GetPath(getter_Copies(oldPrefPathStr));
+  rv = oldPrefPathFile->GetNativePath(oldPrefPathStr);
   if (NS_FAILED(rv)) return rv;
 
   rv = NS_NewFileSpec(getter_AddRefs(oldPrefPath));
@@ -1597,7 +1580,7 @@ GetCookieLine(nsInputFileStream &strm, nsAutoString& aLine)
 
     /* stop at the '\r' */
     if (c != '\r') {
-      aLine.AppendWithConversion(c);
+      aLine.Append(PRUnichar(c));
     }
     else {
       break;
@@ -2028,7 +2011,7 @@ nsPrefMigration::DetermineOldPath(nsIFileSpec *profilePath, const char *oldPathN
 	rv = bundle->GetStringFromName(entityName.get(), getter_Copies(localizedDirName));
 	if (NS_FAILED(rv)) return rv;
 
-	rv = oldLocalFile->AppendRelativeUnicodePath((const PRUnichar *)localizedDirName);
+	rv = oldLocalFile->AppendRelativePath(localizedDirName);
 	if (NS_FAILED(rv)) return rv;
 
 	PRBool exists = PR_FALSE;
@@ -2045,10 +2028,10 @@ nsPrefMigration::DetermineOldPath(nsIFileSpec *profilePath, const char *oldPathN
 	}
 
 	/* at this point, the folder with the localized name exists, so use it */
-	nsXPIDLCString persistentDescriptor;
-	rv = oldLocalFile->GetPersistentDescriptor(getter_Copies(persistentDescriptor));
+	nsCAutoString persistentDescriptor;
+	rv = oldLocalFile->GetPersistentDescriptor(persistentDescriptor);
 	if (NS_FAILED(rv)) return rv;
-	rv = oldPath->SetPersistentDescriptorString((const char *)persistentDescriptor);
+	rv = oldPath->SetPersistentDescriptorString(persistentDescriptor.get());
 	if (NS_FAILED(rv)) return rv;
 
 	return NS_OK;
@@ -2279,7 +2262,7 @@ nsPrefConverter::GetPlatformCharset(nsAutoString& aCharset)
   // we may cache it since the platform charset will not change through application life
   nsCOMPtr <nsIPlatformCharset> platformCharset = do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
   if (NS_SUCCEEDED(rv) && platformCharset) {
-   rv = platformCharset->GetCharset(kPlatformCharsetSel_FileName, aCharset);
+   rv = platformCharset->GetCharset(kPlatformCharsetSel_4xPrefsJS, aCharset);
   }
   if (NS_FAILED(rv)) {
    aCharset.Assign(NS_LITERAL_STRING("ISO-8859-1"));  // use ISO-8859-1 in case of any error
