@@ -92,6 +92,7 @@
 #include "rdfutil.h"
 #include "nsReadableUtils.h"
 #include "nsIExpatSink.h"
+#include "nsCRT.h"
 
 ////////////////////////////////////////////////////////////////////////
 
@@ -161,7 +162,7 @@ public:
     NS_IMETHOD WillResume(void);
     NS_IMETHOD SetParser(nsIParser* aParser);  
     NS_IMETHOD FlushPendingNotifications() { return NS_OK; }
-    NS_IMETHOD SetDocumentCharset(nsAWritableString& aCharset) { return NS_OK; }
+    NS_IMETHOD SetDocumentCharset(nsAString& aCharset) { return NS_OK; }
 
     // nsIRDFContentSink
     NS_IMETHOD Init(nsIURI* aURL);
@@ -216,7 +217,7 @@ protected:
 
     // namespace management
     nsresult PushNameSpacesFrom(const PRUnichar** aAttributes);
-    PRBool   IsXMLNSDirective(const nsAReadableString& aAttributeKey, nsIAtom** aPrefix = nsnull);
+    PRBool   IsXMLNSDirective(const nsAString& aAttributeKey, nsIAtom** aPrefix = nsnull);
     nsresult PopNameSpaces();
 
     PRUnichar* mText;
@@ -252,12 +253,12 @@ protected:
     GetNameSpaceURI(nsIAtom* aPrefix, const char** aNameSpaceURI);
 
     nsresult
-    ParseTagString(const nsAReadableString& aTagName,
+    ParseTagString(const nsAString& aTagName,
                    const char** aNameSpaceURI,
                    nsIAtom** aTag);
 
     nsresult
-    ParseAttributeString(const nsAReadableString& aAttributeName,
+    ParseAttributeString(const nsAString& aAttributeName,
                          const char** aNameSpaceURI,
                          nsIAtom** aAttribute);
 
@@ -603,7 +604,11 @@ RDFContentSinkImpl::HandleCDataSection(const PRUnichar *aData,
 }
 
 NS_IMETHODIMP 
-RDFContentSinkImpl::HandleDoctypeDecl(const PRUnichar *aDoctype, PRUint32 aLength)
+RDFContentSinkImpl::HandleDoctypeDecl(const nsAString & aSubset, 
+                                      const nsAString & aName, 
+                                      const nsAString & aSystemId, 
+                                      const nsAString & aPublicId,
+                                      nsISupports* aCatalogData)
 {
     return NS_OK;
 }
@@ -1461,7 +1466,7 @@ RDFContentSinkImpl::GetNameSpaceURI(nsIAtom* aPrefix, const char** aNameSpaceURI
 }
 
 nsresult
-RDFContentSinkImpl::ParseTagString(const nsAReadableString& aTagName,
+RDFContentSinkImpl::ParseTagString(const nsAString& aTagName,
                                    const char** aNameSpaceURI,
                                    nsIAtom** aTag)
 {
@@ -1476,7 +1481,7 @@ RDFContentSinkImpl::ParseTagString(const nsAReadableString& aTagName,
 
 
 nsresult
-RDFContentSinkImpl::ParseAttributeString(const nsAReadableString& aAttributeName,
+RDFContentSinkImpl::ParseAttributeString(const nsAString& aAttributeName,
                                          const char** aNameSpaceURI,
                                          nsIAtom** aAttribute)
 {
@@ -1660,7 +1665,7 @@ RDFContentSinkImpl::PopContext(nsIRDFResource         *&aResource,
 // Namespace management
 
 PRBool
-RDFContentSinkImpl::IsXMLNSDirective(const nsAReadableString& aAttributeKey, nsIAtom** aPrefix)
+RDFContentSinkImpl::IsXMLNSDirective(const nsAString& aAttributeKey, nsIAtom** aPrefix)
 {
     nsAutoString attr(aAttributeKey);
 
