@@ -48,6 +48,7 @@ extern "C" {
 #include "nsKeygenThread.h"
 #include "nsReadableUtils.h"
 #include "nsUnicharUtils.h"
+#include "nsCRT.h"
 
 //These defines are taken from the PKCS#11 spec
 #define CKM_RSA_PKCS_KEY_PAIR_GEN     0x00000000
@@ -112,10 +113,10 @@ static const char *mozKeyGen = "-mozilla-keygen";
 static PQGParams *
 decode_pqg_params(char *aStr)
 {
-    unsigned char *buf;
+    unsigned char *buf = nsnull;
     unsigned int len;
-    PRArenaPool *arena;
-    PQGParams *params;
+    PRArenaPool *arena = nsnull;
+    PQGParams *params = nsnull;
     SECStatus status;
 
     arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
