@@ -53,6 +53,7 @@
 #include "nsIEventQueue.h"
 #include "nsIRunnable.h"
 #include "plevent.h"
+#include "nsCRT.h"
 
 #include "nsIWindowWatcher.h"
 #include "nsIPrompt.h"
@@ -66,6 +67,7 @@
 #include "nsIEntropyCollector.h"
 #include "nsIBufEntropyCollector.h"
 #include "nsIServiceManager.h"
+#include "nsILocalFile.h"
 
 #include "nss.h"
 #include "pk11func.h"
@@ -267,9 +269,9 @@ nsNSSComponent::~nsNSSComponent()
 
 #ifdef XP_MAC
 #ifdef DEBUG
-#define LOADABLE_CERTS_MODULE "NSSckbiDebug.shlb"
+#define LOADABLE_CERTS_MODULE NS_LITERAL_CSTRING("NSSckbiDebug.shlb")
 #else
-#define LOADABLE_CERTS_MODULE "NSSckbi.shlb"
+#define LOADABLE_CERTS_MODULE NS_LITERAL_CSTRING("NSSckbi.shlb")
 #endif /*DEBUG*/ 
 #endif /*XP_MAC*/
 
@@ -356,7 +358,7 @@ nsNSSComponent::InstallLoadableRoots()
   }
   if (!hasRoot) {
     nsresult rv;
-    nsString modName;
+    nsAutoString modName;
     rv = GetPIPNSSBundleString(NS_LITERAL_STRING("RootCertModuleName").get(),
                                modName);
     if (NS_FAILED(rv)) return;
@@ -367,7 +369,7 @@ nsNSSComponent::InstallLoadableRoots()
       return;
     
     directoryService->Get( NS_XPCOM_CURRENT_PROCESS_DIR,
-                           NS_GET_IID(nsIFile), 
+                           NS_GET_IID(nsILocalFile), 
                            getter_AddRefs(mozFile));
     
     if (!mozFile) {
@@ -375,23 +377,25 @@ nsNSSComponent::InstallLoadableRoots()
     }
     char *fullModuleName = nsnull;
 #ifdef XP_MAC
-    char *unixModulePath=nsnull;
-    mozFile->Append("Essential Files");
-    mozFile->Append(LOADABLE_CERTS_MODULE);
-    mozFile->GetPath(&fullModuleName);    
+    nsCAutoString nativePath;
+    mozFile->AppendNative(NS_LITERAL_CSTRING("Essential Files"));
+    mozFile->AppendNative(LOADABLE_CERTS_MODULE);
+    mozFile->GetNativePath(nativePath);    
+    fullModuleName = (char *) nativePath.get();
 #else
-    char *processDir = nsnull;
-    mozFile->GetPath(&processDir);
-    fullModuleName = PR_GetLibraryName(processDir, "nssckbi");
-    nsMemory::Free(processDir);
+    nsCAutoString processDir;
+    mozFile->GetNativePath(processDir);
+    fullModuleName = PR_GetLibraryName(processDir.get(), "nssckbi");
 #endif
     /* If a module exists with the same name, delete it. */
     char *modNameCString = ToNewCString(modName);
     int modType;
     SECMOD_DeleteModule(modNameCString, &modType);
     SECMOD_AddNewModule(modNameCString, fullModuleName, 0, 0);
-    nsMemory::Free(fullModuleName);
     nsMemory::Free(modNameCString);
+#ifndef XP_MAC
+    PR_Free(fullModuleName); // allocated by NSPR
+#endif
   }
 }
 
@@ -861,7 +865,7 @@ nsNSSComponent::InitializeNSS()
       certHashtable_valueCompare, 0, 0 );
 
     nsresult rv;
-    nsXPIDLCString profileStr;
+    nsCAutoString profileStr;
     nsCOMPtr<nsIFile> profilePath;
 
     rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR,
@@ -874,12 +878,12 @@ nsNSSComponent::InitializeNSS()
   #ifdef XP_MAC
     // On the Mac we place all NSS DBs in the Security
     // Folder in the profile directory.
-    profilePath->Append("Security");
+    profilePath->AppendNative(NS_LITERAL_CSTRING("Security"));
     profilePath->Create(nsIFile::DIRECTORY_TYPE, 0); //This is for Mac, don't worry about
                                                      //permissions.
   #endif 
 
-    rv = profilePath->GetPath(getter_Copies(profileStr));
+    rv = profilePath->GetNativePath(profileStr);
     if (NS_FAILED(rv)) 
       return rv;
 
@@ -902,7 +906,7 @@ nsNSSComponent::InitializeNSS()
 
     ConfigureInternalPKCS11Token();
 
-    if (::NSS_InitReadWrite(profileStr) != SECSuccess) {
+    if (::NSS_InitReadWrite(profileStr.get()) != SECSuccess) {
       PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("can not init NSS r/w in %s\n", profileStr.get()));
 
       if (supress_warning_preference) {
@@ -913,11 +917,11 @@ nsNSSComponent::InitializeNSS()
       }
 
       // try to init r/o
-      if (NSS_Init(profileStr) != SECSuccess) {
+      if (NSS_Init(profileStr.get()) != SECSuccess) {
         PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("can not init in r/o either\n"));
         which_nss_problem = problem_no_security_at_all;
 
-        NSS_NoDB_Init(profileStr);
+        NSS_NoDB_Init(profileStr.get());
       }
     }
 
@@ -1038,6 +1042,7 @@ nsNSSComponent::ShutdownNSS()
                                 (void*) this);
     }
 
+    SSL_ClearSessionCache();
     ::NSS_Shutdown();
   }
 
@@ -1639,15 +1644,18 @@ PSMContentDownloader::OnStartRequest(nsIRequest* request, nsISupports* context)
   // Get the URI //
   channel->GetURI(getter_AddRefs(mURI));
 
-  rv = channel->GetContentLength(&mContentLength);
-  if (rv != NS_OK || mContentLength == -1)
-    mContentLength = kDefaultCertAllocLength;
+  PRInt32 contentLength;
+  rv = channel->GetContentLength(&contentLength);
+  if (NS_FAILED(rv) || contentLength <= 0)
+    contentLength = kDefaultCertAllocLength;
   
   mBufferOffset = 0;
-  mByteData = (char*) nsMemory::Alloc(mContentLength);
+  mBufferSize = 0;
+  mByteData = (char*) nsMemory::Alloc(contentLength);
   if (!mByteData)
     return NS_ERROR_OUT_OF_MEMORY;
   
+  mBufferSize = contentLength;
   return NS_OK;
 }
 
@@ -1664,21 +1672,21 @@ PSMContentDownloader::OnDataAvailable(nsIRequest* request,
   PRUint32 amt;
   nsresult err;
   //Do a check to see if we need to allocate more memory.
-  if ((mBufferOffset + (PRInt32)aLength) > mContentLength) {
-      size_t newSize = mContentLength + kDefaultCertAllocLength;
+  if ((mBufferOffset + (PRInt32)aLength) > mBufferSize) {
+      size_t newSize = (mBufferOffset + aLength) *2; // grow some more than needed
       char *newBuffer;
       newBuffer = (char*)nsMemory::Realloc(mByteData, newSize);
       if (newBuffer == nsnull) {
         return NS_ERROR_OUT_OF_MEMORY;
       }
       mByteData = newBuffer;
-      mContentLength = newSize;
+      mBufferSize = newSize;
   }
   
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("CertDownloader::OnDataAvailable\n"));
   do {
     err = aIStream->Read(mByteData+mBufferOffset,
-                         mContentLength-mBufferOffset, &amt);
+                         aLength, &amt);
     if (amt == 0) break;
     if (NS_FAILED(err)) return err;
     
