@@ -58,6 +58,7 @@
 #include "nsReadableUtils.h"
 #include "nsVoidArray.h"
 #include "nsHashtable.h"
+#include "nsCRT.h"
 
 #include "ssl.h"
 #include "secerr.h"
@@ -150,7 +151,6 @@ nsSSLIOLayerSetOptions(PRFileDesc *fd, PRBool forTLSStepUp,
 nsNSSSocketInfo::nsNSSSocketInfo()
   : mFd(nsnull),
     mSecurityState(nsIWebProgressListener::STATE_IS_INSECURE),
-    mForceHandshake(PR_FALSE),
     mForTLSStepUp(PR_FALSE),
     mFirstWrite(PR_TRUE),
     mTLSIntolerant(PR_FALSE),
@@ -245,14 +245,14 @@ nsNSSSocketInfo::SetNotificationCallbacks(nsIInterfaceRequestor* aCallbacks)
 }
 
 NS_IMETHODIMP
-nsNSSSocketInfo::GetSecurityState(PRInt32* state)
+nsNSSSocketInfo::GetSecurityState(PRUint32* state)
 {
   *state = mSecurityState;
   return NS_OK;
 }
 
 nsresult
-nsNSSSocketInfo::SetSecurityState(PRInt32 aState)
+nsNSSSocketInfo::SetSecurityState(PRUint32 aState)
 {
   mSecurityState = aState;
   return NS_OK;
@@ -295,14 +295,14 @@ NS_IMETHODIMP nsNSSSocketInfo::GetInterface(const nsIID & uuid, void * *result)
 NS_IMETHODIMP
 nsNSSSocketInfo::GetForceHandshake(PRBool* forceHandshake)
 {
-  *forceHandshake = mForceHandshake;
+  *forceHandshake = PR_FALSE;
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsNSSSocketInfo::SetForceHandshake(PRBool forceHandshake)
 {
-  mForceHandshake = forceHandshake;
+  (void)forceHandshake;
   return NS_OK;
 }
 
@@ -839,17 +839,6 @@ nsSSLIOLayerConnect(PRFileDesc* fd, const PRNetAddr* addr,
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("[%p] Connect: forceHandshake = %d, forTLSStepUp = %d\n",
                                     (void*)fd, forceHandshake, forTLSStepUp));
 
-  if (!forTLSStepUp && forceHandshake) {
-    // Kick-start the handshake in order to work around bug 66706
-    PRInt32 res = PR_Write(fd,0,0);
-    
-    if (res == -1) {
-      PR_LOG(gPIPNSSLog, PR_LOG_ERROR, ("[%p] ForceHandshake failure -- error %d\n",
-                                        (void*)fd, PR_GetError()));
-      status = PR_FAILURE;
-    }
-  }
-
 #if defined(XP_BEOS)  // bug 70217
  loser:
   // We put the Nonblocking bit back to the value it was when 
@@ -889,6 +878,7 @@ nsSSLIOLayerClose(PRFileDesc *fd)
   popped->identity = PR_INVALID_IO_LAYER;
   nsNSSSocketInfo *infoObject = (nsNSSSocketInfo *)popped->secret;
   NS_RELEASE(infoObject);
+  popped->dtor(popped);
   
   return status;
 }
@@ -996,7 +986,7 @@ nsSSLIOLayerWrite(PRFileDesc* fd, const void* buf, PRInt32 amount)
   socketInfo = (nsNSSSocketInfo*)fd->secret;
   NS_ASSERTION(socketInfo,"nsNSSSocketInfo was null for an fd");
   socketInfo->GetFirstWrite(&firstWrite);
-  PRBool oldBlockVal;
+  PRBool oldBlockVal = PR_FALSE;
   PRBool oldBlockReset = PR_FALSE;
   PRSocketOptionData sockopt;
 
@@ -1760,7 +1750,9 @@ nsresult nsGetUserCertChoice(SSM_UserCertChoice* certChoice)
         *certChoice = ASK;
     }
     else {
-		ret = NS_ERROR_FAILURE;
+      // Most likely we see a nickname from a migrated cert.
+      // We do not currently support that, ask the user which cert to use.
+		  *certChoice = ASK;
 	}
 
 loser:
@@ -2240,9 +2232,14 @@ nsSSLIOLayerAddToSocket(const char* host,
     goto loser;
   }
 
-
   infoObject->SetFileDescPtr(sslSock);
 
+  rv = nsSSLIOLayerSetOptions(sslSock, forTLSStepUp, proxyHost, host, port,
+                              infoObject);
+
+  if (NS_FAILED(rv))
+    goto loser;
+
   /* Now, layer ourselves on top of the SSL socket... */
   layer = PR_CreateIOLayerStub(nsSSLIOLayerIdentity,
                                &nsSSLIOLayerMethods);
@@ -2259,15 +2256,6 @@ nsSSLIOLayerAddToSocket(const char* host,
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("[%p] Socket set up\n", (void*)sslSock));
   infoObject->QueryInterface(NS_GET_IID(nsISupports), (void**) (info));
 
-  /* This is rather confusing, but now, "layer" points to the SSL socket,
-     and that's what we should use for manipulating it. */
-
-  rv = nsSSLIOLayerSetOptions(layer, forTLSStepUp, proxyHost, host, port,
-                              infoObject);
-
-  if (NS_FAILED(rv))
-    goto loser;
-
   // We are going use a clear connection first //
   if (forTLSStepUp || proxyHost) {
     infoObject->SetFirstWrite(PR_FALSE);
@@ -2276,6 +2264,8 @@ nsSSLIOLayerAddToSocket(const char* host,
   return NS_OK;
  loser:
   NS_IF_RELEASE(infoObject);
-  PR_FREEIF(layer);
+  if (layer) {
+    layer->dtor(layer);
+  }
   return NS_ERROR_FAILURE;
 }
