@@ -226,10 +226,11 @@ PK11SlotInfo *SECMOD_LookupSlot(SECMODModuleID moduleID,CK_SLOT_ID slotID) {
 
 
 /*
- * find a module by name and delete it of the module list
+ * find a module by name or module pointer and delete it off the module list.
+ * optionally remove it from secmod.db.
  */
 SECStatus
-SECMOD_DeleteModule(char *name, int *type) {
+SECMOD_DeleteModuleEx(char *name, SECMODModule *mod, int *type, PRBool permdb) {
     SECMODModuleList *mlp;
     SECMODModuleList **mlpp;
     SECStatus rv = SECFailure;
@@ -240,7 +241,8 @@ SECMOD_DeleteModule(char *name, int *type) {
     SECMOD_GetWriteLock(moduleLock);
     for(mlpp = &modules,mlp = modules; 
 				mlp != NULL; mlpp = &mlp->next, mlp = *mlpp) {
-	if (PORT_Strcmp(name,mlp->module->commonName) == 0) {
+	if ((name && (PORT_Strcmp(name,mlp->module->commonName) == 0)) ||
+							mod == mlp->module) {
 	    /* don't delete the internal module */
 	    if (!mlp->module->internal) {
 		SECMOD_RemoveList(mlpp,mlp);
@@ -258,14 +260,24 @@ SECMOD_DeleteModule(char *name, int *type) {
 
 
     if (rv == SECSuccess) {
- 	SECMOD_DeletePermDB(mlp->module);
+	if (permdb) {
+ 	    SECMOD_DeletePermDB(mlp->module);
+	}
 	SECMOD_DestroyModuleListElement(mlp);
     }
     return rv;
 }
 
 /*
- * find a module by name and delete it of the module list
+ * find a module by name and delete it off the module list
+ */
+SECStatus
+SECMOD_DeleteModule(char *name, int *type) {
+    return SECMOD_DeleteModuleEx(name, NULL, type, PR_TRUE);
+}
+
+/*
+ * find a module by name and delete it off the module list
  */
 SECStatus
 SECMOD_DeleteInternalModule(char *name) {
