@@ -35,7 +35,7 @@
  * may use your version of this file under either the MPL or the
  * GPL.
  *
- * $Id: sslsock.c,v 1.24.2.1 2002/03/08 22:19:06 wtc%netscape.com Exp $
+ * $Id: sslsock.c,v 1.27.4.1 2002/04/10 03:29:17 cltbld%netscape.com Exp $
  */
 #include "seccomon.h"
 #include "cert.h"
@@ -1473,7 +1473,7 @@ ssl_WriteV(PRFileDesc *fd, const PRIOVec *iov, PRInt32 vectors,
 #define HANDLE_ERR(rv, len) \
     if (rv != len) { \
 	if (rv < 0) { \
-	    if (blocking \
+	    if (!blocking \
 		&& (PR_GetError() == PR_WOULD_BLOCK_ERROR) \
 		&& (sent > 0)) { \
 		return sent; \
@@ -1482,7 +1482,7 @@ ssl_WriteV(PRFileDesc *fd, const PRIOVec *iov, PRInt32 vectors,
 	    } \
 	} \
 	/* Only a nonblocking socket can have partial sends */ \
-	PR_ASSERT(blocking); \
+	PR_ASSERT(!blocking); \
 	return sent; \
     } 
 #define SEND(bfr, len) \
