@@ -36,14 +36,16 @@
  * the terms of any one of the NPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
-  
+
+#ifndef nsCharTraits_h___
+#include "nsCharTraits.h"
+#endif
+
 #include "nsStrPrivate.h"
 #include "nsStr.h"
 #include "bufferRoutines.h"
 #include <stdio.h>  //only used for printf
-#include "nsCRT.h"
-#include "nsDeque.h"
- 
+
 /******************************************************************************************
   MODULE NOTES:
 
@@ -330,45 +332,6 @@ void nsStrPrivate::StrInsert1into2( nsStr& aDest,PRUint32 aDestOffset,const nsSt
   }
 }
 
-void nsStrPrivate::StrInsert2into1( nsStr& aDest,PRUint32 aDestOffset,const nsStr& aSource,PRUint32 aSrcOffset,PRInt32 aCount){
-  NS_ASSERTION(aSource.GetCharSize() == eTwoByte, "Must be 2 byte");
-  NS_ASSERTION(aDest.GetCharSize() == eOneByte, "Must be 1 byte");
-  //there are a few cases for insert:
-  //  1. You're inserting chars into an empty string (assign)
-  //  2. You're inserting onto the end of a string (append)
-  //  3. You're inserting onto the 1..n-1 pos of a string (the hard case).
-  if(0<aSource.mLength){
-    if(aDest.mLength){
-      if(aDestOffset<aDest.mLength){
-        PRInt32 theLength = GetSegmentLength(aSource, aSrcOffset, aCount);
-
-        if(aSrcOffset<aSource.mLength) {
-            //here's the only new case we have to handle. 
-            //chars are really being inserted into our buffer...
-
-          if(aDest.mLength+theLength > aDest.GetCapacity())
-            AppendForInsert(aDest, aDestOffset, aSource, aSrcOffset, theLength);
-          else {
-            //shift the chars right by theDelta...
-            ShiftCharsRight(aDest.mStr, aDest.mLength, aDestOffset, theLength);
-      
-            //now insert new chars, starting at offset
-            CopyChars2To1(aDest.mStr,aDestOffset,aSource.mStr,aSrcOffset,theLength);
-          }
-
-            //finally, make sure to update the string length...
-          aDest.mLength+=theLength;
-          AddNullTerminator(aDest);
-          NSSTR_SEEN(aDest);
-        }//if
-        //else nothing to do!
-      }
-      else StrAppend(aDest,aSource,0,aCount);
-    }
-    else StrAppend(aDest,aSource,0,aCount);
-  }
-}
-
 void nsStrPrivate::StrInsert2into2( nsStr& aDest,PRUint32 aDestOffset,const nsStr& aSource,PRUint32 aSrcOffset,PRInt32 aCount){
   NS_ASSERTION(aSource.GetCharSize() == eTwoByte, "Must be 1 byte");
   NS_ASSERTION(aDest.GetCharSize() == eTwoByte, "Must be 2 byte");
@@ -646,45 +609,6 @@ PRInt32 nsStrPrivate::FindSubstr1in1(const nsStr& aDest,const nsStr& aTarget, PR
   return kNotFound;
 }
 
-PRInt32 nsStrPrivate::FindSubstr2in1(const nsStr& aDest,const nsStr& aTarget, PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) {
-  
-  NS_ASSERTION(aDest.GetCharSize() == eOneByte, "Must be 1 byte");
-  NS_ASSERTION(aTarget.GetCharSize() == eTwoByte, "Must be 2 byte");
-  
-  PRInt32 theMaxPos = aDest.mLength-aTarget.mLength;  //this is the last pos that is feasible for starting the search, with given lengths...
-
-  if (theMaxPos<0) return kNotFound;
-
-  if(anOffset<0)
-    anOffset=0;
-
-  if((aDest.mLength<=0) || (anOffset>theMaxPos) || (aTarget.mLength==0))
-    return kNotFound;
-  
-  if(aCount<0)
-    aCount = MaxInt(theMaxPos,1);
-
-  if (aCount <= 0)
-    return kNotFound;
-
-  const char* root  = aDest.mStr; 
-  const char* left  = root+anOffset;
-  const char* last  = left+aCount;
-  const char* max   = root+theMaxPos;
-  const char* right = (last<max) ? last : max;
-
-  while(left<=right){
-    PRInt32 cmp=Compare1To2(left,aTarget.mUStr,aTarget.mLength,aIgnoreCase);
-    if(0==cmp) {
-      return (left-root);
-    }
-    left++;
-  } //while
-
-
-  return kNotFound;
-}
-
 PRInt32 nsStrPrivate::FindSubstr1in2(const nsStr& aDest,const nsStr& aTarget, PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) {
   
   NS_ASSERTION(aDest.GetCharSize() == eTwoByte, "Must be 2 byte");
@@ -725,7 +649,7 @@ PRInt32 nsStrPrivate::FindSubstr1in2(const nsStr& aDest,const nsStr& aTarget, PR
   return kNotFound;
 }
 
-PRInt32 nsStrPrivate::FindSubstr2in2(const nsStr& aDest,const nsStr& aTarget, PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) {
+PRInt32 nsStrPrivate::FindSubstr2in2(const nsStr& aDest,const nsStr& aTarget, PRInt32 anOffset,PRInt32 aCount) {
   
   NS_ASSERTION(aDest.GetCharSize() == eTwoByte, "Must be 2 byte");
   NS_ASSERTION(aTarget.GetCharSize() == eTwoByte, "Must be 2 byte");
@@ -753,7 +677,7 @@ PRInt32 nsStrPrivate::FindSubstr2in2(const nsStr& aDest,const nsStr& aTarget, PR
   const PRUnichar* right = (last<max) ? last : max;
   
   while(left<=right){
-    PRInt32 cmp=Compare2To2(left,aTarget.mUStr,aTarget.mLength,aIgnoreCase);
+    PRInt32 cmp=Compare2To2(left,aTarget.mUStr,aTarget.mLength);
     if(0==cmp) {
       return (left-root);
     }
@@ -895,46 +819,6 @@ PRInt32 nsStrPrivate::RFindSubstr1in1(const nsStr& aDest,const nsStr& aTarget,PR
   return kNotFound;
 }
 
-PRInt32 nsStrPrivate::RFindSubstr2in1(const nsStr& aDest,const nsStr& aTarget,PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) {
-
-  NS_ASSERTION(aDest.GetCharSize() == eOneByte, "Must be 1 byte");
-  NS_ASSERTION(aTarget.GetCharSize() == eTwoByte, "Must be 2 byte");
-  
-  if(anOffset<0)
-    anOffset=(PRInt32)aDest.mLength-1;
-
-  if(aCount<0)
-    aCount = aDest.mLength;
-
-  if ((aDest.mLength <= 0) || (PRUint32(anOffset)>=aDest.mLength) || (aTarget.mLength==0))
-    return kNotFound;
-  
-  if (aCount<=0)
-    return kNotFound;
-  
-  const char* root      = aDest.mStr;
-  const char* destLast  = root+aDest.mLength; //pts to last char in aDest (likely null)
-  
-  const char* rightmost = root+anOffset;
-  const char* min       = rightmost-aCount+1;
-  
-  const char* leftmost  = (min<root) ? root: min;
-
-  while(leftmost<=rightmost) {
-    //don't forget to divide by delta in next text (bug found by rhp)...
-    if(aTarget.mLength<=PRUint32(destLast-rightmost)) {
-      PRInt32 result=Compare1To2(rightmost,aTarget.mUStr,aTarget.mLength,aIgnoreCase);
-      
-      if(0==result) {
-        return (rightmost-root);
-      }
-    } //if
-    rightmost--;
-  } //while
-
-  return kNotFound;
-}
-
 PRInt32 nsStrPrivate::RFindSubstr1in2(const nsStr& aDest,const nsStr& aTarget,PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) {
 
   NS_ASSERTION(aDest.GetCharSize() == eTwoByte, "Must be 2 byte");
@@ -975,7 +859,7 @@ PRInt32 nsStrPrivate::RFindSubstr1in2(const nsStr& aDest,const nsStr& aTarget,PR
   return kNotFound;
 }
 
-PRInt32 nsStrPrivate::RFindSubstr2in2(const nsStr& aDest,const nsStr& aTarget,PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) {
+PRInt32 nsStrPrivate::RFindSubstr2in2(const nsStr& aDest,const nsStr& aTarget,PRInt32 anOffset,PRInt32 aCount) {
 
   NS_ASSERTION(aDest.GetCharSize() == eTwoByte, "Must be 2 byte");
   NS_ASSERTION(aTarget.GetCharSize() == eTwoByte, "Must be 2 byte");
@@ -1003,7 +887,7 @@ PRInt32 nsStrPrivate::RFindSubstr2in2(const nsStr& aDest,const nsStr& aTarget,PR
   while(leftmost<=rightmost) {
     //don't forget to divide by delta in next text (bug found by rhp)...
     if(aTarget.mLength<=PRUint32(destLast-rightmost)) {
-      PRInt32 result = Compare2To2(rightmost,aTarget.mUStr,aTarget.mLength,aIgnoreCase);
+      PRInt32 result = Compare2To2(rightmost,aTarget.mUStr,aTarget.mLength);
       
       if(0==result) {
         return (rightmost-root);
@@ -1152,20 +1036,6 @@ PRInt32 nsStrPrivate::StrCompare1To1(const nsStr& aDest,const nsStr& aSource,PRI
   return 0;
 }
 
-PRInt32 nsStrPrivate::StrCompare1To2(const nsStr& aDest,const nsStr& aSource,PRInt32 aCount,PRBool aIgnoreCase) {
-  
-  NS_ASSERTION(aDest.GetCharSize() == eOneByte, "Must be 1 byte");
-  NS_ASSERTION(aSource.GetCharSize() == eTwoByte, "Must be 2 byte");
-  if (aCount) {
-    PRInt32 theCount = GetCompareCount(aDest.mLength, aSource.mLength, aCount);
-    PRInt32 result = Compare1To2(aDest.mStr, aSource.mUStr, theCount, aIgnoreCase);
-    result = TranslateCompareResult(aDest.mLength, aSource.mLength, result, aCount);
-    return result;
-  }
-  
-  return 0;
-}
-
 PRInt32 nsStrPrivate::StrCompare2To1(const nsStr& aDest,const nsStr& aSource,PRInt32 aCount,PRBool aIgnoreCase) {
   NS_ASSERTION(aDest.GetCharSize() == eTwoByte, "Must be 2 byte");
   NS_ASSERTION(aSource.GetCharSize() == eOneByte, "Must be 1 byte");
@@ -1180,13 +1050,13 @@ PRInt32 nsStrPrivate::StrCompare2To1(const nsStr& aDest,const nsStr& aSource,PRI
   return 0;
 }
 
-PRInt32 nsStrPrivate::StrCompare2To2(const nsStr& aDest,const nsStr& aSource,PRInt32 aCount,PRBool aIgnoreCase) {
+PRInt32 nsStrPrivate::StrCompare2To2(const nsStr& aDest,const nsStr& aSource,PRInt32 aCount) {
   NS_ASSERTION(aDest.GetCharSize() == eTwoByte, "Must be 2 byte");
   NS_ASSERTION(aSource.GetCharSize() == eTwoByte, "Must be 2 byte");
   
   if (aCount) {
     PRInt32 theCount = GetCompareCount(aDest.mLength, aSource.mLength, aCount);
-    PRInt32 result = Compare2To2(aDest.mUStr, aSource.mUStr, theCount, aIgnoreCase);
+    PRInt32 result = Compare2To2(aDest.mUStr, aSource.mUStr, theCount);
     result = TranslateCompareResult(aDest.mLength, aSource.mLength, result, aCount);
     return result;
   }
@@ -1279,7 +1149,7 @@ CBufDescriptor::CBufDescriptor(char* aString,PRBool aStackBased,PRUint32 aCapaci
   mLength=mCapacity=0;
   if(aString && aCapacity>1) {
     mCapacity=aCapacity-1;
-    mLength=(-1==aLength) ? strlen(aString) : aLength;
+    mLength=(-1==aLength) ? nsCharTraits<char>::length(aString) : aLength;
     if(mLength>PRInt32(mCapacity))
       mLength=mCapacity;
   }
@@ -1293,7 +1163,7 @@ CBufDescriptor::CBufDescriptor(const char* aString,PRBool aStackBased,PRUint32 a
   mLength=mCapacity=0;
   if(aString && aCapacity>1) {
     mCapacity=aCapacity-1;
-    mLength=(-1==aLength) ? strlen(aString) : aLength;
+    mLength=(-1==aLength) ? nsCharTraits<char>::length(aString) : aLength;
     if(mLength>PRInt32(mCapacity))
       mLength=mCapacity;
   }
@@ -1308,7 +1178,7 @@ CBufDescriptor::CBufDescriptor(PRUnichar* aString,PRBool aStackBased,PRUint32 aC
   mIsConst=PR_FALSE;
   if(aString && aCapacity>1) {
     mCapacity=aCapacity-1;
-    mLength=(-1==aLength) ? nsCRT::strlen(aString) : aLength;
+    mLength=(-1==aLength) ? nsCharTraits<PRUnichar>::length(aString) : aLength;
     if(mLength>PRInt32(mCapacity))
       mLength=mCapacity;
   }
@@ -1322,7 +1192,7 @@ CBufDescriptor::CBufDescriptor(const PRUnichar* aString,PRBool aStackBased,PRUin
   mIsConst=PR_TRUE;
   if(aString && aCapacity>1) {
     mCapacity=aCapacity-1;
-    mLength=(-1==aLength) ? nsCRT::strlen(aString) : aLength;
+    mLength=(-1==aLength) ? nsCharTraits<PRUnichar>::length(aString) : aLength;
     if(mLength>PRInt32(mCapacity))
       mLength=mCapacity;
   }
@@ -1333,10 +1203,26 @@ CBufDescriptor::CBufDescriptor(const PRUnichar* aString,PRBool aStackBased,PRUin
 PRUint32
 nsStrPrivate::HashCode(const nsStr& aDest)
 {
-	if (aDest.GetCharSize() == eTwoByte)
-    return nsCRT::HashCode(aDest.mUStr);
-  else 
-    return nsCRT::HashCode(aDest.mStr);
+  PRUint32 h = 0;
+  if (aDest.GetCharSize() == eTwoByte) {
+    const PRUnichar* s = aDest.mUStr;
+
+    if (!s) return h;
+    
+    PRUnichar c;
+    while ( (c = *s++) )
+      h = (h>>28) ^ (h<<4) ^ c;
+    return h;
+  } else {
+    const char* s = aDest.mStr;
+      
+    if (!s) return h;
+      
+    unsigned char c;
+    while ( (c = *s++) )
+      h = (h>>28) ^ (h<<4) ^ c;
+    return h;
+  }
 }
 
 #ifdef NS_STR_STATS
