@@ -44,13 +44,15 @@
 #include "nsString.h"
 #include "nsReadableUtils.h"
 #include "nsDebug.h"
-#include "nsDeque.h"
+
+#ifndef nsCharTraits_h___
+#include "nsCharTraits.h"
+#endif
 
 #ifndef RICKG_TESTBED
 #include "prdtoa.h"
 #include "nsISizeOfHandler.h"
-#endif 
-
+#endif
 
 static const char* kPossibleNull = "Error: possible unintended null in string";
 static const char* kNullPointerError = "Error: unexpected null ptr";
@@ -171,7 +173,7 @@ void nsString::SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const {
 void nsString::SetLength(PRUint32 anIndex) {
   if ( anIndex > GetCapacity() )
     SetCapacity(anIndex);
-    // |SetCapacity| normally doesn't guarantee the use we are putting it to here (see its interface comment in nsAWritableString.h),
+    // |SetCapacity| normally doesn't guarantee the use we are putting it to here (see its interface comment in nsAString.h),
     //  we can only use it since our local implementation, |nsString::SetCapacity|, is known to do what we want
   nsStrPrivate::StrTruncate(*this,anIndex);
 }
@@ -372,12 +374,12 @@ void
 nsString::ReplaceSubstring(const PRUnichar* aTarget,const PRUnichar* aNewValue){
   if(aTarget && aNewValue) {
 
-    PRInt32 len=nsCRT::strlen(aTarget);
+    PRInt32 len=nsCharTraits<PRUnichar>::length(aTarget);
     if(0<len) {
       CBufDescriptor theDesc1(aTarget,PR_TRUE, len+1,len);
       nsAutoString theTarget(theDesc1);
 
-      len=nsCRT::strlen(aNewValue);
+      len=nsCharTraits<PRUnichar>::length(aNewValue);
       if(0<len) {
         CBufDescriptor theDesc2(aNewValue,PR_TRUE, len+1,len);
         nsAutoString theNewValue(theDesc2);
@@ -410,7 +412,7 @@ nsString::ReplaceSubstring(const nsString& aTarget,const nsString& aNewValue){
     }
     else {
       PRInt32 theIndex=0;
-      while(kNotFound!=(theIndex=nsStrPrivate::FindSubstr2in2(*this,aTarget,PR_FALSE,theIndex,mLength))) {
+      while(kNotFound!=(theIndex=nsStrPrivate::FindSubstr2in2(*this,aTarget,theIndex,mLength))) {
         if(aNewValue.mLength<aTarget.mLength) {
           //Since target is longer than newValue, we should delete a few chars first, then overwrite.
           PRInt32 theDelLen=aTarget.mLength-aNewValue.mLength;
@@ -472,20 +474,15 @@ nsString::Trim(const char* aTrimSet, PRBool aEliminateLeading,PRBool aEliminateT
 }
 
 /**
- *  This method strips chars in given set from string.
- *  You can control whether chars are yanked from
- *  start and end of string as well.
- *  
- *  @update  gess 3/31/98
- *  @param   aEliminateLeading controls stripping of leading ws
- *  @param   aEliminateTrailing controls stripping of trailing ws
- *  @return  this
+ * This method strips chars in given set from string.
  */
-void
-nsString::CompressSet(const char* aSet, PRUnichar aChar,PRBool aEliminateLeading,PRBool aEliminateTrailing){
-  if(aSet){
-    ReplaceChar(aSet,aChar);
-    nsStrPrivate::CompressSet2(*this,aSet,aEliminateLeading,aEliminateTrailing);
+static void
+CompressSet(nsString& aStr, const char* aSet, PRUnichar aChar,
+            PRBool aEliminateLeading, PRBool aEliminateTrailing) {
+  if (aSet) {
+    aStr.ReplaceChar(aSet, aChar);
+    nsStrPrivate::CompressSet2(aStr, aSet,
+                               aEliminateLeading, aEliminateTrailing);
   }
 }
 
@@ -501,7 +498,7 @@ nsString::CompressSet(const char* aSet, PRUnichar aChar,PRBool aEliminateLeading
  */
 void
 nsString::CompressWhitespace( PRBool aEliminateLeading,PRBool aEliminateTrailing){
-  CompressSet(kWhitespace,' ',aEliminateLeading,aEliminateTrailing);
+  CompressSet(*this, kWhitespace,' ', aEliminateLeading, aEliminateTrailing);
 }
 
 /**********************************************************************
@@ -695,18 +692,6 @@ void nsString::AssignWithConversion(const char* aCString) {
 }
 
 
-/**
- * assign given char to this string
- * @update  gess 01/04/99
- * @param   aChar: char to be assignd to this
- * @return  this
- */
-void nsString::AssignWithConversion(char aChar) {
-  nsStrPrivate::StrTruncate(*this,0);
-  AppendWithConversion(aChar);
-}
-
-
 /**
  * append given c-string to this string
  * @update  gess 01/04/99
@@ -720,7 +705,7 @@ void nsString::AppendWithConversion(const char* aCString,PRInt32 aCount) {
   if(aCString && aCount){  //if astring is null or count==0 there's nothing to do
     nsStr temp;
     nsStrPrivate::Initialize(temp,eOneByte);
-    temp.mStr=(char*)aCString;
+    temp.mStr = NS_CONST_CAST(char*, aCString);
 
     if(0<aCount) {
       temp.mLength=aCount;
@@ -736,30 +721,13 @@ void nsString::AppendWithConversion(const char* aCString,PRInt32 aCount) {
 #endif
 
     }
-    else aCount=temp.mLength=strlen(aCString);
+    else aCount=temp.mLength=nsCharTraits<char>::length(aCString);
       
     if(0<aCount)
       nsStrPrivate::StrAppend(*this,temp,0,aCount);
   }
 }
 
-/**
- * append given char to this string
- * @update  gess 01/04/99
- * @param   aChar: char to be appended 
- * @return  this
- */
-void nsString::AppendWithConversion(char aChar) {
-  char buf[2]={0,0};
-  buf[0]=aChar;
-
-  nsStr temp;
-  nsStrPrivate::Initialize(temp,eOneByte);
-  temp.mStr=buf;
-  temp.mLength=1;
-  nsStrPrivate::StrAppend(*this,temp,0,1);
-}
-
 /**
  * Append the given integer to this string 
  * @update  gess 01/04/99
@@ -830,7 +798,7 @@ void nsString::InsertWithConversion(const char* aCString,PRUint32 anOffset,PRInt
   if(aCString && aCount){
     nsStr temp;
     nsStrPrivate::Initialize(temp,eOneByte);
-    temp.mStr=(char*)aCString;
+    temp.mStr = NS_CONST_CAST(char*, aCString);
 
     if(0<aCount) {
       temp.mLength=aCount;
@@ -845,7 +813,7 @@ void nsString::InsertWithConversion(const char* aCString,PRUint32 anOffset,PRInt
 #endif
 
     }
-    else aCount=temp.mLength=strlen(aCString);
+    else aCount=temp.mLength=nsCharTraits<char>::length(aCString);
 
     if(0<aCount){
       nsStrPrivate::StrInsert1into2(*this,anOffset,temp,0,aCount);
@@ -863,6 +831,18 @@ void nsString::Adopt(PRUnichar* aPtr, PRInt32 aLength) {
   nsStrPrivate::Initialize(*this, (char*)aPtr, aLength, aLength, eTwoByte, PR_TRUE);
 }
 
+nsAString::size_type
+nsString::Mid( self_type& aResult, index_type aStartPos, size_type aLengthToCopy ) const
+  {
+      // If we're just assigning our entire self, give |aResult| the opportunity to share
+    if ( aStartPos == 0 && aLengthToCopy >= Length() )
+      aResult = *this;
+    else
+      aResult = Substring(*this, aStartPos, aLengthToCopy);
+
+    return aResult.Length();
+  }
+
 
 /**********************************************************************
   Searching methods...                
@@ -885,8 +865,8 @@ PRInt32 nsString::Find(const char* aCString,PRBool aIgnoreCase,PRInt32 anOffset,
   if(aCString) {
     nsStr temp;
     nsStrPrivate::Initialize(temp,eOneByte);
-    temp.mLength=strlen(aCString);
-    temp.mStr=(char*)aCString;
+    temp.mLength=nsCharTraits<char>::length(aCString);
+    temp.mStr = NS_CONST_CAST(char*, aCString);
     result=nsStrPrivate::FindSubstr1in2(*this,temp,aIgnoreCase,anOffset,aCount);
   }
   return result;
@@ -902,35 +882,20 @@ PRInt32 nsString::Find(const char* aCString,PRBool aIgnoreCase,PRInt32 anOffset,
  *  @param   aCount tells us how many iterations to make starting at the given offset
  *  @return  offset in string, or -1 (kNotFound)
  */
-PRInt32 nsString::Find(const PRUnichar* aString,PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) const{
+PRInt32 nsString::Find(const PRUnichar* aString,PRInt32 anOffset,PRInt32 aCount) const{
   NS_ASSERTION(0!=aString,kNullPointerError);
 
   PRInt32 result=kNotFound;
   if(aString) {
     nsStr temp;
     nsStrPrivate::Initialize(temp,eTwoByte);
-    temp.mLength=nsCRT::strlen(aString);
-    temp.mUStr=(PRUnichar*)aString;
-    result=nsStrPrivate::FindSubstr2in2(*this,temp,aIgnoreCase,anOffset,aCount);
+    temp.mLength = nsCharTraits<PRUnichar>::length(aString);
+    temp.mUStr = NS_CONST_CAST(PRUnichar*, aString);
+    result=nsStrPrivate::FindSubstr2in2(*this,temp,anOffset,aCount);
   }
   return result;
 }
 
-/**
- *  Search for given string within this string
- *  
- *  @update  gess 3/25/98
- *  @param   aString - substr to be found
- *  @param   aIgnoreCase tells us whether or not to do caseless compare
- *  @param   anOffset tells us where in this string to start searching
- *  @param   aCount tells us how many iterations to make starting at the given offset
- *  @return  offset in string, or -1 (kNotFound)
- */
-PRInt32 nsString::Find(const nsCString& aString,PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) const{
-  PRInt32 result=nsStrPrivate::FindSubstr1in2(*this,aString,aIgnoreCase,anOffset,aCount);
-  return result;
-}
-
 /**
  *  search for given string within this string
  *  
@@ -941,8 +906,14 @@ PRInt32 nsString::Find(const nsCString& aString,PRBool aIgnoreCase,PRInt32 anOff
  *  @param   aCount tells us how many iterations to make starting at the given offset
  *  @return  offset in string, or -1 (kNotFound)
  */
-PRInt32 nsString::Find(const nsString& aString,PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) const{
-  PRInt32 result=nsStrPrivate::FindSubstr2in2(*this,aString,aIgnoreCase,anOffset,aCount);
+PRInt32 nsString::Find(const nsAFlatString& aString,PRInt32 anOffset,PRInt32 aCount) const{
+
+  nsStr temp;
+  nsStrPrivate::Initialize(temp, eTwoByte);
+  temp.mLength = aString.Length();
+  temp.mUStr = NS_CONST_CAST(PRUnichar*, aString.get());
+  
+  PRInt32 result=nsStrPrivate::FindSubstr2in2(*this,temp,anOffset,aCount);
   return result;
 }
 
@@ -962,8 +933,8 @@ PRInt32 nsString::FindCharInSet(const char* aCStringSet,PRInt32 anOffset) const{
   if(aCStringSet) {
     nsStr temp;
     nsStrPrivate::Initialize(temp,eOneByte);
-    temp.mLength=strlen(aCStringSet);
-    temp.mStr=(char*)aCStringSet;
+    temp.mLength=nsCharTraits<char>::length(aCStringSet);
+    temp.mStr = NS_CONST_CAST(char*, aCStringSet);
     result=nsStrPrivate::FindCharInSet1(*this,temp,PR_FALSE,anOffset);
   }
   return result;
@@ -985,33 +956,13 @@ PRInt32 nsString::FindCharInSet(const PRUnichar* aStringSet,PRInt32 anOffset) co
   if(aStringSet) {
     nsStr temp;
     nsStrPrivate::Initialize(temp,eTwoByte);
-    temp.mLength=nsCRT::strlen(aStringSet);
-    temp.mStr=(char*)aStringSet;
+    temp.mLength=nsCharTraits<PRUnichar>::length(aStringSet);
+    temp.mUStr=NS_CONST_CAST(PRUnichar*, aStringSet);
     result=nsStrPrivate::FindCharInSet2(*this,temp,anOffset);
   }
   return result;
 }
 
-/**
- *  This method finds the offset of the first char in this string that is
- *  a member of the given charset, starting the search at anOffset
- *  
- *  @update  gess 3/25/98
- *  @param   aCStringSet
- *  @param   anOffset -- where in this string to start searching
- *  @return  
- */
-PRInt32 nsString::FindCharInSet(const nsString& aSet,PRInt32 anOffset) const{
-  PRInt32 result=nsStrPrivate::FindCharInSet2(*this,aSet,anOffset);
-  return result;
-}
-
-PRInt32 nsString::FindCharInSet(const nsCString& aSet,PRInt32 anOffset) const{
-  PRInt32 result=nsStrPrivate::FindCharInSet1(*this,aSet,PR_FALSE,anOffset);
-  return result;
-}
-
-
 /**
  *  Reverse search for given string within this string
  *  
@@ -1022,23 +973,26 @@ PRInt32 nsString::FindCharInSet(const nsCString& aSet,PRInt32 anOffset) const{
  *  @param   aCount tells us how many iterations to make starting at the given offset
  *  @return  offset in string, or -1 (kNotFound)
  */
-PRInt32 nsString::RFind(const nsCString& aString,PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) const{
-  PRInt32 result=nsStrPrivate::RFindSubstr1in2(*this,aString,aIgnoreCase,anOffset,aCount);
+PRInt32 nsString::RFind(const nsAFlatString& aString,PRInt32 anOffset,PRInt32 aCount) const
+{
+  nsStr temp;
+  nsStrPrivate::Initialize(temp, eTwoByte);
+  temp.mLength = aString.Length();
+  temp.mUStr = NS_CONST_CAST(PRUnichar*, aString.get());
+  PRInt32 result=nsStrPrivate::RFindSubstr2in2(*this,temp,anOffset,aCount);
   return result;
 }
 
-/**
- *  Reverse search for given string within this string
- *  
- *  @update  gess 3/25/98
- *  @param   aString - substr to be found
- *  @param   aIgnoreCase tells us whether or not to do caseless compare
- *  @param   anOffset tells us where in this string to start searching
- *  @param   aCount tells us how many iterations to make starting at the given offset
- *  @return  offset in string, or -1 (kNotFound)
- */
-PRInt32 nsString::RFind(const nsString& aString,PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) const{
-  PRInt32 result=nsStrPrivate::RFindSubstr2in2(*this,aString,aIgnoreCase,anOffset,aCount);
+PRInt32 nsString::RFind(const PRUnichar* aString, PRInt32 anOffset, PRInt32 aCount) const
+{
+  PRInt32 result=kNotFound;
+  if (aString) {
+    nsStr temp;
+    nsStrPrivate::Initialize(temp, eTwoByte);
+    temp.mLength = nsCharTraits<PRUnichar>::length(aString);
+    temp.mUStr = NS_CONST_CAST(PRUnichar*, aString);
+    result=nsStrPrivate::RFindSubstr2in2(*this,temp,anOffset,aCount);
+  }
   return result;
 }
 
@@ -1059,8 +1013,8 @@ PRInt32 nsString::RFind(const char* aString,PRBool aIgnoreCase,PRInt32 anOffset,
   if(aString) {
     nsStr temp;
     nsStrPrivate::Initialize(temp,eOneByte);
-    temp.mLength=strlen(aString);
-    temp.mStr=(char*)aString;
+    temp.mLength=nsCharTraits<char>::length(aString);
+    temp.mStr = NS_CONST_CAST(char*, aString);
     result=nsStrPrivate::RFindSubstr1in2(*this,temp,aIgnoreCase,anOffset,aCount);
   }
   return result;
@@ -1080,48 +1034,6 @@ PRInt32 nsString::RFindChar(PRUnichar aChar,PRInt32 anOffset,PRInt32 aCount) con
   return result;
 }
 
-/**
- *  Reverse search for char in this string that is also a member of given charset
- *  
- *  @update  gess 3/25/98
- *  @param   aCStringSet
- *  @param   anOffset
- *  @return  offset of found char, or -1 (kNotFound)
- */
-PRInt32 nsString::RFindCharInSet(const char* aCStringSet,PRInt32 anOffset) const{
-  NS_ASSERTION(0!=aCStringSet,kNullPointerError);
-
-  PRInt32 result=kNotFound;
-  if(aCStringSet) {
-    nsStr temp;
-    nsStrPrivate::Initialize(temp,eOneByte);
-    temp.mLength=strlen(aCStringSet);
-    temp.mStr=(char*)aCStringSet;
-    result=nsStrPrivate::RFindCharInSet1(*this,temp,PR_FALSE,anOffset);
-  }
-  return result;
-}
-
-/**
- *  Reverse search for a first char in this string that is a
- *  member of the given char set
- *  
- *  @update  gess 3/25/98
- *  @param   aSet
- *  @param   aIgnoreCase
- *  @param   anOffset
- *  @return  offset of found char, or -1 (kNotFound)
- */
-PRInt32 nsString::RFindCharInSet(const nsString& aSet,PRInt32 anOffset) const{
-  PRInt32 result=nsStrPrivate::RFindCharInSet2(*this,aSet,anOffset);
-  return result;
-}
-
-PRInt32 nsString::RFindCharInSet(const nsCString& aSet,PRInt32 anOffset) const{
-  PRInt32 result=nsStrPrivate::RFindCharInSet1(*this,aSet,PR_FALSE,anOffset);
-  return result;
-}
-
 /**
  *  Reverse search for a first char in this string that is a
  *  member of the given char set
@@ -1139,8 +1051,8 @@ PRInt32 nsString::RFindCharInSet(const PRUnichar* aStringSet,PRInt32 anOffset) c
   if(aStringSet) {
     nsStr temp;
     nsStrPrivate::Initialize(temp,eTwoByte);
-    temp.mLength=nsCRT::strlen(aStringSet);
-    temp.mUStr=(PRUnichar*)aStringSet;
+    temp.mLength = nsCharTraits<PRUnichar>::length(aStringSet);
+    temp.mUStr = NS_CONST_CAST(PRUnichar*, aStringSet);
     result=nsStrPrivate::RFindCharInSet2(*this,temp,anOffset);
   }
   return result;
@@ -1166,75 +1078,19 @@ PRInt32 nsString::CompareWithConversion(const char *aCString,PRBool aIgnoreCase,
     nsStr temp;
     nsStrPrivate::Initialize(temp,eOneByte);
 
-    temp.mLength= (0<aCount) ? aCount : strlen(aCString);
+    temp.mLength= (0<aCount) ? aCount : nsCharTraits<char>::length(aCString);
 
-    temp.mStr=(char*)aCString;
+    temp.mStr = NS_CONST_CAST(char*, aCString);
     return nsStrPrivate::StrCompare2To1(*this,temp,aCount,aIgnoreCase);
   }
 
   return 0;
 }
 
-/**
- * Compares given cstring to this string. 
- * @update  gess 01/04/99
- * @param   aCString pts to a cstring
- * @param   aIgnoreCase tells us how to treat case
- * @param   aCount tells us how many chars to test; -1 implies full length
- * @return  -1,0,1
- */
-PRInt32 nsString::CompareWithConversion(const nsString& aString,PRBool aIgnoreCase,PRInt32 aCount) const {
-  PRInt32 result=nsStrPrivate::StrCompare2To2(*this,aString,aCount,aIgnoreCase);
-  return result;
-}
-
-/**
- * Compares given unistring to this string. 
- * @update  gess 01/04/99
- * @param   aString pts to a uni-string
- * @param   aIgnoreCase tells us how to treat case
- * @param   aCount tells us how many chars to test; -1 implies full length
- * @return  -1,0,1
- */
-PRInt32 nsString::CompareWithConversion(const PRUnichar* aString,PRBool aIgnoreCase,PRInt32 aCount) const {
-  NS_ASSERTION(0!=aString,kNullPointerError);
-
-  if(aString) {
-    nsStr temp;
-    nsStrPrivate::Initialize(temp,eTwoByte);
-    temp.mLength=nsCRT::strlen(aString);
-    temp.mUStr=(PRUnichar*)aString;
-    return nsStrPrivate::StrCompare2To2(*this,temp,aCount,aIgnoreCase);
-  }
-
-   return 0;
-}
-
-
-
-PRBool nsString::EqualsIgnoreCase(const nsString& aString) const {
-  return EqualsWithConversion(aString,PR_TRUE);
-}
-
 PRBool nsString::EqualsIgnoreCase(const char* aString,PRInt32 aLength) const {
   return EqualsWithConversion(aString,PR_TRUE,aLength);
 }
 
-/**
- * Compare this to given string; note that we compare full strings here.
- * 
- * @update gess 01/04/99
- * @param  aString is the other nsString to be compared to
- * @param   aCount tells us how many chars to test; -1 implies full length
- * @return TRUE if equal
- */
-PRBool nsString::EqualsWithConversion(const nsString& aString,PRBool aIgnoreCase,PRInt32 aCount) const {
-  PRInt32 theAnswer=nsStrPrivate::StrCompare2To2(*this,aString,aCount,aIgnoreCase);
-  PRBool  result=PRBool(0==theAnswer);
-  return result;
-
-}
-
 /**
  * Compare this to given c-string; note that we compare full strings here.
  *
@@ -1249,21 +1105,7 @@ PRBool nsString::EqualsWithConversion(const char* aString,PRBool aIgnoreCase,PRI
   return result;
 }
 
-/**
- * Compare this to given unicode string; note that we compare full strings here.
- *
- * @param  aString is the U-String to be compared 
- * @param  aIgnorecase tells us whether to be case sensitive
- * @param   aCount tells us how many chars to test; -1 implies full length
- * @return TRUE if equal
- */
-PRBool nsString::EqualsWithConversion(const PRUnichar* aString,PRBool aIgnoreCase,PRInt32 aCount) const {
-  PRInt32 theAnswer=CompareWithConversion(aString,aIgnoreCase,aCount);
-  PRBool  result=PRBool(0==theAnswer);
-  return result;
-}
-
-PRInt32 Compare2To2(const PRUnichar* aStr1,const PRUnichar* aStr2,PRUint32 aCount,PRBool aIgnoreCase);
+PRInt32 Compare2To2(const PRUnichar* aStr1,const PRUnichar* aStr2,PRUint32 aCount);
 /**
  * Compare this to given atom; note that we compare full strings here.
  * The optional length argument just lets us know how long the given string is.
@@ -1275,7 +1117,7 @@ PRInt32 Compare2To2(const PRUnichar* aStr1,const PRUnichar* aStr2,PRUint32 aCoun
  * @param  aLength -- length of given string.
  * @return TRUE if equal
  */
-PRBool nsString::EqualsAtom(/*FIX: const */nsIAtom* aAtom,PRBool aIgnoreCase) const{
+PRBool nsString::EqualsAtom(nsIAtom* aAtom) const{
   NS_ASSERTION(0!=aAtom,kNullPointerError);
   PRBool result=PR_FALSE;
   if(aAtom){
@@ -1283,33 +1125,13 @@ PRBool nsString::EqualsAtom(/*FIX: const */nsIAtom* aAtom,PRBool aIgnoreCase) co
     const PRUnichar* unicode;
     if (aAtom->GetUnicode(&unicode) != NS_OK || unicode == nsnull)
         return PR_FALSE;
-    cmp=Compare2To2(mUStr,unicode, nsCRT::strlen(mUStr), aIgnoreCase);
+    cmp=Compare2To2(mUStr,unicode, nsCharTraits<PRUnichar>::length(mUStr));
     result=PRBool(0==cmp);
   }
 
    return result;
 }
 
-PRBool nsString::EqualsIgnoreCase(/*FIX: const */nsIAtom *aAtom) const {
-  return EqualsAtom(aAtom,PR_TRUE);
-}
-
-
-/**
- *  Determine if given char in valid alpha range
- *  
- *  @update  gess 3/31/98
- *  @param   aChar is character to be tested
- *  @return  TRUE if in alpha range
- */
-PRBool nsString::IsAlpha(PRUnichar aChar) {
-  // XXX i18n
-  if (((aChar >= 'A') && (aChar <= 'Z')) || ((aChar >= 'a') && (aChar <= 'z'))) {
-    return PR_TRUE;
-  }
-  return PR_FALSE;
-}
-
 /**
  *  Determine if given char is a valid space character
  *  
@@ -1359,19 +1181,6 @@ PRBool nsString::IsASCII(const PRUnichar* aBuffer) {
 }
 
 
-/**
- *  Determine if given char is valid digit
- *  
- *  @update  gess 3/31/98
- *  @param   aChar is character to be tested
- *  @return  TRUE if char is a valid digit
- */
-PRBool nsString::IsDigit(PRUnichar aChar) {
-  // XXX i18n
-  return PRBool((aChar >= '0') && (aChar <= '9'));
-}
-
-
 /***********************************************************************
   IMPLEMENTATION NOTES: AUTOSTRING...
  ***********************************************************************/
@@ -1523,22 +1332,20 @@ NS_ConvertUTF8toUCS2::Init( const nsACString& aCString )
   PRUint32 count = calculator.Length();
 
   if (count) {
-
     // Grow the buffer if we need to.
-    SetCapacity(count);
-      // |SetCapacity| normally doesn't guarantee the use we are putting it to here (see its interface comment in nsAWritableString.h),
-      //  we can only use it since our local implementation, |nsString::SetCapacity|, is known to do what we want
+    SetLength(count);
 
     // All ready? Time to convert
 
     ConvertUTF8toUCS2 converter(mUStr);
     copy_string(aCString.BeginReading(start), aCString.EndReading(end), converter);
     mLength = converter.Length();
-    if (GetCapacity())
-      mUStr[mLength] = '\0'; // null terminate
+    if (mLength != count) {
+      NS_ERROR("Input wasn't UTF8 or incorrect length was calculated");
+      Truncate();
+    }
   }
 
-  NS_ASSERTION(count == mLength, "calculator calculated incorrect length");
 }
 
 /**
