@@ -106,11 +106,7 @@ nsAString::GetBufferHandle() const
 PRUint32
 nsAString::GetImplementationFlags() const
   {
-    PRUint32 flags = 0;
-    const shared_buffer_handle_type* handle = GetSharedBufferHandle();
-    if ( handle )
-      flags = handle->GetImplementationFlags();
-    return flags;
+    return 0;
   }
 
 
@@ -183,26 +179,6 @@ nsAString::CountChar( char_type c ) const
     return 0;
   }
 
-nsAString::size_type
-nsAString::Mid( self_type& aResult, index_type aStartPos, size_type aLengthToCopy ) const
-  {
-      // If we're just assigning our entire self, give |aResult| the opportunity to share
-    if ( aStartPos == 0 && aLengthToCopy >= Length() )
-      aResult = *this;
-    else
-      aResult = Substring(*this, aStartPos, aLengthToCopy);
-
-    return aResult.Length();
-  }
-
-nsAString::size_type
-nsAString::Right( self_type& aResult, size_type aLengthToCopy ) const
-  {
-    size_type myLength = Length();
-    aLengthToCopy = NS_MIN(myLength, aLengthToCopy);
-    return Mid(aResult, myLength-aLengthToCopy, aLengthToCopy);
-  }
-
 PRInt32
 nsAString::FindChar( char_type aChar, PRUint32 aOffset ) const
   {
@@ -225,23 +201,42 @@ nsAString::FindChar( char_type aChar, PRUint32 aOffset ) const
     return -1;
   }
 
+PRBool
+nsAString::IsDependentOn( const self_type& aString ) const
+  {
+    const_fragment_type f1;
+    const char_type* s1 = GetReadableFragment(f1, kFirstFragment);
+    while ( s1 )
+      {
+        const_fragment_type f2;
+        const char_type* s2 = aString.GetReadableFragment(f2, kFirstFragment);
+        while ( s2 )
+          {
+              // if it _isn't_ the case that
+              //  one fragment starts after the other ends,
+              //  or ends before the other starts,
+              // then, they conflict:
+              // !(f2.mStart>=f1.mEnd || f2.mEnd<=f1.mStart)
+              //
+              // Simplified, that gives us:
+            if ( f2.mStart < f1.mEnd && f2.mEnd > f1.mStart )
+              return PR_TRUE;
+
+            s2 = aString.GetReadableFragment(f2, kNextFragment);
+          }
+        s1 = GetReadableFragment(f1, kNextFragment);
+      }
+    return PR_FALSE;
+  }
 
   //
   // |Assign()|
   //
 
 void
-nsAString::AssignFromReadable( const self_type& rhs )
-  {
-    if ( this != &rhs )
-      do_AssignFromReadable(rhs);
-    // else, self-assign is a no-op
-  }
-
-void
-nsAString::AssignFromPromise( const self_type& aReadable )
+nsAString::do_AssignFromReadable( const self_type& aReadable )
     /*
-      ...this function is only called when a promise that somehow references |this| is assigned _into_ |this|.
+      ...we need to check whether the string that's being assigned into |this| somehow references |this|.
       E.g.,
       
         ... writable& w ...
@@ -261,8 +256,12 @@ nsAString::AssignFromPromise( const self_type& aReadable )
       option to fix either the callers or this mechanism.
     */
   {
+      // self-assign is a no-op
+    if ( this == &aReadable)
+      return;
+
     if ( !aReadable.IsDependentOn(*this) )
-      do_AssignFromReadable(aReadable);
+      UncheckedAssignFromReadable(aReadable);
     else
       {
         size_type length = aReadable.Length();
@@ -274,15 +273,15 @@ nsAString::AssignFromPromise( const self_type& aReadable )
             const_iterator fromBegin, fromEnd;
             char_type* toBegin = buffer;
             copy_string(aReadable.BeginReading(fromBegin), aReadable.EndReading(fromEnd), toBegin);
-            do_AssignFromElementPtrLength(buffer, length);
-            delete buffer;
+            UncheckedAssignFromReadable(Substring(buffer, buffer + length));
+            delete[] buffer;
           }
         // else assert?
       }
   }
 
 void
-nsAString::do_AssignFromReadable( const self_type& aReadable )
+nsAString::UncheckedAssignFromReadable( const self_type& aReadable )
   {
     SetLength(0);
     if ( !aReadable.IsEmpty() )
@@ -312,7 +311,7 @@ nsAString::do_AssignFromElementPtrLength( const char_type* aPtr, size_type aLeng
 void
 nsAString::do_AssignFromElement( char_type aChar )
   {
-    do_AssignFromReadable(Substring(&aChar, &aChar+1));
+    UncheckedAssignFromReadable(Substring(&aChar, &aChar+1));
   }
 
 
@@ -322,19 +321,10 @@ nsAString::do_AssignFromElement( char_type aChar )
   //
 
 void
-nsAString::AppendFromReadable( const self_type& aReadable )
-  {
-    if ( this != &aReadable )
-      do_AppendFromReadable(aReadable);
-    else
-      AppendFromPromise(aReadable);
-  }
-
-void
-nsAString::AppendFromPromise( const self_type& aReadable )
+nsAString::do_AppendFromReadable( const self_type& aReadable )
   {
     if ( !aReadable.IsDependentOn(*this) )
-      do_AppendFromReadable(aReadable);
+      UncheckedAppendFromReadable(aReadable);
     else
       {
         size_type length = aReadable.Length();
@@ -344,15 +334,15 @@ nsAString::AppendFromPromise( const self_type& aReadable )
             const_iterator fromBegin, fromEnd;
             char_type* toBegin = buffer;
             copy_string(aReadable.BeginReading(fromBegin), aReadable.EndReading(fromEnd), toBegin);
-            do_AppendFromElementPtrLength(buffer, length);
-            delete buffer;
+            UncheckedAppendFromReadable(Substring(buffer, buffer + length));
+            delete[] buffer;
           }
         // else assert?
       }
   }
 
 void
-nsAString::do_AppendFromReadable( const self_type& aReadable )
+nsAString::UncheckedAppendFromReadable( const self_type& aReadable)
   {
     size_type oldLength = this->Length();
     SetLength(oldLength + aReadable.Length());
@@ -362,6 +352,7 @@ nsAString::do_AppendFromReadable( const self_type& aReadable )
     copy_string(aReadable.BeginReading(fromBegin), aReadable.EndReading(fromEnd), BeginWriting(toBegin).advance( PRInt32(oldLength) ) );
   }
 
+
 void
 nsAString::do_AppendFromElementPtr( const char_type* aPtr )
   {
@@ -377,7 +368,7 @@ nsAString::do_AppendFromElementPtrLength( const char_type* aPtr, size_type aLeng
 void
 nsAString::do_AppendFromElement( char_type aChar )
   {
-    do_AppendFromReadable(Substring(&aChar, &aChar + 1));
+    UncheckedAppendFromReadable(Substring(&aChar, &aChar + 1));
   }
 
 
@@ -387,19 +378,10 @@ nsAString::do_AppendFromElement( char_type aChar )
   //
 
 void
-nsAString::InsertFromReadable( const self_type& aReadable, index_type atPosition )
-  {
-    if ( this != &aReadable )
-      do_InsertFromReadable(aReadable, atPosition);
-    else
-      InsertFromPromise(aReadable, atPosition);
-  }
-
-void
-nsAString::InsertFromPromise( const self_type& aReadable, index_type atPosition )
+nsAString::do_InsertFromReadable( const self_type& aReadable, index_type atPosition )
   {
     if ( !aReadable.IsDependentOn(*this) )
-      do_InsertFromReadable(aReadable, atPosition);
+      UncheckedInsertFromReadable(aReadable, atPosition);
     else
       {
         size_type length = aReadable.Length();
@@ -409,15 +391,15 @@ nsAString::InsertFromPromise( const self_type& aReadable, index_type atPosition
             const_iterator fromBegin, fromEnd;
             char_type* toBegin = buffer;
             copy_string(aReadable.BeginReading(fromBegin), aReadable.EndReading(fromEnd), toBegin);
-            do_InsertFromElementPtrLength(buffer, atPosition, length);
-            delete buffer;
+            UncheckedInsertFromReadable(Substring(buffer, buffer + length), atPosition);
+            delete[] buffer;
           }
         // else assert
       }
   }
 
 void
-nsAString::do_InsertFromReadable( const self_type& aReadable, index_type atPosition )
+nsAString::UncheckedInsertFromReadable( const self_type& aReadable, index_type atPosition )
   {
     size_type oldLength = this->Length();
     SetLength(oldLength + aReadable.Length());
@@ -446,7 +428,7 @@ nsAString::do_InsertFromElementPtrLength( const char_type* aPtr, index_type atPo
 void
 nsAString::do_InsertFromElement( char_type aChar, index_type atPosition )
   {
-    do_InsertFromReadable(Substring(&aChar, &aChar+1), atPosition);
+    UncheckedInsertFromReadable(Substring(&aChar, &aChar+1), atPosition);
   }
 
 
@@ -476,19 +458,10 @@ nsAString::Cut( index_type cutStart, size_type cutLength )
   //
 
 void
-nsAString::ReplaceFromReadable( index_type cutStart, size_type cutLength, const self_type& aReplacement )
-  {
-    if ( this != &aReplacement )
-      do_ReplaceFromReadable(cutStart, cutLength, aReplacement);
-    else
-      ReplaceFromPromise(cutStart, cutLength, aReplacement);
-  }
-
-void
-nsAString::ReplaceFromPromise( index_type cutStart, size_type cutLength, const self_type& aReadable )
+nsAString::do_ReplaceFromReadable( index_type cutStart, size_type cutLength, const self_type& aReadable )
   {
     if ( !aReadable.IsDependentOn(*this) )
-      do_ReplaceFromReadable(cutStart, cutLength, aReadable);
+      UncheckedReplaceFromReadable(cutStart, cutLength, aReadable);
     else
       {
         size_type length = aReadable.Length();
@@ -498,15 +471,15 @@ nsAString::ReplaceFromPromise( index_type cutStart, size_type cutLength, const s
             const_iterator fromBegin, fromEnd;
             char_type* toBegin = buffer;
             copy_string(aReadable.BeginReading(fromBegin), aReadable.EndReading(fromEnd), toBegin);
-            do_ReplaceFromReadable(cutStart, cutLength, nsDependentString(buffer, length));
-            delete buffer;
+            UncheckedReplaceFromReadable(cutStart, cutLength, Substring(buffer, buffer + length));
+            delete[] buffer;
           }
         // else assert?
       }
   }
 
 void
-nsAString::do_ReplaceFromReadable( index_type cutStart, size_type cutLength, const self_type& aReplacement )
+nsAString::UncheckedReplaceFromReadable( index_type cutStart, size_type cutLength, const self_type& aReplacement )
   {
     size_type oldLength = this->Length();
 
@@ -531,6 +504,7 @@ nsAString::do_ReplaceFromReadable( index_type cutStart, size_type cutLength, con
   }
 
 
+
 int
 nsDefaultCStringComparator::operator()( const char_type* lhs, const char_type* rhs, PRUint32 aLength ) const
   {
@@ -554,8 +528,8 @@ nsCaseInsensitiveCStringComparator::operator()( char lhs, char rhs ) const
   {
     if (lhs == rhs) return 0;
     
-    lhs = nsCRT::ToLower(lhs);
-    rhs = nsCRT::ToLower(rhs);
+    lhs = tolower(lhs);
+    rhs = tolower(rhs);
 
     return lhs - rhs;
   }
@@ -627,11 +601,7 @@ nsACString::GetBufferHandle() const
 PRUint32
 nsACString::GetImplementationFlags() const
   {
-    PRUint32 flags = 0;
-    const shared_buffer_handle_type* handle = GetSharedBufferHandle();
-    if ( handle )
-      flags = handle->GetImplementationFlags();
-    return flags;
+    return 0;
   }
 
 
@@ -702,26 +672,6 @@ nsACString::CountChar( char_type c ) const
     return 0;
   }
 
-nsACString::size_type
-nsACString::Mid( self_type& aResult, index_type aStartPos, size_type aLengthToCopy ) const
-  {
-      // If we're just assigning our entire self, give |aResult| the opportunity to share
-    if ( aStartPos == 0 && aLengthToCopy >= Length() )
-      aResult = *this;
-    else
-      aResult = Substring(*this, aStartPos, aLengthToCopy);
-
-    return aResult.Length();
-  }
-
-nsACString::size_type
-nsACString::Right( self_type& aResult, size_type aLengthToCopy ) const
-  {
-    size_type myLength = Length();
-    aLengthToCopy = NS_MIN(myLength, aLengthToCopy);
-    return Mid(aResult, myLength-aLengthToCopy, aLengthToCopy);
-  }
-
 PRInt32
 nsACString::FindChar( char_type aChar, PRUint32 aOffset ) const
   {
@@ -744,23 +694,42 @@ nsACString::FindChar( char_type aChar, PRUint32 aOffset ) const
     return -1;
   }
 
+PRBool
+nsACString::IsDependentOn( const self_type& aString ) const
+  {
+    const_fragment_type f1;
+    const char_type* s1 = GetReadableFragment(f1, kFirstFragment);
+    while ( s1 )
+      {
+        const_fragment_type f2;
+        const char_type* s2 = aString.GetReadableFragment(f2, kFirstFragment);
+        while ( s2 )
+          {
+              // if it _isn't_ the case that
+              //  one fragment starts after the other ends,
+              //  or ends before the other starts,
+              // then, they conflict:
+              // !(f2.mStart>=f1.mEnd || f2.mEnd<=f1.mStart)
+              //
+              // Simplified, that gives us:
+            if ( f2.mStart < f1.mEnd && f2.mEnd > f1.mStart )
+              return PR_TRUE;
+
+            s2 = aString.GetReadableFragment(f2, kNextFragment);
+          }
+        s1 = GetReadableFragment(f1, kNextFragment);
+      }
+    return PR_FALSE;
+  }
 
   //
   // |Assign()|
   //
 
 void
-nsACString::AssignFromReadable( const self_type& rhs )
-  {
-    if ( this != &rhs )
-      do_AssignFromReadable(rhs);
-    // else, self-assign is a no-op
-  }
-
-void
-nsACString::AssignFromPromise( const self_type& aReadable )
+nsACString::do_AssignFromReadable( const self_type& aReadable )
     /*
-      ...this function is only called when a promise that somehow references |this| is assigned _into_ |this|.
+      ...we need to check whether the string that's being assigned into |this| somehow references |this|.
       E.g.,
       
         ... writable& w ...
@@ -780,8 +749,12 @@ nsACString::AssignFromPromise( const self_type& aReadable )
       option to fix either the callers or this mechanism.
     */
   {
+      // self-assign is a no-op
+    if (this == &aReadable)
+      return;
+
     if ( !aReadable.IsDependentOn(*this) )
-      do_AssignFromReadable(aReadable);
+      UncheckedAssignFromReadable(aReadable);
     else
       {
         size_type length = aReadable.Length();
@@ -793,15 +766,15 @@ nsACString::AssignFromPromise( const self_type& aReadable )
             const_iterator fromBegin, fromEnd;
             char_type* toBegin = buffer;
             copy_string(aReadable.BeginReading(fromBegin), aReadable.EndReading(fromEnd), toBegin);
-            do_AssignFromElementPtrLength(buffer, length);
-            delete buffer;
+            UncheckedAssignFromReadable(Substring(buffer, buffer + length));
+            delete[] buffer;
           }
         // else assert?
       }
   }
 
 void
-nsACString::do_AssignFromReadable( const self_type& aReadable )
+nsACString::UncheckedAssignFromReadable( const self_type& aReadable )
   {
     SetLength(0);
     if ( !aReadable.IsEmpty() )
@@ -831,7 +804,7 @@ nsACString::do_AssignFromElementPtrLength( const char_type* aPtr, size_type aLen
 void
 nsACString::do_AssignFromElement( char_type aChar )
   {
-    do_AssignFromReadable(Substring(&aChar, &aChar+1));
+    UncheckedAssignFromReadable(Substring(&aChar, &aChar+1));
   }
 
 
@@ -841,19 +814,10 @@ nsACString::do_AssignFromElement( char_type aChar )
   //
 
 void
-nsACString::AppendFromReadable( const self_type& aReadable )
-  {
-    if ( this != &aReadable )
-      do_AppendFromReadable(aReadable);
-    else
-      AppendFromPromise(aReadable);
-  }
-
-void
-nsACString::AppendFromPromise( const self_type& aReadable )
+nsACString::do_AppendFromReadable( const self_type& aReadable )
   {
     if ( !aReadable.IsDependentOn(*this) )
-      do_AppendFromReadable(aReadable);
+      UncheckedAppendFromReadable(aReadable);
     else
       {
         size_type length = aReadable.Length();
@@ -863,15 +827,15 @@ nsACString::AppendFromPromise( const self_type& aReadable )
             const_iterator fromBegin, fromEnd;
             char_type* toBegin = buffer;
             copy_string(aReadable.BeginReading(fromBegin), aReadable.EndReading(fromEnd), toBegin);
-            do_AppendFromElementPtrLength(buffer, length);
-            delete buffer;
+            UncheckedAppendFromReadable(Substring(buffer, buffer + length));
+            delete[] buffer;
           }
         // else assert?
       }
   }
 
 void
-nsACString::do_AppendFromReadable( const self_type& aReadable )
+nsACString::UncheckedAppendFromReadable( const self_type& aReadable )
   {
     size_type oldLength = this->Length();
     SetLength(oldLength + aReadable.Length());
@@ -896,7 +860,7 @@ nsACString::do_AppendFromElementPtrLength( const char_type* aPtr, size_type aLen
 void
 nsACString::do_AppendFromElement( char_type aChar )
   {
-    do_AppendFromReadable(Substring(&aChar, &aChar + 1));
+    UncheckedAppendFromReadable(Substring(&aChar, &aChar + 1));
   }
 
 
@@ -906,19 +870,10 @@ nsACString::do_AppendFromElement( char_type aChar )
   //
 
 void
-nsACString::InsertFromReadable( const self_type& aReadable, index_type atPosition )
-  {
-    if ( this != &aReadable )
-      do_InsertFromReadable(aReadable, atPosition);
-    else
-      InsertFromPromise(aReadable, atPosition);
-  }
-
-void
-nsACString::InsertFromPromise( const self_type& aReadable, index_type atPosition )
+nsACString::do_InsertFromReadable( const self_type& aReadable, index_type atPosition )
   {
     if ( !aReadable.IsDependentOn(*this) )
-      do_InsertFromReadable(aReadable, atPosition);
+      UncheckedInsertFromReadable(aReadable, atPosition);
     else
       {
         size_type length = aReadable.Length();
@@ -928,15 +883,15 @@ nsACString::InsertFromPromise( const self_type& aReadable, index_type atPosition
             const_iterator fromBegin, fromEnd;
             char_type* toBegin = buffer;
             copy_string(aReadable.BeginReading(fromBegin), aReadable.EndReading(fromEnd), toBegin);
-            do_InsertFromElementPtrLength(buffer, atPosition, length);
-            delete buffer;
+            UncheckedInsertFromReadable(Substring(buffer, buffer + length), atPosition);
+            delete[] buffer;
           }
         // else assert
       }
   }
 
 void
-nsACString::do_InsertFromReadable( const self_type& aReadable, index_type atPosition )
+nsACString::UncheckedInsertFromReadable( const self_type& aReadable, index_type atPosition )
   {
     size_type oldLength = this->Length();
     SetLength(oldLength + aReadable.Length());
@@ -965,7 +920,7 @@ nsACString::do_InsertFromElementPtrLength( const char_type* aPtr, index_type atP
 void
 nsACString::do_InsertFromElement( char_type aChar, index_type atPosition )
   {
-    do_InsertFromReadable(Substring(&aChar, &aChar+1), atPosition);
+    UncheckedInsertFromReadable(Substring(&aChar, &aChar+1), atPosition);
   }
 
 
@@ -995,19 +950,10 @@ nsACString::Cut( index_type cutStart, size_type cutLength )
   //
 
 void
-nsACString::ReplaceFromReadable( index_type cutStart, size_type cutLength, const self_type& aReplacement )
-  {
-    if ( this != &aReplacement )
-      do_ReplaceFromReadable(cutStart, cutLength, aReplacement);
-    else
-      ReplaceFromPromise(cutStart, cutLength, aReplacement);
-  }
-
-void
-nsACString::ReplaceFromPromise( index_type cutStart, size_type cutLength, const self_type& aReadable )
+nsACString::do_ReplaceFromReadable( index_type cutStart, size_type cutLength, const self_type& aReadable )
   {
     if ( !aReadable.IsDependentOn(*this) )
-      do_ReplaceFromReadable(cutStart, cutLength, aReadable);
+      UncheckedReplaceFromReadable(cutStart, cutLength, aReadable);
     else
       {
         size_type length = aReadable.Length();
@@ -1017,15 +963,15 @@ nsACString::ReplaceFromPromise( index_type cutStart, size_type cutLength, const
             const_iterator fromBegin, fromEnd;
             char_type* toBegin = buffer;
             copy_string(aReadable.BeginReading(fromBegin), aReadable.EndReading(fromEnd), toBegin);
-            do_ReplaceFromReadable(cutStart, cutLength, nsDependentCString(buffer, length));
-            delete buffer;
+            UncheckedReplaceFromReadable(cutStart, cutLength, Substring(buffer, buffer + length));
+            delete[] buffer;
           }
         // else assert?
       }
   }
 
 void
-nsACString::do_ReplaceFromReadable( index_type cutStart, size_type cutLength, const self_type& aReplacement )
+nsACString::UncheckedReplaceFromReadable( index_type cutStart, size_type cutLength, const self_type& aReplacement )
   {
     size_type oldLength = this->Length();
 
