@@ -80,17 +80,12 @@ PRLogModuleInfo* gDocLoaderLog = nsnull;
 
 
 #if defined(DEBUG)
-void GetURIStringFromRequest(nsIRequest* request, nsXPIDLCString &aStr)
+void GetURIStringFromRequest(nsIRequest* request, nsACString &name)
 {
-    aStr.Adopt(nsCRT::strdup("???"));
-
-    if (request) {
-        nsXPIDLString name;
-        request->GetName(getter_Copies(name));
-
-        if (name)
-            *getter_Copies(aStr) = ToNewUTF8String(nsDependentString(name));
-    }
+    if (request)
+        request->GetName(name);
+    else
+        name = NS_LITERAL_CSTRING("???");
 }
 #endif /* DEBUG */
 
@@ -112,7 +107,23 @@ struct nsRequestInfo {
 };
 
 
+struct nsListenerInfo {
+  nsListenerInfo(nsIWeakReference *aListener, unsigned long aNotifyMask) 
+    : mWeakListener(aListener),
+      mNotifyMask(aNotifyMask)
+  {
+  }
+
+  // Weak pointer for the nsIWebProgressListener...
+  nsWeakPtr mWeakListener;
+
+  // Mask indicating which notifications the listener wants to receive.
+  unsigned long mNotifyMask;
+};
+
+
 nsDocLoaderImpl::nsDocLoaderImpl()
+                : mListenerInfoList(8)
 {
   NS_INIT_REFCNT();
 
@@ -150,10 +161,7 @@ nsDocLoaderImpl::Init()
     PR_LOG(gDocLoaderLog, PR_LOG_DEBUG, 
            ("DocLoader:%p: load group %x.\n", this, mLoadGroup.get()));
 
-    rv = NS_NewISupportsArray(getter_AddRefs(mChildList));
-    if (NS_FAILED(rv)) return rv;
-
-    return NS_NewISupportsArray(getter_AddRefs(mListenerList));
+    return NS_NewISupportsArray(getter_AddRefs(mChildList));
 }
 
 NS_IMETHODIMP nsDocLoaderImpl::ClearParentDocLoader()
@@ -415,8 +423,22 @@ nsDocLoaderImpl::Destroy()
     mParent = nsnull;
   }
 
+  // Release all the information about network requests...
   ClearRequestInfoList();
 
+  // Release all the information about registered listeners...
+  nsListenerInfo *info;
+  PRInt32 i, count;
+
+  count = mListenerInfoList.Count();
+  for(i=0; i<count; i++) {
+    info = NS_STATIC_CAST(nsListenerInfo*,mListenerInfoList.ElementAt(i));
+    delete info;
+  }
+
+  mListenerInfoList.Clear();
+  mListenerInfoList.Compact();
+
   mDocumentRequest = 0;
 
   mLoadGroup->SetGroupObserver(nsnull);
@@ -431,8 +453,8 @@ nsDocLoaderImpl::OnStartRequest(nsIRequest *request, nsISupports *aCtxt)
 
 #ifdef PR_LOGGING
   if (PR_LOG_TEST(gDocLoaderLog, PR_LOG_DEBUG)) {
-    nsXPIDLString name;
-    request->GetName(getter_Copies(name));
+    nsCAutoString name;
+    request->GetName(name);
 
     PRUint32 count = 0;
     if (mLoadGroup)
@@ -440,7 +462,7 @@ nsDocLoaderImpl::OnStartRequest(nsIRequest *request, nsISupports *aCtxt)
 
     PR_LOG(gDocLoaderLog, PR_LOG_DEBUG,
            ("DocLoader:%p: OnStartRequest[%p](%s) mIsLoadingDocument=%s, %u active URLs",
-            this, request, NS_ConvertUCS2toUTF8(name).get(),
+            this, request, name.get(),
             (mIsLoadingDocument ? "true" : "false"),
             count));
   }
@@ -516,8 +538,8 @@ nsDocLoaderImpl::OnStopRequest(nsIRequest *aRequest,
 
 #ifdef PR_LOGGING
   if (PR_LOG_TEST(gDocLoaderLog, PR_LOG_DEBUG)) {
-    nsXPIDLString name;
-    aRequest->GetName(getter_Copies(name));
+    nsCAutoString name;
+    aRequest->GetName(name);
 
     PRUint32 count = 0;
     if (mLoadGroup)
@@ -525,7 +547,7 @@ nsDocLoaderImpl::OnStopRequest(nsIRequest *aRequest,
 
     PR_LOG(gDocLoaderLog, PR_LOG_DEBUG,
            ("DocLoader:%p: OnStopRequest[%p](%s) status=%x mIsLoadingDocument=%s, %u active URLs",
-           this, aRequest, NS_ConvertUCS2toUTF8(name).get(),
+           this, aRequest, name.get(),
            aStatus, (mIsLoadingDocument ? "true" : "false"),
            count));
   }
@@ -655,13 +677,13 @@ void nsDocLoaderImpl::doStartDocumentLoad(void)
 {
 
 #if defined(DEBUG)
-  nsXPIDLCString buffer;
+  nsCAutoString buffer;
 
   GetURIStringFromRequest(mDocumentRequest, buffer);
   PR_LOG(gDocLoaderLog, PR_LOG_DEBUG, 
          ("DocLoader:%p: ++ Firing OnStateChange for start document load (...)."
           "\tURI: %s \n",
-          this, (const char *) buffer));
+          this, buffer.get()));
 #endif /* DEBUG */
 
   // Fire an OnStatus(...) notification STATE_START.  This indicates
@@ -679,13 +701,13 @@ void nsDocLoaderImpl::doStartDocumentLoad(void)
 void nsDocLoaderImpl::doStartURLLoad(nsIRequest *request)
 {
 #if defined(DEBUG)
-  nsXPIDLCString buffer;
+  nsCAutoString buffer;
 
   GetURIStringFromRequest(request, buffer);
     PR_LOG(gDocLoaderLog, PR_LOG_DEBUG, 
           ("DocLoader:%p: ++ Firing OnStateChange start url load (...)."
            "\tURI: %s\n",
-            this, (const char *) buffer));
+            this, buffer.get()));
 #endif /* DEBUG */
 
   FireOnStateChange(this,
@@ -698,13 +720,13 @@ void nsDocLoaderImpl::doStartURLLoad(nsIRequest *request)
 void nsDocLoaderImpl::doStopURLLoad(nsIRequest *request, nsresult aStatus)
 {
 #if defined(DEBUG)
-  nsXPIDLCString buffer;
+  nsCAutoString buffer;
 
   GetURIStringFromRequest(request, buffer);
     PR_LOG(gDocLoaderLog, PR_LOG_DEBUG, 
           ("DocLoader:%p: ++ Firing OnStateChange for end url load (...)."
            "\tURI: %s status=%x\n",
-            this, (const char *) buffer, aStatus));
+            this, buffer.get(), aStatus));
 #endif /* DEBUG */
 
   FireOnStateChange(this,
@@ -718,17 +740,17 @@ void nsDocLoaderImpl::doStopDocumentLoad(nsIRequest *request,
                                          nsresult aStatus)
 {
 #if defined(DEBUG)
-  nsXPIDLCString buffer;
+  nsCAutoString buffer;
 
   GetURIStringFromRequest(request, buffer);
   PR_LOG(gDocLoaderLog, PR_LOG_DEBUG, 
          ("DocLoader:%p: ++ Firing OnStateChange for end document load (...)."
          "\tURI: %s Status=%x\n",
-          this, (const char *) buffer, aStatus));
+          this, buffer.get(), aStatus));
 #endif /* DEBUG */
 
   //
-  // Fire an OnStatusChange(...) notification indicating the the
+  // Fire an OnStateChange(...) notification indicating the the
   // current document has finished loading...
   //
   FireOnStateChange(this,
@@ -738,7 +760,7 @@ void nsDocLoaderImpl::doStopDocumentLoad(nsIRequest *request,
                     aStatus);
 
   //
-  // Fire a final OnStatusChange(...) notification indicating the the
+  // Fire a final OnStateChange(...) notification indicating the the
   // current document has finished loading...
   //
   FireOnStateChange(this,
@@ -765,32 +787,49 @@ nsresult nsDocLoaderImpl::GetParentWebProgressListener(nsDocLoaderImpl * aDocLoa
 }
 
 NS_IMETHODIMP
-nsDocLoaderImpl::AddProgressListener(nsIWebProgressListener *aListener)
+nsDocLoaderImpl::AddProgressListener(nsIWebProgressListener *aListener,
+                                     PRUint32 aNotifyMask)
 {
   nsresult rv;
-
   nsWeakPtr listener = getter_AddRefs(NS_GetWeakReference(aListener));
-  if (!listener) return NS_ERROR_INVALID_ARG;
-
-  if (mListenerList->IndexOf(listener) == -1) {
-
-    // XXX this method incorrectly returns a bool    
-    rv = mListenerList->AppendElement(listener) ? NS_OK : NS_ERROR_FAILURE;
-  } else {
-    // The listener is already in the list...
-    rv = NS_ERROR_FAILURE;
+  if (!listener) {
+    return NS_ERROR_INVALID_ARG;
   }
+
+  nsListenerInfo* info = GetListenerInfo(listener);
+  if (info) {
+    // The listener is already registered!
+    return NS_ERROR_FAILURE;
+  }
+
+  info = new nsListenerInfo(listener, aNotifyMask);
+  if (!info) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+
+  // XXX this method incorrectly returns a bool    
+  rv = mListenerInfoList.AppendElement(info) ? NS_OK : NS_ERROR_FAILURE;
   return rv;
 }
 
 NS_IMETHODIMP
 nsDocLoaderImpl::RemoveProgressListener(nsIWebProgressListener *aListener)
 {
+  nsresult rv;
   nsWeakPtr listener = getter_AddRefs(NS_GetWeakReference(aListener));
-  if (!listener) return NS_ERROR_INVALID_ARG;
+  if (!listener) {
+    return NS_ERROR_INVALID_ARG;
+  }
 
-  // XXX this method incorrectly returns a bool    
-  return mListenerList->RemoveElement(listener) ? NS_OK : NS_ERROR_FAILURE;
+  nsListenerInfo* info = GetListenerInfo(listener);
+  if (info) {
+    rv = mListenerInfoList.RemoveElement(info) ? NS_OK : NS_ERROR_FAILURE;
+    delete info;
+  } else {
+    // The listener is not registered!
+    rv = NS_ERROR_FAILURE;
+  }
+  return rv;
 }
 
 NS_IMETHODIMP
@@ -815,30 +854,6 @@ nsDocLoaderImpl::GetDOMWindow(nsIDOMWindow **aResult)
   return rv;
 }
 
-nsresult nsDocLoaderImpl::GetProgressStatusFlags(PRInt32 *aProgressStateFlags)
-{
-  *aProgressStateFlags = mProgressStateFlags;
-  return NS_OK;
-}
-
-nsresult nsDocLoaderImpl::GetCurSelfProgress(PRInt32 *aCurSelfProgress)
-{
-  *aCurSelfProgress = mCurrentSelfProgress;
-  return NS_OK;
-}
-
-nsresult nsDocLoaderImpl::GetMaxSelfProgress(PRInt32 *aMaxSelfProgress)
-{
-  *aMaxSelfProgress = mMaxSelfProgress;
-  return NS_OK;
-}
-
-nsresult nsDocLoaderImpl::GetCurTotalProgress(PRInt32 *aCurTotalProgress)
-{
-  *aCurTotalProgress = mCurrentTotalProgress;
-  return NS_OK;
-}
-
 nsresult nsDocLoaderImpl::GetMaxTotalProgress(PRInt32 *aMaxTotalProgress)
 {
   PRUint32 count = 0;
@@ -939,12 +954,12 @@ NS_IMETHODIMP nsDocLoaderImpl::OnProgress(nsIRequest *aRequest, nsISupports* ctx
   //
   else {
 #if defined(DEBUG)
-    nsXPIDLCString buffer;
+    nsCAutoString buffer;
 
     GetURIStringFromRequest(aRequest, buffer);
     PR_LOG(gDocLoaderLog, PR_LOG_DEBUG, 
            ("DocLoader:%p OOPS - No Request Info for: %s\n",
-            this, (const char *)buffer));
+            this, buffer.get()));
 #endif /* DEBUG */
 
     return NS_OK;
@@ -996,8 +1011,6 @@ void nsDocLoaderImpl::FireOnProgressChange(nsDocLoaderImpl *aLoadInitiator,
                                            PRInt32 aTotalProgress,
                                            PRInt32 aMaxTotalProgress)
 {
-  PRUint32 count;
-
   if (mIsLoadingDocument) {
     mCurrentTotalProgress += aProgressDelta;
     GetMaxTotalProgress(&mMaxTotalProgress);
@@ -1007,12 +1020,12 @@ void nsDocLoaderImpl::FireOnProgressChange(nsDocLoaderImpl *aLoadInitiator,
   }
 
 #if defined(DEBUG)
-  nsXPIDLCString buffer;
+  nsCAutoString buffer;
 
   GetURIStringFromRequest(request, buffer);
   PR_LOG(gDocLoaderLog, PR_LOG_DEBUG, 
          ("DocLoader:%p: Progress (%s): curSelf: %d maxSelf: %d curTotal: %d maxTotal %d\n",
-          this, (const char *)buffer, aProgress, aProgressMax, aTotalProgress, aMaxTotalProgress));
+          this, buffer.get(), aProgress, aProgressMax, aTotalProgress, aMaxTotalProgress));
 #endif /* DEBUG */
 
   /*
@@ -1021,19 +1034,23 @@ void nsDocLoaderImpl::FireOnProgressChange(nsDocLoaderImpl *aLoadInitiator,
    * Operate the elements from back to front so that if items get
    * get removed from the list it won't affect our iteration
    */
-  (void)mListenerList->Count(&count);
-  while (count > 0) {
-    nsCOMPtr<nsISupports> supports;
-    nsresult rv = mListenerList->GetElementAt(--count, getter_AddRefs(supports));
-    if (NS_FAILED(rv)) return;
-    nsWeakPtr weakPtr = do_QueryInterface(supports);
+  nsCOMPtr<nsIWebProgressListener> listener;
+  PRInt32 count = mListenerInfoList.Count();
 
-    nsCOMPtr<nsIWebProgressListener> listener;
-    listener = do_QueryReferent(weakPtr);
+  while (--count >= 0) {
+    nsListenerInfo *info;
+
+    info = NS_STATIC_CAST(nsListenerInfo*,mListenerInfoList.ElementAt(count));
+    if (!info || !(info->mNotifyMask & nsIWebProgress::NOTIFY_PROGRESS)) {
+      continue;
+    }
+
+    listener = do_QueryReferent(info->mWeakListener);
     if (!listener) {
-        // the listener went away. gracefully pull it out of the list.
-        mListenerList->RemoveElementAt(count);
-        continue;
+      // the listener went away. gracefully pull it out of the list.
+      mListenerInfoList.RemoveElementAt(count);
+      delete info;
+      continue;
     }
 
     listener->OnProgressChange(aLoadInitiator,request,
@@ -1041,7 +1058,7 @@ void nsDocLoaderImpl::FireOnProgressChange(nsDocLoaderImpl *aLoadInitiator,
                                aTotalProgress, aMaxTotalProgress);
   }
 
-  mListenerList->Compact();
+  mListenerInfoList.Compact();
 
   // Pass the notification up to the parent...
   if (mParent) {
@@ -1059,8 +1076,6 @@ void nsDocLoaderImpl::FireOnStateChange(nsIWebProgress *aProgress,
                                         PRInt32 aStateFlags,
                                         nsresult aStatus)
 {
-  PRUint32 count;
-
   //
   // Remove the STATE_IS_NETWORK bit if necessary.
   //
@@ -1075,12 +1090,12 @@ void nsDocLoaderImpl::FireOnStateChange(nsIWebProgress *aProgress,
   }
 
 #if defined(DEBUG)
-  nsXPIDLCString buffer;
+  nsCAutoString buffer;
 
   GetURIStringFromRequest(aRequest, buffer);
   PR_LOG(gDocLoaderLog, PR_LOG_DEBUG, 
          ("DocLoader:%p: Status (%s): code: %x\n",
-         this, (const char *)buffer, aStateFlags));
+         this, buffer.get(), aStateFlags));
 #endif /* DEBUG */
 
   NS_ASSERTION(aRequest, "Firing OnStateChange(...) notification with a NULL request!");
@@ -1091,25 +1106,29 @@ void nsDocLoaderImpl::FireOnStateChange(nsIWebProgress *aProgress,
    * Operate the elements from back to front so that if items get
    * get removed from the list it won't affect our iteration
    */
-  (void)mListenerList->Count(&count);
-  while (count > 0) {
-    nsCOMPtr<nsISupports> supports;
-    nsresult rv = mListenerList->GetElementAt(--count, getter_AddRefs(supports));
-    if (NS_FAILED(rv)) return;
-    nsWeakPtr weakPtr = do_QueryInterface(supports);
+  nsCOMPtr<nsIWebProgressListener> listener;
+  PRInt32 count = mListenerInfoList.Count();
 
-    nsCOMPtr<nsIWebProgressListener> listener;
-    listener = do_QueryReferent(weakPtr);
+  while (--count >= 0) {
+    nsListenerInfo *info;
+
+    info = NS_STATIC_CAST(nsListenerInfo*,mListenerInfoList.ElementAt(count));
+    if (!info || !(info->mNotifyMask & (aStateFlags >>16))) {
+      continue;
+    }
+
+    listener = do_QueryReferent(info->mWeakListener);
     if (!listener) {
-        // the listener went away, gracefully pull it out of the list.
-        mListenerList->RemoveElementAt(count);
-        continue;
+      // the listener went away. gracefully pull it out of the list.
+      mListenerInfoList.RemoveElementAt(count);
+      delete info;
+      continue;
     }
 
     listener->OnStateChange(aProgress, aRequest, aStateFlags, aStatus);
   }
 
-  mListenerList->Compact();
+  mListenerInfoList.Compact();
 
   // Pass the notification up to the parent...
   if (mParent) {
@@ -1124,27 +1143,36 @@ nsDocLoaderImpl::FireOnLocationChange(nsIWebProgress* aWebProgress,
                                       nsIRequest* aRequest,
                                       nsIURI *aUri)
 {
-  PRUint32 count;
+  /*                                                                           
+   * First notify any listeners of the new state info...
+   *
+   * Operate the elements from back to front so that if items get
+   * get removed from the list it won't affect our iteration
+   */
+  nsCOMPtr<nsIWebProgressListener> listener;
+  PRInt32 count = mListenerInfoList.Count();
 
-  (void)mListenerList->Count(&count);
-  while (count > 0) {
-    nsCOMPtr<nsISupports> supports;
-    nsresult rv = mListenerList->GetElementAt(--count, getter_AddRefs(supports));
-    if (NS_FAILED(rv)) return rv;
-    nsWeakPtr weakPtr = do_QueryInterface(supports);
+  while (--count >= 0) {
+    nsListenerInfo *info;
 
-    nsCOMPtr<nsIWebProgressListener> listener;
-    listener = do_QueryReferent(weakPtr);
+    info = NS_STATIC_CAST(nsListenerInfo*,mListenerInfoList.ElementAt(count));
+    if (!info || !(info->mNotifyMask & nsIWebProgress::NOTIFY_LOCATION)) {
+      continue;
+    }
+
+    listener = do_QueryReferent(info->mWeakListener);
     if (!listener) {
-        // the listener went away. gracefully pull it out of the list.
-        mListenerList->RemoveElementAt(count);
-        continue;
+      // the listener went away. gracefully pull it out of the list.
+      mListenerInfoList.RemoveElementAt(count);
+      delete info;
+      continue;
     }
 
     listener->OnLocationChange(aWebProgress, aRequest, aUri);
   }
 
-  mListenerList->Compact();
+  mListenerInfoList.Compact();
+
   // Pass the notification up to the parent...
   if (mParent) {
     mParent->FireOnLocationChange(aWebProgress, aRequest, aUri);
@@ -1159,30 +1187,62 @@ nsDocLoaderImpl::FireOnStatusChange(nsIWebProgress* aWebProgress,
                                     nsresult aStatus,
                                     const PRUnichar* aMessage)
 {
-  PRUint32 count;
+  /*                                                                           
+   * First notify any listeners of the new state info...
+   *
+   * Operate the elements from back to front so that if items get
+   * get removed from the list it won't affect our iteration
+   */
+  nsCOMPtr<nsIWebProgressListener> listener;
+  PRInt32 count = mListenerInfoList.Count();
 
-  (void)mListenerList->Count(&count);
-  while (count > 0) {
-    nsCOMPtr<nsISupports> supports;
-    nsresult rv = mListenerList->GetElementAt(--count, getter_AddRefs(supports));
-    if (NS_FAILED(rv)) return rv;
-    nsWeakPtr weakPtr = do_QueryInterface(supports);
+  while (--count >= 0) {
+    nsListenerInfo *info;
 
-    nsCOMPtr<nsIWebProgressListener> listener;
-    listener = do_QueryReferent(weakPtr);
+    info = NS_STATIC_CAST(nsListenerInfo*,mListenerInfoList.ElementAt(count));
+    if (!info || !(info->mNotifyMask & nsIWebProgress::NOTIFY_STATUS)) {
+      continue;
+    }
+
+    listener = do_QueryReferent(info->mWeakListener);
     if (!listener) {
-        // the listener went away. gracefully pull it out of the list.
-        mListenerList->RemoveElementAt(count);
-        continue;
+      // the listener went away. gracefully pull it out of the list.
+      mListenerInfoList.RemoveElementAt(count);
+      delete info;
+      continue;
     }
 
     listener->OnStatusChange(aWebProgress, aRequest, aStatus, aMessage);
   }
-  mListenerList->Compact();
+  mListenerInfoList.Compact();
+  
+  // Pass the notification up to the parent...
+  if (mParent) {
+    mParent->FireOnStatusChange(aWebProgress, aRequest, aStatus, aMessage);
+  }
+
 
   return NS_OK;
 }
 
+nsListenerInfo * 
+nsDocLoaderImpl::GetListenerInfo(nsIWeakReference *aListener)
+{
+  PRInt32 i, count;
+  nsListenerInfo *info;
+
+  count = mListenerInfoList.Count();
+  for (i=0; i<count; i++) {
+    info = NS_STATIC_CAST(nsListenerInfo* ,mListenerInfoList.ElementAt(i));
+
+    NS_ASSERTION(info, "There should NEVER be a null listener in the list");
+    if (info && (aListener == info->mWeakListener)) {
+      return info;
+    }
+  }
+  return nsnull;
+}
+
 nsresult nsDocLoaderImpl::AddRequestInfo(nsIRequest *aRequest)
 {
   nsresult rv;
@@ -1205,7 +1265,7 @@ nsRequestInfo * nsDocLoaderImpl::GetRequestInfo(nsIRequest *aRequest)
 
   count = mRequestInfoList.Count();
   for(i=0; i<count; i++) {
-    info = (nsRequestInfo *)mRequestInfoList.ElementAt(i);
+    info = NS_STATIC_CAST(nsRequestInfo*, mRequestInfoList.ElementAt(i));
     if (aRequest == info->mKey) {
       return info;
     }
@@ -1221,7 +1281,7 @@ nsresult nsDocLoaderImpl::ClearRequestInfoList(void)
 
   count = mRequestInfoList.Count();
   for(i=0; i<count; i++) {
-    info = (nsRequestInfo *)mRequestInfoList.ElementAt(i);
+    info = NS_STATIC_CAST(nsRequestInfo*, mRequestInfoList.ElementAt(i));
     delete info;
   }
 
@@ -1240,7 +1300,7 @@ void nsDocLoaderImpl::CalculateMaxProgress(PRInt32 *aMax)
   for(i=0; i<count; i++) {
     nsRequestInfo *info;
   
-    info = (nsRequestInfo *)mRequestInfoList.ElementAt(i);
+    info = NS_STATIC_CAST(nsRequestInfo*, mRequestInfoList.ElementAt(i));
 
     current += info->mCurrentProgress;
     if (max >= 0) {
@@ -1268,6 +1328,7 @@ NS_IMETHODIMP nsDocLoaderImpl::OnRedirect(nsIHttpChannel *aOldChannel, nsIChanne
     rv = aNewChannel->GetURI(getter_AddRefs(newURI));
     if (NS_FAILED(rv)) return rv;
 
+#ifdef HTTP_DOESNT_CALL_CHECKLOADURI
     // verify that this is a legal redirect
     nsCOMPtr<nsIScriptSecurityManager> securityManager = 
              do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
@@ -1275,6 +1336,7 @@ NS_IMETHODIMP nsDocLoaderImpl::OnRedirect(nsIHttpChannel *aOldChannel, nsIChanne
     rv = securityManager->CheckLoadURI(oldURI, newURI,
                                        nsIScriptSecurityManager::DISALLOW_FROM_MAIL);
     if (NS_FAILED(rv)) return rv;
+#endif
 
     nsLoadFlags loadFlags = 0;
     PRInt32 stateFlags = nsIWebProgressListener::STATE_REDIRECTING |
@@ -1299,8 +1361,12 @@ NS_IMETHODIMP nsDocLoaderImpl::OnRedirect(nsIHttpChannel *aOldChannel, nsIChanne
   return NS_OK;
 }
 
+/*
+ * Implementation of nsISecurityEventSink method...
+ */
+
 NS_IMETHODIMP nsDocLoaderImpl::OnSecurityChange(nsISupports * aContext,
-                                                PRInt32 state)
+                                                PRUint32 aState)
 {
   //
   // Fire progress notifications out to any registered nsIWebProgressListeners.  
@@ -1309,30 +1375,39 @@ NS_IMETHODIMP nsDocLoaderImpl::OnSecurityChange(nsISupports * aContext,
   nsCOMPtr<nsIRequest> request = do_QueryInterface(aContext);
   nsIWebProgress* webProgress = NS_STATIC_CAST(nsIWebProgress*, this);
 
-  PRUint32 count;
-  (void)mListenerList->Count(&count);
+  /*                                                                           
+   * First notify any listeners of the new state info...
+   *
+   * Operate the elements from back to front so that if items get
+   * get removed from the list it won't affect our iteration
+   */
+  nsCOMPtr<nsIWebProgressListener> listener;
+  PRInt32 count = mListenerInfoList.Count();
 
-  while (count > 0) {
-    nsCOMPtr<nsISupports> supports;
-    nsresult rv = mListenerList->GetElementAt(--count, getter_AddRefs(supports));
-    if (NS_FAILED(rv)) return rv;
-    nsWeakPtr weakPtr = do_QueryInterface(supports);
+  while (--count >= 0) {
+    nsListenerInfo *info;
 
-    nsCOMPtr<nsIWebProgressListener> listener;
-    listener = do_QueryReferent(weakPtr);
-    if (!listener) {
-        // the listener went away. gracefully pull it out of the list.
-        mListenerList->RemoveElementAt(count);
-        continue;
+    info = NS_STATIC_CAST(nsListenerInfo*,mListenerInfoList.ElementAt(count));
+    if (!info || !(info->mNotifyMask & nsIWebProgress::NOTIFY_SECURITY)) {
+      continue;
     }
-    listener->OnSecurityChange(webProgress, request, state);
+
+    listener = do_QueryReferent(info->mWeakListener);
+    if (!listener) {
+      // the listener went away. gracefully pull it out of the list.
+      mListenerInfoList.RemoveElementAt(count);
+      delete info;
+      continue;
+    }
+
+    listener->OnSecurityChange(webProgress, request, aState);
   }
 
-  mListenerList->Compact();
+  mListenerInfoList.Compact();
 
   // Pass the notification up to the parent...
   if (mParent) {
-    mParent->OnSecurityChange(aContext, state);
+    mParent->OnSecurityChange(aContext, aState);
   }
   return NS_OK;
 }
@@ -1355,15 +1430,15 @@ void nsDocLoaderImpl::DumpChannelInfo()
     info = (nsChannelInfo *)mChannelInfoList.ElementAt(i);
 
 #if defined(DEBUG)
-    nsXPIDLCString buffer;
+    nsCAutoString buffer;
     nsresult rv = NS_OK;
     if (info->mURI) {
-      rv = info->mURI->GetSpec(getter_Copies(buffer));
+      rv = info->mURI->GetSpec(buffer);
     }
 
     printf("  [%d] current=%d  max=%d [%s]\n", i,
            info->mCurrentProgress, 
-           info->mMaxProgress, (const char *)buffer);
+           info->mMaxProgress, buffer.get());
 #endif /* DEBUG */
 
     current += info->mCurrentProgress;
