@@ -35,7 +35,6 @@
 // Interfaces needed to be included
 #include "nsIDocShellTreeItem.h"
 #include "nsIWebProgress.h"
-#include "nsIChannel.h"
 #include "nsIURI.h"
 #include "nsIDOMWindow.h"
 
@@ -136,9 +135,9 @@ NS_IMETHODIMP nsWebBrowserChrome::SetWebBrowser(nsIWebBrowser* aWebBrowser)
 NS_IMETHODIMP nsWebBrowserChrome::GetWebBrowser(nsIWebBrowser** aWebBrowser)
 {
   // Unimplemented, and probably will remain so; xpfe windows have docshells,
-  // not webbrowsers. Note that we're relying on this to differentiate
-  // embedded instances from Mozilla in nsWindowWatcher::InitializeDocshell.
-  // (Don't let me stop you, but keep that in mind.)
+  // not webbrowsers.
+  NS_ENSURE_ARG_POINTER(aWebBrowser);
+  *aWebBrowser = 0;
   return NS_ERROR_FAILURE;
 }
 
@@ -400,7 +399,7 @@ nsWebBrowserChrome::OnProgressChange(nsIWebProgress* aProgress,
 NS_IMETHODIMP
 nsWebBrowserChrome::OnStateChange(nsIWebProgress* aProgress,
                                   nsIRequest* aRequest,
-                                  PRInt32 aProgressStateFlags,
+                                  PRUint32 aProgressStateFlags,
                                   nsresult aStatus)
 {
   if (aProgressStateFlags & STATE_START) {
@@ -495,9 +494,9 @@ nsWebBrowserChrome::OnStatusChange(nsIWebProgress* aWebProgress,
 NS_IMETHODIMP 
 nsWebBrowserChrome::OnSecurityChange(nsIWebProgress *aWebProgress, 
                                      nsIRequest *aRequest, 
-                                     PRInt32 state)
+                                     PRUint32 state)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_OK;
 }
 
 //*****************************************************************************
