@@ -165,23 +165,6 @@ nsChildView::~nsChildView()
     [mView release];
   }
   
-  // notify the children that we're gone
-//FIXME do we still need this?
-  nsCOMPtr<nsIEnumerator> children ( getter_AddRefs(GetChildren()) );
-  if (children)
-  {
-    children->First();
-    do
-    {
-      nsISupports* child;
-      if (NS_SUCCEEDED(children->CurrentItem(&child)))
-      {
-        nsChildView* childWindow = static_cast<nsChildView*>(static_cast<nsIWidget*>(child));
-        NS_RELEASE(child);
-      }
-    } while (NS_SUCCEEDED(children->Next()));     
-  }
-      
   NS_IF_RELEASE(mTempRenderingContext); 
   NS_IF_RELEASE(mFontMetrics);
   
@@ -551,22 +534,22 @@ NS_IMETHODIMP nsChildView::ModalEventFilter(PRBool aRealEvent, void *aEvent,
 }
 
 
-//-------------------------------------------------------------------------
 //
-// Enable/disable this component
+// Enable
+//
+// Enable/disable this view
 //
-//-------------------------------------------------------------------------
 NS_IMETHODIMP nsChildView::Enable(PRBool aState)
 {
-  // unimplemented.
+  // unimplemented;
   return NS_OK;
 }
 
 
 NS_IMETHODIMP nsChildView::IsEnabled(PRBool *aState)
 {
-  // unimplemented.
-  if (*aState)
+  // unimplemented
+  if (aState)
    *aState = PR_TRUE;
   return NS_OK;
 }
@@ -826,12 +809,17 @@ NS_IMETHODIMP nsChildView::ConstrainPosition(PRBool aAllowSlop,
 // aX and aY are in the parent widget coordinate system
 //-------------------------------------------------------------------------
 NS_IMETHODIMP nsChildView::Move(PRInt32 aX, PRInt32 aY)
+{
+  return MoveWithRepaintOption(aX, aY, PR_TRUE);
+}
+
+NS_IMETHODIMP nsChildView::MoveWithRepaintOption(PRInt32 aX, PRInt32 aY, PRBool aRepaint)
 {
   if ((mBounds.x != aX) || (mBounds.y != aY))
   {
     // Invalidate the current location
-    if (mVisible)
-        [[mView superview] setNeedsDisplayInRect: [mView frame]];    //XXX needed?
+    if (mVisible && aRepaint)
+      [[mView superview] setNeedsDisplayInRect: [mView frame]];    //XXX needed?
     
     // Set the bounds
     mBounds.x = aX;
@@ -841,7 +829,7 @@ NS_IMETHODIMP nsChildView::Move(PRInt32 aX, PRInt32 aY)
     ConvertGeckoToCocoaRect(mBounds, r);
     [mView setFrame:r];
 
-    if (mVisible)
+    if (mVisible && aRepaint)
       [mView setNeedsDisplay:YES];
     
     // Report the event
@@ -863,7 +851,7 @@ NS_IMETHODIMP nsChildView::Resize(PRInt32 aWidth, PRInt32 aHeight, PRBool aRepai
     mBounds.width  = aWidth;
     mBounds.height = aHeight;
 
-    if (mVisible)
+    if (mVisible && aRepaint)
       [[mView superview] setNeedsDisplayInRect: [mView frame]];    //XXX needed?
     
   // Recalculate the regions
@@ -872,7 +860,7 @@ NS_IMETHODIMP nsChildView::Resize(PRInt32 aWidth, PRInt32 aHeight, PRBool aRepai
     ConvertGeckoToCocoaRect(mBounds, r);
     [mView setFrame:r];
 
-    if (mVisible)
+    if (mVisible && aRepaint)
       [mView setNeedsDisplay:YES];
 
     // Report the event
@@ -889,7 +877,7 @@ NS_IMETHODIMP nsChildView::Resize(PRInt32 aWidth, PRInt32 aHeight, PRBool aRepai
 //-------------------------------------------------------------------------
 NS_IMETHODIMP nsChildView::Resize(PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight, PRBool aRepaint)
 {
-  Move(aX, aY);
+  MoveWithRepaintOption(aX, aY, aRepaint);
   Resize(aWidth, aHeight, aRepaint);
   return NS_OK;
 }
@@ -1247,22 +1235,36 @@ NS_IMETHODIMP nsChildView::Scroll(PRInt32 aDx, PRInt32 aDy, nsRect *aClipRect)
         nsCOMPtr<nsISupports> child;
         if (NS_SUCCEEDED(children->CurrentItem(getter_AddRefs(child)))) {
             nsCOMPtr<nsIWidget> widget = do_QueryInterface(child);
-            
+
+            // We use resize rather than move since it gives us control
+            // over repainting.  In the case of blitting, Quickdraw views
+            // draw their child widgets on the blit, so we can scroll
+            // like a bat out of hell by not wasting time invalidating
+            // the widgets, since it's completely unnecessary to do so.
             nsRect bounds;
             widget->GetBounds(bounds);
-            widget->Move(bounds.x + aDx, bounds.y + aDy);
+            widget->Resize(bounds.x + aDx, bounds.y + aDy, bounds.width, bounds.height, PR_FALSE);
         }
         } while (NS_SUCCEEDED(children->Next()));     
     }
 
     NSRect frame = [mView bounds];
     NSRect horizInvalid = frame;
-    horizInvalid.size.height = abs(aDy);
-    if ( aDy < 0 )
-        horizInvalid.origin.y = frame.origin.y + frame.size.height + aDy;
+    NSRect vertInvalid = frame;
+
+    horizInvalid.size.width = abs(aDx);
+    vertInvalid.size.height = abs(aDy);
+    if (aDy < 0)
+      vertInvalid.origin.y = frame.origin.y + frame.size.height + aDy;
+    if (aDx < 0)
+      horizInvalid.origin.x = frame.origin.x + frame.size.width + aDx;
+
+    if (aDx != 0)
+      [mView setNeedsDisplayInRect: horizInvalid];
+
     if (aDy != 0)
-        [mView setNeedsDisplayInRect: horizInvalid];
-  
+      [mView setNeedsDisplayInRect: vertInvalid];
+    
     return NS_OK;
 }
 
@@ -1518,48 +1520,6 @@ PRBool nsChildView::PointInWidget(Point aThePoint)
   return(widgetRect.Contains(aThePoint.h, aThePoint.v));
 }
 
-
-//-------------------------------------------------------------------------
-// FindWidgetHit
-//    Recursively look for the widget hit
-//    @param aParent   -- parent widget. 
-//    @param aThePoint -- a point in local coordinates to test for the hit. 
-//-------------------------------------------------------------------------
-nsChildView*  nsChildView::FindWidgetHit(Point aThePoint)
-{
-  if (!mVisible || !PointInWidget(aThePoint))
-    return nsnull;
-
-  nsChildView* widgetHit = this;
-
-  nsCOMPtr<nsIEnumerator> normalEnum ( getter_AddRefs(GetChildren()) );
-  nsCOMPtr<nsIBidirectionalEnumerator> children ( do_QueryInterface(normalEnum) );
-  if (children)
-  {
-    // traverse through all the nsChildViews to find out who got hit, lowest level of course
-    children->Last();
-    do
-    {
-      nsISupports* child;
-      if (NS_SUCCEEDED(children->CurrentItem(&child)))
-      {
-        nsChildView* childWindow = static_cast<nsChildView*>(static_cast<nsIWidget*>(child));
-        NS_RELEASE(child);
-
-        nsChildView* deeperHit = childWindow->FindWidgetHit(aThePoint);
-        if (deeperHit)
-        {
-          widgetHit = deeperHit;
-          break;
-        }
-      }
-    }
-    while (NS_SUCCEEDED(children->Prev()));
-  }
-
-  return widgetHit;
-}
-
 #pragma mark -
 
 
@@ -1723,6 +1683,33 @@ nsChildView::GetQuickDrawPort()
 
 @implementation ChildView
 
+-(NSMenu*)menuForEvent:(NSEvent*)theEvent
+{
+  int button = [theEvent buttonNumber];
+  if (button == 1) {
+    // The right mouse went down.  Fire off a right mouse down and
+    // then send the context menu event.
+    nsMouseEvent geckoEvent;
+    geckoEvent.eventStructType = NS_MOUSE_EVENT;
+    [self convert: theEvent message: NS_MOUSE_RIGHT_BUTTON_DOWN toGeckoEvent:&geckoEvent];
+    geckoEvent.clickCount = 1;
+    mGeckoChild->DispatchMouseEvent(geckoEvent);
+  }
+
+  // Fire the context menu event into Gecko.
+  nsMouseEvent geckoEvent;
+  geckoEvent.eventStructType = NS_MOUSE_EVENT;
+  [self convert:theEvent message:NS_CONTEXTMENU toGeckoEvent:&geckoEvent];
+  geckoEvent.clickCount = 0;
+  
+  // send event into Gecko by going directly to the
+  // the widget.
+  mGeckoChild->DispatchMouseEvent(geckoEvent);
+  
+  // Go up our view chain to fetch the correct menu to return.
+  return nil;
+}
+
 
 //
 // initWithGeckoChild:eventSink:
@@ -1888,7 +1875,13 @@ nsChildView::GetQuickDrawPort()
 
 - (void)mouseDragged:(NSEvent*)theEvent
 {
-  [self mouseMoved: theEvent];
+    nsMouseEvent geckoEvent;
+    geckoEvent.eventStructType = NS_MOUSE_EVENT;
+    [self convert:theEvent message:NS_MOUSE_MOVE toGeckoEvent:&geckoEvent];
+    
+    // send event into Gecko by going directly to the
+    // the widget.
+    mGeckoChild->DispatchMouseEvent(geckoEvent);    
 }
 
 - (void)mouseEntered:(NSEvent*)theEvent
@@ -1901,13 +1894,39 @@ nsChildView::GetQuickDrawPort()
   printf("got mouse EXIT view\n");
 }
 
+- (void)otherMouseDown:(NSEvent *)theEvent
+{
+  nsMouseEvent geckoEvent;
+  geckoEvent.eventStructType = NS_MOUSE_EVENT;
+  [self convert:theEvent message:NS_MOUSE_MIDDLE_BUTTON_DOWN toGeckoEvent:&geckoEvent];
+  geckoEvent.clickCount = [theEvent clickCount];
+  
+  // send event into Gecko by going directly to the
+  // the widget.
+  mGeckoChild->DispatchMouseEvent(geckoEvent);
+  
+} // otherMouseDown
+
+
+- (void)otherMouseUp:(NSEvent *)theEvent
+{
+  nsMouseEvent geckoEvent;
+  geckoEvent.eventStructType = NS_MOUSE_EVENT;
+  [self convert:theEvent message:NS_MOUSE_MIDDLE_BUTTON_UP toGeckoEvent:&geckoEvent];
+  
+  // send event into Gecko by going directly to the
+  // the widget.
+  mGeckoChild->DispatchMouseEvent(geckoEvent);
+  
+} // mouseUp
+
 const PRInt32 kNumLines = 8;
 
 -(void)scrollWheel:(NSEvent*)theEvent
 {
   // XXXdwh. We basically always get 1 or -1 as the delta.  This is treated by 
   // Gecko as the number of lines to scroll.  We go ahead and use a 
-  // default kNumLines of 4 for now (until I learn how we can get settings from
+  // default kNumLines of 8 for now (until I learn how we can get settings from
   // the OS). --dwh
   nsMouseScrollEvent geckoEvent;
   geckoEvent.eventStructType = NS_MOUSE_SCROLL_EVENT;
@@ -1915,9 +1934,6 @@ const PRInt32 kNumLines = 8;
   geckoEvent.delta = PRInt32([theEvent deltaY])*(-kNumLines);
   geckoEvent.scrollFlags |= nsMouseScrollEvent::kIsVertical;
  
-  // obscure the cursor during a wheel scroll
-  ::ObscureCursor();
-
   // send event into Gecko by going directly to the
   // the widget.
   mGeckoChild->DispatchWindowEvent(geckoEvent);
@@ -1937,13 +1953,11 @@ const PRInt32 kNumLines = 8;
   
   if (outGeckoEvent->eventStructType != NS_KEY_EVENT) {
     NSPoint mouseLoc = [inEvent locationInWindow];
-    outGeckoEvent->refPoint.x = NS_STATIC_CAST(nscoord, mouseLoc.x);
-    outGeckoEvent->refPoint.y = NS_STATIC_CAST(nscoord, mouseLoc.y);
     
     // convert point to view coordinate system
     NSPoint localPoint = [self convertPoint:mouseLoc fromView:nil];
-    outGeckoEvent->point.x = NS_STATIC_CAST(nscoord, localPoint.x);
-    outGeckoEvent->point.y = NS_STATIC_CAST(nscoord, localPoint.y);
+    outGeckoEvent->refPoint.x = outGeckoEvent->point.x = NS_STATIC_CAST(nscoord, localPoint.x);
+    outGeckoEvent->refPoint.y = outGeckoEvent->point.y = NS_STATIC_CAST(nscoord, localPoint.y);
 
     if (outGeckoEvent->message == NS_MOUSE_SCROLL) {
       outGeckoEvent->refPoint.x = outGeckoEvent->refPoint.y = 0;
@@ -2013,11 +2027,8 @@ const PRInt32 kNumLines = 8;
       mGeckoChild->DispatchWindowEvent(geckoEvent);
     }
   }
-  else {
-    // XXXdwh. Check to see if we're a context menu key.  If so,
-    // fire a context menu event instead.
+  else
     mGeckoChild->DispatchWindowEvent(geckoEvent);
-  }
 }
 
 - (void)keyUp:(NSEvent*)theEvent;
@@ -2032,6 +2043,39 @@ const PRInt32 kNumLines = 8;
   mGeckoChild->DispatchWindowEvent(geckoEvent);
 }
 
+// This method is called when we are about to be focused.
+- (BOOL)becomeFirstResponder
+{
+  nsFocusEvent event;
+  event.eventStructType = NS_FOCUS_EVENT;
+  event.message = NS_GOTFOCUS;
+  event.widget = mGeckoChild;
+
+  //focus and blur event should go to their base widget loc
+  event.point.x = 0;
+  event.point.y = 0;
+
+  mGeckoChild->DispatchWindowEvent(event);
+  return [super becomeFirstResponder];
+}
+
+// This method is called when are are about to lose focus.
+- (BOOL)resignFirstResponder
+{
+  nsFocusEvent event;
+  event.eventStructType = NS_FOCUS_EVENT;
+  event.message = NS_LOSTFOCUS;
+  event.widget = mGeckoChild;
+
+  //focus and blur event should go to their base widget loc
+  event.point.x = 0;
+  event.point.y = 0;
+
+  mGeckoChild->DispatchWindowEvent(event);
+
+  return [super resignFirstResponder];
+}
+
 //-------------------------------------------------------------------------
 //
 // ConvertMacToRaptorKeyCode
