@@ -52,6 +52,7 @@
 #include "nsMacResources.h"
 #include "nsRegionMac.h"
 #include "nsIRollupListener.h"
+#include "nsCRT.h"
 
 #if TARGET_CARBON
 #include <CFString.h>
@@ -176,7 +177,8 @@ nsMacWindow :: DragReceiveHandler (WindowPtr theWindow, void *handlerRefCon,
 } // DragReceiveHandler
 
 
-NS_IMPL_ISUPPORTS_INHERITED3(nsMacWindow, Inherited, nsIEventSink, nsPIWidgetMac, nsPIEventSinkStandalone);
+NS_IMPL_ISUPPORTS_INHERITED4(nsMacWindow, Inherited, nsIEventSink, nsPIWidgetMac, nsPIEventSinkStandalone, 
+                                          nsIMacTextInputEventSink);
 
 
 //-------------------------------------------------------------------------
@@ -193,11 +195,12 @@ nsMacWindow::nsMacWindow() : Inherited()
   , mAcceptsActivation(PR_TRUE)
   , mIsActive(PR_FALSE)
   , mZoomOnShow(PR_FALSE)
+  , mZooming(PR_FALSE)
+  , mResizeIsFromUs(PR_FALSE)
 #if !TARGET_CARBON
   , mPhantomScrollbar(nsnull)
   , mPhantomScrollbarData(nsnull)
 #endif
-  , mResizeIsFromUs(PR_FALSE)
 {
   mMacEventHandler.reset(new nsMacEventHandler(this));
   WIDGET_SET_CLASSNAME("nsMacWindow");  
@@ -354,14 +357,18 @@ nsresult nsMacWindow::StandardCreate(nsIWidget *aParent,
 #if TARGET_CARBON
               if (nsToolkit::OnMacOSX() && aParent && (aInitData->mBorderStyle & eBorderStyle_sheet))
               {
-                  nsWindowType parentType;
-                  aParent->GetWindowType(parentType);
-                  if (parentType != eWindowType_invisible)
+                nsWindowType parentType;
+                aParent->GetWindowType(parentType);
+                if (parentType != eWindowType_invisible)
+                {
+                  // Mac OS X sheet support
+                  mIsSheet = PR_TRUE;
+                  wDefProcID = kWindowSheetProc;
+                  if (aInitData->mBorderStyle & eBorderStyle_resizeh)
                   {
-                      // Mac OS X sheet support
-                      mIsSheet = PR_TRUE;
-                      wDefProcID = kWindowSheetProc;
+                    resizable = true;
                   }
+                }
               }
               else
 #endif
@@ -541,12 +548,16 @@ nsresult nsMacWindow::StandardCreate(nsIWidget *aParent,
 
     if (mIsSheet)
     {
-        // Mac OS X sheet support
-        ::SetWindowClass(mWindowPtr, kSheetWindowClass);
-        EventTypeSpec windEventList[] = { {kEventClassWindow, kEventWindowUpdate},
-                                          {kEventClassWindow, kEventWindowDrawContent} };
-        OSStatus err1 = ::InstallWindowEventHandler ( mWindowPtr,
-            NewEventHandlerUPP(WindowEventHandler), 2, windEventList, this, NULL );
+      // Mac OS X sheet support
+      if ( resizable )
+      {
+        ::ChangeWindowAttributes ( mWindowPtr, kWindowResizableAttribute, kWindowNoAttributes );
+      }
+      ::SetWindowClass(mWindowPtr, kSheetWindowClass);
+      EventTypeSpec windEventList[] = { {kEventClassWindow, kEventWindowUpdate},
+                                        {kEventClassWindow, kEventWindowDrawContent} };
+      OSStatus err1 = ::InstallWindowEventHandler ( mWindowPtr,
+          NewEventHandlerUPP(WindowEventHandler), 2, windEventList, this, NULL );
     }
     
     // Setup the live window resizing if appropriate
@@ -819,7 +830,16 @@ NS_IMETHODIMP nsMacWindow::Show(PRBool bState)
     if ( mIsSheet && parentWindowRef ) {
         WindowPtr top = GetWindowTop(parentWindowRef);
         if (piParentWidget)
+        {
           piParentWidget->SetIgnoreDeactivate(PR_TRUE);
+          PRBool sheetFlag = PR_FALSE;
+          if (parentWindowRef &&
+            NS_SUCCEEDED(piParentWidget->GetIsSheet(&sheetFlag)) && (sheetFlag))
+          {
+            ::GetSheetWindowParent(parentWindowRef, &top);
+            ::HideSheetWindow(parentWindowRef);
+          }
+        }
         ::ShowSheetWindow(mWindowPtr, top);
         UpdateWindowMenubar(parentWindowRef, PR_FALSE);
         gEventDispatchHandler.DispatchGuiEvent(this, NS_GOTFOCUS);
@@ -853,28 +873,45 @@ NS_IMETHODIMP nsMacWindow::Show(PRBool bState)
 #if TARGET_CARBON
     // Mac OS X sheet support
     if (mIsSheet) {
+      if (piParentWidget)
+        piParentWidget->SetIgnoreDeactivate(PR_FALSE);
+
+      // get sheet's parent *before* hiding the sheet (which breaks the linkage)
+      WindowPtr sheetParent = nsnull;
+      ::GetSheetWindowParent(mWindowPtr, &sheetParent);
+
+      ::HideSheetWindow(mWindowPtr);
+
+      gEventDispatchHandler.DispatchGuiEvent(this, NS_DEACTIVATE);
+
+      // if we had several sheets open, when the last one goes away
+      // we need to ensure that the top app window is active
+      WindowPtr top = GetWindowTop(parentWindowRef);
+      piParentWidget = do_QueryInterface(parentWidget);
+
+      PRBool sheetFlag = PR_FALSE;
+      if (parentWindowRef && piParentWidget &&
+        NS_SUCCEEDED(piParentWidget->GetIsSheet(&sheetFlag)) && (sheetFlag))
+      {
+        nsCOMPtr<nsIWidget> parentWidget;
+        nsToolkit::GetTopWidget(sheetParent, getter_AddRefs(parentWidget));
+        piParentWidget = do_QueryInterface(parentWidget);
         if (piParentWidget)
-          piParentWidget->SetIgnoreDeactivate(PR_FALSE);
-        ::HideSheetWindow(mWindowPtr);
+          piParentWidget->SetIgnoreDeactivate(PR_TRUE);
+        ::ShowSheetWindow(parentWindowRef, sheetParent);
+      }
+      else if ( mAcceptsActivation ) {
+        ::ShowWindow(top);
+      }
+      else {
+        ::ShowHide(top, true);
+        ::BringToFront(top); // competes with ComeToFront, but makes popups work
+      }
+      ComeToFront();
 
-        gEventDispatchHandler.DispatchGuiEvent(this, NS_DEACTIVATE);
-
-        // if we had several sheets open, when the last one goes away
-        // we need to ensure that the top app window is active
-        WindowPtr top = GetWindowTop(parentWindowRef);
-
-        if ( mAcceptsActivation ) {
-            ::ShowWindow(top);
-        }
-        else {
-            ::ShowHide(top, true);
-            ::BringToFront(top); // competes with ComeToFront, but makes popups work
-        }
-        ComeToFront();
-
-        if (top == parentWindowRef) {
-            UpdateWindowMenubar(parentWindowRef, PR_TRUE);
-        }
+      if (top == parentWindowRef) {
+        UpdateWindowMenubar(parentWindowRef, PR_TRUE);
+      }
     }
     else
 #endif
@@ -1180,8 +1217,14 @@ NS_METHOD nsMacWindow::SetSizeMode(PRInt32 aMode)
   if (aMode == nsSizeMode_Minimized) // unlikely on the Mac
     return NS_ERROR_UNEXPECTED;
 
+  // resize during zoom may attempt to unzoom us. here's where we put a stop to that.
+  if (mZooming)
+    return NS_OK;
+
   // already done? it's bad to rezoom a window, so do nothing
   rv = nsBaseWidget::GetSizeMode(&currentMode);
+  if (currentMode == nsSizeMode_Normal && !mVisible && mZoomOnShow)
+    currentMode = nsSizeMode_Maximized;
   if (NS_SUCCEEDED(rv) && currentMode == aMode)
     return NS_OK;
 
@@ -1189,10 +1232,10 @@ NS_METHOD nsMacWindow::SetSizeMode(PRInt32 aMode)
     /* zooming on the Mac doesn't seem to work until the window is visible.
        the rest of the app is structured to zoom before the window is visible
        to avoid flashing. here's where we defeat that. */
-    if (aMode == nsSizeMode_Maximized)
-      mZoomOnShow = PR_TRUE;
+    mZoomOnShow = aMode == nsSizeMode_Maximized;
   } else {
     Rect macRect;
+    mZooming = PR_TRUE;
     rv = nsBaseWidget::SetSizeMode(aMode);
     if (NS_SUCCEEDED(rv)) {
       if (aMode == nsSizeMode_Maximized) {
@@ -1203,12 +1246,47 @@ NS_METHOD nsMacWindow::SetSizeMode(PRInt32 aMode)
       ::GetWindowPortBounds(mWindowPtr, &macRect);
       Resize(macRect.right - macRect.left, macRect.bottom - macRect.top, PR_FALSE);
     }
+    mZooming = PR_FALSE;
   }
 
   return rv;
 }
 
 
+// we're resizing. if we happen to be zoomed ("standard" state),
+// switch to the user state without changing the window size.
+void
+nsMacWindow::UserStateForResize()
+{
+  nsresult rv;
+  PRInt32  currentMode;
+
+  // but not if we're currently zooming
+  if (mZooming)
+    return;
+
+  // nothing to do if we're already in the user state
+  rv = nsBaseWidget::GetSizeMode(&currentMode);
+  if (NS_SUCCEEDED(rv) && currentMode == nsSizeMode_Normal)
+    return;
+
+  // we've just finished sizing, so make our current size our user state,
+  // and switch to the user state
+  StPortSetter portState(mWindowPtr);
+  StOriginSetter originState(mWindowPtr);
+  Rect bounds;
+
+  ::GetWindowPortBounds(mWindowPtr, &bounds);
+  ::LocalToGlobal((Point *)&bounds.top);
+  ::LocalToGlobal((Point *)&bounds.bottom);
+  ::SetWindowUserState(mWindowPtr, &bounds);
+  ::ZoomWindow(mWindowPtr, inZoomIn, false);
+
+  // finally, reset our internal state to match
+  nsBaseWidget::SetSizeMode(nsSizeMode_Normal);
+}
+
+
 //
 // CalculateAndSetZoomedSize
 //
@@ -1321,6 +1399,13 @@ nsMacWindow::SetIgnoreDeactivate(PRBool ignoreDeactivate)
   return(NS_OK);
 }
 
+NS_IMETHODIMP
+nsMacWindow::GetIsSheet(PRBool *_retval)
+{
+  *_retval = mIsSheet;
+  return(NS_OK);
+}
+
 
 //-------------------------------------------------------------------------
 //
@@ -1491,6 +1576,99 @@ NS_IMETHODIMP nsMacWindow::SetTitle(const nsString& aTitle)
 }
 
 
+#pragma mark -
+
+//
+// impleemnt nsIMacTextInputEventSink: forward to the right method in nsMacEventHandler 
+//
+
+/* OSStatus HandleUpdateActiveInputArea (in wstring text, in long textLength, in short script, in short language, in long fixLen, in voidPtr hiliteRng); */
+NS_IMETHODIMP 
+nsMacWindow::HandleUpdateActiveInputArea(const nsAString & text, 
+                                         PRInt16 script, PRInt16 language, PRInt32 fixLen, void * hiliteRng, 
+                                         OSStatus *_retval)
+{
+  *_retval = eventNotHandledErr;
+  NS_ENSURE_TRUE(mMacEventHandler.get(), NS_ERROR_FAILURE);
+  const nsPromiseFlatString& buffer = PromiseFlatString(text);
+  // ignore script and langauge information for now. 
+  nsresult res = mMacEventHandler->UnicodeHandleUpdateInputArea((PRUnichar*)buffer.get(), buffer.Length(), fixLen, (TextRangeArray*) hiliteRng);
+  // we will lost the real OSStatus for now untill we change the nsMacEventHandler
+  if (NS_SUCCEEDED(res))
+    *_retval = noErr;
+  return res;
+}
+
+/* OSStatus HandleUpdateActiveInputAreaForNonUnicode (in string text, in long textLength, in short script, in short language, in long fixLen, in voidPtr hiliteRng); */
+NS_IMETHODIMP 
+nsMacWindow::HandleUpdateActiveInputAreaForNonUnicode(const nsACString & text, 
+                                                      PRInt16 script, PRInt16 language, 
+                                                      PRInt32 fixLen, void * hiliteRng, 
+                                                      OSStatus *_retval)
+{
+  *_retval = eventNotHandledErr;
+  NS_ENSURE_TRUE(mMacEventHandler.get(), NS_ERROR_FAILURE);
+  const nsPromiseFlatCString& buffer = PromiseFlatCString(text);
+  // ignore langauge information for now. 
+  nsresult res = mMacEventHandler->HandleUpdateInputArea((char*)buffer.get(), buffer.Length(), script, fixLen, (TextRangeArray*) hiliteRng);
+  // we will lost the real OSStatus for now untill we change the nsMacEventHandler
+  if (NS_SUCCEEDED(res))
+    *_retval = noErr;
+  return res;
+}
+
+/* OSStatus HandleUnicodeForKeyEvent (in wstring text, in long textLength, in short script, in short language, in voidPtr keyboardEvent); */
+NS_IMETHODIMP 
+nsMacWindow::HandleUnicodeForKeyEvent(const nsAString & text, 
+                                      PRInt16 script, PRInt16 language, void * keyboardEvent, 
+                                      OSStatus *_retval)
+{
+  *_retval = eventNotHandledErr;
+  NS_ENSURE_TRUE(mMacEventHandler.get(), NS_ERROR_FAILURE);
+  // ignore langauge information for now. 
+  // we will lost the real OSStatus for now untill we change the nsMacEventHandler
+  EventRecord* eventPtr = (EventRecord*)keyboardEvent;
+  const nsPromiseFlatString& buffer = PromiseFlatString(text);
+  nsresult res = mMacEventHandler->HandleUKeyEvent((PRUnichar*)buffer.get(), buffer.Length(), *eventPtr);
+  // we will lost the real OSStatus for now untill we change the nsMacEventHandler
+  if(NS_SUCCEEDED(res))
+    *_retval = noErr;
+  return res;
+}
+
+/* OSStatus HandleOffsetToPos (in long offset, out short pointX, out short pointY); */
+NS_IMETHODIMP 
+nsMacWindow::HandleOffsetToPos(PRInt32 offset, PRInt16 *pointX, PRInt16 *pointY, OSStatus *_retval)
+{
+  *_retval = eventNotHandledErr;
+  NS_ENSURE_TRUE(mMacEventHandler.get(), NS_ERROR_FAILURE);
+  *pointX = *pointY = 0;
+  Point thePoint = {0,0};
+  nsresult res = mMacEventHandler->HandleOffsetToPosition(offset, &thePoint);
+  // we will lost the real OSStatus for now untill we change the nsMacEventHandler
+  if(NS_SUCCEEDED(res))
+    *_retval = noErr;
+  *pointX = thePoint.h;
+  *pointY = thePoint.v;  
+  return res;
+}
+
+/* OSStatus 
+HandlePosToOffset (in short currentPointX, in short currentPointY, out long offset, out short regionClass); */
+NS_IMETHODIMP 
+nsMacWindow::HandlePosToOffset(PRInt16 currentPointX, PRInt16 currentPointY, 
+                               PRInt32 *offset, PRInt16 *regionClass, OSStatus *_retval)
+{
+  *_retval = eventNotHandledErr;
+  NS_ENSURE_TRUE(mMacEventHandler.get(), NS_ERROR_FAILURE);
+  *_retval = noErr;
+  Point thePoint;
+  thePoint.h = currentPointX;
+  thePoint.v = currentPointY;
+  *offset = mMacEventHandler->HandlePositionToOffset(thePoint, regionClass);
+  return NS_OK;
+}
+
 #pragma mark -
 
 
