@@ -414,7 +414,7 @@ nsMenuBarX :: ExecuteCommand ( nsIContent* inDispatchTo )
 // for |outHiddenNode| if the caller doesn't care about the hidden node.
 //
 void
-nsMenuBarX :: HideItem ( nsIDOMDocument* inDoc, nsAReadableString & inID, nsIContent** outHiddenNode )
+nsMenuBarX :: HideItem ( nsIDOMDocument* inDoc, const nsAString & inID, nsIContent** outHiddenNode )
 {
   nsCOMPtr<nsIDOMElement> menuItem;
   inDoc->GetElementById(inID, getter_AddRefs(menuItem));  
@@ -760,7 +760,8 @@ nsMenuBarX::ContentChanged( nsIDocument * aDocument, nsIContent * aContent, nsIS
 }
 
 NS_IMETHODIMP
-nsMenuBarX::ContentStatesChanged( nsIDocument * aDocument, nsIContent  * aContent1, nsIContent  * aContent2)
+nsMenuBarX::ContentStatesChanged( nsIDocument * aDocument, nsIContent  * aContent1,
+                                  nsIContent  * aContent2, PRInt32 aStateMask)
 {
   return NS_OK;
 }
