@@ -71,6 +71,7 @@
 #include "nsGUIEvent.h"
 
 #include "nsDynamicMDEF.h"
+#include "nsCRT.h"
 
 
 static OSStatus InstallMyMenuEventHandler(MenuRef menuRef, void* userData, EventHandlerRef* outHandler) ;
@@ -155,7 +156,7 @@ nsMenuX::~nsMenuX()
 // Create
 //
 NS_METHOD 
-nsMenuX::Create(nsISupports * aParent, const nsAReadableString &aLabel, const nsAReadableString &aAccessKey, 
+nsMenuX::Create(nsISupports * aParent, const nsAString &aLabel, const nsAString &aAccessKey, 
                 nsIChangeManager* aManager, nsIWebShell* aShell, nsIContent* aNode )
 {
   mWebShellWeakRef = getter_AddRefs(NS_GetWeakReference(aShell));
@@ -208,7 +209,7 @@ NS_METHOD nsMenuX::GetLabel(nsString &aText)
 
 
 //-------------------------------------------------------------------------
-NS_METHOD nsMenuX::SetLabel(const nsAReadableString &aText)
+NS_METHOD nsMenuX::SetLabel(const nsAString &aText)
 {
   mLabel = aText;
 
@@ -228,7 +229,7 @@ NS_METHOD nsMenuX::GetAccessKey(nsString &aText)
 }
 
 //-------------------------------------------------------------------------
-NS_METHOD nsMenuX::SetAccessKey(const nsAReadableString &aText)
+NS_METHOD nsMenuX::SetAccessKey(const nsAString &aText)
 {
   return NS_OK;
 }
