@@ -67,7 +67,7 @@
 
 #ifndef nscore_h___
 #include "nscore.h"
-  // for |NS_..._CAST|, |NS_EXPORT|
+  // for |NS_..._CAST|, |NS_COM|
 #endif
 
 
@@ -403,16 +403,16 @@ class nsCOMPtr_base
         }
 
 #ifdef NSCAP_FEATURE_FACTOR_DESTRUCTOR
-      NS_EXPORT ~nsCOMPtr_base();
+      NS_COM ~nsCOMPtr_base();
 #else
       // Allow debug builds to link with optimized versions of nsCOMPtr-using
       // plugins (e.g., JVMs).
-      NS_EXPORT ~nsCOMPtr_base() { }
+      NS_COM ~nsCOMPtr_base() { }
 #endif
 
-      NS_EXPORT void    assign_with_AddRef( nsISupports* );
-      NS_EXPORT void    assign_from_helper( const nsCOMPtr_helper&, const nsIID& );
-      NS_EXPORT void**  begin_assignment();
+      NS_COM void    assign_with_AddRef( nsISupports* );
+      NS_COM void    assign_from_helper( const nsCOMPtr_helper&, const nsIID& );
+      NS_COM void**  begin_assignment();
 
     protected:
       nsISupports* mRawPtr;
@@ -682,31 +682,6 @@ class nsCOMPtr
 
 #endif // CANT_RESOLVE_CPP_CONST_AMBIGUITY
 
-      // This is going to become private soon, once all users of
-      // nsCOMPtr stop using it.  It may even become:
-      // void operator&() const {}
-    private:
-      const nsCOMPtr<T>*
-      operator&() const
-          // This is private to prevent accidental use of |operator&|
-          // instead of |getter_AddRefs|, which can happen if the result
-          // is cast.
-
-          // To pass an nsCOMPtr as an out parameter to a function, either
-          // (preferably) pass by reference or use |address_of|.
-        {
-          return this;
-        }
-
-      nsCOMPtr<T>*
-      operator&()
-          // This version is also needed so things will compile before
-          // all the uses are removed and we make it private.  After it's
-          // private, we won't need two anymore.
-        {
-          return this;
-        }
-
     public:
       nsDerivedSafe<T>&
       operator*() const
@@ -903,31 +878,6 @@ class nsCOMPtr<nsISupports>
 
 #endif // CANT_RESOLVE_CPP_CONST_AMBIGUITY
 
-      // This is going to become private soon, once all users of
-      // nsCOMPtr stop using it.  It may even become:
-      // void operator&() const {}
-    private:
-      const nsCOMPtr<nsISupports>*
-      operator&() const
-          // This is private to prevent accidental use of |operator&|
-          // instead of |getter_AddRefs|, which can happen if the result
-          // is cast.
-
-          // To pass an nsCOMPtr as an out parameter to a function, either
-          // (preferably) pass by reference or use |address_of|.
-        {
-          return this;
-        }
-
-      nsCOMPtr<nsISupports>*
-      operator&()
-          // This version is also needed so things will compile before
-          // all the uses are removed and we make it private.  After it's
-          // private, we won't need two anymore.
-        {
-          return this;
-        }
-
     public:
 
       nsDerivedSafe<nsISupports>&
