@@ -40,6 +40,10 @@
 /* Implementation of xptiInterfaceInfoManager. */
 
 #include "xptiprivate.h"
+#include "nsDependentString.h"
+#include "nsString.h"
+
+#define NS_ZIPLOADER_CONTRACTID NS_XPTLOADER_CONTRACTID_PREFIX "zip"
 
 NS_IMPL_THREADSAFE_ISUPPORTS2(xptiInterfaceInfoManager, 
                               nsIInterfaceInfoManager,
@@ -125,7 +129,7 @@ xptiInterfaceInfoManager::xptiInterfaceInfoManager(nsISupportsArray* aSearchPath
     {
         mStatsLogFile = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);         
         if(mStatsLogFile && 
-           NS_SUCCEEDED(mStatsLogFile->InitWithPath(statsFilename)))
+           NS_SUCCEEDED(mStatsLogFile->InitWithNativePath(nsDependentCString(statsFilename))))
         {
             printf("* Logging xptinfo stats to: %s\n", statsFilename);
         }
@@ -141,7 +145,7 @@ xptiInterfaceInfoManager::xptiInterfaceInfoManager(nsISupportsArray* aSearchPath
     {
         mAutoRegLogFile = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);         
         if(mAutoRegLogFile && 
-           NS_SUCCEEDED(mAutoRegLogFile->InitWithPath(autoRegFilename)))
+           NS_SUCCEEDED(mAutoRegLogFile->InitWithNativePath(nsDependentCString(autoRegFilename))))
         {
             printf("* Logging xptinfo autoreg to: %s\n", autoRegFilename);
         }
@@ -158,10 +162,6 @@ xptiInterfaceInfoManager::~xptiInterfaceInfoManager()
     // We only do this on shutdown of the service.
     mWorkingSet.InvalidateInterfaceInfos();
 
-#ifdef XPTI_HAS_ZIP_SUPPORT
-    xptiZipLoader::Shutdown();
-#endif /* XPTI_HAS_ZIP_SUPPORT */
-
     if(mResolveLock)
         PR_DestroyLock(mResolveLock);
     if(mAutoRegLock)
@@ -216,6 +216,7 @@ AppendFromDirServiceList(const char* codename, nsISupportsArray* aPath)
         if(!dir || !aPath->AppendElement(dir))
             return PR_FALSE;
     }
+
     return PR_TRUE;
 }        
 
@@ -330,11 +331,11 @@ xptiInterfaceInfoManager::BuildFileList(nsISupportsArray* aSearchPath,
                 continue;
             }
      
-            nsXPIDLCString name;
-            if(NS_FAILED(file->GetLeafName(getter_Copies(name))))
+            nsCAutoString name;
+            if(NS_FAILED(file->GetNativeLeafName(name)))
                 return PR_FALSE;
 
-            if(xptiFileType::IsUnknown(name))
+            if(xptiFileType::IsUnknown(name.get()))
                 continue;
 
             LOG_AUTOREG(("found file: %s\n", name.get()));
@@ -430,14 +431,13 @@ xptiInterfaceInfoManager::LoadFile(const xptiTypelib& aTypelibRecord,
                                     getter_AddRefs(file))) || !file)
         return PR_FALSE;
 
-    if(NS_FAILED(file->Append(fileRecord->GetName())))
+    if(NS_FAILED(file->AppendNative(nsDependentCString(fileRecord->GetName()))))
         return PR_FALSE;
 
     XPTHeader* header;
 
     if(aTypelibRecord.IsZip())
     {
-#ifdef XPTI_HAS_ZIP_SUPPORT
         zipItem = &aWorkingSet->GetZipItemAt(aTypelibRecord.GetZipItemIndex());
         
         // See the big comment below in the 'non-zip' case...
@@ -452,10 +452,26 @@ xptiInterfaceInfoManager::LoadFile(const xptiTypelib& aTypelibRecord,
         }
         
         LOG_LOAD(("# loading zip item %s::%s\n", fileRecord->GetName(), zipItem->GetName()));
-        header = xptiZipLoader::ReadXPTFileFromZip(file, zipItem->GetName(), aWorkingSet);
-#else
-        header = nsnull;
-#endif /* XPTI_HAS_ZIP_SUPPORT */
+        
+        nsCOMPtr<nsIXPTLoader> loader =
+            do_GetService(NS_ZIPLOADER_CONTRACTID);
+
+        if (loader) {
+            nsresult rv;
+            
+            nsCOMPtr<nsIInputStream> stream;
+            rv = loader->LoadEntry(file, zipItem->GetName(),
+                                   getter_AddRefs(stream));
+
+            if (NS_FAILED(rv))
+                return PR_FALSE;
+            
+            header =
+                xptiZipLoader::ReadXPTFileFromInputStream(stream, aWorkingSet);
+        } else {
+            header = nsnull;
+            NS_WARNING("Could not load XPT Zip loader");
+        }
     } 
     else
     {
@@ -609,27 +625,27 @@ xptiSortFileList(const void * p1, const void *p2, void * closure)
     nsILocalFile* pFile2 = *((nsILocalFile**) p2);
     SortData* data = (SortData*) closure;
     
-    nsXPIDLCString name1;
-    nsXPIDLCString name2;
+    nsCAutoString name1;
+    nsCAutoString name2;
         
-    if(NS_FAILED(pFile1->GetLeafName(getter_Copies(name1))))
+    if(NS_FAILED(pFile1->GetNativeLeafName(name1)))
     {
         NS_ERROR("way bad, with no happy out!");
         return 0;    
     }    
-    if(NS_FAILED(pFile2->GetLeafName(getter_Copies(name2))))
+    if(NS_FAILED(pFile2->GetNativeLeafName(name2)))
     {
         NS_ERROR("way bad, with no happy out!");
         return 0;    
     }    
 
-    int index1 = IndexOfFileWithName(name1, data->mWorkingSet); 
-    int index2 = IndexOfFileWithName(name2, data->mWorkingSet); 
+    int index1 = IndexOfFileWithName(name1.get(), data->mWorkingSet); 
+    int index2 = IndexOfFileWithName(name2.get(), data->mWorkingSet); 
    
     // Get these now in case we need them later.
-    PRBool isXPT1 = xptiFileType::IsXPT(name1);
-    PRBool isXPT2 = xptiFileType::IsXPT(name2);
-    int nameOrder = PL_strcmp(name1, name2);
+    PRBool isXPT1 = xptiFileType::IsXPT(name1.get());
+    PRBool isXPT2 = xptiFileType::IsXPT(name2.get());
+    int nameOrder = Compare(name1, name2);
     
     // both in workingSet, preserve old order
     if(index1 != -1 && index2 != -1)
@@ -771,12 +787,12 @@ xptiInterfaceInfoManager::DetermineAutoRegStrategy(nsISupportsArray* aSearchPath
 
             PRInt64 size;
             PRInt64 date;
-            nsXPIDLCString name;
+            nsCAutoString name;
             PRUint32 directory;
 
             if(NS_FAILED(file->GetFileSize(&size)) ||
                NS_FAILED(file->GetLastModifiedTime(&date)) ||
-               NS_FAILED(file->GetLeafName(getter_Copies(name))) ||
+               NS_FAILED(file->GetNativeLeafName(name)) ||
                !aWorkingSet->FindDirectoryOfFile(file, &directory))
             {
                 NS_ERROR("unexpected!");
@@ -788,7 +804,7 @@ xptiInterfaceInfoManager::DetermineAutoRegStrategy(nsISupportsArray* aSearchPath
                 xptiFile& target = aWorkingSet->GetFileAt(k);
                 
                 if(directory == target.GetDirectory() &&
-                   0 == PL_strcmp(name, target.GetName()))
+                   name.Equals(target.GetName()))
                 {
                     if(nsInt64(size) != target.GetSize() ||
                        nsInt64(date) != target.GetDate())
@@ -820,18 +836,18 @@ xptiInterfaceInfoManager::DetermineAutoRegStrategy(nsISupportsArray* aSearchPath
                 aFileList->QueryElementAt(k, NS_GET_IID(nsILocalFile), getter_AddRefs(file));
                 NS_ASSERTION(file, "loser!");
                 
-                nsXPIDLCString name;
+                nsCAutoString name;
                 PRInt64 size;
                 PRInt64 date;
                 if(NS_FAILED(file->GetFileSize(&size)) ||
                    NS_FAILED(file->GetLastModifiedTime(&date)) ||
-                   NS_FAILED(file->GetLeafName(getter_Copies(name))))
+                   NS_FAILED(file->GetNativeLeafName(name)))
                 {
                     NS_ERROR("unexpected!");
                     return FULL_VALIDATION_REQUIRED;
                 }    
             
-                PRBool sameName = (0 == PL_strcmp(name, target.GetName()));
+                PRBool sameName = name.Equals(target.GetName());
                 if(sameName)
                 {
                     if(nsInt64(size) != target.GetSize() ||
@@ -887,20 +903,20 @@ xptiInterfaceInfoManager::AddOnlyNewFilesFromFileList(nsISupportsArray* aSearchP
     {
         nsILocalFile* file = orderedFileArray[i];
 
-        nsXPIDLCString name;
+        nsCAutoString name;
         PRInt64 size;
         PRInt64 date;
         PRUint32 dir;
         if(NS_FAILED(file->GetFileSize(&size)) ||
            NS_FAILED(file->GetLastModifiedTime(&date)) ||
-           NS_FAILED(file->GetLeafName(getter_Copies(name))) ||
+           NS_FAILED(file->GetNativeLeafName(name)) ||
            !aWorkingSet->FindDirectoryOfFile(file, &dir))
         {
             return PR_FALSE;
         }    
     
 
-        if(xptiWorkingSet::NOT_FOUND != aWorkingSet->FindFile(dir, name))
+        if(xptiWorkingSet::NOT_FOUND != aWorkingSet->FindFile(dir, name.get()))
         {
             // This file was found in the working set, so skip it.       
             continue;
@@ -910,7 +926,7 @@ xptiInterfaceInfoManager::AddOnlyNewFilesFromFileList(nsISupportsArray* aSearchP
 
         xptiFile fileRecord;
         fileRecord = xptiFile(nsInt64(size), nsInt64(date), dir,
-                              name, aWorkingSet);
+                              name.get(), aWorkingSet);
 
         if(xptiFileType::IsXPT(fileRecord.GetName()))
         {
@@ -964,20 +980,29 @@ xptiInterfaceInfoManager::AddOnlyNewFilesFromFileList(nsISupportsArray* aSearchP
             // This will correspond to typelibRecord above.
             aWorkingSet->AppendFile(fileRecord);
         }
-#ifdef XPTI_HAS_ZIP_SUPPORT
-        else // It is a zip file, Oh boy!
+        else // its another kind of archive
         {
-            if(!xptiZipLoader::EnumerateZipEntries(file, 
-                          NS_STATIC_CAST(xptiEntrySink*, this), 
-                          aWorkingSet))
-            {
-                return PR_FALSE;    
+            nsCOMPtr<nsIXPTLoader> loader =
+                do_GetService(NS_ZIPLOADER_CONTRACTID);
+            
+            if (loader) {
+                nsresult rv;
+                
+                nsCOMPtr<nsIXPTLoaderSink> sink =
+                    new xptiZipLoaderSink(this, aWorkingSet);
+                if (!sink)
+                    return PR_FALSE;
+                
+                rv = loader->EnumerateEntries(file, sink);
+                if (NS_FAILED(rv))
+                    return PR_FALSE;
+                // This will correspond to typelibRecord used in
+                // xptiInterfaceInfoManager::FoundEntry.
+                aWorkingSet->AppendFile(fileRecord);
+            } else {
+                NS_WARNING("Could not load XPT Zip loader");
             }
-            // This will correspond to typelibRecord used in
-            // xptiInterfaceInfoManager::FoundEntry.
-            aWorkingSet->AppendFile(fileRecord);
         }
-#endif /* XPTI_HAS_ZIP_SUPPORT */
     }
 
     return PR_TRUE;
@@ -1028,13 +1053,13 @@ xptiInterfaceInfoManager::DoFullValidationMergeFromFileList(nsISupportsArray* aS
     {
         nsILocalFile* file = orderedFileArray[i];
 
-        nsXPIDLCString name;
+        nsCAutoString name;
         PRInt64 size;
         PRInt64 date;
         PRUint32 dir;
         if(NS_FAILED(file->GetFileSize(&size)) ||
            NS_FAILED(file->GetLastModifiedTime(&date)) ||
-           NS_FAILED(file->GetLeafName(getter_Copies(name))) ||
+           NS_FAILED(file->GetNativeLeafName(name)) ||
            !aWorkingSet->FindDirectoryOfFile(file, &dir))
         {
             return PR_FALSE;
@@ -1044,7 +1069,7 @@ xptiInterfaceInfoManager::DoFullValidationMergeFromFileList(nsISupportsArray* aS
     
         xptiFile fileRecord;
         fileRecord = xptiFile(nsInt64(size), nsInt64(date), dir,
-                              name, aWorkingSet);
+                              name.get(), aWorkingSet);
 
 //        printf("* found %s\n", fileRecord.GetName());
 
@@ -1101,30 +1126,59 @@ xptiInterfaceInfoManager::DoFullValidationMergeFromFileList(nsISupportsArray* aS
             // This will correspond to typelibRecord above.
             aWorkingSet->AppendFile(fileRecord);
         }
-#ifdef XPTI_HAS_ZIP_SUPPORT
-        else // It is a zip file, Oh boy!
+
+        else
         {
-            if(!xptiZipLoader::EnumerateZipEntries(file, 
-                          NS_STATIC_CAST(xptiEntrySink*, this), 
-                          aWorkingSet))
-            {
-                return PR_FALSE;    
+            nsCOMPtr<nsIXPTLoader> loader =
+                do_GetService(NS_ZIPLOADER_CONTRACTID);
+            
+            if (loader) {
+                nsresult rv;
+                
+                nsCOMPtr<nsIXPTLoaderSink> sink =
+                    new xptiZipLoaderSink(this, aWorkingSet);
+                if (!sink)
+                    return PR_FALSE;
+                
+                rv = loader->EnumerateEntries(file, sink);
+                if (NS_FAILED(rv))
+                    return PR_FALSE;
+                // This will correspond to typelibRecord used in
+                // xptiInterfaceInfoManager::FoundEntry.
+                aWorkingSet->AppendFile(fileRecord);
+            } else {
+                NS_WARNING("Could not load XPT Zip loader");
             }
-            // This will correspond to typelibRecord used in
-            // xptiInterfaceInfoManager::FoundEntry.
-            aWorkingSet->AppendFile(fileRecord);
         }
-#endif /* XPTI_HAS_ZIP_SUPPORT */
     }
     return PR_TRUE;
 }        
 
+NS_IMPL_ISUPPORTS1(xptiZipLoaderSink, nsIXPTLoaderSink)
+
+// implement nsIXPTLoader
+NS_IMETHODIMP
+xptiZipLoaderSink::FoundEntry(const char* entryName,
+                              PRInt32 index,
+                              nsIInputStream *aStream)
+{
+    XPTHeader *header =
+        xptiZipLoader::ReadXPTFileFromInputStream(aStream, mWorkingSet);
+    if (!header)
+        return NS_ERROR_OUT_OF_MEMORY;
+    
+    if (!mManager->FoundZipEntry(entryName, index, header, mWorkingSet))
+        return NS_ERROR_FAILURE;
+    
+    return NS_OK;
+}
+
 // implement xptiEntrySink
 PRBool 
-xptiInterfaceInfoManager::FoundEntry(const char* entryName,
-                                     int index,
-                                     XPTHeader* header,
-                                     xptiWorkingSet* aWorkingSet)
+xptiInterfaceInfoManager::FoundZipEntry(const char* entryName,
+                                        int index,
+                                        XPTHeader* header,
+                                        xptiWorkingSet* aWorkingSet)
 {
 
     NS_ASSERTION(entryName, "loser!");
@@ -1490,8 +1544,8 @@ xptiInterfaceInfoManager::DEBUG_DumpFileList(nsISupportsArray* aFileList)
         if(!file)
             return PR_FALSE;
 
-        nsXPIDLCString name;
-        if(NS_FAILED(file->GetLeafName(getter_Copies(name))))
+        nsCAutoString name;
+        if(NS_FAILED(file->GetNativeLeafName(name)))
             return PR_FALSE;
 
         printf("* found %s\n", name.get());
@@ -1520,8 +1574,8 @@ xptiInterfaceInfoManager::DEBUG_DumpFileArray(nsILocalFile** aFileArray,
     {
         nsILocalFile* file = aFileArray[i];
     
-        nsXPIDLCString name;
-        if(NS_FAILED(file->GetLeafName(getter_Copies(name))))
+        nsCAutoString name;
+        if(NS_FAILED(file->GetNativeLeafName(name)))
             return PR_FALSE;
 
         printf("found file: %s\n", name.get());
