@@ -386,6 +386,12 @@ nsEventQueueImpl::ProcessPendingEvents()
   }
 #endif
   PL_ProcessPendingEvents(mEventQueue);
+
+  // if we're no longer accepting events and there are still events in the
+  // queue, then process remaining events.
+  if (!mAcceptingEvents && PL_EventAvailable(mEventQueue))
+      PL_ProcessPendingEvents(mEventQueue);
+
   CheckForDeactivation();
 
   if (mElderQueue) {
@@ -566,8 +572,6 @@ nsEventQueueImpl::GetYoungestActive(nsIEventQueue **aQueue)
   if (!answer) {
     if (mAcceptingEvents && mCouldHaveEvents)
       answer = NS_STATIC_CAST(nsIEventQueue *, this);
-    else
-      CheckForDeactivation();
   }
   *aQueue = answer;
   NS_IF_ADDREF(*aQueue);
