@@ -51,6 +51,12 @@ var formIndex = 0;
 var imageIndex = 0;
 var frameCount = 0;
 
+const COPYCOL_NONE = -1;
+const COPYCOL_META_CONTENT = 1;
+const COPYCOL_FORM_ACTION = 3;
+const COPYCOL_LINK_ADDRESS = 2;
+const COPYCOL_IMAGE_ADDRESS = 1;
+
 // a number of services I'll need later
 // the cache services
 const nsICacheService = Components.interfaces.nsICacheService;
@@ -62,6 +68,16 @@ var ftpCacheSession = cacheService.createSession("FTP", 0, true);
 const nsIScriptableDateFormat = Components.interfaces.nsIScriptableDateFormat;
 var dateService = Components.classes["@mozilla.org/intl/scriptabledateformat;1"].getService(nsIScriptableDateFormat);
 
+// clipboard helper
+try
+{
+  const gClipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
+}
+catch(e)
+{
+  // do nothing, later code will handle the error
+}
+
 // namespaces, don't need all of these yet...
 const XLinkNS = "http://www.w3.org/1999/xlink";
 const XULNS   = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
@@ -94,8 +110,12 @@ function onLoadPageInfo()
   } 
   else 
   {
-    theWindow = window.opener.frames[0];
+    if ("gBrowser" in window.opener)
+      theWindow = window.opener.gBrowser.contentWindow;
+    else
+      theWindow = window.opener.frames[0];
     theDocument = theWindow.document;
+
     docTitle = theBundle.getString("pageInfo.title");
   }
 
@@ -117,7 +137,7 @@ function onLoadPageInfo()
 
     if (tabName)
     {
-      var tabControl = document.getElementById("tabcontrol");
+      var tabControl = document.getElementById("tabbox");
       var tab = document.getElementById(tabName);
 
       if (tabControl && tab)
@@ -149,10 +169,10 @@ function makeGeneralTab()
   
   // get the meta tags
   var metaNodes = theDocument.getElementsByTagName("meta");
-  var metaOutliner = document.getElementById("metaoutliner");
-  var metaView = new pageInfoOutlinerView(["meta-name","meta-content"]);
+  var metaTree = document.getElementById("metatree");
+  var metaView = new pageInfoTreeView(["meta-name","meta-content"], COPYCOL_META_CONTENT);
 
-  metaOutliner.outlinerBoxObject.view = metaView;
+  metaTree.treeBoxObject.view = metaView;
 
   var length = metaNodes.length;
   for (var i = 0; i < length; i++)
@@ -180,7 +200,7 @@ function makeGeneralTab()
 
   try
   {
-    var cacheEntryDescriptor = httpCacheSession.openCacheEntry(url, Components.interfaces.nsICache.ACCESS_READ, true);
+    var cacheEntryDescriptor = httpCacheSession.openCacheEntry(url, Components.interfaces.nsICache.ACCESS_READ, false);
     if(cacheEntryDescriptor)
     { 
       switch(cacheEntryDescriptor.deviceID)
@@ -198,7 +218,7 @@ function makeGeneralTab()
 
       pageSize = cacheEntryDescriptor.dataSize;
       kbSize = pageSize / 1024;
-      sizeText = pageSize; // + " (" + 0 + "Kb)";
+      sizeText = theBundle.getFormattedString("generalSize", [Math.round(kbSize*100)/100, pageSize]);
 
       expirationText = formatDate(cacheEntryDescriptor.expirationTime*1000, notSet);
     }
@@ -207,7 +227,7 @@ function makeGeneralTab()
   {
     try
     {
-      cacheEntryDescriptor = ftpCacheSession.openCacheEntry(url, Components.interfaces.nsICache.ACCESS_READ, true);
+      cacheEntryDescriptor = ftpCacheSession.openCacheEntry(url, Components.interfaces.nsICache.ACCESS_READ, false);
       if (cacheEntryDescriptor)
       {
         switch(cacheEntryDescriptor.deviceID)
@@ -225,7 +245,7 @@ function makeGeneralTab()
 
         pageSize = cacheEntryDescriptor.dataSize;
         kbSize = pageSize / 1024;
-        sizeText = pageSize; // + " (" + 0 + "Kb)";
+        sizeText = theBundle.getFormattedString("generalSize", [Math.round(kbSize*100)/100, pageSize]);
 
         expirationText = formatDate(cacheEntryDescriptor.expirationTime*1000, notSet);
       }
@@ -243,14 +263,13 @@ function makeGeneralTab()
 //******** Form Stuff
 function makeFormTab()
 {
-  var formOutliner = document.getElementById("formoutliner");
+  var formTree = document.getElementById("formtree");
   var formPreview = document.getElementById("formpreview");
   
-  var formView = new pageInfoOutlinerView(["form-number","form-name","form-action","form-method"]);
-  var fieldView = new pageInfoOutlinerView(["field-number","field-label","field-field","field-type","field-value"]);
-  formOutliner.outlinerBoxObject.view = formView;
-  formPreview.outlinerBoxObject.view = fieldView;
-
+  var formView = new pageInfoTreeView(["form-number","form-name","form-method","form-action"], COPYCOL_FORM_ACTION);
+  var fieldView = new pageInfoTreeView(["field-number","field-label","field-field","field-type","field-value"], COPYCOL_NONE);
+  formTree.treeBoxObject.view = formView;
+  formPreview.treeBoxObject.view = fieldView;
   formList = grabAllForms(theWindow, theDocument);
   formIndex = 0;
 
@@ -258,7 +277,7 @@ function makeFormTab()
   for (var i = 0; i < length; i++)
   {
     var elem = formList[i];
-    formView.addRow([++formIndex, elem.name, elem.method, elem.action]);
+    formView.addRow([++formIndex, elem.name, elem.method, elem.getAttribute("action")]);  // use getAttribute() because of bug 122128
   }
   formView.rowCountChanged(0, length);
 
@@ -279,21 +298,24 @@ function grabAllForms(aWindow, aDocument)
     }
   }
 
-  return theList.concat(aDocument.forms);
+  if ("forms" in aDocument)
+    return theList.concat(aDocument.forms);
+  else
+    return theList.concat(aDocument.getElementsByTagNameNS(XHTMLNS, "form"));
 }
 
 function onFormSelect()
 {
   var theBundle = document.getElementById("pageinfobundle");
-  var formOutliner = document.getElementById("formoutliner");
-  var formView = formOutliner.outlinerBoxObject.view;
+  var formTree = document.getElementById("formtree");
+  var formView = formTree.treeBoxObject.view;
   if (!formView.rowCount) return;
 
   if (formView.selection.count == 1)
   {
     var formPreview = document.getElementById("formpreview");
-    var fieldView = new pageInfoOutlinerView(["field-number","field-label","field-field","field-type","field-value"]);
-    formPreview.outlinerBoxObject.view = fieldView;
+    var fieldView = new pageInfoTreeView(["field-number","field-label","field-field","field-type","field-value"], COPYCOL_NONE);
+    formPreview.treeBoxObject.view = fieldView;
 
     var clickedRow = formView.selection.currentIndex;
     var formnum = formView.getCellText(clickedRow, "form-number");
@@ -369,8 +391,7 @@ function findFirstControl(node)
     }     
   }
 
-  var nodeFilter = new FormControlFilter;
-  var iterator = theDocument.createTreeWalker(node, NodeFilter.SHOW_ELEMENT, nodeFilter, true);
+  var iterator = theDocument.createTreeWalker(node, NodeFilter.SHOW_ELEMENT, FormControlFilter, true);
 
   return iterator.nextNode();
 }
@@ -380,10 +401,10 @@ function makeLinkTab()
 {
   //var start = new Date();
   var theBundle = document.getElementById("pageinfobundle");
-  var linkOutliner = document.getElementById("linkoutliner");
+  var linkTree = document.getElementById("linktree");
 
-  var linkView = new pageInfoOutlinerView(["link-number","link-name","link-address","link-type"]);
-  linkOutliner.outlinerBoxObject.view = linkView;
+  var linkView = new pageInfoTreeView(["link-number","link-name","link-address","link-type"], COPYCOL_LINK_ADDRESS);
+  linkTree.treeBoxObject.view = linkView;
 
   linkList = grabAllLinks(theWindow, theDocument);
 
@@ -394,6 +415,7 @@ function makeLinkTab()
   var linkRel = theBundle.getString("linkRel");
   var linkStylesheet = theBundle.getString("linkStylesheet");
   var linkRev = theBundle.getString("linkRev");
+  var linkX = theBundle.getString("linkX");
 
   var linktext = null;
   linkIndex = 0;
@@ -412,7 +434,7 @@ function makeLinkTab()
         linkView.addRow([++linkIndex, elem.alt, elem.href, linkArea]);
         break;
       case "input":
-        linkView.addRow([++linkIndex, elem.value || linkSubmit, elem.form.action, linkSubmission]);
+        linkView.addRow([++linkIndex, elem.value || linkSubmit, elem.form.getAttribute("action"), linkSubmission]); // use getAttribute() due to bug 122128
         break;
       case "link":
         if (elem.rel)
@@ -428,7 +450,13 @@ function makeLinkTab()
         linkView.addRow([++linkIndex, elem.rel || elem.rev, elem.href, linktext]);
         break;
       default:
-        dump("Page Info - makeLinkTab(): Hey, that's an odd one! ("+elem+")");
+        if (elem.hasAttributeNS(XLinkNS, "href"))
+        {
+          linktext = getValueText(elem);
+          linkView.AddRow([++linkIndex, linktext, elem.href, linkX]);
+        }
+        else
+          dump("Page Info - makeLinkTab(): Hey, that's an odd one! ("+elem+")");
         break;
     }
   }
@@ -460,9 +488,38 @@ function grabAllLinks(aWindow,aDocument)
     if (inputList[i].type.toLowerCase() == "submit")
       theList = theList.concat(inputList[i]);
 
-  return theList.concat(aDocument.links);
+  theList = theList.concat(grabAllXLinks(aDocument));
+
+  if ("links" in aDocument)
+    return theList.concat(aDocument.links);
+  else
+    return theList.concat(aDocument.getElementsByTagNameNS(XHTMLNS, "a"));
 }
 
+function grabAllXLinks(aDocument)
+{
+  function XLinkFilter() 
+  {
+    this.acceptNode = function(aDocument)
+    {
+      return (aDocument.hasAttributeNS(XLinkNS, "href")) ? NodeFilter.FILTER_ACCEPT : NodeFilter.FILTER_SKIP;
+    }     
+  }
+
+  var nodeFilter = new XLinkFilter;
+  var iterator = aDocument.createTreeWalker(aDocument, NodeFilter.SHOW_ELEMENT, nodeFilter, true);
+// why doesn't this work? the same thing works in findFirstControl() :P
+//  var iterator = aDocument.createTreeWalker(aDocument, NodeFilter.SHOW_ELEMENT, XLinkFilter, true);
+
+  var theList = new Array();
+
+  while(iterator.nextNode())
+    theList = theList.concat(iterator.currentNode);
+
+  return theList;
+}
+
+
 function openURL(target)
 {
   var url = target.parentNode.childNodes[2].value;
@@ -473,10 +530,10 @@ function openURL(target)
 function makeMediaTab()
 {
   var theBundle = document.getElementById("pageinfobundle");
-  var imageOutliner = document.getElementById("imageoutliner");
+  var imageTree = document.getElementById("imagetree");
 
-  var imageView = new pageInfoOutlinerView(["image-number","image-address","image-type"]);
-  imageOutliner.outlinerBoxObject.view = imageView;
+  var imageView = new pageInfoTreeView(["image-number","image-address","image-type"], COPYCOL_IMAGE_ADDRESS);
+  imageTree.treeBoxObject.view = imageView;
 
   imageList = grabAllMedia(theWindow, theDocument);
 
@@ -552,22 +609,56 @@ function grabAllMedia(aWindow, aDocument)
     if(linkList[i].rel.match(/\bicon\b/i))
       theList = theList.concat(linkList[i]);
 
-  return theList.concat(aDocument.images);
+  if ("images" in aDocument)
+    return theList.concat(aDocument.images);
+  else
+    return theList.concat(aDocument.getElementsByTagNameNS(XHTMLNS, "img"));
 }  
 
+function getSource( item )
+{
+  // Return the correct source without strict warnings
+  if (item.href != null) {
+    return item.href;
+  } else if (item.src != null) {
+    return item.src;
+  }
+  return null;
+}
+
+function getSelectedItem(tree)
+{
+  var view = tree.treeBoxObject.view;
+  if (!view.rowCount) return null;
+
+  // Only works if only one item is selected
+  var clickedRow = tree.treeBoxObject.selection.currentIndex;
+  var lineNum = view.getCellText(clickedRow, "image-number");
+  return imageList[lineNum - 1];
+}
+
+function saveMedia()
+{
+  var tree = document.getElementById("imagetree");
+  var item = getSelectedItem(tree);
+  var url = getAbsoluteURL(getSource(item), item);
+
+  if (url) {
+    saveURL(url, null, 'SaveImageTitle', false );
+  }
+}
+
 function onImageSelect()
 {
-  var outliner = document.getElementById("imageoutliner");
-  var preview = document.getElementById("imagepreview");
+  var tree = document.getElementById("imagetree");
+  var saveAsButton = document.getElementById("imagesaveasbutton");
 
-  if (outliner.outlinerBoxObject.selection.count == 1)
+  if (tree.treeBoxObject.selection.count == 1)
   {
-    var view = outliner.outlinerBoxObject.view;
-    if (!view.rowCount) return;
-    var clickedRow = outliner.outlinerBoxObject.selection.currentIndex;
-    var lineNum = view.getCellText(clickedRow, "image-number");
-
-    makePreview(imageList[lineNum-1]);
+    makePreview(getSelectedItem(tree));
+    saveAsButton.setAttribute("disabled", "false");
+  } else {
+    saveAsButton.setAttribute("disabled", "true");
   }
 }
 
@@ -580,6 +671,7 @@ function makePreview(item)
   var url = ("src" in item && item.src) || ("code" in item && item.code) || ("data" in item && item.data) || ("href" in item && item.href) || unknown;  // it better have at least one of those...
   document.getElementById("imageurltext").value = url;
   document.getElementById("imagetitletext").value = item.title || notSet;
+  document.getElementById("imagealttext").value = ("alt" in item && item.alt) || getValueText(item) || notSet;
   document.getElementById("imagelongdesctext").value = ("longDesc" in item && item.longDesc) || notSet;
 
   // find out the mime type
@@ -598,7 +690,7 @@ function makePreview(item)
 
   try
   {
-    var cacheEntryDescriptor = httpCacheSession.openCacheEntry(url, Components.interfaces.nsICache.ACCESS_READ, true);   // open for READ, in blocking mode
+    var cacheEntryDescriptor = httpCacheSession.openCacheEntry(url, Components.interfaces.nsICache.ACCESS_READ, false);   // open for READ, in non-blocking mode
     if (cacheEntryDescriptor)
     {
       switch(cacheEntryDescriptor.deviceID)
@@ -614,7 +706,9 @@ function makePreview(item)
           break;
       }
 
-      sizeText = cacheEntryDescriptor.dataSize;
+      pageSize = cacheEntryDescriptor.dataSize;
+      kbSize = pageSize / 1024;
+      sizeText = theBundle.getFormattedString("generalSize", [Math.round(kbSize*100)/100, pageSize]);
 
       expirationText = formatDate(cacheEntryDescriptor.expirationTime*1000, notSet);
     }
@@ -623,7 +717,7 @@ function makePreview(item)
   {
     try
     {
-      cacheEntryDescriptor = ftpCacheSession.openCacheEntry(url, Components.interfaces.nsICache.ACCESS_READ, true);   // open for READ, in blocking mode
+      cacheEntryDescriptor = ftpCacheSession.openCacheEntry(url, Components.interfaces.nsICache.ACCESS_READ, false);   // open for READ, in non-blocking mode
       if (cacheEntryDescriptor)
       {
         switch(cacheEntryDescriptor.deviceID)
@@ -639,7 +733,9 @@ function makePreview(item)
             break;
         }
 
-        sizeText = cacheEntryDescriptor.dataSize;
+        pageSize = cacheEntryDescriptor.dataSize;
+        kbSize = pageSize / 1024;
+        sizeText = theBundle.getFormattedString("generalSize", [Math.round(kbSize*100)/100, pageSize]);
 
         expirationText = formatDate(cacheEntryDescriptor.expirationTime*1000, notSet);
       }
@@ -653,15 +749,10 @@ function makePreview(item)
   document.getElementById("imageexpirestext").value = expirationText;
   document.getElementById("imagesizetext").value = sizeText;
 
-  // perhaps these can be done in the future
-  //document.getElementById("imageplugintext").value = "--";
-  //document.getElementById("imagecharsettext").value = "--";
-
-  document.getElementById("imagewidth").value = ("width" in item && item.width) || unknown;
-  document.getElementById("imageheight").value = ("height" in item && item.height) || unknown;
-
-  // also can't be done at the moment
-  //document.getElementById("imageencryptiontext").value = "--";
+  var width = ("width" in item && item.width) || "";
+  var height = ("height" in item && item.height) || "";
+  document.getElementById("imagewidth").value = theBundle.getFormattedString("mediaWidth", [width]);
+  document.getElementById("imageheight").value = theBundle.getFormattedString("mediaHeight", [height]);
 
   var imageContainer = document.getElementById("theimagecontainer");
   var oldImage = document.getElementById("thepreviewimage");
@@ -671,7 +762,7 @@ function makePreview(item)
   if (nn == "link" || nn == "input")
   {
     newImage = new Image();
-    newImage.src = getAbsoluteURL(item.href || item.src, item);
+    newImage.src = getAbsoluteURL(getSource(item), item);
   }
   else
   {
@@ -802,25 +893,42 @@ function getAbsoluteURL(url, node)
   return URL.spec;
 }
 
-//******** define a js object to implement nsIOutlinerView
-function pageInfoOutlinerView(columnids)
+function doCopy(event)
+{
+  if (!gClipboardHelper) 
+    return;
+
+  var elem = event.originalTarget;
+  if (elem && "treeBoxObject" in elem)
+    elem.treeBoxObject.view.performActionOnRow("copy", elem.currentIndex);
+
+  var text = elem.getAttribute("copybuffer");
+  if (text) 
+    gClipboardHelper.copyString(text);
+}
+
+//******** define a js object to implement nsITreeView
+function pageInfoTreeView(columnids, copycol)
 {
   // columnids is an array of strings indicating the names of the columns, in order
   this.columnids = columnids;
   this.colcount = columnids.length
+  this.copycol = copycol;
   this.rows = 0;
-  this.outliner = null;
+  this.tree = null;
   this.data = new Array;
   this.selection = null;
+  this.sortcol = null;
+  this.sortdir = 0;
 }
 
-pageInfoOutlinerView.prototype = {
+pageInfoTreeView.prototype = {
   set rowCount(c) { throw "rowCount is a readonly property"; },
   get rowCount() { return this.rows; },
 
-  setOutliner: function(outliner) 
+  setTree: function(tree) 
   {
-    this.outliner = outliner;
+    this.tree = tree;
   },
 
   getCellText: function(row, column)
@@ -858,12 +966,12 @@ pageInfoOutlinerView.prototype = {
 
   rowCountChanged: function(index, count)
   {
-    this.outliner.rowCountChanged(index, count);
+    this.tree.rowCountChanged(index, count);
   },
 
   invalidate: function()
   {
-    this.outliner.invalidate();
+    this.tree.invalidate();
   },
 
   clear: function()
@@ -871,25 +979,39 @@ pageInfoOutlinerView.prototype = {
     this.data = null;
   },
 
+  handleCopy: function(row)
+  {
+    return (row < 0) ? "" : (this.data[row][this.copycol] || "");
+  },
+
+  performActionOnRow: function(action, row)
+  {
+    if (action == "copy")
+      var data = this.handleCopy(row)
+    this.tree.treeBody.parentNode.setAttribute("copybuffer", data);
+  },
+
   getRowProperties: function(row, column, prop) { },
   getCellProperties: function(row, prop) { },
   getColumnProperties: function(column, elem, prop) { },
   isContainer: function(index) { return false; },
   isContainerOpen: function(index) { return false; },
   isSeparator: function(index) { return false; },
-  isSorted: function() { return false; },
+  isSorted: function() { },
   canDropOn: function(index) { return false; },
   canDropBeforeAfter: function(index, before) { return false; },
   drop: function(row, orientation) { return false; },
   getParentIndex: function(index) { return 0; },
   hasNextSibling: function(index, after) { return false; },
   getLevel: function(index) { return 0; },
+  getImageSrc: function(row, column) { },
+  getProgressMode: function(row, column) { },
+  getCellValue: function(row, column) { },
   toggleOpenState: function(index) { },
   cycleHeader: function(col, elem) { },
   selectionChanged: function() { },
   cycleCell: function(row, column) { },
   isEditable: function(row, column) { return false; },
   performAction: function(action) { },
-  performActionOnRow: function(action, row) { },
   performActionOnCell: function(action, row, column) { }
 };
