@@ -345,7 +345,7 @@ function AskChangeDefaultEngine(aSelectedEngine)
 
     var checkbox = {value:0};
     change = promptSvc.confirmCheck(window, title, changeEngineMsg, 
-                   dontAskAgainMsg, checkbox);
+               dontAskAgainMsg, checkbox);
 
     // store "don't ask again" pref from checkbox value (if changed)
     debug("dontAskAgain: " + dontAskAgain);
@@ -357,6 +357,18 @@ function AskChangeDefaultEngine(aSelectedEngine)
   // if confirmed true, i.e., change default engine, then set pref
   if (change)
     nsPreferences.setUnicharPref(kDefaultEnginePref, aSelectedEngine.value);
+
+  disableNavButtons();
+}
+
+function disableNavButtons()
+{
+  var nextButton = document.getElementById("next-results");
+  var prevButton = document.getElementById("prev-results");
+  if (nextButton && nextButton.getAttribute("disabled") != "true")
+    nextButton.setAttribute("disabled", "true");
+  if (prevButton && prevButton.getAttribute("disabled") != "true")
+    prevButton.setAttribute("disabled", "true");
 }
 
 function ensureSearchPref()
@@ -392,8 +404,8 @@ function SearchPanelStartup()
   
   ensureSearchPref()
   
-  var tree = document.getElementById("Tree");
-  tree.database.AddObserver(RDF_observer);
+  var resultList = document.getElementById("resultList");
+  resultList.database.AddObserver(RDF_observer);
 
   var categoryList = document.getElementById("categoryList");
   var internetSearch = Components.classes[ISEARCH_CONTRACTID].getService(nsIInternetSearchService);
@@ -404,11 +416,11 @@ function SearchPanelStartup()
     var ref = categoryList.getAttribute("ref");
     if (ref)
       categoryList.setAttribute("ref", ref);
-    var engineTree = document.getElementById("searchengines");
-    engineTree.database.AddDataSource(catDS);
-    ref = engineTree.getAttribute("ref");
+    var engineList = document.getElementById("searchengines");
+    engineList.database.AddDataSource(catDS);
+    ref = engineList.getAttribute("ref");
     if (ref)
-      engineTree.setAttribute("ref", ref);
+      engineList.setAttribute("ref", ref);
   }
 
   // try and determine last category name used
@@ -459,7 +471,7 @@ function SearchPanelStartup()
 
 function haveSearchResults()
 {
-  var ds = document.getElementById("Tree").database;
+  var ds = document.getElementById("resultList").database;
   if (!ds)
     return false;
 
@@ -482,8 +494,8 @@ function haveSearchResults()
 
 function getNumEngines()
 {
-  var treeChildrenNode = document.getElementById("engineKids");
-  return treeChildrenNode.childNodes.length;
+  var listbox = document.getElementById("searchengines");
+  return listbox.getElementsByTagName("listitem").length;
 }
 
 function chooseCategory(aNode)
@@ -512,24 +524,20 @@ function saveEngines()
   if (!localStore)
     return;
 
-  var engineBox = document.getElementById("engineKids");
+  var engineItems = document.getElementById("searchengines").getElementsByTagName("listitem");
 
   var checkedProperty = rdf.GetResource("http://home.netscape.com/NC-rdf#checked", true);
   var categorySRC = rdf.GetResource(category, true);
 
-  for (var x = 0; x < engineBox.childNodes.length; ++x) {
-    var treeitemNode = engineBox.childNodes[x];
+  for (var x = 0; x < engineItems.length; ++x) {
+    var itemNode = engineItems[x];
+    var engineURI = itemNode.getAttribute("id");
+    var engineSRC = rdf.GetResource(engineURI, true);
 
-    var checkboxNode = treeitemNode.firstChild.firstChild.firstChild;
-    if (checkboxNode) {
-      var engineURI = treeitemNode.getAttribute("id");
-      var engineSRC = rdf.GetResource(engineURI, true);
-
-      if (checkboxNode.checked)
-        localStore.Assert(categorySRC, checkedProperty, engineSRC, true);
-      else
-        localStore.Unassert(categorySRC, checkedProperty, engineSRC, true);
-    }
+    if (itemNode.checked)
+      localStore.Assert(categorySRC, checkedProperty, engineSRC, true);
+    else
+      localStore.Unassert(categorySRC, checkedProperty, engineSRC, true);
   }
 
   // save changes; flush out the localstore
@@ -546,20 +554,19 @@ function loadEngines(aCategory)
   var rdf = Components.classes[RDFSERVICE_CONTRACTID].getService(nsIRDFService);
   var localStore = rdf.GetDataSource("rdf:local-store");
   if (localStore) {
-    var engineBox = document.getElementById("engineKids");
+    var engineBox = document.getElementById("searchengines");
     var numEngines = engineBox.childNodes.length;
     var checkedProperty = rdf.GetResource("http://home.netscape.com/NC-rdf#checked", true);
     var categorySRC = rdf.GetResource(aCategory, true);
     for (var x = 0; x < numEngines; ++x) {
-      var treeitemNode = engineBox.childNodes[x];
-      var engineURI = treeitemNode.getAttribute("id");
-      var engineSRC = rdf.GetResource(engineURI, true);
-
-      var checkboxNode = treeitemNode.firstChild.firstChild.firstChild;
-      if (checkboxNode) {
+      var listitemNode = engineBox.childNodes[x];
+      if (listitemNode.localName == "listitem") {
+        var engineURI = listitemNode.getAttribute("id");
+        var engineSRC = rdf.GetResource(engineURI, true);
+  
         var hasAssertion = localStore.HasAssertion(categorySRC, checkedProperty, engineSRC, true);
         if (hasAssertion)
-          checkboxNode.checked = true;
+          listitemNode.checked = true;
       }
     }
   }
@@ -573,7 +580,7 @@ function focusTextBox()
 
 function SearchPanelShutdown()
 {
-  var tree = document.getElementById("Tree");
+  var tree = document.getElementById("resultList");
   tree.database.RemoveObserver(RDF_observer);
 }
 
@@ -597,11 +604,11 @@ function doStop()
 
   // show appropriate column(s)
   var navWindow = getNavigatorWindow(false);
-  var resultsTree = navWindow ? navWindow._content.document.getElementById("internetresultstree") : null;
-  if (!resultsTree)
+  var resultsList = navWindow ? navWindow._content.document.getElementById("resultsList") : null;
+  if (!resultsList)
     return;
 
-  var searchURL = resultsTree.getAttribute("ref");
+  var searchURL = resultsList.getAttribute("ref");
   if (!searchURL)
     return;
 
@@ -697,24 +704,20 @@ function doSearch()
   var engineURIs = [];
   if (searchMode > 0) {
     var foundEngine = false;
-    var treeitemNode;
-    var engineBox = document.getElementById("engineKids");
+    var itemNode;
+    var engineBox = document.getElementById("searchengines");
 
     // in advanced search mode, get selected search engines
     // (for the current search category)
-    for (var x = 0; x < engineBox.childNodes.length; x++) {
-      treeitemNode = engineBox.childNodes[x];
+    for (var x = 0; x < engineBox.childNodes.length; ++x) {
+      itemNode = engineBox.childNodes[x];
 
-      if (treeitemNode) {
-        var checkboxNode = treeitemNode.firstChild.firstChild.firstChild;
+      if (itemNode.localName == "listitem" && itemNode.checked) {
+        var engineURI = itemNode.id;
 
-        if (checkboxNode && checkboxNode.checked) {
-          var engineURI = treeitemNode.id;
-
-          if (engineURI) {
-            engineURIs[engineURIs.length] = engineURI;
-            foundEngine = true;
-          }
+        if (engineURI) {
+          engineURIs[engineURIs.length] = engineURI;
+          foundEngine = true;
         }
       }
     }
@@ -722,13 +725,13 @@ function doSearch()
     if (!foundEngine) {
       if (getNumEngines() == 1) {
         // only one engine in this category, check it
-        treeitemNode = engineBox.firstChild;
-        engineURIs[engineURIs.length] = treeitemNode.id;
+        itemNode = engineBox.firstChild;
+        engineURIs[engineURIs.length] = itemNode.id;
       }
       else {
         for (var i = 0; i < engineBox.childNodes.length; ++i) {
-          treeitemNode = engineBox.childNodes[i];
-          var theID = treeitemNode.id;
+          itemNode = engineBox.childNodes[i];
+          var theID = itemNode.id;
           if (theID.indexOf("NetscapeSearch.src") != -1) {
             engineURIs[engineURIs.length] = theID;
             foundEngine = true;
@@ -772,10 +775,10 @@ function checkSearchProgress()
   var navWindow = getNavigatorWindow(false);
 
   if (navWindow) {
-    var resultsTree = navWindow._content.document.getElementById("internetresultstree");
-    if (resultsTree) {
-      var treeref = resultsTree.getAttribute("ref");
-      var ds = resultsTree.database;
+    var resultsList = navWindow._content.document.getElementById("resultsList");
+    if (resultsList) {
+      var treeref = resultsList.getAttribute("ref");
+      var ds = resultsList.database;
       if (ds && treeref) {
         try {
           var rdf = Components.classes[RDFSERVICE_CONTRACTID].getService(nsIRDFService);
@@ -801,21 +804,16 @@ function checkSearchProgress()
   return activeSearchFlag;
 }
 
-function sidebarOpenURL(treeitem)
+function sidebarOpenURL(listitem)
 {
-  if (treeitem.getAttribute("container") == "true" ||
-      treeitem.getAttribute("type") == "http://home.netscape.com/NC-rdf#BookmarkSeparator")   {
-    return;
-  }
-
-  var id = treeitem.id;
+  var id = listitem.id;
   if (!id)
     return;
 
   // rjc: add support for anonymous resources; if the node has
   // a "#URL" property, use it, otherwise default to using the id
   try {
-    var ds = document.getElementById("Tree").database;
+    var ds = document.getElementById("resultList").database;
     if (ds) {
       var rdf = Components.classes[RDFSERVICE_CONTRACTID].getService(nsIRDFService);
       var src = rdf.GetResource(id, true);
@@ -829,6 +827,10 @@ function sidebarOpenURL(treeitem)
     }
   } catch (ex) {
   }
+
+  // mark result as visited
+  listitem.setAttribute("visited", "true");
+  
   loadURLInContent(id);
 }
 
@@ -863,9 +865,13 @@ function OpenSearch(aSearchStr, engineURIs)
 
       // look up the correct search URL format for the given engine
       try {
-        var searchURL = searchDS.GetInternetSearchURL(searchEngineURI, escapedSearchStr);
+        var whichButtons = new Object;
+        whichButtons.value = 0;
+        var searchURL = searchDS.GetInternetSearchURL(searchEngineURI, escapedSearchStr, 0, 0, whichButtons);
+        doNavButtonEnabling(whichButtons.value, searchDS, 0);
       }
       catch (ex) {
+        debug("Exception when calling GetInternetSearchURL: " + ex);
         searchURL = "";
       }
 
@@ -889,7 +895,9 @@ function OpenSearch(aSearchStr, engineURIs)
       loadURLInContent("chrome://communicator/content/search/internetresults.xul?" + escape(searchURL));
     }
   }
-  catch (ex) {}
+  catch (ex) {
+    debug("Exception: " + ex);
+  }
 
   setTimeout("checkSearchProgress()", 1000);
 }
@@ -903,7 +911,7 @@ function switchTab(aPageIndex)
   if (aPageIndex != 0)
     return;
 
-  var ds = document.getElementById("Tree").database;
+  var ds = document.getElementById("resultList").database;
   if (!ds)
     return;
 
@@ -919,14 +927,11 @@ function switchTab(aPageIndex)
     if (target)
       haveSearchRef = true;
   }
-
-  var saveQueryButton = document.getElementById("saveQueryButton");
-  saveQueryButton.disabled = !haveSearchRef;
 }
 
 function saveSearch()
 {
-  var ds = document.getElementById("Tree").database;
+  var ds = document.getElementById("resultList").database;
   if (!ds)
     return;
 
@@ -978,6 +983,78 @@ function saveSearch()
   bmks.addBookmarkImmediately(lastSearchURI, searchTitle, bmks.BOOKMARK_SEARCH_TYPE, null);
 }
 
+var gPageNumber = 0;
+/**
+ * showMoreResults
+ *
+ * Run a query to show the next/previous page of search results for the
+ * current search term.
+ * 
+ * @param direction : -1 => previous
+ *                     1 => next
+ */
+function showMoreResults(direction)
+{
+  // XXX check if we are in basic search mode
+
+  // get search engine
+  var engine = document.getElementById("basicEngineMenu").selectedItem;
+  var engineURI = engine.id;
+
+  // get search term
+  var searchTerm = document.getElementById("sidebar-search-text").value;
+  searchTerm = escape(searchTerm);
+
+  // change page number
+  if (direction > 0)
+    ++gPageNumber;
+  else
+    --gPageNumber;
+
+  // get qualified URL
+  var searchService = Components.classes[ISEARCH_CONTRACTID].
+                        getService(nsIInternetSearchService);
+  var whichButtons = new Object;
+  whichButtons.value = 0;
+  var searchURL = searchService.GetInternetSearchURL(engineURI, searchTerm, 
+                    direction, gPageNumber, whichButtons);
+
+  doNavButtonEnabling(whichButtons.value, searchService, gPageNumber);
+
+  // load URL in navigator
+  loadURLInContent(searchURL);
+}
+
+function doNavButtonEnabling(whichButtons, searchService, pageNumber)
+{
+  var nextButton = document.getElementById("next-results");
+  var nextDisabled = nextButton.getAttribute("disabled");
+  var prevButton = document.getElementById("prev-results");
+  var prevDisabled = prevButton.getAttribute("disabled");
+
+  if (whichButtons & searchService.kHaveNext)
+  {
+    if (nextDisabled)
+      nextButton.removeAttribute("disabled");
+  }
+  else 
+  {
+    if (!nextDisabled)
+      nextButton.setAttribute("disabled", "true");
+  }
+  
+  if ((pageNumber > 0) && (whichButtons & searchService.kHavePrev))
+  {
+    if (prevDisabled)
+      prevButton.removeAttribute("disabled");
+  }
+  else 
+  {
+    if (!prevDisabled)
+      prevButton.setAttribute("disabled", "true");
+  }
+}
+
 function doCustomize()
 {
   //Switching from Edit Categories back to All Engines then launching customize window
@@ -1041,59 +1118,95 @@ function getItemNode(aNode,nodeName)
   return node ? node : null;
 }
 
+function getArcValueForID(aArc, aID)
+{
+  var val = null;
+
+  try
+  {
+    var ds = document.getElementById("resultList").database;
+    if (ds)
+    {
+      var rdf = Components.classes[RDFSERVICE_CONTRACTID].
+                  getService(nsIRDFService);
+      var src = rdf.GetResource(aID, true);
+      var prop = rdf.GetResource(
+                  "http://home.netscape.com/NC-rdf#"+aArc, true);
+      val = ds.GetTarget(src, prop, true);        
+      if (val)
+        val = val.QueryInterface(nsIRDFLiteral).Value;
+    }
+  } 
+  catch (ex)
+  {
+    dump("Exception: no value for " + aArc + "!\t" + ex + "\n");
+    val = null;
+  }
+
+  return val;
+}
+
 //Fill in tooltip in teh search results panel
 function FillInDescTooltip(tipElement)
 {
   var retValue = false;
  
-  //Get the cell node and the tree item node of
-  //moused over sherlock result
-  var nodeTreeCell = getItemNode(tipElement, "treecell");
-  var nodeTreeitem = getItemNode(tipElement, "treeitem");
+  //Get the Name of the listitem for first item in the tooltip
+  var nodeLabel = tipElement.getAttribute("label");
+  var nodeID = tipElement.id;
  
-  //Get the Name of the tree cell for first item in the tooltip
-  var nodeLabel = nodeTreeCell.getAttribute("label");
-  var nodeID = nodeTreeitem.id;
- 
-  //Query RDF to get URL of tree item
+  //Query RDF to get URL of listitem
   if (nodeID)
-    try {
-      var ds = document.getElementById("Tree").database;
-      if (ds) {
-        var rdf = Components.classes[RDFSERVICE_CONTRACTID].getService(nsIRDFService);
-        var src = rdf.GetResource(nodeID, true);
-        var prop = rdf.GetResource("http://home.netscape.com/NC-rdf#URL", true);
-        var url = ds.GetTarget(src, prop, true);        
-        if (url)
-          url = url.QueryInterface(nsIRDFLiteral).Value;
-      }
-    } catch (ex) {
-  }
+    var url = getArcValueForID("URL", nodeID);
 
   //Fill in the the text nodes
   //collapse them if there is not a node
-    if (nodeLabel || url) {
-      var tooltipTitle = document.getElementById("titleText");
-      var tooltipUrl = document.getElementById("urlText"); 
-      if (nodeLabel) {
-        if (tooltipTitle.getAttribute("hidden") == "true")
-          tooltipTitle.removeAttribute("hidden");
-        tooltipTitle.setAttribute("value",nodeLabel);
-      } 
-      else  {
-        tooltipTitle.setAttribute("hidden", "true");
-      }
-      if (url) {
-        if (tooltipUrl.getAttribute("hidden") == "true")
-          tooltipUrl.removeAttribute("hidden");
-        if (url.length > 100)
-          url = url.substr(0,100) + "...";
-        tooltipUrl.setAttribute("value",url);
-      }
-      else {
-        tooltipUrl.setAttribute("hidden", "true");
-      }
+  if (nodeLabel || url) {
+    var tooltipTitle = document.getElementById("titleText");
+    var tooltipUrl = document.getElementById("urlText"); 
+    if (nodeLabel) {
+      if (tooltipTitle.getAttribute("hidden") == "true")
+        tooltipTitle.removeAttribute("hidden");
+      tooltipTitle.setAttribute("value",nodeLabel);
+    } 
+    else  {
+      tooltipTitle.setAttribute("hidden", "true");
+    }
+    if (url) {
+      if (tooltipUrl.getAttribute("hidden") == "true")
+        tooltipUrl.removeAttribute("hidden");
+      if (url.length > 100)
+        url = url.substr(0,100) + "...";
+      tooltipUrl.setAttribute("value",url);
+    }
+    else {
+      tooltipUrl.setAttribute("hidden", "true");
+    }
     retValue = true;
   }
   return retValue;
 }
+
+var nsResultDNDObserver = 
+{
+  onDragStart: function(aEvent, aXferData, aDragAction)
+  {
+    var node = getItemNode(aEvent.target, "listitem");
+    var URL = getArcValueForID("URL", node.id);
+    var title = getArcValueForID("Name", node.id);
+    var htmlString = "<a href=\"" + URL + "\">" + title + "</a>";
+    var urlString = URL + "\n" + title;
+
+    aXferData.data = new TransferData();
+    aXferData.data.addDataForFlavour("text/x-moz-url", URL);
+    aXferData.data.addDataForFlavour("text/unicode", urlString);
+    aXferData.data.addDataForFlavour("text/html", htmlString); 
+  }
+};
+
+function HandleResultDragGesture(aEvent)
+{
+  nsDragAndDrop.startDrag(aEvent, nsResultDNDObserver);
+  return true;  
+}
+
