@@ -104,6 +104,7 @@ print "The display version is: $versionMain\n";
 $versionLanguage               = "en";
 $ENV{WIZ_nameCompany}          = "mozilla.org";
 $ENV{WIZ_nameProduct}          = "Mozilla";
+$ENV{WIZ_nameProductNoVersion} = "Mozilla";
 $ENV{WIZ_fileMainExe}          = "Mozilla.exe";
 $ENV{WIZ_fileUninstall}        = $seuFileNameSpecific;
 $ENV{WIZ_fileUninstallZip}     = $seuzFileNameSpecific;
@@ -372,10 +373,10 @@ sub MakeExeZip
 
   $saveCwdir = cwd();
   chdir($aSrcDir);
-  if(system("$ENV{MOZ_TOOLS}\\bin\\zip $inDistPath\\xpi\\$aZipFile $aExeFile"))
+  if(system("zip $inDistPath\\xpi\\$aZipFile $aExeFile"))
   {
     chdir($saveCwdir);
-    die "\n Error: $ENV{MOZ_TOOLS}\\bin\\zip $inDistPath\\xpi\\$aZipFile $aExeFile";
+    die "\n Error: zip $inDistPath\\xpi\\$aZipFile $aExeFile";
   }
   chdir($saveCwdir);
 }
