@@ -10,46 +10,40 @@
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
- * The Original Code is Mozilla Communicator client code, 
- * released March 31, 1998. 
+ * The Original Code is Mozilla Communicator client code,
+ * released March 31, 1998.
  *
- * The Initial Developer of the Original Code is Netscape Communications 
+ * The Initial Developer of the Original Code is Netscape Communications
  * Corporation.  Portions created by Netscape are
  * Copyright (C) 1998 Netscape Communications Corporation. All
  * Rights Reserved.
  *
- * Contributor(s): 
+ * Contributor(s):
  *     Daniel Veditz <dveditz@netscape.com>
  *     Douglas Turner <dougt@netscape.com>
  */
 
 #include "nscore.h"
 #include "nsXPIDLString.h"
-#include "nsFileSpec.h"
-#include "nsFileStream.h"
 #include "nsInstall.h" // for error codes
 #include "prmem.h"
 #include "ScheduledTasks.h"
 #include "InstallCleanupDefines.h"
 
-#include "nsSpecialSystemDirectory.h"
 #include "nsDirectoryService.h"
 #include "nsDirectoryServiceDefs.h"
+#include "nsAppDirectoryServiceDefs.h"
 
 static nsresult 
-GetPersistentStringFromSpec(nsIFile* inSpec, char **string)
+GetPersistentStringFromSpec(nsIFile* inSpec, nsACString &string)
 {
     nsresult rv;
 
-    if (!string) return NS_ERROR_NULL_POINTER;
-
-    nsCOMPtr<nsILocalFile> LocalFile = do_QueryInterface(inSpec, &rv);
-
     if (NS_SUCCEEDED(rv)) {
-        rv = LocalFile->GetPath(string);
+        rv = inSpec->GetNativePath(string);
     } 
     else {
-        *string = nsnull;
+        string.Truncate();
     }
     return rv;
 }
@@ -62,30 +56,20 @@ GetPersistentStringFromSpec(nsIFile* inSpec, char **string)
 #include <sys/stat.h>
 #include <windows.h>
 
-PRInt32 ReplaceExistingWindowsFile(nsIFile* currentSpec, nsIFile* finalSpec)
+PRInt32 ReplaceWindowsSystemFile(nsIFile* currentSpec, nsIFile* finalSpec)
 {
-    // this routine is now for DOS-based windows only. WinNT should 
-    // be taken care of by the XP code
-    //
-    // NOTE for WINNT:
-    //
-    // the MOVEFILE_DELAY_UNTIL_REBOOT option doesn't work on
-    // NT 3.51 SP4 or on NT 4.0 until SP2. On the broken versions
-    // of NT 4.0 Microsoft warns using it can lead to an irreparably 
-    // corrupt  windows' registry "after an unknown number of calls".
-    // Time to reinstall windows when that happens.
-    //
-    // I don't want to risk it, I also don't want two separate code
-    // paths to test, so we do it the lame way on all NT systems
-    // until such time as there are few enough old revs around to
-    // make it worth switching back to MoveFileEx().
-
     PRInt32 err = -1;
 
-    /* Get OS version info */
+    // Get OS version info
     DWORD dwVersion = GetVersion();
 
-    /* Get build numbers for Windows NT or Win32s */
+    nsCAutoString final;
+    nsCAutoString current;
+
+    finalSpec->GetNativePath(final);
+    currentSpec->GetNativePath(current);
+ 
+    // Get build numbers for Windows NT or Win32s 
 
     if (dwVersion > 0x80000000)
     {
@@ -97,88 +81,90 @@ PRInt32 ReplaceExistingWindowsFile(nsIFile* currentSpec, nsIFile* finalSpec)
         int     strlen;
         char    Src[_MAX_PATH];   // 8.3 name
         char    Dest[_MAX_PATH];  // 8.3 name
-
         
-        char* final;
-        char* current;
-
-        finalSpec->GetPath(&final);
-        currentSpec->GetPath(&current);
-        
-        strlen = GetShortPathName( (LPCTSTR)current, (LPTSTR)Src, (DWORD)sizeof(Src) );
+        strlen = GetShortPathName( (LPCTSTR)current.get(), (LPTSTR)Src, (DWORD)sizeof(Src) );
         if ( strlen > 0 ) 
         {
-            free(current);
-            current   = strdup(Src);
+            current = Src;
         }
 
-        strlen = GetShortPathName( (LPCTSTR) final, (LPTSTR) Dest, (DWORD) sizeof(Dest));
+        strlen = GetShortPathName( (LPCTSTR) final.get(), (LPTSTR) Dest, (DWORD) sizeof(Dest));
         if ( strlen > 0 ) 
         {
-            free(final);
-            final   = strdup(Dest);
+            final = Dest;
         }
-        
-        /* NOTE: use OEM filenames! Even though it looks like a Windows
-         *       .INI file, WININIT.INI is processed under DOS 
-         */
-        
-        AnsiToOem( final, final );
-        AnsiToOem( current, current );
 
-        if ( WritePrivateProfileString( "Rename", final, current, "WININIT.INI" ) )
+        // NOTE: use OEM filenames! Even though it looks like a Windows
+        //       .INI file, WININIT.INI is processed under DOS 
+        
+        AnsiToOem( (char*)final.get(), (char*)final.get() );
+        AnsiToOem( (char*)current.get(), (char*)current.get() );
+
+        if ( WritePrivateProfileString( "Rename", final.get(), current.get(), "WININIT.INI" ) )
             err = 0;
-
-        free(final);
-        free(current);
     }
-    
+    else
+    {
+       // Windows NT
+        if ( MoveFileEx(final.get(), current.get(), MOVEFILE_DELAY_UNTIL_REBOOT) )
+          err = 0;
+    }
     return err;
 }
 #endif
 
-char* GetRegFilePath()
+nsresult GetRegFilePath(nsACString &regFilePath)
 {
     nsresult rv;
     nsCOMPtr<nsILocalFile> iFileUtilityPath;
     //Get the program directory
-    nsCOMPtr<nsIProperties> directoryService = 
+    nsCOMPtr<nsIProperties> directoryService =
              do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv);
     if (NS_FAILED(rv))
         return nsnull;
 
-    directoryService->Get(NS_OS_CURRENT_PROCESS_DIR, 
-                          NS_GET_IID(nsIFile), 
-                          getter_AddRefs(iFileUtilityPath));
-    if (!iFileUtilityPath)
-        return nsnull;
+    if (nsSoftwareUpdate::GetProgramDirectory()) // In the stub installer
+    {
+        nsCOMPtr<nsIFile> tmp;
+        rv = nsSoftwareUpdate::GetProgramDirectory()->Clone(getter_AddRefs(tmp));
+
+        if (NS_FAILED(rv) || !tmp)
+            return nsnull;
 
 #if defined (XP_MAC)
-    iFileUtilityPath->Append(ESSENTIAL_FILES);
+        tmp->AppendNative(ESSENTIAL_FILES);
 #endif
-    iFileUtilityPath->Append(CLEANUP_REGISTRY);
+        iFileUtilityPath = do_QueryInterface(tmp);
+    }
+    else
+    {
+        rv = directoryService->Get(NS_APP_INSTALL_CLEANUP_DIR,
+                                  NS_GET_IID(nsIFile),
+                                  getter_AddRefs(iFileUtilityPath));
+    }
+    if (NS_FAILED(rv) || !iFileUtilityPath)
+        return nsnull;
 
-    //Yes, we know using GetPath is buggy on the Mac.
+    iFileUtilityPath->AppendNative(CLEANUP_REGISTRY);
+
+    //Yes, we know using GetNativePath is buggy on the Mac.
     //When libreg is fixed to accept nsIFiles we'll change this to match.
-    char* regFilePath;
-    iFileUtilityPath->GetPath(&regFilePath);
-  
-    return regFilePath;
+    return iFileUtilityPath->GetNativePath(regFilePath);
 }
 
 
 PRInt32 DeleteFileNowOrSchedule(nsIFile* filename)
 {
-    PRBool flagExists;  
+    PRBool flagExists;
     PRInt32 result = nsInstall::SUCCESS;
 
     filename->Remove(PR_FALSE);
     filename->Exists(&flagExists);
     if (flagExists)
         result = ScheduleFileForDeletion(filename);
- 
+
     return result;
-} 
+}
 
 PRInt32 ScheduleFileForDeletion(nsIFile *filename)
 {
@@ -189,10 +175,9 @@ PRInt32 ScheduleFileForDeletion(nsIFile *filename)
     REGERR  err;
     PRInt32 result = nsInstall::UNEXPECTED_ERROR;
 
-    char* regFilePath = GetRegFilePath();
-    err = NR_RegOpen(regFilePath, &reg);
-    if (regFilePath)
-      nsCRT::free(regFilePath);
+    nsCAutoString path;
+    GetRegFilePath(path);
+    err = NR_RegOpen((char*)path.get(), &reg);
 
     if ( err == REGERR_OK )
     {
@@ -204,12 +189,11 @@ PRInt32 ScheduleFileForDeletion(nsIFile *filename)
             err = NR_RegGetUniqueName( reg, valname, sizeof(valname) );
             if ( err == REGERR_OK )
             {
-                nsXPIDLCString nameowner;
-                nsresult rv = GetPersistentStringFromSpec(
-                                filename,getter_Copies(nameowner));
-                if ( NS_SUCCEEDED(rv) && nameowner )
+                nsCAutoString nameowner;
+                nsresult rv = GetPersistentStringFromSpec(filename, nameowner);
+                if ( NS_SUCCEEDED(rv) && !nameowner.IsEmpty() )
                 {
-                    const char *fnamestr = nameowner;
+                    const char *fnamestr = nameowner.get();
                     err = NR_RegSetEntry( reg, newkey, valname, 
                                           REGTYPE_ENTRY_BYTES, 
                                           (void*)fnamestr, 
@@ -255,7 +239,7 @@ PRInt32 ReplaceFileNow(nsIFile* replacementFile, nsIFile* doomedFile )
     nsCOMPtr<nsIFile>      renamedDoomedFile;
     nsCOMPtr<nsILocalFile> tmpLocalFile;
     nsCOMPtr<nsIFile> parent;
-    nsXPIDLCString leafname;
+    nsCAutoString leafname;
     
     doomedFile->Clone(getter_AddRefs(renamedDoomedFile));
     renamedDoomedFile->Exists(&flagExists);
@@ -264,36 +248,35 @@ PRInt32 ReplaceFileNow(nsIFile* replacementFile, nsIFile* doomedFile )
         tmpLocalFile = do_QueryInterface(renamedDoomedFile, &rv); // Convert to an nsILocalFile
 
         //get the leafname so we can convert its extension to .old
-        nsXPIDLCString uniqueLeafName;
-        tmpLocalFile->GetLeafName(getter_Copies(leafname));
-        nsCString newLeafName (leafname); 
+        nsCAutoString uniqueLeafName;
+        tmpLocalFile->GetNativeLeafName(leafname);
 
-        PRInt32 extpos = newLeafName.RFindChar('.');
+        PRInt32 extpos = leafname.RFindChar('.');
         if (extpos != -1)
         {
             // We found the extension; 
-            newLeafName.Truncate(extpos + 1); //strip off the old extension
+            leafname.Truncate(extpos + 1); //strip off the old extension
         }
-        newLeafName.Append("old");
+        leafname.Append(NS_LITERAL_CSTRING("old"));
         
         //Now reset the leafname
-        tmpLocalFile->SetLeafName(newLeafName.get());
+        tmpLocalFile->SetNativeLeafName(leafname);
         
         MakeUnique(tmpLocalFile);                                 //  for the call to MakeUnique
-        
-        tmpLocalFile->GetParent(getter_AddRefs(parent)); //get the parent for later use in MoveTo
-        tmpLocalFile->GetLeafName(getter_Copies(uniqueLeafName));//this is the new "unique" leafname
 
-        rv = doomedFile->Clone(getter_AddRefs(renamedDoomedFile));// Reset renamedDoomed file so doomedfile isn't 
+        tmpLocalFile->GetParent(getter_AddRefs(parent)); //get the parent for later use in MoveTo
+        tmpLocalFile->GetNativeLeafName(uniqueLeafName);//this is the new "unique" leafname
+
+        rv = doomedFile->Clone(getter_AddRefs(renamedDoomedFile));// Reset renamedDoomed file so doomedfile isn't
                                                                   //   changed during the MoveTo call
         if (NS_FAILED(rv)) result = nsInstall::UNEXPECTED_ERROR;
-        rv = renamedDoomedFile->MoveTo(parent, uniqueLeafName);        
+        rv = renamedDoomedFile->MoveToNative(parent, uniqueLeafName);        
 
         if (NS_SUCCEEDED(rv))
         {
-            renamedDoomedFile = parent;                //MoveTo on Mac doesn't reset the tmpFile object to 
-            renamedDoomedFile->Append(uniqueLeafName); //the new name or location. That's why there's this 
-                                                       //explict assignment and Append call.
+            renamedDoomedFile = parent;                      //MoveTo on Mac doesn't reset the tmpFile object to 
+            renamedDoomedFile->AppendNative(uniqueLeafName); //the new name or location. That's why there's this 
+                                                             //explict assignment and Append call.
         }
 
         if (result == nsInstall::UNEXPECTED_ERROR)
@@ -307,6 +290,7 @@ PRInt32 ReplaceFileNow(nsIFile* replacementFile, nsIFile* doomedFile )
     {
         nsCOMPtr<nsIFile> parentofFinalFile;
         nsCOMPtr<nsIFile> parentofReplacementFile;
+        nsCAutoString leafname;
 
         doomedFile->GetParent(getter_AddRefs(parentofFinalFile));
         replacementFile->GetParent(getter_AddRefs(parentofReplacementFile));
@@ -319,15 +303,15 @@ PRInt32 ReplaceFileNow(nsIFile* replacementFile, nsIFile* doomedFile )
         if(!flagIsEqual)
         {
             NS_WARNING("File unpacked into a non-dest dir" );
-            replacementFile->GetLeafName(getter_Copies(leafname));
-            rv = replacementFile->MoveTo(parentofFinalFile, leafname);
+            replacementFile->GetNativeLeafName(leafname);
+            rv = replacementFile->MoveToNative(parentofFinalFile, leafname);
         }
         	
-        rv = doomedFile->GetLeafName(getter_Copies(leafname));
+        rv = doomedFile->GetNativeLeafName(leafname);
         if ( NS_SUCCEEDED(rv))
-            rv = replacementFile->MoveTo(parentofReplacementFile, leafname );
+            rv = replacementFile->MoveToNative(parentofReplacementFile, leafname);
 
-        if ( NS_SUCCEEDED(rv) ) 
+        if ( NS_SUCCEEDED(rv) )
         {
             // we replaced the old file OK, now we have to
             // get rid of it if it was renamed out of the way
@@ -337,7 +321,7 @@ PRInt32 ReplaceFileNow(nsIFile* replacementFile, nsIFile* doomedFile )
         {
             // couldn't rename file, try to put old file back
             renamedDoomedFile->GetParent(getter_AddRefs(parent));
-            renamedDoomedFile->MoveTo(parent, leafname);
+            renamedDoomedFile->MoveToNative(parent, leafname);
         }
     }
 
@@ -348,7 +332,7 @@ PRInt32 ReplaceFileNow(nsIFile* replacementFile, nsIFile* doomedFile )
 
 
 
-PRInt32 ReplaceFileNowOrSchedule(nsIFile* replacementFile, nsIFile* doomedFile )
+PRInt32 ReplaceFileNowOrSchedule(nsIFile* replacementFile, nsIFile* doomedFile, PRInt32 aMode)
 {
     PRInt32 result = ReplaceFileNow( replacementFile, doomedFile );
 
@@ -356,8 +340,9 @@ PRInt32 ReplaceFileNowOrSchedule(nsIFile* replacementFile, nsIFile* doomedFile )
     {
         // if we couldn't replace the file schedule it for later
 #ifdef _WINDOWS
-        if ( ReplaceExistingWindowsFile(replacementFile, doomedFile) == 0 )
-            return nsInstall::REBOOT_NEEDED;
+        if ( (aMode & WIN_SYSTEM_FILE) &&
+             (ReplaceWindowsSystemFile(replacementFile, doomedFile) == 0) )
+                return nsInstall::REBOOT_NEEDED;
 #endif
 
         RKEY    listkey;
@@ -365,11 +350,12 @@ PRInt32 ReplaceFileNowOrSchedule(nsIFile* replacementFile, nsIFile* doomedFile )
         HREG    reg;
         REGERR  err;
 
-        char* regFilePath = GetRegFilePath();
-        if ( REGERR_OK == NR_RegOpen(regFilePath, &reg) ) 
+        nsCAutoString regFilePath;
+        GetRegFilePath(regFilePath);
+        if ( REGERR_OK == NR_RegOpen((char*)regFilePath.get(), &reg) ) 
         {
             err = NR_RegAddKey( reg, ROOTKEY_PRIVATE, REG_REPLACE_LIST_KEY, &listkey );
-            if ( err == REGERR_OK ) 
+            if ( err == REGERR_OK )
             {
                 char     valname[20];
                 REGERR   err2;
@@ -380,21 +366,18 @@ PRInt32 ReplaceFileNowOrSchedule(nsIFile* replacementFile, nsIFile* doomedFile )
                     err = NR_RegAddKey( reg, listkey, valname, &filekey );
                     if ( REGERR_OK == err )
                     {
-                        nsXPIDLCString srcowner;
-                        nsXPIDLCString destowner;
-                        nsresult rv = GetPersistentStringFromSpec(
-                                replacementFile, getter_Copies(srcowner));
-                        nsresult rv2 = GetPersistentStringFromSpec(
-                                doomedFile, getter_Copies(destowner));
+                        nsCAutoString srcowner;
+                        nsCAutoString destowner;
+                        nsresult rv = GetPersistentStringFromSpec(replacementFile, srcowner);
+                        nsresult rv2 = GetPersistentStringFromSpec(doomedFile, destowner);
                         if ( NS_SUCCEEDED(rv) && NS_SUCCEEDED(rv2) )
                         {
-
-                            const char *fsrc  = srcowner;
-                            const char *fdest = destowner;
+                            const char *fsrc  = srcowner.get();
+                            const char *fdest = destowner.get();
                             err = NR_RegSetEntry( reg, filekey, 
                                                   REG_REPLACE_SRCFILE,
-                                                  REGTYPE_ENTRY_BYTES, 
-                                                  (void*)fsrc, 
+                                                  REGTYPE_ENTRY_BYTES,
+                                                  (void*)fsrc,
                                                   strlen(fsrc)+sizeof('\0'));
 
                             err2 = NR_RegSetEntry(reg, filekey,
@@ -416,8 +399,6 @@ PRInt32 ReplaceFileNowOrSchedule(nsIFile* replacementFile, nsIFile* doomedFile )
             }
             NR_RegClose(reg);
         }
-        if (regFilePath)
-            nsCRT::free(regFilePath);
     }
 
     return result;
@@ -450,7 +431,7 @@ void DeleteScheduledFiles( HREG reg )
     REGENUM state = 0;
     nsresult rv = NS_OK;
 
-    /* perform scheduled file deletions  */
+    // perform scheduled file deletions
     if (REGERR_OK == NR_RegGetKey(reg,ROOTKEY_PRIVATE,REG_DELETE_LIST_KEY,&key))
     {
         // the delete key exists, so we loop through its children
@@ -471,12 +452,12 @@ void DeleteScheduledFiles( HREG reg )
                 err = NR_RegGetEntry( reg, key, namebuf, valbuf, &bufsize );
                 if ( err == REGERR_OK )
                 {
-                    // no need to check return value of 
+                    // no need to check return value of
                     // SetPersistentDescriptorString, it's always NS_OK
                     //spec->SetPersistentDescriptorString(valbuf); //nsIFileXXX: Do we still need this instead of InitWithPath?
-                    NS_NewLocalFile((char*)valbuf, PR_TRUE, getter_AddRefs(spec));
+                    NS_NewNativeLocalFile(nsDependentCString(valbuf), PR_TRUE, getter_AddRefs(spec));
                     spec->Clone(getter_AddRefs(doomedFile));
-                    if (NS_SUCCEEDED(rv)) 
+                    if (NS_SUCCEEDED(rv))
                     {
                         PRBool flagExists;
                         doomedFile->Remove(PR_FALSE);
@@ -490,7 +471,7 @@ void DeleteScheduledFiles( HREG reg )
                 }
             }
 
-            /* delete list node if empty */
+            // delete list node if empty
             state = 0;
             err = NR_RegEnumEntries(reg, key, &state, namebuf, sizeof(namebuf), 0);
             if ( err == REGERR_NOMORE )
@@ -507,7 +488,7 @@ void ReplaceScheduledFiles( HREG reg )
 {
     RKEY    key;
 
-    /* replace files if any listed */
+    // replace files if any listed
     if (REGERR_OK == NR_RegGetKey(reg,ROOTKEY_PRIVATE,REG_REPLACE_LIST_KEY,&key))
     {
         char keyname[MAXREGNAMELEN];
@@ -522,7 +503,7 @@ void ReplaceScheduledFiles( HREG reg )
 
         uint32 bufsize;
         REGENUM state = 0;
-        while (REGERR_OK == NR_RegEnumSubkeys( reg, key, &state, 
+        while (REGERR_OK == NR_RegEnumSubkeys( reg, key, &state,
                                keyname, sizeof(keyname), REGENUM_CHILDREN))
         {
             bufsize = sizeof(srcFile);
@@ -535,10 +516,10 @@ void ReplaceScheduledFiles( HREG reg )
 
             if ( err1 == REGERR_OK && err2 == REGERR_OK )
             {
-                rv1 = NS_NewLocalFile((char*)srcFile, PR_TRUE, getter_AddRefs(src));
+                rv1 = NS_NewNativeLocalFile(nsDependentCString(srcFile), PR_TRUE, getter_AddRefs(src));
                 rv1 = src->Clone(getter_AddRefs(srcSpec));
 
-                rv2 = NS_NewLocalFile((char*)doomedFile, PR_TRUE, getter_AddRefs(dest));
+                rv2 = NS_NewNativeLocalFile(nsDependentCString(doomedFile), PR_TRUE, getter_AddRefs(dest));
                 rv2 = dest->Clone(getter_AddRefs(doomedSpec));
 
                 if (NS_SUCCEEDED(rv1) && NS_SUCCEEDED(rv2))
@@ -557,7 +538,7 @@ void ReplaceScheduledFiles( HREG reg )
         }
 
 
-        /* delete list node if empty */
+        // delete list node if empty
         state = 0;
         if (REGERR_NOMORE == NR_RegEnumSubkeys( reg, key, &state, keyname,
                                      sizeof(keyname), REGENUM_CHILDREN ))
