@@ -14,7 +14,7 @@
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
@@ -22,7 +22,7 @@
  * Contributor(s):
  *
  * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
  * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
@@ -36,13 +36,13 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "zlib.h"
-#include "nsFileSpec.h"
 #include "prmem.h"
 #include "nsXPIDLString.h"
 #include "nsInstall.h"
 #include "nsInstallPatch.h"
 #include "nsInstallResources.h"
 #include "nsIDOMInstallVersion.h"
+#include "nsILocalFile.h"
 
 #include "gdiff.h"
 
@@ -98,16 +98,15 @@ nsInstallPatch::nsInstallPatch( nsInstall* inInstall,
 
     PRInt32 err = VR_GetPath( NS_CONST_CAST(char *, NS_ConvertUCS2toUTF8(inVRName).get()),
                               sizeof(tempTargetFile), tempTargetFile );
-    
+
     if (err != REGERR_OK)
     {
         *error = nsInstall::NO_SUCH_COMPONENT;
         return;
     }
-    nsString folderSpec; folderSpec.AssignWithConversion(tempTargetFile);
-	
+
     nsCOMPtr<nsILocalFile> tmp;
-    NS_NewLocalFile((char*)tempTargetFile, PR_TRUE, getter_AddRefs(tmp));
+    NS_NewNativeLocalFile(nsDependentCString(tempTargetFile), PR_TRUE, getter_AddRefs(tmp));
 
     mPatchFile      =   nsnull;
     mTargetFile     =   nsnull;
@@ -116,7 +115,7 @@ nsInstallPatch::nsInstallPatch( nsInstall* inInstall,
     mJarLocation    =   new nsString(inJarLocation);
     mVersionInfo    =   new nsInstallVersion();
     tmp->Clone(getter_AddRefs(mTargetFile));
-    
+
     if (mRegistryName == nsnull ||
         mJarLocation  == nsnull ||
         mTargetFile   == nsnull ||
@@ -142,12 +141,12 @@ nsInstallPatch::nsInstallPatch( nsInstall* inInstall,
 {
     MOZ_COUNT_CTOR(nsInstallPatch);
 
-    if ((inInstall == nsnull) || (inVRName.IsEmpty()) || (inJarLocation.IsEmpty())) 
+    if ((inInstall == nsnull) || (inVRName.IsEmpty()) || (inJarLocation.IsEmpty()))
     {
         *error = nsInstall::INVALID_ARGUMENTS;
         return;
     }
-    
+
     nsCOMPtr<nsIFile> tmp = folderSpec->GetFileSpec();
     if (!tmp)
     {
@@ -171,11 +170,11 @@ nsInstallPatch::nsInstallPatch( nsInstall* inInstall,
         *error = nsInstall::OUT_OF_MEMORY;
         return;
     }
-    
+
     mVersionInfo->Init(inVInfo);
-    
+
     if(! inPartialPath.IsEmpty())
-        mTargetFile->Append(NS_LossyConvertUCS2toASCII(inPartialPath).get());
+        mTargetFile->Append(inPartialPath);
 }
 
 nsInstallPatch::~nsInstallPatch()
@@ -188,13 +187,13 @@ nsInstallPatch::~nsInstallPatch()
 
     if (mJarLocation)
         delete mJarLocation;
-    
+
     if (mRegistryName)
         delete mRegistryName;
 
     //if (mPatchedFile)
     //    delete mPatchedFile;
-    
+
     //if (mPatchFile)
     //    delete mPatchFile;
 
@@ -206,7 +205,7 @@ PRInt32 nsInstallPatch::Prepare()
 {
     PRInt32 err;
     PRBool deleteOldSrc, flagExists, flagIsFile;
-    
+
     if (mTargetFile == nsnull)
         return  nsInstall::INVALID_ARGUMENTS;
 
@@ -229,24 +228,24 @@ PRInt32 nsInstallPatch::Prepare()
     }
 
     if (err != nsInstall::SUCCESS)
-    {   
+    {
         return err;
     }
 
     err =  mInstall->ExtractFileFromJar(*mJarLocation, mTargetFile, getter_AddRefs(mPatchFile));
-   
-    
+
+
     nsCOMPtr<nsIFile> fileName = nsnull;
     //nsVoidKey ikey( HashFilePath( nsFilePath(*mTargetFile) ) );//nsIFileXXX: nsFilePath?
     nsVoidKey ikey( HashFilePath( mTargetFile ));
 
     mInstall->GetPatch(&ikey, getter_AddRefs(fileName));
 
-    if (fileName != nsnull) 
+    if (fileName != nsnull)
     {
         deleteOldSrc = PR_TRUE;
-    } 
-    else 
+    }
+    else
     {
         fileName     = mTargetFile;
         deleteOldSrc = PR_FALSE;
@@ -255,7 +254,7 @@ PRInt32 nsInstallPatch::Prepare()
     err = NativePatch(  fileName,           // the file to patch
                         mPatchFile,         // the patch that was extracted from the jarfile
                         getter_AddRefs(mPatchedFile));     // the new patched file
-    
+
     // clean up extracted diff data file
     mPatchFile->Exists(&flagExists);
     if ( (mPatchFile != nsnull) && (flagExists) )
@@ -265,44 +264,44 @@ PRInt32 nsInstallPatch::Prepare()
 
 
     if (err != nsInstall::SUCCESS)
-    {   
+    {
         // clean up tmp patched file since patching failed
-      mPatchFile->Exists(&flagExists);
- 		  if ((mPatchedFile != nsnull) && (flagExists))
-      {
-			    mPatchedFile->Remove(PR_FALSE);
-      }
-		return err;
+        mPatchFile->Exists(&flagExists);
+        if ((mPatchedFile != nsnull) && (flagExists))
+        {
+          mPatchedFile->Remove(PR_FALSE);
+        }
+        return err;
     }
 
     PR_ASSERT(mPatchedFile != nsnull);
     mInstall->AddPatch(&ikey, mPatchedFile );
 
-    if ( deleteOldSrc ) 
+    if ( deleteOldSrc )
     {
     DeleteFileNowOrSchedule(fileName );
     }
-  
+
     return err;
 }
 
 PRInt32 nsInstallPatch::Complete()
-{  
+{
     PRBool flagEquals;
 
-    if ((mInstall == nsnull) || (mVersionInfo == nsnull) || (mPatchedFile == nsnull) || (mTargetFile == nsnull)) 
+    if ((mInstall == nsnull) || (mVersionInfo == nsnull) || (mPatchedFile == nsnull) || (mTargetFile == nsnull))
     {
         return nsInstall::INVALID_ARGUMENTS;
     }
-    
+
     PRInt32 err = nsInstall::SUCCESS;
 
     nsCOMPtr<nsIFile> fileName = nsnull;
     //nsVoidKey ikey( HashFilePath( nsFilePath(*mTargetFile) )  );//nsIFileXXX: nsFilePath?
     nsVoidKey ikey( HashFilePath( mTargetFile ));
-    
+
     mInstall->GetPatch(&ikey, getter_AddRefs(fileName));
-    
+
     if (fileName == nsnull)
     {
         err = nsInstall::UNEXPECTED_ERROR;
@@ -310,19 +309,19 @@ PRInt32 nsInstallPatch::Complete()
     else
     {
       fileName->Equals(mPatchedFile, &flagEquals);
-      if (flagEquals) 
+      if (flagEquals)
       {
         // the patch has not been superceded--do final replacement
-        err = ReplaceFileNowOrSchedule( mPatchedFile, mTargetFile);
-        if ( 0 == err || nsInstall::REBOOT_NEEDED == err ) 
+        err = ReplaceFileNowOrSchedule( mPatchedFile, mTargetFile, 0);
+        if ( 0 == err || nsInstall::REBOOT_NEEDED == err )
         {
             nsString tempVersionString;
             mVersionInfo->ToString(tempVersionString);
             
-            nsXPIDLCString tempPath;
-            mTargetFile->GetPath(getter_Copies(tempPath));
+            nsCAutoString tempPath;
+            mTargetFile->GetNativePath(tempPath);
 
-            // DO NOT propagate version registry errors, it will abort 
+            // DO NOT propagate version registry errors, it will abort
             // FinalizeInstall() leaving things hosed. These piddly errors
             // aren't worth that.
             VR_Install( NS_CONST_CAST(char *, NS_ConvertUCS2toUTF8(*mRegistryName).get()),
@@ -362,7 +361,7 @@ void nsInstallPatch::Abort()
 
 char* nsInstallPatch::toString()
 {
-	  char* buffer = new char[1024];
+    char* buffer = new char[1024];
     char* rsrcVal = nsnull;
 
     if (buffer == nsnull || !mInstall)
@@ -374,14 +373,14 @@ char* nsInstallPatch::toString()
 
         if (rsrcVal)
         {
-            char* temp;
-            mTargetFile->GetPath(&temp);
-            sprintf( buffer, rsrcVal, temp); 
+            nsCAutoString temp;
+            mTargetFile->GetNativePath(temp);
+            sprintf( buffer, rsrcVal, temp.get()); 
             nsCRT::free(rsrcVal);
         }
     }
 
-	return buffer;
+    return buffer;
 }
 
 
@@ -402,97 +401,93 @@ PRInt32
 nsInstallPatch::NativePatch(nsIFile *sourceFile, nsIFile *patchFile, nsIFile **newFile)  //nsIFileXXX: changed & to *
 {
 
-	PRBool flagExists;
+  PRBool flagExists;
   nsresult rv;
-  DIFFDATA	  *dd;
-	PRInt32		  status		   = GDIFF_ERR_MEM;
-	char 		    *tmpurl		   = NULL;
-	//nsFileSpec  *outFileSpec = new nsFileSpec; 
-  //nsFileSpec  *tempSrcFile = new nsFileSpec;   // TODO: do you need to free?
+  DIFFDATA    *dd;
+  PRInt32     status       = GDIFF_ERR_MEM;
+  char        *tmpurl      = NULL;
   nsCOMPtr<nsIFile> outFileSpec;
   nsCOMPtr<nsIFile> tempSrcFile;
   nsCOMPtr<nsILocalFile> uniqueSrcFile;
   nsCOMPtr<nsILocalFile> patchFileLocal = do_QueryInterface(patchFile, &rv);
   
-  char*        realfile;
-  sourceFile->GetPath(&realfile);
+  nsCAutoString realfile;
+  sourceFile->GetNativePath(realfile);
 
   sourceFile->Clone(getter_AddRefs(outFileSpec));
 
-	dd = (DIFFDATA *)PR_Calloc( 1, sizeof(DIFFDATA));
-	if (dd != NULL)
-	{
-		dd->databuf = (uchar*)PR_Malloc(BUFSIZE);
-		if (dd->databuf == NULL) 
-		{
-			status = GDIFF_ERR_MEM;
-			goto cleanup;
-		}
+  dd = (DIFFDATA *)PR_Calloc( 1, sizeof(DIFFDATA));
+  if (dd != NULL)
+  {
+    dd->databuf = (uchar*)PR_Malloc(BUFSIZE);
+    if (dd->databuf == NULL)
+    {
+      status = GDIFF_ERR_MEM;
+      goto cleanup;
+    }
 
 
-		dd->bufsize = BUFSIZE;
+    dd->bufsize = BUFSIZE;
 
-		// validate patch header & check for special instructions
+    // validate patch header & check for special instructions
     patchFileLocal->OpenNSPRFileDesc(PR_RDONLY, 0666, &dd->fDiff);
 
-		if (dd->fDiff != NULL)
-		{
-			status = gdiff_parseHeader(dd);
-		} 
-    else 
+    if (dd->fDiff != NULL)
     {
-			status = GDIFF_ERR_ACCESS;
-		}
+      status = gdiff_parseHeader(dd);
+    }
+    else
+    {
+      status = GDIFF_ERR_ACCESS;
+    }
 
 
     // in case we need to unbind Win32 images OR encode Mac file
     if (( dd->bWin32BoundImage || dd->bMacAppleSingle) && (status == GDIFF_OK ))
     {
         // make an unique tmp file  (FILENAME-src.EXT)
-        char* leafName;
-        rv = sourceFile->GetLeafName(&leafName);
+        nsAutoString tmpFileName;
+        rv = sourceFile->GetLeafName(tmpFileName);
 
-        nsString tmpName(NS_LITERAL_STRING("-src"));
-        nsString tmpFileName; tmpFileName.AssignWithConversion(leafName);
+        NS_NAMED_LITERAL_STRING(tmpName, "-src");
 
         PRInt32 i;
         if ((i = tmpFileName.RFindChar('.')) > 0)
         {
-            nsString ext;
-            nsString fName;
+            nsAutoString ext;
+            nsAutoString fName;
             tmpFileName.Right(ext, (tmpFileName.Length() - i) );        
             tmpFileName.Left(fName, (tmpFileName.Length() - (tmpFileName.Length() - i)));
-            tmpFileName = fName + tmpName + ext;
+            tmpFileName.Assign(fName + tmpName + ext);
 
         } else {
-               tmpFileName += tmpName;
+            tmpFileName += NS_LITERAL_STRING("-src");
         }
-        
-    
-        rv = sourceFile->Clone(getter_AddRefs(tempSrcFile));  //Clone the sourceFile
-        tempSrcFile->SetLeafName(NS_LossyConvertUCS2toASCII(tmpFileName).get()); //Append the new leafname
-        uniqueSrcFile = do_QueryInterface(tempSrcFile, &rv);  //Create an nsILocalFile version to pass to MakeUnique
-        MakeUnique(uniqueSrcFile); 
 
-       char*        realfile;
-       sourceFile->GetPath(&realfile);
+
+        rv = sourceFile->Clone(getter_AddRefs(tempSrcFile));  //Clone the sourceFile
+        tempSrcFile->SetLeafName(tmpFileName); //Append the new leafname
+        uniqueSrcFile = do_QueryInterface(tempSrcFile, &rv);  //Create an nsILocalFile version to pass to MakeUnique
+        MakeUnique(uniqueSrcFile);
+
+        nsCAutoString realfile;
+        sourceFile->GetNativePath(realfile);
 
 #ifdef WIN32
         // unbind Win32 images
 
-        char* unboundFile;
-        uniqueSrcFile->GetPath(&unboundFile);
+        nsCAutoString unboundFile;
+        uniqueSrcFile->GetNativePath(unboundFile);
 
-        if (su_unbind(realfile, unboundFile))  //
+        if (su_unbind((char*)realfile.get(), (char*)unboundFile.get()))  //
         {
-            PL_strfree(realfile);
-            realfile = PL_strdup(unboundFile);
+            realfile = unboundFile;
         }
         else
         {
             status = GDIFF_ERR_MEM;
         }
-        PL_strfree(unboundFile);
+        unboundFile.Truncate();
 #endif
 #ifdef XP_MAC
    // Encode src file, and put into temp file
@@ -500,14 +495,13 @@ nsInstallPatch::NativePatch(nsIFile *sourceFile, nsIFile *patchFile, nsIFile **n
         nsCOMPtr<nsILocalFileMac> tempSourceFile;
         tempSourceFile = do_QueryInterface(sourceFile, &rv);
         tempSourceFile->GetFSSpec(&sourceSpec);
-    
-        status = PAS_EncodeFile(&sourceSpec, &tempSpec);   
+
+        status = PAS_EncodeFile(&sourceSpec, &tempSpec);
 
         if (status == noErr)
         {
             // set
-            PL_strfree(realfile);
-            tempSrcFile->GetPath(&realfile);
+            tempSrcFile->GetNativePath(realfile);
         }
 #endif
     }
@@ -516,16 +510,15 @@ nsInstallPatch::NativePatch(nsIFile *sourceFile, nsIFile *patchFile, nsIFile **n
     goto cleanup;
 
     // make a unique file at the same location of our source file  (FILENAME-ptch.EXT)
-    nsString patchFileName(NS_LITERAL_STRING("-ptch"));
-    char* leafName;
-    sourceFile->GetLeafName(&leafName);
-    nsString newFileName; newFileName.AssignWithConversion(leafName);;
+    NS_NAMED_LITERAL_STRING(patchFileName, "-ptch");
+    nsAutoString newFileName;
+    sourceFile->GetLeafName(newFileName);
 
     PRInt32 index;
     if ((index = newFileName.RFindChar('.')) > 0)
     {
-        nsString extention;
-        nsString fileName;
+        nsAutoString extention;
+        nsAutoString fileName;
         newFileName.Right(extention, (newFileName.Length() - index) );        
         newFileName.Left(fileName, (newFileName.Length() - (newFileName.Length() - index)));
         newFileName = fileName + patchFileName + extention;
@@ -537,7 +530,7 @@ nsInstallPatch::NativePatch(nsIFile *sourceFile, nsIFile *patchFile, nsIFile **n
     }
 
 
-    outFileSpec->SetLeafName(NS_LossyConvertUCS2toASCII(newFileName).get());  //Set new leafname
+    outFileSpec->SetLeafName(newFileName);  //Set new leafname
     nsCOMPtr<nsILocalFile> outFileLocal = do_QueryInterface(outFileSpec, &rv); //Create an nsILocalFile version 
                                                                                //to send to MakeUnique()
     MakeUnique(outFileLocal);
@@ -546,7 +539,7 @@ nsInstallPatch::NativePatch(nsIFile *sourceFile, nsIFile *patchFile, nsIFile **n
     //dd->fSrc = PR_Open ( realfile, PR_RDONLY, 0666);
     //dd->fOut = PR_Open ( outFile, PR_RDWR|PR_CREATE_FILE|PR_TRUNCATE, 0666);
     nsCOMPtr<nsILocalFile> realFileLocal = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);;
-    realFileLocal->InitWithPath(realfile);
+    realFileLocal->InitWithNativePath(realfile);
 
     realFileLocal->OpenNSPRFileDesc(PR_RDONLY, 0666, &dd->fSrc);
     outFileLocal->OpenNSPRFileDesc(PR_RDWR|PR_CREATE_FILE|PR_TRUNCATE, 0666, &dd->fOut);
@@ -569,8 +562,8 @@ nsInstallPatch::NativePatch(nsIFile *sourceFile, nsIFile *patchFile, nsIFile **n
             status = GDIFF_ERR_CHECKSUM_RESULT;
 
         rv = outFileSpec->Clone(newFile);
-    } 
-    else 
+    }
+    else
     {
         status = GDIFF_ERR_ACCESS;
     }
@@ -579,66 +572,65 @@ nsInstallPatch::NativePatch(nsIFile *sourceFile, nsIFile *patchFile, nsIFile **n
 
 
 #ifdef XP_MAC
-  if ( dd->bMacAppleSingle && status == GDIFF_OK ) 
+  if ( dd->bMacAppleSingle && status == GDIFF_OK )
  {
         // create another file, so that we can decode somewhere
-        //nsFileSpec anotherName = *outFileSpec;
         nsCOMPtr<nsILocalFile> anotherName;
         nsCOMPtr<nsIFile> bsTemp;
-        
+
         outFileSpec->Clone(getter_AddRefs(bsTemp));   //Clone because we'll be changing the name
         anotherName = do_QueryInterface(bsTemp, &rv); //Set the old name
         MakeUnique(anotherName);  //Now give it the new name
-        
-		// Close the out file so that we can read it 		
-		PR_Close( dd->fOut );
-		dd->fOut = NULL;
-		
-		
-		FSSpec outSpec;
-		FSSpec anotherSpec;
-		nsCOMPtr<nsILocalFileMac> outSpecMacSpecific;
-		nsCOMPtr<nsILocalFileMac> anotherNameMacSpecific;
-		
-		anotherNameMacSpecific = do_QueryInterface(anotherName, &rv); //set value to nsILocalFileMac (sheesh)
-		outSpecMacSpecific = do_QueryInterface(outFileSpec, &rv); //ditto 
-		
-		anotherNameMacSpecific->GetFSSpec(&anotherSpec);
-		outSpecMacSpecific->GetFSSpec(&outSpec);
-		
-		outFileSpec->Exists(&flagExists);
-		if ( flagExists )
-		{
-		    PRInt64 fileSize;
-		    outFileSpec->GetFileSize(&fileSize);
-		}
-		
-			
+
+    // Close the out file so that we can read it
+    PR_Close( dd->fOut );
+    dd->fOut = NULL;
+
+
+    FSSpec outSpec;
+    FSSpec anotherSpec;
+    nsCOMPtr<nsILocalFileMac> outSpecMacSpecific;
+    nsCOMPtr<nsILocalFileMac> anotherNameMacSpecific;
+
+    anotherNameMacSpecific = do_QueryInterface(anotherName, &rv); //set value to nsILocalFileMac (sheesh)
+    outSpecMacSpecific = do_QueryInterface(outFileSpec, &rv); //ditto
+
+    anotherNameMacSpecific->GetFSSpec(&anotherSpec);
+    outSpecMacSpecific->GetFSSpec(&outSpec);
+
+    outFileSpec->Exists(&flagExists);
+    if ( flagExists )
+    {
+        PRInt64 fileSize;
+        outFileSpec->GetFileSize(&fileSize);
+    }
+
+
         status =  PAS_DecodeFile(&outSpec, &anotherSpec);
-		if (status != noErr)
-		{
-		   	goto cleanup;
+        if (status != noErr)
+        {
+            goto cleanup;
         }
-		
+
         nsCOMPtr<nsIFile> parent;
-        
+
         outFileSpec->GetParent(getter_AddRefs(parent));
-        
+
         outFileSpec->Remove(PR_FALSE);
         
-        char* leaf;
-        anotherName->GetLeafName(&leaf);
+        nsAutoString leaf;
+        anotherName->GetLeafName(leaf);
         anotherName->CopyTo(parent, leaf);
         
         anotherName->Clone(newFile);
-        
-	}
-	
-#endif 
+
+  }
+
+#endif
 
 
 cleanup:
-    if ( dd != NULL ) 
+    if ( dd != NULL )
     {
         if ( dd->fSrc != nsnull )
             PR_Close( dd->fSrc );
@@ -665,11 +657,6 @@ cleanup:
         PR_DELETE( tmpurl );
     }
 
-    if (realfile != NULL)
-    {
-        PL_strfree(realfile);
-    }
-    
     if (tempSrcFile)
     {
         tempSrcFile->Exists(&flagExists);
@@ -706,32 +693,30 @@ cleanup:
 
     return status;
 
-    // return -1;	//old return value
+    // return -1; //old return value
 }
 
 
-void* 
+void*
 nsInstallPatch::HashFilePath(nsIFile* aPath)
 {
     PRUint32 rv = 0;
 
-    char* cPath;
-    aPath->GetPath(&cPath);
+    nsCAutoString cPath;
+    aPath->GetNativePath(cPath);
     
-    if(cPath != nsnull) 
+    if (!cPath.IsEmpty())
     {
         char  ch;
-        char* pathIndex = cPath;
+        const char* pathIndex = cPath.get();
 
-        while ((ch = *pathIndex++) != 0) 
+        while ((ch = *pathIndex++) != 0)
         {
             // FYI: rv = rv*37 + ch
             rv = ((rv << 5) + (rv << 2) + rv) + ch;
         }
     }
 
-    PL_strfree(cPath);
-
     return (void*)rv;
 }
 
@@ -756,7 +741,7 @@ int32 gdiff_parseHeader( pDIFFDATA dd )
 
     /* Read the fixed-size part of the header */
 
-	nRead = PR_Read (dd->fDiff, header, GDIFF_HEADERSIZE);
+    nRead = PR_Read (dd->fDiff, header, GDIFF_HEADERSIZE);
     if ( nRead != GDIFF_HEADERSIZE ||
          memcmp( header, GDIFF_MAGIC, GDIFF_MAGIC_LEN ) != 0  ||
          header[GDIFF_VER_POS] != GDIFF_VER )
@@ -782,10 +767,10 @@ int32 gdiff_parseHeader( pDIFFDATA dd )
 
             if ( dd->oldChecksum != NULL && dd->newChecksum != NULL )
             {
-				nRead = PR_Read (dd->fDiff, dd->oldChecksum, oldcslen);
+                nRead = PR_Read (dd->fDiff, dd->oldChecksum, oldcslen);
                 if ( nRead == oldcslen )
                 {
-					nRead = PR_Read (dd->fDiff, dd->newChecksum, newcslen);
+          nRead = PR_Read (dd->fDiff, dd->newChecksum, newcslen);
                     if ( nRead != newcslen ) {
                         err = GDIFF_ERR_HEADER;
                     }
@@ -808,19 +793,19 @@ int32 gdiff_parseHeader( pDIFFDATA dd )
             uchar   *buf;
             uchar   lenbuf[GDIFF_APPDATALEN];
 
-			nRead = PR_Read(dd->fDiff, lenbuf, GDIFF_APPDATALEN);
-            if ( nRead == GDIFF_APPDATALEN ) 
+            nRead = PR_Read(dd->fDiff, lenbuf, GDIFF_APPDATALEN);
+            if ( nRead == GDIFF_APPDATALEN )
             {
                 appdataSize = getlong(lenbuf);
 
-                if ( appdataSize > 0 ) 
+                if ( appdataSize > 0 )
                 {
                     buf = (uchar *)PR_MALLOC( appdataSize );
 
                     if ( buf != NULL )
                     {
-						nRead = PR_Read (dd->fDiff, buf, appdataSize);
-                        if ( nRead == appdataSize ) 
+                        nRead = PR_Read (dd->fDiff, buf, appdataSize);
+                        if ( nRead == appdataSize )
                         {
                             if ( 0 == memcmp( buf, APPFLAG_W32BOUND, appdataSize ) )
                                 dd->bWin32BoundImage = TRUE;
@@ -852,16 +837,16 @@ int32 gdiff_parseHeader( pDIFFDATA dd )
 /*---------------------------------------------------------
  *  gdiff_validateFile()
  *
- *  computes the checksum of the file and compares it to 
+ *  computes the checksum of the file and compares it to
  *  the value stored in the GDIFF header
  *---------------------------------------------------------
  */
-static 
+static
 int32 gdiff_validateFile( pDIFFDATA dd, int file )
 {
-    int32		result;
-    PRFileDesc*	fh;
-    uchar*		chksum;
+    int32       result;
+    PRFileDesc* fh;
+    uchar*      chksum;
 
     /* which file are we dealing with? */
     if ( file == SRCFILE ) {
@@ -913,12 +898,12 @@ int32 gdiff_validateFile( pDIFFDATA dd, int file )
 /*---------------------------------------------------------
  *  gdiff_valCRC32()
  *
- *  computes the checksum of the file and compares it to 
+ *  computes the checksum of the file and compares it to
  *  the passed in checksum.  Assumes file is positioned at
  *  beginning.
  *---------------------------------------------------------
  */
-static 
+static
 int32 gdiff_valCRC32( pDIFFDATA dd, PRFileDesc* fh, uint32 chksum )
 {
     uint32 crc;
@@ -926,11 +911,11 @@ int32 gdiff_valCRC32( pDIFFDATA dd, PRFileDesc* fh, uint32 chksum )
 
     crc = crc32(0L, Z_NULL, 0);
 
-	nRead = PR_Read (fh, dd->databuf, dd->bufsize);
-    while ( nRead > 0 ) 
+    nRead = PR_Read (fh, dd->databuf, dd->bufsize);
+    while ( nRead > 0 )
     {
         crc = crc32( crc, dd->databuf, nRead );
-		nRead = PR_Read (fh, dd->databuf, dd->bufsize);
+        nRead = PR_Read (fh, dd->databuf, dd->bufsize);
     }
 
     if ( crc == chksum )
@@ -1073,7 +1058,7 @@ int32 gdiff_getdiff( pDIFFDATA dd, uchar *buffer, uint32 length )
 {
     uint32 bytesRead;
 
-	bytesRead = PR_Read (dd->fDiff, buffer, length);
+    bytesRead = PR_Read (dd->fDiff, buffer, length);
     if ( bytesRead != length )
         return GDIFF_ERR_BADDIFF;
 
@@ -1096,13 +1081,13 @@ int32 gdiff_add( pDIFFDATA dd, uint32 count )
 
     while ( count > 0 ) {
         chunksize = ( count > dd->bufsize) ? dd->bufsize : count;
-		nRead = PR_Read (dd->fDiff, dd->databuf, chunksize);
+        nRead = PR_Read (dd->fDiff, dd->databuf, chunksize);
         if ( nRead != chunksize ) {
             err = GDIFF_ERR_BADDIFF;
             break;
         }
 
-		PR_Write (dd->fOut, dd->databuf, chunksize);
+        PR_Write (dd->fOut, dd->databuf, chunksize);
 
         count -= chunksize;
     }
@@ -1125,18 +1110,18 @@ int32 gdiff_copy( pDIFFDATA dd, uint32 position, uint32 count )
     uint32 nRead;
     uint32 chunksize;
 
-	PR_Seek (dd->fSrc, position, PR_SEEK_SET);
+    PR_Seek (dd->fSrc, position, PR_SEEK_SET);
 
     while ( count > 0 ) {
         chunksize = (count > dd->bufsize) ? dd->bufsize : count;
 
-		nRead = PR_Read (dd->fSrc, dd->databuf, chunksize);
+        nRead = PR_Read (dd->fSrc, dd->databuf, chunksize);
         if ( nRead != chunksize ) {
             err = GDIFF_ERR_OLDFILE;
             break;
         }
 
-		PR_Write (dd->fOut, dd->databuf, chunksize);
+        PR_Write (dd->fOut, dd->databuf, chunksize);
 
         count -= chunksize;
     }
@@ -1153,7 +1138,7 @@ int32 gdiff_copy( pDIFFDATA dd, uint32 position, uint32 count )
  *  executables and .DLL's
  *---------------------------------------------------------
  */
-static 
+static
 PRBool su_unbind(char* oldfile, char* newfile)
 {
     PRBool bSuccess = FALSE;
@@ -1175,29 +1160,29 @@ PRBool su_unbind(char* oldfile, char* newfile)
     PIMAGE_IMPORT_DESCRIPTOR    pImp;
 
     typedef BOOL (__stdcall *BINDIMAGEEX)(DWORD Flags,
-									  LPSTR ImageName,
-									  LPSTR DllPath,
-									  LPSTR SymbolPath,
-									  PVOID StatusRoutine);
+                                      LPSTR ImageName,
+                                      LPSTR DllPath,
+                                      LPSTR SymbolPath,
+                                      PVOID StatusRoutine);
     HINSTANCE   hImageHelp;
     BINDIMAGEEX pfnBindImageEx;
 
     if ( oldfile != NULL && newfile != NULL &&
          CopyFile( oldfile, newfile, FALSE ) )
-    {   
+    {
         /* call BindImage() first to make maximum room for a possible
          * NT-style Bound Import Descriptors which can change various
          * offsets in the file */
-	    hImageHelp = LoadLibrary("IMAGEHLP.DLL");
-	    if ( hImageHelp > (HINSTANCE)HINSTANCE_ERROR ) {
-        	pfnBindImageEx = (BINDIMAGEEX)GetProcAddress(hImageHelp, "BindImageEx");
-    	    if (pfnBindImageEx) {
+        hImageHelp = LoadLibrary("IMAGEHLP.DLL");
+        if ( hImageHelp > (HINSTANCE)HINSTANCE_ERROR ) {
+            pfnBindImageEx = (BINDIMAGEEX)GetProcAddress(hImageHelp, "BindImageEx");
+            if (pfnBindImageEx) {
                 pfnBindImageEx(0, newfile, NULL, NULL, NULL);
             }
-		    FreeLibrary(hImageHelp);
-	    }
-        
-        
+            FreeLibrary(hImageHelp);
+        }
+
+
         fh = fopen( newfile, "r+b" );
         if ( fh == NULL )
             goto bail;
@@ -1213,9 +1198,9 @@ PRBool su_unbind(char* oldfile, char* newfile)
         /* read and validate the NT header */
         fseek( fh, mz.e_lfanew, SEEK_SET );
         nRead = fread( &nt, 1, sizeof(nt), fh );
-        if ( nRead != sizeof(nt) || 
+        if ( nRead != sizeof(nt) ||
              nt.Signature != IMAGE_NT_SIGNATURE ||
-             nt.OptionalHeader.Magic != IMAGE_NT_OPTIONAL_HDR_MAGIC ) 
+             nt.OptionalHeader.Magic != IMAGE_NT_OPTIONAL_HDR_MAGIC )
         {
             goto bail;
         }
@@ -1225,7 +1210,7 @@ PRBool su_unbind(char* oldfile, char* newfile)
         for (i = nt.FileHeader.NumberOfSections; i > 0; i--)
         {
             nRead = fread( &sec, 1, sizeof(sec), fh );
-            if ( nRead != sizeof(sec) ) 
+            if ( nRead != sizeof(sec) )
                 goto bail;
 
             if ( memcmp( sec.Name, ".idata", 6 ) == 0 ) {
@@ -1236,7 +1221,7 @@ PRBool su_unbind(char* oldfile, char* newfile)
 
 
         /* Zap any binding in the imports section */
-        if ( bImports ) 
+        if ( bImports )
         {
             buf = (char*)malloc( sec.SizeOfRawData );
             if ( buf == NULL )
@@ -1248,7 +1233,7 @@ PRBool su_unbind(char* oldfile, char* newfile)
                 free( buf );
                 goto bail;
             }
-            
+
             pImp = (PIMAGE_IMPORT_DESCRIPTOR)buf;
             while ( pImp->OriginalFirstThunk != 0 )
             {
@@ -1258,10 +1243,10 @@ PRBool su_unbind(char* oldfile, char* newfile)
                     pImp->TimeDateStamp = 0;
                     pImp->ForwarderChain = 0;
                     bModified = TRUE;
-    
+
                     pOrigThunk = (PDWORD)(buf + (DWORD)(pImp->OriginalFirstThunk) - sec.VirtualAddress);
                     pBoundThunk = (PDWORD)(buf + (DWORD)(pImp->FirstThunk) - sec.VirtualAddress);
-    
+
                     for ( ; *pOrigThunk != 0; pOrigThunk++, pBoundThunk++ ) {
                         *pBoundThunk = *pOrigThunk;
                     }
@@ -1269,26 +1254,26 @@ PRBool su_unbind(char* oldfile, char* newfile)
                 pImp++;
             }
 
-            if ( bModified ) 
+            if ( bModified )
             {
                 /* it's been changed, write out the section */
                 fseek( fh, sec.PointerToRawData, SEEK_SET );
                 fwrite( buf, 1, sec.SizeOfRawData, fh );
             }
-    
+
             free( buf );
         }
 
 
         /* Check for a Bound Import Directory in the headers */
         pDir = &nt.OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_BOUND_IMPORT];
-        if ( pDir->VirtualAddress != 0 ) 
+        if ( pDir->VirtualAddress != 0 )
         {
             /* we've got one, so stomp it */
             buf = (char*)calloc( pDir->Size, 1 );
             if ( buf == NULL )
                 goto bail;
-        
+
             fseek( fh, pDir->VirtualAddress, SEEK_SET );
             fwrite( buf, pDir->Size, 1, fh );
             free( buf );
@@ -1313,9 +1298,10 @@ PRBool su_unbind(char* oldfile, char* newfile)
     }
 
 bail:
-    if ( fh != NULL ) 
+    if ( fh != NULL )
         fclose(fh);
 
     return bSuccess;
 }
 #endif
+
